import React from "react";
import { Bar } from 'react-chartjs-2';
import _ from 'lodash';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';

import PropTypes from 'prop-types';


/**
 * Component to show data ingested per category in cycle(s).
 */
export default function CycleCategoryWiseData(props) {
    let { data } = props;
    const getBarData = () => {
        return {
            labels: _.map(data, 'cycle'),
            datasets: [

                {
                    label: 'Raw IF',
                    data: _.map(data, 'rawIF'),
                    backgroundColor: '#4472C4',
                    barThickness: 75,
                },
                {
                    label: 'Raw BF',
                    data: _.map(data, 'rawBF'),
                    backgroundColor: '#ED7D31',
                    barThickness: 75,

                },
                {
                    label: 'Preproc IF',
                    data: _.map(data, 'preprocIF'),
                    backgroundColor: '#FFC000',
                    barThickness: 75,
                },
                {
                    label: 'Pulp BF',
                    data: _.map(data, 'pulpBF'),
                    backgroundColor: '#44a3ce',
                    barThickness: 75,
                },
            ]
        };
    }

    const getBarOptions = () => {
        return {
            indexAxis: 'x',
            elements: {
                bar: {
                    borderWidth: 0,
                },
            },
            scales: {
                x: {
                    stacked: true,
                    categoryPercentage: 1.0,
                    barPercentage: 1.0
                },
                y: {
                    stacked: true,
                }
            },
            responsive: true,
            plugins: {
                legend: {
                    position: 'bottom',
                },
                title: {
                    display: true,
                    text: 'Ingested Data per Type (TB)',
                }
            }
        };
    }



    return (
        <React.Fragment>
            <div id={`cycles-category-data`}
                style={{ paddingTop: "10px", paddingBottom: "10px" }}>
                <Bar data={getBarData()} options={getBarOptions()} width="50%" height="20" />
            </div>

            <div className="ttd-details" id={`cycle-ttd-details`}>
                <DataTable value={data} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em', textAlign: 'center' }}>
                    <Column field="cycle" header="Cycle Code"></Column>
                    <Column field="rawIF" header="Raw IF (TB)"></Column>
                    <Column field="rawBF" header="Raw BF (TB)"></Column>
                    <Column field="preprocIF" header="Preproc IF (TB)"></Column>
                    <Column field="pulpBF" header="Pulp BF (TB)"></Column>
                </DataTable>
            </div>

        </React.Fragment>
    )
}
CycleCategoryWiseData.propTypes = {

    data: PropTypes.array,



};