import PropTypes from 'prop-types';
import React  from "react";
import 'chart.js/auto';
import { Bar } from 'react-chartjs-2';
import _ from 'lodash';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';


/** Compoent to show usage in ILT and Local mode */

    export default function CyclesILTLocalUsage(props) {
        let { data } = props;
    /**
     * Function that returns the data required for the chart with lablels, datasets, colors for bar, line, barwidth, etc.,
     * @returns Object - with dataset, labels, colors, barwidth, etc.,
     */
    const getBarData =()=> {
        let reportData = data;
        reportData = reportData?.slice(0,-1);
        return {
            labels: _.map(reportData, 'cycle'),
            datasets:[

                {   label: 'ILT-mode observing', 
                    data:_.map(reportData, 'observing'),
                    backgroundColor: '#4472C4',
                    barThickness: 75,
                    fill: 'origin'    
                },
                {   label: 'ILT-mode idle/test', 
                    data:_.map(reportData, 'idle'),
                    backgroundColor: '#ED7D31',
                    barThickness: 75,
                    fill: 'origin'    
                },
                {   label: 'Local-mode', 
                    data:_.map(reportData, 'local_mode'),
                    backgroundColor: '#A5A5A5',
                    barThickness: 75,
                    fill: 'origin'
                },
            ]
        };
    }

    /**
     * Function that returns the bar options like axis, scales, max axis value, tooltip, etc.,
     * @returns Object
     */
   const getBarOptions =()=> {
        return {
            indexAxis: 'x',
            // Elements options apply to all of the options unless overridden in a dataset
            // In this case, we are setting the border of each horizontal bar to be 1px wide
            elements: {
              bar: {
                borderWidth: 1
              },
            },
            scales: {
                x: {
                    stacked: true,
                },
                y: {
                    beginAtZero: true,
                    stacked: true,   
                }
            },
            responsive: true,
            plugins: {
              filler: {
                  propagate: true
              },
              legend: {
                position: 'bottom',
              },
              title: {
                display: true,
                text: 'ILT Usage Plot',
              },
            }
        };
    }

        let reportData = data;
        return(
            <React.Fragment>
                <div  id={`cycles-ilt-usage`}
                    style={{paddingTop: "10px", paddingBottom: "10px"}}>
                    <Bar data={getBarData()} options={getBarOptions()} width="50%" height="20" />
                </div>
                <div className="ilt-time-details" id={`cycle-ilt-details`}>
                    <DataTable value={reportData} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em', textAlign: 'center'}}>
                        <Column field="cycle" header="Cycle Code"></Column>
                        <Column field="observing" header="ILT-mode observing (hrs)"></Column>
                        <Column field="idle" header="ILT-mode idle/test (hrs)"></Column>
                        <Column field="local_mode" header="Local-mode (hrs)"></Column>
                        <Column field="ilt_total" header="ILT-mode (total, in hrs)"></Column>
                        <Column field="ilt_mode_dutch" header="ILT-mode (Dutch array observing, in hrs)"></Column>
                    </DataTable>
                </div>
            </React.Fragment>
        )
    
}


CyclesILTLocalUsage.propTypes = {
    data: PropTypes.array,
};
