import { useEffect, useState } from 'react';
import { Chart } from 'primereact/chart';
import moment from 'moment';
import { Calendar } from 'primereact/calendar';
import { Button } from 'primereact/button';
import { ProgressBar } from 'primereact/progressbar';
import ReportService from '../../../services/report.service';
import { HistoricData}  from './failure.historic.data'
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import PageHeader from '../../../layout/components/PageHeader';
import FailureBusinessPdf from './failure.business.pdf';
import FailureBusinessTemplate from './failure.business.template';


export default function FailureReport() {

    const [dates, setDates] = useState([new Date("2023-01-01"), new Date("2023-12-31")]);
    const [checked, setChecked] = useState("");
    const [isLoading, setIsLoading] = useState(null);
    const [failureReport, setFailureReport] = useState(null);
    const [chartData_Issue_count, setChartData_Issue_count] = useState({});
    const [chartData_Issue_count_options, setChartData_Issue_count_options] = useState({});
    const [chartData_Issue_total_duration, setChartData_Issue_total_duration] = useState({});
    const [chartData_Issue_total_duration_options, setChartData_Issue_total_duration_options] = useState({});
    const [chartData_sub_Issue_count, setChartData_sub_Issue_count] = useState({});
    const [chartData_sub_Issue_count_options, setChartData_sub_Issue_count_options] = useState({});
    const [chartData_sub_Issue_total_duration, setChartData_sub_Issue_total_duration] = useState({});
    const [chartData_sub_Issue_total_duration_options, setChartData_sub_Issue_total_duration_options] = useState({});
    const [chartObservingTimeBarData, setChartObservingTimeBarData] = useState({});
    const [chartWallTimeData, setChartWallTimeData] = useState({});
    const [chartWallTimeMainData, setChartWallTimeMainData] = useState({});
    const [chartWallTimeSubTypeData, setChartWallTimeSubTypeData] = useState({});
    const [chartOptions, setChartOptions] = useState({});

    useEffect(() => {
        if (!dates || dates?.length < 2 || dates.length == 2 && dates[1] == null) {
            setChecked("")
        } else {
            setChecked("pi pi-check")
        }
    }, [dates]);

    const MakePieChartData = (element, key, labelkey) => {
        const documentStyle = getComputedStyle(document.documentElement);
        return {
            labels: element?.map(item => item[labelkey]),
            datasets: [
                {
                    data: element?.map(item => item[key]),
                    backgroundColor: Colors(documentStyle, "500"),
                    hoverBackgroundColor: Colors(documentStyle, "400")
                }

            ]
        }
    }

    const MakeChartOptions = (title) => {
        return {
            plugins: {
                legend: {
                    labels: {
                        usePointStyle: false,
                        position: "right",
                    },
                },
                title: { text: title, display: true },
            },

        }
    }

    const generatePdf = async () => {
        setIsLoading(true);
        await FailureBusinessPdf.GeneratePdf(failureReport, dates);
        setIsLoading(false);
    }
    function MakePieChart(failureReport) {
        let pieelement = failureReport?.system_event_summary?.by_issue_type;
        if (pieelement !== null) {
            setChartData_Issue_count(MakePieChartData(pieelement, "count", "issue_type__value"));
            setChartData_Issue_count_options(MakeChartOptions("Issue Type count per Primary failure category"));
            setChartData_Issue_total_duration(MakePieChartData(pieelement, "duration_lost_event", "issue_type__value"));
            setChartData_Issue_total_duration_options({ plugins: { legend: { display: false }, title: { text: "Issue Type Duration", display: true } } });
        }
    }

    function makeSubTypePieChart(failureReport) {
        let piesubtypeelement = failureReport?.system_event_summary?.by_issue_subtype;
        if (piesubtypeelement !== null) {
            setChartData_sub_Issue_count(MakePieChartData(piesubtypeelement, "count", "issue_subtype__value"));
            setChartData_sub_Issue_count_options(MakeChartOptions("Issue sub Type per failure sub-category"));
            setChartData_sub_Issue_total_duration(MakePieChartData(piesubtypeelement, "duration_lost_event", "issue_subtype__value"));
            setChartData_sub_Issue_total_duration_options({ plugins: { legend: { display: false }, title: { text: "Sub Issue Type Duration", display: true } } });
        }
    }

    useEffect(() => {
        const documentStyle = getComputedStyle(document.documentElement);
        if (failureReport == null) return

        MakePieChart(failureReport);
        makeSubTypePieChart(failureReport);
        
        MakeLostTimeBar(failureReport, documentStyle);
        MakeOverallWallTimeTypeBar(failureReport, documentStyle);
        MakeWallTimeIssueTypeBar(failureReport, documentStyle);
        MakeWallTimeSubIssueBar(failureReport, documentStyle);
    }, [failureReport]);

    function MakeLostTimeBar(failureReport, documentStyle) {
        let lostTimeHistogram = failureReport?.lost_time_histogram_on_sky;

        if (lostTimeHistogram) {

            let sortedKeys = Object.keys(lostTimeHistogram).sort((a, b) => a.localeCompare(b))
            let sortedValues = sortedKeys.map(key => Math.floor(lostTimeHistogram[key] ));

            setChartObservingTimeBarData({
                labels: sortedKeys,
                datasets: [
                    {
                        label: 'Overview observing hours lost per month',
                        backgroundColor: documentStyle.getPropertyValue('--orange-500'),
                        data: sortedValues,
                    },
                ],
            });
        }
    }
    function MakeOverallWallTimeTypeBar(failureReport, documentStyle) {
        setChartWallTimeMainData(MakeWallTimeBarWithType(failureReport, documentStyle, "by_issue_type", "Percentage Succes versus Failures ", "issue_type__value",true))
    }

    function MakeWallTimeIssueTypeBar(failureReport, documentStyle) {
        setChartWallTimeData(MakeWallTimeBarWithType(failureReport, documentStyle, "by_issue_type", "Percentage Failures by issue type", "issue_type__value",false))
    }

    function MakeWallTimeSubIssueBar(failureReport, documentStyle) {
        setChartWallTimeSubTypeData(MakeWallTimeBarWithType(failureReport, documentStyle, "by_issue_subtype", "Percentage failures by sub issue type", "issue_subtype__value",false))
    }

    function Colors(documentStyle, subfix) {
        return [
            documentStyle.getPropertyValue('--red-' + subfix),
            documentStyle.getPropertyValue('--orange-' + subfix),
            documentStyle.getPropertyValue('--yellow-' + subfix),
            documentStyle.getPropertyValue('--purple-' + subfix),
            documentStyle.getPropertyValue('--pink-' + subfix),
            documentStyle.getPropertyValue('--bluegray-' + subfix),
            documentStyle.getPropertyValue('--indigo-' + subfix),
            documentStyle.getPropertyValue('--brown-' + subfix),
            documentStyle.getPropertyValue('--teal-' + subfix),
            documentStyle.getPropertyValue('--green-' + subfix),

        ]
    }

    function MakeWallTimeBarWithType(failureReport, documentStyle, type, title, typelabel,isOnlyAggregration) {

        let colors = Colors(documentStyle, "500");
        if (failureReport?.system_event_summary) {
            let byIssueType = failureReport?.["system_event_summary"]?.[type]?.map(
                (item, index) => ({
                    type: 'bar',
                    label: item[typelabel],
                    data: [item.percent_of_wall_time_lost_on_sky],
                    backgroundColor: colors[index]
                }));

                
               console.log("Pre isonly" ,byIssueType);

                if ( isOnlyAggregration)
                {
                    let totalFailureSum = byIssueType.reduce((sum, item) => sum + item.data[0], 0)
                    let RemainingBar =  {
                        type: 'bar',
                        label: 'Success ' + parseFloat(100-totalFailureSum).toFixed(3) ,
                        data: [100 - totalFailureSum],
                        backgroundColor: documentStyle.getPropertyValue('--green-500'),
                    }
                    let FailureBar =  {
                        type: 'bar',
                        label: 'Failure ' + parseFloat(totalFailureSum).toFixed(3),
                        data: [ totalFailureSum],
                        backgroundColor: documentStyle.getPropertyValue('--red-500'),
                    }
                    byIssueType = [FailureBar,RemainingBar]
                    console.log("with isOnlyAggregration", byIssueType);
                }

                
            return {
                labels: [title],
                datasets: byIssueType
            };
        }
    }

    function formatDateToYYYYMMDD(date) {
        const year = date.getFullYear();
        const month = (date.getMonth() + 1).toString().padStart(2, '0');
        const day = date.getDate().toString().padStart(2, '0');

        return `${year}-${month}-${day}`;
    }

    const generateReport = async () => {
        setIsLoading(true);
        const requestedFailureReport = await ReportService.getFailureReport(formatDateToYYYYMMDD(dates[0]) + "T00:00:00Z", formatDateToYYYYMMDD(dates[1]) + "T23:59:59Z")
        let filteredAndConvertedHistogram = Object.entries(requestedFailureReport.data.lost_time_histogram_on_sky)
    .filter(([date]) => !date.startsWith("2022"))
    .reduce((acc, [date, value]) => {
        acc[date] = value / 3600;
        return acc;
    }, {});

        requestedFailureReport.data.lost_time_histogram_on_sky = { ...filteredAndConvertedHistogram, ...HistoricData.lost_time_histogram_on_sky} 
        setFailureReport(requestedFailureReport?.data);
        setIsLoading(false);
    }





    useEffect(() => {
        const documentStyle = getComputedStyle(document.documentElement);
        const textColor = documentStyle.getPropertyValue('--text-color');
        const textColorSecondary = documentStyle.getPropertyValue('--text-color-secondary');
        const surfaceBorder = documentStyle.getPropertyValue('--surface-border');
        const options = {
            indexAxis: 'y',
            maintainAspectRatio: false,
            aspectRatio: 0.7,
            
            labels:["Observing Failure Percentage"],
            plugins: {
                title: { text: "Percentage Success/Failure", display: true },
                tooltips: {
                    mode: 'index',
                    intersect: false
                },
                legend: {
                    position: 'bottom',
                    align: 'end',
                    labels: {
                        color: textColor,
                        boxWidth: 10,

                    }
                }
            },
            scales: {
                x: {
                    barPercentage: 0.3,
                    categoryPercentage: 0.3,
                    ticks: {
                        color: textColorSecondary
                    },
                    grid: {
                        color: surfaceBorder
                    },
                },
                y: {
                    ticks: {
                        color: textColorSecondary,
                        callback: function (value) {
                            return value + '%'; // Add percentage symbol to the ticks
                        }
                    },
                    grid: {
                        color: surfaceBorder
                    }
                }
            }
        };
        setChartOptions(options);
    }, []);




    return <div className='routerpage'>
        <PageHeader title={'Failure Reports'} />

        <div className='FailureReport'>
            <span>Date Range&nbsp;</span>
            <Calendar value={dates} onChange={(e) => setDates(e.value)} selectionMode="range" readOnlyInput className='FailureCalender' />
            <Button label="Generate Failure Report" icon={checked} iconPos="right" disabled={checked.length == 0} onClick={() => generateReport()} />
            <Button label="Download Pdf" icon="pi pi-file-pdf" iconPos="right" disabled={!failureReport || isLoading} onClick={() => generatePdf()} />

            <a href={`data:text/json;charset=utf-8,${encodeURIComponent(JSON.stringify(failureReport))}`} download={`${moment(dates[0]).format("DD MMM YYYY")} - ${moment(dates[1]).format("DD MMM YYYY")} failrereport.json`} >
                <Button label="Download Json" icon="pi pi-file-export" iconPos="right" disabled={!failureReport || isLoading} />
            </a>
        </div>
        {isLoading && <ProgressBar mode="indeterminate" style={{ height: '6px' }} className='failureProgress'> </ProgressBar>}

        {failureReport && <>
            <div style={{ display: "flex", flexDirection: "row", marginTop: "10px",flexWrap:"wrap" }} >
                <div className='chartbox' style={{ display: "flex", flexDirection: "row" ,flexWrap:"wrap"}}>
                    <Chart type="pie" data={chartData_Issue_count} options={chartData_Issue_count_options} style={{ position: 'relative', width: '450px' }} id="chart1" />
                    <Chart type="pie" data={chartData_Issue_total_duration} options={chartData_Issue_total_duration_options} style={{ position: 'relative', width: '450px' }} id="chart2" />
                </div>
                <div className='chartbox' style={{ display: "flex", flexDirection: "row" ,flexWrap:"wrap"}}>
                    <Chart type="pie" data={chartData_sub_Issue_count} options={chartData_sub_Issue_count_options} style={{ position: 'relative', width: '450px' }} id="chart3" />
                    <Chart type="pie" data={chartData_sub_Issue_total_duration} options={chartData_sub_Issue_total_duration_options} style={{ position: 'relative', width: '450px' }} id="chart4" />            </div>
            </div>

            <div style={{ display: "flex", flexDirection: "row", flexWrap:"wrap" }} className='pdfPage'>
                <Chart type="bar" data={chartWallTimeMainData} options={chartOptions} style={{ position: 'relative', width: '400px', heigth: '100px' }} className='chartbox chartbar' id="chart8" />
                <Chart type="bar" data={chartWallTimeData} options={chartOptions} style={{ position: 'relative', width: '400px', heigth: '100px' }} className='chartbox ' id="chart5" />
                <Chart type="bar" data={chartWallTimeSubTypeData} options={chartOptions} style={{ position: 'relative', width: '400px', heigth: '100px'  }} className='chartbox' id="chart6" />

                <div style={{ width: '990px' }} className='chartbox' >
                    <div className='yaxesrotate'>hours</div>
                    <Chart type="bar" data={chartObservingTimeBarData} style={{ position: 'relative', width: '995px' }} id="chart7" />
                </div>
            </div>



            <DataTable value={failureReport?.failed_scheduling_units} className='chartbox' id="failuredatatable" pr_id_13="">
                <Column sortable key="name" field="name" header="name" ></Column>
                <Column sortable key="project" field="project" header="project" body={FailureBusinessTemplate.ProjectTemplate}></Column>
                <Column sortable key="status" field="status" header="status" ></Column>
                <Column sortable key="on_sky_start_time" field="on_sky_start_time" header="on sky start time" body={FailureBusinessTemplate.on_sky_start_time_Template} ></Column>
                <Column sortable key="on_sky_duration" field="on_sky_duration" header="on sky duration" body={FailureBusinessTemplate.on_sky_duration_Template}  ></Column>
                <Column sortable key="subtasks" field="subtasks" header="subtasks" body={FailureBusinessTemplate.subtasks_Template}></Column>
                <Column sortable key="issue_type" field="issue_type" header="issue_type" body={FailureBusinessTemplate.issues_Template}></Column>
                <Column sortable key="issue_subtype" field="issue_subtype" header="issue_subtype" body={FailureBusinessTemplate.issuessubtypes_Template}></Column>
            </DataTable></>}
    </div>



}



