import _ from 'lodash';
import React, {Component} from 'react';
import { Link } from 'react-router-dom';
import ViewTable from '../../components/ViewTable';
import UIConstants from '../../utils/ui.constants';
import DetailsSummary from "../Timeline/components/DetailsSummary";

export class TaskSummary extends Component {
    constructor(props) {
        super(props);
        this.state = {
            taskDetails: [],
            subtasks: []
        };
        this.closeTaskDets = this.closeTaskDets.bind(this);
        this.linkInspectionPlot = this.linkInspectionPlot.bind(this);
    }

    /**
     * Function to close the summary panel and call parent callback function to close.
     */
    closeTaskDets() {
        if(this.props.closeCallback) {
            this.props.closeCallback();
        }
    }

    redirectToTaskDetails = () => {
        if (!this.props.viewInNewWindow) {
            this.props.history.push(`/task/view/blueprint/${this.props.task.taskId}`);
        } else {
            window.open(`/task/view/blueprint/${this.props.task.taskId}`, '_blank');
        }
    }

    componentDidMount() {
        this.linkInspectionPlot();
    }

    componentDidUpdate(prevState) {
        if(!_.isEqual(prevState.taskDetails, this.props.taskDetails)) {
            this.linkInspectionPlot();
        }
    }

    linkInspectionPlot() {
        let subtasks = this.props.taskDetails.subtasks;
        subtasks.map(subtask => {
            
            if(subtask.subtask_type === 'observation') {
                subtask['links'] = ['Subtask ID'];
                subtask['linksURL'] = {
                    'Subtask ID': `${process.env.REACT_APP_SERVER_NAME}/inspect/HTML/${subtask.id}/`
                };
            }
        
        })
        this.setState({subtasks: subtasks})
    }

    render() {
        const taskDetails = this.props.taskDetails;
        const task = this.props.task;

        let summaryFields = [
            {labelName:"Name", value: taskDetails.name, formatTime: false},
            {labelName:"Task Id", value: taskDetails.id, formatTime: false},
            {labelName:"Scheduling Unit Id", value: taskDetails.suId, formatTime: false},
            {labelName:"Project", value: taskDetails.project, formatTime: false},
            {labelName:"Start Time", value: taskDetails.start_time, formatTime: true},
            {labelName:"End Time", value: taskDetails.end_time, formatTime: true},
            {labelName:"Status", value: taskDetails.status, formatTime: false}]

        if (taskDetails.status === 'unschedulable') {
            summaryFields.push({labelName:"Unschedule Reason", value: taskDetails.unschedulable_reason, formatTime: false})
        }

        return (
            <React.Fragment>
                { task &&
                    <div className="p-grid timeline-details-pane" style={{marginTop: '10px'}}>
                        <div className="timeline-details-header">
                            <h6 className="col-lg-10 col-sm-10">Task Details</h6>
                            <Link onClick={this.redirectToTaskDetails} title="View Full Details"><i className="fa fa-eye"></i></Link>
                            <Link to={this.props.location?this.props.location.pathname:"/su/timelineview/week"} onClick={this.closeTaskDets} title="Close Details"><i className="pi pi-times"></i></Link>
                        </div>
                        <DetailsSummary fields={summaryFields}/>
                        {taskDetails.status === 'unschedulable' && 
                            <>
                               <div className="col-5"><label>Unschedule Reason:</label></div>
                               <div className="col-7">{taskDetails.unschedulable_reason}</div>
                            </>
                        }
                        <div className="col-12 task-summary">
                            <label>Subtasks:</label>
                            <ViewTable 
                                data={this.state.subtasks} 
                                defaultcolumns={[{subtask_type: "Subtask Type", id: {name:"Subtask ID"}, on_sky_start_time:{name:"Start Time", format:UIConstants.CALENDAR_DATETIME_FORMAT}, on_sky_stop_time:{name:"End Time", format:UIConstants.CALENDAR_DATETIME_FORMAT}, state_value: "Status",  unschedulable_reason: {name:"Unschedulable Reason"}},]}
                                optionalcolumns={[{actionpath: "actionpath"}]}
                                columnclassname={[{"Subtask ID": "filter-input-50","Subtask Type":"filter-input-75", "Start Time": "filter-input-75", "End Time": "filter-input-75",
                                                    "Status": "filter-input-75"}]}
                                defaultSortColumn= {[{id: "ID", desc: false}]}
                                tablename="timeline_task_subtasklist"
                                showTopTotal={false}
                                showGlobalFilter={false}
                                showColumnFilter={false}
                                allowColumnSelection={false}
                            />
                        </div>                    
                    </div>
                }
            </React.Fragment>
        );
    }
}

export default TaskSummary;