import { render, act } from "@testing-library/react";
import TimelineItemPopover, { getFieldsHtml } from "./TimelineItemPopover";
import moment from "moment";
import { removeReact18ConsoleErrors } from "../../../utils/test.helper";
import UIConstants from "../../../utils/ui.constants";
import renderer from 'react-test-renderer';
removeReact18ConsoleErrors()

describe("TimelineItemPopover", () => {
    let mouseOverItemSchedule = {
        type: "SCHEDULE",
        status: "scheduled",
        suId: "123",
        project: "X-men",
        name: "Wolverine",
        scheduler: "dynamic",
        suStartTime: moment.utc("2023-07-27T08:00:00Z", 'YYYY-MM-DD'),
        suStopTime: moment.utc("2023-07-27T10:00:00Z", 'YYYY-MM-DD'),
        antenna_set: "LBA",
        stations: {
            groups: 2,
            counts: 8,
        },
        on_sky_duration: 2.5,
        duration: 3.5,
    };

    test("Render type SCHEDULE for a schedulable item", () => {

        let pagecontent
        act(async () => {
            pagecontent = render(<TimelineItemPopover mouseOverItem={mouseOverItemSchedule} />);
        });



        const renderedFields = pagecontent.container.querySelectorAll(".entry");
        const fieldLabels = ["Project:", "Scheduling Unit:", "Scheduler:", "Start Time:", "End Time:", "Antenna Set:", "Stations:", "Status:", "Exposure Time:", "Duration:"];
        const fieldValues = [mouseOverItemSchedule.project, mouseOverItemSchedule.name, mouseOverItemSchedule.scheduler,
        mouseOverItemSchedule.suStartTime.format(UIConstants.CALENDAR_DATETIME_FORMAT), mouseOverItemSchedule.suStopTime.format(UIConstants.CALENDAR_DATETIME_FORMAT), mouseOverItemSchedule.antenna_set,
        `${mouseOverItemSchedule.stations.groups} : ${mouseOverItemSchedule.stations.counts}`, mouseOverItemSchedule.status, mouseOverItemSchedule.on_sky_duration.toString(), mouseOverItemSchedule.duration.toString(),
        ];

        renderedFields.forEach((field, index) => {
            expect(field.querySelector(".col-5")?.textContent, `test failed for ${fieldLabels[index]}`).toBe(fieldLabels[index]);
            expect(field.querySelector(".col-7")?.textContent, `test failed for ${fieldLabels[index]}`).toBe(fieldValues[index]);
            expect(field.querySelector(".col-5")).toHaveClass(`su-${mouseOverItemSchedule.status}-icon`);
        });
    });

    test("Render a RESERVATION", () => {
        const mouseOverItem = {
            type: "RESERVATION",
            bgColor: "#ffffff",
            color: "#000000",
            name: "Grass mowing",
            desc: "By Fanna Lautenbach",
            activity_type: "Very intense",
            stations: {
                groups: 2,
                counts: 8,
            },
            project: "Maintenance",
            displayStartTime: moment("2023-07-27T12:00:00Z"),
            displayEndTime: moment("2023-07-27T14:00:00Z"),
            duration: 2,
            planned: true,
        };

        let pagecontent
        act(async () => {
            pagecontent = render(<TimelineItemPopover mouseOverItem={mouseOverItem} />);
        });

        const renderedFields = pagecontent.container.querySelectorAll(".entry");
        const fieldLabels = ["Name:", "Description:", "Type:", "Stations:", "Project:", "Start Time:", "End Time:", "Duration:", "Planned:"];
        const fieldValues = [mouseOverItem.name, mouseOverItem.desc, mouseOverItem.activity_type, `${mouseOverItemSchedule.stations.groups} : ${mouseOverItemSchedule.stations.counts}`, mouseOverItem.project,
        mouseOverItem.displayStartTime.format(UIConstants.CALENDAR_DATETIME_FORMAT), mouseOverItem.displayEndTime.format(UIConstants.CALENDAR_DATETIME_FORMAT),
        mouseOverItem.duration.toString(), "Yes"
        ];

        renderedFields.forEach((field, index) => {
            expect(field.querySelector(".col-5").textContent).toBe(fieldLabels[index]);
            expect(field.querySelector(".col-7").textContent).toBe(fieldValues[index]);
        });
    });


    test("Render a RESERVATION without stations", () => {
        const mouseOverItem = {
            type: "RESERVATION",
            bgColor: "#ffffff",
            color: "#000000",
            name: "Grass mowing",
            desc: "By Fanna Lautenbach",
            activity_type: "Very intense",
            project: "Maintenance",
            real_start_time: moment("2023-07-27T12:00:00Z"),
            real_end_time: moment("2023-07-27T14:00:00Z"),
            duration: 2,
            planned: true,
        };
        let pagecontent
        act(async () => {
            pagecontent = render(<TimelineItemPopover mouseOverItem={mouseOverItem} />);
        });

        const renderedFields = pagecontent.container.querySelectorAll(".entry");
        const fieldLabels = ["Name:", "Description:", "Type:", "Stations:", "Project:", "Start Time:", "End Time:", "Duration:", "Planned:"];
        const fieldValues = [mouseOverItem.name, mouseOverItem.desc, mouseOverItem.activity_type, "-", mouseOverItem.project,
        mouseOverItem.real_start_time.format(UIConstants.CALENDAR_DATETIME_FORMAT), mouseOverItem.real_end_time.format(UIConstants.CALENDAR_DATETIME_FORMAT),
        mouseOverItem.duration.toString(), "Yes"
        ];

        renderedFields.forEach((field, index) => {
            expect(field.querySelector(".col-5").textContent).toBe(fieldLabels[index]);
            expect(field.querySelector(".col-7").textContent).toBe(fieldValues[index]);
        });
    });

    test('Render a TASK (same asSTATION_TASK)', () => {
        const mouseOverItem = {
            type: "TASK",
            projectId: "123",
            project: "X-men",
            name: "Wolverine",
            taskId: "456",
            taskName: "Do damage",
            controlId: "1337",
            start_time: moment.utc("2023-07-27T08:00:00Z"),
            end_time: moment.utc("2023-07-27T10:00:00Z"),
            band: "HBA",
            stations: {
                groups: 2,
                counts: 8,
            },
            status: "in-progress",
            duration: "2 hours",
        };

        const { container } = render(<TimelineItemPopover mouseOverItem={mouseOverItem} />);

        const renderedFields = container.querySelectorAll(".entry");
        const fieldLabels = ["Project:", "Scheduling Unit:", "Task Id:", "Task Name:", "Control Id:", "Start Time:", "End Time:", "HBA/LBA:", "Stations:", "Status:", "Duration:"];
        const fieldValues = [mouseOverItem.project, mouseOverItem.name, mouseOverItem.taskId, mouseOverItem.taskName, mouseOverItem.controlId,
        mouseOverItem.start_time.format(UIConstants.CALENDAR_DATETIME_FORMAT), mouseOverItem.end_time.format(UIConstants.CALENDAR_DATETIME_FORMAT),
        mouseOverItem.band, `${mouseOverItemSchedule.stations.groups} : ${mouseOverItemSchedule.stations.counts}`, mouseOverItem.status, mouseOverItem.duration
        ];

        renderedFields.forEach((field, index) => {
            expect(field.querySelector(".col-5").textContent).toBe(fieldLabels[index]);
            expect(field.querySelector(".col-7").textContent).toBe(fieldValues[index]);
            expect(field.querySelector(".col-5")).toHaveClass(`su-${mouseOverItem.status}-icon`);
        });
    })

    test.each([null, undefined, {}])('Render nothing on invalid input: %s', (mouseOverItem) => {
        const { queryByText } = render(<TimelineItemPopover mouseOverItem={mouseOverItem} />);

        expect(queryByText("Reservation Overview")).not.toBeInTheDocument();
        expect(queryByText("Scheduling Unit Overview")).not.toBeInTheDocument();
    });
});

describe("getFieldsHtml", () => {
    const fields = [
        { label: "Project:", value: "X-men" },
        { label: "Scheduling Unit:", value: "Wolverine" },
    ];

    // This is a snapshot test now. This means it validates against a
    // Correct taken snapshot.
    test('Render fields with the correct label, value, and classNames', () => {

        const status = "scheduled";
        const style = { color: "blue" };
        const page = renderer
            .create(getFieldsHtml(fields, status, style))
            .toJSON();
        expect(page).toMatchSnapshot();
    })


    test("Render fields without status and style", () => {
        const page = renderer
            .create(getFieldsHtml(fields))
            .toJSON();
        expect(page).toMatchSnapshot();
    })
});