import  { Component } from 'react';
import { Button } from 'primereact/button';
import SunEditor from 'suneditor-react';
import 'suneditor/dist/css/suneditor.min.css'; // Import Sun Editor's CSS File
import { Checkbox } from 'primereact/checkbox';
import WorkflowService from '../../services/workflow.service';
import { Link } from 'react-router-dom';
import TaskAssigner from './task.assigner';
import { SystemEventList } from '../SystemEvent/system.event.list';

class PIverification extends Component {
    constructor(props) {
        super(props);
        this.state = {
            content: '',
            comment: '',
            showEditor: false,
            pi_accept: true,
            operator_accept: true,
            quality_within_policy: true,
            sos_accept_show_pi: true,
            currentWorkflowTask:{},
        };
        this.user =JSON.parse(localStorage.getItem("user"));
        this.Next = this.Next.bind(this);
        this.handleChange = this.handleChange.bind(this);
        this.onChangePIComment = this.onChangePIComment.bind(this);
        this.getPIVerificationDetails = this.getPIVerificationDetails.bind(this);
        this.assignTaskCB = this.assignTaskCB.bind(this);
    }

    async componentDidMount() {
        let currentWorkflowTask = null;
        let processPermission = null;
        if (this.props.readOnly) {
            this.getPIVerificationDetails();

        }   else {
            currentWorkflowTask = await this.props.getCurrentTaskDetails();
            processPermission = await this.props.processPermission();
        }
        const operatorResponse = await WorkflowService.getQAReportingTo(this.props.process.qa_reporting_to);

        
        let  quality_within_policy = null;
        let  sos_accept_show_pi =null;
        let report = null;
        if (this.props.process.qa_reporting_sos) { // only call this service when we do haven an actual report
              const sosResponse = await WorkflowService.getQAReportingSOS(this.props.process.qa_reporting_sos);
              quality_within_policy = sosResponse.quality_within_policy;
              sos_accept_show_pi = sosResponse.sos_accept_show_pi;
              report = sosResponse.sos_report;
        }

        this.setState({
            operator_accept: operatorResponse.operator_accept,
            content:report ,
            quality_within_policy: quality_within_policy,
            sos_accept_show_pi: sos_accept_show_pi,
            assignTo: this.props.readOnly?this.props.workflowTask.owner:(currentWorkflowTask?currentWorkflowTask.owner_id:null), 
            currentWorkflowTask: currentWorkflowTask,
            processPermission: processPermission,
        });
    }

    async getPIVerificationDetails() {
        if (!this.props.process.pi_verification) {
            return
        }
        const piVerificationResponse = await WorkflowService.getQAPIverification(this.props.process.pi_verification);
        this.setState({
            comment: piVerificationResponse.pi_report,
            pi_accept: piVerificationResponse.pi_accept,
        });
    }
    
    
     /**
     * Method will trigger on change of operator report sun-editor
     */
    handleChange(e) {
        if (e === '<p><br></p>') {
            this.setState({ content: '' });
            return;
        }
        this.setState({ content: e });
    }

    /**
     * Method will trigger on click save button
     * here onNext props coming from parent, where will handle redirection to other page
     */
    async Next() {
        let currentWorkflowTask = await this.props.getCurrentTaskDetails();
        const promise = [];
        promise.push(WorkflowService.updateQA_Perform(this.props.id,{"pi_report": this.state.comment, "pi_accept": this.state.pi_accept}));
        Promise.all(promise).then(async(responses) => {
            if (responses.indexOf(null)<0) {
                // After completing the current task, get the next task triggered in the workflow and assign project role user for the next task.
                currentWorkflowTask = await this.props.getCurrentTaskDetails();
                if (currentWorkflowTask && !currentWorkflowTask.owner_id) {
                    await WorkflowService.updateAssignTo(currentWorkflowTask.id, "project_role=friend_of_project", { owner: this.state.assignTo });
                }
                this.props.onNext({ report:this.state.content, pireport: this.state.comment, pi_accept: this.state.pi_accept});
            }   else {
                this.props.onError('error', 'Unable to proceed', 'Please clear your browser cookies and try again');
            }
        });
    }

     /**
     * Method wiill triigger on change of pi report sun-editor
     */
    onChangePIComment(a) {
        if (a === '<p><br></p>') {
            this.setState({ comment: '' });
            return;
        }
        this.setState({comment: a  });
    }

    /**
     * Callback function passed to the Task Assigner and is called after assigning or reassigning task owner.
     */
    async assignTaskCB() {
        let currentWorkflowTask = await this.props.getCurrentTaskDetails();
        this.setState({currentWorkflowTask: currentWorkflowTask, 
                        assignTo: currentWorkflowTask.owner_id, reassign: false});
    }
    
    render() {
        return (
            <>
             <div>
                    <div className={`p-fluid-grid workflow-wrap`}>
                        <div className="p-field p-grid workflow-wrap" style={{ paddingLeft: '-10px' }}>
                            <label htmlFor="assignTo" className="col-lg-2 col-md-2 col-sm-12">Assign To</label>
                            <div className="col-lg-3 col-md-3 col-sm-12" data-testid="assignTo" >
                            {/* Display the assigned owner of the task */}
                            {this.state.assignTo && 
                                <>{!this.props.readOnly ? this.state.currentWorkflowTask.owner_username: this.props.workflowTask.owner_username }
                                    {/* Allow to edit the owner if the task is not yet completed */}
                                    {!this.props.readOnly && !this.state.reassign &&
                                        <>
                                            {this.state.currentWorkflowTask.editPermissions
                                            ? <Link onClick={() =>this.setState({reassign: true})} style={{marginLeft: '10px'}}>
                                                <i className="pi pi-pencil"></i>
                                              </Link>
                                            : <Link style={{ cursor: 'default', pointerEvents: "none", color: 'grey', marginLeft: '10px'}} onClick={ ()=>this.setState({reassign: true})}>
                                                <i className="pi pi-pencil"></i>
                                              </Link>
                                            }
                                        </>       
                                    }
                                </>
                            }
                            {/* Display the task assigner if owner is not assigned or to reassign to other user */}
                            {(!this.state.assignTo || this.state.reassign) && this.state.currentWorkflowTask &&
                                <TaskAssigner currentWorkflowTask={this.state.currentWorkflowTask} disableAssignToMe = {this.state.currentWorkflowTask.owner_username === this.user.name? true: false}
                                        projectRoles={this.props.projectRoles} callback={this.assignTaskCB} />
                            }
                            </div>
                        </div>
                        <div className="p-fluid-grid">
                            <label htmlFor="operatorReport" style={{ padding: '2px' }} >Operator Report</label>
                            <div className="col-lg-12 col-md-12 col-sm-12"></div>
                            {this.state.showEditor && <SunEditor setDefaultStyle="min-height: 250px; height: auto;" enableToolbar={true}
                                onChange={this.handleChange}
                                setContents={this.state.content}
                                setOptions={{
                                    buttonList: [
                                        ['undo', 'redo', 'bold', 'underline', 'fontColor', 'table', 'link', 'image', 'video', 'italic', 'strike', 'subscript',
                                            'superscript', 'outdent', 'indent', 'fullScreen', 'showBlocks', 'codeView', 'preview', 'print', 'removeFormat']
                                    ]
                                }}
                            />}
                             <div className="operator-report" style={{ padding: '5px' }} dangerouslySetInnerHTML={{ __html: this.state.content }}></div>
                        </div>
                        <div className="p-grid workflow-wrap" style={{ padding: '10px' }}>
                            <label htmlFor="piReport" >PI Report {!this.props.readOnly && <span style={{color:'red'}}>*</span>}</label>
                            <div className="col-lg-12 col-md-12 col-sm-12"></div>
                            <SunEditor setDefaultStyle="min-height: 150px; height: auto;" enableToolbar={!this.props.readOnly}
                                setContents={this.state.comment}
                                disable={this.props.readOnly}
                                onChange={this.onChangePIComment}
                                setOptions={{
                                    buttonList: [
                                        ['undo', 'redo', 'bold', 'underline', 'fontColor', 'table', 'link', 'image', 'video', 'italic', 'strike', 'subscript',
                                            'superscript', 'outdent', 'indent', 'fullScreen', 'showBlocks', 'codeView', 'preview', 'print', 'removeFormat']
                                    ]
                                }} />
                            </div>
                            <div className="p-field p-grid workflow-wrap" style={{marginTop: '1em'}}>
                                <label htmlFor="systemissues" >System Events</label>
                            </div> 
                            <div>
                                <SystemEventList schedulingUnit={this.props.schedulingUnit}/>
                            </div> 
                            <div className="p-col-12" style={{marginTop: '1em', paddingLeft: '0rem'}}>
                                <Checkbox disabled inputId="opertator_accept" checked={this.state.operator_accept} />
                                <label htmlFor="operator_accept"  style={{paddingLeft:"5px"}}>The data quality adheres to policy (Operator evaluation)</label>
                                <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                <Checkbox disabled inputId="quality_within_policy" checked={this.state.quality_within_policy} />
                                <label htmlFor="qualityPolicy"  style={{paddingLeft:"5px"}}>The data quality adheres to policy (SDCO evaluation)</label>
                                <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                <Checkbox disabled inputId="sos_accept_show_pi" checked={this.state.sos_accept_show_pi} />
                                <label htmlFor="sdcoAccept" style={{paddingLeft:"5px"}}>SDCO Accepts</label>
                                <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                <Checkbox disabled = {this.props.readOnly} inputId="piAccept" checked={this.state.pi_accept} onChange={e => this.setState({ pi_accept: e.checked })} />
                                <label htmlFor="piAccept" style={{paddingLeft:"5px"}} >As PI / contact author I accept this data</label>
                            </div>
                            {!this.props.readOnly && <div className="p-grid" style={{ marginTop: '20px' }}>
                                <div className="btn-bar">
                                <Button disabled={this.state.content &&  this.state.comment && this.state.processPermission && this.state.currentWorkflowTask.editPermissions && this.state.currentWorkflowTask.owner_username === this.user.name? false: true } 
                                    label="Next" className="p-button-primary" icon="pi pi-check" onClick={ this.Next } />
                                </div>
                                <div className="btn-bar">
                                    <Button label="Cancel" className="p-button-danger" icon="pi pi-times"  style={{ width : '90px' }}
                                        onClick={ ()=>this.props.onCancel()} />
                                </div>
                            </div>}
                            {!this.props.readOnly && 
                                <>
                                    {this.state.currentWorkflowTask.editPermissions && this.state.currentWorkflowTask.owner_username === this.user.name? 
                                    <span></span>: <span style={{color: 'red'}}>* You can only save and proceed next if you are assigned to this workflow step</span>
                                    }
                                </>
                            }
                            <div>
                                {this.state.comment ? 
                                <span></span>: <span style={{color: 'red'}}>* Comments required</span>
                                }
                            </div>
                        </div>  
                </div>
            </>
        )
    }
}
export default  PIverification;
