import  React, { Component } from 'react';
import _ from 'lodash';
import moment from 'moment';
import {TriStateCheckbox} from 'primereact/tristatecheckbox';

import PageHeader from '../../layout/components/PageHeader';
import AppLoader from "../../layout/components/AppLoader";
import ViewTable from '../../components/ViewTable';

import WorkflowService from  '../../services/workflow.service';
import ScheduleService from  '../../services/schedule.service';
import UIConstants from '../../utils/ui.constants';
import UtilService from '../../services/util.service';
import AuthUtil from '../../utils/auth.util';
import AccessDenied from '../../layout/components/AccessDenied';
import { Dropdown } from 'primereact/dropdown';
import Websocket from 'react-websocket';

class WorkflowList extends Component{
    lsKeySortColumn = 'SortDataWorkflowList';
    defaultSortColumn = [];
    SU_BLUEPRINT_EXPAND = 'draft,draft.scheduling_set,draft.scheduling_set.project,draft.observation_strategy_template';

    constructor(props) {
        super(props);
        this.setToggleBySorting();
        this.workflowUIAttr = UtilService.localStore({ type: 'get', key: 'WORKFLOW_UI_ATTR' }) || {};
        this.state={
            userrole: {},
            ftAssignstatus: 'All',
            activeWorkflow: null,
            showActiveStage: 'Show Active Workflow Process',
            workflowList: [],
            columnOrders: [ 
                "Scheduling Unit Name", 
                "Scheduling Unit ID",
                "Scheduling Unit Status",
                "Project",
                "Assigned To",
                "Current Workflow Stage",
                "Updated At",
                "Accepted",
            ],
            defaultcolumns: [{
                suName: {
                    name: "Scheduling Unit Name",
                },
                su: {
                    name: "Scheduling Unit ID",
                    format: UIConstants.CALENDAR_TIME_FORMAT
                },
                status: {
                    name: "Scheduling Unit Status",
                    showTag: true,
                    tagClass: 'su-$dynamic$' //$dynamic$ will be replace with cell value
                },
                project: {
                    name:"Project",
                },
                owner_username: {
                    name: "Assigned To"
                },
                lastTaskName:  {
                    name:"Current Workflow Stage",
                    filter: "multiselect-filter"
                },
                created: {
                    name:"Updated At",
                    filter:"flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT
                },
                results_accepted: {
                    name: "Accepted",
                    filter: 'switch'
                },
            }],
            optionalcolumns:  [{ 
                actionpath:"actionpath",
            }],
            columnclassname: [{
                "Scheduling Unit ID":"filter-input-50",
                "Updated At":"filter-input-150",}],
        };
        this.wsSUId = 0;
        this.filterQry = '';
        this.orderBy = '';
        this.limit = 10;
        this.currentPageSize = 10;
        this.offset =  0;
        this.totalPage = 0;
        this.statusList = [];
        this.pageUpdated = true;
        this.schedulingUnitList = null;
        this.workflowList = null;
        this.workflowTasksList = [];
        this.tableInstance = null;
        this.schedulingUnitList = [];
        this.redirectionMap = {
            'wait scheduled': 'Waiting To Be Scheduled',
            'wait observing': 'Scheduled',
            // 'wait processed': 'Scheduled',
            'qa reporting to': 'QA Reporting (TO)',
            'qa reporting sos': 'QA Reporting (SDCO)',
            'pi verification': 'PI Verification',
            'decide acceptance': 'Decide Acceptance',
            'wait ingest done': 'Ingesting',
            'decide unpin data': 'Unpin Data',
            'end': 'Done'
         };
        // this.filterslist=[{'name': 'Assigned to me', 'code': 'Assigned to me'},{'name': 'Unassigned', 'code':'Unassigned'},{'name': 'Participated', 'code':'Participated'}];
        this.filterslist=[{'name': 'All', 'code': 'All'},{'name': 'Assigned to me', 'code': 'Assigned to me'},{'name': 'Participated', 'code':'Participated'},{'name': 'Participated and Unassigned', 'code':'Participated and Unassigned'},{'name': 'Unassigned', 'code':'Unassigned'}];
        this.notActiveStages = ['done', 'waiting to be scheduled', 'scheduled' ];
        this.close = this.close.bind(this);
        this.getFilterOptions = this.getFilterOptions.bind(this);
        this.fetchTableData = this.fetchTableData.bind(this);
        this.seTableInstanceRef = this.seTableInstanceRef.bind(this);
        this.onConnect = this.onConnect.bind(this);
        this.handleData = this.handleData.bind(this);
        this.refreshWorkflowList = this.refreshWorkflowList.bind(this);
    }
 
    async componentDidMount() {
        const permission = await AuthUtil.getUserPermissionByModule('project');
        await this.getFilterColumns();
        this.setState({userrole: permission});
        this.pageUpdated = true;
        this.filterTableData();
        this.setToggleBySorting();
    }

    toggleBySorting = (sortData) => {
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: sortData });
    }

    setToggleBySorting() {
        let sortData = null;
        if (this.lsKeySortColumn) {
            sortData = UtilService.localStore({ type: 'get', key: this.lsKeySortColumn });
        }
        if(sortData && !_.isEmpty(sortData)){
            if(Object.prototype.toString.call(sortData) === '[object Array]'){
                this.defaultSortColumn = sortData;
            }
            else{
                this.defaultSortColumn = [{...sortData}];
            }
        }
        else {
            this.defaultSortColumn = [{id: "Updated At", desc: false}];
        }
        this.defaultSortColumn = this.defaultSortColumn || [];
        if (this.lsKeySortColumn) {
            UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: [...this.defaultSortColumn] });
        }
    }

    storeUIAttr(key, value){
        this.workflowUIAttr[key] = value
        UtilService.localStore({ type: 'set', key: 'WORKFLOW_UI_ATTR', value: this.workflowUIAttr })  
    }

    /**
     * Get View Table instance
     * @param {*} instanceRef 
     */
     seTableInstanceRef(instanceRef) {
        this.tableInstance = instanceRef;
    }

    /**
     * Prepare Workflow Task data
     * @param {Boolean} status - If workflow_task has to filtered by status
     * @param {String} filterQry - Query for filtering worflow_task
     * @param {*} orderBy - Sort order
     * @param {*} limit - No. of records per page
     * @param {*} offset - Starting record
     */
    async prepareWorkflowTasklist(filterQry, orderBy, limit, offset) {
        if(this.state.ftAssignstatus === "Participated and Unassigned") {
            filterQry+= '&status=NEW&'
        }
        else {
            filterQry+= '&status=NEW&status=ASSIGNED&'
        }
        let response = await WorkflowService.getActiveWorkflowTasksWithFilter(filterQry, orderBy, limit, offset);
        this.totalPage = response.data.count;
        this.workflowTasksList = response.data.results;
        let workflowList = [];
        let processList =_.map(this.workflowTasksList, 'process');
        let workFlowProcessList = await WorkflowService.getWorkflowProcess(processList.join(','));
        let suList = _.map(workFlowProcessList, 'su');
        if (this.wsSUId === 0) {
            this.schedulingUnitList = await ScheduleService.getWorkflowSU(suList.join(','));
        }   else {
            let schedulingUnitList = await ScheduleService.getWorkflowSU([this.wsSUId]);
            this.wsSUId = 0;
            if (schedulingUnitList && schedulingUnitList.length>0) {
                _.remove(this.schedulingUnitList, function (su) { return su.id === schedulingUnitList[0].id});
                this.schedulingUnitList.push(schedulingUnitList[0]);
            }
        }
        if (this.workflowTasksList) {
            for(const workflowTask of this.workflowTasksList) {
                let workflowProcess = _.find(workFlowProcessList, {id:workflowTask.process})
                let schedulingUnit = _.find(this.schedulingUnitList, {id : workflowProcess.su})
                if(schedulingUnit && workflowTask.flow_task) {
                    workflowTask['suName'] = schedulingUnit.name;
                    workflowTask['su'] = workflowProcess.su;
                    workflowTask['project'] = schedulingUnit.draft.scheduling_set.project_id;
                    workflowTask['status'] = schedulingUnit.status;
                    workflowTask['created'] =  moment(workflowTask['created']).format(UIConstants.UTC_DATE_TIME_FORMAT);
                    workflowTask['lastTaskName'] = this.redirectionMap[workflowTask.flow_task.toLowerCase()];
                    workflowTask['owner'] = workflowTask.owner? workflowTask.owner : '';
                    workflowTask['owner_username'] = workflowTask.owner_username? workflowTask.owner_username : '';
                    workflowTask['actionpath'] = `/schedulingunit/${workflowProcess.su}/workflow`;
                    workflowTask['results_accepted'] = schedulingUnit.results_accepted;
                    workflowList.push(workflowTask);
                }
            }
        }
        this.workflowUIAttr=  UtilService.localStore({ type: 'get', key: 'WORKFLOW_UI_ATTR' }); 
        let ftAssignstatus = this.workflowUIAttr.ftAssignstatus;
        let activeWorkflow =  this.workflowUIAttr.activeWorkflow || this.state.activeWorkflow;
        let showActiveStage = this.workflowUIAttr.showActiveStage ; 
        this.setState({ftAssignstatus: ftAssignstatus?ftAssignstatus: 'All' , activeWorkflow:activeWorkflow, showActiveStage:showActiveStage,workflowList: workflowList, isLoading: false, loadingStatus: false  });
    }

    /**
     * Go back to previous page
     */
    close() {
        this.props.history.push('/');
    }

     /**
     * Get Option-list values for Select Dropdown filter in 'Viewtable'
     * @param {String} id : Column id
     * @returns Array of options
     */
    getFilterOptions(id) {
        let options = null;
        if(id && id === 'Current Workflow Stage') {
            options = _.map(UIConstants.CURRENT_WORKFLOW_STAGE, stage=> {return {name: stage, value: stage}});
        }
        return options;
    }

    /**
     * To get filterable columns from the default colunns
     */
    async getFilterColumns() {
        this.columnMap = [];
        let tmpDefaulColumns = _.cloneDeep(this.state.defaultcolumns[0]);
        tmpDefaulColumns = this.getAPIFilter(tmpDefaulColumns);
        await this.setState({tmpDefaulcolumns: [tmpDefaulColumns], columnMap: this.columnMap})
    }

    /**
     * To set filterable columns from the API options. But currently it is manually set. Later it needs to be set from API options
     * @param {Object} columnDef - Column definitions with original and display names
     * @returns 
     */
    getAPIFilter(columnDef) {
        const defaultColKeys = Object.keys(columnDef);
        defaultColKeys.forEach(key => {
            let tmpColMap = {};
            let tempKey = key;
            tmpColMap['orgField'] = tempKey;
            tmpColMap['tmpField'] = tempKey;
            if(columnDef[key]) {
                tmpColMap['displayName'] = columnDef[key]['name'];
            }
            this.columnMap.push(tmpColMap);
            if (key === 'owner_username') {
                columnDef[key]['disableSortBy'] = true;
                columnDef[key]['disableFilters'] = false;
            } else if(key === 'lastTaskName') {
                columnDef[key]['disableSortBy'] = false;
                columnDef[key]['disableFilters'] = false;
            } else if(key === 'created') {
                columnDef[key]['disableSortBy'] = false;
                columnDef[key]['disableFilters'] = false;
            } 
        })
        return columnDef;
    }
    
    /**
     * Filter the table data with drop down selected
     * @param {String} value - Drop Down filter value
     */
    async assignStatusChangeEvent(value) {
        await this.setState({'ftAssignstatus': value});
        this.storeUIAttr('ftAssignstatus', value)
        this.filterTableData()
    }

    /**
     * Filter the table data with checkbox value, it will fetch active workflow stagerows if the checkbox selected
     * @param {Boolena} value 
     */
    async setWorkflowActiveState(value) {       
        let showActiveStage = '';
        if(value === null) {
            showActiveStage = 'Show Active Workflow Process';
        }   else if(value === true) {
            showActiveStage = 'Show Inactive Workflow Process';
        }   else {
            showActiveStage = 'Show All Workflow Process';
        }
        await this.setState({'activeWorkflow': value, showActiveStage: showActiveStage});
        
        this.storeUIAttr('activeWorkflow', value)
        
        this.storeUIAttr('showActiveStage', showActiveStage )
    }

    /**
     * Filter table data if any changes in filter Dropdown or Checkbox.
     */
    async filterTableData() {
        let workflowList = [];
        let activeWorkflow = this.state.activeWorkflow;
        let filters = UtilService.localStore({ type: 'get', key: "workflow_process_list" });
        let assignStatus = UtilService.localStore({ type: 'get', key: "WORKFLOW_UI_ATTR" }).ftAssignstatus;
        // Get current user details
        let userDetails = localStorage.getItem('user');
        if (userDetails) {
            userDetails = JSON.parse(userDetails);
        }
        // Remove all owner filters and add only the required one
        const ownerFilters = ['ownerUserNameisNull', 'participated', 'Assigned To'];
        filters = _.filter(filters, filter => { return ownerFilters.indexOf(filter.id) < 0});
        if(assignStatus === 'Assigned to me') {
            filters.push({id: 'Assigned To', value:userDetails.name});
        }   else if(assignStatus === "Unassigned") {
            filters.push({id: 'ownerUserNameisNull', value: true});
        }   else if (assignStatus === "Participated" || assignStatus === "Participated and Unassigned") {
            let processList = [];
            let taskParticipated = await WorkflowService.getActiveWorkflowTasksWithFilter('owner_username='+userDetails.name);
            processList = _.union(processList, _.map(taskParticipated.data.results, 'process'));
            filters.push({id: 'participated', value: processList});
        }   else if (!filters) {
                filters = [];
        }
        UtilService.localStore({ type: 'set', key: 'workflow_process_list', value: filters}); 
        this.setTableProperty(filters, 'Filter');
        await this.fetchTableData(null);

        //TODO: comment  for lines 165-166  AND 170-187. Even these lines can be commented out and added when we have Filter options available
        /*const currentuser = userDetails.name;
        let ftAssignstatus = this.state.ftAssignstatus; */
        for( const workflow of this.state.workflowList) {
            var ftAssigned = true;
            /*if(ftAssignstatus.length>0 && ftAssignstatus.length <3) {
                let assignToMe = false;
                if( (ftAssignstatus[0] === 'Assigned to me' || (ftAssignstatus.length === 2 && ftAssignstatus[1] === 'Assigned to me')) && workflow.owner === currentuser) {
                    assignToMe = true;
                }
                let unAssigned = false;
                if( (ftAssignstatus[0] === 'Unassigned' || (ftAssignstatus.length === 2 && ftAssignstatus[1] === 'Unassigned')) && workflow.owner_username === '') {
                    unAssigned = true;
                }
                let isParticipated = false;
                if( (ftAssignstatus[0] === 'Participated' || (ftAssignstatus.length === 2 && ftAssignstatus[1] === 'Participated')) 
                    &&  _.includes(workflow.owner_username.split(','), currentuser)) {
                        isParticipated = true;
                }
                if(!assignToMe && !unAssigned && !isParticipated) {
                    ftAssigned = false;
                }
            } */
            let activestage = false;
            if (activeWorkflow === null) {
                activestage = true;
            }   else if(activeWorkflow === true) {
                activestage = _.includes(this.notActiveStages, workflow.lastTaskName.toLowerCase())? false: true;
            }   else if(activeWorkflow === false) {
                activestage = _.includes(this.notActiveStages, workflow.lastTaskName.toLowerCase())? true: false;
            }
            if(ftAssigned && activestage) {
                workflowList.push(workflow);
            }
        }
        this.setState({filteredWorkflowProcessList: workflowList});
    }

    /**
     * Set View table Properties from parent component
     * @param {*} value - Value to set
     * @param {*} propsType - Property type like to set the Filter, Columns Header
     */
    setTableProperty(value, propsType) {
        if (this.tableInstance) {
            if (propsType === 'Filter') {
                this.tableInstance.state.filters = value;
            }
        }
    }

    /**
     * Fetch data from server side - while doing pagination, filtering, sorting
     * @param {Table State} Table props state 
     * @returns 
     */
    async fetchTableData(state) {
        this.filterQry = '';
        this.orderBy = '';
        this.pageUpdated = true;
        this.setState({loadingStatus:true});
        let filters = UtilService.localStore({ type: 'get', key: "workflow_process_list" });
        let userDetails = localStorage.getItem('user');
        if (userDetails) {
            userDetails = JSON.parse(userDetails);
        }
        if(filters.length > 0) {
            for(const filter of filters) {
                if (filter.id === 'Assigned To') {
                    if(filter.value) {
                        this.filterQry += 'owner_username='+filter.value+'&';
                    }
                    
                } else if (filter.id === 'Current Workflow Stage') {
                    let qryString = [];
                    for ( const value of filter.value) {
                        for (const [redirectionMapKey, redirectionMapValue] of Object.entries(this.redirectionMap)) {
                            if(value === redirectionMapValue) {
                                qryString.push(redirectionMapKey);
                            }
                        }
                    }
                    this.filterQry += 'flow_task=' + qryString.join(",") +'&';
                } else if ((filter.id === 'Updated At') && filter.value !== '') {
                    const values = filter.value;
                    if (values[0] && values[0] !== '') {
                        this.filterQry += 'created_after='+ moment(new Date(values[0])).format("YYYY-MM-DD HH:mm:ss&");
                    }
                    if (values[1] && values[1] !== '') {
                        this.filterQry += 'created_before='+moment(new Date(values[1])).format("YYYY-MM-DD HH:mm:ss&");
                    }
                } else if ((filter.id === 'ownerUserNameisNull') && filter.value !== '') {
                    this.filterQry += "owner_username_isnull="+filter.value+'&';
                } else if ((filter.id === 'participated') && filter.value !== '') {
                    if(filter.value) {
                        for (const process of filter.value) {
                            this.filterQry += "process="+process+'&';
                        }
                    }
                }
            }
        }
        let sortBy = state &&state.sortBy?state.sortBy[0]:null;
        if (sortBy) {
            this.setState({defaultSortColumn: [sortBy]}); 
            UtilService.localStore({ type: 'set', key: this.lsKeySortColumn ,value:sortBy});
            let columnDetails = _.find(this.state.columnMap, {displayName:sortBy.id});
            if(columnDetails) {
                if(columnDetails.orgField === "lastTaskName") {
                    this.orderBy = 'ordering='+((sortBy.desc)?'-':'')+'flow_task';
                } else {
                    this.orderBy = 'ordering='+((sortBy.desc)?'-':'')+columnDetails.orgField;
                }
            }
        }
        this.filterQry = this.filterQry.substring(0,this.filterQry.length-1);
        this.currentPageSize = (state && state.pageSize) ? state.pageSize : this.currentPageSize;
        this.offset = (state && state.pageIndex) ? state.pageIndex*this.currentPageSize : 0;
        await this.prepareWorkflowTasklist(this.filterQry, this.orderBy, this.currentPageSize,  this.offset);
        return [this.state.workflowList, this.totalPage]; 
    }
 
    /**
     * Function to call wnen websocket is connected
     */
     onConnect() {
        try{
            console.log("WS Opened");
            const userDets = localStorage.getItem("user");
            if (userDets) {
                this.websocket.sendMessage(JSON.stringify({"token": JSON.parse(userDets).websocket_token}));
                console.log("Auth token submitted");
            }
        }catch (err) {
            console.log('err',err)
        }
    }

    /**
     * Function to call when websocket is disconnected
     */
    onDisconnect() {
        console.log("WS Closed")
    }

    /**
     * Handles the message received through websocket
     * @param {String} data - String of JSON data
     */
    handleData(data) {
        if (data) {
            console.log('data ', data)
            const jsonData = JSON.parse(data);
            this.refreshWorkflowList(jsonData); 
        }
    }

    /**
     * Refresh the Workflow list list
     * @param {*} jsonData 
     */
    async refreshWorkflowList(jsonData) {
        switch (jsonData.object_type) {
            case 'scheduling_unit_blueprint': {
                this.wsSUId = jsonData.object_details.id;
                this.prepareWorkflowTasklist(this.filterQry, this.orderBy, this.currentPageSize, this.offset);
                break;
            }             
            default: { break; }
        }
    }

    render() {
            const {project} = this.state.userrole;
        return (
            <React.Fragment>
                {project && project.list ?
                    <>
                        <PageHeader location={this.props.location} title={'Workflow - List'} 
                                    actions={[]}/>
                        <div style={{marginTop: '15px'}}>
                        {this.state.isLoading ? <AppLoader/> : 
                            <>
                                <div className="p-select " style={{position: 'relative', marginLeft: '27em', marginTop: '-2em'}}>
                                    <div className="p-field p-grid workflow-wrap">
                                        <div className="col-lg-4 col-md-3 col-sm-12 ms-height">
                                            <span className="p-float-label">
                                                <Dropdown inputId="workflowoverview" id="workflowoverview" optionLabel="name" optionValue="code" filter={true}
                                                    tooltip="Show workflows those are 'Assigned to me' / 'Unassigned'/ 'I Participated'" tooltipOptions={this.tooltipOptions}
                                                    value={this.state.ftAssignstatus}
                                                    options={this.filterslist} 
                                                    onChange={(e) => {this.assignStatusChangeEvent(e.value)}} 
                                                    style={{width: '10em', marginLeft: '0.5em'}}
                                                />
                                                <label htmlFor="workflowoverview" >Show Workflow </label>
                                            </span>
                                        </div>
                                        <div>
                                            <label htmlFor="cb1" className="p-checkbox-label">Active</label>
                                            <TriStateCheckbox  
                                            onChange={(e) => {this.setWorkflowActiveState(e.value)}}  
                                            value={true}
                                            disabled={true} 
                                            style={{marginLeft: '5px', marginTop: '5px'}}
                                            tooltip={this.state.showActiveStage} tooltipOptions={this.tooltipOptions}
                                            ></TriStateCheckbox>
                                        </div>
                                    </div>
                                </div>
                                <ViewTable 
                                    data={this.state.workflowList} 
                                    totalPage={this.totalPage}
                                    defaultcolumns={this.state.tmpDefaulcolumns ? this.state.tmpDefaulcolumns : this.state.defaultcolumns}
                                    optionalcolumns={this.state.optionalcolumns}
                                    columnclassname={this.state.columnclassname}
                                    columnOrders={this.state.columnOrders}
                                    defaultSortColumn={this.defaultSortColumn}
                                    showaction="true"
                                    keyaccessor="su"
                                    paths={this.state.paths}
                                    tablename="workflow_process_list"
                                    toggleBySorting={(sortData) => this.toggleBySorting(sortData)}
                                    lsKeySortColumn={this.lsKeySortColumn}
                                    callBackFunction = {this.fetchTableData}
                                    showGlobalFilter={false} showTopPagination={true}
                                    loadingStatus={this.state.loadingStatus}
                                    showFilterOption={this.getFilterOptions} //Callback function to provide inputs for option-list in Select Dropdown filter
                                    pageUpdated={this.pageUpdated}
                                    storeFilter={true}
                                    tableInstanceRef={this.seTableInstanceRef}
                                    showTopTotal={true}
                                />
                                {!this.state.isLoading &&
                                    <Websocket ref={websocket => this.websocket = websocket} url={process.env.REACT_APP_WEBSOCKET_URL} 
                                        onOpen={this.onConnect} onMessage={this.handleData} onClose={this.onDisconnect} />}
                            </>
                            }
                        </div>
                    </>: <AccessDenied/> }
            </React.Fragment>
        );
    }
}
export default WorkflowList;