import UtilService from "./util.service";
import {getStore, updateStore} from "./store.helper";

jest.mock("./util.service", () => ({
    localStore: jest.fn()
}));

const storeKey = 'Marvel';

describe('getStore', () => {

    it('should return an empty object when UtilService.localStore returns falsy', () => {
        UtilService.localStore.mockReturnValue(undefined);

        const result = getStore(storeKey);

        expect(result).toEqual({});
        expect(UtilService.localStore).toHaveBeenCalledWith({type: 'get', key: storeKey});
    });

    it('should return the value from UtilService.localStore when it returns truthy', () => {
        const mockValue = {key: 'value'};
        UtilService.localStore.mockReturnValue(mockValue);

        const result = getStore(storeKey);

        expect(result).toEqual(mockValue);
        expect(UtilService.localStore).toHaveBeenCalledWith({type: 'get', key: storeKey});
    });
});

describe('updateStore', () => {
    const initialStore = {key: storeKey, value: {hero: 'x-men'}};
    it('should update the store when updatedObject is a valid object with a store that is already filled', () => {
        const updatedObject = {villain: 'magneto'};

        updateStore(storeKey, initialStore.value, updatedObject);
        // we should use the store it's value here for testing, as the initialStore itself also holds the surrounding key :)
        const expectedUpdatedStoreValues = {...initialStore.value, ...updatedObject};

        expect(UtilService.localStore).toHaveBeenCalledWith({
            type: 'set',
            key: storeKey,
            value: expectedUpdatedStoreValues
        });
    });

    it('should update the store when updatedObject is a valid object using ES6 annotation with on initialized store', () => {
        updateStore(storeKey, initialStore.value, {['hero']: 'batman'});

        expect(UtilService.localStore).toHaveBeenCalledWith({
            type: 'set',
            key: storeKey,
            value: {hero: 'batman'}
        });
    });

    it('should not update the store when updatedObject is not a valid object', () => {
        const updatedObject = 'notAnObject';

        updateStore(storeKey, initialStore.value, updatedObject);

        expect(UtilService.localStore).not.toHaveBeenCalled();
    });
});