import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act, fireEvent } from '@testing-library/react';
import { MemoryRouter } from "react-router-dom";
import { Login } from "../authenticate/login";



function MakePrimaryMock() {
    let mock = new MockAdapter(axios);
    mock.onPost("/api/token-auth/",200, { "non_field_errors":["Unable to log in with provided credentials."]});
    return mock
}


function MakeGoodMock(){
    let mock = new MockAdapter(axios);
    mock.onPost("/api/token-auth/",200, {"token":"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"});
}
describe('Login', () => {

    it('Renders Login', async () => {
        MakePrimaryMock();
        let component = await  RenderLogin();
        expect(component.asFragment()).toMatchSnapshot();
        component.unmount();

    });


    it('System Login Bad  Login', async () => {
        MakePrimaryMock();
        const component  = await DoLogin();
        expect(component.asFragment()).toMatchSnapshot("Bad Login");
        component.unmount();
    });


    it('System Login Good Login', async () => {
        MakeGoodMock()
        const component  = await DoLogin();
        expect(component.asFragment()).toMatchSnapshot("Good Login");
        component.unmount();
    });

    it('KeyCloack Login Redirect', async () => {
        MakeGoodMock()
        let component = await RenderLogin();
        const submitloginbutton = component?.container.querySelector('#KeyCloakLoginButton');
        await act(async () => { fireEvent.click(submitloginbutton); });

        
        expect(component.asFragment()).toMatchSnapshot("KeyCloak Redirect");
        component.unmount();
    });

    

    

    
});

async function DoLogin() {
    let component = await RenderLogin();
    
    let aswitch = component?.queryAllByRole('switch');
    await act(async () => { fireEvent.click(aswitch[0]); });
    expect(component?.asFragment()).toMatchSnapshot();

    const submitloginbutton = component?.container.querySelector('#systemloginbutton');
    expect(submitloginbutton).toBeDisabled();

    const usernameinputElement = component?.container.querySelector('#username');
    await act(async () => { fireEvent.change(usernameinputElement, { target: { value: 'abcdef' } }); });
    expect(submitloginbutton).toBeDisabled();


    const passwordinputElement = component?.container.querySelector('#password');
    await act(async () => { fireEvent.change(passwordinputElement, { target: { value: 'hijklmnop' } }); });

    expect(submitloginbutton).not.toBeDisabled();
    await act(async () => { fireEvent.click(submitloginbutton); });
    return component;
}

async function RenderLogin () {
    
    let component
    await act(async () => { component = render(<MemoryRouter><Login  history={{}} location="/login/" /></MemoryRouter>) });
    return component
}


