

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act, fireEvent, waitFor } from '@testing-library/react';

import ProjectReportMain from "../routes/Report/project.report.main";
import project_limit from '../__mocks__/project.limit.json'
import cycle from '../__mocks__/cycle.json'
import resourcetype from '../__mocks__/resourcetype.json'
import project_quota from '../__mocks__/projectquota.json'
import report from '../__mocks__/report.json'

import { MemoryRouter } from "react-router-dom"



function MakePrimaryMock() {
    let mock = new MockAdapter(axios);
    jest.mock('chart.js', () => ({
        ChartJS: jest.fn(),
    }));

    mock.onGet("/api/project/?limit=150").reply(200, project_limit);
    mock.onGet("/api/cycle").reply(200, cycle);
    mock.onGet("/api/resource_type").reply(200, resourcetype);
    mock.onGet("/api/project_quota").reply(200, project_quota);
    mock.onGet("/api/project/Calibration/report/").reply(200, report);
    mock.onGet("/api/project/Calibration/friend").reply(200, ["Man Kind"]);
    
    
    
}



describe('Project Report Render Without Contents', () => {

    it('Project Report Render Without Contents correctly without data', async () => {
        let component
        await act(async () => { component = render(<ProjectReportMain />); });
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();
    });
});

describe('Project Report Render With Contents', () => {
    it('Project Report  renders correctly when clicking generate', async () => {

        MakePrimaryMock();
        let component

        let cinstance
        await act(async () => { component = render(<MemoryRouter><ProjectReportMain ref={(instance) => { cinstance = instance; }}  /></MemoryRouter>); });
        const componentclean = component.asFragment();
        expect(componentclean).toMatchSnapshot("After  initial Load");
        const inputElement = component.container.querySelector('[role="combobox"]');
        fireEvent.change(inputElement, { target: { value: 'Calibration' } });
        cinstance.searchProjects({ query: ""})
        cinstance.searchProjects({ query: "Calibration"});
        let projects = cinstance.SearchForProjects("Calibration");
        act( () => {  cinstance.selectProjects(projects);});

        const componentCalibration = component.asFragment();
        expect(componentCalibration).toMatchSnapshot("After  Setting Project");
        const buttonElement = component.container.querySelector('.p-button-generate');
        await waitFor(() => {
            expect(buttonElement).not.toHaveAttribute('disabled');
          }, {timeout:5000}  );

           act( () => { fireEvent.click(buttonElement);});

        const componentSnapshotAfterClick = component.asFragment();
        expect(componentSnapshotAfterClick).toMatchSnapshot("After click Generate");
           await waitFor(() => {
            expect(component.container.querySelector('.failureProgress')).toBeNull();
        }, { timeout: 5000 });

        await waitFor(() => {
            const element = component.container.querySelector('.report-div');
            expect(element).toBeInTheDocument();
          });

        const componentSnapshotAfterDataLoading = component.asFragment();
        expect(componentSnapshotAfterDataLoading).toMatchSnapshot("After data Loading");


        const csvanchorElement = component.container.querySelector('[title="Download Report Data in CSV format"]');
        act( () => { fireEvent.click(csvanchorElement);});
        
        


        await waitFor(() => {
            expect(component.container.querySelector('.p-autocomplete-loader')).toBeNull();
        }, { timeout: 5000 });

        const componentSnapshotAfterCsv = component.asFragment();
        expect(componentSnapshotAfterCsv).toMatchSnapshot("After CSV Generation");


         cinstance.clearAll();
        
         await waitFor(() => {
            expect(component.container.querySelector('.p-autocomplete-loader')).toBeNull();
        }, { timeout: 5000 });


        const componentSnapshotAfterClear = component.asFragment();
        expect(componentSnapshotAfterClear).toMatchSnapshot("After Clear ALl");
        
        


        component.unmount();
    });
});






