

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act } from '@testing-library/react';
import { TaskList}     from "../routes/Task/list";
import taskdraftoptions from '../__mocks__/task.draft.options.json'
import taskblueprintoptions from '../__mocks__/task.blueprint.options.json'
import subtasktemplates  from '../__mocks__/subtask.template.json'
import subtasktemplateslimit20 from '../__mocks__/subtask.template.limit20.json'
import taskdraftexpanded from '../__mocks__/task.draft.expanded.json'
import taskblueprintexpanded from '../__mocks__/task.blueprint.expanded.json'

import { MockAxiosSecurity  } from './axiosmocks.'
import { MemoryRouter } from "react-router-dom";


import PermissionStackUtil from "../authenticate/permission.stack.handler";
import { giveConsistentNodes } from "../utils/test.helper";



function MakePrimaryMock() {
    let mock = new MockAdapter(axios);

    mock.onOptions("/api/task_draft/").reply(200, taskdraftoptions);
    mock.onOptions("/api/task_blueprint/").reply(200, taskblueprintoptions);
    
    mock.onGet("/api/subtask_template/").reply(200, subtasktemplates);
    mock.onGet("/api/subtask_template/?limit=20&offset=50").reply(200, subtasktemplateslimit20);
    mock.onGet("/api/task_draft/?expand=task_blueprints,task_blueprints,scheduling_unit_draft,scheduling_unit_draft.scheduling_set&fields=id,name,description,short_description,task_type,url,duration,relative_start_time,relative_stop_time,created_at,updated_at,task_blueprints.name,task_blueprints.id,task_blueprints_ids,tags,specifications_template_id,scheduling_unit_draft_id,scheduling_unit_draft.id,scheduling_unit_draft.name,scheduling_unit_draft.scheduling_set.project,scheduling_unit_draft.scheduling_set.project_id&ordering=scheduling_unit_draft&limit=10&offset=0").reply(200, taskdraftexpanded);
    mock.onGet("/api/task_blueprint/?expand=subtasks,draft,scheduling_unit_blueprint,draft.scheduling_unit_draft,draft.scheduling_unit_draft.scheduling_set&fields=id,name,description,short_description,status,unschedulable_reason,task_type,url,on_sky_start_time,on_sky_stop_time,process_start_time,process_stop_time,duration,relative_start_time,relative_stop_time,obsolete_since,created_at,updated_at,draft_id,tags,specifications_template_id,scheduling_unit_blueprint_id,scheduling_unit_blueprint.id,scheduling_unit_blueprint.name,draft.scheduling_unit_draft.scheduling_set.project,draft.scheduling_unit_draft.scheduling_set.project_id,subtasks.id,subtasks.primary,subtasks.total_nr_of_dataproducts,subtasks.total_size_of_dataproducts,subtasks.total_size_of_dataproducts_on_filesysem&ordering=scheduling_unit_draft&limit=10&offset=0").reply(200, taskblueprintexpanded);
    mock.onGet("/api/task_draft/640/?fields=name&limit=1").reply(200, {"name":"Ingest"},{ "access-control-allow-methods": "GET, POST, PUT, PATCH, DELETE, copy, create_task_blueprint, create_task_blueprint_and_subtasks, predecessors" });
    mock.onGet("/api/task_draft/601/?fields=name&limit=1").reply(200, {"name":"Ingest"},{ "access-control-allow-methods": "GET, POST, PUT, PATCH, DELETE, copy, create_task_blueprint, create_task_blueprint_and_subtasks, predecessors" });
    mock.onGet("/api/task_blueprint/484/?fields=name&limit=1").reply(200, {"name":"Ingest"},{ "access-control-allow-methods": "GET, cancel, copy_to_new_draft, create_subtasks, mark_as_obsolete, predecessors, restart_when_on_hold, schedule_independent_subtasks, successors, system_events" });
    mock.onGet("/api/task_blueprint/493/?fields=name&limit=1").reply(200, {"name":"Ingest"},{ "access-control-allow-methods": "GET, cancel, copy_to_new_draft, create_subtasks, mark_as_obsolete, predecessors, restart_when_on_hold, schedule_independent_subtasks, successors, system_events" });
    

    return mock

}

describe('Task List ', () => {

    it('Task List  renders correctly without proper security', async () => {
        MakePrimaryMock();
        let component
        await act(async () => { component = render(<MemoryRouter><TaskList  history={{}} location="/task/" /></MemoryRouter>) });
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });

    it('Task List  correctly with data', async () => {
        let {  component } = await RenderTaskList();
        const componentSnapshot = giveConsistentNodes(component.container.outerHTML)
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });
    it('Task List correctly with blueprint data', async () => {
        let { taskListInstance, component } = await RenderTaskList();
        await act( async() => {
            await taskListInstance.changeTaskType('Blueprint');
        });
        const componentSnapshot = giveConsistentNodes(component.container.outerHTML);
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();
      });
});



async function RenderTaskList() {
    MockAxiosSecurity(MakePrimaryMock());
    await PermissionStackUtil.getPermissions(true);
    let component;
    let taskListInstance;
    await act(async () => {
        component = render(<MemoryRouter><TaskList history={{}} location="/task/" ref={(instance) => { taskListInstance = instance; } } /></MemoryRouter>);
    });
    return { taskListInstance, component };
}

