import _ from 'lodash';
import {parseTemplatesToCorrectJSONFormat} from "../services/service.helper";
const templates = (require("../../../tmss_webapp/src/__mocks__/task_templates.json")).templates;
      
const TaskServiceMock= {
    taskTemplates: [
      {
        "id": 1,
        "url": "http://localhost:3000/api/task_template/1",
        "created_at": "2022-02-04T05:10:04.178899",
        "description": "This schema defines the parameters to setup a dataproduct(s) cleanup task.",
        "name": "cleanup",
        "schema": {
          "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/1#",
          "type": "object",
          "title": "cleanup",
          "$schema": "http://json-schema.org/draft-06/schema#",
          "version": 1,
          "required": [],
          "properties": {},
          "description": "This schema defines the parameters to setup a dataproduct(s) cleanup task."
        },
        "tags": [],
        "type": "http://localhost:3000/api/task_type/cleanup",
        "type_value": "cleanup",
        "updated_at": "2022-02-04T05:10:04.178911",
        "version": 1
      },
      {
        "id": 2,
        "url": "http://localhost:3000/api/task_template/2",
        "created_at": "2022-02-04T05:10:04.240678",
        "description": "This schema combines the target and beamformer observation task schema's for parallel observing.",
        "name": "parallel target and beamforming observation",
        "schema": {
          "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20target%20and%20beamforming%20observation/1#",
          "type": "object",
          "title": "parallel target and beamforming observation",
          "$schema": "http://json-schema.org/draft-06/schema#",
          "version": 1,
          "required": [
            "target",
            "beamforming"
          ],
          "properties": {
            "target": {
              "type": "object",
              "default": {},
              "required": [
                "station_groups",
                "antenna_set",
                "filter",
                "SAPs",
                "duration",
                "correlator"
              ],
              "properties": {
                "QA": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/1#/definitions/QA",
                  "default": {}
                },
                "SAPs": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/SAPs",
                  "default": [
                    {}
                  ],
                  "minItems": 1
                },
                "filter": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/filter",
                  "default": "HBA_110_190"
                },
                "duration": {
                  "$id": "#duration",
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/1/#/definitions/timedelta",
                  "title": "Duration",
                  "default": 300,
                  "minimum": 1,
                  "description": "Duration of this observation (seconds)"
                },
                "tile_beam": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pointing/1/#/definitions/pointing",
                  "title": "Tile beam",
                  "default": {},
                  "description": "HBA only"
                },
                "correlator": {
                  "type": "object",
                  "title": "Correlator Settings",
                  "default": {},
                  "required": [
                    "channels_per_subband",
                    "topocentric_frequency_correction",
                    "integration_time",
                    "storage_cluster"
                  ],
                  "properties": {
                    "storage_cluster": {
                      "enum": [
                        "CEP4",
                        "DragNet"
                      ],
                      "type": "string",
                      "title": "Storage cluster",
                      "default": "CEP4",
                      "description": "Cluster to write output to"
                    },
                    "integration_time": {
                      "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/1/#/definitions/timedelta",
                      "title": "Integration time",
                      "default": 1,
                      "minimum": 0.1,
                      "description": "Desired integration period (seconds)"
                    },
                    "channels_per_subband": {
                      "enum": [
                        8,
                        16,
                        32,
                        64,
                        128,
                        256,
                        512,
                        1024
                      ],
                      "type": "integer",
                      "title": "Channels/subband",
                      "default": 64,
                      "minimum": 8,
                      "description": "Number of frequency bands per subband"
                    },
                    "topocentric_frequency_correction": {
                      "type": "boolean",
                      "title": "Topocentric Frequency Correction",
                      "default": false,
                      "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation."
                    }
                  },
                  "additionalProperties": false
                },
                "antenna_set": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/antenna_set",
                  "default": "HBA_DUAL"
                },
                "station_groups": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1#/definitions/station_groups",
                  "default": [
                    {
                      "stations": [
                        "CS002",
                        "CS003",
                        "CS004",
                        "CS005",
                        "CS006",
                        "CS007"
                      ],
                      "max_nr_missing": 1
                    }
                  ]
                }
              },
              "description": "This subschema defines the parameters to setup the target observation part."
            },
            "beamforming": {
              "type": "object",
              "default": {},
              "required": [
                "station_groups",
                "antenna_set",
                "filter",
                "tile_beam",
                "SAPs",
                "duration",
                "beamformers",
                "processing"
              ],
              "properties": {
                "SAPs": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/SAPs",
                  "default": [
                    {}
                  ],
                  "minItems": 0
                },
                "filter": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/filter",
                  "default": "HBA_110_190"
                },
                "duration": {
                  "$id": "#duration",
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/1/#/definitions/timedelta",
                  "title": "Duration",
                  "default": 300,
                  "minimum": 1,
                  "description": "Duration of this observation (seconds)"
                },
                "tile_beam": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pointing/1/#/definitions/pointing",
                  "title": "Tile beam",
                  "default": {},
                  "description": "HBA only"
                },
                "processing": {
                  "type": "object",
                  "title": "Processing",
                  "default": {},
                  "required": [
                    "beamformer_ppf"
                  ],
                  "properties": {
                    "beamformer_ppf": {
                      "type": "boolean",
                      "title": "Use PPF when beamforming",
                      "default": false,
                      "description": "Use a PPF for the channellisation for beamforming. If not, an FFT is used."
                    }
                  },
                  "description": "General processing parameters",
                  "additionalProperties": false
                },
                "antenna_set": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/antenna_set",
                  "default": "HBA_DUAL"
                },
                "beamformers": {
                  "type": "array",
                  "items": {
                    "type": "object",
                    "title": "Beamformer",
                    "default": {},
                    "required": [
                      "name"
                    ],
                    "properties": {
                      "name": {
                        "type": "string",
                        "title": "Name",
                        "default": "_beamformer_name_",
                        "minLength": 1,
                        "description": "Beamformer name, used for identification purposes."
                      },
                      "coherent": {
                        "type": "object",
                        "title": "Coherent Tied-Array Beams",
                        "default": {},
                        "required": [
                          "SAPs",
                          "settings"
                        ],
                        "properties": {
                          "SAPs": {
                            "type": "array",
                            "items": {
                              "type": "object",
                              "required": [
                                "name",
                                "tabs"
                              ],
                              "properties": {
                                "name": {
                                  "type": "string",
                                  "title": "SAP name",
                                  "default": "_SAP_to_beamform_",
                                  "minLength": 1,
                                  "description": "Name of the SAP to beamform"
                                },
                                "tabs": {
                                  "type": "array",
                                  "items": {
                                    "type": "object",
                                    "title": "Tied-Array Beam",
                                    "default": {},
                                    "required": [
                                      "pointing",
                                      "relative"
                                    ],
                                    "properties": {
                                      "pointing": {
                                        "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pointing/1/#/definitions/pointing",
                                        "default": {}
                                      },
                                      "relative": {
                                        "type": "boolean",
                                        "title": "Relative to SAP",
                                        "default": false,
                                        "description": "Angular offset applied to the TAB pointing"
                                      }
                                    },
                                    "headerTemplate": "TAB {{ self.index }}",
                                    "additionalProperties": false
                                  },
                                  "title": "Tied-Array Beams",
                                  "default": [],
                                  "description": "Tied-array beams to form",
                                  "additionalItems": false
                                },
                                "subbands": {
                                  "$ref": "#/definitions/subband_selection",
                                  "default": {}
                                },
                                "tab_rings": {
                                  "type": "object",
                                  "title": "Tied-Array Rings",
                                  "default": {},
                                  "properties": {
                                    "count": {
                                      "type": "integer",
                                      "title": "Number of rings",
                                      "default": 0,
                                      "maximum": 11,
                                      "minimum": 0
                                    },
                                    "width": {
                                      "type": "number",
                                      "title": "Ring width",
                                      "default": 0.01,
                                      "minimum": 0,
                                      "description": "Distance between pointings."
                                    }
                                  },
                                  "description": "Rings of TABs around the center of the beam.",
                                  "additonalProperties": false
                                }
                              },
                              "additionalProperties": false
                            },
                            "title": "SAPs",
                            "default": [],
                            "minItems": 0,
                            "description": "Which SAPs in the observation to beamform.",
                            "additionalItems": false
                          },
                          "settings": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/1#/definitions/stokes_settings",
                            "default": {}
                          }
                        },
                        "additionalProperties": false
                      },
                      "flys eye": {
                        "type": "object",
                        "title": "Fly's Eye Settings",
                        "default": {},
                        "required": [
                          "enabled"
                        ],
                        "properties": {
                          "enabled": {
                            "type": "boolean",
                            "title": "Enable Fly's Eye",
                            "default": false
                          },
                          "settings": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/1#/definitions/stokes_settings",
                            "default": {}
                          }
                        },
                        "description": "Produce beams containing the individual station signals",
                        "additionalProperties": false
                      },
                      "incoherent": {
                        "type": "object",
                        "title": "Incoherent Tied-Array Beams",
                        "default": {},
                        "required": [
                          "settings",
                          "SAPs"
                        ],
                        "properties": {
                          "SAPs": {
                            "type": "array",
                            "items": {
                              "type": "object",
                              "default": {},
                              "required": [
                                "name",
                                "subbands"
                              ],
                              "properties": {
                                "name": {
                                  "type": "string",
                                  "title": "SAP name",
                                  "default": "_SAP_to_beamform_",
                                  "minLength": 1,
                                  "description": "Name of the SAP to beamform"
                                },
                                "subbands": {
                                  "$ref": "#/definitions/subband_selection",
                                  "default": {}
                                }
                              },
                              "additionalProperties": false
                            },
                            "title": "SAPs",
                            "default": [],
                            "minItems": 0,
                            "description": "Which SAPs in the observation to create incoherent TABs for (empty list = all).",
                            "additionalItems": false
                          },
                          "settings": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/1#/definitions/stokes_settings",
                            "default": {}
                          }
                        },
                        "additionalProperties": false
                      },
                      "station_groups": {
                        "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1#/definitions/station_groups",
                        "default": [
                          {
                            "stations": [
                              "CS001",
                              "CS002",
                              "CS003",
                              "CS004",
                              "CS005",
                              "CS006",
                              "CS007",
                              "CS011",
                              "CS013",
                              "CS017",
                              "CS021",
                              "CS024",
                              "CS026",
                              "CS028",
                              "CS030",
                              "CS031",
                              "CS032",
                              "CS301",
                              "CS302",
                              "CS401",
                              "CS501"
                            ],
                            "max_nr_missing": 1
                          }
                        ],
                        "minItems": 1,
                        "description": "While observing, COBALT will beamform on the intersection of all stations in this list and the used stations in the observation. So, specifying all possible stations here means that all observation-stations are used. Specifying a small subset here means that only the observing-stations in this small list are used. By default we let COBALT beamform on the Core stations."
                      }
                    },
                    "headerTemplate": "Beamformer {{ self.index }}",
                    "additionalProperties": false
                  },
                  "title": "Beamformers",
                  "default": [
                    {}
                  ],
                  "minItems": 1,
                  "additionalItems": false
                },
                "station_groups": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1#/definitions/station_groups",
                  "default": [
                    {
                      "stations": [
                        "CS002",
                        "CS003",
                        "CS004",
                        "CS005",
                        "CS006",
                        "CS007"
                      ],
                      "max_nr_missing": 1
                    }
                  ]
                }
              },
              "description": "This subschema defines the parameters for an observation that forms tied-array beams in COBALT."
            }
          },
          "definitions": {
            "subband_selection": {
              "type": "object",
              "title": "Subband selection",
              "default": {},
              "required": [
                "method"
              ],
              "properties": {
                "list": {
                  "type": "array",
                  "items": {
                    "type": "integer",
                    "title": "Subband",
                    "maximum": 511,
                    "minimum": 0
                  },
                  "title": "Subset selection",
                  "default": [],
                  "minItems": 0,
                  "description": "If method is 'select subset', only beamform these subbands, and only if they occur in the SAP.",
                  "additionalItems": false
                },
                "method": {
                  "enum": [
                    "copy",
                    "largest continuous subset",
                    "select subset"
                  ],
                  "type": "string",
                  "title": "Method",
                  "default": "copy",
                  "description": "How to select the subbands to beam form"
                }
              },
              "additionalProperties": false
            }
          },
          "description": "This schema combines the target and beamformer observation task schema's for parallel observing."
        },
        "tags": [],
        "type": "http://localhost:3000/api/task_type/observation",
        "type_value": "observation",
        "updated_at": "2022-02-04T05:10:04.240692",
        "version": 1
      },
      {
        "id": 3,
        "url": "http://localhost:3000/api/task_template/3",
        "created_at": "2022-02-04T05:10:04.245231",
        "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
        "name": "beamforming observation",
        "schema": {
          "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/1#",
          "type": "object",
          "title": "beamforming observation",
          "$schema": "http://json-schema.org/draft-06/schema#",
          "default": {},
          "version": 1,
          "required": [
            "station_groups",
            "antenna_set",
            "filter",
            "tile_beam",
            "SAPs",
            "duration",
            "beamformers",
            "processing"
          ],
          "properties": {
            "SAPs": {
              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/SAPs",
              "default": [
                {}
              ],
              "minItems": 0
            },
            "filter": {
              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/filter",
              "default": "HBA_110_190"
            },
            "duration": {
              "$id": "#duration",
              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/1/#/definitions/timedelta",
              "title": "Duration",
              "default": 300,
              "minimum": 1,
              "description": "Duration of this observation (seconds)"
            },
            "tile_beam": {
              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pointing/1/#/definitions/pointing",
              "title": "Tile beam",
              "default": {},
              "description": "HBA only"
            },
            "processing": {
              "type": "object",
              "title": "Processing",
              "default": {},
              "required": [
                "beamformer_ppf"
              ],
              "properties": {
                "beamformer_ppf": {
                  "type": "boolean",
                  "title": "Use PPF when beamforming",
                  "default": false,
                  "description": "Use a PPF for the channellisation for beamforming. If not, an FFT is used."
                }
              },
              "description": "General processing parameters",
              "additionalProperties": false
            },
            "antenna_set": {
              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/antenna_set",
              "default": "HBA_DUAL"
            },
            "beamformers": {
              "type": "array",
              "items": {
                "type": "object",
                "title": "Beamformer",
                "default": {},
                "required": [
                  "name"
                ],
                "properties": {
                  "name": {
                    "type": "string",
                    "title": "Name",
                    "default": "_beamformer_name_",
                    "minLength": 1,
                    "description": "Beamformer name, used for identification purposes."
                  },
                  "coherent": {
                    "type": "object",
                    "title": "Coherent Tied-Array Beams",
                    "default": {},
                    "required": [
                      "SAPs",
                      "settings"
                    ],
                    "properties": {
                      "SAPs": {
                        "type": "array",
                        "items": {
                          "type": "object",
                          "required": [
                            "name",
                            "tabs"
                          ],
                          "properties": {
                            "name": {
                              "type": "string",
                              "title": "SAP name",
                              "default": "_SAP_to_beamform_",
                              "minLength": 1,
                              "description": "Name of the SAP to beamform"
                            },
                            "tabs": {
                              "type": "array",
                              "items": {
                                "type": "object",
                                "title": "Tied-Array Beam",
                                "default": {},
                                "required": [
                                  "pointing",
                                  "relative"
                                ],
                                "properties": {
                                  "pointing": {
                                    "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pointing/1/#/definitions/pointing",
                                    "default": {}
                                  },
                                  "relative": {
                                    "type": "boolean",
                                    "title": "Relative to SAP",
                                    "default": false,
                                    "description": "Angular offset applied to the TAB pointing"
                                  }
                                },
                                "headerTemplate": "TAB {{ self.index }}",
                                "additionalProperties": false
                              },
                              "title": "Tied-Array Beams",
                              "default": [],
                              "description": "Tied-array beams to form",
                              "additionalItems": false
                            },
                            "subbands": {
                              "$ref": "#/definitions/subband_selection",
                              "default": {}
                            },
                            "tab_rings": {
                              "type": "object",
                              "title": "Tied-Array Rings",
                              "default": {},
                              "properties": {
                                "count": {
                                  "type": "integer",
                                  "title": "Number of rings",
                                  "default": 0,
                                  "maximum": 11,
                                  "minimum": 0
                                },
                                "width": {
                                  "type": "number",
                                  "title": "Ring width",
                                  "default": 0.01,
                                  "minimum": 0,
                                  "description": "Distance between pointings."
                                }
                              },
                              "description": "Rings of TABs around the center of the beam.",
                              "additonalProperties": false
                            }
                          },
                          "additionalProperties": false
                        },
                        "title": "SAPs",
                        "default": [],
                        "minItems": 0,
                        "description": "Which SAPs in the observation to beamform.",
                        "additionalItems": false
                      },
                      "settings": {
                        "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/1#/definitions/stokes_settings",
                        "default": {}
                      }
                    },
                    "additionalProperties": false
                  },
                  "flys eye": {
                    "type": "object",
                    "title": "Fly's Eye Settings",
                    "default": {},
                    "required": [
                      "enabled"
                    ],
                    "properties": {
                      "enabled": {
                        "type": "boolean",
                        "title": "Enable Fly's Eye",
                        "default": false
                      },
                      "settings": {
                        "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/1#/definitions/stokes_settings",
                        "default": {}
                      }
                    },
                    "description": "Produce beams containing the individual station signals",
                    "additionalProperties": false
                  },
                  "incoherent": {
                    "type": "object",
                    "title": "Incoherent Tied-Array Beams",
                    "default": {},
                    "required": [
                      "settings",
                      "SAPs"
                    ],
                    "properties": {
                      "SAPs": {
                        "type": "array",
                        "items": {
                          "type": "object",
                          "default": {},
                          "required": [
                            "name",
                            "subbands"
                          ],
                          "properties": {
                            "name": {
                              "type": "string",
                              "title": "SAP name",
                              "default": "_SAP_to_beamform_",
                              "minLength": 1,
                              "description": "Name of the SAP to beamform"
                            },
                            "subbands": {
                              "$ref": "#/definitions/subband_selection",
                              "default": {}
                            }
                          },
                          "additionalProperties": false
                        },
                        "title": "SAPs",
                        "default": [],
                        "minItems": 0,
                        "description": "Which SAPs in the observation to create incoherent TABs for (empty list = all).",
                        "additionalItems": false
                      },
                      "settings": {
                        "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/1#/definitions/stokes_settings",
                        "default": {}
                      }
                    },
                    "additionalProperties": false
                  },
                  "station_groups": {
                    "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1#/definitions/station_groups",
                    "default": [
                      {
                        "stations": [
                          "CS001",
                          "CS002",
                          "CS003",
                          "CS004",
                          "CS005",
                          "CS006",
                          "CS007",
                          "CS011",
                          "CS013",
                          "CS017",
                          "CS021",
                          "CS024",
                          "CS026",
                          "CS028",
                          "CS030",
                          "CS031",
                          "CS032",
                          "CS301",
                          "CS302",
                          "CS401",
                          "CS501"
                        ],
                        "max_nr_missing": 1
                      }
                    ],
                    "minItems": 1,
                    "description": "While observing, COBALT will beamform on the intersection of all stations in this list and the used stations in the observation. So, specifying all possible stations here means that all observation-stations are used. Specifying a small subset here means that only the observing-stations in this small list are used. By default we let COBALT beamform on the Core stations."
                  }
                },
                "headerTemplate": "Beamformer {{ self.index }}",
                "additionalProperties": false
              },
              "title": "Beamformers",
              "default": [
                {}
              ],
              "minItems": 1,
              "additionalItems": false
            },
            "station_groups": {
              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1#/definitions/station_groups",
              "default": [
                {
                  "stations": [
                    "CS002",
                    "CS003",
                    "CS004",
                    "CS005",
                    "CS006",
                    "CS007"
                  ],
                  "max_nr_missing": 1
                }
              ]
            }
          },
          "definitions": {
            "subband_selection": {
              "type": "object",
              "title": "Subband selection",
              "default": {},
              "required": [
                "method"
              ],
              "properties": {
                "list": {
                  "type": "array",
                  "items": {
                    "type": "integer",
                    "title": "Subband",
                    "maximum": 511,
                    "minimum": 0
                  },
                  "title": "Subset selection",
                  "default": [],
                  "minItems": 0,
                  "description": "If method is 'select subset', only beamform these subbands, and only if they occur in the SAP.",
                  "additionalItems": false
                },
                "method": {
                  "enum": [
                    "copy",
                    "largest continuous subset",
                    "select subset"
                  ],
                  "type": "string",
                  "title": "Method",
                  "default": "copy",
                  "description": "How to select the subbands to beam form"
                }
              },
              "additionalProperties": false
            }
          },
          "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT."
        },
        "tags": [],
        "type": "http://localhost:3000/api/task_type/observation",
        "type_value": "observation",
        "updated_at": "2022-02-04T05:10:04.245245",
        "version": 1
      },
      {
        "id": 4,
        "url": "http://localhost:3000/api/task_template/4",
        "created_at": "2022-02-04T05:10:04.248713",
        "description": "This schema defines the parameters to setup an ingest task.",
        "name": "ingest",
        "schema": {
          "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/1#",
          "type": "object",
          "title": "ingest",
          "$schema": "http://json-schema.org/draft-06/schema#",
          "version": 1,
          "required": [],
          "properties": {},
          "description": "This schema defines the parameters to setup an ingest task."
        },
        "tags": [],
        "type": "http://localhost:3000/api/task_type/ingest",
        "type_value": "ingest",
        "updated_at": "2022-02-04T05:10:04.248761",
        "version": 1
      },
      {
        "id": 5,
        "url": "http://localhost:3000/api/task_template/5",
        "created_at": "2022-02-04T05:10:04.281229",
        "description": "This schema defines the (extra) parameters to setup a calibrator observation task, which uses all paramters from the target observation task which it is linked to, plus these calibrator overrides.",
        "name": "calibrator observation",
        "schema": {
          "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/1#",
          "type": "object",
          "title": "calibrator observation",
          "$schema": "http://json-schema.org/draft-06/schema#",
          "version": 1,
          "required": [
            "autoselect",
            "duration",
            "pointing",
            "name"
          ],
          "properties": {
            "name": {
              "type": "string",
              "title": "Name",
              "default": "calibrator",
              "description": "Name of the calibrator SAP"
            },
            "duration": {
              "$id": "#duration",
              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/1/#/definitions/timedelta",
              "title": "Duration",
              "default": 600,
              "minimum": 1,
              "description": "Duration of this observation (seconds)"
            },
            "pointing": {
              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pointing/1/#/definitions/pointing",
              "title": "Digital pointing",
              "default": {},
              "description": "Manually selected calibrator"
            },
            "autoselect": {
              "type": "boolean",
              "title": "Auto-select",
              "default": true,
              "description": "Auto-select calibrator based on elevation"
            }
          },
          "description": "This schema defines the (extra) parameters to setup a calibrator observation task, which uses all paramters from the target observation task which it is linked to, plus these calibrator overrides."
        },
        "tags": [],
        "type": "http://localhost:3000/api/task_type/observation",
        "type_value": "observation",
        "updated_at": "2022-02-04T05:10:04.281244",
        "version": 1
      },
      {
        "id": 6,
        "url": "http://localhost:3000/api/task_template/6",
        "created_at": "2022-02-04T05:10:04.704390",
        "description": "This schema defines the parameters to setup a target observation task.",
        "name": "target observation",
        "schema": {
          "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/1#",
          "type": "object",
          "title": "target observation",
          "$schema": "http://json-schema.org/draft-06/schema#",
          "version": 1,
          "required": [
              "station_groups",
              "antenna_set",
              "filter",
              "SAPs",
              "duration",
              "correlator"
          ],
          "properties": {
              "QA": {
                  "$ref": "#/definitions/QA",
                  "default": {}
              },
              "SAPs": {
                  "$ref": "#/definitions/SAPs",
                  "default": [
                      {}
                  ],
                  "minItems": 1
              },
              "filter": {
                  "$ref": "#/definitions/filter",
                  "default": "HBA_110_190"
              },
              "duration": {
                  "$id": "#duration",
                  "$ref": "#/definitions/timedelta",
                  "title": "Duration",
                  "default": 300,
                  "minimum": 1,
                  "description": "Duration of this observation (seconds)"
              },
              "tile_beam": {
                  "$ref": "#/definitions/pointing",
                  "title": "Tile beam",
                  "default": {},
                  "description": "HBA only"
              },
              "correlator": {
                  "type": "object",
                  "title": "Correlator Settings",
                  "default": {},
                  "required": [
                      "channels_per_subband",
                      "topocentric_frequency_correction",
                      "integration_time",
                      "storage_cluster"
                  ],
                  "properties": {
                      "storage_cluster": {
                          "enum": [
                              "CEP4",
                              "DragNet"
                          ],
                          "type": "string",
                          "title": "Storage cluster",
                          "default": "CEP4",
                          "description": "Cluster to write output to"
                      },
                      "integration_time": {
                          "$ref": "#/definitions/timedelta",
                          "title": "Integration time",
                          "default": 1,
                          "minimum": 0.1,
                          "description": "Desired integration period (seconds)"
                      },
                        "channels_per_subband": {
                          "enum": [
                              8,
                              16,
                              32,
                              64,
                              128,
                              256,
                              512,
                              1024
                          ],
                          "type": "integer",
                          "title": "Channels/subband",
                          "default": 64,
                          "minimum": 8,
                          "description": "Number of frequency bands per subband"
                      },
                      "topocentric_frequency_correction": {
                          "type": "boolean",
                          "title": "Topocentric Frequency Correction",
                          "default": false,
                          "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation."
                      }
                  },
                  "additionalProperties": false
              },
              "antenna_set": {
                  "$ref": "#/definitions/antenna_set",
                  "default": "HBA_DUAL"
              },
              "station_groups": {
                  "$ref": "#/definitions/station_groups",
                  "default": [
                      {
                          "stations": [
                              "CS002",
                              "CS003",
                              "CS004",
                              "CS005",
                              "CS006",
                              "CS007"
                          ],
                          "max_nr_missing": 1
                      }
                  ]
              }
          },
          "description": "This schema defines the parameters to setup a target observation task.",
          "definitions": {
              "QA": {
                  "type": "object",
                  "title": "QA",
                  "default": {},
                  "required": [
                      "file_conversion",
                      "plots",
                      "inspection_plots"
                  ],
                  "properties": {
                      "plots": {
                          "$ref": "#/definitions/plots",
                          "default": {}
                      },
                      "file_conversion": {
                          "$ref": "#/definitions/file_conversion",
                          "default": {}
                      },
                      "inspection_plots": {
                          "$ref": "#/definitions/inspection_plots",
                          "default": "msplots"
                      }
                  },
                  "description": "Perform all Quality Assurance (QA) tasks, including file conversion and plotting.",
                  "additionalProperties": false
              },
              "plots": {
                  "type": "object",
                  "title": "Plots",
                  "default": {},
                  "required": [
                      "enabled",
                      "autocorrelation",
                      "crosscorrelation"
                  ],
                  "properties": {
                      "enabled": {
                          "type": "boolean",
                          "title": "enabled",
                          "default": true,
                          "description": "Do/Don't create plots from the QA file from the observation"
                      },
                      "autocorrelation": {
                          "type": "boolean",
                          "title": "autocorrelation",
                          "default": true,
                          "description": "Create autocorrelation plots for all stations"
                      },
                      "crosscorrelation": {
                          "type": "boolean",
                          "title": "crosscorrelation",
                          "default": true,
                          "description": "Create crosscorrelation plots for all baselines"
                      }
                  },
                  "description": "Create dynamic spectrum plots",
                  "additionalProperties": false
              },
              "file_conversion": {
                  "type": "object",
                  "title": "File Conversion",
                  "default": {},
                  "required": [
                      "enabled",
                      "nr_of_subbands",
                      "nr_of_timestamps"
                  ],
                  "properties": {
                      "enabled": {
                          "type": "boolean",
                          "title": "enabled",
                          "default": true,
                          "description": "Do/Don't create a QA file for the observation"
                      },
                      "nr_of_subbands": {
                          "type": "integer",
                          "title": "#subbands",
                          "default": -1,
                          "description": "Keep this number of subbands from the observation in the QA file, or all if -1"
                      },
                      "nr_of_timestamps": {
                          "type": "integer",
                          "title": "#timestamps",
                          "default": 256,
                          "minimum": 1,
                          "description": "Extract this number of timestamps from the observation in the QA file (equidistantanly sampled, no averaging/interpolation)"
                      }
                  },
                  "description": "Create a QA file for the observation",
                  "additionalProperties": false
              },
              "inspection_plots": {
                  "enum": [
                      "msplots",
                      "dynspec",
                      "none"
                  ],
                  "type": "string",
                  "title": "Inspection Plots",
                  "default": "msplots",
                  "decription": "Type of inspection plots to run"
              },
              "SAPs": {
                  "type": "array",
                  "items": {
                      "type": "object",
                      "title": "SAP",
                      "default": {},
                      "required": [
                          "name",
                          "digital_pointing",
                          "subbands"
                      ],
                      "properties": {
                          "name": {
                              "type": "string",
                              "title": "Name",
                              "default": "_SAP_name_",
                              "minLength": 1,
                              "description": "Identifier for this beam"
                          },
                          "subbands": {
                              "type": "array",
                              "items": {
                                  "type": "integer",
                                  "title": "Subband",
                                  "maximum": 511,
                                  "minimum": 0,
                                  "maxLength": 488,
                                  "minLength": 1
                              },
                              "title": "Subband list",
                              "default": [],
                              "additionalItems": false
                          },
                          "digital_pointing": {
                              "$id": "#target_pointing",
                              "$ref": "#/definitions/pointing",
                              "title": "Digital pointing",
                              "default": {}
                          }
                      },
                      "headerTemplate": "{{ i0 }} - {{ self.name }}",
                      "additionalProperties": false
                  },
                  "title": "SAPs",
                  "default": [
                      {}
                  ],
                  "minItems": 0,
                  "description": "Station beams",
                  "additionalItems": false
              },
              "pointing": {
                  "type": "object",
                  "required": [
                      "angle1",
                      "angle2",
                      "target"
                  ],
                  "properties": {
                      "angle1": {
                          "type": "number",
                          "title": "Angle 1",
                          "default": 0.6624317181687094,
                          "description": "First angle (e.g. RA)"
                      },
                      "angle2": {
                          "type": "number",
                          "title": "Angle 2",
                          "default": 1.5579526427549426,
                          "description": "Second angle (e.g. DEC)"
                      },
                      "target": {
                          "type": "string",
                          "title": "Target",
                          "default": "_target_name_",
                          "minLength": 1,
                          "description": "Description of where this beam points at"
                      },
                      "direction_type": {
                          "enum": [
                              "J2000",
                              "AZELGEO",
                              "LMN",
                              "SUN",
                              "MOON",
                              "MERCURY",
                              "VENUS",
                              "MARS",
                              "JUPITER",
                              "SATURN",
                              "URANUS",
                              "NEPTUNE",
                              "PLUTO"
                          ],
                          "type": "string",
                          "title": "Reference frame",
                          "default": "J2000",
                          "description": ""
                      }
                  },
                  "additionalProperties": false
              },
              "filter": {
                  "enum": [
                      "LBA_10_70",
                      "LBA_30_70",
                      "LBA_10_90",
                      "LBA_30_90",
                      "HBA_110_190",
                      "HBA_210_250"
                  ],
                  "type": "string",
                  "title": "Band-pass filter",
                  "default": "HBA_110_190",
                  "description": "Must match antenna type"
              },
              "timedelta": {
                  "type": "number",
                  "default": 0,
                  "description": "A time duration or delta expressed in seconds"
              },
              "antenna_set": {
                  "enum": [
                      "HBA_DUAL",
                      "HBA_DUAL_INNER",
                      "HBA_ONE",
                      "HBA_ONE_INNER",
                      "HBA_ZERO",
                      "HBA_ZERO_INNER",
                      "LBA_INNER",
                      "LBA_OUTER",
                      "LBA_SPARSE_EVEN",
                      "LBA_SPARSE_ODD",
                      "LBA_ALL"
                  ],
                  "type": "string",
                  "title": "Antenna set",
                  "default": "HBA_DUAL",
                  "description": "Fields & antennas to use"
              },
              "station_groups": {
                  "type": "array",
                  "items": {
                      "$ref": "#/definitions/station_group"
                  },
                  "title": "Station groups",
                  "default": [
                      {
                          "stations": [
                              "CS002",
                              "CS003",
                              "CS004",
                              "CS005",
                              "CS006",
                              "CS007"
                          ],
                          "max_nr_missing": 1
                      }
                  ],
                  "minItems": 1,
                  "description": "One or more predefined or custom groups of stations",
                  "additionalItems": false,
                  "additionalProperties": false
              },
              "station_group": {
                  "type": "object",
                  "anyOf": [
                      {
                          "type": "object",
                          "title": "Superterp",
                          "default": {
                              "stations": [
                                  "CS002",
                                  "CS003",
                                  "CS004",
                                  "CS005",
                                  "CS006",
                                  "CS007"
                              ],
                              "max_nr_missing": 0
                          },
                          "required": [
                              "stations",
                              "max_nr_missing"
                          ],
                          "properties": {
                              "stations": {
                                  "$ref": "#/definitions/station_list",
                                  "enum": [
                                      [
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007"
                                      ]
                                  ]
                              },
                              "max_nr_missing": {
                                  "$ref": "#/definitions/max_number_of_missing_stations"
                              }
                          },
                          "description": "The group of all stations on the Superterp",
                          "additionalProperties": false,
                          "definitions": {
                              "station_list": {
                                  "type": "array",
                                  "items": {
                                      "enum": [
                                          "CS001",
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007",
                                          "CS011",
                                          "CS013",
                                          "CS017",
                                          "CS021",
                                          "CS024",
                                          "CS026",
                                          "CS028",
                                          "CS030",
                                          "CS031",
                                          "CS032",
                                          "CS101",
                                          "CS103",
                                          "CS201",
                                          "CS301",
                                          "CS302",
                                          "CS401",
                                          "CS501",
                                          "RS106",
                                          "RS205",
                                          "RS208",
                                          "RS210",
                                          "RS305",
                                          "RS306",
                                          "RS307",
                                          "RS310",
                                          "RS406",
                                          "RS407",
                                          "RS409",
                                          "RS503",
                                          "RS508",
                                          "RS509",
                                          "DE601",
                                          "DE602",
                                          "DE603",
                                          "DE604",
                                          "DE605",
                                          "FR606",
                                          "SE607",
                                          "UK608",
                                          "DE609",
                                          "PL610",
                                          "PL611",
                                          "PL612",
                                          "IE613",
                                          "LV614"
                                      ],
                                      "type": "string",
                                      "title": "Station",
                                      "description": "These are the LOFAR stations"
                                  },
                                  "default": [],
                                  "minItems": 0,
                                  "uniqueItems": true,
                                  "additionalItems": false,
                                  "additionalProperties": false
                              },
                              "max_number_of_missing_stations": {
                                  "type": "integer",
                                  "title": "Maximum number of stations to omit",
                                  "default": 0,
                                  "minimum": 0,
                                  "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)"
                              }
                          }
                      },
                      {
                          "type": "object",
                          "title": "Core",
                          "default": {
                              "stations": [
                                  "CS001",
                                  "CS002",
                                  "CS003",
                                  "CS004",
                                  "CS005",
                                  "CS006",
                                  "CS007",
                                  "CS011",
                                  "CS013",
                                  "CS017",
                                  "CS021",
                                  "CS024",
                                  "CS026",
                                  "CS028",
                                  "CS030",
                                  "CS031",
                                  "CS032",
                                  "CS101",
                                  "CS103",
                                  "CS201",
                                  "CS301",
                                  "CS302",
                                  "CS401",
                                  "CS501"
                              ],
                              "max_nr_missing": 4
                          },
                          "required": [
                              "stations",
                              "max_nr_missing"
                          ],
                          "properties": {
                              "stations": {
                                  "$ref": "#/definitions/station_list",
                                  "enum": [
                                      [
                                          "CS001",
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007",
                                          "CS011",
                                          "CS013",
                                          "CS017",
                                          "CS021",
                                          "CS024",
                                          "CS026",
                                          "CS028",
                                          "CS030",
                                          "CS031",
                                          "CS032",
                                          "CS101",
                                          "CS103",
                                          "CS201",
                                          "CS301",
                                          "CS302",
                                          "CS401",
                                          "CS501"
                                      ]
                                  ]
                              },
                              "max_nr_missing": {
                                  "$ref": "#/definitions/max_number_of_missing_stations"
                              }
                          },
                          "description": "The group of all Core stations",
                          "additionalProperties": false,
                          "definitions": {
                              "station_list": {
                                  "type": "array",
                                  "items": {
                                      "enum": [
                                          "CS001",
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007",
                                          "CS011",
                                          "CS013",
                                          "CS017",
                                          "CS021",
                                          "CS024",
                                          "CS026",
                                          "CS028",
                                          "CS030",
                                          "CS031",
                                          "CS032",
                                          "CS101",
                                          "CS103",
                                          "CS201",
                                          "CS301",
                                          "CS302",
                                          "CS401",
                                          "CS501",
                                          "RS106",
                                          "RS205",
                                          "RS208",
                                          "RS210",
                                          "RS305",
                                          "RS306",
                                          "RS307",
                                          "RS310",
                                          "RS406",
                                          "RS407",
                                          "RS409",
                                          "RS503",
                                          "RS508",
                                          "RS509",
                                          "DE601",
                                          "DE602",
                                          "DE603",
                                          "DE604",
                                          "DE605",
                                          "FR606",
                                          "SE607",
                                          "UK608",
                                          "DE609",
                                          "PL610",
                                          "PL611",
                                          "PL612",
                                          "IE613",
                                          "LV614"
                                      ],
                                      "type": "string",
                                      "title": "Station",
                                      "description": "These are the LOFAR stations"
                                  },
                                  "default": [],
                                  "minItems": 0,
                                  "uniqueItems": true,
                                  "additionalItems": false,
                                  "additionalProperties": false
                              },
                              "max_number_of_missing_stations": {
                                  "type": "integer",
                                  "title": "Maximum number of stations to omit",
                                  "default": 0,
                                  "minimum": 0,
                                  "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)"
                              }
                          }
                      },
                      {
                          "type": "object",
                          "title": "Remote",
                          "default": {
                              "stations": [
                                  "RS106",
                                  "RS205",
                                  "RS208",
                                  "RS210",
                                  "RS305",
                                  "RS306",
                                  "RS307",
                                  "RS310",
                                  "RS406",
                                  "RS407",
                                  "RS409",
                                  "RS503",
                                  "RS508",
                                  "RS509"
                              ],
                              "max_nr_missing": 4
                          },
                          "required": [
                              "stations",
                              "max_nr_missing"
                          ],
                          "properties": {
                              "stations": {
                                  "$ref": "#/definitions/station_list",
                                  "enum": [
                                      [
                                          "RS106",
                                          "RS205",
                                          "RS208",
                                          "RS210",
                                          "RS305",
                                          "RS306",
                                          "RS307",
                                          "RS310",
                                          "RS406",
                                          "RS407",
                                          "RS409",
                                          "RS503",
                                          "RS508",
                                          "RS509"
                                      ]
                                  ]
                              },
                              "max_nr_missing": {
                                  "$ref": "#/definitions/max_number_of_missing_stations"
                              }
                          },
                          "description": "The group of all Dutch remote stations",
                          "additionalProperties": false,
                          "definitions": {
                              "station_list": {
                                  "type": "array",
                                  "items": {
                                      "enum": [
                                          "CS001",
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007",
                                          "CS011",
                                          "CS013",
                                          "CS017",
                                          "CS021",
                                          "CS024",
                                          "CS026",
                                          "CS028",
                                          "CS030",
                                          "CS031",
                                          "CS032",
                                          "CS101",
                                          "CS103",
                                          "CS201",
                                          "CS301",
                                          "CS302",
                                          "CS401",
                                          "CS501",
                                          "RS106",
                                          "RS205",
                                          "RS208",
                                          "RS210",
                                          "RS305",
                                          "RS306",
                                          "RS307",
                                          "RS310",
                                          "RS406",
                                          "RS407",
                                          "RS409",
                                          "RS503",
                                          "RS508",
                                          "RS509",
                                          "DE601",
                                          "DE602",
                                          "DE603",
                                          "DE604",
                                          "DE605",
                                          "FR606",
                                          "SE607",
                                          "UK608",
                                          "DE609",
                                          "PL610",
                                          "PL611",
                                          "PL612",
                                          "IE613",
                                          "LV614"
                                      ],
                                      "type": "string",
                                      "title": "Station",
                                      "description": "These are the LOFAR stations"
                                  },
                                  "default": [],
                                  "minItems": 0,
                                  "uniqueItems": true,
                                  "additionalItems": false,
                                  "additionalProperties": false
                              },
                              "max_number_of_missing_stations": {
                                  "type": "integer",
                                  "title": "Maximum number of stations to omit",
                                  "default": 0,
                                  "minimum": 0,
                                  "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)"
                              }
                          }
                      },
                      {
                          "type": "object",
                          "title": "Dutch",
                          "default": {
                              "stations": [
                                  "CS001",
                                  "CS002",
                                  "CS003",
                                  "CS004",
                                  "CS005",
                                  "CS006",
                                  "CS007",
                                  "CS011",
                                  "CS013",
                                  "CS017",
                                  "CS021",
                                  "CS024",
                                  "CS026",
                                  "CS028",
                                  "CS030",
                                  "CS031",
                                  "CS032",
                                  "CS101",
                                  "CS103",
                                  "CS201",
                                  "CS301",
                                  "CS302",
                                  "CS401",
                                  "CS501",
                                  "RS106",
                                  "RS205",
                                  "RS208",
                                  "RS210",
                                  "RS305",
                                  "RS306",
                                  "RS307",
                                  "RS310",
                                  "RS406",
                                  "RS407",
                                  "RS409",
                                  "RS503",
                                  "RS508",
                                  "RS509"
                              ],
                              "max_nr_missing": 4
                          },
                          "required": [
                              "stations",
                              "max_nr_missing"
                          ],
                          "properties": {
                              "stations": {
                                  "$ref": "#/definitions/station_list",
                                  "enum": [
                                      [
                                          "CS001",
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007",
                                          "CS011",
                                          "CS013",
                                          "CS017",
                                          "CS021",
                                          "CS024",
                                          "CS026",
                                          "CS028",
                                          "CS030",
                                          "CS031",
                                          "CS032",
                                          "CS101",
                                          "CS103",
                                          "CS201",
                                          "CS301",
                                          "CS302",
                                          "CS401",
                                          "CS501",
                                          "RS106",
                                          "RS205",
                                          "RS208",
                                          "RS210",
                                          "RS305",
                                          "RS306",
                                          "RS307",
                                          "RS310",
                                          "RS406",
                                          "RS407",
                                          "RS409",
                                          "RS503",
                                          "RS508",
                                          "RS509"
                                      ]
                                  ]
                              },
                              "max_nr_missing": {
                                  "$ref": "#/definitions/max_number_of_missing_stations"
                              }
                          },
                          "description": "The group of all Dutch (Core + Remote) stations",
                          "additionalProperties": false,
                          "definitions": {
                              "station_list": {
                                  "type": "array",
                                  "items": {
                                      "enum": [
                                          "CS001",
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007",
                                          "CS011",
                                          "CS013",
                                          "CS017",
                                          "CS021",
                                          "CS024",
                                          "CS026",
                                          "CS028",
                                          "CS030",
                                          "CS031",
                                          "CS032",
                                          "CS101",
                                          "CS103",
                                          "CS201",
                                          "CS301",
                                          "CS302",
                                          "CS401",
                                          "CS501",
                                          "RS106",
                                          "RS205",
                                          "RS208",
                                          "RS210",
                                          "RS305",
                                          "RS306",
                                          "RS307",
                                          "RS310",
                                          "RS406",
                                          "RS407",
                                          "RS409",
                                          "RS503",
                                          "RS508",
                                          "RS509",
                                          "DE601",
                                          "DE602",
                                          "DE603",
                                          "DE604",
                                          "DE605",
                                          "FR606",
                                          "SE607",
                                          "UK608",
                                          "DE609",
                                          "PL610",
                                          "PL611",
                                          "PL612",
                                          "IE613",
                                          "LV614"
                                      ],
                                      "type": "string",
                                      "title": "Station",
                                      "description": "These are the LOFAR stations"
                                  },
                                  "default": [],
                                  "minItems": 0,
                                  "uniqueItems": true,
                                  "additionalItems": false,
                                  "additionalProperties": false
                              },
                              "max_number_of_missing_stations": {
                                  "type": "integer",
                                  "title": "Maximum number of stations to omit",
                                  "default": 0,
                                  "minimum": 0,
                                  "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)"
                              }
                          }
                      },
                      {
                          "type": "object",
                          "title": "International",
                          "default": {
                              "stations": [
                                  "DE601",
                                  "DE602",
                                  "DE603",
                                  "DE604",
                                  "DE605",
                                  "DE609",
                                  "FR606",
                                  "SE607",
                                  "UK608",
                                  "PL610",
                                  "PL611",
                                  "PL612",
                                  "IE613",
                                  "LV614"
                              ],
                              "max_nr_missing": 2
                          },
                          "required": [
                              "stations",
                              "max_nr_missing"
                          ],
                          "properties": {
                              "stations": {
                                  "$ref": "#/definitions/station_list",
                                  "enum": [
                                      [
                                          "DE601",
                                          "DE602",
                                          "DE603",
                                          "DE604",
                                          "DE605",
                                          "DE609",
                                          "FR606",
                                          "SE607",
                                          "UK608",
                                          "PL610",
                                          "PL611",
                                          "PL612",
                                          "IE613",
                                          "LV614"
                                      ]
                                  ]
                              },
                              "max_nr_missing": {
                                  "$ref": "#/definitions/max_number_of_missing_stations"
                              }
                          },
                          "description": "The group of all international stations",
                          "additionalProperties": false,
                          "definitions": {
                              "station_list": {
                                  "type": "array",
                                  "items": {
                                      "enum": [
                                          "CS001",
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007",
                                          "CS011",
                                          "CS013",
                                          "CS017",
                                          "CS021",
                                          "CS024",
                                          "CS026",
                                          "CS028",
                                          "CS030",
                                          "CS031",
                                          "CS032",
                                          "CS101",
                                          "CS103",
                                          "CS201",
                                          "CS301",
                                          "CS302",
                                          "CS401",
                                          "CS501",
                                          "RS106",
                                          "RS205",
                                          "RS208",
                                          "RS210",
                                          "RS305",
                                          "RS306",
                                          "RS307",
                                          "RS310",
                                          "RS406",
                                          "RS407",
                                          "RS409",
                                          "RS503",
                                          "RS508",
                                          "RS509",
                                          "DE601",
                                          "DE602",
                                          "DE603",
                                          "DE604",
                                          "DE605",
                                          "FR606",
                                          "SE607",
                                          "UK608",
                                          "DE609",
                                          "PL610",
                                          "PL611",
                                          "PL612",
                                          "IE613",
                                          "LV614"
                                      ],
                                      "type": "string",
                                      "title": "Station",
                                      "description": "These are the LOFAR stations"
                                  },
                                  "default": [],
                                  "minItems": 0,
                                  "uniqueItems": true,
                                  "additionalItems": false,
                                  "additionalProperties": false
                              },
                              "max_number_of_missing_stations": {
                                  "type": "integer",
                                  "title": "Maximum number of stations to omit",
                                  "default": 0,
                                  "minimum": 0,
                                  "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)"
                              }
                          }
                      },
                      {
                          "type": "object",
                          "title": "International required",
                          "default": {
                              "stations": [
                                  "DE601",
                                  "DE605"
                              ],
                              "max_nr_missing": 1
                          },
                          "required": [
                              "stations",
                              "max_nr_missing"
                          ],
                          "properties": {
                              "stations": {
                                  "$ref": "#/definitions/station_list",
                                  "enum": [
                                      [
                                          "DE601",
                                          "DE605"
                                      ]
                                  ]
                              },
                              "max_nr_missing": {
                                  "$ref": "#/definitions/max_number_of_missing_stations"
                              }
                          },
                          "description": "A subgroup of the international stations which are required when doing observation with international stations",
                          "additionalProperties": false,
                          "definitions": {
                              "station_list": {
                                  "type": "array",
                                  "items": {
                                      "enum": [
                                          "CS001",
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007",
                                          "CS011",
                                          "CS013",
                                          "CS017",
                                          "CS021",
                                          "CS024",
                                          "CS026",
                                          "CS028",
                                          "CS030",
                                          "CS031",
                                          "CS032",
                                          "CS101",
                                          "CS103",
                                          "CS201",
                                          "CS301",
                                          "CS302",
                                          "CS401",
                                          "CS501",
                                          "RS106",
                                          "RS205",
                                          "RS208",
                                          "RS210",
                                          "RS305",
                                          "RS306",
                                          "RS307",
                                          "RS310",
                                          "RS406",
                                          "RS407",
                                          "RS409",
                                          "RS503",
                                          "RS508",
                                          "RS509",
                                          "DE601",
                                          "DE602",
                                          "DE603",
                                          "DE604",
                                          "DE605",
                                          "FR606",
                                          "SE607",
                                          "UK608",
                                          "DE609",
                                          "PL610",
                                          "PL611",
                                          "PL612",
                                          "IE613",
                                          "LV614"
                                      ],
                                      "type": "string",
                                      "title": "Station",
                                      "description": "These are the LOFAR stations"
                                  },
                                  "default": [],
                                  "minItems": 0,
                                  "uniqueItems": true,
                                  "additionalItems": false,
                                  "additionalProperties": false
                              },
                              "max_number_of_missing_stations": {
                                  "type": "integer",
                                  "title": "Maximum number of stations to omit",
                                  "default": 0,
                                  "minimum": 0,
                                  "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)"
                              }
                          }
                      },
                      {
                          "type": "object",
                          "title": "All",
                          "default": {
                              "stations": [
                                  "CS001",
                                  "CS002",
                                  "CS003",
                                  "CS004",
                                  "CS005",
                                  "CS006",
                                  "CS007",
                                  "CS011",
                                  "CS013",
                                  "CS017",
                                  "CS021",
                                  "CS024",
                                  "CS026",
                                  "CS028",
                                  "CS030",
                                  "CS031",
                                  "CS032",
                                  "CS101",
                                  "CS103",
                                  "CS201",
                                  "CS301",
                                  "CS302",
                                  "CS401",
                                  "CS501",
                                  "RS106",
                                  "RS205",
                                  "RS208",
                                  "RS210",
                                  "RS305",
                                  "RS306",
                                  "RS307",
                                  "RS310",
                                  "RS406",
                                  "RS407",
                                  "RS409",
                                  "RS503",
                                  "RS508",
                                  "RS509",
                                  "DE601",
                                  "DE602",
                                  "DE603",
                                  "DE604",
                                  "DE605",
                                  "DE609",
                                  "FR606",
                                  "SE607",
                                  "UK608",
                                  "PL610",
                                  "PL611",
                                  "PL612",
                                  "IE613",
                                  "LV614"
                              ],
                              "max_nr_missing": 6
                          },
                          "required": [
                              "stations",
                              "max_nr_missing"
                          ],
                          "properties": {
                              "stations": {
                                  "$ref": "#/definitions/station_list",
                                  "enum": [
                                      [
                                          "CS001",
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007",
                                          "CS011",
                                          "CS013",
                                          "CS017",
                                          "CS021",
                                          "CS024",
                                          "CS026",
                                          "CS028",
                                          "CS030",
                                          "CS031",
                                          "CS032",
                                          "CS101",
                                          "CS103",
                                          "CS201",
                                          "CS301",
                                          "CS302",
                                          "CS401",
                                          "CS501",
                                          "RS106",
                                          "RS205",
                                          "RS208",
                                          "RS210",
                                          "RS305",
                                          "RS306",
                                          "RS307",
                                          "RS310",
                                          "RS406",
                                          "RS407",
                                          "RS409",
                                          "RS503",
                                          "RS508",
                                          "RS509",
                                          "DE601",
                                          "DE602",
                                          "DE603",
                                          "DE604",
                                          "DE605",
                                          "DE609",
                                          "FR606",
                                          "SE607",
                                          "UK608",
                                          "PL610",
                                          "PL611",
                                          "PL612",
                                          "IE613",
                                          "LV614"
                                      ]
                                  ]
                              },
                              "max_nr_missing": {
                                  "$ref": "#/definitions/max_number_of_missing_stations"
                              }
                          },
                          "description": "The group of all (Core + Remote + International) stations",
                          "additionalProperties": false,
                          "definitions": {
                              "station_list": {
                                  "type": "array",
                                  "items": {
                                      "enum": [
                                          "CS001",
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007",
                                          "CS011",
                                          "CS013",
                                          "CS017",
                                          "CS021",
                                          "CS024",
                                          "CS026",
                                          "CS028",
                                          "CS030",
                                          "CS031",
                                          "CS032",
                                          "CS101",
                                          "CS103",
                                          "CS201",
                                          "CS301",
                                          "CS302",
                                          "CS401",
                                          "CS501",
                                          "RS106",
                                          "RS205",
                                          "RS208",
                                          "RS210",
                                          "RS305",
                                          "RS306",
                                          "RS307",
                                          "RS310",
                                          "RS406",
                                          "RS407",
                                          "RS409",
                                          "RS503",
                                          "RS508",
                                          "RS509",
                                          "DE601",
                                          "DE602",
                                          "DE603",
                                          "DE604",
                                          "DE605",
                                          "FR606",
                                          "SE607",
                                          "UK608",
                                          "DE609",
                                          "PL610",
                                          "PL611",
                                          "PL612",
                                          "IE613",
                                          "LV614"
                                      ],
                                      "type": "string",
                                      "title": "Station",
                                      "description": "These are the LOFAR stations"
                                  },
                                  "default": [],
                                  "minItems": 0,
                                  "uniqueItems": true,
                                  "additionalItems": false,
                                  "additionalProperties": false
                              },
                              "max_number_of_missing_stations": {
                                  "type": "integer",
                                  "title": "Maximum number of stations to omit",
                                  "default": 0,
                                  "minimum": 0,
                                  "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)"
                              }
                          }
                      },
                      {
                          "type": "object",
                          "title": "Custom",
                          "default": {
                              "stations": [
                                  "CS001"
                              ],
                              "max_nr_missing": 0
                          },
                          "required": [
                              "stations",
                              "max_nr_missing"
                          ],
                          "properties": {
                              "stations": {
                                  "$ref": "#/definitions/station_list"
                              },
                              "max_nr_missing": {
                                  "$ref": "#/definitions/max_number_of_missing_stations"
                              }
                          },
                          "description": "A custom group of stations which can be defined by the user",
                          "additionalProperties": false,
                          "definitions": {
                              "station_list": {
                                  "type": "array",
                                  "items": {
                                      "enum": [
                                          "CS001",
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007",
                                          "CS011",
                                          "CS013",
                                          "CS017",
                                          "CS021",
                                          "CS024",
                                          "CS026",
                                          "CS028",
                                          "CS030",
                                          "CS031",
                                          "CS032",
                                          "CS101",
                                          "CS103",
                                          "CS201",
                                          "CS301",
                                          "CS302",
                                          "CS401",
                                          "CS501",
                                          "RS106",
                                          "RS205",
                                          "RS208",
                                          "RS210",
                                          "RS305",
                                          "RS306",
                                          "RS307",
                                          "RS310",
                                          "RS406",
                                          "RS407",
                                          "RS409",
                                          "RS503",
                                          "RS508",
                                          "RS509",
                                          "DE601",
                                          "DE602",
                                          "DE603",
                                          "DE604",
                                          "DE605",
                                          "FR606",
                                          "SE607",
                                          "UK608",
                                          "DE609",
                                          "PL610",
                                          "PL611",
                                          "PL612",
                                          "IE613",
                                          "LV614"
                                      ],
                                      "type": "string",
                                      "title": "Station",
                                      "description": "These are the LOFAR stations"
                                  },
                                  "default": [],
                                  "minItems": 0,
                                  "uniqueItems": true,
                                  "additionalItems": false,
                                  "additionalProperties": false
                              },
                              "max_number_of_missing_stations": {
                                  "type": "integer",
                                  "title": "Maximum number of stations to omit",
                                  "default": 0,
                                  "minimum": 0,
                                  "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)"
                              }
                          }
                      }
                  ],
                  "title": "Station group",
                  "default": {
                      "stations": [
                          "CS002",
                          "CS003",
                          "CS004",
                          "CS005",
                          "CS006",
                          "CS007"
                      ],
                      "max_nr_missing": 1
                  },
                  "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)"
              },
              "station_list": {
                  "type": "array",
                  "items": {
                      "enum": [
                          "CS001",
                          "CS002",
                          "CS003",
                          "CS004",
                          "CS005",
                          "CS006",
                          "CS007",
                          "CS011",
                          "CS013",
                          "CS017",
                          "CS021",
                          "CS024",
                          "CS026",
                          "CS028",
                          "CS030",
                          "CS031",
                          "CS032",
                          "CS101",
                          "CS103",
                          "CS201",
                          "CS301",
                          "CS302",
                          "CS401",
                          "CS501",
                          "RS106",
                          "RS205",
                          "RS208",
                          "RS210",
                          "RS305",
                          "RS306",
                          "RS307",
                          "RS310",
                          "RS406",
                          "RS407",
                          "RS409",
                          "RS503",
                          "RS508",
                          "RS509",
                          "DE601",
                          "DE602",
                          "DE603",
                          "DE604",
                          "DE605",
                          "FR606",
                          "SE607",
                          "UK608",
                          "DE609",
                          "PL610",
                          "PL611",
                          "PL612",
                          "IE613",
                          "LV614"
                      ],
                      "type": "string",
                      "title": "Station",
                      "description": "These are the LOFAR stations"
                  },
                  "default": [],
                  "minItems": 0,
                  "uniqueItems": true,
                  "additionalItems": false,
                  "additionalProperties": false
              },
              "max_number_of_missing_stations": {
                  "type": "integer",
                  "title": "Maximum number of stations to omit",
                  "default": 0,
                  "minimum": 0,
                  "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)"
              }
          }
      },
        "tags": [],
        "type": "http://localhost:3000/api/task_type/observation",
        "type_value": "observation",
        "updated_at": "2022-02-04T05:10:04.704405",
        "version": 1
      },
      {
        "id": 7,
        "url": "http://localhost:3000/api/task_template/7",
        "created_at": "2022-02-04T05:10:04.781114",
        "description": "This schema combines the calibrator, target and beamformer observation task schema's for parallel observing.",
        "name": "parallel calibrator target and beamforming observation",
        "schema": {
          "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20and%20beamforming%20observation/1#",
          "type": "object",
          "title": "parallel calibrator target and beamforming observation",
          "$schema": "http://json-schema.org/draft-06/schema#",
          "version": 1,
          "required": [
            "calibrator",
            "target",
            "beamforming"
          ],
          "properties": {
            "target": {
              "type": "object",
              "default": {},
              "required": [
                "station_groups",
                "antenna_set",
                "filter",
                "SAPs",
                "duration",
                "correlator"
              ],
              "properties": {
                "QA": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/1#/definitions/QA",
                  "default": {}
                },
                "SAPs": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/SAPs",
                  "default": [
                    {}
                  ],
                  "minItems": 1
                },
                "filter": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/filter",
                  "default": "HBA_110_190"
                },
                "duration": {
                  "$id": "#duration",
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/1/#/definitions/timedelta",
                  "title": "Duration",
                  "default": 300,
                  "minimum": 1,
                  "description": "Duration of this observation (seconds)"
                },
                "tile_beam": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pointing/1/#/definitions/pointing",
                  "title": "Tile beam",
                  "default": {},
                  "description": "HBA only"
                },
                "correlator": {
                  "type": "object",
                  "title": "Correlator Settings",
                  "default": {},
                  "required": [
                    "channels_per_subband",
                    "topocentric_frequency_correction",
                    "integration_time",
                    "storage_cluster"
                  ],
                  "properties": {
                    "storage_cluster": {
                      "enum": [
                        "CEP4",
                        "DragNet"
                      ],
                      "type": "string",
                      "title": "Storage cluster",
                      "default": "CEP4",
                      "description": "Cluster to write output to"
                    },
                    "integration_time": {
                      "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/1/#/definitions/timedelta",
                      "title": "Integration time",
                      "default": 1,
                      "minimum": 0.1,
                      "description": "Desired integration period (seconds)"
                    },
                    "channels_per_subband": {
                      "enum": [
                        8,
                        16,
                        32,
                        64,
                        128,
                        256,
                        512,
                        1024
                      ],
                      "type": "integer",
                      "title": "Channels/subband",
                      "default": 64,
                      "minimum": 8,
                      "description": "Number of frequency bands per subband"
                    },
                    "topocentric_frequency_correction": {
                      "type": "boolean",
                      "title": "Topocentric Frequency Correction",
                      "default": false,
                      "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation."
                    }
                  },
                  "additionalProperties": false
                },
                "antenna_set": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/antenna_set",
                  "default": "HBA_DUAL"
                },
                "station_groups": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1#/definitions/station_groups",
                  "default": [
                    {
                      "stations": [
                        "CS002",
                        "CS003",
                        "CS004",
                        "CS005",
                        "CS006",
                        "CS007"
                      ],
                      "max_nr_missing": 1
                    }
                  ]
                }
              },
              "description": "This subschema defines the parameters to setup the target observation part."
            },
            "calibrator": {
              "type": "object",
              "default": {},
              "required": [
                "autoselect",
                "pointing",
                "name"
              ],
              "properties": {
                "name": {
                  "type": "string",
                  "title": "Name",
                  "default": "calibrator",
                  "description": "Name of the calibrator SAP"
                },
                "pointing": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pointing/1/#/definitions/pointing",
                  "title": "Digital pointing",
                  "default": {},
                  "description": "Manually selected calibrator"
                },
                "autoselect": {
                  "type": "boolean",
                  "title": "Auto-select",
                  "default": true,
                  "description": "Auto-select calibrator based on elevation"
                }
              },
              "description": "This subschema defines the (extra) parameters to setup a calibrator observation task, which uses all paramters from the target observation task which it is linked to, plus these calibrator overrides."
            },
            "beamforming": {
              "type": "object",
              "default": {},
              "required": [
                "station_groups",
                "antenna_set",
                "filter",
                "tile_beam",
                "SAPs",
                "duration",
                "beamformers",
                "processing"
              ],
              "properties": {
                "SAPs": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/SAPs",
                  "default": [
                    {}
                  ],
                  "minItems": 0
                },
                "filter": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/filter",
                  "default": "HBA_110_190"
                },
                "duration": {
                  "$id": "#duration",
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/1/#/definitions/timedelta",
                  "title": "Duration",
                  "default": 300,
                  "minimum": 1,
                  "description": "Duration of this observation (seconds)"
                },
                "tile_beam": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pointing/1/#/definitions/pointing",
                  "title": "Tile beam",
                  "default": {},
                  "description": "HBA only"
                },
                "processing": {
                  "type": "object",
                  "title": "Processing",
                  "default": {},
                  "required": [
                    "beamformer_ppf"
                  ],
                  "properties": {
                    "beamformer_ppf": {
                      "type": "boolean",
                      "title": "Use PPF when beamforming",
                      "default": false,
                      "description": "Use a PPF for the channellisation for beamforming. If not, an FFT is used."
                    }
                  },
                  "description": "General processing parameters",
                  "additionalProperties": false
                },
                "antenna_set": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/antenna_set",
                  "default": "HBA_DUAL"
                },
                "beamformers": {
                  "type": "array",
                  "items": {
                    "type": "object",
                    "title": "Beamformer",
                    "default": {},
                    "required": [
                      "name"
                    ],
                    "properties": {
                      "name": {
                        "type": "string",
                        "title": "Name",
                        "default": "_beamformer_name_",
                        "minLength": 1,
                        "description": "Beamformer name, used for identification purposes."
                      },
                      "coherent": {
                        "type": "object",
                        "title": "Coherent Tied-Array Beams",
                        "default": {},
                        "required": [
                          "SAPs",
                          "settings"
                        ],
                        "properties": {
                          "SAPs": {
                            "type": "array",
                            "items": {
                              "type": "object",
                              "required": [
                                "name",
                                "tabs"
                              ],
                              "properties": {
                                "name": {
                                  "type": "string",
                                  "title": "SAP name",
                                  "default": "_SAP_to_beamform_",
                                  "minLength": 1,
                                  "description": "Name of the SAP to beamform"
                                },
                                "tabs": {
                                  "type": "array",
                                  "items": {
                                    "type": "object",
                                    "title": "Tied-Array Beam",
                                    "default": {},
                                    "required": [
                                      "pointing",
                                      "relative"
                                    ],
                                    "properties": {
                                      "pointing": {
                                        "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pointing/1/#/definitions/pointing",
                                        "default": {}
                                      },
                                      "relative": {
                                        "type": "boolean",
                                        "title": "Relative to SAP",
                                        "default": false,
                                        "description": "Angular offset applied to the TAB pointing"
                                      }
                                    },
                                    "headerTemplate": "TAB {{ self.index }}",
                                    "additionalProperties": false
                                  },
                                  "title": "Tied-Array Beams",
                                  "default": [],
                                  "description": "Tied-array beams to form",
                                  "additionalItems": false
                                },
                                "subbands": {
                                  "$ref": "#/definitions/subband_selection",
                                  "default": {}
                                },
                                "tab_rings": {
                                  "type": "object",
                                  "title": "Tied-Array Rings",
                                  "default": {},
                                  "properties": {
                                    "count": {
                                      "type": "integer",
                                      "title": "Number of rings",
                                      "default": 0,
                                      "maximum": 11,
                                      "minimum": 0
                                    },
                                    "width": {
                                      "type": "number",
                                      "title": "Ring width",
                                      "default": 0.01,
                                      "minimum": 0,
                                      "description": "Distance between pointings."
                                    }
                                  },
                                  "description": "Rings of TABs around the center of the beam.",
                                  "additonalProperties": false
                                }
                              },
                              "additionalProperties": false
                            },
                            "title": "SAPs",
                            "default": [],
                            "minItems": 0,
                            "description": "Which SAPs in the observation to beamform.",
                            "additionalItems": false
                          },
                          "settings": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/1#/definitions/stokes_settings",
                            "default": {}
                          }
                        },
                        "additionalProperties": false
                      },
                      "flys eye": {
                        "type": "object",
                        "title": "Fly's Eye Settings",
                        "default": {},
                        "required": [
                          "enabled"
                        ],
                        "properties": {
                          "enabled": {
                            "type": "boolean",
                            "title": "Enable Fly's Eye",
                            "default": false
                          },
                          "settings": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/1#/definitions/stokes_settings",
                            "default": {}
                          }
                        },
                        "description": "Produce beams containing the individual station signals",
                        "additionalProperties": false
                      },
                      "incoherent": {
                        "type": "object",
                        "title": "Incoherent Tied-Array Beams",
                        "default": {},
                        "required": [
                          "settings",
                          "SAPs"
                        ],
                        "properties": {
                          "SAPs": {
                            "type": "array",
                            "items": {
                              "type": "object",
                              "default": {},
                              "required": [
                                "name",
                                "subbands"
                              ],
                              "properties": {
                                "name": {
                                  "type": "string",
                                  "title": "SAP name",
                                  "default": "_SAP_to_beamform_",
                                  "minLength": 1,
                                  "description": "Name of the SAP to beamform"
                                },
                                "subbands": {
                                  "$ref": "#/definitions/subband_selection",
                                  "default": {}
                                }
                              },
                              "additionalProperties": false
                            },
                            "title": "SAPs",
                            "default": [],
                            "minItems": 0,
                            "description": "Which SAPs in the observation to create incoherent TABs for (empty list = all).",
                            "additionalItems": false
                          },
                          "settings": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/1#/definitions/stokes_settings",
                            "default": {}
                          }
                        },
                        "additionalProperties": false
                      },
                      "station_groups": {
                        "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1#/definitions/station_groups",
                        "default": [
                          {
                            "stations": [
                              "CS001",
                              "CS002",
                              "CS003",
                              "CS004",
                              "CS005",
                              "CS006",
                              "CS007",
                              "CS011",
                              "CS013",
                              "CS017",
                              "CS021",
                              "CS024",
                              "CS026",
                              "CS028",
                              "CS030",
                              "CS031",
                              "CS032",
                              "CS301",
                              "CS302",
                              "CS401",
                              "CS501"
                            ],
                            "max_nr_missing": 1
                          }
                        ],
                        "minItems": 1,
                        "description": "While observing, COBALT will beamform on the intersection of all stations in this list and the used stations in the observation. So, specifying all possible stations here means that all observation-stations are used. Specifying a small subset here means that only the observing-stations in this small list are used. By default we let COBALT beamform on the Core stations."
                      }
                    },
                    "headerTemplate": "Beamformer {{ self.index }}",
                    "additionalProperties": false
                  },
                  "title": "Beamformers",
                  "default": [
                    {}
                  ],
                  "minItems": 1,
                  "additionalItems": false
                },
                "station_groups": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1#/definitions/station_groups",
                  "default": [
                    {
                      "stations": [
                        "CS002",
                        "CS003",
                        "CS004",
                        "CS005",
                        "CS006",
                        "CS007"
                      ],
                      "max_nr_missing": 1
                    }
                  ]
                }
              },
              "description": "This subschema defines the parameters for an observation that forms tied-array beams in COBALT."
            }
          },
          "definitions": {
            "subband_selection": {
              "type": "object",
              "title": "Subband selection",
              "default": {},
              "required": [
                "method"
              ],
              "properties": {
                "list": {
                  "type": "array",
                  "items": {
                    "type": "integer",
                    "title": "Subband",
                    "maximum": 511,
                    "minimum": 0
                  },
                  "title": "Subset selection",
                  "default": [],
                  "minItems": 0,
                  "description": "If method is 'select subset', only beamform these subbands, and only if they occur in the SAP.",
                  "additionalItems": false
                },
                "method": {
                  "enum": [
                    "copy",
                    "largest continuous subset",
                    "select subset"
                  ],
                  "type": "string",
                  "title": "Method",
                  "default": "copy",
                  "description": "How to select the subbands to beam form"
                }
              },
              "additionalProperties": false
            }
          },
          "description": "This schema combines the calibrator, target and beamformer observation task schema's for parallel observing."
        },
        "tags": [],
        "type": "http://localhost:3000/api/task_type/observation",
        "type_value": "observation",
        "updated_at": "2022-02-04T05:10:04.781151",
        "version": 1
      },
      {
        "id": 8,
        "url": "http://localhost:3000/api/task_template/8",
        "created_at": "2022-02-04T05:10:04.810942",
        "description": "This schema defines the parameters for a preprocessing pipeline.",
        "name": "preprocessing pipeline",
        "schema": {
          "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/1#",
          "type": "object",
          "title": "preprocessing pipeline",
          "$schema": "http://json-schema.org/draft-06/schema#",
          "version": 1,
          "required": [
            "average"
          ],
          "properties": {
            "flag": {
              "type": "object",
              "title": "Flagging",
              "default": {},
              "required": [
                "outerchannels",
                "autocorrelations",
                "rfi_strategy"
              ],
              "properties": {
                "rfi_strategy": {
                  "enum": [
                    "none",
                    "HBAdefault",
                    "LBAdefault"
                  ],
                  "type": "string",
                  "title": "RFI flagging strategy",
                  "default": "HBAdefault"
                },
                "outerchannels": {
                  "type": "boolean",
                  "title": "Flag outer channels",
                  "default": true
                },
                "autocorrelations": {
                  "type": "boolean",
                  "title": "Flag auto correlations",
                  "default": true
                }
              },
              "additionalProperties": false
            },
            "demix": {
              "type": "object",
              "title": "Demixing",
              "default": {},
              "required": [],
              "properties": {
                "sources": {
                  "type": "array",
                  "items": {
                    "enum": [
                      "CasA",
                      "CygA",
                      "HerA",
                      "HydraA",
                      "TauA",
                      "VirA"
                    ],
                    "type": "string",
                    "default": "CasA"
                  },
                  "maxItems": 2,
                  "minItems": 0,
                  "uniqueItems": true,
                  "additionalItems": false
                },
                "time_steps": {
                  "type": "integer",
                  "title": "Time steps",
                  "default": 10,
                  "minimum": 1,
                  "description": "Must be a multiple of the averaging time steps"
                },
                "ignore_target": {
                  "type": "boolean",
                  "title": "Ignore target",
                  "default": false
                },
                "frequency_steps": {
                  "type": "integer",
                  "title": "Frequency steps",
                  "default": 64,
                  "minimum": 1,
                  "description": "Must be a multiple of the averaging frequency steps"
                }
              },
              "additionalProperties": false
            },
            "average": {
              "type": "object",
              "title": "Averaging",
              "default": {},
              "required": [
                "frequency_steps",
                "time_steps"
              ],
              "properties": {
                "time_steps": {
                  "type": "integer",
                  "title": "Time steps",
                  "default": 1,
                  "minimum": 1
                },
                "frequency_steps": {
                  "type": "integer",
                  "title": "Frequency steps",
                  "default": 4,
                  "minimum": 1
                }
              },
              "additionalProperties": false
            },
            "storagemanager": {
              "enum": [
                "standard",
                "dysco"
              ],
              "type": "string",
              "title": "Storage Manager",
              "default": "dysco"
            },
            "cluster_resources": {
              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/1#/definitions/cluster_resources",
              "default": {}
            }
          },
          "description": "This schema defines the parameters for a preprocessing pipeline."
        },
        "tags": [],
        "type": "http://localhost:3000/api/task_type/pipeline",
        "type_value": "pipeline",
        "updated_at": "2022-02-04T05:10:04.810955",
        "version": 1
      },
      {
        "id": 9,
        "url": "http://localhost:3000/api/task_template/9",
        "created_at": "2022-02-04T05:10:04.835682",
        "description": "This schema defines the parameters for a pulsar pipeline.",
        "name": "pulsar pipeline",
        "schema": {
          "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/pulsar%20pipeline/1#",
          "type": "object",
          "title": "pulsar pipeline",
          "$schema": "http://json-schema.org/draft-07/schema#",
          "version": 1,
          "required": [
            "pulsar",
            "presto",
            "dspsr",
            "output"
          ],
          "properties": {
            "dspsr": {
              "type": "object",
              "title": "DSPSR",
              "default": {},
              "properties": {
                "digifil": {
                  "type": "object",
                  "title": "Digifil",
                  "default": {},
                  "properties": {
                    "dm": {
                      "type": "number",
                      "title": "DM",
                      "default": -1,
                      "minimum": -1,
                      "desciption": "Dispersion Measure (-1 for none)"
                    },
                    "integration_time": {
                      "type": "number",
                      "title": "Integration time",
                      "default": 4,
                      "minimum": 0.1
                    },
                    "frequency_channels": {
                      "type": "integer",
                      "title": "Frequency channels",
                      "default": 512,
                      "maximum": 512,
                      "minimum": 1,
                      "description": "Number of frequency channels (multiple of subbands/part)"
                    },
                    "coherent_dedispersion": {
                      "type": "boolean",
                      "title": "Coherent Dedispersion",
                      "default": true
                    }
                  },
                  "description": "Processes single pulses in CV data if single-pulse search is enabled",
                  "additionalProperties": false
                },
                "enabled": {
                  "type": "boolean",
                  "title": "Enabled",
                  "default": true
                },
                "filterbank": {
                  "type": "object",
                  "title": "Create filter bank",
                  "default": {},
                  "properties": {
                    "enabled": {
                      "type": "boolean",
                      "title": "Enabled",
                      "default": false
                    },
                    "frequency_channels": {
                      "type": "integer",
                      "title": "Frequency channels",
                      "default": 120,
                      "maximum": 512,
                      "minimum": 1,
                      "description": "Number of frequency channels (multiple of subbands/part)"
                    },
                    "coherent_dedispersion": {
                      "type": "boolean",
                      "title": "Coherent Dedispersion",
                      "default": true
                    }
                  },
                  "additionalProperties": false
                },
                "rfi_excision": {
                  "type": "boolean",
                  "title": "RFI excision",
                  "default": true,
                  "description": "Excise/clean/remove detected RFI"
                },
                "optimise_period_dm": {
                  "type": "boolean",
                  "title": "Optimise period & DM",
                  "default": true
                },
                "subintegration_length": {
                  "type": "integer",
                  "title": "Subintegration length",
                  "default": -1,
                  "minimum": -1
                },
                "single_pulse_subintegration": {
                  "type": "boolean",
                  "title": "Single pulse subintegration",
                  "default": false,
                  "description": "Create single-pulse subintegrations with inter-channel dispersion delays removed."
                }
              },
              "additionalProperties": false
            },
            "output": {
              "type": "object",
              "title": "Output",
              "default": {},
              "properties": {
                "quantisation": {
                  "type": "object",
                  "title": "Quantisation",
                  "default": {},
                  "properties": {
                    "scale": {
                      "type": "number",
                      "title": "Conversion sigma",
                      "default": 5,
                      "minimum": 1,
                      "description": "Conversion sigma to use when converting to 8-bit samples"
                    },
                    "enabled": {
                      "type": "boolean",
                      "title": "Enabled",
                      "default": false
                    }
                  },
                  "description": "Quantise output into 8-bit samples",
                  "additionalProperties": false
                },
                "dynamic_spectrum": {
                  "type": "object",
                  "title": "Dynamic Spectrum",
                  "default": {},
                  "properties": {
                    "enabled": {
                      "type": "boolean",
                      "title": "Enabled",
                      "default": false
                    },
                    "time_average": {
                      "type": "number",
                      "title": "Time average",
                      "default": 0.5,
                      "minimum": 0.01
                    }
                  },
                  "additionalProperties": false
                }
              },
              "additionalProperties": false
            },
            "presto": {
              "type": "object",
              "title": "PRESTO",
              "default": {},
              "properties": {
                "input": {
                  "type": "object",
                  "title": "Input",
                  "properties": {
                    "nr_blocks": {
                      "type": "integer",
                      "title": "Nr of blocks",
                      "default": 100,
                      "minimum": 1,
                      "description": "Number of blocks to read at a time"
                    },
                    "decode_sigma": {
                      "type": "number",
                      "title": "Decode sigma",
                      "default": 3,
                      "minimum": 1,
                      "description": "Sigma threshold for decoding"
                    },
                    "samples_per_block": {
                      "type": "integer",
                      "title": "Block size (samples)",
                      "default": 8192,
                      "minimum": 16
                    }
                  },
                  "additionalProperties": false
                },
                "rrats": {
                  "type": "object",
                  "title": "RRATs analysis",
                  "default": {},
                  "properties": {
                    "enabled": {
                      "type": "boolean",
                      "title": "Enabled",
                      "default": false
                    },
                    "dm_range": {
                      "type": "number",
                      "title": "DM range",
                      "default": 5,
                      "minimum": 0
                    }
                  },
                  "additionalProperties": false
                },
                "rfifind": {
                  "type": "object",
                  "title": "rfifind",
                  "default": {},
                  "properties": {
                    "blocks": {
                      "type": "integer",
                      "title": "blocks",
                      "default": 16,
                      "minimum": 1
                    }
                  },
                  "additionalProperties": false
                },
                "prepdata": {
                  "type": "object",
                  "title": "prepdata",
                  "default": {},
                  "properties": {
                    "dm": {
                      "type": "number",
                      "title": "DM",
                      "default": -1,
                      "minimum": -1,
                      "desciption": "Dispersion Measure (-1 for none)"
                    }
                  },
                  "additionalProperties": false
                },
                "prepfold": {
                  "type": "boolean",
                  "title": "Enable prepfold",
                  "default": true
                },
                "fold_profile": {
                  "type": "boolean",
                  "title": "Fold",
                  "default": true,
                  "description": "Fold the pulsar profile"
                }
              },
              "additionalProperties": false
            },
            "pulsar": {
              "type": "object",
              "title": "Pulsar to fold",
              "default": {},
              "properties": {
                "name": {
                  "type": "string",
                  "title": "Name",
                  "default": "",
                  "description": "Name of the pulsar to fold, if strategy=manual"
                },
                "strategy": {
                  "enum": [
                    "manual",
                    "meta",
                    "sapfind",
                    "sapfind3",
                    "tabfind",
                    "tabfind+"
                  ],
                  "type": "string",
                  "title": "Strategy",
                  "default": "manual",
                  "description": "How to look up the pulsar to fold",
                  "enum_titles": [
                    "Pulsar name specified here",
                    "SAP target is pulsar name",
                    "Brightest known pulsar in SAP",
                    "Brightest 3 known pulsars in SAP",
                    "Brightest known pulsar in TAB",
                    "Brightest 3 known pulsars in TAB"
                  ]
                }
              },
              "additionalProperties": false
            },
            "cluster_resources": {
              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/1#/definitions/cluster_resources",
              "default": {}
            },
            "single_pulse_search": {
              "type": "boolean",
              "title": "Single-pulse search",
              "default": false,
              "description": "Instructs PRESTO to process single pulses, and enables Digifil for CV data"
            }
          },
          "description": "This schema defines the parameters for a pulsar pipeline."
        },
        "tags": [],
        "type": "http://localhost:3000/api/task_type/pipeline",
        "type_value": "pipeline",
        "updated_at": "2022-02-04T05:10:04.835700",
        "version": 1
      },
      {
        "id": 10,
        "url": "http://localhost:3000/api/task_template/10",
        "created_at": "2022-02-04T05:10:04.847301",
        "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
        "name": "parallel calibrator target observation",
        "schema": {
          "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/1#",
          "type": "object",
          "title": "parallel calibrator target observation",
          "$schema": "http://json-schema.org/draft-06/schema#",
          "version": 1,
          "required": [
            "calibrator",
            "target"
          ],
          "properties": {
            "target": {
              "type": "object",
              "default": {},
              "required": [
                "station_groups",
                "antenna_set",
                "filter",
                "SAPs",
                "duration",
                "correlator"
              ],
              "properties": {
                "QA": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/1#/definitions/QA",
                  "default": {}
                },
                "SAPs": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/SAPs",
                  "default": [
                    {}
                  ],
                  "minItems": 1
                },
                "filter": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/filter",
                  "default": "HBA_110_190"
                },
                "duration": {
                  "$id": "#duration",
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/1/#/definitions/timedelta",
                  "title": "Duration",
                  "default": 300,
                  "minimum": 1,
                  "description": "Duration of this observation (seconds)"
                },
                "tile_beam": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pointing/1/#/definitions/pointing",
                  "title": "Tile beam",
                  "default": {},
                  "description": "HBA only"
                },
                "correlator": {
                  "type": "object",
                  "title": "Correlator Settings",
                  "default": {},
                  "required": [
                    "channels_per_subband",
                    "topocentric_frequency_correction",
                    "integration_time",
                    "storage_cluster"
                  ],
                  "properties": {
                    "storage_cluster": {
                      "enum": [
                        "CEP4",
                        "DragNet"
                      ],
                      "type": "string",
                      "title": "Storage cluster",
                      "default": "CEP4",
                      "description": "Cluster to write output to"
                    },
                    "integration_time": {
                      "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/1/#/definitions/timedelta",
                      "title": "Integration time",
                      "default": 1,
                      "minimum": 0.1,
                      "description": "Desired integration period (seconds)"
                    },
                    "channels_per_subband": {
                      "enum": [
                        8,
                        16,
                        32,
                        64,
                        128,
                        256,
                        512,
                        1024
                      ],
                      "type": "integer",
                      "title": "Channels/subband",
                      "default": 64,
                      "minimum": 8,
                      "description": "Number of frequency bands per subband"
                    },
                    "topocentric_frequency_correction": {
                      "type": "boolean",
                      "title": "Topocentric Frequency Correction",
                      "default": false,
                      "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation."
                    }
                  },
                  "additionalProperties": false
                },
                "antenna_set": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1/#/definitions/antenna_set",
                  "default": "HBA_DUAL"
                },
                "station_groups": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/stations/1#/definitions/station_groups",
                  "default": [
                    {
                      "stations": [
                        "CS002",
                        "CS003",
                        "CS004",
                        "CS005",
                        "CS006",
                        "CS007"
                      ],
                      "max_nr_missing": 1
                    }
                  ]
                }
              },
              "description": "This subschema defines the parameters to setup the target observation part."
            },
            "calibrator": {
              "type": "object",
              "default": {},
              "required": [
                "autoselect",
                "pointing",
                "name"
              ],
              "properties": {
                "name": {
                  "type": "string",
                  "title": "Name",
                  "default": "calibrator",
                  "description": "Name of the calibrator SAP"
                },
                "pointing": {
                  "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pointing/1/#/definitions/pointing",
                  "title": "Digital pointing",
                  "default": {},
                  "description": "Manually selected calibrator"
                },
                "autoselect": {
                  "type": "boolean",
                  "title": "Auto-select",
                  "default": true,
                  "description": "Auto-select calibrator based on elevation"
                }
              },
              "description": "This subschema defines the (extra) parameters to setup a calibrator observation task, which uses all paramters from the target observation task which it is linked to, plus these calibrator overrides."
            }
          },
          "description": "This schema combines the calibrator and target observation task schema's for parallel observing."
        },
        "tags": [],
        "type": "http://localhost:3000/api/task_type/observation",
        "type_value": "observation",
        "updated_at": "2022-02-04T05:10:04.847333",
        "version": 1
      }
    ],
    getTaskTemplates: () => {
      return parseTemplatesToCorrectJSONFormat(templates);
    },
    getTaskFilterDefinition: {
        'data':{
          "name": "Task Draft List",
          "description": "",
          "renders": [
              "application/json",
              "text/html"
          ],
          "parses": [
              "application/json",
              "application/x-www-form-urlencoded",
              "multipart/form-data"
          ],
          "actions": {
              "POST": {
                  "id": {
                      "type": "integer",
                      "required": false,
                      "read_only": true,
                      "label": "ID"
                  },
                  "url": {
                      "type": "field",
                      "required": false,
                      "read_only": true,
                      "label": "Url"
                  },
                  "connector_types": {
                      "type": "field",
                      "required": false,
                      "read_only": true,
                      "label": "connector_types",
                      "help_text": "The connector types which define what kind of data this task consumes/produces.",
                      "child": {
                          "type": "nested object",
                          "required": false,
                          "read_only": true,
                          "label": "connector_types",
                          "help_text": "The connector types which define what kind of data this task consumes/produces.",
                          "children": {
                              "id": {
                                  "type": "integer",
                                  "required": false,
                                  "read_only": true,
                                  "label": "ID"
                              },
                              "url": {
                                  "type": "field",
                                  "required": false,
                                  "read_only": true,
                                  "label": "Url"
                              },
                              "dataformat": {
                                  "type": "field",
                                  "required": true,
                                  "read_only": false,
                                  "label": "Dataformat"
                              },
                              "dataformat_value": {
                                  "type": "field",
                                  "required": false,
                                  "read_only": true,
                                  "label": "dataformat_value"
                              },
                              "datatype": {
                                  "type": "field",
                                  "required": true,
                                  "read_only": false,
                                  "label": "Datatype"
                              },
                              "datatype_value": {
                                  "type": "field",
                                  "required": false,
                                  "read_only": true,
                                  "label": "datatype_value"
                              },
                              "iotype": {
                                  "type": "field",
                                  "required": true,
                                  "read_only": false,
                                  "label": "Iotype",
                                  "help_text": "Is this connector an input or output"
                              },
                              "iotype_value": {
                                  "type": "field",
                                  "required": false,
                                  "read_only": true,
                                  "label": "iotype_value"
                              },
                              "role": {
                                  "type": "field",
                                  "required": true,
                                  "read_only": false,
                                  "label": "Role"
                              },
                              "role_value": {
                                  "type": "field",
                                  "required": false,
                                  "read_only": true,
                                  "label": "role_value"
                              },
                              "task_template": {
                                  "type": "field",
                                  "required": true,
                                  "read_only": false,
                                  "label": "Task template"
                              },
                              "task_template_id": {
                                  "type": "field",
                                  "required": false,
                                  "read_only": true,
                                  "label": "Task template id"
                              }
                          }
                      }
                  },
                  "consumed_by": {
                      "type": "field",
                      "required": true,
                      "read_only": false,
                      "label": "Consumed by"
                  },
                  "consumed_by_ids": {
                      "type": "field",
                      "required": false,
                      "read_only": true,
                      "label": "consumed_by_ids"
                  },
                  "created_at": {
                      "type": "datetime",
                      "required": false,
                      "read_only": true,
                      "label": "Created at",
                      "help_text": "Moment of object creation."
                  },
                  "description": {
                      "type": "string",
                      "required": false,
                      "read_only": false,
                      "label": "Description",
                      "help_text": "A longer description of this object.",
                      "max_length": 255
                  },
                  "duration": {
                      "type": "float",
                      "required": false,
                      "read_only": true,
                      "label": "Duration"
                  },
                  "first_scheduling_relation": {
                      "type": "field",
                      "required": true,
                      "read_only": false,
                      "label": "First scheduling relation"
                  },
                  "first_scheduling_relation_ids": {
                      "type": "field",
                      "required": false,
                      "read_only": true,
                      "label": "first_scheduling_relation_ids"
                  },
                  "name": {
                      "type": "string",
                      "required": true,
                      "read_only": false,
                      "label": "Name",
                      "help_text": "Human-readable name of this object.",
                      "max_length": 128
                  },
                  "output_pinned": {
                      "type": "boolean",
                      "required": false,
                      "read_only": false,
                      "label": "Output pinned",
                      "help_text": "True if the output of this task is pinned to disk, that is, forbidden to be removed."
                  },
                  "produced_by": {
                      "type": "field",
                      "required": true,
                      "read_only": false,
                      "label": "Produced by"
                  },
                  "produced_by_ids": {
                      "type": "field",
                      "required": false,
                      "read_only": true,
                      "label": "produced_by_ids"
                  },
                  "relative_start_time": {
                      "type": "float",
                      "required": false,
                      "read_only": true,
                      "label": "Relative start time"
                  },
                  "relative_stop_time": {
                      "type": "float",
                      "required": false,
                      "read_only": true,
                      "label": "Relative stop time"
                  },
                  "scheduling_unit_draft": {
                      "type": "field",
                      "required": true,
                      "read_only": false,
                      "label": "Scheduling unit draft",
                      "help_text": "Scheduling Unit draft to which this task draft belongs."
                  },
                  "scheduling_unit_draft_id": {
                      "type": "field",
                      "required": false,
                      "read_only": true,
                      "label": "Scheduling unit draft id"
                  },
                  "second_scheduling_relation": {
                      "type": "field",
                      "required": true,
                      "read_only": false,
                      "label": "Second scheduling relation"
                  },
                  "second_scheduling_relation_ids": {
                      "type": "field",
                      "required": false,
                      "read_only": true,
                      "label": "second_scheduling_relation_ids"
                  },
                  "short_description": {
                      "type": "string",
                      "required": false,
                      "read_only": false,
                      "label": "Short description",
                      "help_text": "A short description of this task, usually the name of the target and abbreviated task type.",
                      "max_length": 32
                  },
                  "specifications_doc": {
                      "type": "field",
                      "required": true,
                      "read_only": false,
                      "label": "Specifications doc"
                  },
                  "specifications_template": {
                      "type": "field",
                      "required": true,
                      "read_only": false,
                      "label": "Specifications template",
                      "help_text": "Schema used for specifications_doc."
                  },
                  "specifications_template_id": {
                      "type": "field",
                      "required": false,
                      "read_only": true,
                      "label": "Specifications template id"
                  },
                  "tags": {
                      "type": "list",
                      "required": false,
                      "read_only": false,
                      "label": "Tags",
                      "help_text": "User-defined search keywords for object.",
                      "child": {
                          "type": "string",
                          "required": true,
                          "read_only": false,
                          "label": "Tags",
                          "max_length": 128
                      }
                  },
                  "task_blueprints": {
                      "type": "field",
                      "required": true,
                      "read_only": false,
                      "label": "Task blueprints"
                  },
                  "task_blueprints_ids": {
                      "type": "field",
                      "required": false,
                      "read_only": true,
                      "label": "task_blueprints_ids"
                  },
                  "task_type": {
                      "type": "field",
                      "required": false,
                      "read_only": true,
                      "label": "task_type",
                      "help_text": "The task type as defined in the specifications template."
                  },
                  "updated_at": {
                      "type": "datetime",
                      "required": false,
                      "read_only": true,
                      "label": "Updated at",
                      "help_text": "Moment of last object update."
                  }
              }
          },
          "filters": {
              "id": {
                  "type": "NumberInFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "tags": {
                  "type": "CharFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "created_at": {
                  "type": "IsoDateTimeFromToRangeFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "updated_at": {
                  "type": "IsoDateTimeFromToRangeFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "name": {
                  "type": "CharFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "description": {
                  "type": "CharFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "short_description": {
                  "type": "CharFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "specifications_doc": {
                  "type": "CharFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "scheduling_unit_draft": {
                  "type": "ModelChoiceInFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "specifications_template": {
                  "type": "ModelChoiceFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "output_pinned": {
                  "type": "BooleanFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "id_min": {
                  "type": "NumberFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "id_max": {
                  "type": "NumberFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "scheduling_unit_draft_min": {
                  "type": "NumberFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "scheduling_unit_draft_max": {
                  "type": "NumberFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "scheduling_unit_draft_name": {
                  "type": "CharFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "project": {
                  "type": "PropertyCharFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "task_blueprints": {
                  "type": "ModelMultipleChoiceFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "duration_min": {
                  "type": "PropertyDurationFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "duration_max": {
                  "type": "PropertyDurationFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "relative_start_time_min": {
                  "type": "PropertyDurationFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "relative_start_time_max": {
                  "type": "PropertyDurationFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "relative_stop_time_min": {
                  "type": "PropertyDurationFilter",
                  "lookup_types": [
                      "exact"
                  ]
              },
              "relative_stop_time_max": {
                  "type": "PropertyDurationFilter",
                  "lookup_types": [
                      "exact"
                  ]
              }
          },
          "ordering": [
              "id",
              "tags",
              "created_at",
              "updated_at",
              "name",
              "description",
              "short_description",
              "specifications_doc",
              "scheduling_unit_draft",
              "specifications_template",
              "output_pinned"
          ]
      }
    },
    subTaskTypes: [
      {
        "value": "observation",
        "url": "http://localhost:3000/api/subtask_type/observation"
      },
      {
        "value": "pipeline",
        "url": "http://localhost:3000/api/subtask_type/pipeline"
      },
      {
        "value": "ingest",
        "url": "http://localhost:3000/api/subtask_type/ingest"
      },
      {
        "value": "copy",
        "url": "http://localhost:3000/api/subtask_type/copy"
      },
      {
        "value": "inspection",
        "url": "http://localhost:3000/api/subtask_type/inspection"
      },
      {
        "value": "qa_files",
        "url": "http://localhost:3000/api/subtask_type/qa_files"
      },
      {
        "value": "qa_plots",
        "url": "http://localhost:3000/api/subtask_type/qa_plots"
      },
      {
        "value": "cleanup",
        "url": "http://localhost:3000/api/subtask_type/cleanup"
      },
      {
        "value": "manual",
        "url": "http://localhost:3000/api/subtask_type/manual"
      },
      {
        "value": "other",
        "url": "http://localhost:3000/api/subtask_type/other"
      }
    ],
  taskDetails: {
    "id": 1,
    "url": "http://localhost:3000/api/task_blueprint/1",
    "actual_on_sky_start_time": null,
    "actual_on_sky_stop_time": null,
    "actual_process_start_time": null,
    "actual_process_stop_time": null,
    "consumed_by": [],
    "consumed_by_ids": [],
    "created_at": "2022-02-10T08:19:03.451690",
    "description": "A simple short test observation",
    "draft": "http://localhost:3000/api/task_draft/1",
    "draft_id": 1,
    "duration": 2400,
    "first_scheduling_relation": [],
    "first_scheduling_relation_ids": [],
    "name": "Observation",
    "obsolete_since": null,
    "on_sky_duration": 1800,
    "on_sky_start_time": "2022-02-10T08:24:02.993999",
    "on_sky_stop_time": "2022-02-10T08:54:02.993999",
    "output_pinned": false,
    "process_start_time": "2022-02-10T08:24:02.993999",
    "process_stop_time": "2022-02-10T09:04:02.993999",
    "produced_by": [],
    "produced_by_ids": [],
    "relative_start_time": 0,
    "relative_stop_time": 1800,
    "scheduled_start_time": "2022-02-10T08:24:02.993999",
    "scheduled_stop_time": "2022-02-10T09:04:02.993999",
    "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
    "scheduling_unit_blueprint_id": 1,
    "second_scheduling_relation": [],
    "second_scheduling_relation_ids": [],
    "short_description": "",
    "specifications_doc": {
      "QA": {
        "plots": {
          "enabled": true,
          "autocorrelation": true,
          "crosscorrelation": true
        },
        "file_conversion": {
          "enabled": true,
          "nr_of_subbands": -1,
          "nr_of_timestamps": 256
        },
        "inspection_plots": "msplots"
      },
      "SAPs": [
        {
          "name": "Polaris",
          "subbands": [
            0,
            1,
            2,
            3,
            4,
            5,
            6,
            7,
            8,
            9,
            10,
            11,
            12,
            13,
            14,
            15,
            16,
            17,
            18,
            19,
            20,
            21,
            22,
            23,
            24,
            25,
            26,
            27,
            28,
            29,
            30,
            31,
            32,
            33,
            34,
            35,
            36,
            37,
            38,
            39,
            40,
            41,
            42,
            43,
            44,
            45,
            46,
            47,
            48,
            49,
            50,
            51,
            52,
            53,
            54,
            55,
            56,
            57,
            58,
            59,
            60,
            61,
            62,
            63,
            64,
            65,
            66,
            67,
            68,
            69,
            70,
            71,
            72,
            73,
            74,
            75,
            76,
            77,
            78,
            79,
            80,
            81,
            82,
            83,
            84,
            85,
            86,
            87,
            88,
            89,
            90,
            91,
            92,
            93,
            94,
            95,
            96,
            97,
            98,
            99,
            100,
            101,
            102,
            103,
            104,
            105,
            106,
            107,
            108,
            109,
            110,
            111,
            112,
            113,
            114,
            115,
            116,
            117,
            118,
            119,
            120,
            121,
            122,
            123,
            124,
            125,
            126,
            127,
            128,
            129,
            130,
            131,
            132,
            133,
            134,
            135,
            136,
            137,
            138,
            139,
            140,
            141,
            142,
            143,
            144,
            145,
            146,
            147,
            148,
            149,
            150,
            151,
            152,
            153,
            154,
            155,
            156,
            157,
            158,
            159,
            160,
            161,
            162,
            163,
            164,
            165,
            166,
            167,
            168,
            169,
            170,
            171,
            172,
            173,
            174,
            175,
            176,
            177,
            178,
            179,
            180,
            181,
            182,
            183,
            184,
            185,
            186,
            187,
            188,
            189,
            190,
            191,
            192,
            193,
            194,
            195,
            196,
            197,
            198,
            199,
            200,
            201,
            202,
            203,
            204,
            205,
            206,
            207,
            208,
            209,
            210,
            211,
            212,
            213,
            214,
            215,
            216,
            217,
            218,
            219,
            220,
            221,
            222,
            223,
            224,
            225,
            226,
            227,
            228,
            229,
            230,
            231,
            232,
            233,
            234,
            235,
            236,
            237,
            238,
            239,
            240,
            241,
            242,
            243
          ],
          "digital_pointing": {
            "angle1": 0.6624317181687094,
            "angle2": 1.5579526427549426,
            "target": "target1",
            "direction_type": "J2000"
          }
        }
      ],
      "filter": "HBA_110_190",
      "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/1#",
      "duration": 1800,
      "tile_beam": {
        "angle1": 0.6624317181687094,
        "angle2": 1.5579526427549426,
        "target": "target1",
        "direction_type": "J2000"
      },
      "correlator": {
        "storage_cluster": "CEP4",
        "integration_time": 1,
        "channels_per_subband": 64,
        "topocentric_frequency_correction": false
      },
      "antenna_set": "HBA_DUAL_INNER",
      "station_groups": [
        {
          "stations": [
            "CS002",
            "CS003",
            "CS004",
            "CS005",
            "CS006",
            "CS007"
          ],
          "max_nr_missing": 1
        }
      ]
    },
    "specifications_template": "http://localhost:3000/api/task_template/6",
    "specifications_template_id": 6,
    "status": "scheduled",
    "status_value": "scheduled",
    "subtasks": [
      "http://localhost:3000/api/subtask/2000001",
      "http://localhost:3000/api/subtask/2000002",
      "http://localhost:3000/api/subtask/2000000"
    ],
    "subtasks_ids": [
      2000001,
      2000002,
      2000000
    ],
    "tags": [],
    "task_type": "observation",
    "updated_at": "2022-02-10T08:19:03.451741"
  },
  taskDrafts: [
    {
      "id": 1,
      "url": "http://localhost:3000/api/task_draft/1",
      "connector_types": [
          {
              "id": 3,
              "url": "http://localhost:3000/api/task_connector_type/3",
              "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
              "dataformat_value": "MeasurementSet",
              "datatype": "http://localhost:3000/api/datatype/visibilities",
              "datatype_value": "visibilities",
              "iotype": "http://localhost:3000/api/iotype/output",
              "iotype_value": "output",
              "role": "http://localhost:3000/api/role/correlator",
              "role_value": "correlator",
              "task_template": "http://localhost:3000/api/task_template/5",
              "task_template_id": 5
          }
      ],
      "consumed_by": [
          "http://localhost:3000/api/task_relation_draft/1",
          "http://localhost:3000/api/task_relation_draft/2",
          "http://localhost:3000/api/task_relation_draft/3",
          "http://localhost:3000/api/task_relation_draft/4",
          "http://localhost:3000/api/task_relation_draft/5",
          "http://localhost:3000/api/task_relation_draft/6"
      ],
      "consumed_by_ids": [
          1,
          2,
          3,
          4,
          5,
          6
      ],
      "created_at": "2022-04-14T06:20:19.292483",
      "description": "Combined parallel Calibrator & Target Observation for LBA with 5 target beams",
      "duration": 120,
      "first_scheduling_relation": [],
      "first_scheduling_relation_ids": [],
      "name": "Combined Observation",
      "output_pinned": false,
      "produced_by": [],
      "produced_by_ids": [],
      "relative_start_time": 0,
      "relative_stop_time": 120,
      "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
      "scheduling_unit_draft_id": 1,
      "second_scheduling_relation": [],
      "second_scheduling_relation_ids": [],
      "short_description": "oXXX Paaa+01 3Cabc",
      "specifications_doc": {
          "QA": {
              "plots": {
                  "enabled": true,
                  "autocorrelation": true,
                  "crosscorrelation": true
              },
              "file_conversion": {
                  "enabled": true,
                  "nr_of_subbands": -1,
                  "nr_of_timestamps": 256
              },
              "inspection_plots": "msplots"
          },
          "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
          "duration": 120,
          "calibrator": {
              "name": "calibrator",
              "pointing": {
                  "angle1": 0.6624317181687094,
                  "angle2": 1.5579526427549426,
                  "target": "_calibrator_name_",
                  "direction_type": "J2000"
              },
              "autoselect": false
          },
          "correlator": {
              "storage_cluster": "CEP4",
              "integration_time": 1,
              "channels_per_subband": 64,
              "topocentric_frequency_correction": false
          },
          "station_configuration": {
              "SAPs": [
                  {
                      "name": "target1",
                      "subbands": [
                          74,
                          75,
                          76,
                          77,
                          78,
                          79,
                          80,
                          81,
                          82,
                          83,
                          84,
                          85,
                          86,
                          87,
                          88,
                          89,
                          90,
                          91,
                          92,
                          93,
                          94,
                          95,
                          96,
                          97,
                          98,
                          99,
                          100,
                          101,
                          102,
                          103,
                          104,
                          105,
                          106,
                          107,
                          108,
                          109,
                          110,
                          111,
                          112,
                          113,
                          114,
                          115,
                          116,
                          117,
                          118,
                          119,
                          120,
                          121,
                          122,
                          123,
                          124,
                          125,
                          126,
                          127,
                          128,
                          129,
                          130,
                          131,
                          132,
                          133,
                          134,
                          135,
                          136,
                          137,
                          138,
                          139,
                          140,
                          141,
                          142,
                          143,
                          144,
                          145,
                          146,
                          147,
                          148,
                          149,
                          150,
                          151,
                          152,
                          153,
                          154
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+01",
                          "direction_type": "J2000"
                      }
                  },
                  {
                      "name": "target2",
                      "subbands": [
                          74,
                          75,
                          76,
                          77,
                          78,
                          79,
                          80,
                          81,
                          82,
                          83,
                          84,
                          85,
                          86,
                          87,
                          88,
                          89,
                          90,
                          91,
                          92,
                          93,
                          94,
                          95,
                          96,
                          97,
                          98,
                          99,
                          100,
                          101,
                          102,
                          103,
                          104,
                          105,
                          106,
                          107,
                          108,
                          109,
                          110,
                          111,
                          112,
                          113,
                          114,
                          115,
                          116,
                          117,
                          118,
                          119,
                          120,
                          121,
                          122,
                          123,
                          124,
                          125,
                          126,
                          127,
                          128,
                          129,
                          130,
                          131,
                          132,
                          133,
                          134,
                          135,
                          136,
                          137,
                          138,
                          139,
                          140,
                          141,
                          142,
                          143,
                          144,
                          145,
                          146,
                          147,
                          148,
                          149,
                          150,
                          151,
                          152,
                          153,
                          154
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+02",
                          "direction_type": "J2000"
                      }
                  },
                  {
                      "name": "target3",
                      "subbands": [
                          74,
                          75,
                          76,
                          77,
                          78,
                          79,
                          80,
                          81,
                          82,
                          83,
                          84,
                          85,
                          86,
                          87,
                          88,
                          89,
                          90,
                          91,
                          92,
                          93,
                          94,
                          95,
                          96,
                          97,
                          98,
                          99,
                          100,
                          101,
                          102,
                          103,
                          104,
                          105,
                          106,
                          107,
                          108,
                          109,
                          110,
                          111,
                          112,
                          113,
                          114,
                          115,
                          116,
                          117,
                          118,
                          119,
                          120,
                          121,
                          122,
                          123,
                          124,
                          125,
                          126,
                          127,
                          128,
                          129,
                          130,
                          131,
                          132,
                          133,
                          134,
                          135,
                          136,
                          137,
                          138,
                          139,
                          140,
                          141,
                          142,
                          143,
                          144,
                          145,
                          146,
                          147,
                          148,
                          149,
                          150,
                          151,
                          152,
                          153,
                          154
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+03",
                          "direction_type": "J2000"
                      }
                  },
                  {
                      "name": "target4",
                      "subbands": [
                          74,
                          75,
                          76,
                          77,
                          78,
                          79,
                          80,
                          81,
                          82,
                          83,
                          84,
                          85,
                          86,
                          87,
                          88,
                          89,
                          90,
                          91,
                          92,
                          93,
                          94,
                          95,
                          96,
                          97,
                          98,
                          99,
                          100,
                          101,
                          102,
                          103,
                          104,
                          105,
                          106,
                          107,
                          108,
                          109,
                          110,
                          111,
                          112,
                          113,
                          114,
                          115,
                          116,
                          117,
                          118,
                          119,
                          120,
                          121,
                          122,
                          123,
                          124,
                          125,
                          126,
                          127,
                          128,
                          129,
                          130,
                          131,
                          132,
                          133,
                          134,
                          135,
                          136,
                          137,
                          138,
                          139,
                          140,
                          141,
                          142,
                          143,
                          144,
                          145,
                          146,
                          147,
                          148,
                          149,
                          150,
                          151,
                          152,
                          153,
                          154
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+04",
                          "direction_type": "J2000"
                      }
                  },
                  {
                      "name": "target5",
                      "subbands": [
                          74,
                          75,
                          76,
                          77,
                          78,
                          79,
                          80,
                          81,
                          82,
                          83,
                          84,
                          85,
                          86,
                          87,
                          88,
                          89,
                          90,
                          91,
                          92,
                          93,
                          94,
                          95,
                          96,
                          97,
                          98,
                          99,
                          100,
                          101,
                          102,
                          103,
                          104,
                          105,
                          106,
                          107,
                          108,
                          109,
                          110,
                          111,
                          112,
                          113,
                          114,
                          115,
                          116,
                          117,
                          118,
                          119,
                          120,
                          121,
                          122,
                          123,
                          124,
                          125,
                          126,
                          127,
                          128,
                          129,
                          130,
                          131,
                          132,
                          133,
                          134,
                          135,
                          136,
                          137,
                          138,
                          139,
                          140,
                          141,
                          142,
                          143,
                          144,
                          145,
                          146,
                          147,
                          148,
                          149,
                          150,
                          151,
                          152,
                          153,
                          154
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+05",
                          "direction_type": "J2000"
                      }
                  }
              ],
              "filter": "LBA_10_90",
              "tile_beam": {
                  "angle1": 0.6624317181687094,
                  "angle2": 1.5579526427549426,
                  "target": "_target_name_",
                  "direction_type": "J2000"
              },
              "antenna_set": "LBA_SPARSE_EVEN",
              "station_groups": [
                  {
                      "stations": [
                          "CS001",
                          "CS002",
                          "CS003",
                          "CS004",
                          "CS005",
                          "CS006",
                          "CS007",
                          "CS011",
                          "CS013",
                          "CS017",
                          "CS021",
                          "CS024",
                          "CS026",
                          "CS028",
                          "CS030",
                          "CS031",
                          "CS032",
                          "CS101",
                          "CS103",
                          "CS201",
                          "CS301",
                          "CS302",
                          "CS401",
                          "CS501",
                          "RS106",
                          "RS205",
                          "RS208",
                          "RS210",
                          "RS305",
                          "RS306",
                          "RS307",
                          "RS310",
                          "RS406",
                          "RS407",
                          "RS409",
                          "RS503",
                          "RS508",
                          "RS509"
                      ],
                      "max_nr_missing": 4
                  },
                  {
                      "stations": [
                          "RS508",
                          "RS509"
                      ],
                      "max_nr_missing": 1
                  },
                  {
                      "stations": [
                          "RS310",
                          "RS210"
                      ],
                      "max_nr_missing": 0
                  }
              ]
          }
      },
      "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "parallel calibrator target observation"})).id,
      "specifications_template_id": (_.find(templates, {"name": "parallel calibrator target observation"})).id,
      "tags": [],
      "task_blueprints": [
          "http://localhost:3000/api/task_blueprint/1"
      ],
      "task_blueprints_ids": [
          1
      ],
      "task_type": "observation",
      "updated_at": "2022-04-14T06:20:19.292495",
      "predecessors": [],
      "successors": [
        {
            "id": 2,
            "url": "http://localhost:3000/api/task_draft/2",
            "connector_types": [
                {
                    "id": 13,
                    "url": "http://localhost:3000/api/task_connector_type/13",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                },
                {
                    "id": 12,
                    "url": "http://localhost:3000/api/task_connector_type/12",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/7"
            ],
            "consumed_by_ids": [
                7
            ],
            "created_at": "2022-04-14T06:20:19.318002",
            "description": "Preprocessing Pipeline for Calibrator Observation",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Calibrator Pipeline",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/1"
            ],
            "produced_by_ids": [
                1
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
            "scheduling_unit_draft_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "3Cabc/1.0/CP",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 10,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 1,
                    "frequency_steps": 1
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/2"
            ],
            "task_blueprints_ids": [
                2
            ],
            "task_type": "pipeline",
            "updated_at": "2022-04-14T06:20:19.318066"
        },
        {
            "id": 3,
            "url": "http://localhost:3000/api/task_draft/3",
            "connector_types": [
                {
                    "id": 13,
                    "url": "http://localhost:3000/api/task_connector_type/13",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                },
                {
                    "id": 12,
                    "url": "http://localhost:3000/api/task_connector_type/12",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/12"
            ],
            "consumed_by_ids": [
                12
            ],
            "created_at": "2022-04-14T06:20:19.325739",
            "description": "Preprocessing Pipeline for Target Observation target5",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target5",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/6"
            ],
            "produced_by_ids": [
                6
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
            "scheduling_unit_draft_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "Paaa+05/1.4/PP",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 10,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 1,
                    "frequency_steps": 1
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/3"
            ],
            "task_blueprints_ids": [
                3
            ],
            "task_type": "pipeline",
            "updated_at": "2022-04-14T06:20:19.325751"
        },
        {
            "id": 4,
            "url": "http://localhost:3000/api/task_draft/4",
            "connector_types": [
                {
                    "id": 13,
                    "url": "http://localhost:3000/api/task_connector_type/13",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                },
                {
                    "id": 12,
                    "url": "http://localhost:3000/api/task_connector_type/12",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/11"
            ],
            "consumed_by_ids": [
                11
            ],
            "created_at": "2022-04-14T06:20:19.334536",
            "description": "Preprocessing Pipeline for Target Observation target4",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target4",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/5"
            ],
            "produced_by_ids": [
                5
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
            "scheduling_unit_draft_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "Paaa+04/1.3/PP",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 10,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 1,
                    "frequency_steps": 1
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/4"
            ],
            "task_blueprints_ids": [
                4
            ],
            "task_type": "pipeline",
            "updated_at": "2022-04-14T06:20:19.334549"
        },
        {
            "id": 5,
            "url": "http://localhost:3000/api/task_draft/5",
            "connector_types": [
                {
                    "id": 13,
                    "url": "http://localhost:3000/api/task_connector_type/13",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                },
                {
                    "id": 12,
                    "url": "http://localhost:3000/api/task_connector_type/12",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/10"
            ],
            "consumed_by_ids": [
                10
            ],
            "created_at": "2022-04-14T06:20:19.341298",
            "description": "Preprocessing Pipeline for Target Observation target3",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target3",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/4"
            ],
            "produced_by_ids": [
                4
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
            "scheduling_unit_draft_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "Paaa+03/1.2/PP",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 10,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 1,
                    "frequency_steps": 1
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/5"
            ],
            "task_blueprints_ids": [
                5
            ],
            "task_type": "pipeline",
            "updated_at": "2022-04-14T06:20:19.341309"
        },
        {
            "id": 6,
            "url": "http://localhost:3000/api/task_draft/6",
            "connector_types": [
                {
                    "id": 13,
                    "url": "http://localhost:3000/api/task_connector_type/13",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                },
                {
                    "id": 12,
                    "url": "http://localhost:3000/api/task_connector_type/12",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/9"
            ],
            "consumed_by_ids": [
                9
            ],
            "created_at": "2022-04-14T06:20:19.349574",
            "description": "Preprocessing Pipeline for Target Observation target2",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target2",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/3"
            ],
            "produced_by_ids": [
                3
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
            "scheduling_unit_draft_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "Paaa+02/1.1/PP",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 10,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 1,
                    "frequency_steps": 1
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/6"
            ],
            "task_blueprints_ids": [
                6
            ],
            "task_type": "pipeline",
            "updated_at": "2022-04-14T06:20:19.349585"
        },
        {
            "id": 7,
            "url": "http://localhost:3000/api/task_draft/7",
            "connector_types": [
                {
                    "id": 13,
                    "url": "http://localhost:3000/api/task_connector_type/13",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                },
                {
                    "id": 12,
                    "url": "http://localhost:3000/api/task_connector_type/12",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/8"
            ],
            "consumed_by_ids": [
                8
            ],
            "created_at": "2022-04-14T06:20:19.356637",
            "description": "Preprocessing Pipeline for Target Observation target1",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target1",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/2"
            ],
            "produced_by_ids": [
                2
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
            "scheduling_unit_draft_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "Paaa+01/1.0/PP",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 10,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 1,
                    "frequency_steps": 1
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/7"
            ],
            "task_blueprints_ids": [
                7
            ],
            "task_type": "pipeline",
            "updated_at": "2022-04-14T06:20:19.356648"
        }
    ]
    },
    {
      "id": 2,
      "url": "http://localhost:3000/api/task_draft/2",
      "connector_types": [
          {
              "id": 13,
              "url": "http://localhost:3000/api/task_connector_type/13",
              "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
              "dataformat_value": "MeasurementSet",
              "datatype": "http://localhost:3000/api/datatype/visibilities",
              "datatype_value": "visibilities",
              "iotype": "http://localhost:3000/api/iotype/output",
              "iotype_value": "output",
              "role": "http://localhost:3000/api/role/any",
              "role_value": "any",
              "task_template": "http://localhost:3000/api/task_template/10",
              "task_template_id": 10
          },
          {
              "id": 12,
              "url": "http://localhost:3000/api/task_connector_type/12",
              "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
              "dataformat_value": "MeasurementSet",
              "datatype": "http://localhost:3000/api/datatype/visibilities",
              "datatype_value": "visibilities",
              "iotype": "http://localhost:3000/api/iotype/input",
              "iotype_value": "input",
              "role": "http://localhost:3000/api/role/any",
              "role_value": "any",
              "task_template": "http://localhost:3000/api/task_template/10",
              "task_template_id": 10
          }
      ],
      "consumed_by": [
          "http://localhost:3000/api/task_relation_draft/7"
      ],
      "consumed_by_ids": [
          7
      ],
      "created_at": "2022-04-14T06:20:19.318002",
      "description": "Preprocessing Pipeline for Calibrator Observation",
      "duration": 0,
      "first_scheduling_relation": [],
      "first_scheduling_relation_ids": [],
      "name": "Calibrator Pipeline",
      "output_pinned": false,
      "produced_by": [
          "http://localhost:3000/api/task_relation_draft/1"
      ],
      "produced_by_ids": [
          1
      ],
      "relative_start_time": 0,
      "relative_stop_time": 0,
      "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
      "scheduling_unit_draft_id": 1,
      "second_scheduling_relation": [],
      "second_scheduling_relation_ids": [],
      "short_description": "3Cabc/1.0/CP",
      "specifications_doc": {
          "flag": {
              "rfi_strategy": "LBAdefault",
              "outerchannels": true,
              "autocorrelations": true
          },
          "demix": {
              "sources": [],
              "time_steps": 10,
              "ignore_target": false,
              "frequency_steps": 64
          },
          "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
          "average": {
              "time_steps": 1,
              "frequency_steps": 1
          },
          "storagemanager": "dysco",
          "cluster_resources": {
              "where": {
                  "cluster": "CEP4",
                  "partition": "cpu"
              },
              "cores_per_task": 2,
              "parallel_tasks": 122
          }
      },
      "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "preprocessing pipeline"})).id,
      "specifications_template_id": (_.find(templates, {"name": "preprocessing pipeline"})).id,
      "tags": [],
      "task_blueprints": [
          "http://localhost:3000/api/task_blueprint/2"
      ],
      "task_blueprints_ids": [
          2
      ],
      "task_type": "pipeline",
      "updated_at": "2022-04-14T06:20:19.318066",
      "predecessors": [
        {
            "id": 1,
            "url": "http://localhost:3000/api/task_draft/1",
            "connector_types": [
                {
                    "id": 3,
                    "url": "http://localhost:3000/api/task_connector_type/3",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/correlator",
                    "role_value": "correlator",
                    "task_template": "http://localhost:3000/api/task_template/5",
                    "task_template_id": 5
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/1",
                "http://localhost:3000/api/task_relation_draft/2",
                "http://localhost:3000/api/task_relation_draft/3",
                "http://localhost:3000/api/task_relation_draft/4",
                "http://localhost:3000/api/task_relation_draft/5",
                "http://localhost:3000/api/task_relation_draft/6"
            ],
            "consumed_by_ids": [
                1,
                2,
                3,
                4,
                5,
                6
            ],
            "created_at": "2022-04-14T06:20:19.292483",
            "description": "Combined parallel Calibrator & Target Observation for LBA with 5 target beams",
            "duration": 120,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Combined Observation",
            "output_pinned": false,
            "produced_by": [],
            "produced_by_ids": [],
            "relative_start_time": 0,
            "relative_stop_time": 120,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
            "scheduling_unit_draft_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "oXXX Paaa+01 3Cabc",
            "specifications_doc": {
                "QA": {
                    "plots": {
                        "enabled": true,
                        "autocorrelation": true,
                        "crosscorrelation": true
                    },
                    "file_conversion": {
                        "enabled": true,
                        "nr_of_subbands": -1,
                        "nr_of_timestamps": 256
                    },
                    "inspection_plots": "msplots"
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                "duration": 120,
                "calibrator": {
                    "name": "calibrator",
                    "pointing": {
                        "angle1": 0.6624317181687094,
                        "angle2": 1.5579526427549426,
                        "target": "_calibrator_name_",
                        "direction_type": "J2000"
                    },
                    "autoselect": false
                },
                "correlator": {
                    "storage_cluster": "CEP4",
                    "integration_time": 1,
                    "channels_per_subband": 64,
                    "topocentric_frequency_correction": false
                },
                "station_configuration": {
                    "SAPs": [
                        {
                            "name": "target1",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+01",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target2",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+02",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target3",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+03",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target4",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+04",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target5",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+05",
                                "direction_type": "J2000"
                            }
                        }
                    ],
                    "filter": "LBA_10_90",
                    "tile_beam": {
                        "angle1": 0.6624317181687094,
                        "angle2": 1.5579526427549426,
                        "target": "_target_name_",
                        "direction_type": "J2000"
                    },
                    "antenna_set": "LBA_SPARSE_EVEN",
                    "station_groups": [
                        {
                            "stations": [
                                "CS001",
                                "CS002",
                                "CS003",
                                "CS004",
                                "CS005",
                                "CS006",
                                "CS007",
                                "CS011",
                                "CS013",
                                "CS017",
                                "CS021",
                                "CS024",
                                "CS026",
                                "CS028",
                                "CS030",
                                "CS031",
                                "CS032",
                                "CS101",
                                "CS103",
                                "CS201",
                                "CS301",
                                "CS302",
                                "CS401",
                                "CS501",
                                "RS106",
                                "RS205",
                                "RS208",
                                "RS210",
                                "RS305",
                                "RS306",
                                "RS307",
                                "RS310",
                                "RS406",
                                "RS407",
                                "RS409",
                                "RS503",
                                "RS508",
                                "RS509"
                            ],
                            "max_nr_missing": 4
                        },
                        {
                            "stations": [
                                "RS508",
                                "RS509"
                            ],
                            "max_nr_missing": 1
                        },
                        {
                            "stations": [
                                "RS310",
                                "RS210"
                            ],
                            "max_nr_missing": 0
                        }
                    ]
                }
            },
            "specifications_template": {
                "id": 5,
                "url": "http://localhost:3000/api/task_template/5",
                "created_at": "2022-04-14T06:20:07.273898",
                "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                "name": "parallel calibrator target observation",
                "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                "purpose_value": "technical_commissioning",
                "ref_resolved_schema": {
                    "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                    "type": "object",
                    "title": "parallel calibrator target observation",
                    "$schema": "http://json-schema.org/draft-06/schema#",
                    "version": 2,
                    "required": [
                        "calibrator",
                        "correlator",
                        "duration",
                        "station_configuration"
                    ],
                    "properties": {
                        "QA": {
                            "$ref": "#/definitions/QA",
                            "default": {}
                        },
                        "duration": {
                            "$id": "#duration",
                            "$ref": "#/definitions/timedelta",
                            "title": "Duration",
                            "default": 600,
                            "minimum": 1,
                            "description": "Duration of this observation (seconds)"
                        },
                        "calibrator": {
                            "$ref": "#/definitions/calibrator",
                            "default": {}
                        },
                        "correlator": {
                            "$ref": "#/definitions/correlator",
                            "default": {}
                        },
                        "station_configuration": {
                            "$ref": "#/definitions/station_configuration",
                            "default": {}
                        }
                    },
                    "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                    "patternProperties": {
                        "^[$]schema$": {}
                    },
                    "additionalProperties": false,
                    "definitions": {
                        "QA": {
                            "additionalProperties": false,
                            "default": {},
                            "description": "Perform all Quality Assurance (QA) tasks, including file conversion and plotting.",
                            "properties": {
                                "file_conversion": {
                                    "$ref": "#/definitions/file_conversion",
                                    "default": {}
                                },
                                "inspection_plots": {
                                    "$ref": "#/definitions/inspection_plots",
                                    "default": "msplots"
                                },
                                "plots": {
                                    "$ref": "#/definitions/plots",
                                    "default": {}
                                }
                            },
                            "required": [
                                "file_conversion",
                                "plots",
                                "inspection_plots"
                            ],
                            "title": "QA",
                            "type": "object"
                        },
                        "plots": {
                            "additionalProperties": false,
                            "default": {},
                            "description": "Create dynamic spectrum plots",
                            "properties": {
                                "autocorrelation": {
                                    "default": true,
                                    "description": "Create autocorrelation plots for all stations",
                                    "title": "autocorrelation",
                                    "type": "boolean"
                                },
                                "crosscorrelation": {
                                    "default": true,
                                    "description": "Create crosscorrelation plots for all baselines",
                                    "title": "crosscorrelation",
                                    "type": "boolean"
                                },
                                "enabled": {
                                    "default": true,
                                    "description": "Do/Don't create plots from the QA file from the observation",
                                    "title": "enabled",
                                    "type": "boolean"
                                }
                            },
                            "required": [
                                "enabled",
                                "autocorrelation",
                                "crosscorrelation"
                            ],
                            "title": "Plots",
                            "type": "object"
                        },
                        "file_conversion": {
                            "additionalProperties": false,
                            "default": {},
                            "description": "Create a QA file for the observation",
                            "properties": {
                                "enabled": {
                                    "default": true,
                                    "description": "Do/Don't create a QA file for the observation",
                                    "title": "enabled",
                                    "type": "boolean"
                                },
                                "nr_of_subbands": {
                                    "default": -1,
                                    "description": "Keep this number of subbands from the observation in the QA file, or all if -1",
                                    "title": "#subbands",
                                    "type": "integer"
                                },
                                "nr_of_timestamps": {
                                    "default": 256,
                                    "description": "Extract this number of timestamps from the observation in the QA file (equidistantanly sampled, no averaging/interpolation)",
                                    "minimum": 1,
                                    "title": "#timestamps",
                                    "type": "integer"
                                }
                            },
                            "required": [
                                "enabled",
                                "nr_of_subbands",
                                "nr_of_timestamps"
                            ],
                            "title": "File Conversion",
                            "type": "object"
                        },
                        "inspection_plots": {
                            "decription": "Type of inspection plots to run",
                            "default": "msplots",
                            "enum": [
                                "msplots",
                                "dynspec",
                                "none"
                            ],
                            "title": "Inspection Plots",
                            "type": "string"
                        },
                        "timedelta": {
                            "default": 0,
                            "description": "A time duration or delta expressed in seconds",
                            "type": "number"
                        },
                        "timestamp": {
                            "default": "1970-01-01T00:00:00Z",
                            "description": "A timestamp defined in UTC",
                            "format": "date-time",
                            "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                            "type": "string"
                        },
                        "calibrator": {
                            "additionalProperties": false,
                            "default": {},
                            "properties": {
                                "autoselect": {
                                    "default": true,
                                    "description": "Auto-select calibrator based on elevation",
                                    "title": "Auto-select",
                                    "type": "boolean"
                                },
                                "name": {
                                    "default": "calibrator",
                                    "description": "Name of the calibrator SAP",
                                    "title": "Name",
                                    "type": "string"
                                },
                                "pointing": {
                                    "$ref": "#/definitions/pointing",
                                    "default": {},
                                    "description": "Manually selected calibrator",
                                    "title": "Digital pointing"
                                }
                            },
                            "required": [
                                "autoselect",
                                "pointing",
                                "name"
                            ],
                            "type": "object"
                        },
                        "pointing": {
                            "additionalProperties": false,
                            "properties": {
                                "angle1": {
                                    "default": 0.6624317181687094,
                                    "description": "First angle (e.g. RA)",
                                    "title": "Angle 1",
                                    "type": "number"
                                },
                                "angle2": {
                                    "default": 1.5579526427549426,
                                    "description": "Second angle (e.g. DEC)",
                                    "title": "Angle 2",
                                    "type": "number"
                                },
                                "direction_type": {
                                    "default": "J2000",
                                    "description": "",
                                    "enum": [
                                        "J2000",
                                        "AZELGEO",
                                        "LMN",
                                        "SUN",
                                        "MOON",
                                        "MERCURY",
                                        "VENUS",
                                        "MARS",
                                        "JUPITER",
                                        "SATURN",
                                        "URANUS",
                                        "NEPTUNE",
                                        "PLUTO"
                                    ],
                                    "title": "Reference frame",
                                    "type": "string"
                                },
                                "target": {
                                    "default": "_target_name_",
                                    "description": "Description of where this beam points at",
                                    "minLength": 1,
                                    "title": "Target",
                                    "type": "string"
                                }
                            },
                            "required": [
                                "angle1",
                                "angle2",
                                "target"
                            ],
                            "type": "object"
                        },
                        "correlator": {
                            "additionalProperties": false,
                            "default": {},
                            "properties": {
                                "channels_per_subband": {
                                    "default": 64,
                                    "description": "Number of frequency bands per subband",
                                    "enum": [
                                        8,
                                        16,
                                        32,
                                        64,
                                        128,
                                        256,
                                        512,
                                        1024
                                    ],
                                    "minimum": 8,
                                    "title": "Channels/subband",
                                    "type": "integer"
                                },
                                "integration_time": {
                                    "$ref": "#/definitions/timedelta",
                                    "default": 1,
                                    "description": "Desired integration period (seconds)",
                                    "minimum": 0.1,
                                    "title": "Integration time"
                                },
                                "storage_cluster": {
                                    "default": "CEP4",
                                    "description": "Cluster to write output to",
                                    "enum": [
                                        "CEP4",
                                        "DragNet"
                                    ],
                                    "title": "Storage cluster",
                                    "type": "string"
                                },
                                "topocentric_frequency_correction": {
                                    "default": false,
                                    "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation.",
                                    "title": "Topocentric Frequency Correction",
                                    "type": "boolean"
                                }
                            }
                        },
                        "station_configuration": {
                            "default": {},
                            "properties": {
                                "SAPs": {
                                    "$ref": "#/definitions/SAPs",
                                    "default": [
                                        {}
                                    ],
                                    "minItems": 1
                                },
                                "antenna_set": {
                                    "$ref": "#/definitions/antenna_set",
                                    "default": "HBA_DUAL"
                                },
                                "filter": {
                                    "$ref": "#/definitions/filter",
                                    "default": "HBA_110_190"
                                },
                                "station_groups": {
                                    "$ref": "#/definitions/station_groups",
                                    "default": [
                                        {
                                            "max_nr_missing": 1,
                                            "stations": [
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007"
                                            ]
                                        }
                                    ]
                                },
                                "tile_beam": {
                                    "$ref": "#/definitions/pointing",
                                    "default": {},
                                    "description": "HBA only",
                                    "title": "Tile beam"
                                }
                            },
                            "required": [
                                "station_groups",
                                "antenna_set",
                                "filter",
                                "SAPs"
                            ],
                            "title": "station_configuration",
                            "type": "object"
                        },
                        "station": {
                            "description": "These are the LOFAR stations",
                            "enum": [
                                "CS001",
                                "CS002",
                                "CS003",
                                "CS004",
                                "CS005",
                                "CS006",
                                "CS007",
                                "CS011",
                                "CS013",
                                "CS017",
                                "CS021",
                                "CS024",
                                "CS026",
                                "CS028",
                                "CS030",
                                "CS031",
                                "CS032",
                                "CS101",
                                "CS103",
                                "CS201",
                                "CS301",
                                "CS302",
                                "CS401",
                                "CS501",
                                "RS106",
                                "RS205",
                                "RS208",
                                "RS210",
                                "RS305",
                                "RS306",
                                "RS307",
                                "RS310",
                                "RS406",
                                "RS407",
                                "RS409",
                                "RS503",
                                "RS508",
                                "RS509",
                                "DE601",
                                "DE602",
                                "DE603",
                                "DE604",
                                "DE605",
                                "FR606",
                                "SE607",
                                "UK608",
                                "DE609",
                                "PL610",
                                "PL611",
                                "PL612",
                                "IE613",
                                "LV614"
                            ],
                            "title": "Station",
                            "type": "string"
                        },
                        "station_groups": {
                            "additionalItems": false,
                            "additionalProperties": false,
                            "default": [
                                {
                                    "max_nr_missing": 1,
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                }
                            ],
                            "description": "One or more predefined or custom groups of stations",
                            "items": {
                                "$ref": "#/definitions/station_group"
                            },
                            "minItems": 1,
                            "title": "Station groups",
                            "type": "array"
                        },
                        "max_number_of_missing_stations": {
                            "default": 0,
                            "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                            "minimum": 0,
                            "title": "Maximum number of stations to omit",
                            "type": "integer"
                        },
                        "SAPs": {
                            "additionalItems": false,
                            "default": [
                                {}
                            ],
                            "description": "Station beams",
                            "items": {
                                "additionalProperties": false,
                                "default": {},
                                "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                "properties": {
                                    "digital_pointing": {
                                        "$id": "#target_pointing",
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "title": "Digital pointing"
                                    },
                                    "name": {
                                        "default": "_SAP_name_",
                                        "description": "Identifier for this beam",
                                        "minLength": 1,
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "subbands": {
                                        "additionalItems": false,
                                        "default": [
                                            255
                                        ],
                                        "items": {
                                            "maximum": 511,
                                            "minimum": 0,
                                            "title": "Subband",
                                            "type": "integer"
                                        },
                                        "maxItems": 488,
                                        "minItems": 1,
                                        "title": "Subband list",
                                        "type": "array"
                                    }
                                },
                                "required": [
                                    "name",
                                    "digital_pointing",
                                    "subbands"
                                ],
                                "title": "SAP",
                                "type": "object"
                            },
                            "minItems": 0,
                            "title": "SAPs",
                            "type": "array"
                        },
                        "antenna_set": {
                            "default": "HBA_DUAL",
                            "description": "Fields & antennas to use",
                            "enum": [
                                "HBA_DUAL",
                                "HBA_DUAL_INNER",
                                "HBA_ONE",
                                "HBA_ONE_INNER",
                                "HBA_ZERO",
                                "HBA_ZERO_INNER",
                                "LBA_INNER",
                                "LBA_OUTER",
                                "LBA_SPARSE_EVEN",
                                "LBA_SPARSE_ODD",
                                "LBA_ALL"
                            ],
                            "title": "Antenna set",
                            "type": "string"
                        },
                        "filter": {
                            "default": "HBA_110_190",
                            "description": "Must match antenna type",
                            "enum": [
                                "LBA_10_70",
                                "LBA_30_70",
                                "LBA_10_90",
                                "LBA_30_90",
                                "HBA_110_190",
                                "HBA_210_250"
                            ],
                            "title": "Band-pass filter",
                            "type": "string"
                        },
                        "station_list": {
                            "additionalItems": false,
                            "additionalProperties": false,
                            "default": [],
                            "items": {
                                "$ref": "#/definitions/station"
                            },
                            "minItems": 0,
                            "type": "array",
                            "uniqueItems": true
                        },
                        "station_group": {
                            "anyOf": [
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 0,
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ]
                                    },
                                    "description": "The group of all stations on the Superterp",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Superterp",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 4,
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501"
                                        ]
                                    },
                                    "description": "The group of all Core stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS101",
                                                    "CS103",
                                                    "CS201",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Core",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 4,
                                        "stations": [
                                            "RS106",
                                            "RS205",
                                            "RS208",
                                            "RS210",
                                            "RS305",
                                            "RS306",
                                            "RS307",
                                            "RS310",
                                            "RS406",
                                            "RS407",
                                            "RS409",
                                            "RS503",
                                            "RS508",
                                            "RS509"
                                        ]
                                    },
                                    "description": "The group of all Dutch remote stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "RS106",
                                                    "RS205",
                                                    "RS208",
                                                    "RS210",
                                                    "RS305",
                                                    "RS306",
                                                    "RS307",
                                                    "RS310",
                                                    "RS406",
                                                    "RS407",
                                                    "RS409",
                                                    "RS503",
                                                    "RS508",
                                                    "RS509"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Remote",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 4,
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501",
                                            "RS106",
                                            "RS205",
                                            "RS208",
                                            "RS210",
                                            "RS305",
                                            "RS306",
                                            "RS307",
                                            "RS310",
                                            "RS406",
                                            "RS407",
                                            "RS409",
                                            "RS503",
                                            "RS508",
                                            "RS509"
                                        ]
                                    },
                                    "description": "The group of all Dutch (Core + Remote) stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS101",
                                                    "CS103",
                                                    "CS201",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501",
                                                    "RS106",
                                                    "RS205",
                                                    "RS208",
                                                    "RS210",
                                                    "RS305",
                                                    "RS306",
                                                    "RS307",
                                                    "RS310",
                                                    "RS406",
                                                    "RS407",
                                                    "RS409",
                                                    "RS503",
                                                    "RS508",
                                                    "RS509"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Dutch",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 2,
                                        "stations": [
                                            "DE601",
                                            "DE602",
                                            "DE603",
                                            "DE604",
                                            "DE605",
                                            "DE609",
                                            "FR606",
                                            "SE607",
                                            "UK608",
                                            "PL610",
                                            "PL611",
                                            "PL612",
                                            "IE613",
                                            "LV614"
                                        ]
                                    },
                                    "description": "The group of all international stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "DE601",
                                                    "DE602",
                                                    "DE603",
                                                    "DE604",
                                                    "DE605",
                                                    "DE609",
                                                    "FR606",
                                                    "SE607",
                                                    "UK608",
                                                    "PL610",
                                                    "PL611",
                                                    "PL612",
                                                    "IE613",
                                                    "LV614"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "International",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 1,
                                        "stations": [
                                            "DE601",
                                            "DE605"
                                        ]
                                    },
                                    "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "DE601",
                                                    "DE605"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "International required",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 6,
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501",
                                            "RS106",
                                            "RS205",
                                            "RS208",
                                            "RS210",
                                            "RS305",
                                            "RS306",
                                            "RS307",
                                            "RS310",
                                            "RS406",
                                            "RS407",
                                            "RS409",
                                            "RS503",
                                            "RS508",
                                            "RS509",
                                            "DE601",
                                            "DE602",
                                            "DE603",
                                            "DE604",
                                            "DE605",
                                            "DE609",
                                            "FR606",
                                            "SE607",
                                            "UK608",
                                            "PL610",
                                            "PL611",
                                            "PL612",
                                            "IE613",
                                            "LV614"
                                        ]
                                    },
                                    "description": "The group of all (Core + Remote + International) stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS101",
                                                    "CS103",
                                                    "CS201",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501",
                                                    "RS106",
                                                    "RS205",
                                                    "RS208",
                                                    "RS210",
                                                    "RS305",
                                                    "RS306",
                                                    "RS307",
                                                    "RS310",
                                                    "RS406",
                                                    "RS407",
                                                    "RS409",
                                                    "RS503",
                                                    "RS508",
                                                    "RS509",
                                                    "DE601",
                                                    "DE602",
                                                    "DE603",
                                                    "DE604",
                                                    "DE605",
                                                    "DE609",
                                                    "FR606",
                                                    "SE607",
                                                    "UK608",
                                                    "PL610",
                                                    "PL611",
                                                    "PL612",
                                                    "IE613",
                                                    "LV614"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "All",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 0,
                                        "stations": [
                                            "CS001"
                                        ]
                                    },
                                    "description": "A custom group of stations which can be defined by the user",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list"
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Custom",
                                    "type": "object"
                                }
                            ],
                            "default": {
                                "max_nr_missing": 1,
                                "stations": [
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007"
                                ]
                            },
                            "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                            "title": "Station group",
                            "type": "object"
                        }
                    }
                },
                "schema": {
                    "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2#",
                    "type": "object",
                    "title": "parallel calibrator target observation",
                    "$schema": "http://json-schema.org/draft-06/schema#",
                    "version": 2,
                    "required": [
                        "calibrator",
                        "correlator",
                        "duration",
                        "station_configuration"
                    ],
                    "properties": {
                        "QA": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/2#/definitions/QA",
                            "default": {}
                        },
                        "duration": {
                            "$id": "#duration",
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                            "title": "Duration",
                            "default": 600,
                            "minimum": 1,
                            "description": "Duration of this observation (seconds)"
                        },
                        "calibrator": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/calibrator/1#/definitions/calibrator",
                            "default": {}
                        },
                        "correlator": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/correlator/1#/definitions/correlator",
                            "default": {}
                        },
                        "station_configuration": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                            "default": {}
                        }
                    },
                    "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                    "patternProperties": {
                        "^[$]schema$": {}
                    },
                    "additionalProperties": false
                },
                "state": "http://localhost:3000/api/template_state/active",
                "state_value": "active",
                "tags": [],
                "type": "http://localhost:3000/api/task_type/observation",
                "type_value": "observation",
                "updated_at": "2022-04-14T06:20:07.273910",
                "version": 2
            },
            "specifications_template_id": 5,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/1"
            ],
            "task_blueprints_ids": [
                1
            ],
            "task_type": "observation",
            "updated_at": "2022-04-14T06:20:19.292495"
        }
      ],
      "successors": [
        {
            "id": 8,
            "url": "http://localhost:3000/api/task_draft/8",
            "connector_types": [
                {
                    "id": 26,
                    "url": "http://localhost:3000/api/task_connector_type/26",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/correlator",
                    "role_value": "correlator",
                    "task_template": "http://localhost:3000/api/task_template/2",
                    "task_template_id": 2
                },
                {
                    "id": 21,
                    "url": "http://localhost:3000/api/task_connector_type/21",
                    "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                    "dataformat_value": "Beamformed",
                    "datatype": "http://localhost:3000/api/datatype/time%20series",
                    "datatype_value": "time series",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/2",
                    "task_template_id": 2
                },
                {
                    "id": 20,
                    "url": "http://localhost:3000/api/task_connector_type/20",
                    "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                    "dataformat_value": "Beamformed",
                    "datatype": "http://localhost:3000/api/datatype/time%20series",
                    "datatype_value": "time series",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/beamformer",
                    "role_value": "beamformer",
                    "task_template": "http://localhost:3000/api/task_template/2",
                    "task_template_id": 2
                },
                {
                    "id": 18,
                    "url": "http://localhost:3000/api/task_connector_type/18",
                    "dataformat": "http://localhost:3000/api/dataformat/pulp%20summary",
                    "dataformat_value": "pulp summary",
                    "datatype": "http://localhost:3000/api/datatype/quality",
                    "datatype_value": "quality",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/2",
                    "task_template_id": 2
                },
                {
                    "id": 16,
                    "url": "http://localhost:3000/api/task_connector_type/16",
                    "dataformat": "http://localhost:3000/api/dataformat/pulp%20analysis",
                    "dataformat_value": "pulp analysis",
                    "datatype": "http://localhost:3000/api/datatype/pulsar%20profile",
                    "datatype_value": "pulsar profile",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/2",
                    "task_template_id": 2
                },
                {
                    "id": 14,
                    "url": "http://localhost:3000/api/task_connector_type/14",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/2",
                    "task_template_id": 2
                }
            ],
            "consumed_by": [],
            "consumed_by_ids": [],
            "created_at": "2022-04-14T06:20:19.364055",
            "description": "Ingest all preprocessed dataproducts",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Ingest",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/7",
                "http://localhost:3000/api/task_relation_draft/8",
                "http://localhost:3000/api/task_relation_draft/9",
                "http://localhost:3000/api/task_relation_draft/10",
                "http://localhost:3000/api/task_relation_draft/11",
                "http://localhost:3000/api/task_relation_draft/12"
            ],
            "produced_by_ids": [
                7,
                8,
                9,
                10,
                11,
                12
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
            "scheduling_unit_draft_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "",
            "specifications_doc": {
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
            },
            "specifications_template": "http://localhost:3000/api/task_template/2",
            "specifications_template_id": 2,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/8"
            ],
            "task_blueprints_ids": [
                8
            ],
            "task_type": "ingest",
            "updated_at": "2022-04-14T06:20:19.364067"
        }
      ]
    },
    {
      "id": 7,
      "url": "http://localhost:3000/api/task_draft/7",
      "connector_types": [
          {
              "id": 13,
              "url": "http://localhost:3000/api/task_connector_type/13",
              "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
              "dataformat_value": "MeasurementSet",
              "datatype": "http://localhost:3000/api/datatype/visibilities",
              "datatype_value": "visibilities",
              "iotype": "http://localhost:3000/api/iotype/output",
              "iotype_value": "output",
              "role": "http://localhost:3000/api/role/any",
              "role_value": "any",
              "task_template": "http://localhost:3000/api/task_template/10",
              "task_template_id": 10
          },
          {
              "id": 12,
              "url": "http://localhost:3000/api/task_connector_type/12",
              "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
              "dataformat_value": "MeasurementSet",
              "datatype": "http://localhost:3000/api/datatype/visibilities",
              "datatype_value": "visibilities",
              "iotype": "http://localhost:3000/api/iotype/input",
              "iotype_value": "input",
              "role": "http://localhost:3000/api/role/any",
              "role_value": "any",
              "task_template": "http://localhost:3000/api/task_template/10",
              "task_template_id": 10
          }
      ],
      "consumed_by": [
          "http://localhost:3000/api/task_relation_draft/8"
      ],
      "consumed_by_ids": [
          8
      ],
      "created_at": "2022-04-14T06:20:19.356637",
      "description": "Preprocessing Pipeline for Target Observation target1",
      "duration": 0,
      "first_scheduling_relation": [],
      "first_scheduling_relation_ids": [],
      "name": "Pipeline target1",
      "output_pinned": false,
      "produced_by": [
          "http://localhost:3000/api/task_relation_draft/2"
      ],
      "produced_by_ids": [
          2
      ],
      "relative_start_time": 0,
      "relative_stop_time": 0,
      "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
      "scheduling_unit_draft_id": 1,
      "second_scheduling_relation": [],
      "second_scheduling_relation_ids": [],
      "short_description": "Paaa+01/1.0/PP",
      "specifications_doc": {
          "flag": {
              "rfi_strategy": "LBAdefault",
              "outerchannels": true,
              "autocorrelations": true
          },
          "demix": {
              "sources": [],
              "time_steps": 10,
              "ignore_target": false,
              "frequency_steps": 64
          },
          "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
          "average": {
              "time_steps": 1,
              "frequency_steps": 1
          },
          "storagemanager": "dysco",
          "cluster_resources": {
              "where": {
                  "cluster": "CEP4",
                  "partition": "cpu"
              },
              "cores_per_task": 2,
              "parallel_tasks": 122
          }
      },
      "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "preprocessing pipeline"})).id,
      "specifications_template_id": (_.find(templates, {"name": "preprocessing pipeline"})).id,
      "tags": [],
      "task_blueprints": [
          "http://localhost:3000/api/task_blueprint/7"
      ],
      "task_blueprints_ids": [
          7
      ],
      "task_type": "pipeline",
      "updated_at": "2022-04-14T06:20:19.356648",
      "successors": [
        {
            "id": 8,
            "url": "http://localhost:3000/api/task_draft/8",
            "connector_types": [
                {
                    "id": 26,
                    "url": "http://localhost:3000/api/task_connector_type/26",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/correlator",
                    "role_value": "correlator",
                    "task_template": "http://localhost:3000/api/task_template/2",
                    "task_template_id": 2
                },
                {
                    "id": 21,
                    "url": "http://localhost:3000/api/task_connector_type/21",
                    "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                    "dataformat_value": "Beamformed",
                    "datatype": "http://localhost:3000/api/datatype/time%20series",
                    "datatype_value": "time series",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/2",
                    "task_template_id": 2
                },
                {
                    "id": 20,
                    "url": "http://localhost:3000/api/task_connector_type/20",
                    "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                    "dataformat_value": "Beamformed",
                    "datatype": "http://localhost:3000/api/datatype/time%20series",
                    "datatype_value": "time series",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/beamformer",
                    "role_value": "beamformer",
                    "task_template": "http://localhost:3000/api/task_template/2",
                    "task_template_id": 2
                },
                {
                    "id": 18,
                    "url": "http://localhost:3000/api/task_connector_type/18",
                    "dataformat": "http://localhost:3000/api/dataformat/pulp%20summary",
                    "dataformat_value": "pulp summary",
                    "datatype": "http://localhost:3000/api/datatype/quality",
                    "datatype_value": "quality",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/2",
                    "task_template_id": 2
                },
                {
                    "id": 16,
                    "url": "http://localhost:3000/api/task_connector_type/16",
                    "dataformat": "http://localhost:3000/api/dataformat/pulp%20analysis",
                    "dataformat_value": "pulp analysis",
                    "datatype": "http://localhost:3000/api/datatype/pulsar%20profile",
                    "datatype_value": "pulsar profile",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/2",
                    "task_template_id": 2
                },
                {
                    "id": 14,
                    "url": "http://localhost:3000/api/task_connector_type/14",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/2",
                    "task_template_id": 2
                }
            ],
            "consumed_by": [],
            "consumed_by_ids": [],
            "created_at": "2022-04-14T06:20:19.364055",
            "description": "Ingest all preprocessed dataproducts",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Ingest",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/7",
                "http://localhost:3000/api/task_relation_draft/8",
                "http://localhost:3000/api/task_relation_draft/9",
                "http://localhost:3000/api/task_relation_draft/10",
                "http://localhost:3000/api/task_relation_draft/11",
                "http://localhost:3000/api/task_relation_draft/12"
            ],
            "produced_by_ids": [
                7,
                8,
                9,
                10,
                11,
                12
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
            "scheduling_unit_draft_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "",
            "specifications_doc": {
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
            },
            "specifications_template": "http://localhost:3000/api/task_template/2",
            "specifications_template_id": 2,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/8"
            ],
            "task_blueprints_ids": [
                8
            ],
            "task_type": "ingest",
            "updated_at": "2022-04-14T06:20:19.364067"
        }
      ],
      "predecessors": [
        {
            "id": 1,
            "url": "http://localhost:3000/api/task_draft/1",
            "connector_types": [
                {
                    "id": 3,
                    "url": "http://localhost:3000/api/task_connector_type/3",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/correlator",
                    "role_value": "correlator",
                    "task_template": "http://localhost:3000/api/task_template/5",
                    "task_template_id": 5
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/1",
                "http://localhost:3000/api/task_relation_draft/2",
                "http://localhost:3000/api/task_relation_draft/3",
                "http://localhost:3000/api/task_relation_draft/4",
                "http://localhost:3000/api/task_relation_draft/5",
                "http://localhost:3000/api/task_relation_draft/6"
            ],
            "consumed_by_ids": [
                1,
                2,
                3,
                4,
                5,
                6
            ],
            "created_at": "2022-04-14T06:20:19.292483",
            "description": "Combined parallel Calibrator & Target Observation for LBA with 5 target beams",
            "duration": 120,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Combined Observation",
            "output_pinned": false,
            "produced_by": [],
            "produced_by_ids": [],
            "relative_start_time": 0,
            "relative_stop_time": 120,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
            "scheduling_unit_draft_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "oXXX Paaa+01 3Cabc",
            "specifications_doc": {
                "QA": {
                    "plots": {
                        "enabled": true,
                        "autocorrelation": true,
                        "crosscorrelation": true
                    },
                    "file_conversion": {
                        "enabled": true,
                        "nr_of_subbands": -1,
                        "nr_of_timestamps": 256
                    },
                    "inspection_plots": "msplots"
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                "duration": 120,
                "calibrator": {
                    "name": "calibrator",
                    "pointing": {
                        "angle1": 0.6624317181687094,
                        "angle2": 1.5579526427549426,
                        "target": "_calibrator_name_",
                        "direction_type": "J2000"
                    },
                    "autoselect": false
                },
                "correlator": {
                    "storage_cluster": "CEP4",
                    "integration_time": 1,
                    "channels_per_subband": 64,
                    "topocentric_frequency_correction": false
                },
                "station_configuration": {
                    "SAPs": [
                        {
                            "name": "target1",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+01",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target2",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+02",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target3",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+03",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target4",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+04",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target5",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+05",
                                "direction_type": "J2000"
                            }
                        }
                    ],
                    "filter": "LBA_10_90",
                    "tile_beam": {
                        "angle1": 0.6624317181687094,
                        "angle2": 1.5579526427549426,
                        "target": "_target_name_",
                        "direction_type": "J2000"
                    },
                    "antenna_set": "LBA_SPARSE_EVEN",
                    "station_groups": [
                        {
                            "stations": [
                                "CS001",
                                "CS002",
                                "CS003",
                                "CS004",
                                "CS005",
                                "CS006",
                                "CS007",
                                "CS011",
                                "CS013",
                                "CS017",
                                "CS021",
                                "CS024",
                                "CS026",
                                "CS028",
                                "CS030",
                                "CS031",
                                "CS032",
                                "CS101",
                                "CS103",
                                "CS201",
                                "CS301",
                                "CS302",
                                "CS401",
                                "CS501",
                                "RS106",
                                "RS205",
                                "RS208",
                                "RS210",
                                "RS305",
                                "RS306",
                                "RS307",
                                "RS310",
                                "RS406",
                                "RS407",
                                "RS409",
                                "RS503",
                                "RS508",
                                "RS509"
                            ],
                            "max_nr_missing": 4
                        },
                        {
                            "stations": [
                                "RS508",
                                "RS509"
                            ],
                            "max_nr_missing": 1
                        },
                        {
                            "stations": [
                                "RS310",
                                "RS210"
                            ],
                            "max_nr_missing": 0
                        }
                    ]
                }
            },
            "specifications_template": {
                "id": 5,
                "url": "http://localhost:3000/api/task_template/5",
                "created_at": "2022-04-14T06:20:07.273898",
                "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                "name": "parallel calibrator target observation",
                "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                "purpose_value": "technical_commissioning",
                "ref_resolved_schema": {
                    "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                    "type": "object",
                    "title": "parallel calibrator target observation",
                    "$schema": "http://json-schema.org/draft-06/schema#",
                    "version": 2,
                    "required": [
                        "calibrator",
                        "correlator",
                        "duration",
                        "station_configuration"
                    ],
                    "properties": {
                        "QA": {
                            "$ref": "#/definitions/QA",
                            "default": {}
                        },
                        "duration": {
                            "$id": "#duration",
                            "$ref": "#/definitions/timedelta",
                            "title": "Duration",
                            "default": 600,
                            "minimum": 1,
                            "description": "Duration of this observation (seconds)"
                        },
                        "calibrator": {
                            "$ref": "#/definitions/calibrator",
                            "default": {}
                        },
                        "correlator": {
                            "$ref": "#/definitions/correlator",
                            "default": {}
                        },
                        "station_configuration": {
                            "$ref": "#/definitions/station_configuration",
                            "default": {}
                        }
                    },
                    "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                    "patternProperties": {
                        "^[$]schema$": {}
                    },
                    "additionalProperties": false,
                    "definitions": {
                        "QA": {
                            "additionalProperties": false,
                            "default": {},
                            "description": "Perform all Quality Assurance (QA) tasks, including file conversion and plotting.",
                            "properties": {
                                "file_conversion": {
                                    "$ref": "#/definitions/file_conversion",
                                    "default": {}
                                },
                                "inspection_plots": {
                                    "$ref": "#/definitions/inspection_plots",
                                    "default": "msplots"
                                },
                                "plots": {
                                    "$ref": "#/definitions/plots",
                                    "default": {}
                                }
                            },
                            "required": [
                                "file_conversion",
                                "plots",
                                "inspection_plots"
                            ],
                            "title": "QA",
                            "type": "object"
                        },
                        "plots": {
                            "additionalProperties": false,
                            "default": {},
                            "description": "Create dynamic spectrum plots",
                            "properties": {
                                "autocorrelation": {
                                    "default": true,
                                    "description": "Create autocorrelation plots for all stations",
                                    "title": "autocorrelation",
                                    "type": "boolean"
                                },
                                "crosscorrelation": {
                                    "default": true,
                                    "description": "Create crosscorrelation plots for all baselines",
                                    "title": "crosscorrelation",
                                    "type": "boolean"
                                },
                                "enabled": {
                                    "default": true,
                                    "description": "Do/Don't create plots from the QA file from the observation",
                                    "title": "enabled",
                                    "type": "boolean"
                                }
                            },
                            "required": [
                                "enabled",
                                "autocorrelation",
                                "crosscorrelation"
                            ],
                            "title": "Plots",
                            "type": "object"
                        },
                        "file_conversion": {
                            "additionalProperties": false,
                            "default": {},
                            "description": "Create a QA file for the observation",
                            "properties": {
                                "enabled": {
                                    "default": true,
                                    "description": "Do/Don't create a QA file for the observation",
                                    "title": "enabled",
                                    "type": "boolean"
                                },
                                "nr_of_subbands": {
                                    "default": -1,
                                    "description": "Keep this number of subbands from the observation in the QA file, or all if -1",
                                    "title": "#subbands",
                                    "type": "integer"
                                },
                                "nr_of_timestamps": {
                                    "default": 256,
                                    "description": "Extract this number of timestamps from the observation in the QA file (equidistantanly sampled, no averaging/interpolation)",
                                    "minimum": 1,
                                    "title": "#timestamps",
                                    "type": "integer"
                                }
                            },
                            "required": [
                                "enabled",
                                "nr_of_subbands",
                                "nr_of_timestamps"
                            ],
                            "title": "File Conversion",
                            "type": "object"
                        },
                        "inspection_plots": {
                            "decription": "Type of inspection plots to run",
                            "default": "msplots",
                            "enum": [
                                "msplots",
                                "dynspec",
                                "none"
                            ],
                            "title": "Inspection Plots",
                            "type": "string"
                        },
                        "timedelta": {
                            "default": 0,
                            "description": "A time duration or delta expressed in seconds",
                            "type": "number"
                        },
                        "timestamp": {
                            "default": "1970-01-01T00:00:00Z",
                            "description": "A timestamp defined in UTC",
                            "format": "date-time",
                            "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                            "type": "string"
                        },
                        "calibrator": {
                            "additionalProperties": false,
                            "default": {},
                            "properties": {
                                "autoselect": {
                                    "default": true,
                                    "description": "Auto-select calibrator based on elevation",
                                    "title": "Auto-select",
                                    "type": "boolean"
                                },
                                "name": {
                                    "default": "calibrator",
                                    "description": "Name of the calibrator SAP",
                                    "title": "Name",
                                    "type": "string"
                                },
                                "pointing": {
                                    "$ref": "#/definitions/pointing",
                                    "default": {},
                                    "description": "Manually selected calibrator",
                                    "title": "Digital pointing"
                                }
                            },
                            "required": [
                                "autoselect",
                                "pointing",
                                "name"
                            ],
                            "type": "object"
                        },
                        "pointing": {
                            "additionalProperties": false,
                            "properties": {
                                "angle1": {
                                    "default": 0.6624317181687094,
                                    "description": "First angle (e.g. RA)",
                                    "title": "Angle 1",
                                    "type": "number"
                                },
                                "angle2": {
                                    "default": 1.5579526427549426,
                                    "description": "Second angle (e.g. DEC)",
                                    "title": "Angle 2",
                                    "type": "number"
                                },
                                "direction_type": {
                                    "default": "J2000",
                                    "description": "",
                                    "enum": [
                                        "J2000",
                                        "AZELGEO",
                                        "LMN",
                                        "SUN",
                                        "MOON",
                                        "MERCURY",
                                        "VENUS",
                                        "MARS",
                                        "JUPITER",
                                        "SATURN",
                                        "URANUS",
                                        "NEPTUNE",
                                        "PLUTO"
                                    ],
                                    "title": "Reference frame",
                                    "type": "string"
                                },
                                "target": {
                                    "default": "_target_name_",
                                    "description": "Description of where this beam points at",
                                    "minLength": 1,
                                    "title": "Target",
                                    "type": "string"
                                }
                            },
                            "required": [
                                "angle1",
                                "angle2",
                                "target"
                            ],
                            "type": "object"
                        },
                        "correlator": {
                            "additionalProperties": false,
                            "default": {},
                            "properties": {
                                "channels_per_subband": {
                                    "default": 64,
                                    "description": "Number of frequency bands per subband",
                                    "enum": [
                                        8,
                                        16,
                                        32,
                                        64,
                                        128,
                                        256,
                                        512,
                                        1024
                                    ],
                                    "minimum": 8,
                                    "title": "Channels/subband",
                                    "type": "integer"
                                },
                                "integration_time": {
                                    "$ref": "#/definitions/timedelta",
                                    "default": 1,
                                    "description": "Desired integration period (seconds)",
                                    "minimum": 0.1,
                                    "title": "Integration time"
                                },
                                "storage_cluster": {
                                    "default": "CEP4",
                                    "description": "Cluster to write output to",
                                    "enum": [
                                        "CEP4",
                                        "DragNet"
                                    ],
                                    "title": "Storage cluster",
                                    "type": "string"
                                },
                                "topocentric_frequency_correction": {
                                    "default": false,
                                    "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation.",
                                    "title": "Topocentric Frequency Correction",
                                    "type": "boolean"
                                }
                            }
                        },
                        "station_configuration": {
                            "default": {},
                            "properties": {
                                "SAPs": {
                                    "$ref": "#/definitions/SAPs",
                                    "default": [
                                        {}
                                    ],
                                    "minItems": 1
                                },
                                "antenna_set": {
                                    "$ref": "#/definitions/antenna_set",
                                    "default": "HBA_DUAL"
                                },
                                "filter": {
                                    "$ref": "#/definitions/filter",
                                    "default": "HBA_110_190"
                                },
                                "station_groups": {
                                    "$ref": "#/definitions/station_groups",
                                    "default": [
                                        {
                                            "max_nr_missing": 1,
                                            "stations": [
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007"
                                            ]
                                        }
                                    ]
                                },
                                "tile_beam": {
                                    "$ref": "#/definitions/pointing",
                                    "default": {},
                                    "description": "HBA only",
                                    "title": "Tile beam"
                                }
                            },
                            "required": [
                                "station_groups",
                                "antenna_set",
                                "filter",
                                "SAPs"
                            ],
                            "title": "station_configuration",
                            "type": "object"
                        },
                        "station": {
                            "description": "These are the LOFAR stations",
                            "enum": [
                                "CS001",
                                "CS002",
                                "CS003",
                                "CS004",
                                "CS005",
                                "CS006",
                                "CS007",
                                "CS011",
                                "CS013",
                                "CS017",
                                "CS021",
                                "CS024",
                                "CS026",
                                "CS028",
                                "CS030",
                                "CS031",
                                "CS032",
                                "CS101",
                                "CS103",
                                "CS201",
                                "CS301",
                                "CS302",
                                "CS401",
                                "CS501",
                                "RS106",
                                "RS205",
                                "RS208",
                                "RS210",
                                "RS305",
                                "RS306",
                                "RS307",
                                "RS310",
                                "RS406",
                                "RS407",
                                "RS409",
                                "RS503",
                                "RS508",
                                "RS509",
                                "DE601",
                                "DE602",
                                "DE603",
                                "DE604",
                                "DE605",
                                "FR606",
                                "SE607",
                                "UK608",
                                "DE609",
                                "PL610",
                                "PL611",
                                "PL612",
                                "IE613",
                                "LV614"
                            ],
                            "title": "Station",
                            "type": "string"
                        },
                        "station_groups": {
                            "additionalItems": false,
                            "additionalProperties": false,
                            "default": [
                                {
                                    "max_nr_missing": 1,
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                }
                            ],
                            "description": "One or more predefined or custom groups of stations",
                            "items": {
                                "$ref": "#/definitions/station_group"
                            },
                            "minItems": 1,
                            "title": "Station groups",
                            "type": "array"
                        },
                        "max_number_of_missing_stations": {
                            "default": 0,
                            "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                            "minimum": 0,
                            "title": "Maximum number of stations to omit",
                            "type": "integer"
                        },
                        "SAPs": {
                            "additionalItems": false,
                            "default": [
                                {}
                            ],
                            "description": "Station beams",
                            "items": {
                                "additionalProperties": false,
                                "default": {},
                                "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                "properties": {
                                    "digital_pointing": {
                                        "$id": "#target_pointing",
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "title": "Digital pointing"
                                    },
                                    "name": {
                                        "default": "_SAP_name_",
                                        "description": "Identifier for this beam",
                                        "minLength": 1,
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "subbands": {
                                        "additionalItems": false,
                                        "default": [
                                            255
                                        ],
                                        "items": {
                                            "maximum": 511,
                                            "minimum": 0,
                                            "title": "Subband",
                                            "type": "integer"
                                        },
                                        "maxItems": 488,
                                        "minItems": 1,
                                        "title": "Subband list",
                                        "type": "array"
                                    }
                                },
                                "required": [
                                    "name",
                                    "digital_pointing",
                                    "subbands"
                                ],
                                "title": "SAP",
                                "type": "object"
                            },
                            "minItems": 0,
                            "title": "SAPs",
                            "type": "array"
                        },
                        "antenna_set": {
                            "default": "HBA_DUAL",
                            "description": "Fields & antennas to use",
                            "enum": [
                                "HBA_DUAL",
                                "HBA_DUAL_INNER",
                                "HBA_ONE",
                                "HBA_ONE_INNER",
                                "HBA_ZERO",
                                "HBA_ZERO_INNER",
                                "LBA_INNER",
                                "LBA_OUTER",
                                "LBA_SPARSE_EVEN",
                                "LBA_SPARSE_ODD",
                                "LBA_ALL"
                            ],
                            "title": "Antenna set",
                            "type": "string"
                        },
                        "filter": {
                            "default": "HBA_110_190",
                            "description": "Must match antenna type",
                            "enum": [
                                "LBA_10_70",
                                "LBA_30_70",
                                "LBA_10_90",
                                "LBA_30_90",
                                "HBA_110_190",
                                "HBA_210_250"
                            ],
                            "title": "Band-pass filter",
                            "type": "string"
                        },
                        "station_list": {
                            "additionalItems": false,
                            "additionalProperties": false,
                            "default": [],
                            "items": {
                                "$ref": "#/definitions/station"
                            },
                            "minItems": 0,
                            "type": "array",
                            "uniqueItems": true
                        },
                        "station_group": {
                            "anyOf": [
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 0,
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ]
                                    },
                                    "description": "The group of all stations on the Superterp",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Superterp",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 4,
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501"
                                        ]
                                    },
                                    "description": "The group of all Core stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS101",
                                                    "CS103",
                                                    "CS201",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Core",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 4,
                                        "stations": [
                                            "RS106",
                                            "RS205",
                                            "RS208",
                                            "RS210",
                                            "RS305",
                                            "RS306",
                                            "RS307",
                                            "RS310",
                                            "RS406",
                                            "RS407",
                                            "RS409",
                                            "RS503",
                                            "RS508",
                                            "RS509"
                                        ]
                                    },
                                    "description": "The group of all Dutch remote stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "RS106",
                                                    "RS205",
                                                    "RS208",
                                                    "RS210",
                                                    "RS305",
                                                    "RS306",
                                                    "RS307",
                                                    "RS310",
                                                    "RS406",
                                                    "RS407",
                                                    "RS409",
                                                    "RS503",
                                                    "RS508",
                                                    "RS509"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Remote",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 4,
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501",
                                            "RS106",
                                            "RS205",
                                            "RS208",
                                            "RS210",
                                            "RS305",
                                            "RS306",
                                            "RS307",
                                            "RS310",
                                            "RS406",
                                            "RS407",
                                            "RS409",
                                            "RS503",
                                            "RS508",
                                            "RS509"
                                        ]
                                    },
                                    "description": "The group of all Dutch (Core + Remote) stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS101",
                                                    "CS103",
                                                    "CS201",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501",
                                                    "RS106",
                                                    "RS205",
                                                    "RS208",
                                                    "RS210",
                                                    "RS305",
                                                    "RS306",
                                                    "RS307",
                                                    "RS310",
                                                    "RS406",
                                                    "RS407",
                                                    "RS409",
                                                    "RS503",
                                                    "RS508",
                                                    "RS509"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Dutch",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 2,
                                        "stations": [
                                            "DE601",
                                            "DE602",
                                            "DE603",
                                            "DE604",
                                            "DE605",
                                            "DE609",
                                            "FR606",
                                            "SE607",
                                            "UK608",
                                            "PL610",
                                            "PL611",
                                            "PL612",
                                            "IE613",
                                            "LV614"
                                        ]
                                    },
                                    "description": "The group of all international stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "DE601",
                                                    "DE602",
                                                    "DE603",
                                                    "DE604",
                                                    "DE605",
                                                    "DE609",
                                                    "FR606",
                                                    "SE607",
                                                    "UK608",
                                                    "PL610",
                                                    "PL611",
                                                    "PL612",
                                                    "IE613",
                                                    "LV614"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "International",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 1,
                                        "stations": [
                                            "DE601",
                                            "DE605"
                                        ]
                                    },
                                    "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "DE601",
                                                    "DE605"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "International required",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 6,
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501",
                                            "RS106",
                                            "RS205",
                                            "RS208",
                                            "RS210",
                                            "RS305",
                                            "RS306",
                                            "RS307",
                                            "RS310",
                                            "RS406",
                                            "RS407",
                                            "RS409",
                                            "RS503",
                                            "RS508",
                                            "RS509",
                                            "DE601",
                                            "DE602",
                                            "DE603",
                                            "DE604",
                                            "DE605",
                                            "DE609",
                                            "FR606",
                                            "SE607",
                                            "UK608",
                                            "PL610",
                                            "PL611",
                                            "PL612",
                                            "IE613",
                                            "LV614"
                                        ]
                                    },
                                    "description": "The group of all (Core + Remote + International) stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS101",
                                                    "CS103",
                                                    "CS201",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501",
                                                    "RS106",
                                                    "RS205",
                                                    "RS208",
                                                    "RS210",
                                                    "RS305",
                                                    "RS306",
                                                    "RS307",
                                                    "RS310",
                                                    "RS406",
                                                    "RS407",
                                                    "RS409",
                                                    "RS503",
                                                    "RS508",
                                                    "RS509",
                                                    "DE601",
                                                    "DE602",
                                                    "DE603",
                                                    "DE604",
                                                    "DE605",
                                                    "DE609",
                                                    "FR606",
                                                    "SE607",
                                                    "UK608",
                                                    "PL610",
                                                    "PL611",
                                                    "PL612",
                                                    "IE613",
                                                    "LV614"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "All",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 0,
                                        "stations": [
                                            "CS001"
                                        ]
                                    },
                                    "description": "A custom group of stations which can be defined by the user",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list"
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Custom",
                                    "type": "object"
                                }
                            ],
                            "default": {
                                "max_nr_missing": 1,
                                "stations": [
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007"
                                ]
                            },
                            "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                            "title": "Station group",
                            "type": "object"
                        }
                    }
                },
                "schema": {
                    "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2#",
                    "type": "object",
                    "title": "parallel calibrator target observation",
                    "$schema": "http://json-schema.org/draft-06/schema#",
                    "version": 2,
                    "required": [
                        "calibrator",
                        "correlator",
                        "duration",
                        "station_configuration"
                    ],
                    "properties": {
                        "QA": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/2#/definitions/QA",
                            "default": {}
                        },
                        "duration": {
                            "$id": "#duration",
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                            "title": "Duration",
                            "default": 600,
                            "minimum": 1,
                            "description": "Duration of this observation (seconds)"
                        },
                        "calibrator": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/calibrator/1#/definitions/calibrator",
                            "default": {}
                        },
                        "correlator": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/correlator/1#/definitions/correlator",
                            "default": {}
                        },
                        "station_configuration": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                            "default": {}
                        }
                    },
                    "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                    "patternProperties": {
                        "^[$]schema$": {}
                    },
                    "additionalProperties": false
                },
                "state": "http://localhost:3000/api/template_state/active",
                "state_value": "active",
                "tags": [],
                "type": "http://localhost:3000/api/task_type/observation",
                "type_value": "observation",
                "updated_at": "2022-04-14T06:20:07.273910",
                "version": 2
            },
            "specifications_template_id": 5,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/1"
            ],
            "task_blueprints_ids": [
                1
            ],
            "task_type": "observation",
            "updated_at": "2022-04-14T06:20:19.292495"
        }
      ]
    },
    {
      "id": 9,
      "url": "http://localhost:3000/api/task_draft/9",
      "connector_types": [
          {
              "id": 3,
              "url": "http://localhost:3000/api/task_connector_type/3",
              "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
              "dataformat_value": "MeasurementSet",
              "datatype": "http://localhost:3000/api/datatype/visibilities",
              "datatype_value": "visibilities",
              "iotype": "http://localhost:3000/api/iotype/output",
              "iotype_value": "output",
              "role": "http://localhost:3000/api/role/correlator",
              "role_value": "correlator",
              "task_template": "http://localhost:3000/api/task_template/5",
              "task_template_id": 5
          }
      ],
      "consumed_by": [
          "http://localhost:3000/api/task_relation_draft/13",
          "http://localhost:3000/api/task_relation_draft/14",
          "http://localhost:3000/api/task_relation_draft/15",
          "http://localhost:3000/api/task_relation_draft/16",
          "http://localhost:3000/api/task_relation_draft/21"
      ],
      "consumed_by_ids": [
          13,
          14,
          15,
          16,
          21
      ],
      "created_at": "2022-04-14T06:20:22.242382",
      "description": "Combined parallel Calibrator & Target Observation for LBA with 3 target beams",
      "duration": 120,
      "first_scheduling_relation": [],
      "first_scheduling_relation_ids": [],
      "name": "Combined Observation",
      "output_pinned": false,
      "produced_by": [],
      "produced_by_ids": [],
      "relative_start_time": 0,
      "relative_stop_time": 120,
      "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/2",
      "scheduling_unit_draft_id": 2,
      "second_scheduling_relation": [],
      "second_scheduling_relation_ids": [],
      "short_description": "OOO.O Paaa+01 Paaa+02 Paaa+03",
      "specifications_doc": {
          "QA": {
              "plots": {
                  "enabled": true,
                  "autocorrelation": true,
                  "crosscorrelation": true
              },
              "file_conversion": {
                  "enabled": true,
                  "nr_of_subbands": -1,
                  "nr_of_timestamps": 256
              },
              "inspection_plots": "msplots"
          },
          "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
          "duration": 120,
          "calibrator": {
              "name": "calibrator",
              "pointing": {
                  "angle1": 0.6624317181687094,
                  "angle2": 1.5579526427549426,
                  "target": "3Cabc",
                  "direction_type": "J2000"
              },
              "autoselect": false
          },
          "correlator": {
              "storage_cluster": "CEP4",
              "integration_time": 1,
              "channels_per_subband": 64,
              "topocentric_frequency_correction": false
          },
          "station_configuration": {
              "SAPs": [
                  {
                      "name": "target1",
                      "subbands": [
                          217,
                          218,
                          219,
                          220,
                          221,
                          222,
                          223,
                          224,
                          225,
                          226,
                          227,
                          228,
                          229,
                          230,
                          231,
                          232,
                          233,
                          234,
                          235,
                          236,
                          237,
                          238,
                          239,
                          240,
                          241,
                          242,
                          243,
                          244,
                          245,
                          246,
                          247,
                          248,
                          249,
                          250,
                          251,
                          252,
                          253,
                          254,
                          255,
                          256,
                          257,
                          258,
                          259,
                          260,
                          261,
                          262,
                          263,
                          264,
                          265,
                          266,
                          267,
                          268,
                          269,
                          270,
                          271,
                          272,
                          273,
                          274,
                          275,
                          276,
                          277,
                          278,
                          279,
                          280,
                          281,
                          282,
                          283,
                          284,
                          285,
                          286,
                          287,
                          288,
                          289,
                          290,
                          291,
                          292,
                          293,
                          294,
                          295,
                          296,
                          297,
                          298,
                          299,
                          300,
                          301,
                          302,
                          303,
                          304,
                          305,
                          306,
                          307,
                          308,
                          309,
                          310,
                          311,
                          312,
                          313,
                          314,
                          315,
                          316,
                          317,
                          318,
                          319,
                          320,
                          321,
                          322,
                          323,
                          324,
                          325,
                          326,
                          327,
                          328,
                          329,
                          330,
                          331,
                          332,
                          333,
                          334,
                          335,
                          336,
                          337,
                          338
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+01",
                          "direction_type": "J2000"
                      }
                  },
                  {
                      "name": "target2",
                      "subbands": [
                          217,
                          218,
                          219,
                          220,
                          221,
                          222,
                          223,
                          224,
                          225,
                          226,
                          227,
                          228,
                          229,
                          230,
                          231,
                          232,
                          233,
                          234,
                          235,
                          236,
                          237,
                          238,
                          239,
                          240,
                          241,
                          242,
                          243,
                          244,
                          245,
                          246,
                          247,
                          248,
                          249,
                          250,
                          251,
                          252,
                          253,
                          254,
                          255,
                          256,
                          257,
                          258,
                          259,
                          260,
                          261,
                          262,
                          263,
                          264,
                          265,
                          266,
                          267,
                          268,
                          269,
                          270,
                          271,
                          272,
                          273,
                          274,
                          275,
                          276,
                          277,
                          278,
                          279,
                          280,
                          281,
                          282,
                          283,
                          284,
                          285,
                          286,
                          287,
                          288,
                          289,
                          290,
                          291,
                          292,
                          293,
                          294,
                          295,
                          296,
                          297,
                          298,
                          299,
                          300,
                          301,
                          302,
                          303,
                          304,
                          305,
                          306,
                          307,
                          308,
                          309,
                          310,
                          311,
                          312,
                          313,
                          314,
                          315,
                          316,
                          317,
                          318,
                          319,
                          320,
                          321,
                          322,
                          323,
                          324,
                          325,
                          326,
                          327,
                          328,
                          329,
                          330,
                          331,
                          332,
                          333,
                          334,
                          335,
                          336,
                          337,
                          338
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+02",
                          "direction_type": "J2000"
                      }
                  },
                  {
                      "name": "target3",
                      "subbands": [
                          217,
                          218,
                          219,
                          220,
                          221,
                          222,
                          223,
                          224,
                          225,
                          226,
                          227,
                          228,
                          229,
                          230,
                          231,
                          232,
                          233,
                          234,
                          235,
                          236,
                          237,
                          238,
                          239,
                          240,
                          241,
                          242,
                          243,
                          244,
                          245,
                          246,
                          247,
                          248,
                          249,
                          250,
                          251,
                          252,
                          253,
                          254,
                          255,
                          256,
                          257,
                          258,
                          259,
                          260,
                          261,
                          262,
                          263,
                          264,
                          265,
                          266,
                          267,
                          268,
                          269,
                          270,
                          271,
                          272,
                          273,
                          274,
                          275,
                          276,
                          277,
                          278,
                          279,
                          280,
                          281,
                          282,
                          283,
                          284,
                          285,
                          286,
                          287,
                          288,
                          289,
                          290,
                          291,
                          292,
                          293,
                          294,
                          295,
                          296,
                          297,
                          298,
                          299,
                          300,
                          301,
                          302,
                          303,
                          304,
                          305,
                          306,
                          307,
                          308,
                          309,
                          310,
                          311,
                          312,
                          313,
                          314,
                          315,
                          316,
                          317,
                          318,
                          319,
                          320,
                          321,
                          322,
                          323,
                          324,
                          325,
                          326,
                          327,
                          328,
                          329,
                          330,
                          331,
                          332,
                          333,
                          334,
                          335,
                          336,
                          337,
                          338
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+03",
                          "direction_type": "J2000"
                      }
                  }
              ],
              "filter": "LBA_30_90",
              "tile_beam": {
                  "angle1": 0.6624317181687094,
                  "angle2": 1.5579526427549426,
                  "target": "_target_name_",
                  "direction_type": "J2000"
              },
              "antenna_set": "LBA_SPARSE_EVEN",
              "station_groups": [
                  {
                      "stations": [
                          "CS001",
                          "CS002",
                          "CS003",
                          "CS004",
                          "CS005",
                          "CS006",
                          "CS007",
                          "CS011",
                          "CS013",
                          "CS017",
                          "CS021",
                          "CS024",
                          "CS026",
                          "CS028",
                          "CS030",
                          "CS031",
                          "CS032",
                          "CS101",
                          "CS103",
                          "CS201",
                          "CS301",
                          "CS302",
                          "CS401",
                          "CS501",
                          "RS106",
                          "RS205",
                          "RS208",
                          "RS210",
                          "RS305",
                          "RS306",
                          "RS307",
                          "RS310",
                          "RS406",
                          "RS407",
                          "RS409",
                          "RS503",
                          "RS508",
                          "RS509"
                      ],
                      "max_nr_missing": 4
                  },
                  {
                      "stations": [
                          "RS508",
                          "RS509"
                      ],
                      "max_nr_missing": 1
                  },
                  {
                      "stations": [
                          "RS310",
                          "RS210"
                      ],
                      "max_nr_missing": 0
                  }
              ]
          }
      },
      "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "parallel calibrator target observation"})).id,
      "specifications_template_id": (_.find(templates, {"name": "parallel calibrator target observation"})).id,
      "tags": [],
      "task_blueprints": [
          "http://localhost:3000/api/task_blueprint/9"
      ],
      "task_blueprints_ids": [
          9
      ],
      "task_type": "observation",
      "updated_at": "2022-04-14T06:20:22.242394",
      "predecessors": [],
      "successors": [
        {
            "id": 11,
            "url": "http://localhost:3000/api/task_draft/11",
            "connector_types": [
                {
                    "id": 13,
                    "url": "http://localhost:3000/api/task_connector_type/13",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                },
                {
                    "id": 12,
                    "url": "http://localhost:3000/api/task_connector_type/12",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/20",
                "http://localhost:3000/api/task_relation_draft/24"
            ],
            "consumed_by_ids": [
                20,
                24
            ],
            "created_at": "2022-04-14T06:20:22.259971",
            "description": "Preprocessing Pipeline for Target Observation target3",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target3",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/16"
            ],
            "produced_by_ids": [
                16
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/2",
            "scheduling_unit_draft_id": 2,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "c17 oOOO.O Paaa+03",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 8,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 4,
                    "frequency_steps": 8
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/11"
            ],
            "task_blueprints_ids": [
                11
            ],
            "task_type": "pipeline",
            "updated_at": "2022-04-14T06:20:22.259982"
        },
        {
            "id": 13,
            "url": "http://localhost:3000/api/task_draft/13",
            "connector_types": [
                {
                    "id": 13,
                    "url": "http://localhost:3000/api/task_connector_type/13",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                },
                {
                    "id": 12,
                    "url": "http://localhost:3000/api/task_connector_type/12",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/18",
                "http://localhost:3000/api/task_relation_draft/22"
            ],
            "consumed_by_ids": [
                18,
                22
            ],
            "created_at": "2022-04-14T06:20:22.275062",
            "description": "Preprocessing Pipeline for Target Observation target1",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target1",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/14"
            ],
            "produced_by_ids": [
                14
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/2",
            "scheduling_unit_draft_id": 2,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "c17 oOOO.O Paaa+01",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 8,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 4,
                    "frequency_steps": 8
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/13"
            ],
            "task_blueprints_ids": [
                13
            ],
            "task_type": "pipeline",
            "updated_at": "2022-04-14T06:20:22.275073"
        },
        {
            "id": 10,
            "url": "http://localhost:3000/api/task_draft/10",
            "connector_types": [
                {
                    "id": 13,
                    "url": "http://localhost:3000/api/task_connector_type/13",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                },
                {
                    "id": 12,
                    "url": "http://localhost:3000/api/task_connector_type/12",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/17"
            ],
            "consumed_by_ids": [
                17
            ],
            "created_at": "2022-04-14T06:20:22.251986",
            "description": "Preprocessing Pipeline for Calibrator Observation",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Calibrator Pipeline",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/13"
            ],
            "produced_by_ids": [
                13
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/2",
            "scheduling_unit_draft_id": 2,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "c17 oOOO.O 3Cabc",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 8,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 4,
                    "frequency_steps": 8
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/10"
            ],
            "task_blueprints_ids": [
                10
            ],
            "task_type": "pipeline",
            "updated_at": "2022-04-14T06:20:22.251997"
        },
        {
            "id": 14,
            "url": "http://localhost:3000/api/task_draft/14",
            "connector_types": [
                {
                    "id": 48,
                    "url": "http://localhost:3000/api/task_connector_type/48",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/correlator",
                    "role_value": "correlator",
                    "task_template": "http://localhost:3000/api/task_template/1",
                    "task_template_id": 1
                },
                {
                    "id": 43,
                    "url": "http://localhost:3000/api/task_connector_type/43",
                    "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                    "dataformat_value": "Beamformed",
                    "datatype": "http://localhost:3000/api/datatype/time%20series",
                    "datatype_value": "time series",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/1",
                    "task_template_id": 1
                },
                {
                    "id": 42,
                    "url": "http://localhost:3000/api/task_connector_type/42",
                    "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                    "dataformat_value": "Beamformed",
                    "datatype": "http://localhost:3000/api/datatype/time%20series",
                    "datatype_value": "time series",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/beamformer",
                    "role_value": "beamformer",
                    "task_template": "http://localhost:3000/api/task_template/1",
                    "task_template_id": 1
                },
                {
                    "id": 40,
                    "url": "http://localhost:3000/api/task_connector_type/40",
                    "dataformat": "http://localhost:3000/api/dataformat/pulp%20summary",
                    "dataformat_value": "pulp summary",
                    "datatype": "http://localhost:3000/api/datatype/quality",
                    "datatype_value": "quality",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/1",
                    "task_template_id": 1
                },
                {
                    "id": 38,
                    "url": "http://localhost:3000/api/task_connector_type/38",
                    "dataformat": "http://localhost:3000/api/dataformat/pulp%20analysis",
                    "dataformat_value": "pulp analysis",
                    "datatype": "http://localhost:3000/api/datatype/pulsar%20profile",
                    "datatype_value": "pulsar profile",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/1",
                    "task_template_id": 1
                },
                {
                    "id": 36,
                    "url": "http://localhost:3000/api/task_connector_type/36",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/1",
                    "task_template_id": 1
                }
            ],
            "consumed_by": [],
            "consumed_by_ids": [],
            "created_at": "2022-04-14T06:20:22.280424",
            "description": "Clean up all dataproducts from disk after ingest",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Cleanup",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/21",
                "http://localhost:3000/api/task_relation_draft/22",
                "http://localhost:3000/api/task_relation_draft/23",
                "http://localhost:3000/api/task_relation_draft/24"
            ],
            "produced_by_ids": [
                21,
                22,
                23,
                24
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/2",
            "scheduling_unit_draft_id": 2,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "",
            "specifications_doc": {
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2/ref_resolved"
            },
            "specifications_template": "http://localhost:3000/api/task_template/1",
            "specifications_template_id": 1,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/14"
            ],
            "task_blueprints_ids": [
                14
            ],
            "task_type": "cleanup",
            "updated_at": "2022-04-14T06:20:22.280436"
        },
        {
            "id": 12,
            "url": "http://localhost:3000/api/task_draft/12",
            "connector_types": [
                {
                    "id": 13,
                    "url": "http://localhost:3000/api/task_connector_type/13",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                },
                {
                    "id": 12,
                    "url": "http://localhost:3000/api/task_connector_type/12",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/19",
                "http://localhost:3000/api/task_relation_draft/23"
            ],
            "consumed_by_ids": [
                19,
                23
            ],
            "created_at": "2022-04-14T06:20:22.267248",
            "description": "Preprocessing Pipeline for Target Observation target2",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target2",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/15"
            ],
            "produced_by_ids": [
                15
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/2",
            "scheduling_unit_draft_id": 2,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "c17 oOOO.O Paaa+02",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 8,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 4,
                    "frequency_steps": 8
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/12"
            ],
            "task_blueprints_ids": [
                12
            ],
            "task_type": "pipeline",
            "updated_at": "2022-04-14T06:20:22.267260"
        }
      ]
    },
    {
        "id": 21,
        "url": "http://127.0.0.100:8008/api/task_draft/21",
        "connector_types": [
            {
                "id": 1,
                "url": "http://127.0.0.100:8008/api/task_connector_type/1",
                "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                "dataformat_value": "MeasurementSet",
                "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                "datatype_value": "visibilities",
                "iotype": "http://127.0.0.100:8008/api/iotype/output",
                "iotype_value": "output",
                "role": "http://127.0.0.100:8008/api/role/correlator",
                "role_value": "correlator",
                "task_template": "http://127.0.0.100:8008/api/task_template/7",
                "task_template_id": 7
            }
        ],
        "consumed_by": [
            "http://127.0.0.100:8008/api/task_relation_draft/1"
        ],
        "consumed_by_ids": [
            1
        ],
        "created_at": "2022-05-06T06:40:12.523377",
        "description": "Calibrator Observation 1. Before the Target Observation",
        "duration": 600.0,
        "first_scheduling_relation": [
            "http://127.0.0.100:8008/api/task_scheduling_relation_draft/1"
        ],
        "first_scheduling_relation_ids": [
            1
        ],
        "name": "Calibrator Observation 1",
        "output_pinned": false,
        "produced_by": [],
        "produced_by_ids": [],
        "relative_start_time": 0.0,
        "relative_stop_time": 600.0,
        "scheduling_unit_draft": "http://127.0.0.100:8008/api/scheduling_unit_draft/7",
        "scheduling_unit_draft_id": 7,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "Cal1 3Cabc",
        "specifications_doc": {
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2/ref_resolved",
            "duration": 600,
            "calibrator": {
                "name": "calibrator1",
                "pointing": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "3Cabc",
                    "direction_type": "J2000"
                },
                "autoselect": false
            }
        },
        "specifications_template": "http://127.0.0.100:8008/api/task_template/"+(_.find(templates, {"name": "calibrator observation"})).id,
        "specifications_template_id": (_.find(templates, {"name": "calibrator observation"})).id,
        "tags": [],
        "task_blueprints": [],
        "task_blueprints_ids": [],
        "task_type": "observation",
        "updated_at": "2022-05-06T06:40:14.359591",
        "predecessors": [],
        "successors": [
            {
                "id": 22,
                "url": "http://localhost:3000/api/task_draft/22",
                "connector_types": [
                    {
                        "id": 13,
                        "url": "http://localhost:3000/api/task_connector_type/13",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/10",
                        "task_template_id": 10
                    },
                    {
                        "id": 12,
                        "url": "http://localhost:3000/api/task_connector_type/12",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/10",
                        "task_template_id": 10
                    }
                ],
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_draft/38"
                ],
                "consumed_by_ids": [
                    38
                ],
                "created_at": "2022-05-09T05:13:04.176326",
                "description": "Preprocessing Pipeline for Calibrator Observation 1",
                "duration": 0,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Calibrator Pipeline 1",
                "output_pinned": false,
                "produced_by": [
                    "http://localhost:3000/api/task_relation_draft/34"
                ],
                "produced_by_ids": [
                    34
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/4",
                "scheduling_unit_draft_id": 4,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Cal1 3Cabc/PP",
                "specifications_doc": {
                    "flag": {
                        "rfi_strategy": "HBAdefault",
                        "outerchannels": true,
                        "autocorrelations": true
                    },
                    "demix": {
                        "sources": [],
                        "time_steps": 10,
                        "ignore_target": false,
                        "frequency_steps": 64
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "average": {
                        "time_steps": 1,
                        "frequency_steps": 4
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    }
                },
                "specifications_template": "http://localhost:3000/api/task_template/10",
                "specifications_template_id": 10,
                "tags": [],
                "task_blueprints": [
                    "http://localhost:3000/api/task_blueprint/23"
                ],
                "task_blueprints_ids": [
                    23
                ],
                "task_type": "pipeline",
                "updated_at": "2022-05-09T05:13:04.176350"
            }
        ]
    },
    {
        "id": 22,
        "url": "http://127.0.0.100:8008/api/task_draft/22",
        "connector_types": [
            {
                "id": 13,
                "url": "http://127.0.0.100:8008/api/task_connector_type/13",
                "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                "dataformat_value": "MeasurementSet",
                "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                "datatype_value": "visibilities",
                "iotype": "http://127.0.0.100:8008/api/iotype/output",
                "iotype_value": "output",
                "role": "http://127.0.0.100:8008/api/role/any",
                "role_value": "any",
                "task_template": "http://127.0.0.100:8008/api/task_template/10",
                "task_template_id": 10
            },
            {
                "id": 12,
                "url": "http://127.0.0.100:8008/api/task_connector_type/12",
                "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                "dataformat_value": "MeasurementSet",
                "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                "datatype_value": "visibilities",
                "iotype": "http://127.0.0.100:8008/api/iotype/input",
                "iotype_value": "input",
                "role": "http://127.0.0.100:8008/api/role/any",
                "role_value": "any",
                "task_template": "http://127.0.0.100:8008/api/task_template/10",
                "task_template_id": 10
            }
        ],
        "consumed_by": [
            "http://127.0.0.100:8008/api/task_relation_draft/5"
        ],
        "consumed_by_ids": [
            5
        ],
        "created_at": "2022-05-06T06:40:12.532475",
        "description": "Preprocessing Pipeline for Calibrator Observation 1",
        "duration": 0.0,
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Calibrator Pipeline 1",
        "output_pinned": false,
        "produced_by": [
            "http://127.0.0.100:8008/api/task_relation_draft/1"
        ],
        "produced_by_ids": [
            1
        ],
        "relative_start_time": 0.0,
        "relative_stop_time": 0.0,
        "scheduling_unit_draft": "http://127.0.0.100:8008/api/scheduling_unit_draft/7",
        "scheduling_unit_draft_id": 7,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "Cal1 3Cabc/PP",
        "specifications_doc": {
            "flag": {
                "rfi_strategy": "HBAdefault",
                "outerchannels": true,
                "autocorrelations": true
            },
            "demix": {
                "sources": [],
                "time_steps": 10,
                "ignore_target": false,
                "frequency_steps": 64
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
            "average": {
                "time_steps": 1,
                "frequency_steps": 4
            },
            "storagemanager": "dysco",
            "cluster_resources": {
                "where": {
                    "cluster": "CEP4",
                    "partition": "cpu"
                },
                "cores_per_task": 2,
                "parallel_tasks": 122
            }
        },
        "specifications_template": "http://127.0.0.100:8008/api/task_template/"+(_.find(templates, {"name": "preprocessing pipeline"})).id,
        "specifications_template_id": (_.find(templates, {"name": "preprocessing pipeline"})).id,
        "tags": [],
        "task_blueprints": [],
        "task_blueprints_ids": [],
        "task_type": "pipeline",
        "updated_at": "2022-05-06T06:40:14.497477",
        "predecessors": [
            {
              "id": 22,
              "url": "http://localhost:3000/api/task_draft/22",
              "connector_types": [
                {
                  "id": 1,
                  "url": "http://localhost:3000/api/task_connector_type/1",
                  "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                  "dataformat_value": "MeasurementSet",
                  "datatype": "http://localhost:3000/api/datatype/visibilities",
                  "datatype_value": "visibilities",
                  "iotype": "http://localhost:3000/api/iotype/output",
                  "iotype_value": "output",
                  "role": "http://localhost:3000/api/role/correlator",
                  "role_value": "correlator",
                  "task_template": "http://localhost:3000/api/task_template/7",
                  "task_template_id": 7
                }
              ],
              "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/34"
              ],
              "consumed_by_ids": [
                34
              ],
              "created_at": "2022-05-09T05:13:04.155696",
              "description": "Calibrator Observation 1. Before the Target Observation",
              "duration": 600,
              "first_scheduling_relation": [
                "http://localhost:3000/api/task_scheduling_relation_draft/1"
              ],
              "first_scheduling_relation_ids": [
                1
              ],
              "name": "Calibrator Observation 1",
              "output_pinned": false,
              "produced_by": [],
              "produced_by_ids": [],
              "relative_start_time": 0,
              "relative_stop_time": 600,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/4",
              "scheduling_unit_draft_id": 4,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Cal1 3Cabc",
              "specifications_doc": {
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2/ref_resolved",
                "duration": 600,
                "calibrator": {
                  "name": "calibrator1",
                  "pointing": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "3Cabc",
                    "direction_type": "J2000"
                  },
                  "autoselect": false
                }
              },
              "specifications_template": {
                "id": 7,
                "url": "http://localhost:3000/api/task_template/7",
                "created_at": "2022-05-09T05:12:33.393061",
                "description": "This schema defines the (extra) parameters to setup a calibrator observation task, which uses all parameters from the target observation task which it is linked to, plus these calibrator overrides.",
                "name": "calibrator observation",
                "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                "purpose_value": "technical_commissioning",
                "ref_resolved_schema": {
                  "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2/ref_resolved",
                  "title": "calibrator observation",
                  "$schema": "http://json-schema.org/draft-06/schema#",
                  "default": {},
                  "version": 2,
                  "required": [
                    "calibrator",
                    "duration"
                  ],
                  "properties": {
                    "duration": {
                      "$id": "#duration",
                      "$ref": "#/definitions/timedelta",
                      "title": "Duration",
                      "default": 600,
                      "minimum": 1,
                      "description": "Duration of this observation (seconds)"
                    },
                    "calibrator": {
                      "$ref": "#/definitions/calibrator",
                      "default": {}
                    }
                  },
                  "description": "This schema defines the (extra) parameters to setup a calibrator observation task, which uses all parameters from the target observation task which it is linked to, plus these calibrator overrides.",
                  "patternProperties": {
                    "^[$]schema$": {}
                  },
                  "additionalProperties": false,
                  "definitions": {
                    "timedelta": {
                      "default": 0,
                      "description": "A time duration or delta expressed in seconds",
                      "type": "number"
                    },
                    "timestamp": {
                      "default": "1970-01-01T00:00:00Z",
                      "description": "A timestamp defined in UTC",
                      "format": "date-time",
                      "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                      "type": "string"
                    },
                    "calibrator": {
                      "additionalProperties": false,
                      "default": {},
                      "properties": {
                        "autoselect": {
                          "default": true,
                          "description": "Auto-select calibrator based on elevation",
                          "title": "Auto-select",
                          "type": "boolean"
                        },
                        "name": {
                          "default": "calibrator",
                          "description": "Name of the calibrator SAP",
                          "title": "Name",
                          "type": "string"
                        },
                        "pointing": {
                          "$ref": "#/definitions/pointing",
                          "default": {},
                          "description": "Manually selected calibrator",
                          "title": "Digital pointing"
                        }
                      },
                      "required": [
                        "autoselect",
                        "pointing",
                        "name"
                      ],
                      "type": "object"
                    },
                    "pointing": {
                      "additionalProperties": false,
                      "properties": {
                        "angle1": {
                          "default": 0.6624317181687094,
                          "description": "First angle (e.g. RA)",
                          "title": "Angle 1",
                          "type": "number"
                        },
                        "angle2": {
                          "default": 1.5579526427549426,
                          "description": "Second angle (e.g. DEC)",
                          "title": "Angle 2",
                          "type": "number"
                        },
                        "direction_type": {
                          "default": "J2000",
                          "description": "",
                          "enum": [
                            "J2000",
                            "AZELGEO",
                            "LMN",
                            "SUN",
                            "MOON",
                            "MERCURY",
                            "VENUS",
                            "MARS",
                            "JUPITER",
                            "SATURN",
                            "URANUS",
                            "NEPTUNE",
                            "PLUTO"
                          ],
                          "title": "Reference frame",
                          "type": "string"
                        },
                        "target": {
                          "default": "_target_name_",
                          "description": "Description of where this beam points at",
                          "minLength": 1,
                          "title": "Target",
                          "type": "string"
                        }
                      },
                      "required": [
                        "angle1",
                        "angle2",
                        "target"
                      ],
                      "type": "object"
                    }
                  }
                },
                "schema": {
                  "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2#",
                  "title": "calibrator observation",
                  "$schema": "http://json-schema.org/draft-06/schema#",
                  "default": {},
                  "version": 2,
                  "required": [
                    "calibrator",
                    "duration"
                  ],
                  "properties": {
                    "duration": {
                      "$id": "#duration",
                      "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                      "title": "Duration",
                      "default": 600,
                      "minimum": 1,
                      "description": "Duration of this observation (seconds)"
                    },
                    "calibrator": {
                      "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/calibrator/1#/definitions/calibrator",
                      "default": {}
                    }
                  },
                  "description": "This schema defines the (extra) parameters to setup a calibrator observation task, which uses all parameters from the target observation task which it is linked to, plus these calibrator overrides.",
                  "patternProperties": {
                    "^[$]schema$": {}
                  },
                  "additionalProperties": false
                },
                "state": "http://localhost:3000/api/template_state/active",
                "state_value": "active",
                "tags": [],
                "type": "http://localhost:3000/api/task_type/observation",
                "type_value": "observation",
                "updated_at": "2022-05-09T05:12:33.393095",
                "version": 2
              },
              "specifications_template_id": 7,
              "tags": [],
              "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/22"
              ],
              "task_blueprints_ids": [
                22
              ],
              "task_type": "observation",
              "updated_at": "2022-05-09T05:13:04.155710"
            }
          ],
        "successors": [
            {
              "id": 29,
              "url": "http://localhost:3000/api/task_draft/29",
              "connector_types": [
                {
                  "id": 26,
                  "url": "http://localhost:3000/api/task_connector_type/26",
                  "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                  "dataformat_value": "MeasurementSet",
                  "datatype": "http://localhost:3000/api/datatype/visibilities",
                  "datatype_value": "visibilities",
                  "iotype": "http://localhost:3000/api/iotype/input",
                  "iotype_value": "input",
                  "role": "http://localhost:3000/api/role/correlator",
                  "role_value": "correlator",
                  "task_template": "http://localhost:3000/api/task_template/2",
                  "task_template_id": 2
                },
                {
                  "id": 21,
                  "url": "http://localhost:3000/api/task_connector_type/21",
                  "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                  "dataformat_value": "Beamformed",
                  "datatype": "http://localhost:3000/api/datatype/time%20series",
                  "datatype_value": "time series",
                  "iotype": "http://localhost:3000/api/iotype/input",
                  "iotype_value": "input",
                  "role": "http://localhost:3000/api/role/any",
                  "role_value": "any",
                  "task_template": "http://localhost:3000/api/task_template/2",
                  "task_template_id": 2
                },
                {
                  "id": 20,
                  "url": "http://localhost:3000/api/task_connector_type/20",
                  "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                  "dataformat_value": "Beamformed",
                  "datatype": "http://localhost:3000/api/datatype/time%20series",
                  "datatype_value": "time series",
                  "iotype": "http://localhost:3000/api/iotype/input",
                  "iotype_value": "input",
                  "role": "http://localhost:3000/api/role/beamformer",
                  "role_value": "beamformer",
                  "task_template": "http://localhost:3000/api/task_template/2",
                  "task_template_id": 2
                },
                {
                  "id": 18,
                  "url": "http://localhost:3000/api/task_connector_type/18",
                  "dataformat": "http://localhost:3000/api/dataformat/pulp%20summary",
                  "dataformat_value": "pulp summary",
                  "datatype": "http://localhost:3000/api/datatype/quality",
                  "datatype_value": "quality",
                  "iotype": "http://localhost:3000/api/iotype/input",
                  "iotype_value": "input",
                  "role": "http://localhost:3000/api/role/any",
                  "role_value": "any",
                  "task_template": "http://localhost:3000/api/task_template/2",
                  "task_template_id": 2
                },
                {
                  "id": 16,
                  "url": "http://localhost:3000/api/task_connector_type/16",
                  "dataformat": "http://localhost:3000/api/dataformat/pulp%20analysis",
                  "dataformat_value": "pulp analysis",
                  "datatype": "http://localhost:3000/api/datatype/pulsar%20profile",
                  "datatype_value": "pulsar profile",
                  "iotype": "http://localhost:3000/api/iotype/input",
                  "iotype_value": "input",
                  "role": "http://localhost:3000/api/role/any",
                  "role_value": "any",
                  "task_template": "http://localhost:3000/api/task_template/2",
                  "task_template_id": 2
                },
                {
                  "id": 14,
                  "url": "http://localhost:3000/api/task_connector_type/14",
                  "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                  "dataformat_value": "MeasurementSet",
                  "datatype": "http://localhost:3000/api/datatype/visibilities",
                  "datatype_value": "visibilities",
                  "iotype": "http://localhost:3000/api/iotype/input",
                  "iotype_value": "input",
                  "role": "http://localhost:3000/api/role/any",
                  "role_value": "any",
                  "task_template": "http://localhost:3000/api/task_template/2",
                  "task_template_id": 2
                }
              ],
              "consumed_by": [],
              "consumed_by_ids": [],
              "created_at": "2022-05-09T05:13:04.377634",
              "description": "Ingest all preprocessed dataproducts",
              "duration": 0,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Ingest",
              "output_pinned": false,
              "produced_by": [
                "http://localhost:3000/api/task_relation_draft/38",
                "http://localhost:3000/api/task_relation_draft/39",
                "http://localhost:3000/api/task_relation_draft/40",
                "http://localhost:3000/api/task_relation_draft/41"
              ],
              "produced_by_ids": [
                38,
                39,
                40,
                41
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/4",
              "scheduling_unit_draft_id": 4,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "",
              "specifications_doc": {
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
              },
              "specifications_template": "http://localhost:3000/api/task_template/2",
              "specifications_template_id": 2,
              "tags": [],
              "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/29"
              ],
              "task_blueprints_ids": [
                29
              ],
              "task_type": "ingest",
              "updated_at": "2022-05-09T05:13:04.377650"
            }
        ]
    },
    {
        "id": 23,
        "url": "http://localhost:3000/api/task_draft/23",
        "connector_types": [
            {
                "id": 2,
                "url": "http://localhost:3000/api/task_connector_type/2",
                "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                "dataformat_value": "MeasurementSet",
                "datatype": "http://localhost:3000/api/datatype/visibilities",
                "datatype_value": "visibilities",
                "iotype": "http://localhost:3000/api/iotype/output",
                "iotype_value": "output",
                "role": "http://localhost:3000/api/role/correlator",
                "role_value": "correlator",
                "task_template": "http://localhost:3000/api/task_template/5",
                "task_template_id": 5
            }
        ],
        "consumed_by": [
            "http://localhost:3000/api/task_relation_draft/25",
            "http://localhost:3000/api/task_relation_draft/26"
        ],
        "consumed_by_ids": [
            25,
            26
        ],
        "created_at": "2022-05-03T08:40:25.666199",
        "description": "Target Observation for LoTSS scheduling unit. HBA_DUAL_INNER, 120-168 MHz, 1s, 64ch/sb",
        "duration": 28800.0,
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Target Observation",
        "output_pinned": false,
        "produced_by": [],
        "produced_by_ids": [],
        "relative_start_time": 660.0,
        "relative_stop_time": 29460.0,
        "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/7",
        "scheduling_unit_draft_id": 7,
        "second_scheduling_relation": [
            "http://localhost:3000/api/task_scheduling_relation_draft/2",
            "http://localhost:3000/api/task_scheduling_relation_draft/3"
        ],
        "second_scheduling_relation_ids": [
            2,
            3
        ],
        "short_description": "Paaa+01 & Paaa+02",
        "specifications_doc": {
            "QA": {
                "plots": {
                    "enabled": true,
                    "autocorrelation": true,
                    "crosscorrelation": true
                },
                "file_conversion": {
                    "enabled": true,
                    "nr_of_subbands": -1,
                    "nr_of_timestamps": 256
                },
                "inspection_plots": "msplots"
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/2/ref_resolved",
            "duration": 28800,
            "correlator": {
                "storage_cluster": "CEP4",
                "integration_time": 1,
                "channels_per_subband": 64,
                "topocentric_frequency_correction": false
            },
            "station_configuration": {
                "": {
                    "tile_beam": {
                        "angle1": 0,
                        "angle2": 0,
                        "target": "_target_name_"
                    }
                },
                "SAPs": [
                    {
                        "name": "sap1",
                        "subbands": [
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154,
                            155,
                            156,
                            157,
                            158,
                            159,
                            160,
                            161,
                            162,
                            163,
                            165,
                            166,
                            167,
                            168,
                            169,
                            170,
                            171,
                            172,
                            173,
                            174,
                            175,
                            176,
                            177,
                            178,
                            179,
                            180,
                            182,
                            183,
                            184,
                            187,
                            188,
                            189,
                            190,
                            191,
                            192,
                            193,
                            194,
                            195,
                            196,
                            197,
                            198,
                            199,
                            200,
                            201,
                            202,
                            203,
                            204,
                            205,
                            206,
                            207,
                            208,
                            209,
                            212,
                            213,
                            215,
                            216,
                            217,
                            218,
                            219,
                            220,
                            221,
                            222,
                            223,
                            224,
                            225,
                            226,
                            227,
                            228,
                            229,
                            230,
                            231,
                            232,
                            233,
                            234,
                            235,
                            236,
                            237,
                            238,
                            239,
                            240,
                            242,
                            243,
                            244,
                            245,
                            246,
                            247,
                            248,
                            249,
                            250,
                            251,
                            252,
                            253,
                            254,
                            255,
                            257,
                            258,
                            259,
                            260,
                            261,
                            262,
                            263,
                            264,
                            265,
                            266,
                            267,
                            268,
                            269,
                            270,
                            271,
                            272,
                            273,
                            275,
                            276,
                            277,
                            278,
                            279,
                            280,
                            281,
                            282,
                            283,
                            284,
                            285,
                            286,
                            287,
                            288,
                            289,
                            290,
                            291,
                            292,
                            293,
                            294,
                            295,
                            296,
                            297,
                            298,
                            299,
                            300,
                            302,
                            303,
                            304,
                            305,
                            306,
                            307,
                            308,
                            309,
                            310,
                            311,
                            312,
                            313,
                            314,
                            315,
                            316,
                            317,
                            318,
                            319,
                            320,
                            321,
                            322,
                            323,
                            324,
                            325,
                            326,
                            327,
                            328,
                            330,
                            331,
                            332,
                            333,
                            334,
                            335,
                            336,
                            337,
                            338,
                            339,
                            340,
                            341,
                            342,
                            343,
                            344,
                            345,
                            346,
                            347,
                            349,
                            364,
                            372,
                            380,
                            388,
                            396,
                            404,
                            413,
                            421,
                            430,
                            438,
                            447
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+01",
                            "direction_type": "J2000"
                        }
                    },
                    {
                        "name": "sap2",
                        "subbands": [
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154,
                            155,
                            156,
                            157,
                            158,
                            159,
                            160,
                            161,
                            162,
                            163,
                            165,
                            166,
                            167,
                            168,
                            169,
                            170,
                            171,
                            172,
                            173,
                            174,
                            175,
                            176,
                            177,
                            178,
                            179,
                            180,
                            182,
                            183,
                            184,
                            187,
                            188,
                            189,
                            190,
                            191,
                            192,
                            193,
                            194,
                            195,
                            196,
                            197,
                            198,
                            199,
                            200,
                            201,
                            202,
                            203,
                            204,
                            205,
                            206,
                            207,
                            208,
                            209,
                            212,
                            213,
                            215,
                            216,
                            217,
                            218,
                            219,
                            220,
                            221,
                            222,
                            223,
                            224,
                            225,
                            226,
                            227,
                            228,
                            229,
                            230,
                            231,
                            232,
                            233,
                            234,
                            235,
                            236,
                            237,
                            238,
                            239,
                            240,
                            242,
                            243,
                            244,
                            245,
                            246,
                            247,
                            248,
                            249,
                            250,
                            251,
                            252,
                            253,
                            254,
                            255,
                            257,
                            258,
                            259,
                            260,
                            261,
                            262,
                            263,
                            264,
                            265,
                            266,
                            267,
                            268,
                            269,
                            270,
                            271,
                            272,
                            273,
                            275,
                            276,
                            277,
                            278,
                            279,
                            280,
                            281,
                            282,
                            283,
                            284,
                            285,
                            286,
                            287,
                            288,
                            289,
                            290,
                            291,
                            292,
                            293,
                            294,
                            295,
                            296,
                            297,
                            298,
                            299,
                            300,
                            302,
                            303,
                            304,
                            305,
                            306,
                            307,
                            308,
                            309,
                            310,
                            311,
                            312,
                            313,
                            314,
                            315,
                            316,
                            317,
                            318,
                            319,
                            320,
                            321,
                            322,
                            323,
                            324,
                            325,
                            326,
                            327,
                            328,
                            330,
                            331,
                            332,
                            333,
                            334,
                            335,
                            336,
                            337,
                            338,
                            339,
                            340,
                            341,
                            342,
                            343,
                            344,
                            345,
                            346,
                            347,
                            349,
                            364,
                            372,
                            380,
                            388,
                            396,
                            404,
                            413,
                            421,
                            430,
                            438,
                            447
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+02",
                            "direction_type": "J2000"
                        }
                    }
                ],
                "filter": "HBA_110_190",
                "tile_beam": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "Paaa+01Paaa+02REF",
                    "direction_type": "J2000"
                },
                "antenna_set": "HBA_DUAL_INNER",
                "station_groups": [
                    {
                        "stations": [
                            "CS001",
                            "CS002",
                            "CS003",
                            "CS004",
                            "CS005",
                            "CS006",
                            "CS007",
                            "CS011",
                            "CS013",
                            "CS017",
                            "CS021",
                            "CS024",
                            "CS026",
                            "CS028",
                            "CS030",
                            "CS031",
                            "CS032",
                            "CS101",
                            "CS103",
                            "CS201",
                            "CS301",
                            "CS302",
                            "CS401",
                            "CS501",
                            "RS106",
                            "RS205",
                            "RS208",
                            "RS210",
                            "RS305",
                            "RS306",
                            "RS307",
                            "RS310",
                            "RS406",
                            "RS407",
                            "RS409",
                            "RS503",
                            "RS508",
                            "RS509"
                        ],
                        "max_nr_missing": 4
                    },
                    {
                        "stations": [
                            "DE601",
                            "DE602",
                            "DE603",
                            "DE604",
                            "DE605",
                            "DE609",
                            "FR606",
                            "SE607",
                            "UK608",
                            "PL610",
                            "PL611",
                            "PL612",
                            "IE613",
                            "LV614"
                        ],
                        "max_nr_missing": 2
                    },
                    {
                        "stations": [
                            "DE601",
                            "DE605"
                        ],
                        "max_nr_missing": 1
                    }
                ]
            }
        },
        "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "target observation"})).id,
        "specifications_template_id": (_.find(templates, {"name": "target observation"})).id,
        "tags": [],
        "task_blueprints": [],
        "task_blueprints_ids": [],
        "task_type": "observation",
        "updated_at": "2022-05-03T08:40:26.857829",
        "predecessors": [],
        "successors": [
            {
                "id": 27,
                "url": "http://localhost:3000/api/task_draft/27",
                "connector_types": [
                    {
                        "id": 13,
                        "url": "http://localhost:3000/api/task_connector_type/13",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/10",
                        "task_template_id": 10
                    },
                    {
                        "id": 12,
                        "url": "http://localhost:3000/api/task_connector_type/12",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/10",
                        "task_template_id": 10
                    }
                ],
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_draft/41"
                ],
                "consumed_by_ids": [
                    41
                ],
                "created_at": "2022-05-09T05:13:04.335917",
                "description": "Preprocessing Pipeline for Target Observation target2, SAP001, 120-168 MHz, 1s, 16ch/sb",
                "duration": 0,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Pipeline target2",
                "output_pinned": false,
                "produced_by": [
                    "http://localhost:3000/api/task_relation_draft/37"
                ],
                "produced_by_ids": [
                    37
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/4",
                "scheduling_unit_draft_id": 4,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Paaa+02/TP",
                "specifications_doc": {
                    "flag": {
                        "rfi_strategy": "HBAdefault",
                        "outerchannels": true,
                        "autocorrelations": true
                    },
                    "demix": {
                        "sources": [],
                        "time_steps": 10,
                        "ignore_target": false,
                        "frequency_steps": 64
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "average": {
                        "time_steps": 1,
                        "frequency_steps": 4
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    }
                },
                "specifications_template": "http://localhost:3000/api/task_template/10",
                "specifications_template_id": 10,
                "tags": [],
                "task_blueprints": [
                    "http://localhost:3000/api/task_blueprint/27"
                ],
                "task_blueprints_ids": [
                    27
                ],
                "task_type": "pipeline",
                "updated_at": "2022-05-09T05:13:04.335942"
            },
            {
                "id": 28,
                "url": "http://localhost:3000/api/task_draft/28",
                "connector_types": [
                    {
                        "id": 13,
                        "url": "http://localhost:3000/api/task_connector_type/13",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/10",
                        "task_template_id": 10
                    },
                    {
                        "id": 12,
                        "url": "http://localhost:3000/api/task_connector_type/12",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/10",
                        "task_template_id": 10
                    }
                ],
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_draft/40"
                ],
                "consumed_by_ids": [
                    40
                ],
                "created_at": "2022-05-09T05:13:04.359065",
                "description": "Preprocessing Pipeline for Target Observation target1, SAP000, 120-168 MHz, 1s, 16ch/sb",
                "duration": 0,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Pipeline target1",
                "output_pinned": false,
                "produced_by": [
                    "http://localhost:3000/api/task_relation_draft/36"
                ],
                "produced_by_ids": [
                    36
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/4",
                "scheduling_unit_draft_id": 4,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Paaa+01/TP",
                "specifications_doc": {
                    "flag": {
                        "rfi_strategy": "HBAdefault",
                        "outerchannels": true,
                        "autocorrelations": true
                    },
                    "demix": {
                        "sources": [],
                        "time_steps": 10,
                        "ignore_target": false,
                        "frequency_steps": 64
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "average": {
                        "time_steps": 1,
                        "frequency_steps": 4
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    }
                },
                "specifications_template": "http://localhost:3000/api/task_template/10",
                "specifications_template_id": 10,
                "tags": [],
                "task_blueprints": [
                    "http://localhost:3000/api/task_blueprint/28"
                ],
                "task_blueprints_ids": [
                    28
                ],
                "task_type": "pipeline",
                "updated_at": "2022-05-09T05:13:04.359096"
            }
        ]
    },
    {
        "id": 50,
        "url": "http://localhost:3000/api/task_draft/50",
        "connector_types": [
            {
                "id": 11,
                "url": "http://localhost:3000/api/task_connector_type/11",
                "dataformat": "http://localhost:3000/api/dataformat/pulp%20analysis",
                "dataformat_value": "pulp analysis",
                "datatype": "http://localhost:3000/api/datatype/pulsar%20profile",
                "datatype_value": "pulsar profile",
                "iotype": "http://localhost:3000/api/iotype/output",
                "iotype_value": "output",
                "role": "http://localhost:3000/api/role/any",
                "role_value": "any",
                "task_template": "http://localhost:3000/api/task_template/9",
                "task_template_id": 9
            },
            {
                "id": 10,
                "url": "http://localhost:3000/api/task_connector_type/10",
                "dataformat": "http://localhost:3000/api/dataformat/pulp%20summary",
                "dataformat_value": "pulp summary",
                "datatype": "http://localhost:3000/api/datatype/quality",
                "datatype_value": "quality",
                "iotype": "http://localhost:3000/api/iotype/output",
                "iotype_value": "output",
                "role": "http://localhost:3000/api/role/any",
                "role_value": "any",
                "task_template": "http://localhost:3000/api/task_template/9",
                "task_template_id": 9
            },
            {
                "id": 9,
                "url": "http://localhost:3000/api/task_connector_type/9",
                "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                "dataformat_value": "Beamformed",
                "datatype": "http://localhost:3000/api/datatype/time%20series",
                "datatype_value": "time series",
                "iotype": "http://localhost:3000/api/iotype/input",
                "iotype_value": "input",
                "role": "http://localhost:3000/api/role/beamformer",
                "role_value": "beamformer",
                "task_template": "http://localhost:3000/api/task_template/9",
                "task_template_id": 9
            }
        ],
        "consumed_by": [
            "http://localhost:3000/api/task_relation_draft/67",
            "http://localhost:3000/api/task_relation_draft/68",
            "http://localhost:3000/api/task_relation_draft/70",
            "http://localhost:3000/api/task_relation_draft/71"
        ],
        "consumed_by_ids": [
            67,
            68,
            70,
            71
        ],
        "created_at": "2022-04-21T08:16:30.745911",
        "description": "Pulsar Pipeline for default HBA pulsar timing observation",
        "duration": 0,
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Pipeline",
        "output_pinned": false,
        "produced_by": [
            "http://localhost:3000/api/task_relation_draft/66"
        ],
        "produced_by_ids": [
            66
        ],
        "relative_start_time": 0,
        "relative_stop_time": 0,
        "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/9",
        "scheduling_unit_draft_id": 9,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "",
        "specifications_doc": {
            "dspsr": {
                "digifil": {
                    "dm": 0,
					"channels_per_part": 320,
					"coherent_dedispersion": true,
					"integration_time_factor": 4
                },
                "enabled": true,
                "filterbank": {
                    "enabled": false,
                    "channels_per_part": 120,
                    "coherent_dedispersion": true
                },
                "rfi_excision": true,
                "optimise_period_dm": true,
                "subintegration_length": -1,
                "single_pulse_subintegration": false
            },
            "output": {
                "quantisation": {
                    "scale": 5,
                    "enabled": false
                },
                "dynamic_spectrum": {
                    "enabled": false,
                    "time_average": 0.5
                }
            },
            "presto": {
                "input": {
                    "nr_blocks": 100,
                    "decode_sigma": 3,
                    "samples_per_block": 8192
                },
                "rrats": {
                    "enabled": false,
                    "dm_range": 5
                },
                "rfifind": {
                    "blocks": 16
                },
                "prepdata": {
                    "dm": -1
                },
                "prepfold": false,
                "fold_profile": true
            },
            "pulsar": {
                "name": "",
                "strategy": "meta"
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/pulsar%20pipeline/2/ref_resolved",
            "cluster_resources": {
                "where": {
                    "cluster": "CEP4",
                    "partition": "cpu"
                },
                "cores_per_task": 2,
                "parallel_tasks": 122
            },
            "single_pulse_search": false
        },
        "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "pulsar pipeline", "state": "active"})).id,
        "specifications_template_id": (_.find(templates, {"name": "pulsar pipeline", "state": "active"})).id,
        "tags": [],
        "task_blueprints": [
            "http://localhost:3000/api/task_blueprint/50"
        ],
        "task_blueprints_ids": [
            50
        ],
        "task_type": "pipeline",
        "updated_at": "2022-04-21T08:16:30.745924",
        "predecessors": [
            {
                "id": 49,
                "url": "http://localhost:3000/api/task_draft/49",
                "connector_types": [
                    {
                        "id": 6,
                        "url": "http://localhost:3000/api/task_connector_type/6",
                        "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://localhost:3000/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://localhost:3000/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://localhost:3000/api/role/beamformer",
                        "role_value": "beamformer",
                        "task_template": "http://localhost:3000/api/task_template/3",
                        "task_template_id": 3
                    }
                ],
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_draft/66",
                    "http://localhost:3000/api/task_relation_draft/69"
                ],
                "consumed_by_ids": [
                    66,
                    69
                ],
                "created_at": "2022-04-21T08:16:30.727162",
                "description": "Beamforming observation for default pulsar timing with HBA",
                "duration": 120,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Observation",
                "output_pinned": false,
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 0,
                "relative_stop_time": 120,
                "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/9",
                "scheduling_unit_draft_id": 9,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
                    "duration": 120,
                    "beamformer": {
                        "ppf": false,
                        "pipelines": [
                            {
                                "name": "Beamformer CV",
                                "coherent": {
                                    "SAPs": [
                                        {
                                            "name": "SAP0",
                                            "tabs": [
                                                {
                                                    "duration": 0,
                                                    "pointing": {
                                                        "angle1": 0,
                                                        "angle2": 0,
                                                        "target": "target1",
                                                        "direction_type": "J2000"
                                                    },
                                                    "relative": true,
                                                    "allow_after_previous": true
                                                }
                                            ],
                                            "subbands": {
                                                "list": [
                                                    51,
                                                    52,
                                                    53,
                                                    54,
                                                    55,
                                                    56,
                                                    57,
                                                    58,
                                                    59,
                                                    60,
                                                    61,
                                                    62,
                                                    63,
                                                    64,
                                                    65,
                                                    66,
                                                    67,
                                                    68,
                                                    69,
                                                    70,
                                                    71,
                                                    72,
                                                    73,
                                                    74,
                                                    75,
                                                    76,
                                                    77,
                                                    78,
                                                    79,
                                                    80,
                                                    81,
                                                    82,
                                                    83,
                                                    84,
                                                    85,
                                                    86,
                                                    87,
                                                    88,
                                                    89,
                                                    90,
                                                    91,
                                                    92,
                                                    93,
                                                    94,
                                                    95,
                                                    96,
                                                    97,
                                                    98,
                                                    99,
                                                    100,
                                                    101,
                                                    102,
                                                    103,
                                                    104,
                                                    105,
                                                    106,
                                                    107,
                                                    108,
                                                    109,
                                                    110,
                                                    111,
                                                    112,
                                                    113,
                                                    114,
                                                    115,
                                                    116,
                                                    117,
                                                    118,
                                                    119,
                                                    120,
                                                    121,
                                                    122,
                                                    123,
                                                    124,
                                                    125,
                                                    126,
                                                    127,
                                                    128,
                                                    129,
                                                    130,
                                                    131,
                                                    132,
                                                    133,
                                                    134,
                                                    135,
                                                    136,
                                                    137,
                                                    138,
                                                    139,
                                                    140,
                                                    141,
                                                    142,
                                                    143,
                                                    144,
                                                    145,
                                                    146,
                                                    147,
                                                    148,
                                                    149,
                                                    150,
                                                    151,
                                                    152,
                                                    153,
                                                    154,
                                                    155,
                                                    156,
                                                    157,
                                                    158,
                                                    159,
                                                    160,
                                                    161,
                                                    162,
                                                    163,
                                                    164,
                                                    165,
                                                    166,
                                                    167,
                                                    168,
                                                    169,
                                                    170,
                                                    171,
                                                    172,
                                                    173,
                                                    174,
                                                    175,
                                                    176,
                                                    177,
                                                    178,
                                                    179,
                                                    180,
                                                    181,
                                                    182,
                                                    183,
                                                    184,
                                                    185,
                                                    186,
                                                    187,
                                                    188,
                                                    189,
                                                    190,
                                                    191,
                                                    192,
                                                    193,
                                                    194,
                                                    195,
                                                    196,
                                                    197,
                                                    198,
                                                    199,
                                                    200,
                                                    201,
                                                    202,
                                                    203,
                                                    204,
                                                    205,
                                                    206,
                                                    207,
                                                    208,
                                                    209,
                                                    210,
                                                    211,
                                                    212,
                                                    213,
                                                    214,
                                                    215,
                                                    216,
                                                    217,
                                                    218,
                                                    219,
                                                    220,
                                                    221,
                                                    222,
                                                    223,
                                                    224,
                                                    225,
                                                    226,
                                                    227,
                                                    228,
                                                    229,
                                                    230,
                                                    231,
                                                    232,
                                                    233,
                                                    234,
                                                    235,
                                                    236,
                                                    237,
                                                    238,
                                                    239,
                                                    240,
                                                    241,
                                                    242,
                                                    243,
                                                    244,
                                                    245,
                                                    246,
                                                    247,
                                                    248,
                                                    249,
                                                    250,
                                                    251,
                                                    252,
                                                    253,
                                                    254,
                                                    255,
                                                    256,
                                                    257,
                                                    258,
                                                    259,
                                                    260,
                                                    261,
                                                    262,
                                                    263,
                                                    264,
                                                    265,
                                                    266,
                                                    267,
                                                    268,
                                                    269,
                                                    270,
                                                    271,
                                                    272,
                                                    273,
                                                    274,
                                                    275,
                                                    276,
                                                    277,
                                                    278,
                                                    279,
                                                    280,
                                                    281,
                                                    282,
                                                    283,
                                                    284,
                                                    285,
                                                    286,
                                                    287,
                                                    288,
                                                    289,
                                                    290,
                                                    291,
                                                    292,
                                                    293,
                                                    294,
                                                    295,
                                                    296,
                                                    297,
                                                    298,
                                                    299,
                                                    300,
                                                    301,
                                                    302,
                                                    303,
                                                    304,
                                                    305,
                                                    306,
                                                    307,
                                                    308,
                                                    309,
                                                    310,
                                                    311,
                                                    312,
                                                    313,
                                                    314,
                                                    315,
                                                    316,
                                                    317,
                                                    318,
                                                    319,
                                                    320,
                                                    321,
                                                    322,
                                                    323,
                                                    324,
                                                    325,
                                                    326,
                                                    327,
                                                    328,
                                                    329,
                                                    330,
                                                    331,
                                                    332,
                                                    333,
                                                    334,
                                                    335,
                                                    336,
                                                    337,
                                                    338,
                                                    339,
                                                    340,
                                                    341,
                                                    342,
                                                    343,
                                                    344,
                                                    345,
                                                    346,
                                                    347,
                                                    348,
                                                    349,
                                                    350,
                                                    351,
                                                    352,
                                                    353,
                                                    354,
                                                    355,
                                                    356,
                                                    357,
                                                    358,
                                                    359,
                                                    360,
                                                    361,
                                                    362,
                                                    363,
                                                    364,
                                                    365,
                                                    366,
                                                    367,
                                                    368,
                                                    369,
                                                    370,
                                                    371,
                                                    372,
                                                    373,
                                                    374,
                                                    375,
                                                    376,
                                                    377,
                                                    378,
                                                    379,
                                                    380,
                                                    381,
                                                    382,
                                                    383,
                                                    384,
                                                    385,
                                                    386,
                                                    387,
                                                    388,
                                                    389,
                                                    390,
                                                    391,
                                                    392,
                                                    393,
                                                    394,
                                                    395,
                                                    396,
                                                    397,
                                                    398,
                                                    399,
                                                    400,
                                                    401,
                                                    402,
                                                    403,
                                                    404,
                                                    405,
                                                    406,
                                                    407,
                                                    408,
                                                    409,
                                                    410,
                                                    411,
                                                    412,
                                                    413,
                                                    414,
                                                    415,
                                                    416,
                                                    417,
                                                    418,
                                                    419,
                                                    420,
                                                    421,
                                                    422,
                                                    423,
                                                    424,
                                                    425,
                                                    426,
                                                    427,
                                                    428,
                                                    429,
                                                    430,
                                                    431,
                                                    432,
                                                    433,
                                                    434,
                                                    435,
                                                    436,
                                                    437,
                                                    438,
                                                    439,
                                                    440,
                                                    441,
                                                    442,
                                                    443,
                                                    444,
                                                    445,
                                                    446,
                                                    447,
                                                    448,
                                                    449,
                                                    450
                                                ],
                                                "method": "copy"
                                            },
                                            "tab_rings": {
                                                "count": 0,
                                                "width": 0.01
                                            }
                                        }
                                    ],
                                    "settings": {
                                        "stokes": "XXYY",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 20,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "flys eye": {
                                    "enabled": false,
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "incoherent": {
                                    "SAPs": [],
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "station_groups": [
                                    {
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501"
                                        ],
                                        "max_nr_missing": 1
                                    }
                                ]
                            }
                        ]
                    },
                    "station_configuration": {
                        "SAPs": [
                            {
                                "name": "SAP0",
                                "subbands": [
                                    51,
                                    52,
                                    53,
                                    54,
                                    55,
                                    56,
                                    57,
                                    58,
                                    59,
                                    60,
                                    61,
                                    62,
                                    63,
                                    64,
                                    65,
                                    66,
                                    67,
                                    68,
                                    69,
                                    70,
                                    71,
                                    72,
                                    73,
                                    74,
                                    75,
                                    76,
                                    77,
                                    78,
                                    79,
                                    80,
                                    81,
                                    82,
                                    83,
                                    84,
                                    85,
                                    86,
                                    87,
                                    88,
                                    89,
                                    90,
                                    91,
                                    92,
                                    93,
                                    94,
                                    95,
                                    96,
                                    97,
                                    98,
                                    99,
                                    100,
                                    101,
                                    102,
                                    103,
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    137,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    164,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    181,
                                    182,
                                    183,
                                    184,
                                    185,
                                    186,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    210,
                                    211,
                                    212,
                                    213,
                                    214,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    241,
                                    242,
                                    243,
                                    244,
                                    245,
                                    246,
                                    247,
                                    248,
                                    249,
                                    250,
                                    251,
                                    252,
                                    253,
                                    254,
                                    255,
                                    256,
                                    257,
                                    258,
                                    259,
                                    260,
                                    261,
                                    262,
                                    263,
                                    264,
                                    265,
                                    266,
                                    267,
                                    268,
                                    269,
                                    270,
                                    271,
                                    272,
                                    273,
                                    274,
                                    275,
                                    276,
                                    277,
                                    278,
                                    279,
                                    280,
                                    281,
                                    282,
                                    283,
                                    284,
                                    285,
                                    286,
                                    287,
                                    288,
                                    289,
                                    290,
                                    291,
                                    292,
                                    293,
                                    294,
                                    295,
                                    296,
                                    297,
                                    298,
                                    299,
                                    300,
                                    301,
                                    302,
                                    303,
                                    304,
                                    305,
                                    306,
                                    307,
                                    308,
                                    309,
                                    310,
                                    311,
                                    312,
                                    313,
                                    314,
                                    315,
                                    316,
                                    317,
                                    318,
                                    319,
                                    320,
                                    321,
                                    322,
                                    323,
                                    324,
                                    325,
                                    326,
                                    327,
                                    328,
                                    329,
                                    330,
                                    331,
                                    332,
                                    333,
                                    334,
                                    335,
                                    336,
                                    337,
                                    338,
                                    339,
                                    340,
                                    341,
                                    342,
                                    343,
                                    344,
                                    345,
                                    346,
                                    347,
                                    348,
                                    349,
                                    350,
                                    351,
                                    352,
                                    353,
                                    354,
                                    355,
                                    356,
                                    357,
                                    358,
                                    359,
                                    360,
                                    361,
                                    362,
                                    363,
                                    364,
                                    365,
                                    366,
                                    367,
                                    368,
                                    369,
                                    370,
                                    371,
                                    372,
                                    373,
                                    374,
                                    375,
                                    376,
                                    377,
                                    378,
                                    379,
                                    380,
                                    381,
                                    382,
                                    383,
                                    384,
                                    385,
                                    386,
                                    387,
                                    388,
                                    389,
                                    390,
                                    391,
                                    392,
                                    393,
                                    394,
                                    395,
                                    396,
                                    397,
                                    398,
                                    399,
                                    400,
                                    401,
                                    402,
                                    403,
                                    404,
                                    405,
                                    406,
                                    407,
                                    408,
                                    409,
                                    410,
                                    411,
                                    412,
                                    413,
                                    414,
                                    415,
                                    416,
                                    417,
                                    418,
                                    419,
                                    420,
                                    421,
                                    422,
                                    423,
                                    424,
                                    425,
                                    426,
                                    427,
                                    428,
                                    429,
                                    430,
                                    431,
                                    432,
                                    433,
                                    434,
                                    435,
                                    436,
                                    437,
                                    438,
                                    439,
                                    440,
                                    441,
                                    442,
                                    443,
                                    444,
                                    445,
                                    446,
                                    447,
                                    448,
                                    449,
                                    450
                                ],
                                "digital_pointing": {
                                    "angle1": 0.92934186635,
                                    "angle2": 0.952579228492,
                                    "target": "target1",
                                    "direction_type": "J2000"
                                }
                            }
                        ],
                        "filter": "HBA_110_190",
                        "tile_beam": {
                            "angle1": 0.92934186635,
                            "angle2": 0.952579228492,
                            "target": "target1",
                            "direction_type": "J2000"
                        },
                        "antenna_set": "HBA_DUAL_INNER",
                        "station_groups": [
                            {
                                "stations": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                },
                "specifications_template": {
                    "id": 3,
                    "url": "http://localhost:3000/api/task_template/3",
                    "created_at": "2022-04-21T08:15:52.853823",
                    "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                    "name": "beamforming observation",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
                        "type": "object",
                        "title": "beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "beamformer",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "#/definitions/timedelta",
                                "title": "Duration",
                                "default": 300,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "#/definitions/beamformer",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "timedelta": {
                                "default": 0,
                                "description": "A time duration or delta expressed in seconds",
                                "type": "number"
                            },
                            "timestamp": {
                                "default": "1970-01-01T00:00:00Z",
                                "description": "A timestamp defined in UTC",
                                "format": "date-time",
                                "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                                "type": "string"
                            },
                            "beamformer": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "pipelines": {
                                        "additionalItems": false,
                                        "default": [
                                            {}
                                        ],
                                        "description": "This array of beamformer pipelines is currently restricted to exactly 1 item. Support for handling multiple pipelines (in cobalt) will be added in the future.",
                                        "items": {
                                            "$ref": "#/definitions/beamformer_pipeline"
                                        },
                                        "maxItems": 1,
                                        "minItems": 1,
                                        "title": "Pipelines",
                                        "type": "array"
                                    },
                                    "ppf": {
                                        "default": false,
                                        "description": "Use a PPF for the channellisation for beamforming. If not, an FFT is used.",
                                        "title": "Use PPF when beamforming",
                                        "type": "boolean"
                                    }
                                },
                                "required": [
                                    "pipelines",
                                    "ppf"
                                ],
                                "title": "Beamformer",
                                "type": "object"
                            },
                            "beamformer_pipeline": {
                                "additionalProperties": false,
                                "default": {},
                                "headerTemplate": "Beamformer Pipeline {{ self.index }}",
                                "properties": {
                                    "coherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to beamform.",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        },
                                                        "tab_rings": {
                                                            "additonalProperties": false,
                                                            "default": {},
                                                            "description": "Rings of TABs around the center of the beam.",
                                                            "properties": {
                                                                "count": {
                                                                    "default": 0,
                                                                    "maximum": 11,
                                                                    "minimum": 0,
                                                                    "title": "Number of rings",
                                                                    "type": "integer"
                                                                },
                                                                "width": {
                                                                    "default": 0.01,
                                                                    "description": "Distance between pointings.",
                                                                    "minimum": 0,
                                                                    "title": "Ring width",
                                                                    "type": "number"
                                                                }
                                                            },
                                                            "title": "Tied-Array Rings",
                                                            "type": "object"
                                                        },
                                                        "tabs": {
                                                            "additionalItems": false,
                                                            "default": [],
                                                            "description": "Tied-array beams to form",
                                                            "items": {
                                                                "additionalProperties": false,
                                                                "default": {},
                                                                "headerTemplate": "TAB {{ self.index }}",
                                                                "properties": {
                                                                    "allow_after_previous": {
                                                                        "default": true,
                                                                        "description": "This TAB can be recorded after the previous TAB, if duration allows.",
                                                                        "title": "Allow after previous",
                                                                        "type": "boolean"
                                                                    },
                                                                    "duration": {
                                                                        "default": 0,
                                                                        "description": "Exposure time of the TAB. 0 = observation duration.",
                                                                        "maximum": 0,
                                                                        "minimum": 0,
                                                                        "title": "Duration",
                                                                        "type": "integer"
                                                                    },
                                                                    "pointing": {
                                                                        "$ref": "#/definitions/pointing",
                                                                        "default": {}
                                                                    },
                                                                    "relative": {
                                                                        "default": false,
                                                                        "description": "Angular offset applied to the TAB pointing",
                                                                        "title": "Relative to SAP",
                                                                        "type": "boolean"
                                                                    }
                                                                },
                                                                "required": [
                                                                    "pointing",
                                                                    "relative"
                                                                ],
                                                                "title": "Tied-Array Beam",
                                                                "type": "object"
                                                            },
                                                            "title": "Tied-Array Beams",
                                                            "type": "array"
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "tabs"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "SAPs",
                                            "settings"
                                        ],
                                        "title": "Coherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "flys eye": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "description": "Produce beams containing the individual station signals",
                                        "properties": {
                                            "enabled": {
                                                "default": false,
                                                "title": "Enable Fly's Eye",
                                                "type": "boolean"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "enabled"
                                        ],
                                        "title": "Fly's Eye Settings",
                                        "type": "object"
                                    },
                                    "incoherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to create incoherent TABs for (empty list = all).",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "default": {},
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "subbands"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "settings",
                                            "SAPs"
                                        ],
                                        "title": "Incoherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "name": {
                                        "default": "_beamformer_name_",
                                        "description": "Beamformer name, used for identification purposes.",
                                        "minLength": 1,
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501"
                                                ]
                                            }
                                        ],
                                        "description": "While observing, COBALT will beamform on the intersection of all stations in this list and the used stations in the observation. So, specifying all possible stations here means that all observation-stations are used. Specifying a small subset here means that only the observing-stations in this small list are used. By default we let COBALT beamform on the Core stations.",
                                        "minItems": 1
                                    }
                                },
                                "required": [
                                    "name"
                                ],
                                "title": "Beamformer Pipeline",
                                "type": "object"
                            },
                            "subband_selection": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "list": {
                                        "additionalItems": false,
                                        "default": [],
                                        "description": "If method is 'select subset', only beamform these subbands, and only if they occur in the SAP.",
                                        "items": {
                                            "maximum": 511,
                                            "minimum": 0,
                                            "title": "Subband",
                                            "type": "integer"
                                        },
                                        "minItems": 0,
                                        "title": "Subset selection",
                                        "type": "array"
                                    },
                                    "method": {
                                        "default": "copy",
                                        "description": "How to select the subbands to beam form",
                                        "enum": [
                                            "copy",
                                            "largest continuous subset",
                                            "select subset"
                                        ],
                                        "title": "Method",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "method"
                                ],
                                "title": "Subband selection",
                                "type": "object"
                            },
                            "station_list": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [],
                                "items": {
                                    "$ref": "#/definitions/station"
                                },
                                "minItems": 0,
                                "type": "array",
                                "uniqueItems": true
                            },
                            "pointing": {
                                "additionalProperties": false,
                                "properties": {
                                    "angle1": {
                                        "default": 0.6624317181687094,
                                        "description": "First angle (e.g. RA)",
                                        "title": "Angle 1",
                                        "type": "number"
                                    },
                                    "angle2": {
                                        "default": 1.5579526427549426,
                                        "description": "Second angle (e.g. DEC)",
                                        "title": "Angle 2",
                                        "type": "number"
                                    },
                                    "direction_type": {
                                        "default": "J2000",
                                        "description": "",
                                        "enum": [
                                            "J2000",
                                            "AZELGEO",
                                            "LMN",
                                            "SUN",
                                            "MOON",
                                            "MERCURY",
                                            "VENUS",
                                            "MARS",
                                            "JUPITER",
                                            "SATURN",
                                            "URANUS",
                                            "NEPTUNE",
                                            "PLUTO"
                                        ],
                                        "title": "Reference frame",
                                        "type": "string"
                                    },
                                    "target": {
                                        "default": "_target_name_",
                                        "description": "Description of where this beam points at",
                                        "minLength": 1,
                                        "title": "Target",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "angle1",
                                    "angle2",
                                    "target"
                                ],
                                "type": "object"
                            },
                            "stokes_settings": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "channels_per_subband": {
                                        "default": 1,
                                        "description": "Number of frequency bands per subband",
                                        "enum": [
                                            1,
                                            8,
                                            16,
                                            32,
                                            64,
                                            128,
                                            256,
                                            512,
                                            1024
                                        ],
                                        "minimum": 1,
                                        "title": "Channels/subband",
                                        "type": "integer"
                                    },
                                    "quantisation": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "bits": {
                                                "default": 8,
                                                "description": "Number of bits for a single value",
                                                "enum": [
                                                    8
                                                ],
                                                "title": "Bits/sample",
                                                "type": "integer"
                                            },
                                            "enabled": {
                                                "default": false,
                                                "title": "Output quantisation into integers",
                                                "type": "boolean"
                                            },
                                            "scale_max": {
                                                "default": 5,
                                                "description": "Cut off values above this treshold",
                                                "title": "Maximum value",
                                                "type": "number"
                                            },
                                            "scale_min": {
                                                "default": -5,
                                                "description": "Cut off values below this treshold",
                                                "title": "Minimum value",
                                                "type": "number"
                                            }
                                        },
                                        "required": [
                                            "enabled",
                                            "bits",
                                            "scale_min",
                                            "scale_max"
                                        ],
                                        "title": "Output quantisation settings",
                                        "type": "object"
                                    },
                                    "stokes": {
                                        "$ref": "#/definitions/stokes",
                                        "default": "I"
                                    },
                                    "subbands_per_file": {
                                        "default": 488,
                                        "description": "The maximum number of subbands to write in each output dataproduct.",
                                        "maximum": 488,
                                        "minimum": 1,
                                        "title": "Subbands per file",
                                        "type": "integer"
                                    },
                                    "time_integration_factor": {
                                        "default": 1,
                                        "description": "The number of samples to integrate over",
                                        "maximum": 12288,
                                        "minimum": 1,
                                        "title": "Time integration",
                                        "type": "integer"
                                    }
                                },
                                "required": [
                                    "stokes",
                                    "time_integration_factor",
                                    "channels_per_subband"
                                ],
                                "type": "object"
                            },
                            "station": {
                                "description": "These are the LOFAR stations",
                                "enum": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501",
                                    "RS106",
                                    "RS205",
                                    "RS208",
                                    "RS210",
                                    "RS305",
                                    "RS306",
                                    "RS307",
                                    "RS310",
                                    "RS406",
                                    "RS407",
                                    "RS409",
                                    "RS503",
                                    "RS508",
                                    "RS509",
                                    "DE601",
                                    "DE602",
                                    "DE603",
                                    "DE604",
                                    "DE605",
                                    "FR606",
                                    "SE607",
                                    "UK608",
                                    "DE609",
                                    "PL610",
                                    "PL611",
                                    "PL612",
                                    "IE613",
                                    "LV614"
                                ],
                                "title": "Station",
                                "type": "string"
                            },
                            "max_number_of_missing_stations": {
                                "default": 0,
                                "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                                "minimum": 0,
                                "title": "Maximum number of stations to omit",
                                "type": "integer"
                            },
                            "stokes": {
                                "default": "I",
                                "description": "Which Stokes to produce",
                                "enum": [
                                    "I",
                                    "IQUV",
                                    "XXYY"
                                ],
                                "title": "Stokes",
                                "type": "string"
                            },
                            "station_groups": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [
                                    {
                                        "max_nr_missing": 1,
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ]
                                    }
                                ],
                                "description": "One or more predefined or custom groups of stations",
                                "items": {
                                    "$ref": "#/definitions/station_group"
                                },
                                "minItems": 1,
                                "title": "Station groups",
                                "type": "array"
                            },
                            "station_group": {
                                "anyOf": [
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007"
                                            ]
                                        },
                                        "description": "The group of all stations on the Superterp",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Superterp",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501"
                                            ]
                                        },
                                        "description": "The group of all Core stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Core",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch remote stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Remote",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch (Core + Remote) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Dutch",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 2,
                                            "stations": [
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 1,
                                            "stations": [
                                                "DE601",
                                                "DE605"
                                            ]
                                        },
                                        "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE605"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International required",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 6,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509",
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all (Core + Remote + International) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509",
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "All",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS001"
                                            ]
                                        },
                                        "description": "A custom group of stations which can be defined by the user",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list"
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Custom",
                                        "type": "object"
                                    }
                                ],
                                "default": {
                                    "max_nr_missing": 1,
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                },
                                "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                                "title": "Station group",
                                "type": "object"
                            },
                            "station_configuration": {
                                "default": {},
                                "properties": {
                                    "SAPs": {
                                        "$ref": "#/definitions/SAPs",
                                        "default": [
                                            {}
                                        ],
                                        "minItems": 1
                                    },
                                    "antenna_set": {
                                        "$ref": "#/definitions/antenna_set",
                                        "default": "HBA_DUAL"
                                    },
                                    "filter": {
                                        "$ref": "#/definitions/filter",
                                        "default": "HBA_110_190"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007"
                                                ]
                                            }
                                        ]
                                    },
                                    "tile_beam": {
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "description": "HBA only",
                                        "title": "Tile beam"
                                    }
                                },
                                "required": [
                                    "station_groups",
                                    "antenna_set",
                                    "filter",
                                    "SAPs"
                                ],
                                "title": "station_configuration",
                                "type": "object"
                            },
                            "filter": {
                                "default": "HBA_110_190",
                                "description": "Must match antenna type",
                                "enum": [
                                    "LBA_10_70",
                                    "LBA_30_70",
                                    "LBA_10_90",
                                    "LBA_30_90",
                                    "HBA_110_190",
                                    "HBA_210_250"
                                ],
                                "title": "Band-pass filter",
                                "type": "string"
                            },
                            "antenna_set": {
                                "default": "HBA_DUAL",
                                "description": "Fields & antennas to use",
                                "enum": [
                                    "HBA_DUAL",
                                    "HBA_DUAL_INNER",
                                    "HBA_ONE",
                                    "HBA_ONE_INNER",
                                    "HBA_ZERO",
                                    "HBA_ZERO_INNER",
                                    "LBA_INNER",
                                    "LBA_OUTER",
                                    "LBA_SPARSE_EVEN",
                                    "LBA_SPARSE_ODD",
                                    "LBA_ALL"
                                ],
                                "title": "Antenna set",
                                "type": "string"
                            },
                            "SAPs": {
                                "additionalItems": false,
                                "default": [
                                    {}
                                ],
                                "description": "Station beams",
                                "items": {
                                    "additionalProperties": false,
                                    "default": {},
                                    "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                    "properties": {
                                        "digital_pointing": {
                                            "$id": "#target_pointing",
                                            "$ref": "#/definitions/pointing",
                                            "default": {},
                                            "title": "Digital pointing"
                                        },
                                        "name": {
                                            "default": "_SAP_name_",
                                            "description": "Identifier for this beam",
                                            "minLength": 1,
                                            "title": "Name",
                                            "type": "string"
                                        },
                                        "subbands": {
                                            "additionalItems": false,
                                            "default": [
                                                255
                                            ],
                                            "items": {
                                                "maximum": 511,
                                                "minimum": 0,
                                                "title": "Subband",
                                                "type": "integer"
                                            },
                                            "maxItems": 488,
                                            "minItems": 1,
                                            "title": "Subband list",
                                            "type": "array"
                                        }
                                    },
                                    "required": [
                                        "name",
                                        "digital_pointing",
                                        "subbands"
                                    ],
                                    "title": "SAP",
                                    "type": "object"
                                },
                                "minItems": 0,
                                "title": "SAPs",
                                "type": "array"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2#",
                        "type": "object",
                        "title": "beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "beamformer",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                "title": "Duration",
                                "default": 300,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/2#/definitions/beamformer",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/observation",
                    "type_value": "observation",
                    "updated_at": "2022-04-21T08:15:52.853846",
                    "version": 2
                },
                "specifications_template_id": 3,
                "tags": [],
                "task_blueprints": [
                    "http://localhost:3000/api/task_blueprint/49"
                ],
                "task_blueprints_ids": [
                    49
                ],
                "task_type": "observation",
                "updated_at": "2022-04-21T08:16:30.727178"
            }
        ],
        "successors": [
            {
                "id": 52,
                "url": "http://localhost:3000/api/task_draft/52",
                "connector_types": [
                    {
                        "id": 26,
                        "url": "http://localhost:3000/api/task_connector_type/26",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/correlator",
                        "role_value": "correlator",
                        "task_template": "http://localhost:3000/api/task_template/2",
                        "task_template_id": 2
                    },
                    {
                        "id": 21,
                        "url": "http://localhost:3000/api/task_connector_type/21",
                        "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://localhost:3000/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/2",
                        "task_template_id": 2
                    },
                    {
                        "id": 20,
                        "url": "http://localhost:3000/api/task_connector_type/20",
                        "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://localhost:3000/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/beamformer",
                        "role_value": "beamformer",
                        "task_template": "http://localhost:3000/api/task_template/2",
                        "task_template_id": 2
                    },
                    {
                        "id": 18,
                        "url": "http://localhost:3000/api/task_connector_type/18",
                        "dataformat": "http://localhost:3000/api/dataformat/pulp%20summary",
                        "dataformat_value": "pulp summary",
                        "datatype": "http://localhost:3000/api/datatype/quality",
                        "datatype_value": "quality",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/2",
                        "task_template_id": 2
                    },
                    {
                        "id": 16,
                        "url": "http://localhost:3000/api/task_connector_type/16",
                        "dataformat": "http://localhost:3000/api/dataformat/pulp%20analysis",
                        "dataformat_value": "pulp analysis",
                        "datatype": "http://localhost:3000/api/datatype/pulsar%20profile",
                        "datatype_value": "pulsar profile",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/2",
                        "task_template_id": 2
                    },
                    {
                        "id": 14,
                        "url": "http://localhost:3000/api/task_connector_type/14",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/2",
                        "task_template_id": 2
                    }
                ],
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-04-21T08:16:30.766446",
                "description": "Ingest the pipeline outputs dataproducts",
                "duration": 0,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Ingest",
                "output_pinned": false,
                "produced_by": [
                    "http://localhost:3000/api/task_relation_draft/67",
                    "http://localhost:3000/api/task_relation_draft/68"
                ],
                "produced_by_ids": [
                    67,
                    68
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/9",
                "scheduling_unit_draft_id": 9,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
                },
                "specifications_template": "http://localhost:3000/api/task_template/2",
                "specifications_template_id": 2,
                "tags": [],
                "task_blueprints": [
                    "http://localhost:3000/api/task_blueprint/52"
                ],
                "task_blueprints_ids": [
                    52
                ],
                "task_type": "ingest",
                "updated_at": "2022-04-21T08:16:30.766460"
            },
            {
                "id": 51,
                "url": "http://localhost:3000/api/task_draft/51",
                "connector_types": [
                    {
                        "id": 48,
                        "url": "http://localhost:3000/api/task_connector_type/48",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/correlator",
                        "role_value": "correlator",
                        "task_template": "http://localhost:3000/api/task_template/1",
                        "task_template_id": 1
                    },
                    {
                        "id": 43,
                        "url": "http://localhost:3000/api/task_connector_type/43",
                        "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://localhost:3000/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/1",
                        "task_template_id": 1
                    },
                    {
                        "id": 42,
                        "url": "http://localhost:3000/api/task_connector_type/42",
                        "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://localhost:3000/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/beamformer",
                        "role_value": "beamformer",
                        "task_template": "http://localhost:3000/api/task_template/1",
                        "task_template_id": 1
                    },
                    {
                        "id": 40,
                        "url": "http://localhost:3000/api/task_connector_type/40",
                        "dataformat": "http://localhost:3000/api/dataformat/pulp%20summary",
                        "dataformat_value": "pulp summary",
                        "datatype": "http://localhost:3000/api/datatype/quality",
                        "datatype_value": "quality",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/1",
                        "task_template_id": 1
                    },
                    {
                        "id": 38,
                        "url": "http://localhost:3000/api/task_connector_type/38",
                        "dataformat": "http://localhost:3000/api/dataformat/pulp%20analysis",
                        "dataformat_value": "pulp analysis",
                        "datatype": "http://localhost:3000/api/datatype/pulsar%20profile",
                        "datatype_value": "pulsar profile",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/1",
                        "task_template_id": 1
                    },
                    {
                        "id": 36,
                        "url": "http://localhost:3000/api/task_connector_type/36",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/1",
                        "task_template_id": 1
                    }
                ],
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-04-21T08:16:30.756024",
                "description": "Cleanup all dataproducts from disk",
                "duration": 0,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Cleanup",
                "output_pinned": false,
                "produced_by": [
                    "http://localhost:3000/api/task_relation_draft/69",
                    "http://localhost:3000/api/task_relation_draft/70",
                    "http://localhost:3000/api/task_relation_draft/71"
                ],
                "produced_by_ids": [
                    69,
                    70,
                    71
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/9",
                "scheduling_unit_draft_id": 9,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2/ref_resolved"
                },
                "specifications_template": "http://localhost:3000/api/task_template/1",
                "specifications_template_id": 1,
                "tags": [],
                "task_blueprints": [
                    "http://localhost:3000/api/task_blueprint/51"
                ],
                "task_blueprints_ids": [
                    51
                ],
                "task_type": "cleanup",
                "updated_at": "2022-04-21T08:16:30.756038"
            }
        ]
    },
    {
        "id": 53,
        "url": "http://localhost:3000/api/task_draft/53",
        "connector_types": [
            {
                "id": 6,
                "url": "http://localhost:3000/api/task_connector_type/6",
                "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                "dataformat_value": "Beamformed",
                "datatype": "http://localhost:3000/api/datatype/time%20series",
                "datatype_value": "time series",
                "iotype": "http://localhost:3000/api/iotype/output",
                "iotype_value": "output",
                "role": "http://localhost:3000/api/role/beamformer",
                "role_value": "beamformer",
                "task_template": "http://localhost:3000/api/task_template/3",
                "task_template_id": 3
            }
        ],
        "consumed_by": [],
        "consumed_by_ids": [],
        "created_at": "2022-04-21T08:16:32.621923",
        "description": "A simple short test beamforming observation",
        "duration": 120,
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Observation",
        "output_pinned": false,
        "produced_by": [],
        "produced_by_ids": [],
        "relative_start_time": 0,
        "relative_stop_time": 120,
        "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/10",
        "scheduling_unit_draft_id": 10,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "",
        "specifications_doc": {
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
            "duration": 120,
            "beamformer": {
                "ppf": false,
                "pipelines": [
                    {
                        "name": "B0329+54",
                        "coherent": {
                            "SAPs": [
                                {
                                    "name": "B0329+54",
                                    "tabs": [
                                        {
                                            "duration": 0,
                                            "pointing": {
                                                "angle1": 0.8115781021773631,
                                                "angle2": 0.952579228492,
                                                "target": "target1",
                                                "direction_type": "J2000"
                                            },
                                            "relative": false,
                                            "allow_after_previous": true
                                        }
                                    ],
                                    "subbands": {
                                        "list": [
                                            0,
                                            1,
                                            2,
                                            3,
                                            4,
                                            5,
                                            6,
                                            7,
                                            8,
                                            9,
                                            10,
                                            11,
                                            12,
                                            13,
                                            14,
                                            15,
                                            16,
                                            17,
                                            18,
                                            19,
                                            20,
                                            21,
                                            22,
                                            23,
                                            24,
                                            25,
                                            26,
                                            27,
                                            28,
                                            29,
                                            30,
                                            31,
                                            32,
                                            33,
                                            34,
                                            35,
                                            36,
                                            37,
                                            38,
                                            39,
                                            40,
                                            41,
                                            42,
                                            43,
                                            44,
                                            45,
                                            46,
                                            47,
                                            48,
                                            49,
                                            50,
                                            51,
                                            52,
                                            53,
                                            54,
                                            55,
                                            56,
                                            57,
                                            58,
                                            59,
                                            60,
                                            61,
                                            62,
                                            63,
                                            64,
                                            65,
                                            66,
                                            67,
                                            68,
                                            69,
                                            70,
                                            71,
                                            72,
                                            73,
                                            74,
                                            75,
                                            76,
                                            77,
                                            78,
                                            79,
                                            80,
                                            81,
                                            82,
                                            83,
                                            84,
                                            85,
                                            86,
                                            87,
                                            88,
                                            89,
                                            90,
                                            91,
                                            92,
                                            93,
                                            94,
                                            95,
                                            96,
                                            97,
                                            98,
                                            99,
                                            100,
                                            101,
                                            102,
                                            103,
                                            104,
                                            105,
                                            106,
                                            107,
                                            108,
                                            109,
                                            110,
                                            111,
                                            112,
                                            113,
                                            114,
                                            115,
                                            116,
                                            117,
                                            118,
                                            119,
                                            120,
                                            121,
                                            122,
                                            123,
                                            124,
                                            125,
                                            126,
                                            127,
                                            128,
                                            129,
                                            130,
                                            131,
                                            132,
                                            133,
                                            134,
                                            135,
                                            136,
                                            137,
                                            138,
                                            139,
                                            140,
                                            141,
                                            142,
                                            143,
                                            144,
                                            145,
                                            146,
                                            147,
                                            148,
                                            149,
                                            150,
                                            151,
                                            152,
                                            153,
                                            154,
                                            155,
                                            156,
                                            157,
                                            158,
                                            159,
                                            160,
                                            161,
                                            162,
                                            163,
                                            164,
                                            165,
                                            166,
                                            167,
                                            168,
                                            169,
                                            170,
                                            171,
                                            172,
                                            173,
                                            174,
                                            175,
                                            176,
                                            177,
                                            178,
                                            179,
                                            180,
                                            181,
                                            182,
                                            183,
                                            184,
                                            185,
                                            186,
                                            187,
                                            188,
                                            189,
                                            190,
                                            191,
                                            192,
                                            193,
                                            194,
                                            195,
                                            196,
                                            197,
                                            198,
                                            199,
                                            200,
                                            201,
                                            202,
                                            203,
                                            204,
                                            205,
                                            206,
                                            207,
                                            208,
                                            209,
                                            210,
                                            211,
                                            212,
                                            213,
                                            214,
                                            215,
                                            216,
                                            217,
                                            218,
                                            219,
                                            220,
                                            221,
                                            222,
                                            223,
                                            224,
                                            225,
                                            226,
                                            227,
                                            228,
                                            229,
                                            230,
                                            231,
                                            232,
                                            233,
                                            234,
                                            235,
                                            236,
                                            237,
                                            238,
                                            239,
                                            240,
                                            241,
                                            242,
                                            243
                                        ],
                                        "method": "copy"
                                    },
                                    "tab_rings": {
                                        "count": 0,
                                        "width": 0.01
                                    }
                                }
                            ],
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "flys eye": {
                            "enabled": false,
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "incoherent": {
                            "SAPs": [],
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "station_groups": [
                            {
                                "stations": [
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                ]
            },
            "station_configuration": {
                "SAPs": [
                    {
                        "name": "B0329+54",
                        "subbands": [
                            0,
                            1,
                            2,
                            3,
                            4,
                            5,
                            6,
                            7,
                            8,
                            9,
                            10,
                            11,
                            12,
                            13,
                            14,
                            15,
                            16,
                            17,
                            18,
                            19,
                            20,
                            21,
                            22,
                            23,
                            24,
                            25,
                            26,
                            27,
                            28,
                            29,
                            30,
                            31,
                            32,
                            33,
                            34,
                            35,
                            36,
                            37,
                            38,
                            39,
                            40,
                            41,
                            42,
                            43,
                            44,
                            45,
                            46,
                            47,
                            48,
                            49,
                            50,
                            51,
                            52,
                            53,
                            54,
                            55,
                            56,
                            57,
                            58,
                            59,
                            60,
                            61,
                            62,
                            63,
                            64,
                            65,
                            66,
                            67,
                            68,
                            69,
                            70,
                            71,
                            72,
                            73,
                            74,
                            75,
                            76,
                            77,
                            78,
                            79,
                            80,
                            81,
                            82,
                            83,
                            84,
                            85,
                            86,
                            87,
                            88,
                            89,
                            90,
                            91,
                            92,
                            93,
                            94,
                            95,
                            96,
                            97,
                            98,
                            99,
                            100,
                            101,
                            102,
                            103,
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            137,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154,
                            155,
                            156,
                            157,
                            158,
                            159,
                            160,
                            161,
                            162,
                            163,
                            164,
                            165,
                            166,
                            167,
                            168,
                            169,
                            170,
                            171,
                            172,
                            173,
                            174,
                            175,
                            176,
                            177,
                            178,
                            179,
                            180,
                            181,
                            182,
                            183,
                            184,
                            185,
                            186,
                            187,
                            188,
                            189,
                            190,
                            191,
                            192,
                            193,
                            194,
                            195,
                            196,
                            197,
                            198,
                            199,
                            200,
                            201,
                            202,
                            203,
                            204,
                            205,
                            206,
                            207,
                            208,
                            209,
                            210,
                            211,
                            212,
                            213,
                            214,
                            215,
                            216,
                            217,
                            218,
                            219,
                            220,
                            221,
                            222,
                            223,
                            224,
                            225,
                            226,
                            227,
                            228,
                            229,
                            230,
                            231,
                            232,
                            233,
                            234,
                            235,
                            236,
                            237,
                            238,
                            239,
                            240,
                            241,
                            242,
                            243
                        ],
                        "digital_pointing": {
                            "angle1": 0.92934186635,
                            "angle2": 0.952579228492,
                            "target": "target1",
                            "direction_type": "J2000"
                        }
                    }
                ],
                "filter": "HBA_110_190",
                "tile_beam": {
                    "angle1": 0.92934186635,
                    "angle2": 0.952579228492,
                    "target": "target1",
                    "direction_type": "J2000"
                },
                "antenna_set": "HBA_DUAL_INNER",
                "station_groups": [
                    {
                        "stations": [
                            "CS002",
                            "CS003",
                            "CS004",
                            "CS005",
                            "CS006",
                            "CS007"
                        ],
                        "max_nr_missing": 1
                    }
                ]
            }
        },
        "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "beamforming observation"})).id,
        "specifications_template_id": (_.find(templates, {"name": "beamforming observation"})).id,
        "tags": [],
        "task_blueprints": [
            "http://localhost:3000/api/task_blueprint/53"
        ],
        "task_blueprints_ids": [
            53
        ],
        "task_type": "observation",
        "updated_at": "2022-04-21T08:16:32.621939",
        "predecessors": [],
        "successors": []
    },
    {
        "id": 68,
        "url": "http://localhost:3000/api/task_draft/68",
        "connector_types": [
            {
                "id": 8,
                "url": "http://localhost:3000/api/task_connector_type/8",
                "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                "dataformat_value": "Beamformed",
                "datatype": "http://localhost:3000/api/datatype/time%20series",
                "datatype_value": "time series",
                "iotype": "http://localhost:3000/api/iotype/output",
                "iotype_value": "output",
                "role": "http://localhost:3000/api/role/beamformer",
                "role_value": "beamformer",
                "task_template": "http://localhost:3000/api/task_template/6",
                "task_template_id": 6
            },
            {
                "id": 5,
                "url": "http://localhost:3000/api/task_connector_type/5",
                "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                "dataformat_value": "MeasurementSet",
                "datatype": "http://localhost:3000/api/datatype/visibilities",
                "datatype_value": "visibilities",
                "iotype": "http://localhost:3000/api/iotype/output",
                "iotype_value": "output",
                "role": "http://localhost:3000/api/role/correlator",
                "role_value": "correlator",
                "task_template": "http://localhost:3000/api/task_template/6",
                "task_template_id": 6
            }
        ],
        "consumed_by": [],
        "consumed_by_ids": [],
        "created_at": "2022-04-22T07:25:37.890328",
        "description": "IM BF Cal&Tgt",
        "duration": 600,
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Observation",
        "output_pinned": false,
        "produced_by": [],
        "produced_by_ids": [],
        "relative_start_time": 0,
        "relative_stop_time": 600,
        "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/15",
        "scheduling_unit_draft_id": 15,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "",
        "specifications_doc": {
            "QA": {
                "plots": {
                    "enabled": true,
                    "autocorrelation": true,
                    "crosscorrelation": true
                },
                "file_conversion": {
                    "enabled": true,
                    "nr_of_subbands": -1,
                    "nr_of_timestamps": 256
                },
                "inspection_plots": "msplots"
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20and%20beamforming%20observation/2/ref_resolved",
            "duration": 600,
            "beamformer": {
                "ppf": false,
                "pipelines": [
                    {
                        "name": "_beamformer_name_",
                        "coherent": {
                            "SAPs": [],
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "flys eye": {
                            "enabled": false,
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "incoherent": {
                            "SAPs": [],
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "station_groups": [
                            {
                                "stations": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                ]
            },
            "calibrator": {
                "name": "calibrator",
                "pointing": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "_target_name_",
                    "direction_type": "J2000"
                },
                "autoselect": true
            },
            "correlator": {
                "storage_cluster": "CEP4",
                "integration_time": 1,
                "channels_per_subband": 64,
                "topocentric_frequency_correction": false
            },
            "station_configuration": {
                "SAPs": [
                    {
                        "name": "_SAP_name_",
                        "subbands": [
                            255
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "_target_name_",
                            "direction_type": "J2000"
                        }
                    }
                ],
                "filter": "HBA_110_190",
                "tile_beam": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "_target_name_",
                    "direction_type": "J2000"
                },
                "antenna_set": "HBA_DUAL",
                "station_groups": [
                    {
                        "stations": [
                            "CS002",
                            "CS003",
                            "CS004",
                            "CS005",
                            "CS006",
                            "CS007"
                        ],
                        "max_nr_missing": 1
                    }
                ]
            }
        },
        "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "parallel calibrator target and beamforming observation"})).id,
        "specifications_template_id": (_.find(templates, {"name": "parallel calibrator target and beamforming observation"})).id,
        "tags": [],
        "task_blueprints": [],
        "task_blueprints_ids": [],
        "task_type": "observation",
        "updated_at": "2022-04-22T07:25:38.319179",
        "predecessors": [],
        "successors": []
    },
    {
        "id": 69,
        "url": "http://localhost:8008/api/task_draft/69",
        "connector_types": [
            {
                "id": 9,
                "url": "http://localhost:8008/api/task_connector_type/9",
                "dataformat": "http://localhost:8008/api/dataformat/Beamformed",
                "dataformat_value": "Beamformed",
                "datatype": "http://localhost:8008/api/datatype/time%20series",
                "datatype_value": "time series",
                "iotype": "http://localhost:8008/api/iotype/output",
                "iotype_value": "output",
                "role": "http://localhost:8008/api/role/beamformer",
                "role_value": "beamformer",
                "task_template": "http://localhost:8008/api/task_template/8",
                "task_template_id": 8
            },
            {
                "id": 5,
                "url": "http://localhost:8008/api/task_connector_type/5",
                "dataformat": "http://localhost:8008/api/dataformat/MeasurementSet",
                "dataformat_value": "MeasurementSet",
                "datatype": "http://localhost:8008/api/datatype/visibilities",
                "datatype_value": "visibilities",
                "iotype": "http://localhost:8008/api/iotype/output",
                "iotype_value": "output",
                "role": "http://localhost:8008/api/role/correlator",
                "role_value": "correlator",
                "task_template": "http://localhost:8008/api/task_template/8",
                "task_template_id": 8
            }
        ],
        "consumed_by": [
            "http://localhost:8008/api/task_relation_draft/3",
            "http://localhost:8008/api/task_relation_draft/4",
            "http://localhost:8008/api/task_relation_draft/9",
            "http://localhost:8008/api/task_relation_draft/10",
            "http://localhost:8008/api/task_relation_draft/15"
        ],
        "consumed_by_ids": [
            3,
            4,
            9,
            10,
            15
        ],
        "created_at": "2022-05-26T05:21:21.738115",
        "description": "IM+BF Observation",
        "duration": 600.0,
        "first_scheduling_relation": [
            "http://localhost:8008/api/task_scheduling_relation_draft/4",
            "http://localhost:8008/api/task_scheduling_relation_draft/5",
            "http://localhost:8008/api/task_scheduling_relation_draft/6"
        ],
        "first_scheduling_relation_ids": [
            4,
            5,
            6
        ],
        "name": "IM+BF",
        "output_pinned": false,
        "produced_by": [],
        "produced_by_ids": [],
        "relative_start_time": 0.0,
        "relative_stop_time": 600.0,
        "scheduling_unit_draft": "http://localhost:8008/api/scheduling_unit_draft/16",
        "scheduling_unit_draft_id": 16,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "IM+BF",
        "specifications_doc": {
            "QA": {
                "plots": {
                    "enabled": false,
                    "autocorrelation": true,
                    "crosscorrelation": true
                },
                "file_conversion": {
                    "enabled": false,
                    "nr_of_subbands": -1,
                    "nr_of_timestamps": 256
                },
                "inspection_plots": "dynspec"
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20and%20beamforming%20observation/2/ref_resolved",
            "duration": 600,
            "beamformer": {
                "ppf": false,
                "pipelines": [
                    {
                        "name": "MultiRings_Beamformer",
                        "coherent": {
                            "SAPs": [
                                {
                                    "name": "IMBF_Sun",
                                    "tabs": [],
                                    "subbands": {
                                        "list": [
                                            96,
                                            97,
                                            98,
                                            99,
                                            100,
                                            101,
                                            102,
                                            103,
                                            104,
                                            105,
                                            106,
                                            107,
                                            108,
                                            109,
                                            110,
                                            111,
                                            112,
                                            113,
                                            114,
                                            115,
                                            116,
                                            117,
                                            118,
                                            119,
                                            120,
                                            121,
                                            122,
                                            123,
                                            124,
                                            125,
                                            126,
                                            127,
                                            128,
                                            129,
                                            130,
                                            131,
                                            132,
                                            133,
                                            134,
                                            135,
                                            136,
                                            137,
                                            138,
                                            139,
                                            140,
                                            141,
                                            142,
                                            143,
                                            144,
                                            145,
                                            146,
                                            147,
                                            148,
                                            149,
                                            150,
                                            151,
                                            152,
                                            153,
                                            154,
                                            155,
                                            156,
                                            157,
                                            158,
                                            159,
                                            160,
                                            161,
                                            162,
                                            163,
                                            164,
                                            165,
                                            166,
                                            167,
                                            168,
                                            169,
                                            170,
                                            171,
                                            172,
                                            173,
                                            174,
                                            175,
                                            176,
                                            177,
                                            178,
                                            179,
                                            180,
                                            181,
                                            182,
                                            183,
                                            184,
                                            185,
                                            186,
                                            187,
                                            188,
                                            189,
                                            190,
                                            191,
                                            192,
                                            193,
                                            194,
                                            195,
                                            196,
                                            197,
                                            198,
                                            199,
                                            200,
                                            201,
                                            202,
                                            203,
                                            204,
                                            205,
                                            206,
                                            207,
                                            208,
                                            209,
                                            210,
                                            211,
                                            212,
                                            213,
                                            214,
                                            215,
                                            216,
                                            217,
                                            218,
                                            219,
                                            220,
                                            221,
                                            222,
                                            223,
                                            224,
                                            225,
                                            226,
                                            227,
                                            228,
                                            229,
                                            230,
                                            231,
                                            232,
                                            233,
                                            234,
                                            235,
                                            236,
                                            237,
                                            238,
                                            239,
                                            240,
                                            241,
                                            242,
                                            243,
                                            244,
                                            245,
                                            246,
                                            247,
                                            248,
                                            249,
                                            250,
                                            251,
                                            252,
                                            253,
                                            254,
                                            255,
                                            256,
                                            257,
                                            258,
                                            259,
                                            260,
                                            261,
                                            262,
                                            263,
                                            264,
                                            265,
                                            266,
                                            267,
                                            268,
                                            269,
                                            270,
                                            271,
                                            272,
                                            273,
                                            274,
                                            282,
                                            284,
                                            286,
                                            288,
                                            290,
                                            292,
                                            294,
                                            296,
                                            298,
                                            300,
                                            302,
                                            304,
                                            306,
                                            308,
                                            310,
                                            312,
                                            314,
                                            316,
                                            318,
                                            320,
                                            322,
                                            324,
                                            326,
                                            328,
                                            330,
                                            332,
                                            334,
                                            336,
                                            338,
                                            340,
                                            342,
                                            344,
                                            346,
                                            348,
                                            350,
                                            352,
                                            354,
                                            356,
                                            358,
                                            360,
                                            362,
                                            364,
                                            366,
                                            368,
                                            370,
                                            372,
                                            374,
                                            376,
                                            378,
                                            380,
                                            382,
                                            384,
                                            386,
                                            388,
                                            390,
                                            392,
                                            394,
                                            396,
                                            398,
                                            400,
                                            402,
                                            404,
                                            406,
                                            408,
                                            410
                                        ],
                                        "method": "copy"
                                    },
                                    "tab_rings": {
                                        "count": 6,
                                        "width": 0.00174533
                                    }
                                }
                            ],
                            "settings": {
                                "stokes": "IQUV",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 16,
                                "time_integration_factor": 128
                            }
                        },
                        "flys eye": {
                            "enabled": false,
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "incoherent": {
                            "SAPs": [],
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "station_groups": [
                            {
                                "stations": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                ]
            },
            "calibrator": {
                "name": "calibrator",
                "pointing": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "_target_name_",
                    "direction_type": "J2000"
                },
                "autoselect": false
            },
            "correlator": {
                "storage_cluster": "CEP4",
                "integration_time": 0.25,
                "channels_per_subband": 16,
                "topocentric_frequency_correction": false
            },
            "station_configuration": {
                "SAPs": [
                    {
                        "name": "IMBF_Sun",
                        "subbands": [
                            96,
                            97,
                            98,
                            99,
                            100,
                            101,
                            102,
                            103,
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            137,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154,
                            155,
                            156,
                            157,
                            158,
                            159,
                            160,
                            161,
                            162,
                            163,
                            164,
                            165,
                            166,
                            167,
                            168,
                            169,
                            170,
                            171,
                            172,
                            173,
                            174,
                            175,
                            176,
                            177,
                            178,
                            179,
                            180,
                            181,
                            182,
                            183,
                            184,
                            185,
                            186,
                            187,
                            188,
                            189,
                            190,
                            191,
                            192,
                            193,
                            194,
                            195,
                            196,
                            197,
                            198,
                            199,
                            200,
                            201,
                            202,
                            203,
                            204,
                            205,
                            206,
                            207,
                            208,
                            209,
                            210,
                            211,
                            212,
                            213,
                            214,
                            215,
                            216,
                            217,
                            218,
                            219,
                            220,
                            221,
                            222,
                            223,
                            224,
                            225,
                            226,
                            227,
                            228,
                            229,
                            230,
                            231,
                            232,
                            233,
                            234,
                            235,
                            236,
                            237,
                            238,
                            239,
                            240,
                            241,
                            242,
                            243,
                            244,
                            245,
                            246,
                            247,
                            248,
                            249,
                            250,
                            251,
                            252,
                            253,
                            254,
                            255,
                            256,
                            257,
                            258,
                            259,
                            260,
                            261,
                            262,
                            263,
                            264,
                            265,
                            266,
                            267,
                            268,
                            269,
                            270,
                            271,
                            272,
                            273,
                            274,
                            282,
                            284,
                            286,
                            288,
                            290,
                            292,
                            294,
                            296,
                            298,
                            300,
                            302,
                            304,
                            306,
                            308,
                            310,
                            312,
                            314,
                            316,
                            318,
                            320,
                            322,
                            324,
                            326,
                            328,
                            330,
                            332,
                            334,
                            336,
                            338,
                            340,
                            342,
                            344,
                            346,
                            348,
                            350,
                            352,
                            354,
                            356,
                            358,
                            360,
                            362,
                            364,
                            366,
                            368,
                            370,
                            372,
                            374,
                            376,
                            378,
                            380,
                            382,
                            384,
                            386,
                            388,
                            390,
                            392,
                            394,
                            396,
                            398,
                            400,
                            402,
                            404,
                            406,
                            408,
                            410
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "_target_name_",
                            "direction_type": "J2000"
                        }
                    }
                ],
                "filter": "HBA_110_190",
                "tile_beam": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "_target_name_",
                    "direction_type": "J2000"
                },
                "antenna_set": "HBA_DUAL",
                "station_groups": [
                    {
                        "stations": [
                            "DE601",
                            "DE602",
                            "DE603",
                            "DE604",
                            "DE605",
                            "DE609",
                            "FR606",
                            "SE607",
                            "UK608",
                            "PL610",
                            "PL611",
                            "PL612",
                            "IE613",
                            "LV614"
                        ],
                        "max_nr_missing": 12
                    },
                    {
                        "stations": [
                            "RS310",
                            "RS509"
                        ],
                        "max_nr_missing": 2
                    }
                ]
            }
        },
        "specifications_template": "http://localhost:8008/api/task_template/"+(_.find(templates, {"name": "parallel calibrator target and beamforming observation"})).id,
        "specifications_template_id": +(_.find(templates, {"name": "parallel calibrator target and beamforming observation"})).id,
        "tags": [],
        "task_blueprints": [],
        "task_blueprints_ids": [],
        "task_type": "observation",
        "updated_at": "2022-05-26T05:21:28.889045"
    },
    {
        "id": 70,
        "url": "http://localhost:3000/api/task_draft/70",
        "connector_types": [
            {
                "id": 2,
                "url": "http://localhost:3000/api/task_connector_type/2",
                "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                "dataformat_value": "MeasurementSet",
                "datatype": "http://localhost:3000/api/datatype/visibilities",
                "datatype_value": "visibilities",
                "iotype": "http://localhost:3000/api/iotype/output",
                "iotype_value": "output",
                "role": "http://localhost:3000/api/role/correlator",
                "role_value": "correlator",
                "task_template": "http://localhost:3000/api/task_template/6",
                "task_template_id": 6
            }
        ],
        "consumed_by": [],
        "consumed_by_ids": [],
        "created_at": "2022-06-17T05:53:55.117168",
        "description": "A simple short test observation",
        "duration": 120,
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Observation",
        "output_pinned": false,
        "produced_by": [],
        "produced_by_ids": [],
        "relative_start_time": 0,
        "relative_stop_time": 120,
        "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/17",
        "scheduling_unit_draft_id": 1,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "",
        "specifications_doc": {
            "QA": {
                "plots": {
                    "enabled": true,
                    "autocorrelation": true,
                    "crosscorrelation": true
                },
                "file_conversion": {
                    "enabled": true,
                    "nr_of_subbands": -1,
                    "nr_of_timestamps": 256
                },
                "inspection_plots": "msplots"
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/2/ref_resolved",
            "duration": 120,
            "correlator": {
                "storage_cluster": "CEP4",
                "integration_time": 1,
                "channels_per_subband": 64,
                "topocentric_frequency_correction": false
            },
            "station_configuration": {
                "SAPs": [
                    {
                        "name": "Polaris",
                        "subbands": [
                            0,
                            1,
                            2,
                            3,
                            4,
                            5,
                            6,
                            7,
                            8,
                            9,
                            10,
                            11,
                            12,
                            13,
                            14,
                            15,
                            16,
                            17,
                            18,
                            19,
                            20,
                            21,
                            22,
                            23,
                            24,
                            25,
                            26,
                            27,
                            28,
                            29,
                            30,
                            31,
                            32,
                            33,
                            34,
                            35,
                            36,
                            37,
                            38,
                            39,
                            40,
                            41,
                            42,
                            43,
                            44,
                            45,
                            46,
                            47,
                            48,
                            49,
                            50,
                            51,
                            52,
                            53,
                            54,
                            55,
                            56,
                            57,
                            58,
                            59,
                            60,
                            61,
                            62,
                            63,
                            64,
                            65,
                            66,
                            67,
                            68,
                            69,
                            70,
                            71,
                            72,
                            73,
                            74,
                            75,
                            76,
                            77,
                            78,
                            79,
                            80,
                            81,
                            82,
                            83,
                            84,
                            85,
                            86,
                            87,
                            88,
                            89,
                            90,
                            91,
                            92,
                            93,
                            94,
                            95,
                            96,
                            97,
                            98,
                            99,
                            100,
                            101,
                            102,
                            103,
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            137,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154,
                            155,
                            156,
                            157,
                            158,
                            159,
                            160,
                            161,
                            162,
                            163,
                            164,
                            165,
                            166,
                            167,
                            168,
                            169,
                            170,
                            171,
                            172,
                            173,
                            174,
                            175,
                            176,
                            177,
                            178,
                            179,
                            180,
                            181,
                            182,
                            183,
                            184,
                            185,
                            186,
                            187,
                            188,
                            189,
                            190,
                            191,
                            192,
                            193,
                            194,
                            195,
                            196,
                            197,
                            198,
                            199,
                            200,
                            201,
                            202,
                            203,
                            204,
                            205,
                            206,
                            207,
                            208,
                            209,
                            210,
                            211,
                            212,
                            213,
                            214,
                            215,
                            216,
                            217,
                            218,
                            219,
                            220,
                            221,
                            222,
                            223,
                            224,
                            225,
                            226,
                            227,
                            228,
                            229,
                            230,
                            231,
                            232,
                            233,
                            234,
                            235,
                            236,
                            237,
                            238,
                            239,
                            240,
                            241,
                            242,
                            243
                        ],
                        "digital_pointing": {
                            "angle1": 0.8115781021773631,
                            "angle2": 1.570505438586231,
                            "target": "target1",
                            "direction_type": "J2000"
                        }
                    }
                ],
                "filter": "HBA_110_190",
                "tile_beam": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "target1",
                    "direction_type": "J2000"
                },
                "antenna_set": "HBA_DUAL_INNER",
                "station_groups": [
                    {
                        "stations": [
                            "CS002",
                            "CS003",
                            "CS004",
                            "CS005",
                            "CS006",
                            "CS007"
                        ],
                        "max_nr_missing": 1
                    }
                ]
            }
        },
        "specifications_template": "http://localhost:3000/api/task_template/"+ (_.find(templates, {"name": "target observation"})).id,
        "specifications_template_id": (_.find(templates, {"name": "target observation"})).id,
        "tags": [],
        "task_blueprints": [],
        "task_blueprints_ids": [],
        "task_type": "observation",
        "updated_at": "2022-06-17T05:53:55.858755"
    },
    {
        "id": 71,
        "url": "http://localhost:3000/api/task_draft/71",
        "connector_types": [
            {
                "id": 2,
                "url": "http://localhost:3000/api/task_connector_type/2",
                "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                "dataformat_value": "MeasurementSet",
                "datatype": "http://localhost:3000/api/datatype/visibilities",
                "datatype_value": "visibilities",
                "iotype": "http://localhost:3000/api/iotype/output",
                "iotype_value": "output",
                "role": "http://localhost:3000/api/role/correlator",
                "role_value": "correlator",
                "task_template": "http://localhost:3000/api/task_template/7",
                "task_template_id": 7
            }
        ],
        "consumed_by": [],
        "consumed_by_ids": [],
        "created_at": "2022-06-21T05:32:23.844672",
        "description": "A simple short test observation",
        "duration": 120,
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Observation",
        "output_pinned": false,
        "produced_by": [],
        "produced_by_ids": [],
        "relative_start_time": 0,
        "relative_stop_time": 120,
        "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/18",
        "scheduling_unit_draft_id": 18,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "",
        "specifications_doc": {
            "QA": {
                "plots": {
                    "enabled": true,
                    "autocorrelation": true,
                    "crosscorrelation": true
                },
                "file_conversion": {
                    "enabled": true,
                    "nr_of_subbands": -1,
                    "nr_of_timestamps": 256
                },
                "inspection_plots": "msplots"
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/2/ref_resolved",
            "duration": 120,
            "correlator": {
                "storage_cluster": "CEP4",
                "integration_time": 1,
                "channels_per_subband": 64,
                "topocentric_frequency_correction": false
            },
            "station_configuration": {
                "SAPs": [
                    {
                        "name": "Polaris",
                        "subbands": [
                            0,
                            1,
                            2,
                            3,
                            4,
                            5,
                            6,
                            7,
                            8,
                            9,
                            10,
                            11,
                            12,
                            13,
                            14,
                            15,
                            16,
                            17,
                            18,
                            19,
                            20,
                            21,
                            22,
                            23,
                            24,
                            25,
                            26,
                            27,
                            28,
                            29,
                            30,
                            31,
                            32,
                            33,
                            34,
                            35,
                            36,
                            37,
                            38,
                            39,
                            40,
                            41,
                            42,
                            43,
                            44,
                            45,
                            46,
                            47,
                            48,
                            49,
                            50,
                            51,
                            52,
                            53,
                            54,
                            55,
                            56,
                            57,
                            58,
                            59,
                            60,
                            61,
                            62,
                            63,
                            64,
                            65,
                            66,
                            67,
                            68,
                            69,
                            70,
                            71,
                            72,
                            73,
                            74,
                            75,
                            76,
                            77,
                            78,
                            79,
                            80,
                            81,
                            82,
                            83,
                            84,
                            85,
                            86,
                            87,
                            88,
                            89,
                            90,
                            91,
                            92,
                            93,
                            94,
                            95,
                            96,
                            97,
                            98,
                            99,
                            100,
                            101,
                            102,
                            103,
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            137,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154,
                            155,
                            156,
                            157,
                            158,
                            159,
                            160,
                            161,
                            162,
                            163,
                            164,
                            165,
                            166,
                            167,
                            168,
                            169,
                            170,
                            171,
                            172,
                            173,
                            174,
                            175,
                            176,
                            177,
                            178,
                            179,
                            180,
                            181,
                            182,
                            183,
                            184,
                            185,
                            186,
                            187,
                            188,
                            189,
                            190,
                            191,
                            192,
                            193,
                            194,
                            195,
                            196,
                            197,
                            198,
                            199,
                            200,
                            201,
                            202,
                            203,
                            204,
                            205,
                            206,
                            207,
                            208,
                            209,
                            210,
                            211,
                            212,
                            213,
                            214,
                            215,
                            216,
                            217,
                            218,
                            219,
                            220,
                            221,
                            222,
                            223,
                            224,
                            225,
                            226,
                            227,
                            228,
                            229,
                            230,
                            231,
                            232,
                            233,
                            234,
                            235,
                            236,
                            237,
                            238,
                            239,
                            240,
                            241,
                            242,
                            243
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "target1",
                            "direction_type": "J2000"
                        }
                    }
                ],
                "filter": "HBA_110_190",
                "tile_beam": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "target1",
                    "direction_type": "J2000"
                },
                "antenna_set": "HBA_DUAL_INNER",
                "station_groups": [
                    {
                        "stations": [
                            "CS002",
                            "CS003",
                            "CS004",
                            "CS005",
                            "CS006",
                            "CS007"
                        ],
                        "max_nr_missing": 1
                    }
                ]
            }
        },
        "specifications_template": "http://localhost:3000/api/task_template/7",
        "specifications_template_id": 7,
        "tags": [],
        "task_blueprints": [],
        "task_blueprints_ids": [],
        "task_type": "observation",
        "updated_at": "2022-06-21T05:32:24.460594"
    }
  ],
  taskBlueprints: [
    {
      "id": 1,
      "url": "http://localhost:3000/api/task_blueprint/1",
      "actual_on_sky_start_time": null,
      "actual_on_sky_stop_time": null,
      "actual_process_start_time": null,
      "actual_process_stop_time": null,
      "consumed_by": [
          "http://localhost:3000/api/task_relation_blueprint/1",
          "http://localhost:3000/api/task_relation_blueprint/2",
          "http://localhost:3000/api/task_relation_blueprint/3",
          "http://localhost:3000/api/task_relation_blueprint/4",
          "http://localhost:3000/api/task_relation_blueprint/5",
          "http://localhost:3000/api/task_relation_blueprint/6"
      ],
      "consumed_by_ids": [
          1,
          2,
          3,
          4,
          5,
          6
      ],
      "created_at": "2022-04-14T06:20:19.668460",
      "description": "Combined parallel Calibrator & Target Observation for LBA with 5 target beams",
      "draft": "http://localhost:3000/api/task_draft/1",
      "draft_id": 1,
      "draftObject": {
        "id": 1,
        "url": "http://localhost:3000/api/task_draft/1",
        "connector_types": [
            {
                "id": 3,
                "url": "http://localhost:3000/api/task_connector_type/3",
                "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                "dataformat_value": "MeasurementSet",
                "datatype": "http://localhost:3000/api/datatype/visibilities",
                "datatype_value": "visibilities",
                "iotype": "http://localhost:3000/api/iotype/output",
                "iotype_value": "output",
                "role": "http://localhost:3000/api/role/correlator",
                "role_value": "correlator",
                "task_template": "http://localhost:3000/api/task_template/5",
                "task_template_id": 5
            }
        ],
        "consumed_by": [
            "http://localhost:3000/api/task_relation_draft/1",
            "http://localhost:3000/api/task_relation_draft/2",
            "http://localhost:3000/api/task_relation_draft/3",
            "http://localhost:3000/api/task_relation_draft/4",
            "http://localhost:3000/api/task_relation_draft/5",
            "http://localhost:3000/api/task_relation_draft/6"
        ],
        "consumed_by_ids": [
            1,
            2,
            3,
            4,
            5,
            6
        ],
        "created_at": "2022-04-14T06:20:19.292483",
        "description": "Combined parallel Calibrator & Target Observation for LBA with 5 target beams",
        "duration": 120,
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Combined Observation",
        "output_pinned": false,
        "produced_by": [],
        "produced_by_ids": [],
        "relative_start_time": 0,
        "relative_stop_time": 120,
        "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
        "scheduling_unit_draft_id": 1,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "oXXX Paaa+01 3Cabc",
        "specifications_doc": {
            "QA": {
                "plots": {
                    "enabled": true,
                    "autocorrelation": true,
                    "crosscorrelation": true
                },
                "file_conversion": {
                    "enabled": true,
                    "nr_of_subbands": -1,
                    "nr_of_timestamps": 256
                },
                "inspection_plots": "msplots"
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
            "duration": 120,
            "calibrator": {
                "name": "calibrator",
                "pointing": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "_calibrator_name_",
                    "direction_type": "J2000"
                },
                "autoselect": false
            },
            "correlator": {
                "storage_cluster": "CEP4",
                "integration_time": 1,
                "channels_per_subband": 64,
                "topocentric_frequency_correction": false
            },
            "station_configuration": {
                "SAPs": [
                    {
                        "name": "target1",
                        "subbands": [
                            74,
                            75,
                            76,
                            77,
                            78,
                            79,
                            80,
                            81,
                            82,
                            83,
                            84,
                            85,
                            86,
                            87,
                            88,
                            89,
                            90,
                            91,
                            92,
                            93,
                            94,
                            95,
                            96,
                            97,
                            98,
                            99,
                            100,
                            101,
                            102,
                            103,
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            137,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+01",
                            "direction_type": "J2000"
                        }
                    },
                    {
                        "name": "target2",
                        "subbands": [
                            74,
                            75,
                            76,
                            77,
                            78,
                            79,
                            80,
                            81,
                            82,
                            83,
                            84,
                            85,
                            86,
                            87,
                            88,
                            89,
                            90,
                            91,
                            92,
                            93,
                            94,
                            95,
                            96,
                            97,
                            98,
                            99,
                            100,
                            101,
                            102,
                            103,
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            137,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+02",
                            "direction_type": "J2000"
                        }
                    },
                    {
                        "name": "target3",
                        "subbands": [
                            74,
                            75,
                            76,
                            77,
                            78,
                            79,
                            80,
                            81,
                            82,
                            83,
                            84,
                            85,
                            86,
                            87,
                            88,
                            89,
                            90,
                            91,
                            92,
                            93,
                            94,
                            95,
                            96,
                            97,
                            98,
                            99,
                            100,
                            101,
                            102,
                            103,
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            137,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+03",
                            "direction_type": "J2000"
                        }
                    },
                    {
                        "name": "target4",
                        "subbands": [
                            74,
                            75,
                            76,
                            77,
                            78,
                            79,
                            80,
                            81,
                            82,
                            83,
                            84,
                            85,
                            86,
                            87,
                            88,
                            89,
                            90,
                            91,
                            92,
                            93,
                            94,
                            95,
                            96,
                            97,
                            98,
                            99,
                            100,
                            101,
                            102,
                            103,
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            137,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+04",
                            "direction_type": "J2000"
                        }
                    },
                    {
                        "name": "target5",
                        "subbands": [
                            74,
                            75,
                            76,
                            77,
                            78,
                            79,
                            80,
                            81,
                            82,
                            83,
                            84,
                            85,
                            86,
                            87,
                            88,
                            89,
                            90,
                            91,
                            92,
                            93,
                            94,
                            95,
                            96,
                            97,
                            98,
                            99,
                            100,
                            101,
                            102,
                            103,
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            137,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+05",
                            "direction_type": "J2000"
                        }
                    }
                ],
                "filter": "LBA_10_90",
                "tile_beam": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "_target_name_",
                    "direction_type": "J2000"
                },
                "antenna_set": "LBA_SPARSE_EVEN",
                "station_groups": [
                    {
                        "stations": [
                            "CS001",
                            "CS002",
                            "CS003",
                            "CS004",
                            "CS005",
                            "CS006",
                            "CS007",
                            "CS011",
                            "CS013",
                            "CS017",
                            "CS021",
                            "CS024",
                            "CS026",
                            "CS028",
                            "CS030",
                            "CS031",
                            "CS032",
                            "CS101",
                            "CS103",
                            "CS201",
                            "CS301",
                            "CS302",
                            "CS401",
                            "CS501",
                            "RS106",
                            "RS205",
                            "RS208",
                            "RS210",
                            "RS305",
                            "RS306",
                            "RS307",
                            "RS310",
                            "RS406",
                            "RS407",
                            "RS409",
                            "RS503",
                            "RS508",
                            "RS509"
                        ],
                        "max_nr_missing": 4
                    },
                    {
                        "stations": [
                            "RS508",
                            "RS509"
                        ],
                        "max_nr_missing": 1
                    },
                    {
                        "stations": [
                            "RS310",
                            "RS210"
                        ],
                        "max_nr_missing": 0
                    }
                ]
            }
        },
        "specifications_template": "http://localhost:3000/api/task_template/5",
        "specifications_template_id": 5,
        "tags": [],
        "task_blueprints": [
            "http://localhost:3000/api/task_blueprint/1"
        ],
        "task_blueprints_ids": [
            1
        ],
        "task_type": "observation",
        "updated_at": "2022-04-14T06:20:19.292495"
      },
      "duration": 720,
      "error_reason": "",
      "first_scheduling_relation": [],
      "first_scheduling_relation_ids": [],
      "name": "Combined Observation",
      "obsolete_since": null,
      "on_sky_duration": 120,
      "on_sky_start_time": "2022-04-14T06:25:19.054619",
      "on_sky_stop_time": "2022-04-14T06:27:19.054619",
      "output_pinned": false,
      "process_start_time": "2022-04-14T06:25:19.054619",
      "process_stop_time": "2022-04-14T06:37:19.054619",
      "produced_by": [],
      "produced_by_ids": [],
      "relative_start_time": 0,
      "relative_stop_time": 120,
      "scheduled_start_time": "2022-04-14T06:25:19.054619",
      "scheduled_stop_time": "2022-04-14T06:37:19.054619",
      "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
      "scheduling_unit_blueprint_id": 1,
      "second_scheduling_relation": [],
      "second_scheduling_relation_ids": [],
      "short_description": "oXXX Paaa+01 3Cabc",
      "specifications_doc": {
          "QA": {
              "plots": {
                  "enabled": true,
                  "autocorrelation": true,
                  "crosscorrelation": true
              },
              "file_conversion": {
                  "enabled": true,
                  "nr_of_subbands": -1,
                  "nr_of_timestamps": 256
              },
              "inspection_plots": "msplots"
          },
          "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
          "duration": 120,
          "calibrator": {
              "name": "calibrator",
              "pointing": {
                  "angle1": 0.6624317181687094,
                  "angle2": 1.5579526427549426,
                  "target": "_calibrator_name_",
                  "direction_type": "J2000"
              },
              "autoselect": false
          },
          "correlator": {
              "storage_cluster": "CEP4",
              "integration_time": 1,
              "channels_per_subband": 64,
              "topocentric_frequency_correction": false
          },
          "station_configuration": {
              "SAPs": [
                  {
                      "name": "target1",
                      "subbands": [
                          74,
                          75,
                          76,
                          77,
                          78,
                          79,
                          80,
                          81,
                          82,
                          83,
                          84,
                          85,
                          86,
                          87,
                          88,
                          89,
                          90,
                          91,
                          92,
                          93,
                          94,
                          95,
                          96,
                          97,
                          98,
                          99,
                          100,
                          101,
                          102,
                          103,
                          104,
                          105,
                          106,
                          107,
                          108,
                          109,
                          110,
                          111,
                          112,
                          113,
                          114,
                          115,
                          116,
                          117,
                          118,
                          119,
                          120,
                          121,
                          122,
                          123,
                          124,
                          125,
                          126,
                          127,
                          128,
                          129,
                          130,
                          131,
                          132,
                          133,
                          134,
                          135,
                          136,
                          137,
                          138,
                          139,
                          140,
                          141,
                          142,
                          143,
                          144,
                          145,
                          146,
                          147,
                          148,
                          149,
                          150,
                          151,
                          152,
                          153,
                          154
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+01",
                          "direction_type": "J2000"
                      }
                  },
                  {
                      "name": "target2",
                      "subbands": [
                          74,
                          75,
                          76,
                          77,
                          78,
                          79,
                          80,
                          81,
                          82,
                          83,
                          84,
                          85,
                          86,
                          87,
                          88,
                          89,
                          90,
                          91,
                          92,
                          93,
                          94,
                          95,
                          96,
                          97,
                          98,
                          99,
                          100,
                          101,
                          102,
                          103,
                          104,
                          105,
                          106,
                          107,
                          108,
                          109,
                          110,
                          111,
                          112,
                          113,
                          114,
                          115,
                          116,
                          117,
                          118,
                          119,
                          120,
                          121,
                          122,
                          123,
                          124,
                          125,
                          126,
                          127,
                          128,
                          129,
                          130,
                          131,
                          132,
                          133,
                          134,
                          135,
                          136,
                          137,
                          138,
                          139,
                          140,
                          141,
                          142,
                          143,
                          144,
                          145,
                          146,
                          147,
                          148,
                          149,
                          150,
                          151,
                          152,
                          153,
                          154
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+02",
                          "direction_type": "J2000"
                      }
                  },
                  {
                      "name": "target3",
                      "subbands": [
                          74,
                          75,
                          76,
                          77,
                          78,
                          79,
                          80,
                          81,
                          82,
                          83,
                          84,
                          85,
                          86,
                          87,
                          88,
                          89,
                          90,
                          91,
                          92,
                          93,
                          94,
                          95,
                          96,
                          97,
                          98,
                          99,
                          100,
                          101,
                          102,
                          103,
                          104,
                          105,
                          106,
                          107,
                          108,
                          109,
                          110,
                          111,
                          112,
                          113,
                          114,
                          115,
                          116,
                          117,
                          118,
                          119,
                          120,
                          121,
                          122,
                          123,
                          124,
                          125,
                          126,
                          127,
                          128,
                          129,
                          130,
                          131,
                          132,
                          133,
                          134,
                          135,
                          136,
                          137,
                          138,
                          139,
                          140,
                          141,
                          142,
                          143,
                          144,
                          145,
                          146,
                          147,
                          148,
                          149,
                          150,
                          151,
                          152,
                          153,
                          154
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+03",
                          "direction_type": "J2000"
                      }
                  },
                  {
                      "name": "target4",
                      "subbands": [
                          74,
                          75,
                          76,
                          77,
                          78,
                          79,
                          80,
                          81,
                          82,
                          83,
                          84,
                          85,
                          86,
                          87,
                          88,
                          89,
                          90,
                          91,
                          92,
                          93,
                          94,
                          95,
                          96,
                          97,
                          98,
                          99,
                          100,
                          101,
                          102,
                          103,
                          104,
                          105,
                          106,
                          107,
                          108,
                          109,
                          110,
                          111,
                          112,
                          113,
                          114,
                          115,
                          116,
                          117,
                          118,
                          119,
                          120,
                          121,
                          122,
                          123,
                          124,
                          125,
                          126,
                          127,
                          128,
                          129,
                          130,
                          131,
                          132,
                          133,
                          134,
                          135,
                          136,
                          137,
                          138,
                          139,
                          140,
                          141,
                          142,
                          143,
                          144,
                          145,
                          146,
                          147,
                          148,
                          149,
                          150,
                          151,
                          152,
                          153,
                          154
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+04",
                          "direction_type": "J2000"
                      }
                  },
                  {
                      "name": "target5",
                      "subbands": [
                          74,
                          75,
                          76,
                          77,
                          78,
                          79,
                          80,
                          81,
                          82,
                          83,
                          84,
                          85,
                          86,
                          87,
                          88,
                          89,
                          90,
                          91,
                          92,
                          93,
                          94,
                          95,
                          96,
                          97,
                          98,
                          99,
                          100,
                          101,
                          102,
                          103,
                          104,
                          105,
                          106,
                          107,
                          108,
                          109,
                          110,
                          111,
                          112,
                          113,
                          114,
                          115,
                          116,
                          117,
                          118,
                          119,
                          120,
                          121,
                          122,
                          123,
                          124,
                          125,
                          126,
                          127,
                          128,
                          129,
                          130,
                          131,
                          132,
                          133,
                          134,
                          135,
                          136,
                          137,
                          138,
                          139,
                          140,
                          141,
                          142,
                          143,
                          144,
                          145,
                          146,
                          147,
                          148,
                          149,
                          150,
                          151,
                          152,
                          153,
                          154
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+05",
                          "direction_type": "J2000"
                      }
                  }
              ],
              "filter": "LBA_10_90",
              "tile_beam": {
                  "angle1": 0.6624317181687094,
                  "angle2": 1.5579526427549426,
                  "target": "_target_name_",
                  "direction_type": "J2000"
              },
              "antenna_set": "LBA_SPARSE_EVEN",
              "station_groups": [
                  {
                      "stations": [
                          "CS001",
                          "CS002",
                          "CS003",
                          "CS004",
                          "CS005",
                          "CS006",
                          "CS007",
                          "CS011",
                          "CS013",
                          "CS017",
                          "CS021",
                          "CS024",
                          "CS026",
                          "CS028",
                          "CS030",
                          "CS031",
                          "CS032",
                          "CS101",
                          "CS103",
                          "CS201",
                          "CS301",
                          "CS302",
                          "CS401",
                          "CS501",
                          "RS106",
                          "RS205",
                          "RS208",
                          "RS210",
                          "RS305",
                          "RS306",
                          "RS307",
                          "RS310",
                          "RS406",
                          "RS407",
                          "RS409",
                          "RS503",
                          "RS508",
                          "RS509"
                      ],
                      "max_nr_missing": 4
                  },
                  {
                      "stations": [
                          "RS508",
                          "RS509"
                      ],
                      "max_nr_missing": 1
                  },
                  {
                      "stations": [
                          "RS310",
                          "RS210"
                      ],
                      "max_nr_missing": 0
                  }
              ]
          }
      },
      "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "parallel calibrator target observation"})).id,
      "specifications_template_id": (_.find(templates, {"name": "parallel calibrator target observation"})).id,
      "status": "scheduled",
      "status_value": "scheduled",
      "subtasks": [
          {
              "id": 2000001,
              "url": "http://localhost:3000/api/subtask/2000001",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "cluster": "http://localhost:3000/api/cluster/1",
              "cluster_id": 1,
              "cluster_name": "CEP4",
              "created_at": "2022-04-14T06:20:20.561919",
              "created_or_updated_by_user": null,
              "created_or_updated_by_user_id": null,
              "duration": 300,
              "error_reason": null,
              "global_identifier": "http://localhost:3000/api/sip_identifier/4",
              "global_identifier_id": 4,
              "global_parset_identifier": "http://localhost:3000/api/sip_identifier/5",
              "global_parset_identifier_id": 5,
              "inputs": [
                  "http://localhost:3000/api/subtask_input/1"
              ],
              "inputs_ids": [
                  1
              ],
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "outputs": [
                  "http://localhost:3000/api/subtask_output/2"
              ],
              "outputs_ids": [
                  2
              ],
              "primary": false,
              "process_start_time": "2022-04-14T06:27:19.054619",
              "process_stop_time": "2022-04-14T06:32:19.054619",
              "processing_feedback_doc": null,
              "processing_feedback_template": null,
              "processing_feedback_template_id": null,
              "scheduled_start_time": "2022-04-14T06:27:19.054619",
              "scheduled_stop_time": "2022-04-14T06:32:19.054619",
              "specifications_doc": {
                  "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/QA%20file%20conversion/2/ref_resolved",
                  "file_conversion": {
                      "enabled": true,
                      "nr_of_subbands": -1,
                      "nr_of_timestamps": 256
                  }
              },
              "specifications_template": "http://localhost:3000/api/subtask_template/5",
              "specifications_template_id": 5,
              "state": "http://localhost:3000/api/subtask_state/defined",
              "state_value": "defined",
              "subtask_type": "qa_files",
              "tags": [],
              "task_blueprint": "http://localhost:3000/api/task_blueprint/1",
              "task_blueprint_id": 1,
              "total_nr_of_dataproducts": 0,
              "total_size_of_dataproducts": 0,
              "total_size_of_dataproducts_on_filesysem": 0,
              "unschedulable_reason": null,
              "updated_at": "2022-04-14T06:21:07.213955"
          },
          {
              "id": 2000002,
              "url": "http://localhost:3000/api/subtask/2000002",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "cluster": "http://localhost:3000/api/cluster/1",
              "cluster_id": 1,
              "cluster_name": "CEP4",
              "created_at": "2022-04-14T06:20:20.655886",
              "created_or_updated_by_user": null,
              "created_or_updated_by_user_id": null,
              "duration": 300,
              "error_reason": null,
              "global_identifier": "http://localhost:3000/api/sip_identifier/6",
              "global_identifier_id": 6,
              "global_parset_identifier": "http://localhost:3000/api/sip_identifier/7",
              "global_parset_identifier_id": 7,
              "inputs": [
                  "http://localhost:3000/api/subtask_input/2"
              ],
              "inputs_ids": [
                  2
              ],
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "outputs": [
                  "http://localhost:3000/api/subtask_output/3"
              ],
              "outputs_ids": [
                  3
              ],
              "primary": false,
              "process_start_time": "2022-04-14T06:32:19.054619",
              "process_stop_time": "2022-04-14T06:37:19.054619",
              "processing_feedback_doc": null,
              "processing_feedback_template": null,
              "processing_feedback_template_id": null,
              "scheduled_start_time": "2022-04-14T06:32:19.054619",
              "scheduled_stop_time": "2022-04-14T06:37:19.054619",
              "specifications_doc": {
                  "plots": {
                      "enabled": true,
                      "autocorrelation": true,
                      "crosscorrelation": true
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/QA%20plots/2/ref_resolved"
              },
              "specifications_template": "http://localhost:3000/api/subtask_template/3",
              "specifications_template_id": 3,
              "state": "http://localhost:3000/api/subtask_state/defined",
              "state_value": "defined",
              "subtask_type": "qa_plots",
              "tags": [],
              "task_blueprint": "http://localhost:3000/api/task_blueprint/1",
              "task_blueprint_id": 1,
              "total_nr_of_dataproducts": 0,
              "total_size_of_dataproducts": 0,
              "total_size_of_dataproducts_on_filesysem": 0,
              "unschedulable_reason": null,
              "updated_at": "2022-04-14T06:21:07.229090"
          },
          {
              "id": 2000000,
              "url": "http://localhost:3000/api/subtask/2000000",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "cluster": "http://localhost:3000/api/cluster/1",
              "cluster_id": 1,
              "cluster_name": "CEP4",
              "created_at": "2022-04-14T06:20:20.442495",
              "created_or_updated_by_user": null,
              "created_or_updated_by_user_id": null,
              "duration": 120,
              "error_reason": null,
              "global_identifier": "http://localhost:3000/api/sip_identifier/2",
              "global_identifier_id": 2,
              "global_parset_identifier": "http://localhost:3000/api/sip_identifier/3",
              "global_parset_identifier_id": 3,
              "inputs": [],
              "inputs_ids": [],
              "obsolete_since": null,
              "on_sky_duration": "00:02:00",
              "on_sky_start_time": "2022-04-14T06:25:19.054619",
              "on_sky_stop_time": "2022-04-14T06:27:19.054619",
              "outputs": [
                  "http://localhost:3000/api/subtask_output/1"
              ],
              "outputs_ids": [
                  1
              ],
              "primary": true,
              "process_start_time": "2022-04-14T06:25:19.054619",
              "process_stop_time": "2022-04-14T06:27:19.054619",
              "processing_feedback_doc": null,
              "processing_feedback_template": null,
              "processing_feedback_template_id": null,
              "scheduled_start_time": "2022-04-14T06:25:19.054619",
              "scheduled_stop_time": "2022-04-14T06:27:19.054619",
              "specifications_doc": {
                  "QA": {
                      "inspection_plots": "msplots"
                  },
                  "COBALT": {
                      "debug": {
                          "write_output": true,
                          "benchmark_file": "",
                          "annotate_broken_antennas": true
                      },
                      "version": 1,
                      "blocksize": 195584,
                      "resources": {
                          "writer_nodes": [
                              "cpu01.cep4.infiniband.lofar",
                              "cpu02.cep4.infiniband.lofar",
                              "cpu03.cep4.infiniband.lofar",
                              "cpu04.cep4.infiniband.lofar",
                              "cpu05.cep4.infiniband.lofar"
                          ],
                          "processing_nodes": [
                              "cbt201_0",
                              "cbt201_1",
                              "cbt202_0",
                              "cbt202_1",
                              "cbt203_0",
                              "cbt203_1",
                              "cbt204_0",
                              "cbt204_1",
                              "cbt205_0",
                              "cbt205_1",
                              "cbt207_0",
                              "cbt207_1",
                              "cbt208_0",
                              "cbt208_1",
                              "cbt209_0",
                              "cbt209_1",
                              "cbt210_0",
                              "cbt210_1",
                              "cbt211_0",
                              "cbt211_1",
                              "cbt212_0",
                              "cbt212_1"
                          ]
                      },
                      "beamformer": {
                          "input_ppf": false,
                          "tab_pipelines": [],
                          "flyseye_pipelines": []
                      },
                      "correlator": {
                          "enabled": true,
                          "doppler_correction": false,
                          "channels_per_subband": 64,
                          "blocks_per_integration": 1,
                          "integrations_per_block": 1
                      },
                      "delay_compensation": true,
                      "bandpass_correction": true
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/observation%20control/2/ref_resolved",
                  "stations": {
                      "filter": "LBA_10_90",
                      "antenna_set": "LBA_SPARSE_EVEN",
                      "station_list": [
                          "CS001",
                          "CS002",
                          "CS003",
                          "CS004",
                          "CS005",
                          "CS006",
                          "CS007",
                          "CS011",
                          "CS013",
                          "CS017",
                          "CS021",
                          "CS024",
                          "CS026",
                          "CS028",
                          "CS030",
                          "CS031",
                          "CS032",
                          "CS101",
                          "CS103",
                          "CS201",
                          "CS301",
                          "CS302",
                          "CS401",
                          "CS501",
                          "RS106",
                          "RS205",
                          "RS208",
                          "RS210",
                          "RS305",
                          "RS306",
                          "RS307",
                          "RS310",
                          "RS406",
                          "RS407",
                          "RS409",
                          "RS503",
                          "RS508",
                          "RS509"
                      ],
                      "analog_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "_calibrator_name_",
                          "direction_type": "J2000"
                      },
                      "digital_pointings": [
                          {
                              "name": "target1",
                              "pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+01",
                                  "direction_type": "J2000"
                              },
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ]
                          },
                          {
                              "name": "target2",
                              "pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+02",
                                  "direction_type": "J2000"
                              },
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ]
                          },
                          {
                              "name": "target3",
                              "pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+03",
                                  "direction_type": "J2000"
                              },
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ]
                          },
                          {
                              "name": "target4",
                              "pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+04",
                                  "direction_type": "J2000"
                              },
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ]
                          },
                          {
                              "name": "target5",
                              "pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+05",
                                  "direction_type": "J2000"
                              },
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ]
                          },
                          {
                              "name": "calibrator",
                              "pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "_calibrator_name_",
                                  "direction_type": "J2000"
                              },
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ]
                          }
                      ]
                  }
              },
              "specifications_template": "http://localhost:3000/api/subtask_template/4",
              "specifications_template_id": 4,
              "state": "http://localhost:3000/api/subtask_state/scheduled",
              "state_value": "scheduled",
              "subtask_type": "observation",
              "tags": [],
              "task_blueprint": "http://localhost:3000/api/task_blueprint/1",
              "task_blueprint_id": 1,
              "total_nr_of_dataproducts": 0,
              "total_size_of_dataproducts": 0,
              "total_size_of_dataproducts_on_filesysem": 0,
              "unschedulable_reason": null,
              "updated_at": "2022-04-14T06:21:07.190136",
              "predecessors": [],
              "successors": [
                {
                    "id": 3,
                    "url": "http://localhost:3000/api/task_blueprint/3",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "consumed_by": [
                        "http://localhost:3000/api/task_relation_blueprint/12"
                    ],
                    "consumed_by_ids": [
                        12
                    ],
                    "created_at": "2022-04-14T06:20:19.774241",
                    "description": "Preprocessing Pipeline for Target Observation target5",
                    "draft": "http://localhost:3000/api/task_draft/3",
                    "draft_id": 3,
                    "duration": 120,
                    "error_reason": "",
                    "first_scheduling_relation": [],
                    "first_scheduling_relation_ids": [],
                    "name": "Pipeline target5",
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "output_pinned": false,
                    "process_start_time": "2022-04-14T06:27:19.054619",
                    "process_stop_time": "2022-04-14T06:29:19.054619",
                    "produced_by": [
                        "http://localhost:3000/api/task_relation_blueprint/2"
                    ],
                    "produced_by_ids": [
                        2
                    ],
                    "relative_start_time": 0,
                    "relative_stop_time": 0,
                    "scheduled_start_time": "2022-04-14T06:27:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                    "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
                    "scheduling_unit_blueprint_id": 1,
                    "second_scheduling_relation": [],
                    "second_scheduling_relation_ids": [],
                    "short_description": "Paaa+05/1.4/PP",
                    "specifications_doc": {
                        "flag": {
                            "rfi_strategy": "LBAdefault",
                            "outerchannels": true,
                            "autocorrelations": true
                        },
                        "demix": {
                            "sources": [],
                            "time_steps": 10,
                            "ignore_target": false,
                            "frequency_steps": 64
                        },
                        "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "average": {
                            "time_steps": 1,
                            "frequency_steps": 1
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/task_template/10",
                    "specifications_template_id": 10,
                    "status": "schedulable",
                    "status_value": "schedulable",
                    "subtasks": [
                        {
                            "id": 2000003,
                            "url": "http://localhost:3000/api/subtask/2000003",
                            "actual_on_sky_start_time": null,
                            "actual_on_sky_stop_time": null,
                            "actual_process_start_time": null,
                            "actual_process_stop_time": null,
                            "cluster": "http://localhost:3000/api/cluster/1",
                            "cluster_id": 1,
                            "cluster_name": "CEP4",
                            "created_at": "2022-04-14T06:20:20.722939",
                            "created_or_updated_by_user": null,
                            "created_or_updated_by_user_id": null,
                            "duration": 120,
                            "error_reason": null,
                            "global_identifier": "http://localhost:3000/api/sip_identifier/8",
                            "global_identifier_id": 8,
                            "global_parset_identifier": "http://localhost:3000/api/sip_identifier/9",
                            "global_parset_identifier_id": 9,
                            "inputs": [
                                "http://localhost:3000/api/subtask_input/3"
                            ],
                            "inputs_ids": [
                                3
                            ],
                            "obsolete_since": null,
                            "on_sky_duration": null,
                            "on_sky_start_time": null,
                            "on_sky_stop_time": null,
                            "outputs": [
                                "http://localhost:3000/api/subtask_output/4"
                            ],
                            "outputs_ids": [
                                4
                            ],
                            "primary": true,
                            "process_start_time": "2022-04-14T06:27:19.054619",
                            "process_stop_time": "2022-04-14T06:29:19.054619",
                            "processing_feedback_doc": null,
                            "processing_feedback_template": null,
                            "processing_feedback_template_id": null,
                            "scheduled_start_time": "2022-04-14T06:27:19.054619",
                            "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                            "specifications_doc": {
                                "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                                "demixer": {
                                    "enabled": true,
                                    "baselines": "CS*,RS*&",
                                    "time_steps": 1,
                                    "demix_always": [],
                                    "ignore_target": false,
                                    "demix_if_needed": [],
                                    "frequency_steps": 1,
                                    "demix_time_steps": 10,
                                    "demix_frequency_steps": 64
                                },
                                "aoflagger": {
                                    "enabled": true,
                                    "strategy": "LBAdefault"
                                },
                                "preflagger0": {
                                    "enabled": true,
                                    "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                                },
                                "preflagger1": {
                                    "enabled": true,
                                    "corrtype": "auto"
                                },
                                "storagemanager": "dysco",
                                "cluster_resources": {
                                    "where": {
                                        "cluster": "CEP4",
                                        "partition": "cpu"
                                    },
                                    "cores_per_task": 2,
                                    "parallel_tasks": 122
                                }
                            },
                            "specifications_template": "http://localhost:3000/api/subtask_template/7",
                            "specifications_template_id": 7,
                            "state": "http://localhost:3000/api/subtask_state/defined",
                            "state_value": "defined",
                            "subtask_type": "pipeline",
                            "tags": [],
                            "task_blueprint": "http://localhost:3000/api/task_blueprint/3",
                            "task_blueprint_id": 3,
                            "total_nr_of_dataproducts": 0,
                            "total_size_of_dataproducts": 0,
                            "total_size_of_dataproducts_on_filesysem": 0,
                            "unschedulable_reason": null,
                            "updated_at": "2022-04-14T06:21:07.482780"
                        }
                    ],
                    "subtasks_ids": [
                        2000003
                    ],
                    "tags": [],
                    "task_type": "pipeline",
                    "unschedulable_reason": "",
                    "updated_at": "2022-04-14T06:20:19.774253"
                },
                {
                    "id": 5,
                    "url": "http://localhost:3000/api/task_blueprint/5",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "consumed_by": [
                        "http://localhost:3000/api/task_relation_blueprint/10"
                    ],
                    "consumed_by_ids": [
                        10
                    ],
                    "created_at": "2022-04-14T06:20:19.856020",
                    "description": "Preprocessing Pipeline for Target Observation target3",
                    "draft": "http://localhost:3000/api/task_draft/5",
                    "draft_id": 5,
                    "duration": 120,
                    "error_reason": "",
                    "first_scheduling_relation": [],
                    "first_scheduling_relation_ids": [],
                    "name": "Pipeline target3",
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "output_pinned": false,
                    "process_start_time": "2022-04-14T06:27:19.054619",
                    "process_stop_time": "2022-04-14T06:29:19.054619",
                    "produced_by": [
                        "http://localhost:3000/api/task_relation_blueprint/4"
                    ],
                    "produced_by_ids": [
                        4
                    ],
                    "relative_start_time": 0,
                    "relative_stop_time": 0,
                    "scheduled_start_time": "2022-04-14T06:27:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                    "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
                    "scheduling_unit_blueprint_id": 1,
                    "second_scheduling_relation": [],
                    "second_scheduling_relation_ids": [],
                    "short_description": "Paaa+03/1.2/PP",
                    "specifications_doc": {
                        "flag": {
                            "rfi_strategy": "LBAdefault",
                            "outerchannels": true,
                            "autocorrelations": true
                        },
                        "demix": {
                            "sources": [],
                            "time_steps": 10,
                            "ignore_target": false,
                            "frequency_steps": 64
                        },
                        "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "average": {
                            "time_steps": 1,
                            "frequency_steps": 1
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/task_template/10",
                    "specifications_template_id": 10,
                    "status": "schedulable",
                    "status_value": "schedulable",
                    "subtasks": [
                        {
                            "id": 2000004,
                            "url": "http://localhost:3000/api/subtask/2000004",
                            "actual_on_sky_start_time": null,
                            "actual_on_sky_stop_time": null,
                            "actual_process_start_time": null,
                            "actual_process_stop_time": null,
                            "cluster": "http://localhost:3000/api/cluster/1",
                            "cluster_id": 1,
                            "cluster_name": "CEP4",
                            "created_at": "2022-04-14T06:20:20.825054",
                            "created_or_updated_by_user": null,
                            "created_or_updated_by_user_id": null,
                            "duration": 120,
                            "error_reason": null,
                            "global_identifier": "http://localhost:3000/api/sip_identifier/10",
                            "global_identifier_id": 10,
                            "global_parset_identifier": "http://localhost:3000/api/sip_identifier/11",
                            "global_parset_identifier_id": 11,
                            "inputs": [
                                "http://localhost:3000/api/subtask_input/4"
                            ],
                            "inputs_ids": [
                                4
                            ],
                            "obsolete_since": null,
                            "on_sky_duration": null,
                            "on_sky_start_time": null,
                            "on_sky_stop_time": null,
                            "outputs": [
                                "http://localhost:3000/api/subtask_output/5"
                            ],
                            "outputs_ids": [
                                5
                            ],
                            "primary": true,
                            "process_start_time": "2022-04-14T06:27:19.054619",
                            "process_stop_time": "2022-04-14T06:29:19.054619",
                            "processing_feedback_doc": null,
                            "processing_feedback_template": null,
                            "processing_feedback_template_id": null,
                            "scheduled_start_time": "2022-04-14T06:27:19.054619",
                            "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                            "specifications_doc": {
                                "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                                "demixer": {
                                    "enabled": true,
                                    "baselines": "CS*,RS*&",
                                    "time_steps": 1,
                                    "demix_always": [],
                                    "ignore_target": false,
                                    "demix_if_needed": [],
                                    "frequency_steps": 1,
                                    "demix_time_steps": 10,
                                    "demix_frequency_steps": 64
                                },
                                "aoflagger": {
                                    "enabled": true,
                                    "strategy": "LBAdefault"
                                },
                                "preflagger0": {
                                    "enabled": true,
                                    "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                                },
                                "preflagger1": {
                                    "enabled": true,
                                    "corrtype": "auto"
                                },
                                "storagemanager": "dysco",
                                "cluster_resources": {
                                    "where": {
                                        "cluster": "CEP4",
                                        "partition": "cpu"
                                    },
                                    "cores_per_task": 2,
                                    "parallel_tasks": 122
                                }
                            },
                            "specifications_template": "http://localhost:3000/api/subtask_template/7",
                            "specifications_template_id": 7,
                            "state": "http://localhost:3000/api/subtask_state/defined",
                            "state_value": "defined",
                            "subtask_type": "pipeline",
                            "tags": [],
                            "task_blueprint": "http://localhost:3000/api/task_blueprint/5",
                            "task_blueprint_id": 5,
                            "total_nr_of_dataproducts": 0,
                            "total_size_of_dataproducts": 0,
                            "total_size_of_dataproducts_on_filesysem": 0,
                            "unschedulable_reason": null,
                            "updated_at": "2022-04-14T06:21:07.306954"
                        }
                    ],
                    "subtasks_ids": [
                        2000004
                    ],
                    "tags": [],
                    "task_type": "pipeline",
                    "unschedulable_reason": "",
                    "updated_at": "2022-04-14T06:20:19.856102"
                },
                {
                    "id": 4,
                    "url": "http://localhost:3000/api/task_blueprint/4",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "consumed_by": [
                        "http://localhost:3000/api/task_relation_blueprint/11"
                    ],
                    "consumed_by_ids": [
                        11
                    ],
                    "created_at": "2022-04-14T06:20:19.813073",
                    "description": "Preprocessing Pipeline for Target Observation target4",
                    "draft": "http://localhost:3000/api/task_draft/4",
                    "draft_id": 4,
                    "duration": 120,
                    "error_reason": "",
                    "first_scheduling_relation": [],
                    "first_scheduling_relation_ids": [],
                    "name": "Pipeline target4",
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "output_pinned": false,
                    "process_start_time": "2022-04-14T06:27:19.054619",
                    "process_stop_time": "2022-04-14T06:29:19.054619",
                    "produced_by": [
                        "http://localhost:3000/api/task_relation_blueprint/3"
                    ],
                    "produced_by_ids": [
                        3
                    ],
                    "relative_start_time": 0,
                    "relative_stop_time": 0,
                    "scheduled_start_time": "2022-04-14T06:27:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                    "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
                    "scheduling_unit_blueprint_id": 1,
                    "second_scheduling_relation": [],
                    "second_scheduling_relation_ids": [],
                    "short_description": "Paaa+04/1.3/PP",
                    "specifications_doc": {
                        "flag": {
                            "rfi_strategy": "LBAdefault",
                            "outerchannels": true,
                            "autocorrelations": true
                        },
                        "demix": {
                            "sources": [],
                            "time_steps": 10,
                            "ignore_target": false,
                            "frequency_steps": 64
                        },
                        "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "average": {
                            "time_steps": 1,
                            "frequency_steps": 1
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/task_template/10",
                    "specifications_template_id": 10,
                    "status": "schedulable",
                    "status_value": "schedulable",
                    "subtasks": [
                        {
                            "id": 2000005,
                            "url": "http://localhost:3000/api/subtask/2000005",
                            "actual_on_sky_start_time": null,
                            "actual_on_sky_stop_time": null,
                            "actual_process_start_time": null,
                            "actual_process_stop_time": null,
                            "cluster": "http://localhost:3000/api/cluster/1",
                            "cluster_id": 1,
                            "cluster_name": "CEP4",
                            "created_at": "2022-04-14T06:20:20.929183",
                            "created_or_updated_by_user": null,
                            "created_or_updated_by_user_id": null,
                            "duration": 120,
                            "error_reason": null,
                            "global_identifier": "http://localhost:3000/api/sip_identifier/12",
                            "global_identifier_id": 12,
                            "global_parset_identifier": "http://localhost:3000/api/sip_identifier/13",
                            "global_parset_identifier_id": 13,
                            "inputs": [
                                "http://localhost:3000/api/subtask_input/5"
                            ],
                            "inputs_ids": [
                                5
                            ],
                            "obsolete_since": null,
                            "on_sky_duration": null,
                            "on_sky_start_time": null,
                            "on_sky_stop_time": null,
                            "outputs": [
                                "http://localhost:3000/api/subtask_output/6"
                            ],
                            "outputs_ids": [
                                6
                            ],
                            "primary": true,
                            "process_start_time": "2022-04-14T06:27:19.054619",
                            "process_stop_time": "2022-04-14T06:29:19.054619",
                            "processing_feedback_doc": null,
                            "processing_feedback_template": null,
                            "processing_feedback_template_id": null,
                            "scheduled_start_time": "2022-04-14T06:27:19.054619",
                            "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                            "specifications_doc": {
                                "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                                "demixer": {
                                    "enabled": true,
                                    "baselines": "CS*,RS*&",
                                    "time_steps": 1,
                                    "demix_always": [],
                                    "ignore_target": false,
                                    "demix_if_needed": [],
                                    "frequency_steps": 1,
                                    "demix_time_steps": 10,
                                    "demix_frequency_steps": 64
                                },
                                "aoflagger": {
                                    "enabled": true,
                                    "strategy": "LBAdefault"
                                },
                                "preflagger0": {
                                    "enabled": true,
                                    "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                                },
                                "preflagger1": {
                                    "enabled": true,
                                    "corrtype": "auto"
                                },
                                "storagemanager": "dysco",
                                "cluster_resources": {
                                    "where": {
                                        "cluster": "CEP4",
                                        "partition": "cpu"
                                    },
                                    "cores_per_task": 2,
                                    "parallel_tasks": 122
                                }
                            },
                            "specifications_template": "http://localhost:3000/api/subtask_template/7",
                            "specifications_template_id": 7,
                            "state": "http://localhost:3000/api/subtask_state/defined",
                            "state_value": "defined",
                            "subtask_type": "pipeline",
                            "tags": [],
                            "task_blueprint": "http://localhost:3000/api/task_blueprint/4",
                            "task_blueprint_id": 4,
                            "total_nr_of_dataproducts": 0,
                            "total_size_of_dataproducts": 0,
                            "total_size_of_dataproducts_on_filesysem": 0,
                            "unschedulable_reason": null,
                            "updated_at": "2022-04-14T06:21:07.436107"
                        }
                    ],
                    "subtasks_ids": [
                        2000005
                    ],
                    "tags": [],
                    "task_type": "pipeline",
                    "unschedulable_reason": "",
                    "updated_at": "2022-04-14T06:20:19.813116"
                },
                {
                    "id": 2,
                    "url": "http://localhost:3000/api/task_blueprint/2",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "consumed_by": [
                        "http://localhost:3000/api/task_relation_blueprint/7"
                    ],
                    "consumed_by_ids": [
                        7
                    ],
                    "created_at": "2022-04-14T06:20:19.736118",
                    "description": "Preprocessing Pipeline for Calibrator Observation",
                    "draft": "http://localhost:3000/api/task_draft/2",
                    "draft_id": 2,
                    "duration": 120,
                    "error_reason": "",
                    "first_scheduling_relation": [],
                    "first_scheduling_relation_ids": [],
                    "name": "Calibrator Pipeline",
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "output_pinned": false,
                    "process_start_time": "2022-04-14T06:27:19.054619",
                    "process_stop_time": "2022-04-14T06:29:19.054619",
                    "produced_by": [
                        "http://localhost:3000/api/task_relation_blueprint/1"
                    ],
                    "produced_by_ids": [
                        1
                    ],
                    "relative_start_time": 0,
                    "relative_stop_time": 0,
                    "scheduled_start_time": "2022-04-14T06:27:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                    "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
                    "scheduling_unit_blueprint_id": 1,
                    "second_scheduling_relation": [],
                    "second_scheduling_relation_ids": [],
                    "short_description": "3Cabc/1.0/CP",
                    "specifications_doc": {
                        "flag": {
                            "rfi_strategy": "LBAdefault",
                            "outerchannels": true,
                            "autocorrelations": true
                        },
                        "demix": {
                            "sources": [],
                            "time_steps": 10,
                            "ignore_target": false,
                            "frequency_steps": 64
                        },
                        "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "average": {
                            "time_steps": 1,
                            "frequency_steps": 1
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/task_template/10",
                    "specifications_template_id": 10,
                    "status": "schedulable",
                    "status_value": "schedulable",
                    "subtasks": [
                        {
                            "id": 2000006,
                            "url": "http://localhost:3000/api/subtask/2000006",
                            "actual_on_sky_start_time": null,
                            "actual_on_sky_stop_time": null,
                            "actual_process_start_time": null,
                            "actual_process_stop_time": null,
                            "cluster": "http://localhost:3000/api/cluster/1",
                            "cluster_id": 1,
                            "cluster_name": "CEP4",
                            "created_at": "2022-04-14T06:20:21.031702",
                            "created_or_updated_by_user": null,
                            "created_or_updated_by_user_id": null,
                            "duration": 120,
                            "error_reason": null,
                            "global_identifier": "http://localhost:3000/api/sip_identifier/14",
                            "global_identifier_id": 14,
                            "global_parset_identifier": "http://localhost:3000/api/sip_identifier/15",
                            "global_parset_identifier_id": 15,
                            "inputs": [
                                "http://localhost:3000/api/subtask_input/6"
                            ],
                            "inputs_ids": [
                                6
                            ],
                            "obsolete_since": null,
                            "on_sky_duration": null,
                            "on_sky_start_time": null,
                            "on_sky_stop_time": null,
                            "outputs": [
                                "http://localhost:3000/api/subtask_output/7"
                            ],
                            "outputs_ids": [
                                7
                            ],
                            "primary": true,
                            "process_start_time": "2022-04-14T06:27:19.054619",
                            "process_stop_time": "2022-04-14T06:29:19.054619",
                            "processing_feedback_doc": null,
                            "processing_feedback_template": null,
                            "processing_feedback_template_id": null,
                            "scheduled_start_time": "2022-04-14T06:27:19.054619",
                            "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                            "specifications_doc": {
                                "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                                "demixer": {
                                    "enabled": true,
                                    "baselines": "CS*,RS*&",
                                    "time_steps": 1,
                                    "demix_always": [],
                                    "ignore_target": false,
                                    "demix_if_needed": [],
                                    "frequency_steps": 1,
                                    "demix_time_steps": 10,
                                    "demix_frequency_steps": 64
                                },
                                "aoflagger": {
                                    "enabled": true,
                                    "strategy": "LBAdefault"
                                },
                                "preflagger0": {
                                    "enabled": true,
                                    "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                                },
                                "preflagger1": {
                                    "enabled": true,
                                    "corrtype": "auto"
                                },
                                "storagemanager": "dysco",
                                "cluster_resources": {
                                    "where": {
                                        "cluster": "CEP4",
                                        "partition": "cpu"
                                    },
                                    "cores_per_task": 2,
                                    "parallel_tasks": 122
                                }
                            },
                            "specifications_template": "http://localhost:3000/api/subtask_template/7",
                            "specifications_template_id": 7,
                            "state": "http://localhost:3000/api/subtask_state/defined",
                            "state_value": "defined",
                            "subtask_type": "pipeline",
                            "tags": [],
                            "task_blueprint": "http://localhost:3000/api/task_blueprint/2",
                            "task_blueprint_id": 2,
                            "total_nr_of_dataproducts": 0,
                            "total_size_of_dataproducts": 0,
                            "total_size_of_dataproducts_on_filesysem": 0,
                            "unschedulable_reason": null,
                            "updated_at": "2022-04-14T06:21:07.392191"
                        }
                    ],
                    "subtasks_ids": [
                        2000006
                    ],
                    "tags": [],
                    "task_type": "pipeline",
                    "unschedulable_reason": "",
                    "updated_at": "2022-04-14T06:20:19.736130"
                },
                {
                    "id": 7,
                    "url": "http://localhost:3000/api/task_blueprint/7",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "consumed_by": [
                        "http://localhost:3000/api/task_relation_blueprint/8"
                    ],
                    "consumed_by_ids": [
                        8
                    ],
                    "created_at": "2022-04-14T06:20:19.939457",
                    "description": "Preprocessing Pipeline for Target Observation target1",
                    "draft": "http://localhost:3000/api/task_draft/7",
                    "draft_id": 7,
                    "duration": 120,
                    "error_reason": "",
                    "first_scheduling_relation": [],
                    "first_scheduling_relation_ids": [],
                    "name": "Pipeline target1",
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "output_pinned": false,
                    "process_start_time": "2022-04-14T06:27:19.054619",
                    "process_stop_time": "2022-04-14T06:29:19.054619",
                    "produced_by": [
                        "http://localhost:3000/api/task_relation_blueprint/6"
                    ],
                    "produced_by_ids": [
                        6
                    ],
                    "relative_start_time": 0,
                    "relative_stop_time": 0,
                    "scheduled_start_time": "2022-04-14T06:27:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                    "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
                    "scheduling_unit_blueprint_id": 1,
                    "second_scheduling_relation": [],
                    "second_scheduling_relation_ids": [],
                    "short_description": "Paaa+01/1.0/PP",
                    "specifications_doc": {
                        "flag": {
                            "rfi_strategy": "LBAdefault",
                            "outerchannels": true,
                            "autocorrelations": true
                        },
                        "demix": {
                            "sources": [],
                            "time_steps": 10,
                            "ignore_target": false,
                            "frequency_steps": 64
                        },
                        "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "average": {
                            "time_steps": 1,
                            "frequency_steps": 1
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/task_template/10",
                    "specifications_template_id": 10,
                    "status": "schedulable",
                    "status_value": "schedulable",
                    "subtasks": [
                        {
                            "id": 2000007,
                            "url": "http://localhost:3000/api/subtask/2000007",
                            "actual_on_sky_start_time": null,
                            "actual_on_sky_stop_time": null,
                            "actual_process_start_time": null,
                            "actual_process_stop_time": null,
                            "cluster": "http://localhost:3000/api/cluster/1",
                            "cluster_id": 1,
                            "cluster_name": "CEP4",
                            "created_at": "2022-04-14T06:20:21.134705",
                            "created_or_updated_by_user": null,
                            "created_or_updated_by_user_id": null,
                            "duration": 120,
                            "error_reason": null,
                            "global_identifier": "http://localhost:3000/api/sip_identifier/16",
                            "global_identifier_id": 16,
                            "global_parset_identifier": "http://localhost:3000/api/sip_identifier/17",
                            "global_parset_identifier_id": 17,
                            "inputs": [
                                "http://localhost:3000/api/subtask_input/7"
                            ],
                            "inputs_ids": [
                                7
                            ],
                            "obsolete_since": null,
                            "on_sky_duration": null,
                            "on_sky_start_time": null,
                            "on_sky_stop_time": null,
                            "outputs": [
                                "http://localhost:3000/api/subtask_output/8"
                            ],
                            "outputs_ids": [
                                8
                            ],
                            "primary": true,
                            "process_start_time": "2022-04-14T06:27:19.054619",
                            "process_stop_time": "2022-04-14T06:29:19.054619",
                            "processing_feedback_doc": null,
                            "processing_feedback_template": null,
                            "processing_feedback_template_id": null,
                            "scheduled_start_time": "2022-04-14T06:27:19.054619",
                            "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                            "specifications_doc": {
                                "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                                "demixer": {
                                    "enabled": true,
                                    "baselines": "CS*,RS*&",
                                    "time_steps": 1,
                                    "demix_always": [],
                                    "ignore_target": false,
                                    "demix_if_needed": [],
                                    "frequency_steps": 1,
                                    "demix_time_steps": 10,
                                    "demix_frequency_steps": 64
                                },
                                "aoflagger": {
                                    "enabled": true,
                                    "strategy": "LBAdefault"
                                },
                                "preflagger0": {
                                    "enabled": true,
                                    "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                                },
                                "preflagger1": {
                                    "enabled": true,
                                    "corrtype": "auto"
                                },
                                "storagemanager": "dysco",
                                "cluster_resources": {
                                    "where": {
                                        "cluster": "CEP4",
                                        "partition": "cpu"
                                    },
                                    "cores_per_task": 2,
                                    "parallel_tasks": 122
                                }
                            },
                            "specifications_template": "http://localhost:3000/api/subtask_template/7",
                            "specifications_template_id": 7,
                            "state": "http://localhost:3000/api/subtask_state/defined",
                            "state_value": "defined",
                            "subtask_type": "pipeline",
                            "tags": [],
                            "task_blueprint": "http://localhost:3000/api/task_blueprint/7",
                            "task_blueprint_id": 7,
                            "total_nr_of_dataproducts": 0,
                            "total_size_of_dataproducts": 0,
                            "total_size_of_dataproducts_on_filesysem": 0,
                            "unschedulable_reason": null,
                            "updated_at": "2022-04-14T06:21:07.261428"
                        }
                    ],
                    "subtasks_ids": [
                        2000007
                    ],
                    "tags": [],
                    "task_type": "pipeline",
                    "unschedulable_reason": "",
                    "updated_at": "2022-04-14T06:20:19.939486"
                },
                {
                    "id": 6,
                    "url": "http://localhost:3000/api/task_blueprint/6",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "consumed_by": [
                        "http://localhost:3000/api/task_relation_blueprint/9"
                    ],
                    "consumed_by_ids": [
                        9
                    ],
                    "created_at": "2022-04-14T06:20:19.899035",
                    "description": "Preprocessing Pipeline for Target Observation target2",
                    "draft": "http://localhost:3000/api/task_draft/6",
                    "draft_id": 6,
                    "duration": 120,
                    "error_reason": "",
                    "first_scheduling_relation": [],
                    "first_scheduling_relation_ids": [],
                    "name": "Pipeline target2",
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "output_pinned": false,
                    "process_start_time": "2022-04-14T06:27:19.054619",
                    "process_stop_time": "2022-04-14T06:29:19.054619",
                    "produced_by": [
                        "http://localhost:3000/api/task_relation_blueprint/5"
                    ],
                    "produced_by_ids": [
                        5
                    ],
                    "relative_start_time": 0,
                    "relative_stop_time": 0,
                    "scheduled_start_time": "2022-04-14T06:27:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                    "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
                    "scheduling_unit_blueprint_id": 1,
                    "second_scheduling_relation": [],
                    "second_scheduling_relation_ids": [],
                    "short_description": "Paaa+02/1.1/PP",
                    "specifications_doc": {
                        "flag": {
                            "rfi_strategy": "LBAdefault",
                            "outerchannels": true,
                            "autocorrelations": true
                        },
                        "demix": {
                            "sources": [],
                            "time_steps": 10,
                            "ignore_target": false,
                            "frequency_steps": 64
                        },
                        "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "average": {
                            "time_steps": 1,
                            "frequency_steps": 1
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/task_template/10",
                    "specifications_template_id": 10,
                    "status": "schedulable",
                    "status_value": "schedulable",
                    "subtasks": [
                        {
                            "id": 2000008,
                            "url": "http://localhost:3000/api/subtask/2000008",
                            "actual_on_sky_start_time": null,
                            "actual_on_sky_stop_time": null,
                            "actual_process_start_time": null,
                            "actual_process_stop_time": null,
                            "cluster": "http://localhost:3000/api/cluster/1",
                            "cluster_id": 1,
                            "cluster_name": "CEP4",
                            "created_at": "2022-04-14T06:20:21.238072",
                            "created_or_updated_by_user": null,
                            "created_or_updated_by_user_id": null,
                            "duration": 120,
                            "error_reason": null,
                            "global_identifier": "http://localhost:3000/api/sip_identifier/18",
                            "global_identifier_id": 18,
                            "global_parset_identifier": "http://localhost:3000/api/sip_identifier/19",
                            "global_parset_identifier_id": 19,
                            "inputs": [
                                "http://localhost:3000/api/subtask_input/8"
                            ],
                            "inputs_ids": [
                                8
                            ],
                            "obsolete_since": null,
                            "on_sky_duration": null,
                            "on_sky_start_time": null,
                            "on_sky_stop_time": null,
                            "outputs": [
                                "http://localhost:3000/api/subtask_output/9"
                            ],
                            "outputs_ids": [
                                9
                            ],
                            "primary": true,
                            "process_start_time": "2022-04-14T06:27:19.054619",
                            "process_stop_time": "2022-04-14T06:29:19.054619",
                            "processing_feedback_doc": null,
                            "processing_feedback_template": null,
                            "processing_feedback_template_id": null,
                            "scheduled_start_time": "2022-04-14T06:27:19.054619",
                            "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                            "specifications_doc": {
                                "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                                "demixer": {
                                    "enabled": true,
                                    "baselines": "CS*,RS*&",
                                    "time_steps": 1,
                                    "demix_always": [],
                                    "ignore_target": false,
                                    "demix_if_needed": [],
                                    "frequency_steps": 1,
                                    "demix_time_steps": 10,
                                    "demix_frequency_steps": 64
                                },
                                "aoflagger": {
                                    "enabled": true,
                                    "strategy": "LBAdefault"
                                },
                                "preflagger0": {
                                    "enabled": true,
                                    "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                                },
                                "preflagger1": {
                                    "enabled": true,
                                    "corrtype": "auto"
                                },
                                "storagemanager": "dysco",
                                "cluster_resources": {
                                    "where": {
                                        "cluster": "CEP4",
                                        "partition": "cpu"
                                    },
                                    "cores_per_task": 2,
                                    "parallel_tasks": 122
                                }
                            },
                            "specifications_template": "http://localhost:3000/api/subtask_template/7",
                            "specifications_template_id": 7,
                            "state": "http://localhost:3000/api/subtask_state/defined",
                            "state_value": "defined",
                            "subtask_type": "pipeline",
                            "tags": [],
                            "task_blueprint": "http://localhost:3000/api/task_blueprint/6",
                            "task_blueprint_id": 6,
                            "total_nr_of_dataproducts": 0,
                            "total_size_of_dataproducts": 0,
                            "total_size_of_dataproducts_on_filesysem": 0,
                            "unschedulable_reason": null,
                            "updated_at": "2022-04-14T06:21:07.349438"
                        }
                    ],
                    "subtasks_ids": [
                        2000008
                    ],
                    "tags": [],
                    "task_type": "pipeline",
                    "unschedulable_reason": "",
                    "updated_at": "2022-04-14T06:20:19.899049"
                }
            ]
          }
      ],
      "subtasks_ids": [
          2000001,
          2000002,
          2000000
      ],
      "tags": [],
      "task_type": "observation",
      "unschedulable_reason": "",
      "updated_at": "2022-04-14T06:20:19.668472",
      "predecessors": [],
      "successors": [
        {
            "id": 3,
            "url": "http://localhost:3000/api/task_blueprint/3",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/12"
            ],
            "consumed_by_ids": [
                12
            ],
            "created_at": "2022-04-14T06:20:19.774241",
            "description": "Preprocessing Pipeline for Target Observation target5",
            "draft": "http://localhost:3000/api/task_draft/3",
            "draft_id": 3,
            "duration": 120,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target5",
            "obsolete_since": null,
            "on_sky_duration": null,
            "on_sky_start_time": null,
            "on_sky_stop_time": null,
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:27:19.054619",
            "process_stop_time": "2022-04-14T06:29:19.054619",
            "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/2"
            ],
            "produced_by_ids": [
                2
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduled_start_time": "2022-04-14T06:27:19.054619",
            "scheduled_stop_time": "2022-04-14T06:29:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
            "scheduling_unit_blueprint_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "Paaa+05/1.4/PP",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 10,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 1,
                    "frequency_steps": 1
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "status": "schedulable",
            "status_value": "schedulable",
            "subtasks": [
                {
                    "id": 2000003,
                    "url": "http://localhost:3000/api/subtask/2000003",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "cluster": "http://localhost:3000/api/cluster/1",
                    "cluster_id": 1,
                    "cluster_name": "CEP4",
                    "created_at": "2022-04-14T06:20:20.722939",
                    "created_or_updated_by_user": null,
                    "created_or_updated_by_user_id": null,
                    "duration": 120,
                    "error_reason": null,
                    "global_identifier": "http://localhost:3000/api/sip_identifier/8",
                    "global_identifier_id": 8,
                    "global_parset_identifier": "http://localhost:3000/api/sip_identifier/9",
                    "global_parset_identifier_id": 9,
                    "inputs": [
                        "http://localhost:3000/api/subtask_input/3"
                    ],
                    "inputs_ids": [
                        3
                    ],
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "outputs": [
                        "http://localhost:3000/api/subtask_output/4"
                    ],
                    "outputs_ids": [
                        4
                    ],
                    "primary": true,
                    "process_start_time": "2022-04-14T06:27:19.054619",
                    "process_stop_time": "2022-04-14T06:29:19.054619",
                    "processing_feedback_doc": null,
                    "processing_feedback_template": null,
                    "processing_feedback_template_id": null,
                    "scheduled_start_time": "2022-04-14T06:27:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                    "specifications_doc": {
                        "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "demixer": {
                            "enabled": true,
                            "baselines": "CS*,RS*&",
                            "time_steps": 1,
                            "demix_always": [],
                            "ignore_target": false,
                            "demix_if_needed": [],
                            "frequency_steps": 1,
                            "demix_time_steps": 10,
                            "demix_frequency_steps": 64
                        },
                        "aoflagger": {
                            "enabled": true,
                            "strategy": "LBAdefault"
                        },
                        "preflagger0": {
                            "enabled": true,
                            "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                        },
                        "preflagger1": {
                            "enabled": true,
                            "corrtype": "auto"
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/subtask_template/7",
                    "specifications_template_id": 7,
                    "state": "http://localhost:3000/api/subtask_state/defined",
                    "state_value": "defined",
                    "subtask_type": "pipeline",
                    "tags": [],
                    "task_blueprint": "http://localhost:3000/api/task_blueprint/3",
                    "task_blueprint_id": 3,
                    "total_nr_of_dataproducts": 0,
                    "total_size_of_dataproducts": 0,
                    "total_size_of_dataproducts_on_filesysem": 0,
                    "unschedulable_reason": null,
                    "updated_at": "2022-04-14T06:21:07.482780"
                }
            ],
            "subtasks_ids": [
                2000003
            ],
            "tags": [],
            "task_type": "pipeline",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:19.774253"
        },
        {
            "id": 5,
            "url": "http://localhost:3000/api/task_blueprint/5",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/10"
            ],
            "consumed_by_ids": [
                10
            ],
            "created_at": "2022-04-14T06:20:19.856020",
            "description": "Preprocessing Pipeline for Target Observation target3",
            "draft": "http://localhost:3000/api/task_draft/5",
            "draft_id": 5,
            "duration": 120,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target3",
            "obsolete_since": null,
            "on_sky_duration": null,
            "on_sky_start_time": null,
            "on_sky_stop_time": null,
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:27:19.054619",
            "process_stop_time": "2022-04-14T06:29:19.054619",
            "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/4"
            ],
            "produced_by_ids": [
                4
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduled_start_time": "2022-04-14T06:27:19.054619",
            "scheduled_stop_time": "2022-04-14T06:29:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
            "scheduling_unit_blueprint_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "Paaa+03/1.2/PP",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 10,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 1,
                    "frequency_steps": 1
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "status": "schedulable",
            "status_value": "schedulable",
            "subtasks": [
                {
                    "id": 2000004,
                    "url": "http://localhost:3000/api/subtask/2000004",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "cluster": "http://localhost:3000/api/cluster/1",
                    "cluster_id": 1,
                    "cluster_name": "CEP4",
                    "created_at": "2022-04-14T06:20:20.825054",
                    "created_or_updated_by_user": null,
                    "created_or_updated_by_user_id": null,
                    "duration": 120,
                    "error_reason": null,
                    "global_identifier": "http://localhost:3000/api/sip_identifier/10",
                    "global_identifier_id": 10,
                    "global_parset_identifier": "http://localhost:3000/api/sip_identifier/11",
                    "global_parset_identifier_id": 11,
                    "inputs": [
                        "http://localhost:3000/api/subtask_input/4"
                    ],
                    "inputs_ids": [
                        4
                    ],
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "outputs": [
                        "http://localhost:3000/api/subtask_output/5"
                    ],
                    "outputs_ids": [
                        5
                    ],
                    "primary": true,
                    "process_start_time": "2022-04-14T06:27:19.054619",
                    "process_stop_time": "2022-04-14T06:29:19.054619",
                    "processing_feedback_doc": null,
                    "processing_feedback_template": null,
                    "processing_feedback_template_id": null,
                    "scheduled_start_time": "2022-04-14T06:27:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                    "specifications_doc": {
                        "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "demixer": {
                            "enabled": true,
                            "baselines": "CS*,RS*&",
                            "time_steps": 1,
                            "demix_always": [],
                            "ignore_target": false,
                            "demix_if_needed": [],
                            "frequency_steps": 1,
                            "demix_time_steps": 10,
                            "demix_frequency_steps": 64
                        },
                        "aoflagger": {
                            "enabled": true,
                            "strategy": "LBAdefault"
                        },
                        "preflagger0": {
                            "enabled": true,
                            "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                        },
                        "preflagger1": {
                            "enabled": true,
                            "corrtype": "auto"
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/subtask_template/7",
                    "specifications_template_id": 7,
                    "state": "http://localhost:3000/api/subtask_state/defined",
                    "state_value": "defined",
                    "subtask_type": "pipeline",
                    "tags": [],
                    "task_blueprint": "http://localhost:3000/api/task_blueprint/5",
                    "task_blueprint_id": 5,
                    "total_nr_of_dataproducts": 0,
                    "total_size_of_dataproducts": 0,
                    "total_size_of_dataproducts_on_filesysem": 0,
                    "unschedulable_reason": null,
                    "updated_at": "2022-04-14T06:21:07.306954"
                }
            ],
            "subtasks_ids": [
                2000004
            ],
            "tags": [],
            "task_type": "pipeline",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:19.856102"
        },
        {
            "id": 4,
            "url": "http://localhost:3000/api/task_blueprint/4",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/11"
            ],
            "consumed_by_ids": [
                11
            ],
            "created_at": "2022-04-14T06:20:19.813073",
            "description": "Preprocessing Pipeline for Target Observation target4",
            "draft": "http://localhost:3000/api/task_draft/4",
            "draft_id": 4,
            "duration": 120,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target4",
            "obsolete_since": null,
            "on_sky_duration": null,
            "on_sky_start_time": null,
            "on_sky_stop_time": null,
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:27:19.054619",
            "process_stop_time": "2022-04-14T06:29:19.054619",
            "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/3"
            ],
            "produced_by_ids": [
                3
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduled_start_time": "2022-04-14T06:27:19.054619",
            "scheduled_stop_time": "2022-04-14T06:29:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
            "scheduling_unit_blueprint_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "Paaa+04/1.3/PP",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 10,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 1,
                    "frequency_steps": 1
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "status": "schedulable",
            "status_value": "schedulable",
            "subtasks": [
                {
                    "id": 2000005,
                    "url": "http://localhost:3000/api/subtask/2000005",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "cluster": "http://localhost:3000/api/cluster/1",
                    "cluster_id": 1,
                    "cluster_name": "CEP4",
                    "created_at": "2022-04-14T06:20:20.929183",
                    "created_or_updated_by_user": null,
                    "created_or_updated_by_user_id": null,
                    "duration": 120,
                    "error_reason": null,
                    "global_identifier": "http://localhost:3000/api/sip_identifier/12",
                    "global_identifier_id": 12,
                    "global_parset_identifier": "http://localhost:3000/api/sip_identifier/13",
                    "global_parset_identifier_id": 13,
                    "inputs": [
                        "http://localhost:3000/api/subtask_input/5"
                    ],
                    "inputs_ids": [
                        5
                    ],
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "outputs": [
                        "http://localhost:3000/api/subtask_output/6"
                    ],
                    "outputs_ids": [
                        6
                    ],
                    "primary": true,
                    "process_start_time": "2022-04-14T06:27:19.054619",
                    "process_stop_time": "2022-04-14T06:29:19.054619",
                    "processing_feedback_doc": null,
                    "processing_feedback_template": null,
                    "processing_feedback_template_id": null,
                    "scheduled_start_time": "2022-04-14T06:27:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                    "specifications_doc": {
                        "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "demixer": {
                            "enabled": true,
                            "baselines": "CS*,RS*&",
                            "time_steps": 1,
                            "demix_always": [],
                            "ignore_target": false,
                            "demix_if_needed": [],
                            "frequency_steps": 1,
                            "demix_time_steps": 10,
                            "demix_frequency_steps": 64
                        },
                        "aoflagger": {
                            "enabled": true,
                            "strategy": "LBAdefault"
                        },
                        "preflagger0": {
                            "enabled": true,
                            "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                        },
                        "preflagger1": {
                            "enabled": true,
                            "corrtype": "auto"
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/subtask_template/7",
                    "specifications_template_id": 7,
                    "state": "http://localhost:3000/api/subtask_state/defined",
                    "state_value": "defined",
                    "subtask_type": "pipeline",
                    "tags": [],
                    "task_blueprint": "http://localhost:3000/api/task_blueprint/4",
                    "task_blueprint_id": 4,
                    "total_nr_of_dataproducts": 0,
                    "total_size_of_dataproducts": 0,
                    "total_size_of_dataproducts_on_filesysem": 0,
                    "unschedulable_reason": null,
                    "updated_at": "2022-04-14T06:21:07.436107"
                }
            ],
            "subtasks_ids": [
                2000005
            ],
            "tags": [],
            "task_type": "pipeline",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:19.813116"
        },
        {
            "id": 2,
            "url": "http://localhost:3000/api/task_blueprint/2",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/7"
            ],
            "consumed_by_ids": [
                7
            ],
            "created_at": "2022-04-14T06:20:19.736118",
            "description": "Preprocessing Pipeline for Calibrator Observation",
            "draft": "http://localhost:3000/api/task_draft/2",
            "draft_id": 2,
            "duration": 120,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Calibrator Pipeline",
            "obsolete_since": null,
            "on_sky_duration": null,
            "on_sky_start_time": null,
            "on_sky_stop_time": null,
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:27:19.054619",
            "process_stop_time": "2022-04-14T06:29:19.054619",
            "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/1"
            ],
            "produced_by_ids": [
                1
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduled_start_time": "2022-04-14T06:27:19.054619",
            "scheduled_stop_time": "2022-04-14T06:29:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
            "scheduling_unit_blueprint_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "3Cabc/1.0/CP",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 10,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 1,
                    "frequency_steps": 1
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "status": "schedulable",
            "status_value": "schedulable",
            "subtasks": [
                {
                    "id": 2000006,
                    "url": "http://localhost:3000/api/subtask/2000006",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "cluster": "http://localhost:3000/api/cluster/1",
                    "cluster_id": 1,
                    "cluster_name": "CEP4",
                    "created_at": "2022-04-14T06:20:21.031702",
                    "created_or_updated_by_user": null,
                    "created_or_updated_by_user_id": null,
                    "duration": 120,
                    "error_reason": null,
                    "global_identifier": "http://localhost:3000/api/sip_identifier/14",
                    "global_identifier_id": 14,
                    "global_parset_identifier": "http://localhost:3000/api/sip_identifier/15",
                    "global_parset_identifier_id": 15,
                    "inputs": [
                        "http://localhost:3000/api/subtask_input/6"
                    ],
                    "inputs_ids": [
                        6
                    ],
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "outputs": [
                        "http://localhost:3000/api/subtask_output/7"
                    ],
                    "outputs_ids": [
                        7
                    ],
                    "primary": true,
                    "process_start_time": "2022-04-14T06:27:19.054619",
                    "process_stop_time": "2022-04-14T06:29:19.054619",
                    "processing_feedback_doc": null,
                    "processing_feedback_template": null,
                    "processing_feedback_template_id": null,
                    "scheduled_start_time": "2022-04-14T06:27:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                    "specifications_doc": {
                        "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "demixer": {
                            "enabled": true,
                            "baselines": "CS*,RS*&",
                            "time_steps": 1,
                            "demix_always": [],
                            "ignore_target": false,
                            "demix_if_needed": [],
                            "frequency_steps": 1,
                            "demix_time_steps": 10,
                            "demix_frequency_steps": 64
                        },
                        "aoflagger": {
                            "enabled": true,
                            "strategy": "LBAdefault"
                        },
                        "preflagger0": {
                            "enabled": true,
                            "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                        },
                        "preflagger1": {
                            "enabled": true,
                            "corrtype": "auto"
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/subtask_template/7",
                    "specifications_template_id": 7,
                    "state": "http://localhost:3000/api/subtask_state/defined",
                    "state_value": "defined",
                    "subtask_type": "pipeline",
                    "tags": [],
                    "task_blueprint": "http://localhost:3000/api/task_blueprint/2",
                    "task_blueprint_id": 2,
                    "total_nr_of_dataproducts": 0,
                    "total_size_of_dataproducts": 0,
                    "total_size_of_dataproducts_on_filesysem": 0,
                    "unschedulable_reason": null,
                    "updated_at": "2022-04-14T06:21:07.392191"
                }
            ],
            "subtasks_ids": [
                2000006
            ],
            "tags": [],
            "task_type": "pipeline",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:19.736130"
        },
        {
            "id": 7,
            "url": "http://localhost:3000/api/task_blueprint/7",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/8"
            ],
            "consumed_by_ids": [
                8
            ],
            "created_at": "2022-04-14T06:20:19.939457",
            "description": "Preprocessing Pipeline for Target Observation target1",
            "draft": "http://localhost:3000/api/task_draft/7",
            "draft_id": 7,
            "duration": 120,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target1",
            "obsolete_since": null,
            "on_sky_duration": null,
            "on_sky_start_time": null,
            "on_sky_stop_time": null,
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:27:19.054619",
            "process_stop_time": "2022-04-14T06:29:19.054619",
            "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/6"
            ],
            "produced_by_ids": [
                6
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduled_start_time": "2022-04-14T06:27:19.054619",
            "scheduled_stop_time": "2022-04-14T06:29:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
            "scheduling_unit_blueprint_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "Paaa+01/1.0/PP",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 10,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 1,
                    "frequency_steps": 1
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "status": "schedulable",
            "status_value": "schedulable",
            "subtasks": [
                {
                    "id": 2000007,
                    "url": "http://localhost:3000/api/subtask/2000007",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "cluster": "http://localhost:3000/api/cluster/1",
                    "cluster_id": 1,
                    "cluster_name": "CEP4",
                    "created_at": "2022-04-14T06:20:21.134705",
                    "created_or_updated_by_user": null,
                    "created_or_updated_by_user_id": null,
                    "duration": 120,
                    "error_reason": null,
                    "global_identifier": "http://localhost:3000/api/sip_identifier/16",
                    "global_identifier_id": 16,
                    "global_parset_identifier": "http://localhost:3000/api/sip_identifier/17",
                    "global_parset_identifier_id": 17,
                    "inputs": [
                        "http://localhost:3000/api/subtask_input/7"
                    ],
                    "inputs_ids": [
                        7
                    ],
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "outputs": [
                        "http://localhost:3000/api/subtask_output/8"
                    ],
                    "outputs_ids": [
                        8
                    ],
                    "primary": true,
                    "process_start_time": "2022-04-14T06:27:19.054619",
                    "process_stop_time": "2022-04-14T06:29:19.054619",
                    "processing_feedback_doc": null,
                    "processing_feedback_template": null,
                    "processing_feedback_template_id": null,
                    "scheduled_start_time": "2022-04-14T06:27:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                    "specifications_doc": {
                        "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "demixer": {
                            "enabled": true,
                            "baselines": "CS*,RS*&",
                            "time_steps": 1,
                            "demix_always": [],
                            "ignore_target": false,
                            "demix_if_needed": [],
                            "frequency_steps": 1,
                            "demix_time_steps": 10,
                            "demix_frequency_steps": 64
                        },
                        "aoflagger": {
                            "enabled": true,
                            "strategy": "LBAdefault"
                        },
                        "preflagger0": {
                            "enabled": true,
                            "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                        },
                        "preflagger1": {
                            "enabled": true,
                            "corrtype": "auto"
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/subtask_template/7",
                    "specifications_template_id": 7,
                    "state": "http://localhost:3000/api/subtask_state/defined",
                    "state_value": "defined",
                    "subtask_type": "pipeline",
                    "tags": [],
                    "task_blueprint": "http://localhost:3000/api/task_blueprint/7",
                    "task_blueprint_id": 7,
                    "total_nr_of_dataproducts": 0,
                    "total_size_of_dataproducts": 0,
                    "total_size_of_dataproducts_on_filesysem": 0,
                    "unschedulable_reason": null,
                    "updated_at": "2022-04-14T06:21:07.261428"
                }
            ],
            "subtasks_ids": [
                2000007
            ],
            "tags": [],
            "task_type": "pipeline",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:19.939486"
        },
        {
            "id": 6,
            "url": "http://localhost:3000/api/task_blueprint/6",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/9"
            ],
            "consumed_by_ids": [
                9
            ],
            "created_at": "2022-04-14T06:20:19.899035",
            "description": "Preprocessing Pipeline for Target Observation target2",
            "draft": "http://localhost:3000/api/task_draft/6",
            "draft_id": 6,
            "duration": 120,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target2",
            "obsolete_since": null,
            "on_sky_duration": null,
            "on_sky_start_time": null,
            "on_sky_stop_time": null,
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:27:19.054619",
            "process_stop_time": "2022-04-14T06:29:19.054619",
            "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/5"
            ],
            "produced_by_ids": [
                5
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduled_start_time": "2022-04-14T06:27:19.054619",
            "scheduled_stop_time": "2022-04-14T06:29:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
            "scheduling_unit_blueprint_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "Paaa+02/1.1/PP",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 10,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 1,
                    "frequency_steps": 1
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "status": "schedulable",
            "status_value": "schedulable",
            "subtasks": [
                {
                    "id": 2000008,
                    "url": "http://localhost:3000/api/subtask/2000008",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "cluster": "http://localhost:3000/api/cluster/1",
                    "cluster_id": 1,
                    "cluster_name": "CEP4",
                    "created_at": "2022-04-14T06:20:21.238072",
                    "created_or_updated_by_user": null,
                    "created_or_updated_by_user_id": null,
                    "duration": 120,
                    "error_reason": null,
                    "global_identifier": "http://localhost:3000/api/sip_identifier/18",
                    "global_identifier_id": 18,
                    "global_parset_identifier": "http://localhost:3000/api/sip_identifier/19",
                    "global_parset_identifier_id": 19,
                    "inputs": [
                        "http://localhost:3000/api/subtask_input/8"
                    ],
                    "inputs_ids": [
                        8
                    ],
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "outputs": [
                        "http://localhost:3000/api/subtask_output/9"
                    ],
                    "outputs_ids": [
                        9
                    ],
                    "primary": true,
                    "process_start_time": "2022-04-14T06:27:19.054619",
                    "process_stop_time": "2022-04-14T06:29:19.054619",
                    "processing_feedback_doc": null,
                    "processing_feedback_template": null,
                    "processing_feedback_template_id": null,
                    "scheduled_start_time": "2022-04-14T06:27:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:29:19.054619",
                    "specifications_doc": {
                        "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "demixer": {
                            "enabled": true,
                            "baselines": "CS*,RS*&",
                            "time_steps": 1,
                            "demix_always": [],
                            "ignore_target": false,
                            "demix_if_needed": [],
                            "frequency_steps": 1,
                            "demix_time_steps": 10,
                            "demix_frequency_steps": 64
                        },
                        "aoflagger": {
                            "enabled": true,
                            "strategy": "LBAdefault"
                        },
                        "preflagger0": {
                            "enabled": true,
                            "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                        },
                        "preflagger1": {
                            "enabled": true,
                            "corrtype": "auto"
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/subtask_template/7",
                    "specifications_template_id": 7,
                    "state": "http://localhost:3000/api/subtask_state/defined",
                    "state_value": "defined",
                    "subtask_type": "pipeline",
                    "tags": [],
                    "task_blueprint": "http://localhost:3000/api/task_blueprint/6",
                    "task_blueprint_id": 6,
                    "total_nr_of_dataproducts": 0,
                    "total_size_of_dataproducts": 0,
                    "total_size_of_dataproducts_on_filesysem": 0,
                    "unschedulable_reason": null,
                    "updated_at": "2022-04-14T06:21:07.349438"
                }
            ],
            "subtasks_ids": [
                2000008
            ],
            "tags": [],
            "task_type": "pipeline",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:19.899049"
        }
      ]
    },
    {
      "id": 2,
      "url": "http://localhost:3000/api/task_blueprint/2",
      "actual_on_sky_start_time": null,
      "actual_on_sky_stop_time": null,
      "actual_process_start_time": null,
      "actual_process_stop_time": null,
      "consumed_by": [
          "http://localhost:3000/api/task_relation_blueprint/7"
      ],
      "consumed_by_ids": [
          7
      ],
      "created_at": "2022-04-14T06:20:19.736118",
      "description": "Preprocessing Pipeline for Calibrator Observation",
      "draft": "http://localhost:3000/api/task_draft/2",
      "draft_id": 2,
      "draftObject": {
        "id": 2,
        "url": "http://localhost:3000/api/task_draft/2",
        "connector_types": [
            {
                "id": 13,
                "url": "http://localhost:3000/api/task_connector_type/13",
                "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                "dataformat_value": "MeasurementSet",
                "datatype": "http://localhost:3000/api/datatype/visibilities",
                "datatype_value": "visibilities",
                "iotype": "http://localhost:3000/api/iotype/output",
                "iotype_value": "output",
                "role": "http://localhost:3000/api/role/any",
                "role_value": "any",
                "task_template": "http://localhost:3000/api/task_template/10",
                "task_template_id": 10
            },
            {
                "id": 12,
                "url": "http://localhost:3000/api/task_connector_type/12",
                "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                "dataformat_value": "MeasurementSet",
                "datatype": "http://localhost:3000/api/datatype/visibilities",
                "datatype_value": "visibilities",
                "iotype": "http://localhost:3000/api/iotype/input",
                "iotype_value": "input",
                "role": "http://localhost:3000/api/role/any",
                "role_value": "any",
                "task_template": "http://localhost:3000/api/task_template/10",
                "task_template_id": 10
            }
        ],
        "consumed_by": [
            "http://localhost:3000/api/task_relation_draft/7"
        ],
        "consumed_by_ids": [
            7
        ],
        "created_at": "2022-04-14T06:20:19.318002",
        "description": "Preprocessing Pipeline for Calibrator Observation",
        "duration": 0,
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Calibrator Pipeline",
        "output_pinned": false,
        "produced_by": [
            "http://localhost:3000/api/task_relation_draft/1"
        ],
        "produced_by_ids": [
            1
        ],
        "relative_start_time": 0,
        "relative_stop_time": 0,
        "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
        "scheduling_unit_draft_id": 1,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "3Cabc/1.0/CP",
        "specifications_doc": {
            "flag": {
                "rfi_strategy": "LBAdefault",
                "outerchannels": true,
                "autocorrelations": true
            },
            "demix": {
                "sources": [],
                "time_steps": 10,
                "ignore_target": false,
                "frequency_steps": 64
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
            "average": {
                "time_steps": 1,
                "frequency_steps": 1
            },
            "storagemanager": "dysco",
            "cluster_resources": {
                "where": {
                    "cluster": "CEP4",
                    "partition": "cpu"
                },
                "cores_per_task": 2,
                "parallel_tasks": 122
            }
        },
        "specifications_template": "http://localhost:3000/api/task_template/10",
        "specifications_template_id": 10,
        "tags": [],
        "task_blueprints": [
            "http://localhost:3000/api/task_blueprint/2"
        ],
        "task_blueprints_ids": [
            2
        ],
        "task_type": "pipeline",
        "updated_at": "2022-04-14T06:20:19.318066"
      },
      "duration": 120,
      "error_reason": "",
      "first_scheduling_relation": [],
      "first_scheduling_relation_ids": [],
      "name": "Calibrator Pipeline",
      "obsolete_since": null,
      "on_sky_duration": null,
      "on_sky_start_time": null,
      "on_sky_stop_time": null,
      "output_pinned": false,
      "process_start_time": "2022-04-14T06:27:19.054619",
      "process_stop_time": "2022-04-14T06:29:19.054619",
      "produced_by": [
          "http://localhost:3000/api/task_relation_blueprint/1"
      ],
      "produced_by_ids": [
          1
      ],
      "relative_start_time": 0,
      "relative_stop_time": 0,
      "scheduled_start_time": "2022-04-14T06:27:19.054619",
      "scheduled_stop_time": "2022-04-14T06:29:19.054619",
      "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
      "scheduling_unit_blueprint_id": 1,
      "second_scheduling_relation": [],
      "second_scheduling_relation_ids": [],
      "short_description": "3Cabc/1.0/CP",
      "specifications_doc": {
          "flag": {
              "rfi_strategy": "LBAdefault",
              "outerchannels": true,
              "autocorrelations": true
          },
          "demix": {
              "sources": [],
              "time_steps": 10,
              "ignore_target": false,
              "frequency_steps": 64
          },
          "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
          "average": {
              "time_steps": 1,
              "frequency_steps": 1
          },
          "storagemanager": "dysco",
          "cluster_resources": {
              "where": {
                  "cluster": "CEP4",
                  "partition": "cpu"
              },
              "cores_per_task": 2,
              "parallel_tasks": 122
          }
      },
      "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "preprocessing pipeline"})).id,
      "specifications_template_id": (_.find(templates, {"name": "preprocessing pipeline"})).id,
      "status": "schedulable",
      "status_value": "schedulable",
      "subtasks": [
          {
              "id": 2000006,
              "url": "http://localhost:3000/api/subtask/2000006",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "cluster": "http://localhost:3000/api/cluster/1",
              "cluster_id": 1,
              "cluster_name": "CEP4",
              "created_at": "2022-04-14T06:20:21.031702",
              "created_or_updated_by_user": null,
              "created_or_updated_by_user_id": null,
              "duration": 120,
              "error_reason": null,
              "global_identifier": "http://localhost:3000/api/sip_identifier/14",
              "global_identifier_id": 14,
              "global_parset_identifier": "http://localhost:3000/api/sip_identifier/15",
              "global_parset_identifier_id": 15,
              "inputs": [
                  "http://localhost:3000/api/subtask_input/6"
              ],
              "inputs_ids": [
                  6
              ],
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "outputs": [
                  "http://localhost:3000/api/subtask_output/7"
              ],
              "outputs_ids": [
                  7
              ],
              "primary": true,
              "process_start_time": "2022-04-14T06:27:19.054619",
              "process_stop_time": "2022-04-14T06:29:19.054619",
              "processing_feedback_doc": null,
              "processing_feedback_template": null,
              "processing_feedback_template_id": null,
              "scheduled_start_time": "2022-04-14T06:27:19.054619",
              "scheduled_stop_time": "2022-04-14T06:29:19.054619",
              "specifications_doc": {
                  "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "demixer": {
                      "enabled": true,
                      "baselines": "CS*,RS*&",
                      "time_steps": 1,
                      "demix_always": [],
                      "ignore_target": false,
                      "demix_if_needed": [],
                      "frequency_steps": 1,
                      "demix_time_steps": 10,
                      "demix_frequency_steps": 64
                  },
                  "aoflagger": {
                      "enabled": true,
                      "strategy": "LBAdefault"
                  },
                  "preflagger0": {
                      "enabled": true,
                      "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                  },
                  "preflagger1": {
                      "enabled": true,
                      "corrtype": "auto"
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": "http://localhost:3000/api/subtask_template/7",
              "specifications_template_id": 7,
              "state": "http://localhost:3000/api/subtask_state/defined",
              "state_value": "defined",
              "subtask_type": "pipeline",
              "tags": [],
              "task_blueprint": "http://localhost:3000/api/task_blueprint/2",
              "task_blueprint_id": 2,
              "total_nr_of_dataproducts": 0,
              "total_size_of_dataproducts": 0,
              "total_size_of_dataproducts_on_filesysem": 0,
              "unschedulable_reason": null,
              "updated_at": "2022-04-14T06:21:07.392191"
          }
      ],
      "subtasks_ids": [
          2000006
      ],
      "tags": [],
      "task_type": "pipeline",
      "unschedulable_reason": "",
      "updated_at": "2022-04-14T06:20:19.736130",
      "predecessors": [
        {
            "id": 1,
            "url": "http://localhost:3000/api/task_blueprint/1",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/1",
                "http://localhost:3000/api/task_relation_blueprint/2",
                "http://localhost:3000/api/task_relation_blueprint/3",
                "http://localhost:3000/api/task_relation_blueprint/4",
                "http://localhost:3000/api/task_relation_blueprint/5",
                "http://localhost:3000/api/task_relation_blueprint/6"
            ],
            "consumed_by_ids": [
                1,
                2,
                3,
                4,
                5,
                6
            ],
            "created_at": "2022-04-14T06:20:19.668460",
            "description": "Combined parallel Calibrator & Target Observation for LBA with 5 target beams",
            "draft": "http://localhost:3000/api/task_draft/1",
            "draft_id": 1,
            "duration": 720,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Combined Observation",
            "obsolete_since": null,
            "on_sky_duration": 120,
            "on_sky_start_time": "2022-04-14T06:25:19.054619",
            "on_sky_stop_time": "2022-04-14T06:27:19.054619",
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:25:19.054619",
            "process_stop_time": "2022-04-14T06:37:19.054619",
            "produced_by": [],
            "produced_by_ids": [],
            "relative_start_time": 0,
            "relative_stop_time": 120,
            "scheduled_start_time": "2022-04-14T06:25:19.054619",
            "scheduled_stop_time": "2022-04-14T06:37:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
            "scheduling_unit_blueprint_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "oXXX Paaa+01 3Cabc",
            "specifications_doc": {
                "QA": {
                    "plots": {
                        "enabled": true,
                        "autocorrelation": true,
                        "crosscorrelation": true
                    },
                    "file_conversion": {
                        "enabled": true,
                        "nr_of_subbands": -1,
                        "nr_of_timestamps": 256
                    },
                    "inspection_plots": "msplots"
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                "duration": 120,
                "calibrator": {
                    "name": "calibrator",
                    "pointing": {
                        "angle1": 0.6624317181687094,
                        "angle2": 1.5579526427549426,
                        "target": "_calibrator_name_",
                        "direction_type": "J2000"
                    },
                    "autoselect": false
                },
                "correlator": {
                    "storage_cluster": "CEP4",
                    "integration_time": 1,
                    "channels_per_subband": 64,
                    "topocentric_frequency_correction": false
                },
                "station_configuration": {
                    "SAPs": [
                        {
                            "name": "target1",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+01",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target2",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+02",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target3",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+03",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target4",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+04",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target5",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+05",
                                "direction_type": "J2000"
                            }
                        }
                    ],
                    "filter": "LBA_10_90",
                    "tile_beam": {
                        "angle1": 0.6624317181687094,
                        "angle2": 1.5579526427549426,
                        "target": "_target_name_",
                        "direction_type": "J2000"
                    },
                    "antenna_set": "LBA_SPARSE_EVEN",
                    "station_groups": [
                        {
                            "stations": [
                                "CS001",
                                "CS002",
                                "CS003",
                                "CS004",
                                "CS005",
                                "CS006",
                                "CS007",
                                "CS011",
                                "CS013",
                                "CS017",
                                "CS021",
                                "CS024",
                                "CS026",
                                "CS028",
                                "CS030",
                                "CS031",
                                "CS032",
                                "CS101",
                                "CS103",
                                "CS201",
                                "CS301",
                                "CS302",
                                "CS401",
                                "CS501",
                                "RS106",
                                "RS205",
                                "RS208",
                                "RS210",
                                "RS305",
                                "RS306",
                                "RS307",
                                "RS310",
                                "RS406",
                                "RS407",
                                "RS409",
                                "RS503",
                                "RS508",
                                "RS509"
                            ],
                            "max_nr_missing": 4
                        },
                        {
                            "stations": [
                                "RS508",
                                "RS509"
                            ],
                            "max_nr_missing": 1
                        },
                        {
                            "stations": [
                                "RS310",
                                "RS210"
                            ],
                            "max_nr_missing": 0
                        }
                    ]
                }
            },
            "specifications_template": {
                "id": 5,
                "url": "http://localhost:3000/api/task_template/5",
                "created_at": "2022-04-14T06:20:07.273898",
                "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                "name": "parallel calibrator target observation",
                "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                "purpose_value": "technical_commissioning",
                "ref_resolved_schema": {
                    "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                    "type": "object",
                    "title": "parallel calibrator target observation",
                    "$schema": "http://json-schema.org/draft-06/schema#",
                    "version": 2,
                    "required": [
                        "calibrator",
                        "correlator",
                        "duration",
                        "station_configuration"
                    ],
                    "properties": {
                        "QA": {
                            "$ref": "#/definitions/QA",
                            "default": {}
                        },
                        "duration": {
                            "$id": "#duration",
                            "$ref": "#/definitions/timedelta",
                            "title": "Duration",
                            "default": 600,
                            "minimum": 1,
                            "description": "Duration of this observation (seconds)"
                        },
                        "calibrator": {
                            "$ref": "#/definitions/calibrator",
                            "default": {}
                        },
                        "correlator": {
                            "$ref": "#/definitions/correlator",
                            "default": {}
                        },
                        "station_configuration": {
                            "$ref": "#/definitions/station_configuration",
                            "default": {}
                        }
                    },
                    "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                    "patternProperties": {
                        "^[$]schema$": {}
                    },
                    "additionalProperties": false,
                    "definitions": {
                        "QA": {
                            "additionalProperties": false,
                            "default": {},
                            "description": "Perform all Quality Assurance (QA) tasks, including file conversion and plotting.",
                            "properties": {
                                "file_conversion": {
                                    "$ref": "#/definitions/file_conversion",
                                    "default": {}
                                },
                                "inspection_plots": {
                                    "$ref": "#/definitions/inspection_plots",
                                    "default": "msplots"
                                },
                                "plots": {
                                    "$ref": "#/definitions/plots",
                                    "default": {}
                                }
                            },
                            "required": [
                                "file_conversion",
                                "plots",
                                "inspection_plots"
                            ],
                            "title": "QA",
                            "type": "object"
                        },
                        "plots": {
                            "additionalProperties": false,
                            "default": {},
                            "description": "Create dynamic spectrum plots",
                            "properties": {
                                "autocorrelation": {
                                    "default": true,
                                    "description": "Create autocorrelation plots for all stations",
                                    "title": "autocorrelation",
                                    "type": "boolean"
                                },
                                "crosscorrelation": {
                                    "default": true,
                                    "description": "Create crosscorrelation plots for all baselines",
                                    "title": "crosscorrelation",
                                    "type": "boolean"
                                },
                                "enabled": {
                                    "default": true,
                                    "description": "Do/Don't create plots from the QA file from the observation",
                                    "title": "enabled",
                                    "type": "boolean"
                                }
                            },
                            "required": [
                                "enabled",
                                "autocorrelation",
                                "crosscorrelation"
                            ],
                            "title": "Plots",
                            "type": "object"
                        },
                        "file_conversion": {
                            "additionalProperties": false,
                            "default": {},
                            "description": "Create a QA file for the observation",
                            "properties": {
                                "enabled": {
                                    "default": true,
                                    "description": "Do/Don't create a QA file for the observation",
                                    "title": "enabled",
                                    "type": "boolean"
                                },
                                "nr_of_subbands": {
                                    "default": -1,
                                    "description": "Keep this number of subbands from the observation in the QA file, or all if -1",
                                    "title": "#subbands",
                                    "type": "integer"
                                },
                                "nr_of_timestamps": {
                                    "default": 256,
                                    "description": "Extract this number of timestamps from the observation in the QA file (equidistantanly sampled, no averaging/interpolation)",
                                    "minimum": 1,
                                    "title": "#timestamps",
                                    "type": "integer"
                                }
                            },
                            "required": [
                                "enabled",
                                "nr_of_subbands",
                                "nr_of_timestamps"
                            ],
                            "title": "File Conversion",
                            "type": "object"
                        },
                        "inspection_plots": {
                            "decription": "Type of inspection plots to run",
                            "default": "msplots",
                            "enum": [
                                "msplots",
                                "dynspec",
                                "none"
                            ],
                            "title": "Inspection Plots",
                            "type": "string"
                        },
                        "timedelta": {
                            "default": 0,
                            "description": "A time duration or delta expressed in seconds",
                            "type": "number"
                        },
                        "timestamp": {
                            "default": "1970-01-01T00:00:00Z",
                            "description": "A timestamp defined in UTC",
                            "format": "date-time",
                            "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                            "type": "string"
                        },
                        "calibrator": {
                            "additionalProperties": false,
                            "default": {},
                            "properties": {
                                "autoselect": {
                                    "default": true,
                                    "description": "Auto-select calibrator based on elevation",
                                    "title": "Auto-select",
                                    "type": "boolean"
                                },
                                "name": {
                                    "default": "calibrator",
                                    "description": "Name of the calibrator SAP",
                                    "title": "Name",
                                    "type": "string"
                                },
                                "pointing": {
                                    "$ref": "#/definitions/pointing",
                                    "default": {},
                                    "description": "Manually selected calibrator",
                                    "title": "Digital pointing"
                                }
                            },
                            "required": [
                                "autoselect",
                                "pointing",
                                "name"
                            ],
                            "type": "object"
                        },
                        "pointing": {
                            "additionalProperties": false,
                            "properties": {
                                "angle1": {
                                    "default": 0.6624317181687094,
                                    "description": "First angle (e.g. RA)",
                                    "title": "Angle 1",
                                    "type": "number"
                                },
                                "angle2": {
                                    "default": 1.5579526427549426,
                                    "description": "Second angle (e.g. DEC)",
                                    "title": "Angle 2",
                                    "type": "number"
                                },
                                "direction_type": {
                                    "default": "J2000",
                                    "description": "",
                                    "enum": [
                                        "J2000",
                                        "AZELGEO",
                                        "LMN",
                                        "SUN",
                                        "MOON",
                                        "MERCURY",
                                        "VENUS",
                                        "MARS",
                                        "JUPITER",
                                        "SATURN",
                                        "URANUS",
                                        "NEPTUNE",
                                        "PLUTO"
                                    ],
                                    "title": "Reference frame",
                                    "type": "string"
                                },
                                "target": {
                                    "default": "_target_name_",
                                    "description": "Description of where this beam points at",
                                    "minLength": 1,
                                    "title": "Target",
                                    "type": "string"
                                }
                            },
                            "required": [
                                "angle1",
                                "angle2",
                                "target"
                            ],
                            "type": "object"
                        },
                        "correlator": {
                            "additionalProperties": false,
                            "default": {},
                            "properties": {
                                "channels_per_subband": {
                                    "default": 64,
                                    "description": "Number of frequency bands per subband",
                                    "enum": [
                                        8,
                                        16,
                                        32,
                                        64,
                                        128,
                                        256,
                                        512,
                                        1024
                                    ],
                                    "minimum": 8,
                                    "title": "Channels/subband",
                                    "type": "integer"
                                },
                                "integration_time": {
                                    "$ref": "#/definitions/timedelta",
                                    "default": 1,
                                    "description": "Desired integration period (seconds)",
                                    "minimum": 0.1,
                                    "title": "Integration time"
                                },
                                "storage_cluster": {
                                    "default": "CEP4",
                                    "description": "Cluster to write output to",
                                    "enum": [
                                        "CEP4",
                                        "DragNet"
                                    ],
                                    "title": "Storage cluster",
                                    "type": "string"
                                },
                                "topocentric_frequency_correction": {
                                    "default": false,
                                    "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation.",
                                    "title": "Topocentric Frequency Correction",
                                    "type": "boolean"
                                }
                            }
                        },
                        "station_configuration": {
                            "default": {},
                            "properties": {
                                "SAPs": {
                                    "$ref": "#/definitions/SAPs",
                                    "default": [
                                        {}
                                    ],
                                    "minItems": 1
                                },
                                "antenna_set": {
                                    "$ref": "#/definitions/antenna_set",
                                    "default": "HBA_DUAL"
                                },
                                "filter": {
                                    "$ref": "#/definitions/filter",
                                    "default": "HBA_110_190"
                                },
                                "station_groups": {
                                    "$ref": "#/definitions/station_groups",
                                    "default": [
                                        {
                                            "max_nr_missing": 1,
                                            "stations": [
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007"
                                            ]
                                        }
                                    ]
                                },
                                "tile_beam": {
                                    "$ref": "#/definitions/pointing",
                                    "default": {},
                                    "description": "HBA only",
                                    "title": "Tile beam"
                                }
                            },
                            "required": [
                                "station_groups",
                                "antenna_set",
                                "filter",
                                "SAPs"
                            ],
                            "title": "station_configuration",
                            "type": "object"
                        },
                        "station": {
                            "description": "These are the LOFAR stations",
                            "enum": [
                                "CS001",
                                "CS002",
                                "CS003",
                                "CS004",
                                "CS005",
                                "CS006",
                                "CS007",
                                "CS011",
                                "CS013",
                                "CS017",
                                "CS021",
                                "CS024",
                                "CS026",
                                "CS028",
                                "CS030",
                                "CS031",
                                "CS032",
                                "CS101",
                                "CS103",
                                "CS201",
                                "CS301",
                                "CS302",
                                "CS401",
                                "CS501",
                                "RS106",
                                "RS205",
                                "RS208",
                                "RS210",
                                "RS305",
                                "RS306",
                                "RS307",
                                "RS310",
                                "RS406",
                                "RS407",
                                "RS409",
                                "RS503",
                                "RS508",
                                "RS509",
                                "DE601",
                                "DE602",
                                "DE603",
                                "DE604",
                                "DE605",
                                "FR606",
                                "SE607",
                                "UK608",
                                "DE609",
                                "PL610",
                                "PL611",
                                "PL612",
                                "IE613",
                                "LV614"
                            ],
                            "title": "Station",
                            "type": "string"
                        },
                        "station_groups": {
                            "additionalItems": false,
                            "additionalProperties": false,
                            "default": [
                                {
                                    "max_nr_missing": 1,
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                }
                            ],
                            "description": "One or more predefined or custom groups of stations",
                            "items": {
                                "$ref": "#/definitions/station_group"
                            },
                            "minItems": 1,
                            "title": "Station groups",
                            "type": "array"
                        },
                        "max_number_of_missing_stations": {
                            "default": 0,
                            "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                            "minimum": 0,
                            "title": "Maximum number of stations to omit",
                            "type": "integer"
                        },
                        "SAPs": {
                            "additionalItems": false,
                            "default": [
                                {}
                            ],
                            "description": "Station beams",
                            "items": {
                                "additionalProperties": false,
                                "default": {},
                                "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                "properties": {
                                    "digital_pointing": {
                                        "$id": "#target_pointing",
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "title": "Digital pointing"
                                    },
                                    "name": {
                                        "default": "_SAP_name_",
                                        "description": "Identifier for this beam",
                                        "minLength": 1,
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "subbands": {
                                        "additionalItems": false,
                                        "default": [
                                            255
                                        ],
                                        "items": {
                                            "maximum": 511,
                                            "minimum": 0,
                                            "title": "Subband",
                                            "type": "integer"
                                        },
                                        "maxItems": 488,
                                        "minItems": 1,
                                        "title": "Subband list",
                                        "type": "array"
                                    }
                                },
                                "required": [
                                    "name",
                                    "digital_pointing",
                                    "subbands"
                                ],
                                "title": "SAP",
                                "type": "object"
                            },
                            "minItems": 0,
                            "title": "SAPs",
                            "type": "array"
                        },
                        "antenna_set": {
                            "default": "HBA_DUAL",
                            "description": "Fields & antennas to use",
                            "enum": [
                                "HBA_DUAL",
                                "HBA_DUAL_INNER",
                                "HBA_ONE",
                                "HBA_ONE_INNER",
                                "HBA_ZERO",
                                "HBA_ZERO_INNER",
                                "LBA_INNER",
                                "LBA_OUTER",
                                "LBA_SPARSE_EVEN",
                                "LBA_SPARSE_ODD",
                                "LBA_ALL"
                            ],
                            "title": "Antenna set",
                            "type": "string"
                        },
                        "filter": {
                            "default": "HBA_110_190",
                            "description": "Must match antenna type",
                            "enum": [
                                "LBA_10_70",
                                "LBA_30_70",
                                "LBA_10_90",
                                "LBA_30_90",
                                "HBA_110_190",
                                "HBA_210_250"
                            ],
                            "title": "Band-pass filter",
                            "type": "string"
                        },
                        "station_list": {
                            "additionalItems": false,
                            "additionalProperties": false,
                            "default": [],
                            "items": {
                                "$ref": "#/definitions/station"
                            },
                            "minItems": 0,
                            "type": "array",
                            "uniqueItems": true
                        },
                        "station_group": {
                            "anyOf": [
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 0,
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ]
                                    },
                                    "description": "The group of all stations on the Superterp",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Superterp",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 4,
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501"
                                        ]
                                    },
                                    "description": "The group of all Core stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS101",
                                                    "CS103",
                                                    "CS201",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Core",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 4,
                                        "stations": [
                                            "RS106",
                                            "RS205",
                                            "RS208",
                                            "RS210",
                                            "RS305",
                                            "RS306",
                                            "RS307",
                                            "RS310",
                                            "RS406",
                                            "RS407",
                                            "RS409",
                                            "RS503",
                                            "RS508",
                                            "RS509"
                                        ]
                                    },
                                    "description": "The group of all Dutch remote stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "RS106",
                                                    "RS205",
                                                    "RS208",
                                                    "RS210",
                                                    "RS305",
                                                    "RS306",
                                                    "RS307",
                                                    "RS310",
                                                    "RS406",
                                                    "RS407",
                                                    "RS409",
                                                    "RS503",
                                                    "RS508",
                                                    "RS509"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Remote",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 4,
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501",
                                            "RS106",
                                            "RS205",
                                            "RS208",
                                            "RS210",
                                            "RS305",
                                            "RS306",
                                            "RS307",
                                            "RS310",
                                            "RS406",
                                            "RS407",
                                            "RS409",
                                            "RS503",
                                            "RS508",
                                            "RS509"
                                        ]
                                    },
                                    "description": "The group of all Dutch (Core + Remote) stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS101",
                                                    "CS103",
                                                    "CS201",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501",
                                                    "RS106",
                                                    "RS205",
                                                    "RS208",
                                                    "RS210",
                                                    "RS305",
                                                    "RS306",
                                                    "RS307",
                                                    "RS310",
                                                    "RS406",
                                                    "RS407",
                                                    "RS409",
                                                    "RS503",
                                                    "RS508",
                                                    "RS509"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Dutch",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 2,
                                        "stations": [
                                            "DE601",
                                            "DE602",
                                            "DE603",
                                            "DE604",
                                            "DE605",
                                            "DE609",
                                            "FR606",
                                            "SE607",
                                            "UK608",
                                            "PL610",
                                            "PL611",
                                            "PL612",
                                            "IE613",
                                            "LV614"
                                        ]
                                    },
                                    "description": "The group of all international stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "DE601",
                                                    "DE602",
                                                    "DE603",
                                                    "DE604",
                                                    "DE605",
                                                    "DE609",
                                                    "FR606",
                                                    "SE607",
                                                    "UK608",
                                                    "PL610",
                                                    "PL611",
                                                    "PL612",
                                                    "IE613",
                                                    "LV614"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "International",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 1,
                                        "stations": [
                                            "DE601",
                                            "DE605"
                                        ]
                                    },
                                    "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "DE601",
                                                    "DE605"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "International required",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 6,
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501",
                                            "RS106",
                                            "RS205",
                                            "RS208",
                                            "RS210",
                                            "RS305",
                                            "RS306",
                                            "RS307",
                                            "RS310",
                                            "RS406",
                                            "RS407",
                                            "RS409",
                                            "RS503",
                                            "RS508",
                                            "RS509",
                                            "DE601",
                                            "DE602",
                                            "DE603",
                                            "DE604",
                                            "DE605",
                                            "DE609",
                                            "FR606",
                                            "SE607",
                                            "UK608",
                                            "PL610",
                                            "PL611",
                                            "PL612",
                                            "IE613",
                                            "LV614"
                                        ]
                                    },
                                    "description": "The group of all (Core + Remote + International) stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS101",
                                                    "CS103",
                                                    "CS201",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501",
                                                    "RS106",
                                                    "RS205",
                                                    "RS208",
                                                    "RS210",
                                                    "RS305",
                                                    "RS306",
                                                    "RS307",
                                                    "RS310",
                                                    "RS406",
                                                    "RS407",
                                                    "RS409",
                                                    "RS503",
                                                    "RS508",
                                                    "RS509",
                                                    "DE601",
                                                    "DE602",
                                                    "DE603",
                                                    "DE604",
                                                    "DE605",
                                                    "DE609",
                                                    "FR606",
                                                    "SE607",
                                                    "UK608",
                                                    "PL610",
                                                    "PL611",
                                                    "PL612",
                                                    "IE613",
                                                    "LV614"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "All",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 0,
                                        "stations": [
                                            "CS001"
                                        ]
                                    },
                                    "description": "A custom group of stations which can be defined by the user",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list"
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Custom",
                                    "type": "object"
                                }
                            ],
                            "default": {
                                "max_nr_missing": 1,
                                "stations": [
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007"
                                ]
                            },
                            "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                            "title": "Station group",
                            "type": "object"
                        }
                    }
                },
                "schema": {
                    "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2#",
                    "type": "object",
                    "title": "parallel calibrator target observation",
                    "$schema": "http://json-schema.org/draft-06/schema#",
                    "version": 2,
                    "required": [
                        "calibrator",
                        "correlator",
                        "duration",
                        "station_configuration"
                    ],
                    "properties": {
                        "QA": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/2#/definitions/QA",
                            "default": {}
                        },
                        "duration": {
                            "$id": "#duration",
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                            "title": "Duration",
                            "default": 600,
                            "minimum": 1,
                            "description": "Duration of this observation (seconds)"
                        },
                        "calibrator": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/calibrator/1#/definitions/calibrator",
                            "default": {}
                        },
                        "correlator": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/correlator/1#/definitions/correlator",
                            "default": {}
                        },
                        "station_configuration": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                            "default": {}
                        }
                    },
                    "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                    "patternProperties": {
                        "^[$]schema$": {}
                    },
                    "additionalProperties": false
                },
                "state": "http://localhost:3000/api/template_state/active",
                "state_value": "active",
                "tags": [],
                "type": "http://localhost:3000/api/task_type/observation",
                "type_value": "observation",
                "updated_at": "2022-04-14T06:20:07.273910",
                "version": 2
            },
            "specifications_template_id": 5,
            "status": "scheduled",
            "status_value": "scheduled",
            "subtasks": [
                "http://localhost:3000/api/subtask/2000001",
                "http://localhost:3000/api/subtask/2000002",
                "http://localhost:3000/api/subtask/2000000"
            ],
            "subtasks_ids": [
                2000001,
                2000002,
                2000000
            ],
            "tags": [],
            "task_type": "observation",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:19.668472"
        }
      ],
      "successors": [
        {
            "id": 8,
            "url": "http://localhost:3000/api/task_blueprint/8",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [],
            "consumed_by_ids": [],
            "created_at": "2022-04-14T06:20:19.977881",
            "description": "Ingest all preprocessed dataproducts",
            "draft": "http://localhost:3000/api/task_draft/8",
            "draft_id": 8,
            "duration": 300,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Ingest",
            "obsolete_since": null,
            "on_sky_duration": null,
            "on_sky_start_time": null,
            "on_sky_stop_time": null,
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:29:19.054619",
            "process_stop_time": "2022-04-14T06:34:19.054619",
            "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/7",
                "http://localhost:3000/api/task_relation_blueprint/8",
                "http://localhost:3000/api/task_relation_blueprint/9",
                "http://localhost:3000/api/task_relation_blueprint/10",
                "http://localhost:3000/api/task_relation_blueprint/11",
                "http://localhost:3000/api/task_relation_blueprint/12"
            ],
            "produced_by_ids": [
                7,
                8,
                9,
                10,
                11,
                12
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduled_start_time": "2022-04-14T06:29:19.054619",
            "scheduled_stop_time": "2022-04-14T06:34:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
            "scheduling_unit_blueprint_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "",
            "specifications_doc": {
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
            },
            "specifications_template": "http://localhost:3000/api/task_template/2",
            "specifications_template_id": 2,
            "status": "schedulable",
            "status_value": "schedulable",
            "subtasks": [
                {
                    "id": 2000009,
                    "url": "http://localhost:3000/api/subtask/2000009",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "cluster": "http://localhost:3000/api/cluster/1",
                    "cluster_id": 1,
                    "cluster_name": "CEP4",
                    "created_at": "2022-04-14T06:20:21.282039",
                    "created_or_updated_by_user": null,
                    "created_or_updated_by_user_id": null,
                    "duration": 300,
                    "error_reason": null,
                    "global_identifier": "http://localhost:3000/api/sip_identifier/20",
                    "global_identifier_id": 20,
                    "global_parset_identifier": "http://localhost:3000/api/sip_identifier/21",
                    "global_parset_identifier_id": 21,
                    "inputs": [
                        "http://localhost:3000/api/subtask_input/9",
                        "http://localhost:3000/api/subtask_input/10",
                        "http://localhost:3000/api/subtask_input/11",
                        "http://localhost:3000/api/subtask_input/12",
                        "http://localhost:3000/api/subtask_input/13",
                        "http://localhost:3000/api/subtask_input/14"
                    ],
                    "inputs_ids": [
                        9,
                        10,
                        11,
                        12,
                        13,
                        14
                    ],
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "outputs": [],
                    "outputs_ids": [],
                    "primary": true,
                    "process_start_time": "2022-04-14T06:29:19.054619",
                    "process_stop_time": "2022-04-14T06:34:19.054619",
                    "processing_feedback_doc": null,
                    "processing_feedback_template": null,
                    "processing_feedback_template_id": null,
                    "scheduled_start_time": "2022-04-14T06:29:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:34:19.054619",
                    "specifications_doc": {
                        "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/ingest%20control/2/ref_resolved"
                    },
                    "specifications_template": "http://localhost:3000/api/subtask_template/2",
                    "specifications_template_id": 2,
                    "state": "http://localhost:3000/api/subtask_state/defined",
                    "state_value": "defined",
                    "subtask_type": "ingest",
                    "tags": [],
                    "task_blueprint": "http://localhost:3000/api/task_blueprint/8",
                    "task_blueprint_id": 8,
                    "total_nr_of_dataproducts": 0,
                    "total_size_of_dataproducts": 0,
                    "total_size_of_dataproducts_on_filesysem": 0,
                    "unschedulable_reason": null,
                    "updated_at": "2022-04-14T06:21:07.499484"
                }
            ],
            "subtasks_ids": [
                2000009
            ],
            "tags": [],
            "task_type": "ingest",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:19.977893"
        }
      ]
    },
    {
      "id": 7,
      "url": "http://localhost:3000/api/task_blueprint/7",
      "actual_on_sky_start_time": null,
      "actual_on_sky_stop_time": null,
      "actual_process_start_time": null,
      "actual_process_stop_time": null,
      "consumed_by": [
          "http://localhost:3000/api/task_relation_blueprint/8"
      ],
      "consumed_by_ids": [
          8
      ],
      "created_at": "2022-04-14T06:20:19.939457",
      "description": "Preprocessing Pipeline for Target Observation target1",
      "draft": "http://localhost:3000/api/task_draft/7",
      "draft_id": 7,
      "draftObject": {
        "id": 7,
        "url": "http://localhost:3000/api/task_draft/7",
        "connector_types": [
            {
                "id": 13,
                "url": "http://localhost:3000/api/task_connector_type/13",
                "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                "dataformat_value": "MeasurementSet",
                "datatype": "http://localhost:3000/api/datatype/visibilities",
                "datatype_value": "visibilities",
                "iotype": "http://localhost:3000/api/iotype/output",
                "iotype_value": "output",
                "role": "http://localhost:3000/api/role/any",
                "role_value": "any",
                "task_template": "http://localhost:3000/api/task_template/10",
                "task_template_id": 10
            },
            {
                "id": 12,
                "url": "http://localhost:3000/api/task_connector_type/12",
                "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                "dataformat_value": "MeasurementSet",
                "datatype": "http://localhost:3000/api/datatype/visibilities",
                "datatype_value": "visibilities",
                "iotype": "http://localhost:3000/api/iotype/input",
                "iotype_value": "input",
                "role": "http://localhost:3000/api/role/any",
                "role_value": "any",
                "task_template": "http://localhost:3000/api/task_template/10",
                "task_template_id": 10
            }
        ],
        "consumed_by": [
            "http://localhost:3000/api/task_relation_draft/8"
        ],
        "consumed_by_ids": [
            8
        ],
        "created_at": "2022-04-14T06:20:19.356637",
        "description": "Preprocessing Pipeline for Target Observation target1",
        "duration": 0,
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Pipeline target1",
        "output_pinned": false,
        "produced_by": [
            "http://localhost:3000/api/task_relation_draft/2"
        ],
        "produced_by_ids": [
            2
        ],
        "relative_start_time": 0,
        "relative_stop_time": 0,
        "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
        "scheduling_unit_draft_id": 1,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "Paaa+01/1.0/PP",
        "specifications_doc": {
            "flag": {
                "rfi_strategy": "LBAdefault",
                "outerchannels": true,
                "autocorrelations": true
            },
            "demix": {
                "sources": [],
                "time_steps": 10,
                "ignore_target": false,
                "frequency_steps": 64
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
            "average": {
                "time_steps": 1,
                "frequency_steps": 1
            },
            "storagemanager": "dysco",
            "cluster_resources": {
                "where": {
                    "cluster": "CEP4",
                    "partition": "cpu"
                },
                "cores_per_task": 2,
                "parallel_tasks": 122
            }
        },
        "specifications_template": "http://localhost:3000/api/task_template/10",
        "specifications_template_id": 10,
        "tags": [],
        "task_blueprints": [
            "http://localhost:3000/api/task_blueprint/7"
        ],
        "task_blueprints_ids": [
            7
        ],
        "task_type": "pipeline",
        "updated_at": "2022-04-14T06:20:19.356648"
      },
      "duration": 120,
      "error_reason": "",
      "first_scheduling_relation": [],
      "first_scheduling_relation_ids": [],
      "name": "Pipeline target1",
      "obsolete_since": null,
      "on_sky_duration": null,
      "on_sky_start_time": null,
      "on_sky_stop_time": null,
      "output_pinned": false,
      "process_start_time": "2022-04-14T06:27:19.054619",
      "process_stop_time": "2022-04-14T06:29:19.054619",
      "produced_by": [
          "http://localhost:3000/api/task_relation_blueprint/6"
      ],
      "produced_by_ids": [
          6
      ],
      "relative_start_time": 0,
      "relative_stop_time": 0,
      "scheduled_start_time": "2022-04-14T06:27:19.054619",
      "scheduled_stop_time": "2022-04-14T06:29:19.054619",
      "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
      "scheduling_unit_blueprint_id": 1,
      "second_scheduling_relation": [],
      "second_scheduling_relation_ids": [],
      "short_description": "Paaa+01/1.0/PP",
      "specifications_doc": {
          "flag": {
              "rfi_strategy": "LBAdefault",
              "outerchannels": true,
              "autocorrelations": true
          },
          "demix": {
              "sources": [],
              "time_steps": 10,
              "ignore_target": false,
              "frequency_steps": 64
          },
          "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
          "average": {
              "time_steps": 1,
              "frequency_steps": 1
          },
          "storagemanager": "dysco",
          "cluster_resources": {
              "where": {
                  "cluster": "CEP4",
                  "partition": "cpu"
              },
              "cores_per_task": 2,
              "parallel_tasks": 122
          }
      },
      "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "preprocessing pipeline"})).id,
      "specifications_template_id": (_.find(templates, {"name": "preprocessing pipeline"})).id,
      "status": "schedulable",
      "status_value": "schedulable",
      "subtasks": [
          {
              "id": 2000007,
              "url": "http://localhost:3000/api/subtask/2000007",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "cluster": "http://localhost:3000/api/cluster/1",
              "cluster_id": 1,
              "cluster_name": "CEP4",
              "created_at": "2022-04-14T06:20:21.134705",
              "created_or_updated_by_user": null,
              "created_or_updated_by_user_id": null,
              "duration": 120,
              "error_reason": null,
              "global_identifier": "http://localhost:3000/api/sip_identifier/16",
              "global_identifier_id": 16,
              "global_parset_identifier": "http://localhost:3000/api/sip_identifier/17",
              "global_parset_identifier_id": 17,
              "inputs": [
                  "http://localhost:3000/api/subtask_input/7"
              ],
              "inputs_ids": [
                  7
              ],
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "outputs": [
                  "http://localhost:3000/api/subtask_output/8"
              ],
              "outputs_ids": [
                  8
              ],
              "primary": true,
              "process_start_time": "2022-04-14T06:27:19.054619",
              "process_stop_time": "2022-04-14T06:29:19.054619",
              "processing_feedback_doc": null,
              "processing_feedback_template": null,
              "processing_feedback_template_id": null,
              "scheduled_start_time": "2022-04-14T06:27:19.054619",
              "scheduled_stop_time": "2022-04-14T06:29:19.054619",
              "specifications_doc": {
                  "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "demixer": {
                      "enabled": true,
                      "baselines": "CS*,RS*&",
                      "time_steps": 1,
                      "demix_always": [],
                      "ignore_target": false,
                      "demix_if_needed": [],
                      "frequency_steps": 1,
                      "demix_time_steps": 10,
                      "demix_frequency_steps": 64
                  },
                  "aoflagger": {
                      "enabled": true,
                      "strategy": "LBAdefault"
                  },
                  "preflagger0": {
                      "enabled": true,
                      "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                  },
                  "preflagger1": {
                      "enabled": true,
                      "corrtype": "auto"
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": "http://localhost:3000/api/subtask_template/7",
              "specifications_template_id": 7,
              "state": "http://localhost:3000/api/subtask_state/defined",
              "state_value": "defined",
              "subtask_type": "pipeline",
              "tags": [],
              "task_blueprint": "http://localhost:3000/api/task_blueprint/7",
              "task_blueprint_id": 7,
              "total_nr_of_dataproducts": 0,
              "total_size_of_dataproducts": 0,
              "total_size_of_dataproducts_on_filesysem": 0,
              "unschedulable_reason": null,
              "updated_at": "2022-04-14T06:21:07.261428"
          }
      ],
      "subtasks_ids": [
          2000007
      ],
      "tags": [],
      "task_type": "pipeline",
      "unschedulable_reason": "",
      "updated_at": "2022-04-14T06:20:19.939486",
      "predecessors": [
        {
            "id": 1,
            "url": "http://localhost:3000/api/task_blueprint/1",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/1",
                "http://localhost:3000/api/task_relation_blueprint/2",
                "http://localhost:3000/api/task_relation_blueprint/3",
                "http://localhost:3000/api/task_relation_blueprint/4",
                "http://localhost:3000/api/task_relation_blueprint/5",
                "http://localhost:3000/api/task_relation_blueprint/6"
            ],
            "consumed_by_ids": [
                1,
                2,
                3,
                4,
                5,
                6
            ],
            "created_at": "2022-04-14T06:20:19.668460",
            "description": "Combined parallel Calibrator & Target Observation for LBA with 5 target beams",
            "draft": "http://localhost:3000/api/task_draft/1",
            "draft_id": 1,
            "duration": 720,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Combined Observation",
            "obsolete_since": null,
            "on_sky_duration": 120,
            "on_sky_start_time": "2022-04-14T06:25:19.054619",
            "on_sky_stop_time": "2022-04-14T06:27:19.054619",
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:25:19.054619",
            "process_stop_time": "2022-04-14T06:37:19.054619",
            "produced_by": [],
            "produced_by_ids": [],
            "relative_start_time": 0,
            "relative_stop_time": 120,
            "scheduled_start_time": "2022-04-14T06:25:19.054619",
            "scheduled_stop_time": "2022-04-14T06:37:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
            "scheduling_unit_blueprint_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "oXXX Paaa+01 3Cabc",
            "specifications_doc": {
                "QA": {
                    "plots": {
                        "enabled": true,
                        "autocorrelation": true,
                        "crosscorrelation": true
                    },
                    "file_conversion": {
                        "enabled": true,
                        "nr_of_subbands": -1,
                        "nr_of_timestamps": 256
                    },
                    "inspection_plots": "msplots"
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                "duration": 120,
                "calibrator": {
                    "name": "calibrator",
                    "pointing": {
                        "angle1": 0.6624317181687094,
                        "angle2": 1.5579526427549426,
                        "target": "_calibrator_name_",
                        "direction_type": "J2000"
                    },
                    "autoselect": false
                },
                "correlator": {
                    "storage_cluster": "CEP4",
                    "integration_time": 1,
                    "channels_per_subband": 64,
                    "topocentric_frequency_correction": false
                },
                "station_configuration": {
                    "SAPs": [
                        {
                            "name": "target1",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+01",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target2",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+02",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target3",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+03",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target4",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+04",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "target5",
                            "subbands": [
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+05",
                                "direction_type": "J2000"
                            }
                        }
                    ],
                    "filter": "LBA_10_90",
                    "tile_beam": {
                        "angle1": 0.6624317181687094,
                        "angle2": 1.5579526427549426,
                        "target": "_target_name_",
                        "direction_type": "J2000"
                    },
                    "antenna_set": "LBA_SPARSE_EVEN",
                    "station_groups": [
                        {
                            "stations": [
                                "CS001",
                                "CS002",
                                "CS003",
                                "CS004",
                                "CS005",
                                "CS006",
                                "CS007",
                                "CS011",
                                "CS013",
                                "CS017",
                                "CS021",
                                "CS024",
                                "CS026",
                                "CS028",
                                "CS030",
                                "CS031",
                                "CS032",
                                "CS101",
                                "CS103",
                                "CS201",
                                "CS301",
                                "CS302",
                                "CS401",
                                "CS501",
                                "RS106",
                                "RS205",
                                "RS208",
                                "RS210",
                                "RS305",
                                "RS306",
                                "RS307",
                                "RS310",
                                "RS406",
                                "RS407",
                                "RS409",
                                "RS503",
                                "RS508",
                                "RS509"
                            ],
                            "max_nr_missing": 4
                        },
                        {
                            "stations": [
                                "RS508",
                                "RS509"
                            ],
                            "max_nr_missing": 1
                        },
                        {
                            "stations": [
                                "RS310",
                                "RS210"
                            ],
                            "max_nr_missing": 0
                        }
                    ]
                }
            },
            "specifications_template": {
                "id": 5,
                "url": "http://localhost:3000/api/task_template/5",
                "created_at": "2022-04-14T06:20:07.273898",
                "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                "name": "parallel calibrator target observation",
                "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                "purpose_value": "technical_commissioning",
                "ref_resolved_schema": {
                    "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                    "type": "object",
                    "title": "parallel calibrator target observation",
                    "$schema": "http://json-schema.org/draft-06/schema#",
                    "version": 2,
                    "required": [
                        "calibrator",
                        "correlator",
                        "duration",
                        "station_configuration"
                    ],
                    "properties": {
                        "QA": {
                            "$ref": "#/definitions/QA",
                            "default": {}
                        },
                        "duration": {
                            "$id": "#duration",
                            "$ref": "#/definitions/timedelta",
                            "title": "Duration",
                            "default": 600,
                            "minimum": 1,
                            "description": "Duration of this observation (seconds)"
                        },
                        "calibrator": {
                            "$ref": "#/definitions/calibrator",
                            "default": {}
                        },
                        "correlator": {
                            "$ref": "#/definitions/correlator",
                            "default": {}
                        },
                        "station_configuration": {
                            "$ref": "#/definitions/station_configuration",
                            "default": {}
                        }
                    },
                    "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                    "patternProperties": {
                        "^[$]schema$": {}
                    },
                    "additionalProperties": false,
                    "definitions": {
                        "QA": {
                            "additionalProperties": false,
                            "default": {},
                            "description": "Perform all Quality Assurance (QA) tasks, including file conversion and plotting.",
                            "properties": {
                                "file_conversion": {
                                    "$ref": "#/definitions/file_conversion",
                                    "default": {}
                                },
                                "inspection_plots": {
                                    "$ref": "#/definitions/inspection_plots",
                                    "default": "msplots"
                                },
                                "plots": {
                                    "$ref": "#/definitions/plots",
                                    "default": {}
                                }
                            },
                            "required": [
                                "file_conversion",
                                "plots",
                                "inspection_plots"
                            ],
                            "title": "QA",
                            "type": "object"
                        },
                        "plots": {
                            "additionalProperties": false,
                            "default": {},
                            "description": "Create dynamic spectrum plots",
                            "properties": {
                                "autocorrelation": {
                                    "default": true,
                                    "description": "Create autocorrelation plots for all stations",
                                    "title": "autocorrelation",
                                    "type": "boolean"
                                },
                                "crosscorrelation": {
                                    "default": true,
                                    "description": "Create crosscorrelation plots for all baselines",
                                    "title": "crosscorrelation",
                                    "type": "boolean"
                                },
                                "enabled": {
                                    "default": true,
                                    "description": "Do/Don't create plots from the QA file from the observation",
                                    "title": "enabled",
                                    "type": "boolean"
                                }
                            },
                            "required": [
                                "enabled",
                                "autocorrelation",
                                "crosscorrelation"
                            ],
                            "title": "Plots",
                            "type": "object"
                        },
                        "file_conversion": {
                            "additionalProperties": false,
                            "default": {},
                            "description": "Create a QA file for the observation",
                            "properties": {
                                "enabled": {
                                    "default": true,
                                    "description": "Do/Don't create a QA file for the observation",
                                    "title": "enabled",
                                    "type": "boolean"
                                },
                                "nr_of_subbands": {
                                    "default": -1,
                                    "description": "Keep this number of subbands from the observation in the QA file, or all if -1",
                                    "title": "#subbands",
                                    "type": "integer"
                                },
                                "nr_of_timestamps": {
                                    "default": 256,
                                    "description": "Extract this number of timestamps from the observation in the QA file (equidistantanly sampled, no averaging/interpolation)",
                                    "minimum": 1,
                                    "title": "#timestamps",
                                    "type": "integer"
                                }
                            },
                            "required": [
                                "enabled",
                                "nr_of_subbands",
                                "nr_of_timestamps"
                            ],
                            "title": "File Conversion",
                            "type": "object"
                        },
                        "inspection_plots": {
                            "decription": "Type of inspection plots to run",
                            "default": "msplots",
                            "enum": [
                                "msplots",
                                "dynspec",
                                "none"
                            ],
                            "title": "Inspection Plots",
                            "type": "string"
                        },
                        "timedelta": {
                            "default": 0,
                            "description": "A time duration or delta expressed in seconds",
                            "type": "number"
                        },
                        "timestamp": {
                            "default": "1970-01-01T00:00:00Z",
                            "description": "A timestamp defined in UTC",
                            "format": "date-time",
                            "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                            "type": "string"
                        },
                        "calibrator": {
                            "additionalProperties": false,
                            "default": {},
                            "properties": {
                                "autoselect": {
                                    "default": true,
                                    "description": "Auto-select calibrator based on elevation",
                                    "title": "Auto-select",
                                    "type": "boolean"
                                },
                                "name": {
                                    "default": "calibrator",
                                    "description": "Name of the calibrator SAP",
                                    "title": "Name",
                                    "type": "string"
                                },
                                "pointing": {
                                    "$ref": "#/definitions/pointing",
                                    "default": {},
                                    "description": "Manually selected calibrator",
                                    "title": "Digital pointing"
                                }
                            },
                            "required": [
                                "autoselect",
                                "pointing",
                                "name"
                            ],
                            "type": "object"
                        },
                        "pointing": {
                            "additionalProperties": false,
                            "properties": {
                                "angle1": {
                                    "default": 0.6624317181687094,
                                    "description": "First angle (e.g. RA)",
                                    "title": "Angle 1",
                                    "type": "number"
                                },
                                "angle2": {
                                    "default": 1.5579526427549426,
                                    "description": "Second angle (e.g. DEC)",
                                    "title": "Angle 2",
                                    "type": "number"
                                },
                                "direction_type": {
                                    "default": "J2000",
                                    "description": "",
                                    "enum": [
                                        "J2000",
                                        "AZELGEO",
                                        "LMN",
                                        "SUN",
                                        "MOON",
                                        "MERCURY",
                                        "VENUS",
                                        "MARS",
                                        "JUPITER",
                                        "SATURN",
                                        "URANUS",
                                        "NEPTUNE",
                                        "PLUTO"
                                    ],
                                    "title": "Reference frame",
                                    "type": "string"
                                },
                                "target": {
                                    "default": "_target_name_",
                                    "description": "Description of where this beam points at",
                                    "minLength": 1,
                                    "title": "Target",
                                    "type": "string"
                                }
                            },
                            "required": [
                                "angle1",
                                "angle2",
                                "target"
                            ],
                            "type": "object"
                        },
                        "correlator": {
                            "additionalProperties": false,
                            "default": {},
                            "properties": {
                                "channels_per_subband": {
                                    "default": 64,
                                    "description": "Number of frequency bands per subband",
                                    "enum": [
                                        8,
                                        16,
                                        32,
                                        64,
                                        128,
                                        256,
                                        512,
                                        1024
                                    ],
                                    "minimum": 8,
                                    "title": "Channels/subband",
                                    "type": "integer"
                                },
                                "integration_time": {
                                    "$ref": "#/definitions/timedelta",
                                    "default": 1,
                                    "description": "Desired integration period (seconds)",
                                    "minimum": 0.1,
                                    "title": "Integration time"
                                },
                                "storage_cluster": {
                                    "default": "CEP4",
                                    "description": "Cluster to write output to",
                                    "enum": [
                                        "CEP4",
                                        "DragNet"
                                    ],
                                    "title": "Storage cluster",
                                    "type": "string"
                                },
                                "topocentric_frequency_correction": {
                                    "default": false,
                                    "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation.",
                                    "title": "Topocentric Frequency Correction",
                                    "type": "boolean"
                                }
                            }
                        },
                        "station_configuration": {
                            "default": {},
                            "properties": {
                                "SAPs": {
                                    "$ref": "#/definitions/SAPs",
                                    "default": [
                                        {}
                                    ],
                                    "minItems": 1
                                },
                                "antenna_set": {
                                    "$ref": "#/definitions/antenna_set",
                                    "default": "HBA_DUAL"
                                },
                                "filter": {
                                    "$ref": "#/definitions/filter",
                                    "default": "HBA_110_190"
                                },
                                "station_groups": {
                                    "$ref": "#/definitions/station_groups",
                                    "default": [
                                        {
                                            "max_nr_missing": 1,
                                            "stations": [
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007"
                                            ]
                                        }
                                    ]
                                },
                                "tile_beam": {
                                    "$ref": "#/definitions/pointing",
                                    "default": {},
                                    "description": "HBA only",
                                    "title": "Tile beam"
                                }
                            },
                            "required": [
                                "station_groups",
                                "antenna_set",
                                "filter",
                                "SAPs"
                            ],
                            "title": "station_configuration",
                            "type": "object"
                        },
                        "station": {
                            "description": "These are the LOFAR stations",
                            "enum": [
                                "CS001",
                                "CS002",
                                "CS003",
                                "CS004",
                                "CS005",
                                "CS006",
                                "CS007",
                                "CS011",
                                "CS013",
                                "CS017",
                                "CS021",
                                "CS024",
                                "CS026",
                                "CS028",
                                "CS030",
                                "CS031",
                                "CS032",
                                "CS101",
                                "CS103",
                                "CS201",
                                "CS301",
                                "CS302",
                                "CS401",
                                "CS501",
                                "RS106",
                                "RS205",
                                "RS208",
                                "RS210",
                                "RS305",
                                "RS306",
                                "RS307",
                                "RS310",
                                "RS406",
                                "RS407",
                                "RS409",
                                "RS503",
                                "RS508",
                                "RS509",
                                "DE601",
                                "DE602",
                                "DE603",
                                "DE604",
                                "DE605",
                                "FR606",
                                "SE607",
                                "UK608",
                                "DE609",
                                "PL610",
                                "PL611",
                                "PL612",
                                "IE613",
                                "LV614"
                            ],
                            "title": "Station",
                            "type": "string"
                        },
                        "station_groups": {
                            "additionalItems": false,
                            "additionalProperties": false,
                            "default": [
                                {
                                    "max_nr_missing": 1,
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                }
                            ],
                            "description": "One or more predefined or custom groups of stations",
                            "items": {
                                "$ref": "#/definitions/station_group"
                            },
                            "minItems": 1,
                            "title": "Station groups",
                            "type": "array"
                        },
                        "max_number_of_missing_stations": {
                            "default": 0,
                            "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                            "minimum": 0,
                            "title": "Maximum number of stations to omit",
                            "type": "integer"
                        },
                        "SAPs": {
                            "additionalItems": false,
                            "default": [
                                {}
                            ],
                            "description": "Station beams",
                            "items": {
                                "additionalProperties": false,
                                "default": {},
                                "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                "properties": {
                                    "digital_pointing": {
                                        "$id": "#target_pointing",
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "title": "Digital pointing"
                                    },
                                    "name": {
                                        "default": "_SAP_name_",
                                        "description": "Identifier for this beam",
                                        "minLength": 1,
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "subbands": {
                                        "additionalItems": false,
                                        "default": [
                                            255
                                        ],
                                        "items": {
                                            "maximum": 511,
                                            "minimum": 0,
                                            "title": "Subband",
                                            "type": "integer"
                                        },
                                        "maxItems": 488,
                                        "minItems": 1,
                                        "title": "Subband list",
                                        "type": "array"
                                    }
                                },
                                "required": [
                                    "name",
                                    "digital_pointing",
                                    "subbands"
                                ],
                                "title": "SAP",
                                "type": "object"
                            },
                            "minItems": 0,
                            "title": "SAPs",
                            "type": "array"
                        },
                        "antenna_set": {
                            "default": "HBA_DUAL",
                            "description": "Fields & antennas to use",
                            "enum": [
                                "HBA_DUAL",
                                "HBA_DUAL_INNER",
                                "HBA_ONE",
                                "HBA_ONE_INNER",
                                "HBA_ZERO",
                                "HBA_ZERO_INNER",
                                "LBA_INNER",
                                "LBA_OUTER",
                                "LBA_SPARSE_EVEN",
                                "LBA_SPARSE_ODD",
                                "LBA_ALL"
                            ],
                            "title": "Antenna set",
                            "type": "string"
                        },
                        "filter": {
                            "default": "HBA_110_190",
                            "description": "Must match antenna type",
                            "enum": [
                                "LBA_10_70",
                                "LBA_30_70",
                                "LBA_10_90",
                                "LBA_30_90",
                                "HBA_110_190",
                                "HBA_210_250"
                            ],
                            "title": "Band-pass filter",
                            "type": "string"
                        },
                        "station_list": {
                            "additionalItems": false,
                            "additionalProperties": false,
                            "default": [],
                            "items": {
                                "$ref": "#/definitions/station"
                            },
                            "minItems": 0,
                            "type": "array",
                            "uniqueItems": true
                        },
                        "station_group": {
                            "anyOf": [
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 0,
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ]
                                    },
                                    "description": "The group of all stations on the Superterp",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Superterp",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 4,
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501"
                                        ]
                                    },
                                    "description": "The group of all Core stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS101",
                                                    "CS103",
                                                    "CS201",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Core",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 4,
                                        "stations": [
                                            "RS106",
                                            "RS205",
                                            "RS208",
                                            "RS210",
                                            "RS305",
                                            "RS306",
                                            "RS307",
                                            "RS310",
                                            "RS406",
                                            "RS407",
                                            "RS409",
                                            "RS503",
                                            "RS508",
                                            "RS509"
                                        ]
                                    },
                                    "description": "The group of all Dutch remote stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "RS106",
                                                    "RS205",
                                                    "RS208",
                                                    "RS210",
                                                    "RS305",
                                                    "RS306",
                                                    "RS307",
                                                    "RS310",
                                                    "RS406",
                                                    "RS407",
                                                    "RS409",
                                                    "RS503",
                                                    "RS508",
                                                    "RS509"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Remote",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 4,
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501",
                                            "RS106",
                                            "RS205",
                                            "RS208",
                                            "RS210",
                                            "RS305",
                                            "RS306",
                                            "RS307",
                                            "RS310",
                                            "RS406",
                                            "RS407",
                                            "RS409",
                                            "RS503",
                                            "RS508",
                                            "RS509"
                                        ]
                                    },
                                    "description": "The group of all Dutch (Core + Remote) stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS101",
                                                    "CS103",
                                                    "CS201",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501",
                                                    "RS106",
                                                    "RS205",
                                                    "RS208",
                                                    "RS210",
                                                    "RS305",
                                                    "RS306",
                                                    "RS307",
                                                    "RS310",
                                                    "RS406",
                                                    "RS407",
                                                    "RS409",
                                                    "RS503",
                                                    "RS508",
                                                    "RS509"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Dutch",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 2,
                                        "stations": [
                                            "DE601",
                                            "DE602",
                                            "DE603",
                                            "DE604",
                                            "DE605",
                                            "DE609",
                                            "FR606",
                                            "SE607",
                                            "UK608",
                                            "PL610",
                                            "PL611",
                                            "PL612",
                                            "IE613",
                                            "LV614"
                                        ]
                                    },
                                    "description": "The group of all international stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "DE601",
                                                    "DE602",
                                                    "DE603",
                                                    "DE604",
                                                    "DE605",
                                                    "DE609",
                                                    "FR606",
                                                    "SE607",
                                                    "UK608",
                                                    "PL610",
                                                    "PL611",
                                                    "PL612",
                                                    "IE613",
                                                    "LV614"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "International",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 1,
                                        "stations": [
                                            "DE601",
                                            "DE605"
                                        ]
                                    },
                                    "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "DE601",
                                                    "DE605"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "International required",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 6,
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501",
                                            "RS106",
                                            "RS205",
                                            "RS208",
                                            "RS210",
                                            "RS305",
                                            "RS306",
                                            "RS307",
                                            "RS310",
                                            "RS406",
                                            "RS407",
                                            "RS409",
                                            "RS503",
                                            "RS508",
                                            "RS509",
                                            "DE601",
                                            "DE602",
                                            "DE603",
                                            "DE604",
                                            "DE605",
                                            "DE609",
                                            "FR606",
                                            "SE607",
                                            "UK608",
                                            "PL610",
                                            "PL611",
                                            "PL612",
                                            "IE613",
                                            "LV614"
                                        ]
                                    },
                                    "description": "The group of all (Core + Remote + International) stations",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list",
                                            "enum": [
                                                [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS101",
                                                    "CS103",
                                                    "CS201",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501",
                                                    "RS106",
                                                    "RS205",
                                                    "RS208",
                                                    "RS210",
                                                    "RS305",
                                                    "RS306",
                                                    "RS307",
                                                    "RS310",
                                                    "RS406",
                                                    "RS407",
                                                    "RS409",
                                                    "RS503",
                                                    "RS508",
                                                    "RS509",
                                                    "DE601",
                                                    "DE602",
                                                    "DE603",
                                                    "DE604",
                                                    "DE605",
                                                    "DE609",
                                                    "FR606",
                                                    "SE607",
                                                    "UK608",
                                                    "PL610",
                                                    "PL611",
                                                    "PL612",
                                                    "IE613",
                                                    "LV614"
                                                ]
                                            ]
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "All",
                                    "type": "object"
                                },
                                {
                                    "additionalProperties": false,
                                    "default": {
                                        "max_nr_missing": 0,
                                        "stations": [
                                            "CS001"
                                        ]
                                    },
                                    "description": "A custom group of stations which can be defined by the user",
                                    "properties": {
                                        "max_nr_missing": {
                                            "$ref": "#/definitions/max_number_of_missing_stations"
                                        },
                                        "stations": {
                                            "$ref": "#/definitions/station_list"
                                        }
                                    },
                                    "required": [
                                        "stations",
                                        "max_nr_missing"
                                    ],
                                    "title": "Custom",
                                    "type": "object"
                                }
                            ],
                            "default": {
                                "max_nr_missing": 1,
                                "stations": [
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007"
                                ]
                            },
                            "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                            "title": "Station group",
                            "type": "object"
                        }
                    }
                },
                "schema": {
                    "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2#",
                    "type": "object",
                    "title": "parallel calibrator target observation",
                    "$schema": "http://json-schema.org/draft-06/schema#",
                    "version": 2,
                    "required": [
                        "calibrator",
                        "correlator",
                        "duration",
                        "station_configuration"
                    ],
                    "properties": {
                        "QA": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/2#/definitions/QA",
                            "default": {}
                        },
                        "duration": {
                            "$id": "#duration",
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                            "title": "Duration",
                            "default": 600,
                            "minimum": 1,
                            "description": "Duration of this observation (seconds)"
                        },
                        "calibrator": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/calibrator/1#/definitions/calibrator",
                            "default": {}
                        },
                        "correlator": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/correlator/1#/definitions/correlator",
                            "default": {}
                        },
                        "station_configuration": {
                            "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                            "default": {}
                        }
                    },
                    "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                    "patternProperties": {
                        "^[$]schema$": {}
                    },
                    "additionalProperties": false
                },
                "state": "http://localhost:3000/api/template_state/active",
                "state_value": "active",
                "tags": [],
                "type": "http://localhost:3000/api/task_type/observation",
                "type_value": "observation",
                "updated_at": "2022-04-14T06:20:07.273910",
                "version": 2
            },
            "specifications_template_id": 5,
            "status": "scheduled",
            "status_value": "scheduled",
            "subtasks": [
                "http://localhost:3000/api/subtask/2000001",
                "http://localhost:3000/api/subtask/2000002",
                "http://localhost:3000/api/subtask/2000000"
            ],
            "subtasks_ids": [
                2000001,
                2000002,
                2000000
            ],
            "tags": [],
            "task_type": "observation",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:19.668472"
        }
      ],
      "successors": [
        {
            "id": 8,
            "url": "http://localhost:3000/api/task_blueprint/8",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [],
            "consumed_by_ids": [],
            "created_at": "2022-04-14T06:20:19.977881",
            "description": "Ingest all preprocessed dataproducts",
            "draft": "http://localhost:3000/api/task_draft/8",
            "draft_id": 8,
            "duration": 300,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Ingest",
            "obsolete_since": null,
            "on_sky_duration": null,
            "on_sky_start_time": null,
            "on_sky_stop_time": null,
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:29:19.054619",
            "process_stop_time": "2022-04-14T06:34:19.054619",
            "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/7",
                "http://localhost:3000/api/task_relation_blueprint/8",
                "http://localhost:3000/api/task_relation_blueprint/9",
                "http://localhost:3000/api/task_relation_blueprint/10",
                "http://localhost:3000/api/task_relation_blueprint/11",
                "http://localhost:3000/api/task_relation_blueprint/12"
            ],
            "produced_by_ids": [
                7,
                8,
                9,
                10,
                11,
                12
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduled_start_time": "2022-04-14T06:29:19.054619",
            "scheduled_stop_time": "2022-04-14T06:34:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
            "scheduling_unit_blueprint_id": 1,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "",
            "specifications_doc": {
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
            },
            "specifications_template": "http://localhost:3000/api/task_template/2",
            "specifications_template_id": 2,
            "status": "schedulable",
            "status_value": "schedulable",
            "subtasks": [
                {
                    "id": 2000009,
                    "url": "http://localhost:3000/api/subtask/2000009",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "cluster": "http://localhost:3000/api/cluster/1",
                    "cluster_id": 1,
                    "cluster_name": "CEP4",
                    "created_at": "2022-04-14T06:20:21.282039",
                    "created_or_updated_by_user": null,
                    "created_or_updated_by_user_id": null,
                    "duration": 300,
                    "error_reason": null,
                    "global_identifier": "http://localhost:3000/api/sip_identifier/20",
                    "global_identifier_id": 20,
                    "global_parset_identifier": "http://localhost:3000/api/sip_identifier/21",
                    "global_parset_identifier_id": 21,
                    "inputs": [
                        "http://localhost:3000/api/subtask_input/9",
                        "http://localhost:3000/api/subtask_input/10",
                        "http://localhost:3000/api/subtask_input/11",
                        "http://localhost:3000/api/subtask_input/12",
                        "http://localhost:3000/api/subtask_input/13",
                        "http://localhost:3000/api/subtask_input/14"
                    ],
                    "inputs_ids": [
                        9,
                        10,
                        11,
                        12,
                        13,
                        14
                    ],
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "outputs": [],
                    "outputs_ids": [],
                    "primary": true,
                    "process_start_time": "2022-04-14T06:29:19.054619",
                    "process_stop_time": "2022-04-14T06:34:19.054619",
                    "processing_feedback_doc": null,
                    "processing_feedback_template": null,
                    "processing_feedback_template_id": null,
                    "scheduled_start_time": "2022-04-14T06:29:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:34:19.054619",
                    "specifications_doc": {
                        "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/ingest%20control/2/ref_resolved"
                    },
                    "specifications_template": "http://localhost:3000/api/subtask_template/2",
                    "specifications_template_id": 2,
                    "state": "http://localhost:3000/api/subtask_state/defined",
                    "state_value": "defined",
                    "subtask_type": "ingest",
                    "tags": [],
                    "task_blueprint": "http://localhost:3000/api/task_blueprint/8",
                    "task_blueprint_id": 8,
                    "total_nr_of_dataproducts": 0,
                    "total_size_of_dataproducts": 0,
                    "total_size_of_dataproducts_on_filesysem": 0,
                    "unschedulable_reason": null,
                    "updated_at": "2022-04-14T06:21:07.499484"
                }
            ],
            "subtasks_ids": [
                2000009
            ],
            "tags": [],
            "task_type": "ingest",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:19.977893"
        }
      ]
    },
    {
      "id": 9,
      "url": "http://localhost:3000/api/task_blueprint/9",
      "actual_on_sky_start_time": null,
      "actual_on_sky_stop_time": null,
      "actual_process_start_time": null,
      "actual_process_stop_time": null,
      "consumed_by": [
          "http://localhost:3000/api/task_relation_blueprint/13",
          "http://localhost:3000/api/task_relation_blueprint/14",
          "http://localhost:3000/api/task_relation_blueprint/15",
          "http://localhost:3000/api/task_relation_blueprint/16",
          "http://localhost:3000/api/task_relation_blueprint/17"
      ],
      "consumed_by_ids": [
          13,
          14,
          15,
          16,
          17
      ],
      "created_at": "2022-04-14T06:20:22.577268",
      "description": "Combined parallel Calibrator & Target Observation for LBA with 3 target beams",
      "draft": "http://localhost:3000/api/task_draft/9",
      "draft_id": 9,
      "draftObject": {
        "id": 9,
        "url": "http://localhost:3000/api/task_draft/9",
        "connector_types": [
            {
                "id": 3,
                "url": "http://localhost:3000/api/task_connector_type/3",
                "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                "dataformat_value": "MeasurementSet",
                "datatype": "http://localhost:3000/api/datatype/visibilities",
                "datatype_value": "visibilities",
                "iotype": "http://localhost:3000/api/iotype/output",
                "iotype_value": "output",
                "role": "http://localhost:3000/api/role/correlator",
                "role_value": "correlator",
                "task_template": "http://localhost:3000/api/task_template/5",
                "task_template_id": 5
            }
        ],
        "consumed_by": [
            "http://localhost:3000/api/task_relation_draft/13",
            "http://localhost:3000/api/task_relation_draft/14",
            "http://localhost:3000/api/task_relation_draft/15",
            "http://localhost:3000/api/task_relation_draft/16",
            "http://localhost:3000/api/task_relation_draft/21"
        ],
        "consumed_by_ids": [
            13,
            14,
            15,
            16,
            21
        ],
        "created_at": "2022-04-14T06:20:22.242382",
        "description": "Combined parallel Calibrator & Target Observation for LBA with 3 target beams",
        "duration": 120,
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Combined Observation",
        "output_pinned": false,
        "produced_by": [],
        "produced_by_ids": [],
        "relative_start_time": 0,
        "relative_stop_time": 120,
        "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/2",
        "scheduling_unit_draft_id": 2,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "OOO.O Paaa+01 Paaa+02 Paaa+03",
        "specifications_doc": {
            "QA": {
                "plots": {
                    "enabled": true,
                    "autocorrelation": true,
                    "crosscorrelation": true
                },
                "file_conversion": {
                    "enabled": true,
                    "nr_of_subbands": -1,
                    "nr_of_timestamps": 256
                },
                "inspection_plots": "msplots"
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
            "duration": 120,
            "calibrator": {
                "name": "calibrator",
                "pointing": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "3Cabc",
                    "direction_type": "J2000"
                },
                "autoselect": false
            },
            "correlator": {
                "storage_cluster": "CEP4",
                "integration_time": 1,
                "channels_per_subband": 64,
                "topocentric_frequency_correction": false
            },
            "station_configuration": {
                "SAPs": [
                    {
                        "name": "target1",
                        "subbands": [
                            217,
                            218,
                            219,
                            220,
                            221,
                            222,
                            223,
                            224,
                            225,
                            226,
                            227,
                            228,
                            229,
                            230,
                            231,
                            232,
                            233,
                            234,
                            235,
                            236,
                            237,
                            238,
                            239,
                            240,
                            241,
                            242,
                            243,
                            244,
                            245,
                            246,
                            247,
                            248,
                            249,
                            250,
                            251,
                            252,
                            253,
                            254,
                            255,
                            256,
                            257,
                            258,
                            259,
                            260,
                            261,
                            262,
                            263,
                            264,
                            265,
                            266,
                            267,
                            268,
                            269,
                            270,
                            271,
                            272,
                            273,
                            274,
                            275,
                            276,
                            277,
                            278,
                            279,
                            280,
                            281,
                            282,
                            283,
                            284,
                            285,
                            286,
                            287,
                            288,
                            289,
                            290,
                            291,
                            292,
                            293,
                            294,
                            295,
                            296,
                            297,
                            298,
                            299,
                            300,
                            301,
                            302,
                            303,
                            304,
                            305,
                            306,
                            307,
                            308,
                            309,
                            310,
                            311,
                            312,
                            313,
                            314,
                            315,
                            316,
                            317,
                            318,
                            319,
                            320,
                            321,
                            322,
                            323,
                            324,
                            325,
                            326,
                            327,
                            328,
                            329,
                            330,
                            331,
                            332,
                            333,
                            334,
                            335,
                            336,
                            337,
                            338
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+01",
                            "direction_type": "J2000"
                        }
                    },
                    {
                        "name": "target2",
                        "subbands": [
                            217,
                            218,
                            219,
                            220,
                            221,
                            222,
                            223,
                            224,
                            225,
                            226,
                            227,
                            228,
                            229,
                            230,
                            231,
                            232,
                            233,
                            234,
                            235,
                            236,
                            237,
                            238,
                            239,
                            240,
                            241,
                            242,
                            243,
                            244,
                            245,
                            246,
                            247,
                            248,
                            249,
                            250,
                            251,
                            252,
                            253,
                            254,
                            255,
                            256,
                            257,
                            258,
                            259,
                            260,
                            261,
                            262,
                            263,
                            264,
                            265,
                            266,
                            267,
                            268,
                            269,
                            270,
                            271,
                            272,
                            273,
                            274,
                            275,
                            276,
                            277,
                            278,
                            279,
                            280,
                            281,
                            282,
                            283,
                            284,
                            285,
                            286,
                            287,
                            288,
                            289,
                            290,
                            291,
                            292,
                            293,
                            294,
                            295,
                            296,
                            297,
                            298,
                            299,
                            300,
                            301,
                            302,
                            303,
                            304,
                            305,
                            306,
                            307,
                            308,
                            309,
                            310,
                            311,
                            312,
                            313,
                            314,
                            315,
                            316,
                            317,
                            318,
                            319,
                            320,
                            321,
                            322,
                            323,
                            324,
                            325,
                            326,
                            327,
                            328,
                            329,
                            330,
                            331,
                            332,
                            333,
                            334,
                            335,
                            336,
                            337,
                            338
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+02",
                            "direction_type": "J2000"
                        }
                    },
                    {
                        "name": "target3",
                        "subbands": [
                            217,
                            218,
                            219,
                            220,
                            221,
                            222,
                            223,
                            224,
                            225,
                            226,
                            227,
                            228,
                            229,
                            230,
                            231,
                            232,
                            233,
                            234,
                            235,
                            236,
                            237,
                            238,
                            239,
                            240,
                            241,
                            242,
                            243,
                            244,
                            245,
                            246,
                            247,
                            248,
                            249,
                            250,
                            251,
                            252,
                            253,
                            254,
                            255,
                            256,
                            257,
                            258,
                            259,
                            260,
                            261,
                            262,
                            263,
                            264,
                            265,
                            266,
                            267,
                            268,
                            269,
                            270,
                            271,
                            272,
                            273,
                            274,
                            275,
                            276,
                            277,
                            278,
                            279,
                            280,
                            281,
                            282,
                            283,
                            284,
                            285,
                            286,
                            287,
                            288,
                            289,
                            290,
                            291,
                            292,
                            293,
                            294,
                            295,
                            296,
                            297,
                            298,
                            299,
                            300,
                            301,
                            302,
                            303,
                            304,
                            305,
                            306,
                            307,
                            308,
                            309,
                            310,
                            311,
                            312,
                            313,
                            314,
                            315,
                            316,
                            317,
                            318,
                            319,
                            320,
                            321,
                            322,
                            323,
                            324,
                            325,
                            326,
                            327,
                            328,
                            329,
                            330,
                            331,
                            332,
                            333,
                            334,
                            335,
                            336,
                            337,
                            338
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+03",
                            "direction_type": "J2000"
                        }
                    }
                ],
                "filter": "LBA_30_90",
                "tile_beam": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "_target_name_",
                    "direction_type": "J2000"
                },
                "antenna_set": "LBA_SPARSE_EVEN",
                "station_groups": [
                    {
                        "stations": [
                            "CS001",
                            "CS002",
                            "CS003",
                            "CS004",
                            "CS005",
                            "CS006",
                            "CS007",
                            "CS011",
                            "CS013",
                            "CS017",
                            "CS021",
                            "CS024",
                            "CS026",
                            "CS028",
                            "CS030",
                            "CS031",
                            "CS032",
                            "CS101",
                            "CS103",
                            "CS201",
                            "CS301",
                            "CS302",
                            "CS401",
                            "CS501",
                            "RS106",
                            "RS205",
                            "RS208",
                            "RS210",
                            "RS305",
                            "RS306",
                            "RS307",
                            "RS310",
                            "RS406",
                            "RS407",
                            "RS409",
                            "RS503",
                            "RS508",
                            "RS509"
                        ],
                        "max_nr_missing": 4
                    },
                    {
                        "stations": [
                            "RS508",
                            "RS509"
                        ],
                        "max_nr_missing": 1
                    },
                    {
                        "stations": [
                            "RS310",
                            "RS210"
                        ],
                        "max_nr_missing": 0
                    }
                ]
            }
        },
        "specifications_template": "http://localhost:3000/api/task_template/5",
        "specifications_template_id": 5,
        "tags": [],
        "task_blueprints": [
            "http://localhost:3000/api/task_blueprint/9"
        ],
        "task_blueprints_ids": [
            9
        ],
        "task_type": "observation",
        "updated_at": "2022-04-14T06:20:22.242394"
      },
      "duration": 720,
      "error_reason": "",
      "first_scheduling_relation": [],
      "first_scheduling_relation_ids": [],
      "name": "Combined Observation",
      "obsolete_since": null,
      "on_sky_duration": 120,
      "on_sky_start_time": "2022-04-14T06:42:19.054619",
      "on_sky_stop_time": "2022-04-14T06:44:19.054619",
      "output_pinned": false,
      "process_start_time": "2022-04-14T06:42:19.054619",
      "process_stop_time": "2022-04-14T06:54:19.054619",
      "produced_by": [],
      "produced_by_ids": [],
      "relative_start_time": 0,
      "relative_stop_time": 120,
      "scheduled_start_time": "2022-04-14T06:42:19.054619",
      "scheduled_stop_time": "2022-04-14T06:54:19.054619",
      "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/2",
      "scheduling_unit_blueprint_id": 2,
      "second_scheduling_relation": [],
      "second_scheduling_relation_ids": [],
      "short_description": "OOO.O Paaa+01 Paaa+02 Paaa+03",
      "specifications_doc": {
          "QA": {
              "plots": {
                  "enabled": true,
                  "autocorrelation": true,
                  "crosscorrelation": true
              },
              "file_conversion": {
                  "enabled": true,
                  "nr_of_subbands": -1,
                  "nr_of_timestamps": 256
              },
              "inspection_plots": "msplots"
          },
          "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
          "duration": 120,
          "calibrator": {
              "name": "calibrator",
              "pointing": {
                  "angle1": 0.6624317181687094,
                  "angle2": 1.5579526427549426,
                  "target": "3Cabc",
                  "direction_type": "J2000"
              },
              "autoselect": false
          },
          "correlator": {
              "storage_cluster": "CEP4",
              "integration_time": 1,
              "channels_per_subband": 64,
              "topocentric_frequency_correction": false
          },
          "station_configuration": {
              "SAPs": [
                  {
                      "name": "target1",
                      "subbands": [
                          217,
                          218,
                          219,
                          220,
                          221,
                          222,
                          223,
                          224,
                          225,
                          226,
                          227,
                          228,
                          229,
                          230,
                          231,
                          232,
                          233,
                          234,
                          235,
                          236,
                          237,
                          238,
                          239,
                          240,
                          241,
                          242,
                          243,
                          244,
                          245,
                          246,
                          247,
                          248,
                          249,
                          250,
                          251,
                          252,
                          253,
                          254,
                          255,
                          256,
                          257,
                          258,
                          259,
                          260,
                          261,
                          262,
                          263,
                          264,
                          265,
                          266,
                          267,
                          268,
                          269,
                          270,
                          271,
                          272,
                          273,
                          274,
                          275,
                          276,
                          277,
                          278,
                          279,
                          280,
                          281,
                          282,
                          283,
                          284,
                          285,
                          286,
                          287,
                          288,
                          289,
                          290,
                          291,
                          292,
                          293,
                          294,
                          295,
                          296,
                          297,
                          298,
                          299,
                          300,
                          301,
                          302,
                          303,
                          304,
                          305,
                          306,
                          307,
                          308,
                          309,
                          310,
                          311,
                          312,
                          313,
                          314,
                          315,
                          316,
                          317,
                          318,
                          319,
                          320,
                          321,
                          322,
                          323,
                          324,
                          325,
                          326,
                          327,
                          328,
                          329,
                          330,
                          331,
                          332,
                          333,
                          334,
                          335,
                          336,
                          337,
                          338
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+01",
                          "direction_type": "J2000"
                      }
                  },
                  {
                      "name": "target2",
                      "subbands": [
                          217,
                          218,
                          219,
                          220,
                          221,
                          222,
                          223,
                          224,
                          225,
                          226,
                          227,
                          228,
                          229,
                          230,
                          231,
                          232,
                          233,
                          234,
                          235,
                          236,
                          237,
                          238,
                          239,
                          240,
                          241,
                          242,
                          243,
                          244,
                          245,
                          246,
                          247,
                          248,
                          249,
                          250,
                          251,
                          252,
                          253,
                          254,
                          255,
                          256,
                          257,
                          258,
                          259,
                          260,
                          261,
                          262,
                          263,
                          264,
                          265,
                          266,
                          267,
                          268,
                          269,
                          270,
                          271,
                          272,
                          273,
                          274,
                          275,
                          276,
                          277,
                          278,
                          279,
                          280,
                          281,
                          282,
                          283,
                          284,
                          285,
                          286,
                          287,
                          288,
                          289,
                          290,
                          291,
                          292,
                          293,
                          294,
                          295,
                          296,
                          297,
                          298,
                          299,
                          300,
                          301,
                          302,
                          303,
                          304,
                          305,
                          306,
                          307,
                          308,
                          309,
                          310,
                          311,
                          312,
                          313,
                          314,
                          315,
                          316,
                          317,
                          318,
                          319,
                          320,
                          321,
                          322,
                          323,
                          324,
                          325,
                          326,
                          327,
                          328,
                          329,
                          330,
                          331,
                          332,
                          333,
                          334,
                          335,
                          336,
                          337,
                          338
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+02",
                          "direction_type": "J2000"
                      }
                  },
                  {
                      "name": "target3",
                      "subbands": [
                          217,
                          218,
                          219,
                          220,
                          221,
                          222,
                          223,
                          224,
                          225,
                          226,
                          227,
                          228,
                          229,
                          230,
                          231,
                          232,
                          233,
                          234,
                          235,
                          236,
                          237,
                          238,
                          239,
                          240,
                          241,
                          242,
                          243,
                          244,
                          245,
                          246,
                          247,
                          248,
                          249,
                          250,
                          251,
                          252,
                          253,
                          254,
                          255,
                          256,
                          257,
                          258,
                          259,
                          260,
                          261,
                          262,
                          263,
                          264,
                          265,
                          266,
                          267,
                          268,
                          269,
                          270,
                          271,
                          272,
                          273,
                          274,
                          275,
                          276,
                          277,
                          278,
                          279,
                          280,
                          281,
                          282,
                          283,
                          284,
                          285,
                          286,
                          287,
                          288,
                          289,
                          290,
                          291,
                          292,
                          293,
                          294,
                          295,
                          296,
                          297,
                          298,
                          299,
                          300,
                          301,
                          302,
                          303,
                          304,
                          305,
                          306,
                          307,
                          308,
                          309,
                          310,
                          311,
                          312,
                          313,
                          314,
                          315,
                          316,
                          317,
                          318,
                          319,
                          320,
                          321,
                          322,
                          323,
                          324,
                          325,
                          326,
                          327,
                          328,
                          329,
                          330,
                          331,
                          332,
                          333,
                          334,
                          335,
                          336,
                          337,
                          338
                      ],
                      "digital_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "Paaa+03",
                          "direction_type": "J2000"
                      }
                  }
              ],
              "filter": "LBA_30_90",
              "tile_beam": {
                  "angle1": 0.6624317181687094,
                  "angle2": 1.5579526427549426,
                  "target": "_target_name_",
                  "direction_type": "J2000"
              },
              "antenna_set": "LBA_SPARSE_EVEN",
              "station_groups": [
                  {
                      "stations": [
                          "CS001",
                          "CS002",
                          "CS003",
                          "CS004",
                          "CS005",
                          "CS006",
                          "CS007",
                          "CS011",
                          "CS013",
                          "CS017",
                          "CS021",
                          "CS024",
                          "CS026",
                          "CS028",
                          "CS030",
                          "CS031",
                          "CS032",
                          "CS101",
                          "CS103",
                          "CS201",
                          "CS301",
                          "CS302",
                          "CS401",
                          "CS501",
                          "RS106",
                          "RS205",
                          "RS208",
                          "RS210",
                          "RS305",
                          "RS306",
                          "RS307",
                          "RS310",
                          "RS406",
                          "RS407",
                          "RS409",
                          "RS503",
                          "RS508",
                          "RS509"
                      ],
                      "max_nr_missing": 4
                  },
                  {
                      "stations": [
                          "RS508",
                          "RS509"
                      ],
                      "max_nr_missing": 1
                  },
                  {
                      "stations": [
                          "RS310",
                          "RS210"
                      ],
                      "max_nr_missing": 0
                  }
              ]
          }
      },
      "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "parallel calibrator target observation"})).id,
      "specifications_template_id": (_.find(templates, {"name": "parallel calibrator target observation"})).id,
      "status": "scheduled",
      "status_value": "scheduled",
      "subtasks": [
          {
              "id": 2000011,
              "url": "http://localhost:3000/api/subtask/2000011",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "cluster": "http://localhost:3000/api/cluster/1",
              "cluster_id": 1,
              "cluster_name": "CEP4",
              "created_at": "2022-04-14T06:20:23.306638",
              "created_or_updated_by_user": null,
              "created_or_updated_by_user_id": null,
              "duration": 300,
              "error_reason": null,
              "global_identifier": "http://localhost:3000/api/sip_identifier/25",
              "global_identifier_id": 25,
              "global_parset_identifier": "http://localhost:3000/api/sip_identifier/26",
              "global_parset_identifier_id": 26,
              "inputs": [
                  "http://localhost:3000/api/subtask_input/15"
              ],
              "inputs_ids": [
                  15
              ],
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "outputs": [
                  "http://localhost:3000/api/subtask_output/11"
              ],
              "outputs_ids": [
                  11
              ],
              "primary": false,
              "process_start_time": "2022-04-14T06:44:19.054619",
              "process_stop_time": "2022-04-14T06:49:19.054619",
              "processing_feedback_doc": null,
              "processing_feedback_template": null,
              "processing_feedback_template_id": null,
              "scheduled_start_time": "2022-04-14T06:44:19.054619",
              "scheduled_stop_time": "2022-04-14T06:49:19.054619",
              "specifications_doc": {
                  "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/QA%20file%20conversion/2/ref_resolved",
                  "file_conversion": {
                      "enabled": true,
                      "nr_of_subbands": -1,
                      "nr_of_timestamps": 256
                  }
              },
              "specifications_template": "http://localhost:3000/api/subtask_template/5",
              "specifications_template_id": 5,
              "state": "http://localhost:3000/api/subtask_state/defined",
              "state_value": "defined",
              "subtask_type": "qa_files",
              "tags": [],
              "task_blueprint": "http://localhost:3000/api/task_blueprint/9",
              "task_blueprint_id": 9,
              "total_nr_of_dataproducts": 0,
              "total_size_of_dataproducts": 0,
              "total_size_of_dataproducts_on_filesysem": 0,
              "unschedulable_reason": null,
              "updated_at": "2022-04-14T06:21:05.477098"
          },
          {
              "id": 2000012,
              "url": "http://localhost:3000/api/subtask/2000012",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "cluster": "http://localhost:3000/api/cluster/1",
              "cluster_id": 1,
              "cluster_name": "CEP4",
              "created_at": "2022-04-14T06:20:23.393785",
              "created_or_updated_by_user": null,
              "created_or_updated_by_user_id": null,
              "duration": 300,
              "error_reason": null,
              "global_identifier": "http://localhost:3000/api/sip_identifier/27",
              "global_identifier_id": 27,
              "global_parset_identifier": "http://localhost:3000/api/sip_identifier/28",
              "global_parset_identifier_id": 28,
              "inputs": [
                  "http://localhost:3000/api/subtask_input/16"
              ],
              "inputs_ids": [
                  16
              ],
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "outputs": [
                  "http://localhost:3000/api/subtask_output/12"
              ],
              "outputs_ids": [
                  12
              ],
              "primary": false,
              "process_start_time": "2022-04-14T06:49:19.054619",
              "process_stop_time": "2022-04-14T06:54:19.054619",
              "processing_feedback_doc": null,
              "processing_feedback_template": null,
              "processing_feedback_template_id": null,
              "scheduled_start_time": "2022-04-14T06:49:19.054619",
              "scheduled_stop_time": "2022-04-14T06:54:19.054619",
              "specifications_doc": {
                  "plots": {
                      "enabled": true,
                      "autocorrelation": true,
                      "crosscorrelation": true
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/QA%20plots/2/ref_resolved"
              },
              "specifications_template": "http://localhost:3000/api/subtask_template/3",
              "specifications_template_id": 3,
              "state": "http://localhost:3000/api/subtask_state/defined",
              "state_value": "defined",
              "subtask_type": "qa_plots",
              "tags": [],
              "task_blueprint": "http://localhost:3000/api/task_blueprint/9",
              "task_blueprint_id": 9,
              "total_nr_of_dataproducts": 0,
              "total_size_of_dataproducts": 0,
              "total_size_of_dataproducts_on_filesysem": 0,
              "unschedulable_reason": null,
              "updated_at": "2022-04-14T06:21:05.493633"
          },
          {
              "id": 2000010,
              "url": "http://localhost:3000/api/subtask/2000010",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "cluster": "http://localhost:3000/api/cluster/1",
              "cluster_id": 1,
              "cluster_name": "CEP4",
              "created_at": "2022-04-14T06:20:23.194140",
              "created_or_updated_by_user": null,
              "created_or_updated_by_user_id": null,
              "duration": 120,
              "error_reason": null,
              "global_identifier": "http://localhost:3000/api/sip_identifier/23",
              "global_identifier_id": 23,
              "global_parset_identifier": "http://localhost:3000/api/sip_identifier/24",
              "global_parset_identifier_id": 24,
              "inputs": [],
              "inputs_ids": [],
              "obsolete_since": null,
              "on_sky_duration": "00:02:00",
              "on_sky_start_time": "2022-04-14T06:42:19.054619",
              "on_sky_stop_time": "2022-04-14T06:44:19.054619",
              "outputs": [
                  "http://localhost:3000/api/subtask_output/10"
              ],
              "outputs_ids": [
                  10
              ],
              "primary": true,
              "process_start_time": "2022-04-14T06:42:19.054619",
              "process_stop_time": "2022-04-14T06:44:19.054619",
              "processing_feedback_doc": null,
              "processing_feedback_template": null,
              "processing_feedback_template_id": null,
              "scheduled_start_time": "2022-04-14T06:42:19.054619",
              "scheduled_stop_time": "2022-04-14T06:44:19.054619",
              "specifications_doc": {
                  "QA": {
                      "inspection_plots": "msplots"
                  },
                  "COBALT": {
                      "debug": {
                          "write_output": true,
                          "benchmark_file": "",
                          "annotate_broken_antennas": true
                      },
                      "version": 1,
                      "blocksize": 195584,
                      "resources": {
                          "writer_nodes": [
                              "cpu01.cep4.infiniband.lofar",
                              "cpu02.cep4.infiniband.lofar",
                              "cpu03.cep4.infiniband.lofar",
                              "cpu04.cep4.infiniband.lofar",
                              "cpu05.cep4.infiniband.lofar"
                          ],
                          "processing_nodes": [
                              "cbt201_0",
                              "cbt201_1",
                              "cbt202_0",
                              "cbt202_1",
                              "cbt203_0",
                              "cbt203_1",
                              "cbt204_0",
                              "cbt204_1",
                              "cbt205_0",
                              "cbt205_1",
                              "cbt207_0",
                              "cbt207_1",
                              "cbt208_0",
                              "cbt208_1",
                              "cbt209_0",
                              "cbt209_1",
                              "cbt210_0",
                              "cbt210_1",
                              "cbt211_0",
                              "cbt211_1",
                              "cbt212_0",
                              "cbt212_1"
                          ]
                      },
                      "beamformer": {
                          "input_ppf": false,
                          "tab_pipelines": [],
                          "flyseye_pipelines": []
                      },
                      "correlator": {
                          "enabled": true,
                          "doppler_correction": false,
                          "channels_per_subband": 64,
                          "blocks_per_integration": 1,
                          "integrations_per_block": 1
                      },
                      "delay_compensation": true,
                      "bandpass_correction": true
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/observation%20control/2/ref_resolved",
                  "stations": {
                      "filter": "LBA_30_90",
                      "antenna_set": "LBA_SPARSE_EVEN",
                      "station_list": [
                          "CS001",
                          "CS002",
                          "CS003",
                          "CS004",
                          "CS005",
                          "CS006",
                          "CS007",
                          "CS011",
                          "CS013",
                          "CS017",
                          "CS021",
                          "CS024",
                          "CS026",
                          "CS028",
                          "CS030",
                          "CS031",
                          "CS032",
                          "CS101",
                          "CS103",
                          "CS201",
                          "CS301",
                          "CS302",
                          "CS401",
                          "CS501",
                          "RS106",
                          "RS205",
                          "RS208",
                          "RS210",
                          "RS305",
                          "RS306",
                          "RS307",
                          "RS310",
                          "RS406",
                          "RS407",
                          "RS409",
                          "RS503",
                          "RS508",
                          "RS509"
                      ],
                      "analog_pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "3Cabc",
                          "direction_type": "J2000"
                      },
                      "digital_pointings": [
                          {
                              "name": "target1",
                              "pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+01",
                                  "direction_type": "J2000"
                              },
                              "subbands": [
                                  217,
                                  218,
                                  219,
                                  220,
                                  221,
                                  222,
                                  223,
                                  224,
                                  225,
                                  226,
                                  227,
                                  228,
                                  229,
                                  230,
                                  231,
                                  232,
                                  233,
                                  234,
                                  235,
                                  236,
                                  237,
                                  238,
                                  239,
                                  240,
                                  241,
                                  242,
                                  243,
                                  244,
                                  245,
                                  246,
                                  247,
                                  248,
                                  249,
                                  250,
                                  251,
                                  252,
                                  253,
                                  254,
                                  255,
                                  256,
                                  257,
                                  258,
                                  259,
                                  260,
                                  261,
                                  262,
                                  263,
                                  264,
                                  265,
                                  266,
                                  267,
                                  268,
                                  269,
                                  270,
                                  271,
                                  272,
                                  273,
                                  274,
                                  275,
                                  276,
                                  277,
                                  278,
                                  279,
                                  280,
                                  281,
                                  282,
                                  283,
                                  284,
                                  285,
                                  286,
                                  287,
                                  288,
                                  289,
                                  290,
                                  291,
                                  292,
                                  293,
                                  294,
                                  295,
                                  296,
                                  297,
                                  298,
                                  299,
                                  300,
                                  301,
                                  302,
                                  303,
                                  304,
                                  305,
                                  306,
                                  307,
                                  308,
                                  309,
                                  310,
                                  311,
                                  312,
                                  313,
                                  314,
                                  315,
                                  316,
                                  317,
                                  318,
                                  319,
                                  320,
                                  321,
                                  322,
                                  323,
                                  324,
                                  325,
                                  326,
                                  327,
                                  328,
                                  329,
                                  330,
                                  331,
                                  332,
                                  333,
                                  334,
                                  335,
                                  336,
                                  337,
                                  338
                              ]
                          },
                          {
                              "name": "target2",
                              "pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+02",
                                  "direction_type": "J2000"
                              },
                              "subbands": [
                                  217,
                                  218,
                                  219,
                                  220,
                                  221,
                                  222,
                                  223,
                                  224,
                                  225,
                                  226,
                                  227,
                                  228,
                                  229,
                                  230,
                                  231,
                                  232,
                                  233,
                                  234,
                                  235,
                                  236,
                                  237,
                                  238,
                                  239,
                                  240,
                                  241,
                                  242,
                                  243,
                                  244,
                                  245,
                                  246,
                                  247,
                                  248,
                                  249,
                                  250,
                                  251,
                                  252,
                                  253,
                                  254,
                                  255,
                                  256,
                                  257,
                                  258,
                                  259,
                                  260,
                                  261,
                                  262,
                                  263,
                                  264,
                                  265,
                                  266,
                                  267,
                                  268,
                                  269,
                                  270,
                                  271,
                                  272,
                                  273,
                                  274,
                                  275,
                                  276,
                                  277,
                                  278,
                                  279,
                                  280,
                                  281,
                                  282,
                                  283,
                                  284,
                                  285,
                                  286,
                                  287,
                                  288,
                                  289,
                                  290,
                                  291,
                                  292,
                                  293,
                                  294,
                                  295,
                                  296,
                                  297,
                                  298,
                                  299,
                                  300,
                                  301,
                                  302,
                                  303,
                                  304,
                                  305,
                                  306,
                                  307,
                                  308,
                                  309,
                                  310,
                                  311,
                                  312,
                                  313,
                                  314,
                                  315,
                                  316,
                                  317,
                                  318,
                                  319,
                                  320,
                                  321,
                                  322,
                                  323,
                                  324,
                                  325,
                                  326,
                                  327,
                                  328,
                                  329,
                                  330,
                                  331,
                                  332,
                                  333,
                                  334,
                                  335,
                                  336,
                                  337,
                                  338
                              ]
                          },
                          {
                              "name": "target3",
                              "pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+03",
                                  "direction_type": "J2000"
                              },
                              "subbands": [
                                  217,
                                  218,
                                  219,
                                  220,
                                  221,
                                  222,
                                  223,
                                  224,
                                  225,
                                  226,
                                  227,
                                  228,
                                  229,
                                  230,
                                  231,
                                  232,
                                  233,
                                  234,
                                  235,
                                  236,
                                  237,
                                  238,
                                  239,
                                  240,
                                  241,
                                  242,
                                  243,
                                  244,
                                  245,
                                  246,
                                  247,
                                  248,
                                  249,
                                  250,
                                  251,
                                  252,
                                  253,
                                  254,
                                  255,
                                  256,
                                  257,
                                  258,
                                  259,
                                  260,
                                  261,
                                  262,
                                  263,
                                  264,
                                  265,
                                  266,
                                  267,
                                  268,
                                  269,
                                  270,
                                  271,
                                  272,
                                  273,
                                  274,
                                  275,
                                  276,
                                  277,
                                  278,
                                  279,
                                  280,
                                  281,
                                  282,
                                  283,
                                  284,
                                  285,
                                  286,
                                  287,
                                  288,
                                  289,
                                  290,
                                  291,
                                  292,
                                  293,
                                  294,
                                  295,
                                  296,
                                  297,
                                  298,
                                  299,
                                  300,
                                  301,
                                  302,
                                  303,
                                  304,
                                  305,
                                  306,
                                  307,
                                  308,
                                  309,
                                  310,
                                  311,
                                  312,
                                  313,
                                  314,
                                  315,
                                  316,
                                  317,
                                  318,
                                  319,
                                  320,
                                  321,
                                  322,
                                  323,
                                  324,
                                  325,
                                  326,
                                  327,
                                  328,
                                  329,
                                  330,
                                  331,
                                  332,
                                  333,
                                  334,
                                  335,
                                  336,
                                  337,
                                  338
                              ]
                          },
                          {
                              "name": "calibrator",
                              "pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "3Cabc",
                                  "direction_type": "J2000"
                              },
                              "subbands": [
                                  217,
                                  218,
                                  219,
                                  220,
                                  221,
                                  222,
                                  223,
                                  224,
                                  225,
                                  226,
                                  227,
                                  228,
                                  229,
                                  230,
                                  231,
                                  232,
                                  233,
                                  234,
                                  235,
                                  236,
                                  237,
                                  238,
                                  239,
                                  240,
                                  241,
                                  242,
                                  243,
                                  244,
                                  245,
                                  246,
                                  247,
                                  248,
                                  249,
                                  250,
                                  251,
                                  252,
                                  253,
                                  254,
                                  255,
                                  256,
                                  257,
                                  258,
                                  259,
                                  260,
                                  261,
                                  262,
                                  263,
                                  264,
                                  265,
                                  266,
                                  267,
                                  268,
                                  269,
                                  270,
                                  271,
                                  272,
                                  273,
                                  274,
                                  275,
                                  276,
                                  277,
                                  278,
                                  279,
                                  280,
                                  281,
                                  282,
                                  283,
                                  284,
                                  285,
                                  286,
                                  287,
                                  288,
                                  289,
                                  290,
                                  291,
                                  292,
                                  293,
                                  294,
                                  295,
                                  296,
                                  297,
                                  298,
                                  299,
                                  300,
                                  301,
                                  302,
                                  303,
                                  304,
                                  305,
                                  306,
                                  307,
                                  308,
                                  309,
                                  310,
                                  311,
                                  312,
                                  313,
                                  314,
                                  315,
                                  316,
                                  317,
                                  318,
                                  319,
                                  320,
                                  321,
                                  322,
                                  323,
                                  324,
                                  325,
                                  326,
                                  327,
                                  328,
                                  329,
                                  330,
                                  331,
                                  332,
                                  333,
                                  334,
                                  335,
                                  336,
                                  337,
                                  338
                              ]
                          }
                      ]
                  }
              },
              "specifications_template": "http://localhost:3000/api/subtask_template/4",
              "specifications_template_id": 4,
              "state": "http://localhost:3000/api/subtask_state/scheduled",
              "state_value": "scheduled",
              "subtask_type": "observation",
              "tags": [],
              "task_blueprint": "http://localhost:3000/api/task_blueprint/9",
              "task_blueprint_id": 9,
              "total_nr_of_dataproducts": 0,
              "total_size_of_dataproducts": 0,
              "total_size_of_dataproducts_on_filesysem": 0,
              "unschedulable_reason": null,
              "updated_at": "2022-04-14T06:21:05.404984"
          }
      ],
      "subtasks_ids": [
          2000011,
          2000012,
          2000010
      ],
      "tags": [],
      "task_type": "observation",
      "unschedulable_reason": "",
      "updated_at": "2022-04-14T06:20:22.577280",
      "predecessors": [],
      "successors": [
        {
            "id": 11,
            "url": "http://localhost:3000/api/task_blueprint/11",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/20",
                "http://localhost:3000/api/task_relation_blueprint/24"
            ],
            "consumed_by_ids": [
                20,
                24
            ],
            "created_at": "2022-04-14T06:20:22.664403",
            "description": "Preprocessing Pipeline for Target Observation target3",
            "draft": "http://localhost:3000/api/task_draft/11",
            "draft_id": 11,
            "duration": 120,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target3",
            "obsolete_since": null,
            "on_sky_duration": null,
            "on_sky_start_time": null,
            "on_sky_stop_time": null,
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:44:19.054619",
            "process_stop_time": "2022-04-14T06:46:19.054619",
            "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/14"
            ],
            "produced_by_ids": [
                14
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduled_start_time": "2022-04-14T06:44:19.054619",
            "scheduled_stop_time": "2022-04-14T06:46:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/2",
            "scheduling_unit_blueprint_id": 2,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "c17 oOOO.O Paaa+03",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 8,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 4,
                    "frequency_steps": 8
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "status": "schedulable",
            "status_value": "schedulable",
            "subtasks": [
                {
                    "id": 2000013,
                    "url": "http://localhost:3000/api/subtask/2000013",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "cluster": "http://localhost:3000/api/cluster/1",
                    "cluster_id": 1,
                    "cluster_name": "CEP4",
                    "created_at": "2022-04-14T06:20:23.460156",
                    "created_or_updated_by_user": null,
                    "created_or_updated_by_user_id": null,
                    "duration": 120,
                    "error_reason": null,
                    "global_identifier": "http://localhost:3000/api/sip_identifier/29",
                    "global_identifier_id": 29,
                    "global_parset_identifier": "http://localhost:3000/api/sip_identifier/30",
                    "global_parset_identifier_id": 30,
                    "inputs": [
                        "http://localhost:3000/api/subtask_input/17"
                    ],
                    "inputs_ids": [
                        17
                    ],
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "outputs": [
                        "http://localhost:3000/api/subtask_output/13"
                    ],
                    "outputs_ids": [
                        13
                    ],
                    "primary": true,
                    "process_start_time": "2022-04-14T06:44:19.054619",
                    "process_stop_time": "2022-04-14T06:46:19.054619",
                    "processing_feedback_doc": null,
                    "processing_feedback_template": null,
                    "processing_feedback_template_id": null,
                    "scheduled_start_time": "2022-04-14T06:44:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:46:19.054619",
                    "specifications_doc": {
                        "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "demixer": {
                            "enabled": true,
                            "baselines": "CS*,RS*&",
                            "time_steps": 4,
                            "demix_always": [],
                            "ignore_target": false,
                            "demix_if_needed": [],
                            "frequency_steps": 8,
                            "demix_time_steps": 8,
                            "demix_frequency_steps": 64
                        },
                        "aoflagger": {
                            "enabled": true,
                            "strategy": "LBAdefault"
                        },
                        "preflagger0": {
                            "enabled": true,
                            "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                        },
                        "preflagger1": {
                            "enabled": true,
                            "corrtype": "auto"
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/subtask_template/7",
                    "specifications_template_id": 7,
                    "state": "http://localhost:3000/api/subtask_state/defined",
                    "state_value": "defined",
                    "subtask_type": "pipeline",
                    "tags": [],
                    "task_blueprint": "http://localhost:3000/api/task_blueprint/11",
                    "task_blueprint_id": 11,
                    "total_nr_of_dataproducts": 0,
                    "total_size_of_dataproducts": 0,
                    "total_size_of_dataproducts_on_filesysem": 0,
                    "unschedulable_reason": null,
                    "updated_at": "2022-04-14T06:21:05.614265"
                }
            ],
            "subtasks_ids": [
                2000013
            ],
            "tags": [],
            "task_type": "pipeline",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:22.664415"
        },
        {
            "id": 13,
            "url": "http://localhost:3000/api/task_blueprint/13",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/18",
                "http://localhost:3000/api/task_relation_blueprint/22"
            ],
            "consumed_by_ids": [
                18,
                22
            ],
            "created_at": "2022-04-14T06:20:22.753080",
            "description": "Preprocessing Pipeline for Target Observation target1",
            "draft": "http://localhost:3000/api/task_draft/13",
            "draft_id": 13,
            "duration": 120,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target1",
            "obsolete_since": null,
            "on_sky_duration": null,
            "on_sky_start_time": null,
            "on_sky_stop_time": null,
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:44:19.054619",
            "process_stop_time": "2022-04-14T06:46:19.054619",
            "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/16"
            ],
            "produced_by_ids": [
                16
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduled_start_time": "2022-04-14T06:44:19.054619",
            "scheduled_stop_time": "2022-04-14T06:46:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/2",
            "scheduling_unit_blueprint_id": 2,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "c17 oOOO.O Paaa+01",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 8,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 4,
                    "frequency_steps": 8
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "status": "schedulable",
            "status_value": "schedulable",
            "subtasks": [
                {
                    "id": 2000014,
                    "url": "http://localhost:3000/api/subtask/2000014",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "cluster": "http://localhost:3000/api/cluster/1",
                    "cluster_id": 1,
                    "cluster_name": "CEP4",
                    "created_at": "2022-04-14T06:20:23.561018",
                    "created_or_updated_by_user": null,
                    "created_or_updated_by_user_id": null,
                    "duration": 120,
                    "error_reason": null,
                    "global_identifier": "http://localhost:3000/api/sip_identifier/31",
                    "global_identifier_id": 31,
                    "global_parset_identifier": "http://localhost:3000/api/sip_identifier/32",
                    "global_parset_identifier_id": 32,
                    "inputs": [
                        "http://localhost:3000/api/subtask_input/18"
                    ],
                    "inputs_ids": [
                        18
                    ],
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "outputs": [
                        "http://localhost:3000/api/subtask_output/14"
                    ],
                    "outputs_ids": [
                        14
                    ],
                    "primary": true,
                    "process_start_time": "2022-04-14T06:44:19.054619",
                    "process_stop_time": "2022-04-14T06:46:19.054619",
                    "processing_feedback_doc": null,
                    "processing_feedback_template": null,
                    "processing_feedback_template_id": null,
                    "scheduled_start_time": "2022-04-14T06:44:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:46:19.054619",
                    "specifications_doc": {
                        "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "demixer": {
                            "enabled": true,
                            "baselines": "CS*,RS*&",
                            "time_steps": 4,
                            "demix_always": [],
                            "ignore_target": false,
                            "demix_if_needed": [],
                            "frequency_steps": 8,
                            "demix_time_steps": 8,
                            "demix_frequency_steps": 64
                        },
                        "aoflagger": {
                            "enabled": true,
                            "strategy": "LBAdefault"
                        },
                        "preflagger0": {
                            "enabled": true,
                            "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                        },
                        "preflagger1": {
                            "enabled": true,
                            "corrtype": "auto"
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/subtask_template/7",
                    "specifications_template_id": 7,
                    "state": "http://localhost:3000/api/subtask_state/defined",
                    "state_value": "defined",
                    "subtask_type": "pipeline",
                    "tags": [],
                    "task_blueprint": "http://localhost:3000/api/task_blueprint/13",
                    "task_blueprint_id": 13,
                    "total_nr_of_dataproducts": 0,
                    "total_size_of_dataproducts": 0,
                    "total_size_of_dataproducts_on_filesysem": 0,
                    "unschedulable_reason": null,
                    "updated_at": "2022-04-14T06:21:05.434545"
                }
            ],
            "subtasks_ids": [
                2000014
            ],
            "tags": [],
            "task_type": "pipeline",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:22.753094"
        },
        {
            "id": 10,
            "url": "http://localhost:3000/api/task_blueprint/10",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/21"
            ],
            "consumed_by_ids": [
                21
            ],
            "created_at": "2022-04-14T06:20:22.628307",
            "description": "Preprocessing Pipeline for Calibrator Observation",
            "draft": "http://localhost:3000/api/task_draft/10",
            "draft_id": 10,
            "duration": 120,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Calibrator Pipeline",
            "obsolete_since": null,
            "on_sky_duration": null,
            "on_sky_start_time": null,
            "on_sky_stop_time": null,
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:44:19.054619",
            "process_stop_time": "2022-04-14T06:46:19.054619",
            "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/13"
            ],
            "produced_by_ids": [
                13
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduled_start_time": "2022-04-14T06:44:19.054619",
            "scheduled_stop_time": "2022-04-14T06:46:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/2",
            "scheduling_unit_blueprint_id": 2,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "c17 oOOO.O 3Cabc",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 8,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 4,
                    "frequency_steps": 8
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "status": "schedulable",
            "status_value": "schedulable",
            "subtasks": [
                {
                    "id": 2000015,
                    "url": "http://localhost:3000/api/subtask/2000015",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "cluster": "http://localhost:3000/api/cluster/1",
                    "cluster_id": 1,
                    "cluster_name": "CEP4",
                    "created_at": "2022-04-14T06:20:23.664286",
                    "created_or_updated_by_user": null,
                    "created_or_updated_by_user_id": null,
                    "duration": 120,
                    "error_reason": null,
                    "global_identifier": "http://localhost:3000/api/sip_identifier/33",
                    "global_identifier_id": 33,
                    "global_parset_identifier": "http://localhost:3000/api/sip_identifier/34",
                    "global_parset_identifier_id": 34,
                    "inputs": [
                        "http://localhost:3000/api/subtask_input/19"
                    ],
                    "inputs_ids": [
                        19
                    ],
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "outputs": [
                        "http://localhost:3000/api/subtask_output/15"
                    ],
                    "outputs_ids": [
                        15
                    ],
                    "primary": true,
                    "process_start_time": "2022-04-14T06:44:19.054619",
                    "process_stop_time": "2022-04-14T06:46:19.054619",
                    "processing_feedback_doc": null,
                    "processing_feedback_template": null,
                    "processing_feedback_template_id": null,
                    "scheduled_start_time": "2022-04-14T06:44:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:46:19.054619",
                    "specifications_doc": {
                        "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "demixer": {
                            "enabled": true,
                            "baselines": "CS*,RS*&",
                            "time_steps": 4,
                            "demix_always": [],
                            "ignore_target": false,
                            "demix_if_needed": [],
                            "frequency_steps": 8,
                            "demix_time_steps": 8,
                            "demix_frequency_steps": 64
                        },
                        "aoflagger": {
                            "enabled": true,
                            "strategy": "LBAdefault"
                        },
                        "preflagger0": {
                            "enabled": true,
                            "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                        },
                        "preflagger1": {
                            "enabled": true,
                            "corrtype": "auto"
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/subtask_template/7",
                    "specifications_template_id": 7,
                    "state": "http://localhost:3000/api/subtask_state/defined",
                    "state_value": "defined",
                    "subtask_type": "pipeline",
                    "tags": [],
                    "task_blueprint": "http://localhost:3000/api/task_blueprint/10",
                    "task_blueprint_id": 10,
                    "total_nr_of_dataproducts": 0,
                    "total_size_of_dataproducts": 0,
                    "total_size_of_dataproducts_on_filesysem": 0,
                    "unschedulable_reason": null,
                    "updated_at": "2022-04-14T06:21:05.580304"
                }
            ],
            "subtasks_ids": [
                2000015
            ],
            "tags": [],
            "task_type": "pipeline",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:22.628318"
        },
        {
            "id": 14,
            "url": "http://localhost:3000/api/task_blueprint/14",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [],
            "consumed_by_ids": [],
            "created_at": "2022-04-14T06:20:22.793703",
            "description": "Clean up all dataproducts from disk after ingest",
            "draft": "http://localhost:3000/api/task_draft/14",
            "draft_id": 14,
            "duration": 300,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Cleanup",
            "obsolete_since": null,
            "on_sky_duration": null,
            "on_sky_start_time": null,
            "on_sky_stop_time": null,
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:46:19.054619",
            "process_stop_time": "2022-04-14T06:51:19.054619",
            "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/17",
                "http://localhost:3000/api/task_relation_blueprint/18",
                "http://localhost:3000/api/task_relation_blueprint/19",
                "http://localhost:3000/api/task_relation_blueprint/20"
            ],
            "produced_by_ids": [
                17,
                18,
                19,
                20
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduled_start_time": "2022-04-14T06:46:19.054619",
            "scheduled_stop_time": "2022-04-14T06:51:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/2",
            "scheduling_unit_blueprint_id": 2,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "",
            "specifications_doc": {
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2/ref_resolved"
            },
            "specifications_template": "http://localhost:3000/api/task_template/1",
            "specifications_template_id": 1,
            "status": "schedulable",
            "status_value": "schedulable",
            "subtasks": [
                {
                    "id": 2000018,
                    "url": "http://localhost:3000/api/subtask/2000018",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "cluster": "http://localhost:3000/api/cluster/1",
                    "cluster_id": 1,
                    "cluster_name": "CEP4",
                    "created_at": "2022-04-14T06:20:23.996875",
                    "created_or_updated_by_user": null,
                    "created_or_updated_by_user_id": null,
                    "duration": 300,
                    "error_reason": null,
                    "global_identifier": "http://localhost:3000/api/sip_identifier/39",
                    "global_identifier_id": 39,
                    "global_parset_identifier": "http://localhost:3000/api/sip_identifier/40",
                    "global_parset_identifier_id": 40,
                    "inputs": [
                        "http://localhost:3000/api/subtask_input/25",
                        "http://localhost:3000/api/subtask_input/26",
                        "http://localhost:3000/api/subtask_input/27",
                        "http://localhost:3000/api/subtask_input/28",
                        "http://localhost:3000/api/subtask_input/29"
                    ],
                    "inputs_ids": [
                        25,
                        26,
                        27,
                        28,
                        29
                    ],
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "outputs": [],
                    "outputs_ids": [],
                    "primary": true,
                    "process_start_time": "2022-04-14T06:46:19.054619",
                    "process_stop_time": "2022-04-14T06:51:19.054619",
                    "processing_feedback_doc": null,
                    "processing_feedback_template": null,
                    "processing_feedback_template_id": null,
                    "scheduled_start_time": "2022-04-14T06:46:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:51:19.054619",
                    "specifications_doc": {
                        "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/cleanup/2/ref_resolved"
                    },
                    "specifications_template": "http://localhost:3000/api/subtask_template/1",
                    "specifications_template_id": 1,
                    "state": "http://localhost:3000/api/subtask_state/defined",
                    "state_value": "defined",
                    "subtask_type": "cleanup",
                    "tags": [],
                    "task_blueprint": "http://localhost:3000/api/task_blueprint/14",
                    "task_blueprint_id": 14,
                    "total_nr_of_dataproducts": 0,
                    "total_size_of_dataproducts": 0,
                    "total_size_of_dataproducts_on_filesysem": 0,
                    "unschedulable_reason": null,
                    "updated_at": "2022-04-14T06:21:05.638839"
                }
            ],
            "subtasks_ids": [
                2000018
            ],
            "tags": [],
            "task_type": "cleanup",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:22.793715"
        },
        {
            "id": 12,
            "url": "http://localhost:3000/api/task_blueprint/12",
            "actual_on_sky_start_time": null,
            "actual_on_sky_stop_time": null,
            "actual_process_start_time": null,
            "actual_process_stop_time": null,
            "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/19",
                "http://localhost:3000/api/task_relation_blueprint/23"
            ],
            "consumed_by_ids": [
                19,
                23
            ],
            "created_at": "2022-04-14T06:20:22.708297",
            "description": "Preprocessing Pipeline for Target Observation target2",
            "draft": "http://localhost:3000/api/task_draft/12",
            "draft_id": 12,
            "duration": 120,
            "error_reason": "",
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline target2",
            "obsolete_since": null,
            "on_sky_duration": null,
            "on_sky_start_time": null,
            "on_sky_stop_time": null,
            "output_pinned": false,
            "process_start_time": "2022-04-14T06:44:19.054619",
            "process_stop_time": "2022-04-14T06:46:19.054619",
            "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/15"
            ],
            "produced_by_ids": [
                15
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduled_start_time": "2022-04-14T06:44:19.054619",
            "scheduled_stop_time": "2022-04-14T06:46:19.054619",
            "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/2",
            "scheduling_unit_blueprint_id": 2,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "c17 oOOO.O Paaa+02",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "LBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 8,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 4,
                    "frequency_steps": 8
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "status": "schedulable",
            "status_value": "schedulable",
            "subtasks": [
                {
                    "id": 2000016,
                    "url": "http://localhost:3000/api/subtask/2000016",
                    "actual_on_sky_start_time": null,
                    "actual_on_sky_stop_time": null,
                    "actual_process_start_time": null,
                    "actual_process_stop_time": null,
                    "cluster": "http://localhost:3000/api/cluster/1",
                    "cluster_id": 1,
                    "cluster_name": "CEP4",
                    "created_at": "2022-04-14T06:20:23.770987",
                    "created_or_updated_by_user": null,
                    "created_or_updated_by_user_id": null,
                    "duration": 120,
                    "error_reason": null,
                    "global_identifier": "http://localhost:3000/api/sip_identifier/35",
                    "global_identifier_id": 35,
                    "global_parset_identifier": "http://localhost:3000/api/sip_identifier/36",
                    "global_parset_identifier_id": 36,
                    "inputs": [
                        "http://localhost:3000/api/subtask_input/20"
                    ],
                    "inputs_ids": [
                        20
                    ],
                    "obsolete_since": null,
                    "on_sky_duration": null,
                    "on_sky_start_time": null,
                    "on_sky_stop_time": null,
                    "outputs": [
                        "http://localhost:3000/api/subtask_output/16"
                    ],
                    "outputs_ids": [
                        16
                    ],
                    "primary": true,
                    "process_start_time": "2022-04-14T06:44:19.054619",
                    "process_stop_time": "2022-04-14T06:46:19.054619",
                    "processing_feedback_doc": null,
                    "processing_feedback_template": null,
                    "processing_feedback_template_id": null,
                    "scheduled_start_time": "2022-04-14T06:44:19.054619",
                    "scheduled_stop_time": "2022-04-14T06:46:19.054619",
                    "specifications_doc": {
                        "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "demixer": {
                            "enabled": true,
                            "baselines": "CS*,RS*&",
                            "time_steps": 4,
                            "demix_always": [],
                            "ignore_target": false,
                            "demix_if_needed": [],
                            "frequency_steps": 8,
                            "demix_time_steps": 8,
                            "demix_frequency_steps": 64
                        },
                        "aoflagger": {
                            "enabled": true,
                            "strategy": "LBAdefault"
                        },
                        "preflagger0": {
                            "enabled": true,
                            "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                        },
                        "preflagger1": {
                            "enabled": true,
                            "corrtype": "auto"
                        },
                        "storagemanager": "dysco",
                        "cluster_resources": {
                            "where": {
                                "cluster": "CEP4",
                                "partition": "cpu"
                            },
                            "cores_per_task": 2,
                            "parallel_tasks": 122
                        }
                    },
                    "specifications_template": "http://localhost:3000/api/subtask_template/7",
                    "specifications_template_id": 7,
                    "state": "http://localhost:3000/api/subtask_state/defined",
                    "state_value": "defined",
                    "subtask_type": "pipeline",
                    "tags": [],
                    "task_blueprint": "http://localhost:3000/api/task_blueprint/12",
                    "task_blueprint_id": 12,
                    "total_nr_of_dataproducts": 0,
                    "total_size_of_dataproducts": 0,
                    "total_size_of_dataproducts_on_filesysem": 0,
                    "unschedulable_reason": null,
                    "updated_at": "2022-04-14T06:21:05.520012"
                }
            ],
            "subtasks_ids": [
                2000016
            ],
            "tags": [],
            "task_type": "pipeline",
            "unschedulable_reason": "",
            "updated_at": "2022-04-14T06:20:22.708311"
        }
      ]
    },
    {
        "id": 22,
        "url": "http://localhost:3000/api/task_blueprint/22",
        "actual_on_sky_start_time": null,
        "actual_on_sky_stop_time": null,
        "actual_process_start_time": null,
        "actual_process_stop_time": null,
        "consumed_by": [
            "http://localhost:3000/api/task_relation_blueprint/34"
        ],
        "consumed_by_ids": [
            34
        ],
        "created_at": "2022-05-09T05:13:05.319201",
        "description": "Calibrator Observation 1. Before the Target Observation",
        "draft": "http://localhost:3000/api/task_draft/22",
        "draft_id": 22,
        "draftObject": {
            "id": 22,
            "url": "http://localhost:3000/api/task_draft/22",
            "connector_types": [
                {
                    "id": 1,
                    "url": "http://localhost:3000/api/task_connector_type/1",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/correlator",
                    "role_value": "correlator",
                    "task_template": "http://localhost:3000/api/task_template/7",
                    "task_template_id": 7
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/34"
            ],
            "consumed_by_ids": [
                34
            ],
            "created_at": "2022-05-09T05:13:04.155696",
            "description": "Calibrator Observation 1. Before the Target Observation",
            "duration": 600,
            "first_scheduling_relation": [
                "http://localhost:3000/api/task_scheduling_relation_draft/1"
            ],
            "first_scheduling_relation_ids": [
                1
            ],
            "name": "Calibrator Observation 1",
            "output_pinned": false,
            "produced_by": [],
            "produced_by_ids": [],
            "relative_start_time": 0,
            "relative_stop_time": 600,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/4",
            "scheduling_unit_draft_id": 4,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "Cal1 3Cabc",
            "specifications_doc": {
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2/ref_resolved",
                "duration": 600,
                "calibrator": {
                    "name": "calibrator1",
                    "pointing": {
                        "angle1": 0.6624317181687094,
                        "angle2": 1.5579526427549426,
                        "target": "3Cabc",
                        "direction_type": "J2000"
                    },
                    "autoselect": false
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/7",
            "specifications_template_id": 7,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/22"
            ],
            "task_blueprints_ids": [
                22
            ],
            "task_type": "observation",
            "updated_at": "2022-05-09T05:13:04.155710"
        },
        "duration": 1200,
        "error_reason": "",
        "first_scheduling_relation": [
            "http://localhost:3000/api/task_scheduling_relation_blueprint/1"
        ],
        "first_scheduling_relation_ids": [
            1
        ],
        "name": "Calibrator Observation 1",
        "obsolete_since": null,
        "on_sky_duration": 600,
        "on_sky_start_time": "2022-05-09T06:08:49.932056",
        "on_sky_stop_time": "2022-05-09T06:18:49.932056",
        "output_pinned": false,
        "process_start_time": "2022-05-09T06:08:49.932056",
        "process_stop_time": "2022-05-09T06:28:49.932056",
        "produced_by": [],
        "produced_by_ids": [],
        "relative_start_time": 0,
        "relative_stop_time": 600,
        "scheduled_start_time": "2022-05-09T06:08:49.932056",
        "scheduled_stop_time": "2022-05-09T06:28:49.932056",
        "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
        "scheduling_unit_blueprint_id": 4,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "Cal1 3Cabc",
        "specifications_doc": {
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2/ref_resolved",
            "duration": 600,
            "calibrator": {
                "name": "calibrator1",
                "pointing": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "3Cabc",
                    "direction_type": "J2000"
                },
                "autoselect": false
            }
        },
        "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "calibrator observation"})).id,
        "specifications_template_id": (_.find(templates, {"name": "calibrator observation"})).id,
        "status": "scheduled",
        "status_value": "scheduled",
        "subtasks": [
            {
                "id": 2000037,
                "url": "http://localhost:3000/api/subtask/2000037",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "cluster": "http://localhost:3000/api/cluster/1",
                "cluster_id": 1,
                "cluster_name": "CEP4",
                "created_at": "2022-05-09T05:13:07.640763",
                "created_or_updated_by_user": null,
                "created_or_updated_by_user_id": null,
                "duration": 300,
                "error_reason": null,
                "global_identifier": "http://localhost:3000/api/sip_identifier/79",
                "global_identifier_id": 79,
                "global_parset_identifier": "http://localhost:3000/api/sip_identifier/80",
                "global_parset_identifier_id": 80,
                "inputs": [
                    "http://localhost:3000/api/subtask_input/49"
                ],
                "inputs_ids": [
                    49
                ],
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "outputs": [
                    "http://localhost:3000/api/subtask_output/33"
                ],
                "outputs_ids": [
                    33
                ],
                "primary": false,
                "process_start_time": "2022-05-09T06:23:49.932056",
                "process_stop_time": "2022-05-09T06:28:49.932056",
                "processing_feedback_doc": null,
                "processing_feedback_template": null,
                "processing_feedback_template_id": null,
                "scheduled_start_time": "2022-05-09T06:23:49.932056",
                "scheduled_stop_time": "2022-05-09T06:28:49.932056",
                "specifications_doc": {
                    "plots": {
                        "enabled": true,
                        "autocorrelation": true,
                        "crosscorrelation": true
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/QA%20plots/2/ref_resolved"
                },
                "specifications_template": "http://localhost:3000/api/subtask_template/3",
                "specifications_template_id": 3,
                "state": "http://localhost:3000/api/subtask_state/defined",
                "state_value": "defined",
                "subtask_type": "qa_plots",
                "tags": [],
                "task_blueprint": "http://localhost:3000/api/task_blueprint/22",
                "task_blueprint_id": 22,
                "total_nr_of_dataproducts": 0,
                "total_size_of_dataproducts": 0,
                "total_size_of_dataproducts_on_filesysem": 0,
                "unschedulable_reason": null,
                "updated_at": "2022-05-09T05:14:06.869293"
            },
            {
                "id": 2000035,
                "url": "http://localhost:3000/api/subtask/2000035",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "cluster": "http://localhost:3000/api/cluster/1",
                "cluster_id": 1,
                "cluster_name": "CEP4",
                "created_at": "2022-05-09T05:13:07.420264",
                "created_or_updated_by_user": null,
                "created_or_updated_by_user_id": null,
                "duration": 600,
                "error_reason": null,
                "global_identifier": "http://localhost:3000/api/sip_identifier/75",
                "global_identifier_id": 75,
                "global_parset_identifier": "http://localhost:3000/api/sip_identifier/76",
                "global_parset_identifier_id": 76,
                "inputs": [],
                "inputs_ids": [],
                "obsolete_since": null,
                "on_sky_duration": "00:10:00",
                "on_sky_start_time": "2022-05-09T06:08:49.932056",
                "on_sky_stop_time": "2022-05-09T06:18:49.932056",
                "outputs": [
                    "http://localhost:3000/api/subtask_output/31"
                ],
                "outputs_ids": [
                    31
                ],
                "primary": true,
                "process_start_time": "2022-05-09T06:08:49.932056",
                "process_stop_time": "2022-05-09T06:18:49.932056",
                "processing_feedback_doc": null,
                "processing_feedback_template": null,
                "processing_feedback_template_id": null,
                "scheduled_start_time": "2022-05-09T06:08:49.932056",
                "scheduled_stop_time": "2022-05-09T06:18:49.932056",
                "specifications_doc": {
                    "QA": {
                        "inspection_plots": "none"
                    },
                    "COBALT": {
                        "debug": {
                            "write_output": true,
                            "benchmark_file": "",
                            "annotate_broken_antennas": true
                        },
                        "version": 1,
                        "blocksize": 195584,
                        "resources": {
                            "writer_nodes": [
                                "cpu01.cep4.infiniband.lofar",
                                "cpu02.cep4.infiniband.lofar",
                                "cpu03.cep4.infiniband.lofar",
                                "cpu04.cep4.infiniband.lofar",
                                "cpu05.cep4.infiniband.lofar"
                            ],
                            "processing_nodes": [
                                "cbt201_0",
                                "cbt201_1",
                                "cbt202_0",
                                "cbt202_1",
                                "cbt203_0",
                                "cbt203_1",
                                "cbt204_0",
                                "cbt204_1",
                                "cbt205_0",
                                "cbt205_1",
                                "cbt207_0",
                                "cbt207_1",
                                "cbt208_0",
                                "cbt208_1",
                                "cbt209_0",
                                "cbt209_1",
                                "cbt210_0",
                                "cbt210_1",
                                "cbt211_0",
                                "cbt211_1",
                                "cbt212_0",
                                "cbt212_1"
                            ]
                        },
                        "beamformer": {
                            "input_ppf": false,
                            "tab_pipelines": [],
                            "flyseye_pipelines": []
                        },
                        "correlator": {
                            "enabled": true,
                            "doppler_correction": false,
                            "channels_per_subband": 64,
                            "blocks_per_integration": 1,
                            "integrations_per_block": 1
                        },
                        "delay_compensation": true,
                        "bandpass_correction": true
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/observation%20control/2/ref_resolved",
                    "stations": {
                        "filter": "HBA_110_190",
                        "antenna_set": "HBA_DUAL_INNER",
                        "station_list": [
                            "CS001",
                            "CS002",
                            "CS003",
                            "CS004",
                            "CS005",
                            "CS006",
                            "CS007",
                            "CS011",
                            "CS013",
                            "CS017",
                            "CS021",
                            "CS024",
                            "CS026",
                            "CS028",
                            "CS030",
                            "CS031",
                            "CS032",
                            "CS101",
                            "CS103",
                            "CS201",
                            "CS301",
                            "CS302",
                            "CS401",
                            "CS501",
                            "DE601",
                            "DE602",
                            "DE603",
                            "DE604",
                            "DE605",
                            "DE609",
                            "FR606",
                            "IE613",
                            "LV614",
                            "PL610",
                            "PL611",
                            "PL612",
                            "RS106",
                            "RS205",
                            "RS208",
                            "RS210",
                            "RS305",
                            "RS306",
                            "RS307",
                            "RS310",
                            "RS406",
                            "RS407",
                            "RS409",
                            "RS503",
                            "RS508",
                            "RS509",
                            "SE607",
                            "UK608"
                        ],
                        "analog_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "3Cabc",
                            "direction_type": "J2000"
                        },
                        "digital_pointings": [
                            {
                                "name": "calibrator1",
                                "pointing": {
                                    "angle1": 0.6624317181687094,
                                    "angle2": 1.5579526427549426,
                                    "target": "3Cabc",
                                    "direction_type": "J2000"
                                },
                                "subbands": [
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    182,
                                    183,
                                    184,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    212,
                                    213,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    242,
                                    243,
                                    244,
                                    245,
                                    246,
                                    247,
                                    248,
                                    249,
                                    250,
                                    251,
                                    252,
                                    253,
                                    254,
                                    255,
                                    257,
                                    258,
                                    259,
                                    260,
                                    261,
                                    262,
                                    263,
                                    264,
                                    265,
                                    266,
                                    267,
                                    268,
                                    269,
                                    270,
                                    271,
                                    272,
                                    273,
                                    275,
                                    276,
                                    277,
                                    278,
                                    279,
                                    280,
                                    281,
                                    282,
                                    283,
                                    284,
                                    285,
                                    286,
                                    287,
                                    288,
                                    289,
                                    290,
                                    291,
                                    292,
                                    293,
                                    294,
                                    295,
                                    296,
                                    297,
                                    298,
                                    299,
                                    300,
                                    302,
                                    303,
                                    304,
                                    305,
                                    306,
                                    307,
                                    308,
                                    309,
                                    310,
                                    311,
                                    312,
                                    313,
                                    314,
                                    315,
                                    316,
                                    317,
                                    318,
                                    319,
                                    320,
                                    321,
                                    322,
                                    323,
                                    324,
                                    325,
                                    326,
                                    327,
                                    328,
                                    330,
                                    331,
                                    332,
                                    333,
                                    334,
                                    335,
                                    336,
                                    337,
                                    338,
                                    339,
                                    340,
                                    341,
                                    342,
                                    343,
                                    344,
                                    345,
                                    346,
                                    347,
                                    349,
                                    364,
                                    372,
                                    380,
                                    388,
                                    396,
                                    404,
                                    413,
                                    421,
                                    430,
                                    438,
                                    447
                                ]
                            }
                        ]
                    }
                },
                "specifications_template": "http://localhost:3000/api/subtask_template/4",
                "specifications_template_id": 4,
                "state": "http://localhost:3000/api/subtask_state/scheduled",
                "state_value": "scheduled",
                "subtask_type": "observation",
                "tags": [],
                "task_blueprint": "http://localhost:3000/api/task_blueprint/22",
                "task_blueprint_id": 22,
                "total_nr_of_dataproducts": 0,
                "total_size_of_dataproducts": 0,
                "total_size_of_dataproducts_on_filesysem": 0,
                "unschedulable_reason": null,
                "updated_at": "2022-05-09T05:14:06.781066"
            },
            {
                "id": 2000036,
                "url": "http://localhost:3000/api/subtask/2000036",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "cluster": "http://localhost:3000/api/cluster/1",
                "cluster_id": 1,
                "cluster_name": "CEP4",
                "created_at": "2022-05-09T05:13:07.543346",
                "created_or_updated_by_user": null,
                "created_or_updated_by_user_id": null,
                "duration": 300,
                "error_reason": null,
                "global_identifier": "http://localhost:3000/api/sip_identifier/77",
                "global_identifier_id": 77,
                "global_parset_identifier": "http://localhost:3000/api/sip_identifier/78",
                "global_parset_identifier_id": 78,
                "inputs": [
                    "http://localhost:3000/api/subtask_input/48"
                ],
                "inputs_ids": [
                    48
                ],
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "outputs": [
                    "http://localhost:3000/api/subtask_output/32"
                ],
                "outputs_ids": [
                    32
                ],
                "primary": false,
                "process_start_time": "2022-05-09T06:18:49.932056",
                "process_stop_time": "2022-05-09T06:23:49.932056",
                "processing_feedback_doc": null,
                "processing_feedback_template": null,
                "processing_feedback_template_id": null,
                "scheduled_start_time": "2022-05-09T06:18:49.932056",
                "scheduled_stop_time": "2022-05-09T06:23:49.932056",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/QA%20file%20conversion/2/ref_resolved",
                    "file_conversion": {
                        "enabled": true,
                        "nr_of_subbands": -1,
                        "nr_of_timestamps": 256
                    }
                },
                "specifications_template": "http://localhost:3000/api/subtask_template/5",
                "specifications_template_id": 5,
                "state": "http://localhost:3000/api/subtask_state/defined",
                "state_value": "defined",
                "subtask_type": "qa_files",
                "tags": [],
                "task_blueprint": "http://localhost:3000/api/task_blueprint/22",
                "task_blueprint_id": 22,
                "total_nr_of_dataproducts": 0,
                "total_size_of_dataproducts": 0,
                "total_size_of_dataproducts_on_filesysem": 0,
                "unschedulable_reason": null,
                "updated_at": "2022-05-09T05:14:06.853241"
            }
        ],
        "subtasks_ids": [
            2000037,
            2000035,
            2000036
        ],
        "tags": [],
        "task_type": "observation",
        "unschedulable_reason": "",
        "updated_at": "2022-05-09T05:13:05.319244",
        "predecessors": [],
        "successors": [
            {
              "id": 23,
              "url": "http://localhost:3000/api/task_blueprint/23",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/38"
              ],
              "consumed_by_ids": [
                38
              ],
              "created_at": "2022-05-09T05:13:05.378424",
              "description": "Preprocessing Pipeline for Calibrator Observation 1",
              "draft": "http://localhost:3000/api/task_draft/23",
              "draft_id": 23,
              "duration": 600,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Calibrator Pipeline 1",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-05-09T06:18:49.932056",
              "process_stop_time": "2022-05-09T06:28:49.932056",
              "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/34"
              ],
              "produced_by_ids": [
                34
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-05-09T06:18:49.932056",
              "scheduled_stop_time": "2022-05-09T06:28:49.932056",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
              "scheduling_unit_blueprint_id": 4,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Cal1 3Cabc/PP",
              "specifications_doc": {
                "flag": {
                  "rfi_strategy": "HBAdefault",
                  "outerchannels": true,
                  "autocorrelations": true
                },
                "demix": {
                  "sources": [],
                  "time_steps": 10,
                  "ignore_target": false,
                  "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                  "time_steps": 1,
                  "frequency_steps": 4
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                  "where": {
                    "cluster": "CEP4",
                    "partition": "cpu"
                  },
                  "cores_per_task": 2,
                  "parallel_tasks": 122
                }
              },
              "specifications_template": "http://localhost:3000/api/task_template/10",
              "specifications_template_id": 10,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                {
                  "id": 2000038,
                  "url": "http://localhost:3000/api/subtask/2000038",
                  "actual_on_sky_start_time": null,
                  "actual_on_sky_stop_time": null,
                  "actual_process_start_time": null,
                  "actual_process_stop_time": null,
                  "cluster": "http://localhost:3000/api/cluster/1",
                  "cluster_id": 1,
                  "cluster_name": "CEP4",
                  "created_at": "2022-05-09T05:13:07.708976",
                  "created_or_updated_by_user": null,
                  "created_or_updated_by_user_id": null,
                  "duration": 600,
                  "error_reason": null,
                  "global_identifier": "http://localhost:3000/api/sip_identifier/81",
                  "global_identifier_id": 81,
                  "global_parset_identifier": "http://localhost:3000/api/sip_identifier/82",
                  "global_parset_identifier_id": 82,
                  "inputs": [
                    "http://localhost:3000/api/subtask_input/50"
                  ],
                  "inputs_ids": [
                    50
                  ],
                  "obsolete_since": null,
                  "on_sky_duration": null,
                  "on_sky_start_time": null,
                  "on_sky_stop_time": null,
                  "outputs": [
                    "http://localhost:3000/api/subtask_output/34"
                  ],
                  "outputs_ids": [
                    34
                  ],
                  "primary": true,
                  "process_start_time": "2022-05-09T06:18:49.932056",
                  "process_stop_time": "2022-05-09T06:28:49.932056",
                  "processing_feedback_doc": null,
                  "processing_feedback_template": null,
                  "processing_feedback_template_id": null,
                  "scheduled_start_time": "2022-05-09T06:18:49.932056",
                  "scheduled_stop_time": "2022-05-09T06:28:49.932056",
                  "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "demixer": {
                      "enabled": true,
                      "baselines": "CS*,RS*&",
                      "time_steps": 1,
                      "demix_always": [],
                      "ignore_target": false,
                      "demix_if_needed": [],
                      "frequency_steps": 4,
                      "demix_time_steps": 10,
                      "demix_frequency_steps": 64
                    },
                    "aoflagger": {
                      "enabled": true,
                      "strategy": "HBAdefault"
                    },
                    "preflagger0": {
                      "enabled": true,
                      "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                    },
                    "preflagger1": {
                      "enabled": true,
                      "corrtype": "auto"
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                      "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                    }
                  },
                  "specifications_template": "http://localhost:3000/api/subtask_template/7",
                  "specifications_template_id": 7,
                  "state": "http://localhost:3000/api/subtask_state/defined",
                  "state_value": "defined",
                  "subtask_type": "pipeline",
                  "tags": [],
                  "task_blueprint": "http://localhost:3000/api/task_blueprint/23",
                  "task_blueprint_id": 23,
                  "total_nr_of_dataproducts": 0,
                  "total_size_of_dataproducts": 0,
                  "total_size_of_dataproducts_on_filesysem": 0,
                  "unschedulable_reason": null,
                  "updated_at": "2022-05-09T05:14:06.820657"
                }
              ],
              "subtasks_ids": [
                2000038
              ],
              "tags": [],
              "task_type": "pipeline",
              "unschedulable_reason": "",
              "updated_at": "2022-05-09T05:13:05.378438"
            }
        ]
    },
    {
        "id": 23,
        "url": "http://localhost:3000/api/task_blueprint/23",
        "actual_on_sky_start_time": null,
        "actual_on_sky_stop_time": null,
        "actual_process_start_time": null,
        "actual_process_stop_time": null,
        "consumed_by": [
            "http://localhost:3000/api/task_relation_blueprint/38"
        ],
        "consumed_by_ids": [
            38
        ],
        "created_at": "2022-05-09T05:13:05.378424",
        "description": "Preprocessing Pipeline for Calibrator Observation 1",
        "draft": "http://localhost:3000/api/task_draft/23",
        "draft_id": 23,
        "draftObject": {
            "id": 23,
            "url": "http://localhost:3000/api/task_draft/23",
            "connector_types": [
                {
                    "id": 13,
                    "url": "http://localhost:3000/api/task_connector_type/13",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                },
                {
                    "id": 12,
                    "url": "http://localhost:3000/api/task_connector_type/12",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/10",
                    "task_template_id": 10
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/38"
            ],
            "consumed_by_ids": [
                38
            ],
            "created_at": "2022-05-09T05:13:04.176326",
            "description": "Preprocessing Pipeline for Calibrator Observation 1",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Calibrator Pipeline 1",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/34"
            ],
            "produced_by_ids": [
                34
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/4",
            "scheduling_unit_draft_id": 4,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "Cal1 3Cabc/PP",
            "specifications_doc": {
                "flag": {
                    "rfi_strategy": "HBAdefault",
                    "outerchannels": true,
                    "autocorrelations": true
                },
                "demix": {
                    "sources": [],
                    "time_steps": 10,
                    "ignore_target": false,
                    "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                    "time_steps": 1,
                    "frequency_steps": 4
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/10",
            "specifications_template_id": 10,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/23"
            ],
            "task_blueprints_ids": [
                23
            ],
            "task_type": "pipeline",
            "updated_at": "2022-05-09T05:13:04.176350"
        },
        "duration": 600,
        "error_reason": "",
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Calibrator Pipeline 1",
        "obsolete_since": null,
        "on_sky_duration": null,
        "on_sky_start_time": null,
        "on_sky_stop_time": null,
        "output_pinned": false,
        "process_start_time": "2022-05-09T06:18:49.932056",
        "process_stop_time": "2022-05-09T06:28:49.932056",
        "produced_by": [
            "http://localhost:3000/api/task_relation_blueprint/34"
        ],
        "produced_by_ids": [
            34
        ],
        "relative_start_time": 0,
        "relative_stop_time": 0,
        "scheduled_start_time": "2022-05-09T06:18:49.932056",
        "scheduled_stop_time": "2022-05-09T06:28:49.932056",
        "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
        "scheduling_unit_blueprint_id": 4,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "Cal1 3Cabc/PP",
        "specifications_doc": {
            "flag": {
                "rfi_strategy": "HBAdefault",
                "outerchannels": true,
                "autocorrelations": true
            },
            "demix": {
                "sources": [],
                "time_steps": 10,
                "ignore_target": false,
                "frequency_steps": 64
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
            "average": {
                "time_steps": 1,
                "frequency_steps": 4
            },
            "storagemanager": "dysco",
            "cluster_resources": {
                "where": {
                    "cluster": "CEP4",
                    "partition": "cpu"
                },
                "cores_per_task": 2,
                "parallel_tasks": 122
            }
        },
        "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "preprocessing pipeline"})).id,
        "specifications_template_id": (_.find(templates, {"name": "preprocessing pipeline"})).id,
        "status": "schedulable",
        "status_value": "schedulable",
        "subtasks": [
            {
                "id": 2000038,
                "url": "http://localhost:3000/api/subtask/2000038",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "cluster": "http://localhost:3000/api/cluster/1",
                "cluster_id": 1,
                "cluster_name": "CEP4",
                "created_at": "2022-05-09T05:13:07.708976",
                "created_or_updated_by_user": null,
                "created_or_updated_by_user_id": null,
                "duration": 600,
                "error_reason": null,
                "global_identifier": "http://localhost:3000/api/sip_identifier/81",
                "global_identifier_id": 81,
                "global_parset_identifier": "http://localhost:3000/api/sip_identifier/82",
                "global_parset_identifier_id": 82,
                "inputs": [
                    "http://localhost:3000/api/subtask_input/50"
                ],
                "inputs_ids": [
                    50
                ],
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "outputs": [
                    "http://localhost:3000/api/subtask_output/34"
                ],
                "outputs_ids": [
                    34
                ],
                "primary": true,
                "process_start_time": "2022-05-09T06:18:49.932056",
                "process_stop_time": "2022-05-09T06:28:49.932056",
                "processing_feedback_doc": null,
                "processing_feedback_template": null,
                "processing_feedback_template_id": null,
                "scheduled_start_time": "2022-05-09T06:18:49.932056",
                "scheduled_stop_time": "2022-05-09T06:28:49.932056",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "demixer": {
                        "enabled": true,
                        "baselines": "CS*,RS*&",
                        "time_steps": 1,
                        "demix_always": [],
                        "ignore_target": false,
                        "demix_if_needed": [],
                        "frequency_steps": 4,
                        "demix_time_steps": 10,
                        "demix_frequency_steps": 64
                    },
                    "aoflagger": {
                        "enabled": true,
                        "strategy": "HBAdefault"
                    },
                    "preflagger0": {
                        "enabled": true,
                        "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                    },
                    "preflagger1": {
                        "enabled": true,
                        "corrtype": "auto"
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    }
                },
                "specifications_template": "http://localhost:3000/api/subtask_template/7",
                "specifications_template_id": 7,
                "state": "http://localhost:3000/api/subtask_state/defined",
                "state_value": "defined",
                "subtask_type": "pipeline",
                "tags": [],
                "task_blueprint": "http://localhost:3000/api/task_blueprint/23",
                "task_blueprint_id": 23,
                "total_nr_of_dataproducts": 0,
                "total_size_of_dataproducts": 0,
                "total_size_of_dataproducts_on_filesysem": 0,
                "unschedulable_reason": null,
                "updated_at": "2022-05-09T05:14:06.820657"
            }
        ],
        "subtasks_ids": [
            2000038
        ],
        "tags": [],
        "task_type": "pipeline",
        "unschedulable_reason": "",
        "updated_at": "2022-05-09T05:13:05.378438",
        "predecessors": [
            {
              "id": 22,
              "url": "http://localhost:3000/api/task_blueprint/22",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/34"
              ],
              "consumed_by_ids": [
                34
              ],
              "created_at": "2022-05-09T05:13:05.319201",
              "description": "Calibrator Observation 1. Before the Target Observation",
              "draft": "http://localhost:3000/api/task_draft/22",
              "draft_id": 22,
              "duration": 1200,
              "error_reason": "",
              "first_scheduling_relation": [
                "http://localhost:3000/api/task_scheduling_relation_blueprint/1"
              ],
              "first_scheduling_relation_ids": [
                1
              ],
              "name": "Calibrator Observation 1",
              "obsolete_since": null,
              "on_sky_duration": 600,
              "on_sky_start_time": "2022-05-09T06:08:49.932056",
              "on_sky_stop_time": "2022-05-09T06:18:49.932056",
              "output_pinned": false,
              "process_start_time": "2022-05-09T06:08:49.932056",
              "process_stop_time": "2022-05-09T06:28:49.932056",
              "produced_by": [],
              "produced_by_ids": [],
              "relative_start_time": 0,
              "relative_stop_time": 600,
              "scheduled_start_time": "2022-05-09T06:08:49.932056",
              "scheduled_stop_time": "2022-05-09T06:28:49.932056",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
              "scheduling_unit_blueprint_id": 4,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Cal1 3Cabc",
              "specifications_doc": {
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2/ref_resolved",
                "duration": 600,
                "calibrator": {
                  "name": "calibrator1",
                  "pointing": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "3Cabc",
                    "direction_type": "J2000"
                  },
                  "autoselect": false
                }
              },
              "specifications_template": {
                "id": 7,
                "url": "http://localhost:3000/api/task_template/7",
                "created_at": "2022-05-09T05:12:33.393061",
                "description": "This schema defines the (extra) parameters to setup a calibrator observation task, which uses all parameters from the target observation task which it is linked to, plus these calibrator overrides.",
                "name": "calibrator observation",
                "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                "purpose_value": "technical_commissioning",
                "ref_resolved_schema": {
                  "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2/ref_resolved",
                  "title": "calibrator observation",
                  "$schema": "http://json-schema.org/draft-06/schema#",
                  "default": {},
                  "version": 2,
                  "required": [
                    "calibrator",
                    "duration"
                  ],
                  "properties": {
                    "duration": {
                      "$id": "#duration",
                      "$ref": "#/definitions/timedelta",
                      "title": "Duration",
                      "default": 600,
                      "minimum": 1,
                      "description": "Duration of this observation (seconds)"
                    },
                    "calibrator": {
                      "$ref": "#/definitions/calibrator",
                      "default": {}
                    }
                  },
                  "description": "This schema defines the (extra) parameters to setup a calibrator observation task, which uses all parameters from the target observation task which it is linked to, plus these calibrator overrides.",
                  "patternProperties": {
                    "^[$]schema$": {}
                  },
                  "additionalProperties": false,
                  "definitions": {
                    "timedelta": {
                      "default": 0,
                      "description": "A time duration or delta expressed in seconds",
                      "type": "number"
                    },
                    "timestamp": {
                      "default": "1970-01-01T00:00:00Z",
                      "description": "A timestamp defined in UTC",
                      "format": "date-time",
                      "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                      "type": "string"
                    },
                    "calibrator": {
                      "additionalProperties": false,
                      "default": {},
                      "properties": {
                        "autoselect": {
                          "default": true,
                          "description": "Auto-select calibrator based on elevation",
                          "title": "Auto-select",
                          "type": "boolean"
                        },
                        "name": {
                          "default": "calibrator",
                          "description": "Name of the calibrator SAP",
                          "title": "Name",
                          "type": "string"
                        },
                        "pointing": {
                          "$ref": "#/definitions/pointing",
                          "default": {},
                          "description": "Manually selected calibrator",
                          "title": "Digital pointing"
                        }
                      },
                      "required": [
                        "autoselect",
                        "pointing",
                        "name"
                      ],
                      "type": "object"
                    },
                    "pointing": {
                      "additionalProperties": false,
                      "properties": {
                        "angle1": {
                          "default": 0.6624317181687094,
                          "description": "First angle (e.g. RA)",
                          "title": "Angle 1",
                          "type": "number"
                        },
                        "angle2": {
                          "default": 1.5579526427549426,
                          "description": "Second angle (e.g. DEC)",
                          "title": "Angle 2",
                          "type": "number"
                        },
                        "direction_type": {
                          "default": "J2000",
                          "description": "",
                          "enum": [
                            "J2000",
                            "AZELGEO",
                            "LMN",
                            "SUN",
                            "MOON",
                            "MERCURY",
                            "VENUS",
                            "MARS",
                            "JUPITER",
                            "SATURN",
                            "URANUS",
                            "NEPTUNE",
                            "PLUTO"
                          ],
                          "title": "Reference frame",
                          "type": "string"
                        },
                        "target": {
                          "default": "_target_name_",
                          "description": "Description of where this beam points at",
                          "minLength": 1,
                          "title": "Target",
                          "type": "string"
                        }
                      },
                      "required": [
                        "angle1",
                        "angle2",
                        "target"
                      ],
                      "type": "object"
                    }
                  }
                },
                "schema": {
                  "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2#",
                  "title": "calibrator observation",
                  "$schema": "http://json-schema.org/draft-06/schema#",
                  "default": {},
                  "version": 2,
                  "required": [
                    "calibrator",
                    "duration"
                  ],
                  "properties": {
                    "duration": {
                      "$id": "#duration",
                      "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                      "title": "Duration",
                      "default": 600,
                      "minimum": 1,
                      "description": "Duration of this observation (seconds)"
                    },
                    "calibrator": {
                      "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/calibrator/1#/definitions/calibrator",
                      "default": {}
                    }
                  },
                  "description": "This schema defines the (extra) parameters to setup a calibrator observation task, which uses all parameters from the target observation task which it is linked to, plus these calibrator overrides.",
                  "patternProperties": {
                    "^[$]schema$": {}
                  },
                  "additionalProperties": false
                },
                "state": "http://localhost:3000/api/template_state/active",
                "state_value": "active",
                "tags": [],
                "type": "http://localhost:3000/api/task_type/observation",
                "type_value": "observation",
                "updated_at": "2022-05-09T05:12:33.393095",
                "version": 2
              },
              "specifications_template_id": 7,
              "status": "scheduled",
              "status_value": "scheduled",
              "subtasks": [
                "http://localhost:3000/api/subtask/2000037",
                "http://localhost:3000/api/subtask/2000035",
                "http://localhost:3000/api/subtask/2000036"
              ],
              "subtasks_ids": [
                2000037,
                2000035,
                2000036
              ],
              "tags": [],
              "task_type": "observation",
              "unschedulable_reason": "",
              "updated_at": "2022-05-09T05:13:05.319244"
            }
        ],
        "successors": [
            {
              "id": 29,
              "url": "http://localhost:3000/api/task_blueprint/29",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [],
              "consumed_by_ids": [],
              "created_at": "2022-05-09T05:13:05.717903",
              "description": "Ingest all preprocessed dataproducts",
              "draft": "http://localhost:3000/api/task_draft/29",
              "draft_id": 29,
              "duration": 300,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Ingest",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-05-09T22:19:49.932056",
              "process_stop_time": "2022-05-09T22:24:49.932056",
              "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/38",
                "http://localhost:3000/api/task_relation_blueprint/39",
                "http://localhost:3000/api/task_relation_blueprint/40",
                "http://localhost:3000/api/task_relation_blueprint/41"
              ],
              "produced_by_ids": [
                38,
                39,
                40,
                41
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-05-09T22:19:49.932056",
              "scheduled_stop_time": "2022-05-09T22:24:49.932056",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
              "scheduling_unit_blueprint_id": 4,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "",
              "specifications_doc": {
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
              },
              "specifications_template": "http://localhost:3000/api/task_template/2",
              "specifications_template_id": 2,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                {
                  "id": 2000040,
                  "url": "http://localhost:3000/api/subtask/2000040",
                  "actual_on_sky_start_time": null,
                  "actual_on_sky_stop_time": null,
                  "actual_process_start_time": null,
                  "actual_process_stop_time": null,
                  "cluster": "http://localhost:3000/api/cluster/1",
                  "cluster_id": 1,
                  "cluster_name": "CEP4",
                  "created_at": "2022-05-09T05:13:07.901188",
                  "created_or_updated_by_user": null,
                  "created_or_updated_by_user_id": null,
                  "duration": 300,
                  "error_reason": null,
                  "global_identifier": "http://localhost:3000/api/sip_identifier/85",
                  "global_identifier_id": 85,
                  "global_parset_identifier": "http://localhost:3000/api/sip_identifier/86",
                  "global_parset_identifier_id": 86,
                  "inputs": [
                    "http://localhost:3000/api/subtask_input/52",
                    "http://localhost:3000/api/subtask_input/53",
                    "http://localhost:3000/api/subtask_input/54",
                    "http://localhost:3000/api/subtask_input/55"
                  ],
                  "inputs_ids": [
                    52,
                    53,
                    54,
                    55
                  ],
                  "obsolete_since": null,
                  "on_sky_duration": null,
                  "on_sky_start_time": null,
                  "on_sky_stop_time": null,
                  "outputs": [],
                  "outputs_ids": [],
                  "primary": true,
                  "process_start_time": "2022-05-09T22:19:49.932056",
                  "process_stop_time": "2022-05-09T22:24:49.932056",
                  "processing_feedback_doc": null,
                  "processing_feedback_template": null,
                  "processing_feedback_template_id": null,
                  "scheduled_start_time": "2022-05-09T22:19:49.932056",
                  "scheduled_stop_time": "2022-05-09T22:24:49.932056",
                  "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/ingest%20control/2/ref_resolved"
                  },
                  "specifications_template": "http://localhost:3000/api/subtask_template/2",
                  "specifications_template_id": 2,
                  "state": "http://localhost:3000/api/subtask_state/defined",
                  "state_value": "defined",
                  "subtask_type": "ingest",
                  "tags": [],
                  "task_blueprint": "http://localhost:3000/api/task_blueprint/29",
                  "task_blueprint_id": 29,
                  "total_nr_of_dataproducts": 0,
                  "total_size_of_dataproducts": 0,
                  "total_size_of_dataproducts_on_filesysem": 0,
                  "unschedulable_reason": null,
                  "updated_at": "2022-05-09T05:14:08.429386"
                }
              ],
              "subtasks_ids": [
                2000040
              ],
              "tags": [],
              "task_type": "ingest",
              "unschedulable_reason": "",
              "updated_at": "2022-05-09T05:13:05.717918"
            }
        ]
    },
    {
        "id": 24,
        "url": "http://localhost:3000/api/task_blueprint/24",
        "actual_on_sky_start_time": null,
        "actual_on_sky_stop_time": null,
        "actual_process_start_time": null,
        "actual_process_stop_time": null,
        "consumed_by": [
            "http://localhost:3000/api/task_relation_blueprint/36",
            "http://localhost:3000/api/task_relation_blueprint/37"
        ],
        "consumed_by_ids": [
            36,
            37
        ],
        "created_at": "2022-05-09T05:13:05.477848",
        "description": "Target Observation for LoTSS scheduling unit. HBA_DUAL_INNER, 120-168 MHz, 1s, 64ch/sb ",
        "draft": "http://localhost:3000/api/task_draft/23",
        "draft_id": 23,
        "draftObject": {
            "id": 23,
            "url": "http://localhost:3000/api/task_draft/23",
            "connector_types": [
                {
                    "id": 2,
                    "url": "http://localhost:3000/api/task_connector_type/2",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/correlator",
                    "role_value": "correlator",
                    "task_template": "http://localhost:3000/api/task_template/4",
                    "task_template_id": 4
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/36",
                "http://localhost:3000/api/task_relation_draft/37"
            ],
            "consumed_by_ids": [
                36,
                37
            ],
            "created_at": "2022-05-09T05:13:04.262572",
            "description": "Target Observation for LoTSS scheduling unit. HBA_DUAL_INNER, 120-168 MHz, 1s, 64ch/sb ",
            "duration": 28800,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Target Observation",
            "output_pinned": false,
            "produced_by": [],
            "produced_by_ids": [],
            "relative_start_time": 660,
            "relative_stop_time": 29460,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/4",
            "scheduling_unit_draft_id": 4,
            "second_scheduling_relation": [
                "http://localhost:3000/api/task_scheduling_relation_draft/1",
                "http://localhost:3000/api/task_scheduling_relation_draft/2"
            ],
            "second_scheduling_relation_ids": [
                1,
                2
            ],
            "short_description": "Paaa+01 & Paaa+02",
            "specifications_doc": {
                "QA": {
                    "plots": {
                        "enabled": true,
                        "autocorrelation": true,
                        "crosscorrelation": true
                    },
                    "file_conversion": {
                        "enabled": true,
                        "nr_of_subbands": -1,
                        "nr_of_timestamps": 256
                    },
                    "inspection_plots": "msplots"
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/2/ref_resolved",
                "duration": 28800,
                "correlator": {
                    "storage_cluster": "CEP4",
                    "integration_time": 1,
                    "channels_per_subband": 64,
                    "topocentric_frequency_correction": false
                },
                "station_configuration": {
                    "SAPs": [
                        {
                            "name": "sap1",
                            "subbands": [
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154,
                                155,
                                156,
                                157,
                                158,
                                159,
                                160,
                                161,
                                162,
                                163,
                                165,
                                166,
                                167,
                                168,
                                169,
                                170,
                                171,
                                172,
                                173,
                                174,
                                175,
                                176,
                                177,
                                178,
                                179,
                                180,
                                182,
                                183,
                                184,
                                187,
                                188,
                                189,
                                190,
                                191,
                                192,
                                193,
                                194,
                                195,
                                196,
                                197,
                                198,
                                199,
                                200,
                                201,
                                202,
                                203,
                                204,
                                205,
                                206,
                                207,
                                208,
                                209,
                                212,
                                213,
                                215,
                                216,
                                217,
                                218,
                                219,
                                220,
                                221,
                                222,
                                223,
                                224,
                                225,
                                226,
                                227,
                                228,
                                229,
                                230,
                                231,
                                232,
                                233,
                                234,
                                235,
                                236,
                                237,
                                238,
                                239,
                                240,
                                242,
                                243,
                                244,
                                245,
                                246,
                                247,
                                248,
                                249,
                                250,
                                251,
                                252,
                                253,
                                254,
                                255,
                                257,
                                258,
                                259,
                                260,
                                261,
                                262,
                                263,
                                264,
                                265,
                                266,
                                267,
                                268,
                                269,
                                270,
                                271,
                                272,
                                273,
                                275,
                                276,
                                277,
                                278,
                                279,
                                280,
                                281,
                                282,
                                283,
                                284,
                                285,
                                286,
                                287,
                                288,
                                289,
                                290,
                                291,
                                292,
                                293,
                                294,
                                295,
                                296,
                                297,
                                298,
                                299,
                                300,
                                302,
                                303,
                                304,
                                305,
                                306,
                                307,
                                308,
                                309,
                                310,
                                311,
                                312,
                                313,
                                314,
                                315,
                                316,
                                317,
                                318,
                                319,
                                320,
                                321,
                                322,
                                323,
                                324,
                                325,
                                326,
                                327,
                                328,
                                330,
                                331,
                                332,
                                333,
                                334,
                                335,
                                336,
                                337,
                                338,
                                339,
                                340,
                                341,
                                342,
                                343,
                                344,
                                345,
                                346,
                                347,
                                349,
                                364,
                                372,
                                380,
                                388,
                                396,
                                404,
                                413,
                                421,
                                430,
                                438,
                                447
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+01",
                                "direction_type": "J2000"
                            }
                        },
                        {
                            "name": "sap2",
                            "subbands": [
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154,
                                155,
                                156,
                                157,
                                158,
                                159,
                                160,
                                161,
                                162,
                                163,
                                165,
                                166,
                                167,
                                168,
                                169,
                                170,
                                171,
                                172,
                                173,
                                174,
                                175,
                                176,
                                177,
                                178,
                                179,
                                180,
                                182,
                                183,
                                184,
                                187,
                                188,
                                189,
                                190,
                                191,
                                192,
                                193,
                                194,
                                195,
                                196,
                                197,
                                198,
                                199,
                                200,
                                201,
                                202,
                                203,
                                204,
                                205,
                                206,
                                207,
                                208,
                                209,
                                212,
                                213,
                                215,
                                216,
                                217,
                                218,
                                219,
                                220,
                                221,
                                222,
                                223,
                                224,
                                225,
                                226,
                                227,
                                228,
                                229,
                                230,
                                231,
                                232,
                                233,
                                234,
                                235,
                                236,
                                237,
                                238,
                                239,
                                240,
                                242,
                                243,
                                244,
                                245,
                                246,
                                247,
                                248,
                                249,
                                250,
                                251,
                                252,
                                253,
                                254,
                                255,
                                257,
                                258,
                                259,
                                260,
                                261,
                                262,
                                263,
                                264,
                                265,
                                266,
                                267,
                                268,
                                269,
                                270,
                                271,
                                272,
                                273,
                                275,
                                276,
                                277,
                                278,
                                279,
                                280,
                                281,
                                282,
                                283,
                                284,
                                285,
                                286,
                                287,
                                288,
                                289,
                                290,
                                291,
                                292,
                                293,
                                294,
                                295,
                                296,
                                297,
                                298,
                                299,
                                300,
                                302,
                                303,
                                304,
                                305,
                                306,
                                307,
                                308,
                                309,
                                310,
                                311,
                                312,
                                313,
                                314,
                                315,
                                316,
                                317,
                                318,
                                319,
                                320,
                                321,
                                322,
                                323,
                                324,
                                325,
                                326,
                                327,
                                328,
                                330,
                                331,
                                332,
                                333,
                                334,
                                335,
                                336,
                                337,
                                338,
                                339,
                                340,
                                341,
                                342,
                                343,
                                344,
                                345,
                                346,
                                347,
                                349,
                                364,
                                372,
                                380,
                                388,
                                396,
                                404,
                                413,
                                421,
                                430,
                                438,
                                447
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "Paaa+02",
                                "direction_type": "J2000"
                            }
                        }
                    ],
                    "filter": "HBA_110_190",
                    "tile_beam": {
                        "angle1": 0.6624317181687094,
                        "angle2": 1.5579526427549426,
                        "target": "Paaa+01Paaa+02REF",
                        "direction_type": "J2000"
                    },
                    "antenna_set": "HBA_DUAL_INNER",
                    "station_groups": [
                        {
                            "stations": [
                                "CS001",
                                "CS002",
                                "CS003",
                                "CS004",
                                "CS005",
                                "CS006",
                                "CS007",
                                "CS011",
                                "CS013",
                                "CS017",
                                "CS021",
                                "CS024",
                                "CS026",
                                "CS028",
                                "CS030",
                                "CS031",
                                "CS032",
                                "CS101",
                                "CS103",
                                "CS201",
                                "CS301",
                                "CS302",
                                "CS401",
                                "CS501",
                                "RS106",
                                "RS205",
                                "RS208",
                                "RS210",
                                "RS305",
                                "RS306",
                                "RS307",
                                "RS310",
                                "RS406",
                                "RS407",
                                "RS409",
                                "RS503",
                                "RS508",
                                "RS509"
                            ],
                            "max_nr_missing": 4
                        },
                        {
                            "stations": [
                                "DE601",
                                "DE602",
                                "DE603",
                                "DE604",
                                "DE605",
                                "DE609",
                                "FR606",
                                "SE607",
                                "UK608",
                                "PL610",
                                "PL611",
                                "PL612",
                                "IE613",
                                "LV614"
                            ],
                            "max_nr_missing": 2
                        },
                        {
                            "stations": [
                                "DE601",
                                "DE605"
                            ],
                            "max_nr_missing": 1
                        }
                    ]
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/4",
            "specifications_template_id": 4,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/24"
            ],
            "task_blueprints_ids": [
                24
            ],
            "task_type": "observation",
            "updated_at": "2022-05-09T05:13:04.262586"
        },
        "duration": 29400,
        "error_reason": "",
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Target Observation",
        "obsolete_since": null,
        "on_sky_duration": 28800,
        "on_sky_start_time": "2022-05-09T06:19:49.932056",
        "on_sky_stop_time": "2022-05-09T14:19:49.932056",
        "output_pinned": false,
        "process_start_time": "2022-05-09T06:19:49.932056",
        "process_stop_time": "2022-05-09T14:29:49.932056",
        "produced_by": [],
        "produced_by_ids": [],
        "relative_start_time": 660,
        "relative_stop_time": 29460,
        "scheduled_start_time": "2022-05-09T06:19:49.932056",
        "scheduled_stop_time": "2022-05-09T14:29:49.932056",
        "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
        "scheduling_unit_blueprint_id": 4,
        "second_scheduling_relation": [
            "http://localhost:3000/api/task_scheduling_relation_blueprint/1",
            "http://localhost:3000/api/task_scheduling_relation_blueprint/2"
        ],
        "second_scheduling_relation_ids": [
            1,
            2
        ],
        "short_description": "Paaa+01 & Paaa+02",
        "specifications_doc": {
            "QA": {
                "plots": {
                    "enabled": true,
                    "autocorrelation": true,
                    "crosscorrelation": true
                },
                "file_conversion": {
                    "enabled": true,
                    "nr_of_subbands": -1,
                    "nr_of_timestamps": 256
                },
                "inspection_plots": "msplots"
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/2/ref_resolved",
            "duration": 28800,
            "correlator": {
                "storage_cluster": "CEP4",
                "integration_time": 1,
                "channels_per_subband": 64,
                "topocentric_frequency_correction": false
            },
            "station_configuration": {
                "SAPs": [
                    {
                        "name": "sap1",
                        "subbands": [
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154,
                            155,
                            156,
                            157,
                            158,
                            159,
                            160,
                            161,
                            162,
                            163,
                            165,
                            166,
                            167,
                            168,
                            169,
                            170,
                            171,
                            172,
                            173,
                            174,
                            175,
                            176,
                            177,
                            178,
                            179,
                            180,
                            182,
                            183,
                            184,
                            187,
                            188,
                            189,
                            190,
                            191,
                            192,
                            193,
                            194,
                            195,
                            196,
                            197,
                            198,
                            199,
                            200,
                            201,
                            202,
                            203,
                            204,
                            205,
                            206,
                            207,
                            208,
                            209,
                            212,
                            213,
                            215,
                            216,
                            217,
                            218,
                            219,
                            220,
                            221,
                            222,
                            223,
                            224,
                            225,
                            226,
                            227,
                            228,
                            229,
                            230,
                            231,
                            232,
                            233,
                            234,
                            235,
                            236,
                            237,
                            238,
                            239,
                            240,
                            242,
                            243,
                            244,
                            245,
                            246,
                            247,
                            248,
                            249,
                            250,
                            251,
                            252,
                            253,
                            254,
                            255,
                            257,
                            258,
                            259,
                            260,
                            261,
                            262,
                            263,
                            264,
                            265,
                            266,
                            267,
                            268,
                            269,
                            270,
                            271,
                            272,
                            273,
                            275,
                            276,
                            277,
                            278,
                            279,
                            280,
                            281,
                            282,
                            283,
                            284,
                            285,
                            286,
                            287,
                            288,
                            289,
                            290,
                            291,
                            292,
                            293,
                            294,
                            295,
                            296,
                            297,
                            298,
                            299,
                            300,
                            302,
                            303,
                            304,
                            305,
                            306,
                            307,
                            308,
                            309,
                            310,
                            311,
                            312,
                            313,
                            314,
                            315,
                            316,
                            317,
                            318,
                            319,
                            320,
                            321,
                            322,
                            323,
                            324,
                            325,
                            326,
                            327,
                            328,
                            330,
                            331,
                            332,
                            333,
                            334,
                            335,
                            336,
                            337,
                            338,
                            339,
                            340,
                            341,
                            342,
                            343,
                            344,
                            345,
                            346,
                            347,
                            349,
                            364,
                            372,
                            380,
                            388,
                            396,
                            404,
                            413,
                            421,
                            430,
                            438,
                            447
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+01",
                            "direction_type": "J2000"
                        }
                    },
                    {
                        "name": "sap2",
                        "subbands": [
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154,
                            155,
                            156,
                            157,
                            158,
                            159,
                            160,
                            161,
                            162,
                            163,
                            165,
                            166,
                            167,
                            168,
                            169,
                            170,
                            171,
                            172,
                            173,
                            174,
                            175,
                            176,
                            177,
                            178,
                            179,
                            180,
                            182,
                            183,
                            184,
                            187,
                            188,
                            189,
                            190,
                            191,
                            192,
                            193,
                            194,
                            195,
                            196,
                            197,
                            198,
                            199,
                            200,
                            201,
                            202,
                            203,
                            204,
                            205,
                            206,
                            207,
                            208,
                            209,
                            212,
                            213,
                            215,
                            216,
                            217,
                            218,
                            219,
                            220,
                            221,
                            222,
                            223,
                            224,
                            225,
                            226,
                            227,
                            228,
                            229,
                            230,
                            231,
                            232,
                            233,
                            234,
                            235,
                            236,
                            237,
                            238,
                            239,
                            240,
                            242,
                            243,
                            244,
                            245,
                            246,
                            247,
                            248,
                            249,
                            250,
                            251,
                            252,
                            253,
                            254,
                            255,
                            257,
                            258,
                            259,
                            260,
                            261,
                            262,
                            263,
                            264,
                            265,
                            266,
                            267,
                            268,
                            269,
                            270,
                            271,
                            272,
                            273,
                            275,
                            276,
                            277,
                            278,
                            279,
                            280,
                            281,
                            282,
                            283,
                            284,
                            285,
                            286,
                            287,
                            288,
                            289,
                            290,
                            291,
                            292,
                            293,
                            294,
                            295,
                            296,
                            297,
                            298,
                            299,
                            300,
                            302,
                            303,
                            304,
                            305,
                            306,
                            307,
                            308,
                            309,
                            310,
                            311,
                            312,
                            313,
                            314,
                            315,
                            316,
                            317,
                            318,
                            319,
                            320,
                            321,
                            322,
                            323,
                            324,
                            325,
                            326,
                            327,
                            328,
                            330,
                            331,
                            332,
                            333,
                            334,
                            335,
                            336,
                            337,
                            338,
                            339,
                            340,
                            341,
                            342,
                            343,
                            344,
                            345,
                            346,
                            347,
                            349,
                            364,
                            372,
                            380,
                            388,
                            396,
                            404,
                            413,
                            421,
                            430,
                            438,
                            447
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+02",
                            "direction_type": "J2000"
                        }
                    }
                ],
                "filter": "HBA_110_190",
                "tile_beam": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "Paaa+01Paaa+02REF",
                    "direction_type": "J2000"
                },
                "antenna_set": "HBA_DUAL_INNER",
                "station_groups": [
                    {
                        "stations": [
                            "CS001",
                            "CS002",
                            "CS003",
                            "CS004",
                            "CS005",
                            "CS006",
                            "CS007",
                            "CS011",
                            "CS013",
                            "CS017",
                            "CS021",
                            "CS024",
                            "CS026",
                            "CS028",
                            "CS030",
                            "CS031",
                            "CS032",
                            "CS101",
                            "CS103",
                            "CS201",
                            "CS301",
                            "CS302",
                            "CS401",
                            "CS501",
                            "RS106",
                            "RS205",
                            "RS208",
                            "RS210",
                            "RS305",
                            "RS306",
                            "RS307",
                            "RS310",
                            "RS406",
                            "RS407",
                            "RS409",
                            "RS503",
                            "RS508",
                            "RS509"
                        ],
                        "max_nr_missing": 4
                    },
                    {
                        "stations": [
                            "DE601",
                            "DE602",
                            "DE603",
                            "DE604",
                            "DE605",
                            "DE609",
                            "FR606",
                            "SE607",
                            "UK608",
                            "PL610",
                            "PL611",
                            "PL612",
                            "IE613",
                            "LV614"
                        ],
                        "max_nr_missing": 2
                    },
                    {
                        "stations": [
                            "DE601",
                            "DE605"
                        ],
                        "max_nr_missing": 1
                    }
                ]
            }
        },
        "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "target observation"})).id,
        "specifications_template_id": (_.find(templates, {"name": "target observation"})).id,
        "status": "scheduled",
        "status_value": "scheduled",
        "subtasks": [
            {
                "id": 2000033,
                "url": "http://localhost:3000/api/subtask/2000033",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "cluster": "http://localhost:3000/api/cluster/1",
                "cluster_id": 1,
                "cluster_name": "CEP4",
                "created_at": "2022-05-09T05:13:07.019676",
                "created_or_updated_by_user": null,
                "created_or_updated_by_user_id": null,
                "duration": 300,
                "error_reason": null,
                "global_identifier": "http://localhost:3000/api/sip_identifier/71",
                "global_identifier_id": 71,
                "global_parset_identifier": "http://localhost:3000/api/sip_identifier/72",
                "global_parset_identifier_id": 72,
                "inputs": [
                    "http://localhost:3000/api/subtask_input/46"
                ],
                "inputs_ids": [
                    46
                ],
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "outputs": [
                    "http://localhost:3000/api/subtask_output/29"
                ],
                "outputs_ids": [
                    29
                ],
                "primary": false,
                "process_start_time": "2022-05-09T14:24:49.932056",
                "process_stop_time": "2022-05-09T14:29:49.932056",
                "processing_feedback_doc": null,
                "processing_feedback_template": null,
                "processing_feedback_template_id": null,
                "scheduled_start_time": "2022-05-09T14:24:49.932056",
                "scheduled_stop_time": "2022-05-09T14:29:49.932056",
                "specifications_doc": {
                    "plots": {
                        "enabled": true,
                        "autocorrelation": true,
                        "crosscorrelation": true
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/QA%20plots/2/ref_resolved"
                },
                "specifications_template": "http://localhost:3000/api/subtask_template/3",
                "specifications_template_id": 3,
                "state": "http://localhost:3000/api/subtask_state/defined",
                "state_value": "defined",
                "subtask_type": "qa_plots",
                "tags": [],
                "task_blueprint": "http://localhost:3000/api/task_blueprint/24",
                "task_blueprint_id": 24,
                "total_nr_of_dataproducts": 0,
                "total_size_of_dataproducts": 0,
                "total_size_of_dataproducts_on_filesysem": 0,
                "unschedulable_reason": null,
                "updated_at": "2022-05-09T05:14:08.382249"
            },
            {
                "id": 2000032,
                "url": "http://localhost:3000/api/subtask/2000032",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "cluster": "http://localhost:3000/api/cluster/1",
                "cluster_id": 1,
                "cluster_name": "CEP4",
                "created_at": "2022-05-09T05:13:06.919324",
                "created_or_updated_by_user": null,
                "created_or_updated_by_user_id": null,
                "duration": 300,
                "error_reason": null,
                "global_identifier": "http://localhost:3000/api/sip_identifier/69",
                "global_identifier_id": 69,
                "global_parset_identifier": "http://localhost:3000/api/sip_identifier/70",
                "global_parset_identifier_id": 70,
                "inputs": [
                    "http://localhost:3000/api/subtask_input/45"
                ],
                "inputs_ids": [
                    45
                ],
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "outputs": [
                    "http://localhost:3000/api/subtask_output/28"
                ],
                "outputs_ids": [
                    28
                ],
                "primary": false,
                "process_start_time": "2022-05-09T14:19:49.932056",
                "process_stop_time": "2022-05-09T14:24:49.932056",
                "processing_feedback_doc": null,
                "processing_feedback_template": null,
                "processing_feedback_template_id": null,
                "scheduled_start_time": "2022-05-09T14:19:49.932056",
                "scheduled_stop_time": "2022-05-09T14:24:49.932056",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/QA%20file%20conversion/2/ref_resolved",
                    "file_conversion": {
                        "enabled": true,
                        "nr_of_subbands": -1,
                        "nr_of_timestamps": 256
                    }
                },
                "specifications_template": "http://localhost:3000/api/subtask_template/5",
                "specifications_template_id": 5,
                "state": "http://localhost:3000/api/subtask_state/defined",
                "state_value": "defined",
                "subtask_type": "qa_files",
                "tags": [],
                "task_blueprint": "http://localhost:3000/api/task_blueprint/24",
                "task_blueprint_id": 24,
                "total_nr_of_dataproducts": 0,
                "total_size_of_dataproducts": 0,
                "total_size_of_dataproducts_on_filesysem": 0,
                "unschedulable_reason": null,
                "updated_at": "2022-05-09T05:14:08.364849"
            },
            {
                "id": 2000031,
                "url": "http://localhost:3000/api/subtask/2000031",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "cluster": "http://localhost:3000/api/cluster/1",
                "cluster_id": 1,
                "cluster_name": "CEP4",
                "created_at": "2022-05-09T05:13:06.767385",
                "created_or_updated_by_user": null,
                "created_or_updated_by_user_id": null,
                "duration": 28800,
                "error_reason": null,
                "global_identifier": "http://localhost:3000/api/sip_identifier/67",
                "global_identifier_id": 67,
                "global_parset_identifier": "http://localhost:3000/api/sip_identifier/68",
                "global_parset_identifier_id": 68,
                "inputs": [],
                "inputs_ids": [],
                "obsolete_since": null,
                "on_sky_duration": "08:00:00",
                "on_sky_start_time": "2022-05-09T06:19:49.932056",
                "on_sky_stop_time": "2022-05-09T14:19:49.932056",
                "outputs": [
                    "http://localhost:3000/api/subtask_output/27"
                ],
                "outputs_ids": [
                    27
                ],
                "primary": true,
                "process_start_time": "2022-05-09T06:19:49.932056",
                "process_stop_time": "2022-05-09T14:19:49.932056",
                "processing_feedback_doc": null,
                "processing_feedback_template": null,
                "processing_feedback_template_id": null,
                "scheduled_start_time": "2022-05-09T06:19:49.932056",
                "scheduled_stop_time": "2022-05-09T14:19:49.932056",
                "specifications_doc": {
                    "QA": {
                        "inspection_plots": "msplots"
                    },
                    "COBALT": {
                        "debug": {
                            "write_output": true,
                            "benchmark_file": "",
                            "annotate_broken_antennas": true
                        },
                        "version": 1,
                        "blocksize": 195584,
                        "resources": {
                            "writer_nodes": [
                                "cpu01.cep4.infiniband.lofar",
                                "cpu02.cep4.infiniband.lofar",
                                "cpu03.cep4.infiniband.lofar",
                                "cpu04.cep4.infiniband.lofar",
                                "cpu05.cep4.infiniband.lofar"
                            ],
                            "processing_nodes": [
                                "cbt201_0",
                                "cbt201_1",
                                "cbt202_0",
                                "cbt202_1",
                                "cbt203_0",
                                "cbt203_1",
                                "cbt204_0",
                                "cbt204_1",
                                "cbt205_0",
                                "cbt205_1",
                                "cbt207_0",
                                "cbt207_1",
                                "cbt208_0",
                                "cbt208_1",
                                "cbt209_0",
                                "cbt209_1",
                                "cbt210_0",
                                "cbt210_1",
                                "cbt211_0",
                                "cbt211_1",
                                "cbt212_0",
                                "cbt212_1"
                            ]
                        },
                        "beamformer": {
                            "input_ppf": false,
                            "tab_pipelines": [],
                            "flyseye_pipelines": []
                        },
                        "correlator": {
                            "enabled": true,
                            "doppler_correction": false,
                            "channels_per_subband": 64,
                            "blocks_per_integration": 1,
                            "integrations_per_block": 1
                        },
                        "delay_compensation": true,
                        "bandpass_correction": true
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/observation%20control/2/ref_resolved",
                    "stations": {
                        "filter": "HBA_110_190",
                        "antenna_set": "HBA_DUAL_INNER",
                        "station_list": [
                            "CS001",
                            "CS002",
                            "CS003",
                            "CS004",
                            "CS005",
                            "CS006",
                            "CS007",
                            "CS011",
                            "CS013",
                            "CS017",
                            "CS021",
                            "CS024",
                            "CS026",
                            "CS028",
                            "CS030",
                            "CS031",
                            "CS032",
                            "CS101",
                            "CS103",
                            "CS201",
                            "CS301",
                            "CS302",
                            "CS401",
                            "CS501",
                            "DE601",
                            "DE602",
                            "DE603",
                            "DE604",
                            "DE605",
                            "DE609",
                            "FR606",
                            "IE613",
                            "LV614",
                            "PL610",
                            "PL611",
                            "PL612",
                            "RS106",
                            "RS205",
                            "RS208",
                            "RS210",
                            "RS305",
                            "RS306",
                            "RS307",
                            "RS310",
                            "RS406",
                            "RS407",
                            "RS409",
                            "RS503",
                            "RS508",
                            "RS509",
                            "SE607",
                            "UK608"
                        ],
                        "analog_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+01Paaa+02REF",
                            "direction_type": "J2000"
                        },
                        "digital_pointings": [
                            {
                                "name": "sap1",
                                "pointing": {
                                    "angle1": 0.6624317181687094,
                                    "angle2": 1.5579526427549426,
                                    "target": "Paaa+01",
                                    "direction_type": "J2000"
                                },
                                "subbands": [
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    182,
                                    183,
                                    184,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    212,
                                    213,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    242,
                                    243,
                                    244,
                                    245,
                                    246,
                                    247,
                                    248,
                                    249,
                                    250,
                                    251,
                                    252,
                                    253,
                                    254,
                                    255,
                                    257,
                                    258,
                                    259,
                                    260,
                                    261,
                                    262,
                                    263,
                                    264,
                                    265,
                                    266,
                                    267,
                                    268,
                                    269,
                                    270,
                                    271,
                                    272,
                                    273,
                                    275,
                                    276,
                                    277,
                                    278,
                                    279,
                                    280,
                                    281,
                                    282,
                                    283,
                                    284,
                                    285,
                                    286,
                                    287,
                                    288,
                                    289,
                                    290,
                                    291,
                                    292,
                                    293,
                                    294,
                                    295,
                                    296,
                                    297,
                                    298,
                                    299,
                                    300,
                                    302,
                                    303,
                                    304,
                                    305,
                                    306,
                                    307,
                                    308,
                                    309,
                                    310,
                                    311,
                                    312,
                                    313,
                                    314,
                                    315,
                                    316,
                                    317,
                                    318,
                                    319,
                                    320,
                                    321,
                                    322,
                                    323,
                                    324,
                                    325,
                                    326,
                                    327,
                                    328,
                                    330,
                                    331,
                                    332,
                                    333,
                                    334,
                                    335,
                                    336,
                                    337,
                                    338,
                                    339,
                                    340,
                                    341,
                                    342,
                                    343,
                                    344,
                                    345,
                                    346,
                                    347,
                                    349,
                                    364,
                                    372,
                                    380,
                                    388,
                                    396,
                                    404,
                                    413,
                                    421,
                                    430,
                                    438,
                                    447
                                ]
                            },
                            {
                                "name": "sap2",
                                "pointing": {
                                    "angle1": 0.6624317181687094,
                                    "angle2": 1.5579526427549426,
                                    "target": "Paaa+02",
                                    "direction_type": "J2000"
                                },
                                "subbands": [
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    182,
                                    183,
                                    184,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    212,
                                    213,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    242,
                                    243,
                                    244,
                                    245,
                                    246,
                                    247,
                                    248,
                                    249,
                                    250,
                                    251,
                                    252,
                                    253,
                                    254,
                                    255,
                                    257,
                                    258,
                                    259,
                                    260,
                                    261,
                                    262,
                                    263,
                                    264,
                                    265,
                                    266,
                                    267,
                                    268,
                                    269,
                                    270,
                                    271,
                                    272,
                                    273,
                                    275,
                                    276,
                                    277,
                                    278,
                                    279,
                                    280,
                                    281,
                                    282,
                                    283,
                                    284,
                                    285,
                                    286,
                                    287,
                                    288,
                                    289,
                                    290,
                                    291,
                                    292,
                                    293,
                                    294,
                                    295,
                                    296,
                                    297,
                                    298,
                                    299,
                                    300,
                                    302,
                                    303,
                                    304,
                                    305,
                                    306,
                                    307,
                                    308,
                                    309,
                                    310,
                                    311,
                                    312,
                                    313,
                                    314,
                                    315,
                                    316,
                                    317,
                                    318,
                                    319,
                                    320,
                                    321,
                                    322,
                                    323,
                                    324,
                                    325,
                                    326,
                                    327,
                                    328,
                                    330,
                                    331,
                                    332,
                                    333,
                                    334,
                                    335,
                                    336,
                                    337,
                                    338,
                                    339,
                                    340,
                                    341,
                                    342,
                                    343,
                                    344,
                                    345,
                                    346,
                                    347,
                                    349,
                                    364,
                                    372,
                                    380,
                                    388,
                                    396,
                                    404,
                                    413,
                                    421,
                                    430,
                                    438,
                                    447
                                ]
                            }
                        ]
                    }
                },
                "specifications_template": "http://localhost:3000/api/subtask_template/4",
                "specifications_template_id": 4,
                "state": "http://localhost:3000/api/subtask_state/scheduled",
                "state_value": "scheduled",
                "subtask_type": "observation",
                "tags": [],
                "task_blueprint": "http://localhost:3000/api/task_blueprint/24",
                "task_blueprint_id": 24,
                "total_nr_of_dataproducts": 0,
                "total_size_of_dataproducts": 0,
                "total_size_of_dataproducts_on_filesysem": 0,
                "unschedulable_reason": null,
                "updated_at": "2022-05-09T05:14:08.289861"
            }
        ],
        "subtasks_ids": [
            2000033,
            2000032,
            2000031
        ],
        "tags": [],
        "task_type": "observation",
        "unschedulable_reason": "",
        "updated_at": "2022-05-09T05:13:05.477861",
        "predecessors": [],
        "successors": [
            {
              "id": 28,
              "url": "http://localhost:3000/api/task_blueprint/28",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/40"
              ],
              "consumed_by_ids": [
                40
              ],
              "created_at": "2022-05-09T05:13:05.670728",
              "description": "Preprocessing Pipeline for Target Observation target1, SAP000, 120-168 MHz, 1s, 16ch/sb",
              "draft": "http://localhost:3000/api/task_draft/28",
              "draft_id": 28,
              "duration": 28800,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target1",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-05-09T14:19:49.932056",
              "process_stop_time": "2022-05-09T22:19:49.932056",
              "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/37"
              ],
              "produced_by_ids": [
                37
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-05-09T14:19:49.932056",
              "scheduled_stop_time": "2022-05-09T22:19:49.932056",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
              "scheduling_unit_blueprint_id": 4,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Paaa+01/TP",
              "specifications_doc": {
                "flag": {
                  "rfi_strategy": "HBAdefault",
                  "outerchannels": true,
                  "autocorrelations": true
                },
                "demix": {
                  "sources": [],
                  "time_steps": 10,
                  "ignore_target": false,
                  "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                  "time_steps": 1,
                  "frequency_steps": 4
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                  "where": {
                    "cluster": "CEP4",
                    "partition": "cpu"
                  },
                  "cores_per_task": 2,
                  "parallel_tasks": 122
                }
              },
              "specifications_template": "http://localhost:3000/api/task_template/10",
              "specifications_template_id": 10,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                {
                  "id": 2000039,
                  "url": "http://localhost:3000/api/subtask/2000039",
                  "actual_on_sky_start_time": null,
                  "actual_on_sky_stop_time": null,
                  "actual_process_start_time": null,
                  "actual_process_stop_time": null,
                  "cluster": "http://localhost:3000/api/cluster/1",
                  "cluster_id": 1,
                  "cluster_name": "CEP4",
                  "created_at": "2022-05-09T05:13:07.839386",
                  "created_or_updated_by_user": null,
                  "created_or_updated_by_user_id": null,
                  "duration": 28800,
                  "error_reason": null,
                  "global_identifier": "http://localhost:3000/api/sip_identifier/83",
                  "global_identifier_id": 83,
                  "global_parset_identifier": "http://localhost:3000/api/sip_identifier/84",
                  "global_parset_identifier_id": 84,
                  "inputs": [
                    "http://localhost:3000/api/subtask_input/51"
                  ],
                  "inputs_ids": [
                    51
                  ],
                  "obsolete_since": null,
                  "on_sky_duration": null,
                  "on_sky_start_time": null,
                  "on_sky_stop_time": null,
                  "outputs": [
                    "http://localhost:3000/api/subtask_output/35"
                  ],
                  "outputs_ids": [
                    35
                  ],
                  "primary": true,
                  "process_start_time": "2022-05-09T14:19:49.932056",
                  "process_stop_time": "2022-05-09T22:19:49.932056",
                  "processing_feedback_doc": null,
                  "processing_feedback_template": null,
                  "processing_feedback_template_id": null,
                  "scheduled_start_time": "2022-05-09T14:19:49.932056",
                  "scheduled_stop_time": "2022-05-09T22:19:49.932056",
                  "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "demixer": {
                      "enabled": true,
                      "baselines": "CS*,RS*&",
                      "time_steps": 1,
                      "demix_always": [],
                      "ignore_target": false,
                      "demix_if_needed": [],
                      "frequency_steps": 4,
                      "demix_time_steps": 10,
                      "demix_frequency_steps": 64
                    },
                    "aoflagger": {
                      "enabled": true,
                      "strategy": "HBAdefault"
                    },
                    "preflagger0": {
                      "enabled": true,
                      "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                    },
                    "preflagger1": {
                      "enabled": true,
                      "corrtype": "auto"
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                      "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                    }
                  },
                  "specifications_template": "http://localhost:3000/api/subtask_template/7",
                  "specifications_template_id": 7,
                  "state": "http://localhost:3000/api/subtask_state/defined",
                  "state_value": "defined",
                  "subtask_type": "pipeline",
                  "tags": [],
                  "task_blueprint": "http://localhost:3000/api/task_blueprint/28",
                  "task_blueprint_id": 28,
                  "total_nr_of_dataproducts": 0,
                  "total_size_of_dataproducts": 0,
                  "total_size_of_dataproducts_on_filesysem": 0,
                  "unschedulable_reason": null,
                  "updated_at": "2022-05-09T05:14:08.333426"
                }
              ],
              "subtasks_ids": [
                2000039
              ],
              "tags": [],
              "task_type": "pipeline",
              "unschedulable_reason": "",
              "updated_at": "2022-05-09T05:13:05.670749"
            },
            {
              "id": 27,
              "url": "http://localhost:3000/api/task_blueprint/27",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                "http://localhost:3000/api/task_relation_blueprint/41"
              ],
              "consumed_by_ids": [
                41
              ],
              "created_at": "2022-05-09T05:13:05.624931",
              "description": "Preprocessing Pipeline for Target Observation target2, SAP001, 120-168 MHz, 1s, 16ch/sb",
              "draft": "http://localhost:3000/api/task_draft/27",
              "draft_id": 27,
              "duration": 28800,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target2",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-05-09T14:19:49.932056",
              "process_stop_time": "2022-05-09T22:19:49.932056",
              "produced_by": [
                "http://localhost:3000/api/task_relation_blueprint/36"
              ],
              "produced_by_ids": [
                36
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-05-09T14:19:49.932056",
              "scheduled_stop_time": "2022-05-09T22:19:49.932056",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
              "scheduling_unit_blueprint_id": 4,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Paaa+02/TP",
              "specifications_doc": {
                "flag": {
                  "rfi_strategy": "HBAdefault",
                  "outerchannels": true,
                  "autocorrelations": true
                },
                "demix": {
                  "sources": [],
                  "time_steps": 10,
                  "ignore_target": false,
                  "frequency_steps": 64
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                "average": {
                  "time_steps": 1,
                  "frequency_steps": 4
                },
                "storagemanager": "dysco",
                "cluster_resources": {
                  "where": {
                    "cluster": "CEP4",
                    "partition": "cpu"
                  },
                  "cores_per_task": 2,
                  "parallel_tasks": 122
                }
              },
              "specifications_template": "http://localhost:3000/api/task_template/10",
              "specifications_template_id": 10,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                {
                  "id": 2000034,
                  "url": "http://localhost:3000/api/subtask/2000034",
                  "actual_on_sky_start_time": null,
                  "actual_on_sky_stop_time": null,
                  "actual_process_start_time": null,
                  "actual_process_stop_time": null,
                  "cluster": "http://localhost:3000/api/cluster/1",
                  "cluster_id": 1,
                  "cluster_name": "CEP4",
                  "created_at": "2022-05-09T05:13:07.096931",
                  "created_or_updated_by_user": null,
                  "created_or_updated_by_user_id": null,
                  "duration": 28800,
                  "error_reason": null,
                  "global_identifier": "http://localhost:3000/api/sip_identifier/73",
                  "global_identifier_id": 73,
                  "global_parset_identifier": "http://localhost:3000/api/sip_identifier/74",
                  "global_parset_identifier_id": 74,
                  "inputs": [
                    "http://localhost:3000/api/subtask_input/47"
                  ],
                  "inputs_ids": [
                    47
                  ],
                  "obsolete_since": null,
                  "on_sky_duration": null,
                  "on_sky_start_time": null,
                  "on_sky_stop_time": null,
                  "outputs": [
                    "http://localhost:3000/api/subtask_output/30"
                  ],
                  "outputs_ids": [
                    30
                  ],
                  "primary": true,
                  "process_start_time": "2022-05-09T14:19:49.932056",
                  "process_stop_time": "2022-05-09T22:19:49.932056",
                  "processing_feedback_doc": null,
                  "processing_feedback_template": null,
                  "processing_feedback_template_id": null,
                  "scheduled_start_time": "2022-05-09T14:19:49.932056",
                  "scheduled_stop_time": "2022-05-09T22:19:49.932056",
                  "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "demixer": {
                      "enabled": true,
                      "baselines": "CS*,RS*&",
                      "time_steps": 1,
                      "demix_always": [],
                      "ignore_target": false,
                      "demix_if_needed": [],
                      "frequency_steps": 4,
                      "demix_time_steps": 10,
                      "demix_frequency_steps": 64
                    },
                    "aoflagger": {
                      "enabled": true,
                      "strategy": "HBAdefault"
                    },
                    "preflagger0": {
                      "enabled": true,
                      "channels": "0..nchan/32-1,31*nchan/32..nchan-1"
                    },
                    "preflagger1": {
                      "enabled": true,
                      "corrtype": "auto"
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                      "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                    }
                  },
                  "specifications_template": "http://localhost:3000/api/subtask_template/7",
                  "specifications_template_id": 7,
                  "state": "http://localhost:3000/api/subtask_state/defined",
                  "state_value": "defined",
                  "subtask_type": "pipeline",
                  "tags": [],
                  "task_blueprint": "http://localhost:3000/api/task_blueprint/27",
                  "task_blueprint_id": 27,
                  "total_nr_of_dataproducts": 0,
                  "total_size_of_dataproducts": 0,
                  "total_size_of_dataproducts_on_filesysem": 0,
                  "unschedulable_reason": null,
                  "updated_at": "2022-05-09T05:14:08.411630"
                }
              ],
              "subtasks_ids": [
                2000034
              ],
              "tags": [],
              "task_type": "pipeline",
              "unschedulable_reason": "",
              "updated_at": "2022-05-09T05:13:05.624946"
            }
        ]
    },
    {
        "id": 50,
        "url": "http://localhost:3000/api/task_blueprint/50",
        "actual_on_sky_start_time": null,
        "actual_on_sky_stop_time": null,
        "actual_process_start_time": null,
        "actual_process_stop_time": null,
        "consumed_by": [
            "http://localhost:3000/api/task_relation_blueprint/68",
            "http://localhost:3000/api/task_relation_blueprint/69",
            "http://localhost:3000/api/task_relation_blueprint/70",
            "http://localhost:3000/api/task_relation_blueprint/71"
        ],
        "consumed_by_ids": [
            68,
            69,
            70,
            71
        ],
        "created_at": "2022-04-21T08:16:31.195314",
        "description": "Pulsar Pipeline for default HBA pulsar timing observation",
        "draft": "http://localhost:3000/api/task_draft/50",
        "draft_id": 50,
        "draftObject": {
            "id": 50,
            "url": "http://localhost:3000/api/task_draft/50",
            "connector_types": [
                {
                    "id": 11,
                    "url": "http://localhost:3000/api/task_connector_type/11",
                    "dataformat": "http://localhost:3000/api/dataformat/pulp%20analysis",
                    "dataformat_value": "pulp analysis",
                    "datatype": "http://localhost:3000/api/datatype/pulsar%20profile",
                    "datatype_value": "pulsar profile",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/9",
                    "task_template_id": 9
                },
                {
                    "id": 10,
                    "url": "http://localhost:3000/api/task_connector_type/10",
                    "dataformat": "http://localhost:3000/api/dataformat/pulp%20summary",
                    "dataformat_value": "pulp summary",
                    "datatype": "http://localhost:3000/api/datatype/quality",
                    "datatype_value": "quality",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/any",
                    "role_value": "any",
                    "task_template": "http://localhost:3000/api/task_template/9",
                    "task_template_id": 9
                },
                {
                    "id": 9,
                    "url": "http://localhost:3000/api/task_connector_type/9",
                    "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                    "dataformat_value": "Beamformed",
                    "datatype": "http://localhost:3000/api/datatype/time%20series",
                    "datatype_value": "time series",
                    "iotype": "http://localhost:3000/api/iotype/input",
                    "iotype_value": "input",
                    "role": "http://localhost:3000/api/role/beamformer",
                    "role_value": "beamformer",
                    "task_template": "http://localhost:3000/api/task_template/9",
                    "task_template_id": 9
                }
            ],
            "consumed_by": [
                "http://localhost:3000/api/task_relation_draft/67",
                "http://localhost:3000/api/task_relation_draft/68",
                "http://localhost:3000/api/task_relation_draft/70",
                "http://localhost:3000/api/task_relation_draft/71"
            ],
            "consumed_by_ids": [
                67,
                68,
                70,
                71
            ],
            "created_at": "2022-04-21T08:16:30.745911",
            "description": "Pulsar Pipeline for default HBA pulsar timing observation",
            "duration": 0,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Pipeline",
            "output_pinned": false,
            "produced_by": [
                "http://localhost:3000/api/task_relation_draft/66"
            ],
            "produced_by_ids": [
                66
            ],
            "relative_start_time": 0,
            "relative_stop_time": 0,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/9",
            "scheduling_unit_draft_id": 9,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "",
            "specifications_doc": {
                "dspsr": {
                    "digifil": {
                        "dm": 0,
                        "channels_per_part": 320,
                        "coherent_dedispersion": true,
                        "integration_time_factor": 4
                    },
                    "enabled": true,
                    "filterbank": {
                        "enabled": false,
                        "channels_per_part": 120,
                        "coherent_dedispersion": true
                    },
                    "rfi_excision": true,
                    "optimise_period_dm": true,
                    "subintegration_length": -1,
                    "single_pulse_subintegration": false
                },
                "output": {
                    "quantisation": {
                        "scale": 5,
                        "enabled": false
                    },
                    "dynamic_spectrum": {
                        "enabled": false,
                        "time_average": 0.5
                    }
                },
                "presto": {
                    "input": {
                        "nr_blocks": 100,
                        "decode_sigma": 3,
                        "samples_per_block": 8192
                    },
                    "rrats": {
                        "enabled": false,
                        "dm_range": 5
                    },
                    "rfifind": {
                        "blocks": 16
                    },
                    "prepdata": {
                        "dm": -1
                    },
                    "prepfold": false,
                    "fold_profile": true
                },
                "pulsar": {
                    "name": "",
                    "strategy": "meta"
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/pulsar%20pipeline/2/ref_resolved",
                "cluster_resources": {
                    "where": {
                        "cluster": "CEP4",
                        "partition": "cpu"
                    },
                    "cores_per_task": 2,
                    "parallel_tasks": 122
                },
                "single_pulse_search": false
            },
            "specifications_template": "http://localhost:3000/api/task_template/9",
            "specifications_template_id": 9,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/50"
            ],
            "task_blueprints_ids": [
                50
            ],
            "task_type": "pipeline",
            "updated_at": "2022-04-21T08:16:30.745924"
        },
        "duration": 120,
        "error_reason": "",
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Pipeline",
        "obsolete_since": null,
        "on_sky_duration": null,
        "on_sky_start_time": null,
        "on_sky_stop_time": null,
        "output_pinned": false,
        "process_start_time": "2022-04-22T02:54:06.166227",
        "process_stop_time": "2022-04-22T02:56:06.166227",
        "produced_by": [
            "http://localhost:3000/api/task_relation_blueprint/66"
        ],
        "produced_by_ids": [
            66
        ],
        "relative_start_time": 0,
        "relative_stop_time": 0,
        "scheduled_start_time": "2022-04-22T02:54:06.166227",
        "scheduled_stop_time": "2022-04-22T02:56:06.166227",
        "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/9",
        "scheduling_unit_blueprint_id": 9,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "",
        "specifications_doc": {
            "dspsr": {
                "digifil": {
                    "dm": 0,
                    "channels_per_part": 320,
                    "coherent_dedispersion": true,
                    "integration_time_factor": 4
                },
                "enabled": true,
                "filterbank": {
                    "enabled": false,
                    "channels_per_part": 120,
                    "coherent_dedispersion": true
                },
                "rfi_excision": true,
                "optimise_period_dm": true,
                "subintegration_length": -1,
                "single_pulse_subintegration": false
            },
            "output": {
                "quantisation": {
                    "scale": 5,
                    "enabled": false
                },
                "dynamic_spectrum": {
                    "enabled": false,
                    "time_average": 0.5
                }
            },
            "presto": {
                "input": {
                    "nr_blocks": 100,
                    "decode_sigma": 3,
                    "samples_per_block": 8192
                },
                "rrats": {
                    "enabled": false,
                    "dm_range": 5
                },
                "rfifind": {
                    "blocks": 16
                },
                "prepdata": {
                    "dm": -1
                },
                "prepfold": false,
                "fold_profile": true
            },
            "pulsar": {
                "name": "",
                "strategy": "meta"
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/pulsar%20pipeline/2/ref_resolved",
            "cluster_resources": {
                "where": {
                    "cluster": "CEP4",
                    "partition": "cpu"
                },
                "cores_per_task": 2,
                "parallel_tasks": 122
            },
            "single_pulse_search": false
        },
        "specifications_template": "http://localhost:3000/api/task_template/"+ (_.find(templates, {"name": "pulsar pipeline", "state": "active"})).id,
        "specifications_template_id": (_.find(templates, {"name": "pulsar pipeline", "state": "active"})).id,
        "status": "schedulable",
        "status_value": "schedulable",
        "subtasks": [
            {
                "id": 2000069,
                "url": "http://localhost:3000/api/subtask/2000069",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "cluster": "http://localhost:3000/api/cluster/1",
                "cluster_id": 1,
                "cluster_name": "CEP4",
                "created_at": "2022-04-21T08:16:31.879738",
                "created_or_updated_by_user": null,
                "created_or_updated_by_user_id": null,
                "duration": 120,
                "error_reason": null,
                "global_identifier": "http://localhost:3000/api/sip_identifier/148",
                "global_identifier_id": 148,
                "global_parset_identifier": "http://localhost:3000/api/sip_identifier/149",
                "global_parset_identifier_id": 149,
                "inputs": [
                    "http://localhost:3000/api/subtask_input/90"
                ],
                "inputs_ids": [
                    90
                ],
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "outputs": [
                    "http://localhost:3000/api/subtask_output/61",
                    "http://localhost:3000/api/subtask_output/62"
                ],
                "outputs_ids": [
                    61,
                    62
                ],
                "primary": true,
                "process_start_time": "2022-04-22T02:54:06.166227",
                "process_stop_time": "2022-04-22T02:56:06.166227",
                "processing_feedback_doc": null,
                "processing_feedback_template": null,
                "processing_feedback_template_id": null,
                "scheduled_start_time": "2022-04-22T02:54:06.166227",
                "scheduled_stop_time": "2022-04-22T02:56:06.166227",
                "specifications_doc": {
                    "dspsr": {
                        "norfi": false,
                        "nopdmp": false,
                        "tsubint": -1,
                        "skip_dspsr": false,
                        "dspsr_extra_opts": "-U minX1 -t 1  ",
                        "digifil_extra_opts": "-D 0 -t 4 -F 20"
                    },
                    "output": {
                        "raw_to_8bit": false,
                        "8bit_conversion_sigma": 5,
                        "skip_dynamic_spectrum": true,
                        "dynamic_spectrum_time_average": 0.5
                    },
                    "presto": {
                        "rrats": false,
                        "nofold": false,
                        "decode_sigma": 3,
                        "skip_prepfold": true,
                        "decode_nblocks": 100,
                        "rrats_dm_range": 5,
                        "rfifind_extra_opts": "-blocks 16",
                        "2bf2fits_extra_opts": "-nsamples=8192",
                        "prepdata_extra_opts": "",
                        "prepfold_extra_opts": "",
                        "prepsubband_extra_opts": ""
                    },
                    "pulsar": "meta",
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/pulsar%20pipeline/2/ref_resolved",
                    "threads": 2,
                    "single_pulse": false,
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    }
                },
                "specifications_template": "http://localhost:3000/api/subtask_template/6",
                "specifications_template_id": 6,
                "state": "http://localhost:3000/api/subtask_state/defined",
                "state_value": "defined",
                "subtask_type": "pipeline",
                "tags": [],
                "task_blueprint": "http://localhost:3000/api/task_blueprint/50",
                "task_blueprint_id": 50,
                "total_nr_of_dataproducts": 0,
                "total_size_of_dataproducts": 0,
                "total_size_of_dataproducts_on_filesysem": 0,
                "unschedulable_reason": null,
                "updated_at": "2022-04-21T08:16:50.330052"
            }
        ],
        "subtasks_ids": [
            2000069
        ],
        "tags": [],
        "task_type": "pipeline",
        "unschedulable_reason": "",
        "updated_at": "2022-04-21T08:16:31.195328",
        "predecessors": [
            {
                "id": 49,
                "url": "http://localhost:3000/api/task_blueprint/49",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_blueprint/66",
                    "http://localhost:3000/api/task_relation_blueprint/67"
                ],
                "consumed_by_ids": [
                    66,
                    67
                ],
                "created_at": "2022-04-21T08:16:31.148213",
                "description": "Beamforming observation for default pulsar timing with HBA",
                "draft": "http://localhost:3000/api/task_draft/49",
                "draft_id": 49,
                "duration": 120,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Observation",
                "obsolete_since": null,
                "on_sky_duration": 120,
                "on_sky_start_time": "2022-04-22T02:52:06.166227",
                "on_sky_stop_time": "2022-04-22T02:54:06.166227",
                "output_pinned": false,
                "process_start_time": "2022-04-22T02:52:06.166227",
                "process_stop_time": "2022-04-22T02:54:06.166227",
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 0,
                "relative_stop_time": 120,
                "scheduled_start_time": "2022-04-22T02:52:06.166227",
                "scheduled_stop_time": "2022-04-22T02:54:06.166227",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/9",
                "scheduling_unit_blueprint_id": 9,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
                    "duration": 120,
                    "beamformer": {
                        "ppf": false,
                        "pipelines": [
                            {
                                "name": "Beamformer CV",
                                "coherent": {
                                    "SAPs": [
                                        {
                                            "name": "SAP0",
                                            "tabs": [
                                                {
                                                    "duration": 0,
                                                    "pointing": {
                                                        "angle1": 0,
                                                        "angle2": 0,
                                                        "target": "target1",
                                                        "direction_type": "J2000"
                                                    },
                                                    "relative": true,
                                                    "allow_after_previous": true
                                                }
                                            ],
                                            "subbands": {
                                                "list": [
                                                    51,
                                                    52,
                                                    53,
                                                    54,
                                                    55,
                                                    56,
                                                    57,
                                                    58,
                                                    59,
                                                    60,
                                                    61,
                                                    62,
                                                    63,
                                                    64,
                                                    65,
                                                    66,
                                                    67,
                                                    68,
                                                    69,
                                                    70,
                                                    71,
                                                    72,
                                                    73,
                                                    74,
                                                    75,
                                                    76,
                                                    77,
                                                    78,
                                                    79,
                                                    80,
                                                    81,
                                                    82,
                                                    83,
                                                    84,
                                                    85,
                                                    86,
                                                    87,
                                                    88,
                                                    89,
                                                    90,
                                                    91,
                                                    92,
                                                    93,
                                                    94,
                                                    95,
                                                    96,
                                                    97,
                                                    98,
                                                    99,
                                                    100,
                                                    101,
                                                    102,
                                                    103,
                                                    104,
                                                    105,
                                                    106,
                                                    107,
                                                    108,
                                                    109,
                                                    110,
                                                    111,
                                                    112,
                                                    113,
                                                    114,
                                                    115,
                                                    116,
                                                    117,
                                                    118,
                                                    119,
                                                    120,
                                                    121,
                                                    122,
                                                    123,
                                                    124,
                                                    125,
                                                    126,
                                                    127,
                                                    128,
                                                    129,
                                                    130,
                                                    131,
                                                    132,
                                                    133,
                                                    134,
                                                    135,
                                                    136,
                                                    137,
                                                    138,
                                                    139,
                                                    140,
                                                    141,
                                                    142,
                                                    143,
                                                    144,
                                                    145,
                                                    146,
                                                    147,
                                                    148,
                                                    149,
                                                    150,
                                                    151,
                                                    152,
                                                    153,
                                                    154,
                                                    155,
                                                    156,
                                                    157,
                                                    158,
                                                    159,
                                                    160,
                                                    161,
                                                    162,
                                                    163,
                                                    164,
                                                    165,
                                                    166,
                                                    167,
                                                    168,
                                                    169,
                                                    170,
                                                    171,
                                                    172,
                                                    173,
                                                    174,
                                                    175,
                                                    176,
                                                    177,
                                                    178,
                                                    179,
                                                    180,
                                                    181,
                                                    182,
                                                    183,
                                                    184,
                                                    185,
                                                    186,
                                                    187,
                                                    188,
                                                    189,
                                                    190,
                                                    191,
                                                    192,
                                                    193,
                                                    194,
                                                    195,
                                                    196,
                                                    197,
                                                    198,
                                                    199,
                                                    200,
                                                    201,
                                                    202,
                                                    203,
                                                    204,
                                                    205,
                                                    206,
                                                    207,
                                                    208,
                                                    209,
                                                    210,
                                                    211,
                                                    212,
                                                    213,
                                                    214,
                                                    215,
                                                    216,
                                                    217,
                                                    218,
                                                    219,
                                                    220,
                                                    221,
                                                    222,
                                                    223,
                                                    224,
                                                    225,
                                                    226,
                                                    227,
                                                    228,
                                                    229,
                                                    230,
                                                    231,
                                                    232,
                                                    233,
                                                    234,
                                                    235,
                                                    236,
                                                    237,
                                                    238,
                                                    239,
                                                    240,
                                                    241,
                                                    242,
                                                    243,
                                                    244,
                                                    245,
                                                    246,
                                                    247,
                                                    248,
                                                    249,
                                                    250,
                                                    251,
                                                    252,
                                                    253,
                                                    254,
                                                    255,
                                                    256,
                                                    257,
                                                    258,
                                                    259,
                                                    260,
                                                    261,
                                                    262,
                                                    263,
                                                    264,
                                                    265,
                                                    266,
                                                    267,
                                                    268,
                                                    269,
                                                    270,
                                                    271,
                                                    272,
                                                    273,
                                                    274,
                                                    275,
                                                    276,
                                                    277,
                                                    278,
                                                    279,
                                                    280,
                                                    281,
                                                    282,
                                                    283,
                                                    284,
                                                    285,
                                                    286,
                                                    287,
                                                    288,
                                                    289,
                                                    290,
                                                    291,
                                                    292,
                                                    293,
                                                    294,
                                                    295,
                                                    296,
                                                    297,
                                                    298,
                                                    299,
                                                    300,
                                                    301,
                                                    302,
                                                    303,
                                                    304,
                                                    305,
                                                    306,
                                                    307,
                                                    308,
                                                    309,
                                                    310,
                                                    311,
                                                    312,
                                                    313,
                                                    314,
                                                    315,
                                                    316,
                                                    317,
                                                    318,
                                                    319,
                                                    320,
                                                    321,
                                                    322,
                                                    323,
                                                    324,
                                                    325,
                                                    326,
                                                    327,
                                                    328,
                                                    329,
                                                    330,
                                                    331,
                                                    332,
                                                    333,
                                                    334,
                                                    335,
                                                    336,
                                                    337,
                                                    338,
                                                    339,
                                                    340,
                                                    341,
                                                    342,
                                                    343,
                                                    344,
                                                    345,
                                                    346,
                                                    347,
                                                    348,
                                                    349,
                                                    350,
                                                    351,
                                                    352,
                                                    353,
                                                    354,
                                                    355,
                                                    356,
                                                    357,
                                                    358,
                                                    359,
                                                    360,
                                                    361,
                                                    362,
                                                    363,
                                                    364,
                                                    365,
                                                    366,
                                                    367,
                                                    368,
                                                    369,
                                                    370,
                                                    371,
                                                    372,
                                                    373,
                                                    374,
                                                    375,
                                                    376,
                                                    377,
                                                    378,
                                                    379,
                                                    380,
                                                    381,
                                                    382,
                                                    383,
                                                    384,
                                                    385,
                                                    386,
                                                    387,
                                                    388,
                                                    389,
                                                    390,
                                                    391,
                                                    392,
                                                    393,
                                                    394,
                                                    395,
                                                    396,
                                                    397,
                                                    398,
                                                    399,
                                                    400,
                                                    401,
                                                    402,
                                                    403,
                                                    404,
                                                    405,
                                                    406,
                                                    407,
                                                    408,
                                                    409,
                                                    410,
                                                    411,
                                                    412,
                                                    413,
                                                    414,
                                                    415,
                                                    416,
                                                    417,
                                                    418,
                                                    419,
                                                    420,
                                                    421,
                                                    422,
                                                    423,
                                                    424,
                                                    425,
                                                    426,
                                                    427,
                                                    428,
                                                    429,
                                                    430,
                                                    431,
                                                    432,
                                                    433,
                                                    434,
                                                    435,
                                                    436,
                                                    437,
                                                    438,
                                                    439,
                                                    440,
                                                    441,
                                                    442,
                                                    443,
                                                    444,
                                                    445,
                                                    446,
                                                    447,
                                                    448,
                                                    449,
                                                    450
                                                ],
                                                "method": "copy"
                                            },
                                            "tab_rings": {
                                                "count": 0,
                                                "width": 0.01
                                            }
                                        }
                                    ],
                                    "settings": {
                                        "stokes": "XXYY",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 20,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "flys eye": {
                                    "enabled": false,
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "incoherent": {
                                    "SAPs": [],
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "station_groups": [
                                    {
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501"
                                        ],
                                        "max_nr_missing": 1
                                    }
                                ]
                            }
                        ]
                    },
                    "station_configuration": {
                        "SAPs": [
                            {
                                "name": "SAP0",
                                "subbands": [
                                    51,
                                    52,
                                    53,
                                    54,
                                    55,
                                    56,
                                    57,
                                    58,
                                    59,
                                    60,
                                    61,
                                    62,
                                    63,
                                    64,
                                    65,
                                    66,
                                    67,
                                    68,
                                    69,
                                    70,
                                    71,
                                    72,
                                    73,
                                    74,
                                    75,
                                    76,
                                    77,
                                    78,
                                    79,
                                    80,
                                    81,
                                    82,
                                    83,
                                    84,
                                    85,
                                    86,
                                    87,
                                    88,
                                    89,
                                    90,
                                    91,
                                    92,
                                    93,
                                    94,
                                    95,
                                    96,
                                    97,
                                    98,
                                    99,
                                    100,
                                    101,
                                    102,
                                    103,
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    137,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    164,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    181,
                                    182,
                                    183,
                                    184,
                                    185,
                                    186,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    210,
                                    211,
                                    212,
                                    213,
                                    214,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    241,
                                    242,
                                    243,
                                    244,
                                    245,
                                    246,
                                    247,
                                    248,
                                    249,
                                    250,
                                    251,
                                    252,
                                    253,
                                    254,
                                    255,
                                    256,
                                    257,
                                    258,
                                    259,
                                    260,
                                    261,
                                    262,
                                    263,
                                    264,
                                    265,
                                    266,
                                    267,
                                    268,
                                    269,
                                    270,
                                    271,
                                    272,
                                    273,
                                    274,
                                    275,
                                    276,
                                    277,
                                    278,
                                    279,
                                    280,
                                    281,
                                    282,
                                    283,
                                    284,
                                    285,
                                    286,
                                    287,
                                    288,
                                    289,
                                    290,
                                    291,
                                    292,
                                    293,
                                    294,
                                    295,
                                    296,
                                    297,
                                    298,
                                    299,
                                    300,
                                    301,
                                    302,
                                    303,
                                    304,
                                    305,
                                    306,
                                    307,
                                    308,
                                    309,
                                    310,
                                    311,
                                    312,
                                    313,
                                    314,
                                    315,
                                    316,
                                    317,
                                    318,
                                    319,
                                    320,
                                    321,
                                    322,
                                    323,
                                    324,
                                    325,
                                    326,
                                    327,
                                    328,
                                    329,
                                    330,
                                    331,
                                    332,
                                    333,
                                    334,
                                    335,
                                    336,
                                    337,
                                    338,
                                    339,
                                    340,
                                    341,
                                    342,
                                    343,
                                    344,
                                    345,
                                    346,
                                    347,
                                    348,
                                    349,
                                    350,
                                    351,
                                    352,
                                    353,
                                    354,
                                    355,
                                    356,
                                    357,
                                    358,
                                    359,
                                    360,
                                    361,
                                    362,
                                    363,
                                    364,
                                    365,
                                    366,
                                    367,
                                    368,
                                    369,
                                    370,
                                    371,
                                    372,
                                    373,
                                    374,
                                    375,
                                    376,
                                    377,
                                    378,
                                    379,
                                    380,
                                    381,
                                    382,
                                    383,
                                    384,
                                    385,
                                    386,
                                    387,
                                    388,
                                    389,
                                    390,
                                    391,
                                    392,
                                    393,
                                    394,
                                    395,
                                    396,
                                    397,
                                    398,
                                    399,
                                    400,
                                    401,
                                    402,
                                    403,
                                    404,
                                    405,
                                    406,
                                    407,
                                    408,
                                    409,
                                    410,
                                    411,
                                    412,
                                    413,
                                    414,
                                    415,
                                    416,
                                    417,
                                    418,
                                    419,
                                    420,
                                    421,
                                    422,
                                    423,
                                    424,
                                    425,
                                    426,
                                    427,
                                    428,
                                    429,
                                    430,
                                    431,
                                    432,
                                    433,
                                    434,
                                    435,
                                    436,
                                    437,
                                    438,
                                    439,
                                    440,
                                    441,
                                    442,
                                    443,
                                    444,
                                    445,
                                    446,
                                    447,
                                    448,
                                    449,
                                    450
                                ],
                                "digital_pointing": {
                                    "angle1": 0.92934186635,
                                    "angle2": 0.952579228492,
                                    "target": "target1",
                                    "direction_type": "J2000"
                                }
                            }
                        ],
                        "filter": "HBA_110_190",
                        "tile_beam": {
                            "angle1": 0.92934186635,
                            "angle2": 0.952579228492,
                            "target": "target1",
                            "direction_type": "J2000"
                        },
                        "antenna_set": "HBA_DUAL_INNER",
                        "station_groups": [
                            {
                                "stations": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                },
                "specifications_template": {
                    "id": 3,
                    "url": "http://localhost:3000/api/task_template/3",
                    "created_at": "2022-04-21T08:15:52.853823",
                    "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                    "name": "beamforming observation",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
                        "type": "object",
                        "title": "beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "beamformer",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "#/definitions/timedelta",
                                "title": "Duration",
                                "default": 300,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "#/definitions/beamformer",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "timedelta": {
                                "default": 0,
                                "description": "A time duration or delta expressed in seconds",
                                "type": "number"
                            },
                            "timestamp": {
                                "default": "1970-01-01T00:00:00Z",
                                "description": "A timestamp defined in UTC",
                                "format": "date-time",
                                "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                                "type": "string"
                            },
                            "beamformer": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "pipelines": {
                                        "additionalItems": false,
                                        "default": [
                                            {}
                                        ],
                                        "description": "This array of beamformer pipelines is currently restricted to exactly 1 item. Support for handling multiple pipelines (in cobalt) will be added in the future.",
                                        "items": {
                                            "$ref": "#/definitions/beamformer_pipeline"
                                        },
                                        "maxItems": 1,
                                        "minItems": 1,
                                        "title": "Pipelines",
                                        "type": "array"
                                    },
                                    "ppf": {
                                        "default": false,
                                        "description": "Use a PPF for the channellisation for beamforming. If not, an FFT is used.",
                                        "title": "Use PPF when beamforming",
                                        "type": "boolean"
                                    }
                                },
                                "required": [
                                    "pipelines",
                                    "ppf"
                                ],
                                "title": "Beamformer",
                                "type": "object"
                            },
                            "beamformer_pipeline": {
                                "additionalProperties": false,
                                "default": {},
                                "headerTemplate": "Beamformer Pipeline {{ self.index }}",
                                "properties": {
                                    "coherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to beamform.",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        },
                                                        "tab_rings": {
                                                            "additonalProperties": false,
                                                            "default": {},
                                                            "description": "Rings of TABs around the center of the beam.",
                                                            "properties": {
                                                                "count": {
                                                                    "default": 0,
                                                                    "maximum": 11,
                                                                    "minimum": 0,
                                                                    "title": "Number of rings",
                                                                    "type": "integer"
                                                                },
                                                                "width": {
                                                                    "default": 0.01,
                                                                    "description": "Distance between pointings.",
                                                                    "minimum": 0,
                                                                    "title": "Ring width",
                                                                    "type": "number"
                                                                }
                                                            },
                                                            "title": "Tied-Array Rings",
                                                            "type": "object"
                                                        },
                                                        "tabs": {
                                                            "additionalItems": false,
                                                            "default": [],
                                                            "description": "Tied-array beams to form",
                                                            "items": {
                                                                "additionalProperties": false,
                                                                "default": {},
                                                                "headerTemplate": "TAB {{ self.index }}",
                                                                "properties": {
                                                                    "allow_after_previous": {
                                                                        "default": true,
                                                                        "description": "This TAB can be recorded after the previous TAB, if duration allows.",
                                                                        "title": "Allow after previous",
                                                                        "type": "boolean"
                                                                    },
                                                                    "duration": {
                                                                        "default": 0,
                                                                        "description": "Exposure time of the TAB. 0 = observation duration.",
                                                                        "maximum": 0,
                                                                        "minimum": 0,
                                                                        "title": "Duration",
                                                                        "type": "integer"
                                                                    },
                                                                    "pointing": {
                                                                        "$ref": "#/definitions/pointing",
                                                                        "default": {}
                                                                    },
                                                                    "relative": {
                                                                        "default": false,
                                                                        "description": "Angular offset applied to the TAB pointing",
                                                                        "title": "Relative to SAP",
                                                                        "type": "boolean"
                                                                    }
                                                                },
                                                                "required": [
                                                                    "pointing",
                                                                    "relative"
                                                                ],
                                                                "title": "Tied-Array Beam",
                                                                "type": "object"
                                                            },
                                                            "title": "Tied-Array Beams",
                                                            "type": "array"
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "tabs"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "SAPs",
                                            "settings"
                                        ],
                                        "title": "Coherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "flys eye": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "description": "Produce beams containing the individual station signals",
                                        "properties": {
                                            "enabled": {
                                                "default": false,
                                                "title": "Enable Fly's Eye",
                                                "type": "boolean"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "enabled"
                                        ],
                                        "title": "Fly's Eye Settings",
                                        "type": "object"
                                    },
                                    "incoherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to create incoherent TABs for (empty list = all).",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "default": {},
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "subbands"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "settings",
                                            "SAPs"
                                        ],
                                        "title": "Incoherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "name": {
                                        "default": "_beamformer_name_",
                                        "description": "Beamformer name, used for identification purposes.",
                                        "minLength": 1,
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501"
                                                ]
                                            }
                                        ],
                                        "description": "While observing, COBALT will beamform on the intersection of all stations in this list and the used stations in the observation. So, specifying all possible stations here means that all observation-stations are used. Specifying a small subset here means that only the observing-stations in this small list are used. By default we let COBALT beamform on the Core stations.",
                                        "minItems": 1
                                    }
                                },
                                "required": [
                                    "name"
                                ],
                                "title": "Beamformer Pipeline",
                                "type": "object"
                            },
                            "subband_selection": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "list": {
                                        "additionalItems": false,
                                        "default": [],
                                        "description": "If method is 'select subset', only beamform these subbands, and only if they occur in the SAP.",
                                        "items": {
                                            "maximum": 511,
                                            "minimum": 0,
                                            "title": "Subband",
                                            "type": "integer"
                                        },
                                        "minItems": 0,
                                        "title": "Subset selection",
                                        "type": "array"
                                    },
                                    "method": {
                                        "default": "copy",
                                        "description": "How to select the subbands to beam form",
                                        "enum": [
                                            "copy",
                                            "largest continuous subset",
                                            "select subset"
                                        ],
                                        "title": "Method",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "method"
                                ],
                                "title": "Subband selection",
                                "type": "object"
                            },
                            "station_list": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [],
                                "items": {
                                    "$ref": "#/definitions/station"
                                },
                                "minItems": 0,
                                "type": "array",
                                "uniqueItems": true
                            },
                            "pointing": {
                                "additionalProperties": false,
                                "properties": {
                                    "angle1": {
                                        "default": 0.6624317181687094,
                                        "description": "First angle (e.g. RA)",
                                        "title": "Angle 1",
                                        "type": "number"
                                    },
                                    "angle2": {
                                        "default": 1.5579526427549426,
                                        "description": "Second angle (e.g. DEC)",
                                        "title": "Angle 2",
                                        "type": "number"
                                    },
                                    "direction_type": {
                                        "default": "J2000",
                                        "description": "",
                                        "enum": [
                                            "J2000",
                                            "AZELGEO",
                                            "LMN",
                                            "SUN",
                                            "MOON",
                                            "MERCURY",
                                            "VENUS",
                                            "MARS",
                                            "JUPITER",
                                            "SATURN",
                                            "URANUS",
                                            "NEPTUNE",
                                            "PLUTO"
                                        ],
                                        "title": "Reference frame",
                                        "type": "string"
                                    },
                                    "target": {
                                        "default": "_target_name_",
                                        "description": "Description of where this beam points at",
                                        "minLength": 1,
                                        "title": "Target",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "angle1",
                                    "angle2",
                                    "target"
                                ],
                                "type": "object"
                            },
                            "stokes_settings": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "channels_per_subband": {
                                        "default": 1,
                                        "description": "Number of frequency bands per subband",
                                        "enum": [
                                            1,
                                            8,
                                            16,
                                            32,
                                            64,
                                            128,
                                            256,
                                            512,
                                            1024
                                        ],
                                        "minimum": 1,
                                        "title": "Channels/subband",
                                        "type": "integer"
                                    },
                                    "quantisation": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "bits": {
                                                "default": 8,
                                                "description": "Number of bits for a single value",
                                                "enum": [
                                                    8
                                                ],
                                                "title": "Bits/sample",
                                                "type": "integer"
                                            },
                                            "enabled": {
                                                "default": false,
                                                "title": "Output quantisation into integers",
                                                "type": "boolean"
                                            },
                                            "scale_max": {
                                                "default": 5,
                                                "description": "Cut off values above this treshold",
                                                "title": "Maximum value",
                                                "type": "number"
                                            },
                                            "scale_min": {
                                                "default": -5,
                                                "description": "Cut off values below this treshold",
                                                "title": "Minimum value",
                                                "type": "number"
                                            }
                                        },
                                        "required": [
                                            "enabled",
                                            "bits",
                                            "scale_min",
                                            "scale_max"
                                        ],
                                        "title": "Output quantisation settings",
                                        "type": "object"
                                    },
                                    "stokes": {
                                        "$ref": "#/definitions/stokes",
                                        "default": "I"
                                    },
                                    "subbands_per_file": {
                                        "default": 488,
                                        "description": "The maximum number of subbands to write in each output dataproduct.",
                                        "maximum": 488,
                                        "minimum": 1,
                                        "title": "Subbands per file",
                                        "type": "integer"
                                    },
                                    "time_integration_factor": {
                                        "default": 1,
                                        "description": "The number of samples to integrate over",
                                        "maximum": 12288,
                                        "minimum": 1,
                                        "title": "Time integration",
                                        "type": "integer"
                                    }
                                },
                                "required": [
                                    "stokes",
                                    "time_integration_factor",
                                    "channels_per_subband"
                                ],
                                "type": "object"
                            },
                            "station": {
                                "description": "These are the LOFAR stations",
                                "enum": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501",
                                    "RS106",
                                    "RS205",
                                    "RS208",
                                    "RS210",
                                    "RS305",
                                    "RS306",
                                    "RS307",
                                    "RS310",
                                    "RS406",
                                    "RS407",
                                    "RS409",
                                    "RS503",
                                    "RS508",
                                    "RS509",
                                    "DE601",
                                    "DE602",
                                    "DE603",
                                    "DE604",
                                    "DE605",
                                    "FR606",
                                    "SE607",
                                    "UK608",
                                    "DE609",
                                    "PL610",
                                    "PL611",
                                    "PL612",
                                    "IE613",
                                    "LV614"
                                ],
                                "title": "Station",
                                "type": "string"
                            },
                            "max_number_of_missing_stations": {
                                "default": 0,
                                "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                                "minimum": 0,
                                "title": "Maximum number of stations to omit",
                                "type": "integer"
                            },
                            "stokes": {
                                "default": "I",
                                "description": "Which Stokes to produce",
                                "enum": [
                                    "I",
                                    "IQUV",
                                    "XXYY"
                                ],
                                "title": "Stokes",
                                "type": "string"
                            },
                            "station_groups": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [
                                    {
                                        "max_nr_missing": 1,
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ]
                                    }
                                ],
                                "description": "One or more predefined or custom groups of stations",
                                "items": {
                                    "$ref": "#/definitions/station_group"
                                },
                                "minItems": 1,
                                "title": "Station groups",
                                "type": "array"
                            },
                            "station_group": {
                                "anyOf": [
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007"
                                            ]
                                        },
                                        "description": "The group of all stations on the Superterp",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Superterp",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501"
                                            ]
                                        },
                                        "description": "The group of all Core stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Core",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch remote stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Remote",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch (Core + Remote) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Dutch",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 2,
                                            "stations": [
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 1,
                                            "stations": [
                                                "DE601",
                                                "DE605"
                                            ]
                                        },
                                        "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE605"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International required",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 6,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509",
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all (Core + Remote + International) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509",
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "All",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS001"
                                            ]
                                        },
                                        "description": "A custom group of stations which can be defined by the user",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list"
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Custom",
                                        "type": "object"
                                    }
                                ],
                                "default": {
                                    "max_nr_missing": 1,
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                },
                                "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                                "title": "Station group",
                                "type": "object"
                            },
                            "station_configuration": {
                                "default": {},
                                "properties": {
                                    "SAPs": {
                                        "$ref": "#/definitions/SAPs",
                                        "default": [
                                            {}
                                        ],
                                        "minItems": 1
                                    },
                                    "antenna_set": {
                                        "$ref": "#/definitions/antenna_set",
                                        "default": "HBA_DUAL"
                                    },
                                    "filter": {
                                        "$ref": "#/definitions/filter",
                                        "default": "HBA_110_190"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007"
                                                ]
                                            }
                                        ]
                                    },
                                    "tile_beam": {
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "description": "HBA only",
                                        "title": "Tile beam"
                                    }
                                },
                                "required": [
                                    "station_groups",
                                    "antenna_set",
                                    "filter",
                                    "SAPs"
                                ],
                                "title": "station_configuration",
                                "type": "object"
                            },
                            "filter": {
                                "default": "HBA_110_190",
                                "description": "Must match antenna type",
                                "enum": [
                                    "LBA_10_70",
                                    "LBA_30_70",
                                    "LBA_10_90",
                                    "LBA_30_90",
                                    "HBA_110_190",
                                    "HBA_210_250"
                                ],
                                "title": "Band-pass filter",
                                "type": "string"
                            },
                            "antenna_set": {
                                "default": "HBA_DUAL",
                                "description": "Fields & antennas to use",
                                "enum": [
                                    "HBA_DUAL",
                                    "HBA_DUAL_INNER",
                                    "HBA_ONE",
                                    "HBA_ONE_INNER",
                                    "HBA_ZERO",
                                    "HBA_ZERO_INNER",
                                    "LBA_INNER",
                                    "LBA_OUTER",
                                    "LBA_SPARSE_EVEN",
                                    "LBA_SPARSE_ODD",
                                    "LBA_ALL"
                                ],
                                "title": "Antenna set",
                                "type": "string"
                            },
                            "SAPs": {
                                "additionalItems": false,
                                "default": [
                                    {}
                                ],
                                "description": "Station beams",
                                "items": {
                                    "additionalProperties": false,
                                    "default": {},
                                    "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                    "properties": {
                                        "digital_pointing": {
                                            "$id": "#target_pointing",
                                            "$ref": "#/definitions/pointing",
                                            "default": {},
                                            "title": "Digital pointing"
                                        },
                                        "name": {
                                            "default": "_SAP_name_",
                                            "description": "Identifier for this beam",
                                            "minLength": 1,
                                            "title": "Name",
                                            "type": "string"
                                        },
                                        "subbands": {
                                            "additionalItems": false,
                                            "default": [
                                                255
                                            ],
                                            "items": {
                                                "maximum": 511,
                                                "minimum": 0,
                                                "title": "Subband",
                                                "type": "integer"
                                            },
                                            "maxItems": 488,
                                            "minItems": 1,
                                            "title": "Subband list",
                                            "type": "array"
                                        }
                                    },
                                    "required": [
                                        "name",
                                        "digital_pointing",
                                        "subbands"
                                    ],
                                    "title": "SAP",
                                    "type": "object"
                                },
                                "minItems": 0,
                                "title": "SAPs",
                                "type": "array"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2#",
                        "type": "object",
                        "title": "beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "beamformer",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                "title": "Duration",
                                "default": 300,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/2#/definitions/beamformer",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/observation",
                    "type_value": "observation",
                    "updated_at": "2022-04-21T08:15:52.853846",
                    "version": 2
                },
                "specifications_template_id": 3,
                "status": "scheduled",
                "status_value": "scheduled",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000068"
                ],
                "subtasks_ids": [
                    2000068
                ],
                "tags": [],
                "task_type": "observation",
                "unschedulable_reason": "",
                "updated_at": "2022-04-21T08:16:31.148228"
            }
        ],
        "successors": [
            {
                "id": 51,
                "url": "http://localhost:3000/api/task_blueprint/51",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-04-21T08:16:31.257688",
                "description": "Cleanup all dataproducts from disk",
                "draft": "http://localhost:3000/api/task_draft/51",
                "draft_id": 51,
                "duration": 300,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Cleanup",
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "output_pinned": false,
                "process_start_time": "2022-04-22T02:56:06.166227",
                "process_stop_time": "2022-04-22T03:01:06.166227",
                "produced_by": [
                    "http://localhost:3000/api/task_relation_blueprint/67",
                    "http://localhost:3000/api/task_relation_blueprint/68",
                    "http://localhost:3000/api/task_relation_blueprint/69"
                ],
                "produced_by_ids": [
                    67,
                    68,
                    69
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduled_start_time": "2022-04-22T02:56:06.166227",
                "scheduled_stop_time": "2022-04-22T03:01:06.166227",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/9",
                "scheduling_unit_blueprint_id": 9,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2/ref_resolved"
                },
                "specifications_template": "http://localhost:3000/api/task_template/1",
                "specifications_template_id": 1,
                "status": "schedulable",
                "status_value": "schedulable",
                "subtasks": [
                    {
                        "id": 2000071,
                        "url": "http://localhost:3000/api/subtask/2000071",
                        "actual_on_sky_start_time": null,
                        "actual_on_sky_stop_time": null,
                        "actual_process_start_time": null,
                        "actual_process_stop_time": null,
                        "cluster": "http://localhost:3000/api/cluster/1",
                        "cluster_id": 1,
                        "cluster_name": "CEP4",
                        "created_at": "2022-04-21T08:16:32.064018",
                        "created_or_updated_by_user": null,
                        "created_or_updated_by_user_id": null,
                        "duration": 300,
                        "error_reason": null,
                        "global_identifier": "http://localhost:3000/api/sip_identifier/152",
                        "global_identifier_id": 152,
                        "global_parset_identifier": "http://localhost:3000/api/sip_identifier/153",
                        "global_parset_identifier_id": 153,
                        "inputs": [
                            "http://localhost:3000/api/subtask_input/93",
                            "http://localhost:3000/api/subtask_input/94",
                            "http://localhost:3000/api/subtask_input/95",
                            "http://localhost:3000/api/subtask_input/96"
                        ],
                        "inputs_ids": [
                            93,
                            94,
                            95,
                            96
                        ],
                        "obsolete_since": null,
                        "on_sky_duration": null,
                        "on_sky_start_time": null,
                        "on_sky_stop_time": null,
                        "outputs": [],
                        "outputs_ids": [],
                        "primary": true,
                        "process_start_time": "2022-04-22T02:56:06.166227",
                        "process_stop_time": "2022-04-22T03:01:06.166227",
                        "processing_feedback_doc": null,
                        "processing_feedback_template": null,
                        "processing_feedback_template_id": null,
                        "scheduled_start_time": "2022-04-22T02:56:06.166227",
                        "scheduled_stop_time": "2022-04-22T03:01:06.166227",
                        "specifications_doc": {
                            "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/cleanup/2/ref_resolved"
                        },
                        "specifications_template": "http://localhost:3000/api/subtask_template/1",
                        "specifications_template_id": 1,
                        "state": "http://localhost:3000/api/subtask_state/defined",
                        "state_value": "defined",
                        "subtask_type": "cleanup",
                        "tags": [],
                        "task_blueprint": "http://localhost:3000/api/task_blueprint/51",
                        "task_blueprint_id": 51,
                        "total_nr_of_dataproducts": 0,
                        "total_size_of_dataproducts": 0,
                        "total_size_of_dataproducts_on_filesysem": 0,
                        "unschedulable_reason": null,
                        "updated_at": "2022-04-21T08:16:50.343241"
                    }
                ],
                "subtasks_ids": [
                    2000071
                ],
                "tags": [],
                "task_type": "cleanup",
                "unschedulable_reason": "",
                "updated_at": "2022-04-21T08:16:31.257700"
            },
            {
                "id": 52,
                "url": "http://localhost:3000/api/task_blueprint/52",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-04-21T08:16:31.327526",
                "description": "Ingest the pipeline outputs dataproducts",
                "draft": "http://localhost:3000/api/task_draft/52",
                "draft_id": 52,
                "duration": 300,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Ingest",
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "output_pinned": false,
                "process_start_time": "2022-04-22T02:56:06.166227",
                "process_stop_time": "2022-04-22T03:01:06.166227",
                "produced_by": [
                    "http://localhost:3000/api/task_relation_blueprint/70",
                    "http://localhost:3000/api/task_relation_blueprint/71"
                ],
                "produced_by_ids": [
                    70,
                    71
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduled_start_time": "2022-04-22T02:56:06.166227",
                "scheduled_stop_time": "2022-04-22T03:01:06.166227",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/9",
                "scheduling_unit_blueprint_id": 9,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
                },
                "specifications_template": "http://localhost:3000/api/task_template/2",
                "specifications_template_id": 2,
                "status": "schedulable",
                "status_value": "schedulable",
                "subtasks": [
                    {
                        "id": 2000070,
                        "url": "http://localhost:3000/api/subtask/2000070",
                        "actual_on_sky_start_time": null,
                        "actual_on_sky_stop_time": null,
                        "actual_process_start_time": null,
                        "actual_process_stop_time": null,
                        "cluster": "http://localhost:3000/api/cluster/1",
                        "cluster_id": 1,
                        "cluster_name": "CEP4",
                        "created_at": "2022-04-21T08:16:31.933431",
                        "created_or_updated_by_user": null,
                        "created_or_updated_by_user_id": null,
                        "duration": 300,
                        "error_reason": null,
                        "global_identifier": "http://localhost:3000/api/sip_identifier/150",
                        "global_identifier_id": 150,
                        "global_parset_identifier": "http://localhost:3000/api/sip_identifier/151",
                        "global_parset_identifier_id": 151,
                        "inputs": [
                            "http://localhost:3000/api/subtask_input/91",
                            "http://localhost:3000/api/subtask_input/92"
                        ],
                        "inputs_ids": [
                            91,
                            92
                        ],
                        "obsolete_since": null,
                        "on_sky_duration": null,
                        "on_sky_start_time": null,
                        "on_sky_stop_time": null,
                        "outputs": [],
                        "outputs_ids": [],
                        "primary": true,
                        "process_start_time": "2022-04-22T02:56:06.166227",
                        "process_stop_time": "2022-04-22T03:01:06.166227",
                        "processing_feedback_doc": null,
                        "processing_feedback_template": null,
                        "processing_feedback_template_id": null,
                        "scheduled_start_time": "2022-04-22T02:56:06.166227",
                        "scheduled_stop_time": "2022-04-22T03:01:06.166227",
                        "specifications_doc": {
                            "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/ingest%20control/2/ref_resolved"
                        },
                        "specifications_template": "http://localhost:3000/api/subtask_template/2",
                        "specifications_template_id": 2,
                        "state": "http://localhost:3000/api/subtask_state/defined",
                        "state_value": "defined",
                        "subtask_type": "ingest",
                        "tags": [],
                        "task_blueprint": "http://localhost:3000/api/task_blueprint/52",
                        "task_blueprint_id": 52,
                        "total_nr_of_dataproducts": 0,
                        "total_size_of_dataproducts": 0,
                        "total_size_of_dataproducts_on_filesysem": 0,
                        "unschedulable_reason": null,
                        "updated_at": "2022-04-21T08:16:50.355126"
                    }
                ],
                "subtasks_ids": [
                    2000070
                ],
                "tags": [],
                "task_type": "ingest",
                "unschedulable_reason": "",
                "updated_at": "2022-04-21T08:16:31.327539"
            }
        ]
    },
    {
        "id": 53,
        "url": "http://localhost:3000/api/task_blueprint/53",
        "actual_on_sky_start_time": null,
        "actual_on_sky_stop_time": null,
        "actual_process_start_time": null,
        "actual_process_stop_time": null,
        "consumed_by": [],
        "consumed_by_ids": [],
        "created_at": "2022-04-21T08:16:32.722465",
        "description": "A simple short test beamforming observation",
        "draft": "http://localhost:3000/api/task_draft/53",
        "draft_id": 53,
        "draftObject": {
            "id": 53,
            "url": "http://localhost:3000/api/task_draft/53",
            "connector_types": [
                {
                    "id": 6,
                    "url": "http://localhost:3000/api/task_connector_type/6",
                    "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                    "dataformat_value": "Beamformed",
                    "datatype": "http://localhost:3000/api/datatype/time%20series",
                    "datatype_value": "time series",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/beamformer",
                    "role_value": "beamformer",
                    "task_template": "http://localhost:3000/api/task_template/3",
                    "task_template_id": 3
                }
            ],
            "consumed_by": [],
            "consumed_by_ids": [],
            "created_at": "2022-04-21T08:16:32.621923",
            "description": "A simple short test beamforming observation",
            "duration": 120,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Observation",
            "output_pinned": false,
            "produced_by": [],
            "produced_by_ids": [],
            "relative_start_time": 0,
            "relative_stop_time": 120,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/10",
            "scheduling_unit_draft_id": 10,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "",
            "specifications_doc": {
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
                "duration": 120,
                "beamformer": {
                    "ppf": false,
                    "pipelines": [
                        {
                            "name": "B0329+54",
                            "coherent": {
                                "SAPs": [
                                    {
                                        "name": "B0329+54",
                                        "tabs": [
                                            {
                                                "duration": 0,
                                                "pointing": {
                                                    "angle1": 0.8115781021773631,
                                                    "angle2": 0.952579228492,
                                                    "target": "target1",
                                                    "direction_type": "J2000"
                                                },
                                                "relative": false,
                                                "allow_after_previous": true
                                            }
                                        ],
                                        "subbands": {
                                            "list": [
                                                0,
                                                1,
                                                2,
                                                3,
                                                4,
                                                5,
                                                6,
                                                7,
                                                8,
                                                9,
                                                10,
                                                11,
                                                12,
                                                13,
                                                14,
                                                15,
                                                16,
                                                17,
                                                18,
                                                19,
                                                20,
                                                21,
                                                22,
                                                23,
                                                24,
                                                25,
                                                26,
                                                27,
                                                28,
                                                29,
                                                30,
                                                31,
                                                32,
                                                33,
                                                34,
                                                35,
                                                36,
                                                37,
                                                38,
                                                39,
                                                40,
                                                41,
                                                42,
                                                43,
                                                44,
                                                45,
                                                46,
                                                47,
                                                48,
                                                49,
                                                50,
                                                51,
                                                52,
                                                53,
                                                54,
                                                55,
                                                56,
                                                57,
                                                58,
                                                59,
                                                60,
                                                61,
                                                62,
                                                63,
                                                64,
                                                65,
                                                66,
                                                67,
                                                68,
                                                69,
                                                70,
                                                71,
                                                72,
                                                73,
                                                74,
                                                75,
                                                76,
                                                77,
                                                78,
                                                79,
                                                80,
                                                81,
                                                82,
                                                83,
                                                84,
                                                85,
                                                86,
                                                87,
                                                88,
                                                89,
                                                90,
                                                91,
                                                92,
                                                93,
                                                94,
                                                95,
                                                96,
                                                97,
                                                98,
                                                99,
                                                100,
                                                101,
                                                102,
                                                103,
                                                104,
                                                105,
                                                106,
                                                107,
                                                108,
                                                109,
                                                110,
                                                111,
                                                112,
                                                113,
                                                114,
                                                115,
                                                116,
                                                117,
                                                118,
                                                119,
                                                120,
                                                121,
                                                122,
                                                123,
                                                124,
                                                125,
                                                126,
                                                127,
                                                128,
                                                129,
                                                130,
                                                131,
                                                132,
                                                133,
                                                134,
                                                135,
                                                136,
                                                137,
                                                138,
                                                139,
                                                140,
                                                141,
                                                142,
                                                143,
                                                144,
                                                145,
                                                146,
                                                147,
                                                148,
                                                149,
                                                150,
                                                151,
                                                152,
                                                153,
                                                154,
                                                155,
                                                156,
                                                157,
                                                158,
                                                159,
                                                160,
                                                161,
                                                162,
                                                163,
                                                164,
                                                165,
                                                166,
                                                167,
                                                168,
                                                169,
                                                170,
                                                171,
                                                172,
                                                173,
                                                174,
                                                175,
                                                176,
                                                177,
                                                178,
                                                179,
                                                180,
                                                181,
                                                182,
                                                183,
                                                184,
                                                185,
                                                186,
                                                187,
                                                188,
                                                189,
                                                190,
                                                191,
                                                192,
                                                193,
                                                194,
                                                195,
                                                196,
                                                197,
                                                198,
                                                199,
                                                200,
                                                201,
                                                202,
                                                203,
                                                204,
                                                205,
                                                206,
                                                207,
                                                208,
                                                209,
                                                210,
                                                211,
                                                212,
                                                213,
                                                214,
                                                215,
                                                216,
                                                217,
                                                218,
                                                219,
                                                220,
                                                221,
                                                222,
                                                223,
                                                224,
                                                225,
                                                226,
                                                227,
                                                228,
                                                229,
                                                230,
                                                231,
                                                232,
                                                233,
                                                234,
                                                235,
                                                236,
                                                237,
                                                238,
                                                239,
                                                240,
                                                241,
                                                242,
                                                243
                                            ],
                                            "method": "copy"
                                        },
                                        "tab_rings": {
                                            "count": 0,
                                            "width": 0.01
                                        }
                                    }
                                ],
                                "settings": {
                                    "stokes": "I",
                                    "quantisation": {
                                        "bits": 8,
                                        "enabled": false,
                                        "scale_max": 5,
                                        "scale_min": -5
                                    },
                                    "subbands_per_file": 488,
                                    "channels_per_subband": 1,
                                    "time_integration_factor": 1
                                }
                            },
                            "flys eye": {
                                "enabled": false,
                                "settings": {
                                    "stokes": "I",
                                    "quantisation": {
                                        "bits": 8,
                                        "enabled": false,
                                        "scale_max": 5,
                                        "scale_min": -5
                                    },
                                    "subbands_per_file": 488,
                                    "channels_per_subband": 1,
                                    "time_integration_factor": 1
                                }
                            },
                            "incoherent": {
                                "SAPs": [],
                                "settings": {
                                    "stokes": "I",
                                    "quantisation": {
                                        "bits": 8,
                                        "enabled": false,
                                        "scale_max": 5,
                                        "scale_min": -5
                                    },
                                    "subbands_per_file": 488,
                                    "channels_per_subband": 1,
                                    "time_integration_factor": 1
                                }
                            },
                            "station_groups": [
                                {
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ],
                                    "max_nr_missing": 1
                                }
                            ]
                        }
                    ]
                },
                "station_configuration": {
                    "SAPs": [
                        {
                            "name": "B0329+54",
                            "subbands": [
                                0,
                                1,
                                2,
                                3,
                                4,
                                5,
                                6,
                                7,
                                8,
                                9,
                                10,
                                11,
                                12,
                                13,
                                14,
                                15,
                                16,
                                17,
                                18,
                                19,
                                20,
                                21,
                                22,
                                23,
                                24,
                                25,
                                26,
                                27,
                                28,
                                29,
                                30,
                                31,
                                32,
                                33,
                                34,
                                35,
                                36,
                                37,
                                38,
                                39,
                                40,
                                41,
                                42,
                                43,
                                44,
                                45,
                                46,
                                47,
                                48,
                                49,
                                50,
                                51,
                                52,
                                53,
                                54,
                                55,
                                56,
                                57,
                                58,
                                59,
                                60,
                                61,
                                62,
                                63,
                                64,
                                65,
                                66,
                                67,
                                68,
                                69,
                                70,
                                71,
                                72,
                                73,
                                74,
                                75,
                                76,
                                77,
                                78,
                                79,
                                80,
                                81,
                                82,
                                83,
                                84,
                                85,
                                86,
                                87,
                                88,
                                89,
                                90,
                                91,
                                92,
                                93,
                                94,
                                95,
                                96,
                                97,
                                98,
                                99,
                                100,
                                101,
                                102,
                                103,
                                104,
                                105,
                                106,
                                107,
                                108,
                                109,
                                110,
                                111,
                                112,
                                113,
                                114,
                                115,
                                116,
                                117,
                                118,
                                119,
                                120,
                                121,
                                122,
                                123,
                                124,
                                125,
                                126,
                                127,
                                128,
                                129,
                                130,
                                131,
                                132,
                                133,
                                134,
                                135,
                                136,
                                137,
                                138,
                                139,
                                140,
                                141,
                                142,
                                143,
                                144,
                                145,
                                146,
                                147,
                                148,
                                149,
                                150,
                                151,
                                152,
                                153,
                                154,
                                155,
                                156,
                                157,
                                158,
                                159,
                                160,
                                161,
                                162,
                                163,
                                164,
                                165,
                                166,
                                167,
                                168,
                                169,
                                170,
                                171,
                                172,
                                173,
                                174,
                                175,
                                176,
                                177,
                                178,
                                179,
                                180,
                                181,
                                182,
                                183,
                                184,
                                185,
                                186,
                                187,
                                188,
                                189,
                                190,
                                191,
                                192,
                                193,
                                194,
                                195,
                                196,
                                197,
                                198,
                                199,
                                200,
                                201,
                                202,
                                203,
                                204,
                                205,
                                206,
                                207,
                                208,
                                209,
                                210,
                                211,
                                212,
                                213,
                                214,
                                215,
                                216,
                                217,
                                218,
                                219,
                                220,
                                221,
                                222,
                                223,
                                224,
                                225,
                                226,
                                227,
                                228,
                                229,
                                230,
                                231,
                                232,
                                233,
                                234,
                                235,
                                236,
                                237,
                                238,
                                239,
                                240,
                                241,
                                242,
                                243
                            ],
                            "digital_pointing": {
                                "angle1": 0.92934186635,
                                "angle2": 0.952579228492,
                                "target": "target1",
                                "direction_type": "J2000"
                            }
                        }
                    ],
                    "filter": "HBA_110_190",
                    "tile_beam": {
                        "angle1": 0.92934186635,
                        "angle2": 0.952579228492,
                        "target": "target1",
                        "direction_type": "J2000"
                    },
                    "antenna_set": "HBA_DUAL_INNER",
                    "station_groups": [
                        {
                            "stations": [
                                "CS002",
                                "CS003",
                                "CS004",
                                "CS005",
                                "CS006",
                                "CS007"
                            ],
                            "max_nr_missing": 1
                        }
                    ]
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/3",
            "specifications_template_id": 3,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/53"
            ],
            "task_blueprints_ids": [
                53
            ],
            "task_type": "observation",
            "updated_at": "2022-04-21T08:16:32.621939"
        },
        "duration": 120,
        "error_reason": "",
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Observation",
        "obsolete_since": null,
        "on_sky_duration": 120,
        "on_sky_start_time": "2022-04-22T03:06:06.166227",
        "on_sky_stop_time": "2022-04-22T03:08:06.166227",
        "output_pinned": false,
        "process_start_time": "2022-04-22T03:06:06.166227",
        "process_stop_time": "2022-04-22T03:08:06.166227",
        "produced_by": [],
        "produced_by_ids": [],
        "relative_start_time": 0,
        "relative_stop_time": 120,
        "scheduled_start_time": "2022-04-22T03:06:06.166227",
        "scheduled_stop_time": "2022-04-22T03:08:06.166227",
        "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/10",
        "scheduling_unit_blueprint_id": 10,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "",
        "specifications_doc": {
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
            "duration": 120,
            "beamformer": {
                "ppf": false,
                "pipelines": [
                    {
                        "name": "B0329+54",
                        "coherent": {
                            "SAPs": [
                                {
                                    "name": "B0329+54",
                                    "tabs": [
                                        {
                                            "duration": 0,
                                            "pointing": {
                                                "angle1": 0.8115781021773631,
                                                "angle2": 0.952579228492,
                                                "target": "target1",
                                                "direction_type": "J2000"
                                            },
                                            "relative": false,
                                            "allow_after_previous": true
                                        }
                                    ],
                                    "subbands": {
                                        "list": [
                                            0,
                                            1,
                                            2,
                                            3,
                                            4,
                                            5,
                                            6,
                                            7,
                                            8,
                                            9,
                                            10,
                                            11,
                                            12,
                                            13,
                                            14,
                                            15,
                                            16,
                                            17,
                                            18,
                                            19,
                                            20,
                                            21,
                                            22,
                                            23,
                                            24,
                                            25,
                                            26,
                                            27,
                                            28,
                                            29,
                                            30,
                                            31,
                                            32,
                                            33,
                                            34,
                                            35,
                                            36,
                                            37,
                                            38,
                                            39,
                                            40,
                                            41,
                                            42,
                                            43,
                                            44,
                                            45,
                                            46,
                                            47,
                                            48,
                                            49,
                                            50,
                                            51,
                                            52,
                                            53,
                                            54,
                                            55,
                                            56,
                                            57,
                                            58,
                                            59,
                                            60,
                                            61,
                                            62,
                                            63,
                                            64,
                                            65,
                                            66,
                                            67,
                                            68,
                                            69,
                                            70,
                                            71,
                                            72,
                                            73,
                                            74,
                                            75,
                                            76,
                                            77,
                                            78,
                                            79,
                                            80,
                                            81,
                                            82,
                                            83,
                                            84,
                                            85,
                                            86,
                                            87,
                                            88,
                                            89,
                                            90,
                                            91,
                                            92,
                                            93,
                                            94,
                                            95,
                                            96,
                                            97,
                                            98,
                                            99,
                                            100,
                                            101,
                                            102,
                                            103,
                                            104,
                                            105,
                                            106,
                                            107,
                                            108,
                                            109,
                                            110,
                                            111,
                                            112,
                                            113,
                                            114,
                                            115,
                                            116,
                                            117,
                                            118,
                                            119,
                                            120,
                                            121,
                                            122,
                                            123,
                                            124,
                                            125,
                                            126,
                                            127,
                                            128,
                                            129,
                                            130,
                                            131,
                                            132,
                                            133,
                                            134,
                                            135,
                                            136,
                                            137,
                                            138,
                                            139,
                                            140,
                                            141,
                                            142,
                                            143,
                                            144,
                                            145,
                                            146,
                                            147,
                                            148,
                                            149,
                                            150,
                                            151,
                                            152,
                                            153,
                                            154,
                                            155,
                                            156,
                                            157,
                                            158,
                                            159,
                                            160,
                                            161,
                                            162,
                                            163,
                                            164,
                                            165,
                                            166,
                                            167,
                                            168,
                                            169,
                                            170,
                                            171,
                                            172,
                                            173,
                                            174,
                                            175,
                                            176,
                                            177,
                                            178,
                                            179,
                                            180,
                                            181,
                                            182,
                                            183,
                                            184,
                                            185,
                                            186,
                                            187,
                                            188,
                                            189,
                                            190,
                                            191,
                                            192,
                                            193,
                                            194,
                                            195,
                                            196,
                                            197,
                                            198,
                                            199,
                                            200,
                                            201,
                                            202,
                                            203,
                                            204,
                                            205,
                                            206,
                                            207,
                                            208,
                                            209,
                                            210,
                                            211,
                                            212,
                                            213,
                                            214,
                                            215,
                                            216,
                                            217,
                                            218,
                                            219,
                                            220,
                                            221,
                                            222,
                                            223,
                                            224,
                                            225,
                                            226,
                                            227,
                                            228,
                                            229,
                                            230,
                                            231,
                                            232,
                                            233,
                                            234,
                                            235,
                                            236,
                                            237,
                                            238,
                                            239,
                                            240,
                                            241,
                                            242,
                                            243
                                        ],
                                        "method": "copy"
                                    },
                                    "tab_rings": {
                                        "count": 0,
                                        "width": 0.01
                                    }
                                }
                            ],
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "flys eye": {
                            "enabled": false,
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "incoherent": {
                            "SAPs": [],
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "station_groups": [
                            {
                                "stations": [
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                ]
            },
            "station_configuration": {
                "SAPs": [
                    {
                        "name": "B0329+54",
                        "subbands": [
                            0,
                            1,
                            2,
                            3,
                            4,
                            5,
                            6,
                            7,
                            8,
                            9,
                            10,
                            11,
                            12,
                            13,
                            14,
                            15,
                            16,
                            17,
                            18,
                            19,
                            20,
                            21,
                            22,
                            23,
                            24,
                            25,
                            26,
                            27,
                            28,
                            29,
                            30,
                            31,
                            32,
                            33,
                            34,
                            35,
                            36,
                            37,
                            38,
                            39,
                            40,
                            41,
                            42,
                            43,
                            44,
                            45,
                            46,
                            47,
                            48,
                            49,
                            50,
                            51,
                            52,
                            53,
                            54,
                            55,
                            56,
                            57,
                            58,
                            59,
                            60,
                            61,
                            62,
                            63,
                            64,
                            65,
                            66,
                            67,
                            68,
                            69,
                            70,
                            71,
                            72,
                            73,
                            74,
                            75,
                            76,
                            77,
                            78,
                            79,
                            80,
                            81,
                            82,
                            83,
                            84,
                            85,
                            86,
                            87,
                            88,
                            89,
                            90,
                            91,
                            92,
                            93,
                            94,
                            95,
                            96,
                            97,
                            98,
                            99,
                            100,
                            101,
                            102,
                            103,
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            137,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154,
                            155,
                            156,
                            157,
                            158,
                            159,
                            160,
                            161,
                            162,
                            163,
                            164,
                            165,
                            166,
                            167,
                            168,
                            169,
                            170,
                            171,
                            172,
                            173,
                            174,
                            175,
                            176,
                            177,
                            178,
                            179,
                            180,
                            181,
                            182,
                            183,
                            184,
                            185,
                            186,
                            187,
                            188,
                            189,
                            190,
                            191,
                            192,
                            193,
                            194,
                            195,
                            196,
                            197,
                            198,
                            199,
                            200,
                            201,
                            202,
                            203,
                            204,
                            205,
                            206,
                            207,
                            208,
                            209,
                            210,
                            211,
                            212,
                            213,
                            214,
                            215,
                            216,
                            217,
                            218,
                            219,
                            220,
                            221,
                            222,
                            223,
                            224,
                            225,
                            226,
                            227,
                            228,
                            229,
                            230,
                            231,
                            232,
                            233,
                            234,
                            235,
                            236,
                            237,
                            238,
                            239,
                            240,
                            241,
                            242,
                            243
                        ],
                        "digital_pointing": {
                            "angle1": 0.92934186635,
                            "angle2": 0.952579228492,
                            "target": "target1",
                            "direction_type": "J2000"
                        }
                    }
                ],
                "filter": "HBA_110_190",
                "tile_beam": {
                    "angle1": 0.92934186635,
                    "angle2": 0.952579228492,
                    "target": "target1",
                    "direction_type": "J2000"
                },
                "antenna_set": "HBA_DUAL_INNER",
                "station_groups": [
                    {
                        "stations": [
                            "CS002",
                            "CS003",
                            "CS004",
                            "CS005",
                            "CS006",
                            "CS007"
                        ],
                        "max_nr_missing": 1
                    }
                ]
            }
        },
        "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "beamforming observation"})).id,
        "specifications_template_id": (_.find(templates, {"name": "beamforming observation"})).id,
        "status": "scheduled",
        "status_value": "scheduled",
        "subtasks": [
            {
                "id": 2000072,
                "url": "http://localhost:3000/api/subtask/2000072",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "cluster": "http://localhost:3000/api/cluster/1",
                "cluster_id": 1,
                "cluster_name": "CEP4",
                "created_at": "2022-04-21T08:16:33.130571",
                "created_or_updated_by_user": null,
                "created_or_updated_by_user_id": null,
                "duration": 120,
                "error_reason": null,
                "global_identifier": "http://localhost:3000/api/sip_identifier/155",
                "global_identifier_id": 155,
                "global_parset_identifier": "http://localhost:3000/api/sip_identifier/156",
                "global_parset_identifier_id": 156,
                "inputs": [],
                "inputs_ids": [],
                "obsolete_since": null,
                "on_sky_duration": "00:02:00",
                "on_sky_start_time": "2022-04-22T03:06:06.166227",
                "on_sky_stop_time": "2022-04-22T03:08:06.166227",
                "outputs": [
                    "http://localhost:3000/api/subtask_output/63"
                ],
                "outputs_ids": [
                    63
                ],
                "primary": true,
                "process_start_time": "2022-04-22T03:06:06.166227",
                "process_stop_time": "2022-04-22T03:08:06.166227",
                "processing_feedback_doc": null,
                "processing_feedback_template": null,
                "processing_feedback_template_id": null,
                "scheduled_start_time": "2022-04-22T03:06:06.166227",
                "scheduled_stop_time": "2022-04-22T03:08:06.166227",
                "specifications_doc": {
                    "QA": {
                        "inspection_plots": "none"
                    },
                    "COBALT": {
                        "debug": {
                            "write_output": true,
                            "benchmark_file": "",
                            "annotate_broken_antennas": true
                        },
                        "version": 1,
                        "blocksize": 196608,
                        "resources": {
                            "writer_nodes": [
                                "cpu01.cep4.infiniband.lofar",
                                "cpu02.cep4.infiniband.lofar",
                                "cpu03.cep4.infiniband.lofar",
                                "cpu04.cep4.infiniband.lofar",
                                "cpu05.cep4.infiniband.lofar"
                            ],
                            "processing_nodes": [
                                "cbt201_0",
                                "cbt201_1",
                                "cbt202_0",
                                "cbt202_1",
                                "cbt203_0",
                                "cbt203_1",
                                "cbt204_0",
                                "cbt204_1",
                                "cbt205_0",
                                "cbt205_1",
                                "cbt207_0",
                                "cbt207_1",
                                "cbt208_0",
                                "cbt208_1",
                                "cbt209_0",
                                "cbt209_1",
                                "cbt210_0",
                                "cbt210_1",
                                "cbt211_0",
                                "cbt211_1",
                                "cbt212_0",
                                "cbt212_1"
                            ]
                        },
                        "beamformer": {
                            "input_ppf": false,
                            "tab_pipelines": [
                                {
                                    "SAPs": [
                                        {
                                            "name": "B0329+54",
                                            "tabs": [
                                                {
                                                    "coherent": true,
                                                    "pointings": [
                                                        {
                                                            "duration": 0,
                                                            "pointing": {
                                                                "angle1": 0.92934186635,
                                                                "angle2": 0.952579228492,
                                                                "target": "target1",
                                                                "direction_type": "J2000"
                                                            }
                                                        }
                                                    ]
                                                }
                                            ],
                                            "subbands": [
                                                0,
                                                1,
                                                2,
                                                3,
                                                4,
                                                5,
                                                6,
                                                7,
                                                8,
                                                9,
                                                10,
                                                11,
                                                12,
                                                13,
                                                14,
                                                15,
                                                16,
                                                17,
                                                18,
                                                19,
                                                20,
                                                21,
                                                22,
                                                23,
                                                24,
                                                25,
                                                26,
                                                27,
                                                28,
                                                29,
                                                30,
                                                31,
                                                32,
                                                33,
                                                34,
                                                35,
                                                36,
                                                37,
                                                38,
                                                39,
                                                40,
                                                41,
                                                42,
                                                43,
                                                44,
                                                45,
                                                46,
                                                47,
                                                48,
                                                49,
                                                50,
                                                51,
                                                52,
                                                53,
                                                54,
                                                55,
                                                56,
                                                57,
                                                58,
                                                59,
                                                60,
                                                61,
                                                62,
                                                63,
                                                64,
                                                65,
                                                66,
                                                67,
                                                68,
                                                69,
                                                70,
                                                71,
                                                72,
                                                73,
                                                74,
                                                75,
                                                76,
                                                77,
                                                78,
                                                79,
                                                80,
                                                81,
                                                82,
                                                83,
                                                84,
                                                85,
                                                86,
                                                87,
                                                88,
                                                89,
                                                90,
                                                91,
                                                92,
                                                93,
                                                94,
                                                95,
                                                96,
                                                97,
                                                98,
                                                99,
                                                100,
                                                101,
                                                102,
                                                103,
                                                104,
                                                105,
                                                106,
                                                107,
                                                108,
                                                109,
                                                110,
                                                111,
                                                112,
                                                113,
                                                114,
                                                115,
                                                116,
                                                117,
                                                118,
                                                119,
                                                120,
                                                121,
                                                122,
                                                123,
                                                124,
                                                125,
                                                126,
                                                127,
                                                128,
                                                129,
                                                130,
                                                131,
                                                132,
                                                133,
                                                134,
                                                135,
                                                136,
                                                137,
                                                138,
                                                139,
                                                140,
                                                141,
                                                142,
                                                143,
                                                144,
                                                145,
                                                146,
                                                147,
                                                148,
                                                149,
                                                150,
                                                151,
                                                152,
                                                153,
                                                154,
                                                155,
                                                156,
                                                157,
                                                158,
                                                159,
                                                160,
                                                161,
                                                162,
                                                163,
                                                164,
                                                165,
                                                166,
                                                167,
                                                168,
                                                169,
                                                170,
                                                171,
                                                172,
                                                173,
                                                174,
                                                175,
                                                176,
                                                177,
                                                178,
                                                179,
                                                180,
                                                181,
                                                182,
                                                183,
                                                184,
                                                185,
                                                186,
                                                187,
                                                188,
                                                189,
                                                190,
                                                191,
                                                192,
                                                193,
                                                194,
                                                195,
                                                196,
                                                197,
                                                198,
                                                199,
                                                200,
                                                201,
                                                202,
                                                203,
                                                204,
                                                205,
                                                206,
                                                207,
                                                208,
                                                209,
                                                210,
                                                211,
                                                212,
                                                213,
                                                214,
                                                215,
                                                216,
                                                217,
                                                218,
                                                219,
                                                220,
                                                221,
                                                222,
                                                223,
                                                224,
                                                225,
                                                226,
                                                227,
                                                228,
                                                229,
                                                230,
                                                231,
                                                232,
                                                233,
                                                234,
                                                235,
                                                236,
                                                237,
                                                238,
                                                239,
                                                240,
                                                241,
                                                242,
                                                243
                                            ]
                                        }
                                    ],
                                    "coherent": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    },
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                }
                            ],
                            "flyseye_pipelines": []
                        },
                        "correlator": {
                            "enabled": false,
                            "doppler_correction": false,
                            "channels_per_subband": 64,
                            "blocks_per_integration": 1,
                            "integrations_per_block": 1
                        },
                        "delay_compensation": true,
                        "bandpass_correction": true
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/observation%20control/2/ref_resolved",
                    "stations": {
                        "filter": "HBA_110_190",
                        "antenna_set": "HBA_DUAL_INNER",
                        "station_list": [
                            "CS002",
                            "CS003",
                            "CS004",
                            "CS005",
                            "CS006",
                            "CS007"
                        ],
                        "analog_pointing": {
                            "angle1": 0.92934186635,
                            "angle2": 0.952579228492,
                            "target": "target1",
                            "direction_type": "J2000"
                        },
                        "digital_pointings": [
                            {
                                "name": "B0329+54",
                                "pointing": {
                                    "angle1": 0.92934186635,
                                    "angle2": 0.952579228492,
                                    "target": "target1",
                                    "direction_type": "J2000"
                                },
                                "subbands": [
                                    0,
                                    1,
                                    2,
                                    3,
                                    4,
                                    5,
                                    6,
                                    7,
                                    8,
                                    9,
                                    10,
                                    11,
                                    12,
                                    13,
                                    14,
                                    15,
                                    16,
                                    17,
                                    18,
                                    19,
                                    20,
                                    21,
                                    22,
                                    23,
                                    24,
                                    25,
                                    26,
                                    27,
                                    28,
                                    29,
                                    30,
                                    31,
                                    32,
                                    33,
                                    34,
                                    35,
                                    36,
                                    37,
                                    38,
                                    39,
                                    40,
                                    41,
                                    42,
                                    43,
                                    44,
                                    45,
                                    46,
                                    47,
                                    48,
                                    49,
                                    50,
                                    51,
                                    52,
                                    53,
                                    54,
                                    55,
                                    56,
                                    57,
                                    58,
                                    59,
                                    60,
                                    61,
                                    62,
                                    63,
                                    64,
                                    65,
                                    66,
                                    67,
                                    68,
                                    69,
                                    70,
                                    71,
                                    72,
                                    73,
                                    74,
                                    75,
                                    76,
                                    77,
                                    78,
                                    79,
                                    80,
                                    81,
                                    82,
                                    83,
                                    84,
                                    85,
                                    86,
                                    87,
                                    88,
                                    89,
                                    90,
                                    91,
                                    92,
                                    93,
                                    94,
                                    95,
                                    96,
                                    97,
                                    98,
                                    99,
                                    100,
                                    101,
                                    102,
                                    103,
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    137,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    164,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    181,
                                    182,
                                    183,
                                    184,
                                    185,
                                    186,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    210,
                                    211,
                                    212,
                                    213,
                                    214,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    241,
                                    242,
                                    243
                                ]
                            }
                        ]
                    }
                },
                "specifications_template": "http://localhost:3000/api/subtask_template/4",
                "specifications_template_id": 4,
                "state": "http://localhost:3000/api/subtask_state/scheduled",
                "state_value": "scheduled",
                "subtask_type": "observation",
                "tags": [],
                "task_blueprint": "http://localhost:3000/api/task_blueprint/53",
                "task_blueprint_id": 53,
                "total_nr_of_dataproducts": 0,
                "total_size_of_dataproducts": 0,
                "total_size_of_dataproducts_on_filesysem": 0,
                "unschedulable_reason": null,
                "updated_at": "2022-04-21T08:16:49.410062"
            }
        ],
        "subtasks_ids": [
            2000072
        ],
        "tags": [],
        "task_type": "observation",
        "unschedulable_reason": "",
        "updated_at": "2022-04-21T08:16:32.722481",
        "predecessors": [],
        "successors": []
    },
    {
        "id": 68,
        "url": "http://localhost:3000/api/task_blueprint/68",
        "actual_on_sky_start_time": null,
        "actual_on_sky_stop_time": null,
        "actual_process_start_time": null,
        "actual_process_stop_time": null,
        "consumed_by": [],
        "consumed_by_ids": [],
        "created_at": "2022-04-26T11:43:13.243456",
        "description": "IM BF Cal&Tgt",
        "draft": "http://localhost:3000/api/task_draft/68",
        "draft_id": 68,
        "draftObject": {
            "id": 68,
            "url": "http://localhost:3000/api/task_draft/68",
            "connector_types": [
                {
                    "id": 8,
                    "url": "http://localhost:3000/api/task_connector_type/8",
                    "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                    "dataformat_value": "Beamformed",
                    "datatype": "http://localhost:3000/api/datatype/time%20series",
                    "datatype_value": "time series",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/beamformer",
                    "role_value": "beamformer",
                    "task_template": "http://localhost:3000/api/task_template/6",
                    "task_template_id": 6
                },
                {
                    "id": 5,
                    "url": "http://localhost:3000/api/task_connector_type/5",
                    "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:3000/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:3000/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:3000/api/role/correlator",
                    "role_value": "correlator",
                    "task_template": "http://localhost:3000/api/task_template/6",
                    "task_template_id": 6
                }
            ],
            "consumed_by": [],
            "consumed_by_ids": [],
            "created_at": "2022-04-22T07:25:37.890328",
            "description": "IM BF Cal&Tgt",
            "duration": 600,
            "first_scheduling_relation": [],
            "first_scheduling_relation_ids": [],
            "name": "Observation",
            "output_pinned": false,
            "produced_by": [],
            "produced_by_ids": [],
            "relative_start_time": 0,
            "relative_stop_time": 600,
            "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/15",
            "scheduling_unit_draft_id": 15,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "",
            "specifications_doc": {
                "QA": {
                    "plots": {
                        "enabled": true,
                        "autocorrelation": true,
                        "crosscorrelation": true
                    },
                    "file_conversion": {
                        "enabled": true,
                        "nr_of_subbands": -1,
                        "nr_of_timestamps": 256
                    },
                    "inspection_plots": "msplots"
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20and%20beamforming%20observation/2/ref_resolved",
                "duration": 600,
                "beamformer": {
                    "ppf": false,
                    "pipelines": [
                        {
                            "name": "_beamformer_name_",
                            "coherent": {
                                "SAPs": [],
                                "settings": {
                                    "stokes": "I",
                                    "quantisation": {
                                        "bits": 8,
                                        "enabled": false,
                                        "scale_max": 5,
                                        "scale_min": -5
                                    },
                                    "subbands_per_file": 488,
                                    "channels_per_subband": 1,
                                    "time_integration_factor": 1
                                }
                            },
                            "flys eye": {
                                "enabled": false,
                                "settings": {
                                    "stokes": "I",
                                    "quantisation": {
                                        "bits": 8,
                                        "enabled": false,
                                        "scale_max": 5,
                                        "scale_min": -5
                                    },
                                    "subbands_per_file": 488,
                                    "channels_per_subband": 1,
                                    "time_integration_factor": 1
                                }
                            },
                            "incoherent": {
                                "SAPs": [],
                                "settings": {
                                    "stokes": "I",
                                    "quantisation": {
                                        "bits": 8,
                                        "enabled": false,
                                        "scale_max": 5,
                                        "scale_min": -5
                                    },
                                    "subbands_per_file": 488,
                                    "channels_per_subband": 1,
                                    "time_integration_factor": 1
                                }
                            },
                            "station_groups": [
                                {
                                    "stations": [
                                        "CS001",
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007",
                                        "CS011",
                                        "CS013",
                                        "CS017",
                                        "CS021",
                                        "CS024",
                                        "CS026",
                                        "CS028",
                                        "CS030",
                                        "CS031",
                                        "CS032",
                                        "CS301",
                                        "CS302",
                                        "CS401",
                                        "CS501"
                                    ],
                                    "max_nr_missing": 1
                                }
                            ]
                        }
                    ]
                },
                "calibrator": {
                    "name": "calibrator",
                    "pointing": {
                        "angle1": 0.6624317181687094,
                        "angle2": 1.5579526427549426,
                        "target": "_target_name_",
                        "direction_type": "J2000"
                    },
                    "autoselect": true
                },
                "correlator": {
                    "storage_cluster": "CEP4",
                    "integration_time": 1,
                    "channels_per_subband": 64,
                    "topocentric_frequency_correction": false
                },
                "station_configuration": {
                    "SAPs": [
                        {
                            "name": "_SAP_name_",
                            "subbands": [
                                255
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "_target_name_",
                                "direction_type": "J2000"
                            }
                        }
                    ],
                    "filter": "HBA_110_190",
                    "tile_beam": {
                        "angle1": 0.6624317181687094,
                        "angle2": 1.5579526427549426,
                        "target": "_target_name_",
                        "direction_type": "J2000"
                    },
                    "antenna_set": "HBA_DUAL",
                    "station_groups": [
                        {
                            "stations": [
                                "CS002",
                                "CS003",
                                "CS004",
                                "CS005",
                                "CS006",
                                "CS007"
                            ],
                            "max_nr_missing": 1
                        }
                    ]
                }
            },
            "specifications_template": "http://localhost:3000/api/task_template/5",
            "specifications_template_id": 5,
            "tags": [],
            "task_blueprints": [
                "http://localhost:3000/api/task_blueprint/68"
            ],
            "task_blueprints_ids": [
                68
            ],
            "task_type": "observation",
            "updated_at": "2022-04-22T07:25:38.319179"
        },
        "duration": 1200,
        "error_reason": "",
        "first_scheduling_relation": [],
        "first_scheduling_relation_ids": [],
        "name": "Observation",
        "obsolete_since": null,
        "on_sky_duration": 600,
        "on_sky_start_time": "2022-04-22T07:18:00",
        "on_sky_stop_time": "2022-04-22T07:28:00",
        "output_pinned": false,
        "process_start_time": "2022-04-22T07:18:00",
        "process_stop_time": "2022-04-22T07:38:00",
        "produced_by": [],
        "produced_by_ids": [],
        "relative_start_time": 0,
        "relative_stop_time": 600,
        "scheduled_start_time": "2022-04-22T07:18:00",
        "scheduled_stop_time": "2022-04-22T07:38:00",
        "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/15",
        "scheduling_unit_blueprint_id": 15,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "",
        "specifications_doc": {
            "QA": {
                "plots": {
                    "enabled": true,
                    "autocorrelation": true,
                    "crosscorrelation": true
                },
                "file_conversion": {
                    "enabled": true,
                    "nr_of_subbands": -1,
                    "nr_of_timestamps": 256
                },
                "inspection_plots": "msplots"
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20and%20beamforming%20observation/2/ref_resolved",
            "duration": 600,
            "beamformer": {
                "ppf": false,
                "pipelines": [
                    {
                        "name": "_beamformer_name_",
                        "coherent": {
                            "SAPs": [],
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "flys eye": {
                            "enabled": false,
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "incoherent": {
                            "SAPs": [],
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "station_groups": [
                            {
                                "stations": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                ]
            },
            "calibrator": {
                "name": "calibrator",
                "pointing": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "_target_name_",
                    "direction_type": "J2000"
                },
                "autoselect": true
            },
            "correlator": {
                "storage_cluster": "CEP4",
                "integration_time": 1,
                "channels_per_subband": 64,
                "topocentric_frequency_correction": false
            },
            "station_configuration": {
                "SAPs": [
                    {
                        "name": "_SAP_name_",
                        "subbands": [
                            255
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "_target_name_",
                            "direction_type": "J2000"
                        }
                    }
                ],
                "filter": "HBA_110_190",
                "tile_beam": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "_target_name_",
                    "direction_type": "J2000"
                },
                "antenna_set": "HBA_DUAL",
                "station_groups": [
                    {
                        "stations": [
                            "CS002",
                            "CS003",
                            "CS004",
                            "CS005",
                            "CS006",
                            "CS007"
                        ],
                        "max_nr_missing": 1
                    }
                ]
            }
        },
        "specifications_template": "http://localhost:3000/api/task_template/"+(_.find(templates, {"name": "parallel calibrator target and beamforming observation"})).id,
        "specifications_template_id": (_.find(templates, {"name": "parallel calibrator target and beamforming observation"})).id,
        "status": "schedulable",
        "status_value": "schedulable",
        "subtasks": [
            {
                "id": 2000099,
                "url": "http://localhost:3000/api/subtask/2000099",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "cluster": "http://localhost:3000/api/cluster/1",
                "cluster_id": 1,
                "cluster_name": "CEP4",
                "created_at": "2022-04-26T11:43:13.521693",
                "created_or_updated_by_user": null,
                "created_or_updated_by_user_id": null,
                "duration": 600,
                "error_reason": null,
                "global_identifier": "http://localhost:3000/api/sip_identifier/8778",
                "global_identifier_id": 8778,
                "global_parset_identifier": "http://localhost:3000/api/sip_identifier/8779",
                "global_parset_identifier_id": 8779,
                "inputs": [],
                "inputs_ids": [],
                "obsolete_since": null,
                "on_sky_duration": "00:10:00",
                "on_sky_start_time": "2022-04-22T07:18:00",
                "on_sky_stop_time": "2022-04-22T07:28:00",
                "outputs": [
                    "http://localhost:3000/api/subtask_output/90",
                    "http://localhost:3000/api/subtask_output/89"
                ],
                "outputs_ids": [
                    90,
                    89
                ],
                "primary": true,
                "process_start_time": "2022-04-22T07:18:00",
                "process_stop_time": "2022-04-22T07:28:00",
                "processing_feedback_doc": null,
                "processing_feedback_template": null,
                "processing_feedback_template_id": null,
                "scheduled_start_time": "2022-04-22T07:18:00",
                "scheduled_stop_time": "2022-04-22T07:28:00",
                "specifications_doc": {
                    "QA": {
                        "inspection_plots": "msplots"
                    },
                    "COBALT": {
                        "debug": {
                            "write_output": true,
                            "benchmark_file": "",
                            "annotate_broken_antennas": true
                        },
                        "version": 1,
                        "blocksize": 195584,
                        "resources": {
                            "writer_nodes": [
                                "cpu01.cep4.infiniband.lofar",
                                "cpu02.cep4.infiniband.lofar",
                                "cpu03.cep4.infiniband.lofar",
                                "cpu04.cep4.infiniband.lofar",
                                "cpu05.cep4.infiniband.lofar"
                            ],
                            "processing_nodes": [
                                "cbt201_0",
                                "cbt201_1",
                                "cbt202_0",
                                "cbt202_1",
                                "cbt203_0",
                                "cbt203_1",
                                "cbt204_0",
                                "cbt204_1",
                                "cbt205_0",
                                "cbt205_1",
                                "cbt207_0",
                                "cbt207_1",
                                "cbt208_0",
                                "cbt208_1",
                                "cbt209_0",
                                "cbt209_1",
                                "cbt210_0",
                                "cbt210_1",
                                "cbt211_0",
                                "cbt211_1",
                                "cbt212_0",
                                "cbt212_1"
                            ]
                        },
                        "beamformer": {
                            "input_ppf": false,
                            "tab_pipelines": [],
                            "flyseye_pipelines": []
                        },
                        "correlator": {
                            "enabled": true,
                            "doppler_correction": false,
                            "channels_per_subband": 64,
                            "blocks_per_integration": 1,
                            "integrations_per_block": 1
                        },
                        "delay_compensation": true,
                        "bandpass_correction": true
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/observation%20control/2/ref_resolved",
                    "stations": {
                        "filter": "HBA_110_190",
                        "antenna_set": "HBA_DUAL",
                        "station_list": [
                            "CS002",
                            "CS003",
                            "CS004",
                            "CS005",
                            "CS006",
                            "CS007"
                        ],
                        "analog_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "_target_name_",
                            "direction_type": "J2000"
                        },
                        "digital_pointings": [
                            {
                                "name": "_SAP_name_",
                                "pointing": {
                                    "angle1": 0.6624317181687094,
                                    "angle2": 1.5579526427549426,
                                    "target": "_target_name_",
                                    "direction_type": "J2000"
                                },
                                "subbands": [
                                    255
                                ]
                            },
                            {
                                "name": "calibrator",
                                "pointing": {
                                    "angle1": 0.6624317181687094,
                                    "angle2": 1.5579526427549426,
                                    "target": "_target_name_",
                                    "direction_type": "J2000"
                                },
                                "subbands": [
                                    255
                                ]
                            }
                        ]
                    }
                },
                "specifications_template": "http://localhost:3000/api/subtask_template/4",
                "specifications_template_id": 4,
                "state": "http://localhost:3000/api/subtask_state/defined",
                "state_value": "defined",
                "subtask_type": "observation",
                "tags": [],
                "task_blueprint": "http://localhost:3000/api/task_blueprint/68",
                "task_blueprint_id": 68,
                "total_nr_of_dataproducts": 0,
                "total_size_of_dataproducts": 0,
                "total_size_of_dataproducts_on_filesysem": 0,
                "unschedulable_reason": null,
                "updated_at": "2022-04-26T11:43:13.785134"
            },
            {
                "id": 2000100,
                "url": "http://localhost:3000/api/subtask/2000100",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "cluster": "http://localhost:3000/api/cluster/1",
                "cluster_id": 1,
                "cluster_name": "CEP4",
                "created_at": "2022-04-26T11:43:13.633609",
                "created_or_updated_by_user": null,
                "created_or_updated_by_user_id": null,
                "duration": 300,
                "error_reason": null,
                "global_identifier": "http://localhost:3000/api/sip_identifier/8780",
                "global_identifier_id": 8780,
                "global_parset_identifier": "http://localhost:3000/api/sip_identifier/8781",
                "global_parset_identifier_id": 8781,
                "inputs": [
                    "http://localhost:3000/api/subtask_input/119"
                ],
                "inputs_ids": [
                    119
                ],
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "outputs": [
                    "http://localhost:3000/api/subtask_output/91"
                ],
                "outputs_ids": [
                    91
                ],
                "primary": false,
                "process_start_time": "2022-04-22T07:28:00",
                "process_stop_time": "2022-04-22T07:33:00",
                "processing_feedback_doc": null,
                "processing_feedback_template": null,
                "processing_feedback_template_id": null,
                "scheduled_start_time": "2022-04-22T07:28:00",
                "scheduled_stop_time": "2022-04-22T07:33:00",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/QA%20file%20conversion/2/ref_resolved",
                    "file_conversion": {
                        "enabled": true,
                        "nr_of_subbands": -1,
                        "nr_of_timestamps": 256
                    }
                },
                "specifications_template": "http://localhost:3000/api/subtask_template/5",
                "specifications_template_id": 5,
                "state": "http://localhost:3000/api/subtask_state/defined",
                "state_value": "defined",
                "subtask_type": "qa_files",
                "tags": [],
                "task_blueprint": "http://localhost:3000/api/task_blueprint/68",
                "task_blueprint_id": 68,
                "total_nr_of_dataproducts": 0,
                "total_size_of_dataproducts": 0,
                "total_size_of_dataproducts_on_filesysem": 0,
                "unschedulable_reason": null,
                "updated_at": "2022-04-26T11:43:13.799152"
            },
            {
                "id": 2000101,
                "url": "http://localhost:3000/api/subtask/2000101",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "cluster": "http://localhost:3000/api/cluster/1",
                "cluster_id": 1,
                "cluster_name": "CEP4",
                "created_at": "2022-04-26T11:43:13.730905",
                "created_or_updated_by_user": null,
                "created_or_updated_by_user_id": null,
                "duration": 300,
                "error_reason": null,
                "global_identifier": "http://localhost:3000/api/sip_identifier/8782",
                "global_identifier_id": 8782,
                "global_parset_identifier": "http://localhost:3000/api/sip_identifier/8783",
                "global_parset_identifier_id": 8783,
                "inputs": [
                    "http://localhost:3000/api/subtask_input/120"
                ],
                "inputs_ids": [
                    120
                ],
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "outputs": [
                    "http://localhost:3000/api/subtask_output/92"
                ],
                "outputs_ids": [
                    92
                ],
                "primary": false,
                "process_start_time": "2022-04-22T07:33:00",
                "process_stop_time": "2022-04-22T07:38:00",
                "processing_feedback_doc": null,
                "processing_feedback_template": null,
                "processing_feedback_template_id": null,
                "scheduled_start_time": "2022-04-22T07:33:00",
                "scheduled_stop_time": "2022-04-22T07:38:00",
                "specifications_doc": {
                    "plots": {
                        "enabled": true,
                        "autocorrelation": true,
                        "crosscorrelation": true
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/subtasktemplate/QA%20plots/2/ref_resolved"
                },
                "specifications_template": "http://localhost:3000/api/subtask_template/3",
                "specifications_template_id": 3,
                "state": "http://localhost:3000/api/subtask_state/defined",
                "state_value": "defined",
                "subtask_type": "qa_plots",
                "tags": [],
                "task_blueprint": "http://localhost:3000/api/task_blueprint/68",
                "task_blueprint_id": 68,
                "total_nr_of_dataproducts": 0,
                "total_size_of_dataproducts": 0,
                "total_size_of_dataproducts_on_filesysem": 0,
                "unschedulable_reason": null,
                "updated_at": "2022-04-26T11:43:13.812724"
            }
        ],
        "subtasks_ids": [
            2000099,
            2000100,
            2000101
        ],
        "tags": [],
        "task_type": "observation",
        "unschedulable_reason": "",
        "updated_at": "2022-04-26T11:43:13.243471",
        "predecessors": [],
        "successors": []
    },
    {
        "id": 69,
        "url": "http://localhost:8008/api/task_blueprint/69",
        "actual_on_sky_start_time": null,
        "actual_on_sky_stop_time": null,
        "actual_process_start_time": null,
        "actual_process_stop_time": null,
        "consumed_by": [
            "http://localhost:8008/api/task_relation_blueprint/1",
            "http://localhost:8008/api/task_relation_blueprint/2",
            "http://localhost:8008/api/task_relation_blueprint/5",
            "http://localhost:8008/api/task_relation_blueprint/12",
            "http://localhost:8008/api/task_relation_blueprint/13"
        ],
        "consumed_by_ids": [
            1,
            2,
            5,
            12,
            13
        ],
        "created_at": "2022-05-30T05:49:14.408788",
        "description": "IM+BF Observation",
        "draft": "http://localhost:8008/api/task_draft/69",
        "draft_id": 69,
        "draftObject": {
            "id": 69,
            "url": "http://localhost:8008/api/task_draft/69",
            "connector_types": [
                {
                    "id": 9,
                    "url": "http://localhost:8008/api/task_connector_type/9",
                    "dataformat": "http://localhost:8008/api/dataformat/Beamformed",
                    "dataformat_value": "Beamformed",
                    "datatype": "http://localhost:8008/api/datatype/time%20series",
                    "datatype_value": "time series",
                    "iotype": "http://localhost:8008/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:8008/api/role/beamformer",
                    "role_value": "beamformer",
                    "task_template": "http://localhost:8008/api/task_template/8",
                    "task_template_id": 8
                },
                {
                    "id": 5,
                    "url": "http://localhost:8008/api/task_connector_type/5",
                    "dataformat": "http://localhost:8008/api/dataformat/MeasurementSet",
                    "dataformat_value": "MeasurementSet",
                    "datatype": "http://localhost:8008/api/datatype/visibilities",
                    "datatype_value": "visibilities",
                    "iotype": "http://localhost:8008/api/iotype/output",
                    "iotype_value": "output",
                    "role": "http://localhost:8008/api/role/correlator",
                    "role_value": "correlator",
                    "task_template": "http://localhost:8008/api/task_template/8",
                    "task_template_id": 8
                }
            ],
            "consumed_by": [
                "http://localhost:8008/api/task_relation_draft/3",
                "http://localhost:8008/api/task_relation_draft/4",
                "http://localhost:8008/api/task_relation_draft/9",
                "http://localhost:8008/api/task_relation_draft/10",
                "http://localhost:8008/api/task_relation_draft/15"
            ],
            "consumed_by_ids": [
                3,
                4,
                9,
                10,
                15
            ],
            "created_at": "2022-05-26T05:21:21.738115",
            "description": "IM+BF Observation",
            "duration": 600.0,
            "first_scheduling_relation": [
                "http://localhost:8008/api/task_scheduling_relation_draft/4",
                "http://localhost:8008/api/task_scheduling_relation_draft/5",
                "http://localhost:8008/api/task_scheduling_relation_draft/6"
            ],
            "first_scheduling_relation_ids": [
                4,
                5,
                6
            ],
            "name": "IM+BF",
            "output_pinned": false,
            "produced_by": [],
            "produced_by_ids": [],
            "relative_start_time": 0.0,
            "relative_stop_time": 600.0,
            "scheduling_unit_draft": "http://localhost:8008/api/scheduling_unit_draft/16",
            "scheduling_unit_draft_id": 16,
            "second_scheduling_relation": [],
            "second_scheduling_relation_ids": [],
            "short_description": "IM+BF",
            "specifications_doc": {
                "QA": {
                    "plots": {
                        "enabled": false,
                        "autocorrelation": true,
                        "crosscorrelation": true
                    },
                    "file_conversion": {
                        "enabled": false,
                        "nr_of_subbands": -1,
                        "nr_of_timestamps": 256
                    },
                    "inspection_plots": "dynspec"
                },
                "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20and%20beamforming%20observation/2/ref_resolved",
                "duration": 600,
                "beamformer": {
                    "ppf": false,
                    "pipelines": [
                        {
                            "name": "MultiRings_Beamformer",
                            "coherent": {
                                "SAPs": [
                                    {
                                        "name": "IMBF_Sun",
                                        "tabs": [],
                                        "subbands": {
                                            "list": [
                                                410
                                            ],
                                            "method": "copy"
                                        },
                                        "tab_rings": {
                                            "count": 6,
                                            "width": 0.00174533
                                        }
                                    }
                                ],
                                "settings": {
                                    "stokes": "IQUV",
                                    "quantisation": {
                                        "bits": 8,
                                        "enabled": false,
                                        "scale_max": 5,
                                        "scale_min": -5
                                    },
                                    "subbands_per_file": 488,
                                    "channels_per_subband": 16,
                                    "time_integration_factor": 128
                                }
                            },
                            "flys eye": {
                                "enabled": false,
                                "settings": {
                                    "stokes": "I",
                                    "quantisation": {
                                        "bits": 8,
                                        "enabled": false,
                                        "scale_max": 5,
                                        "scale_min": -5
                                    },
                                    "subbands_per_file": 488,
                                    "channels_per_subband": 1,
                                    "time_integration_factor": 1
                                }
                            },
                            "incoherent": {
                                "SAPs": [],
                                "settings": {
                                    "stokes": "I",
                                    "quantisation": {
                                        "bits": 8,
                                        "enabled": false,
                                        "scale_max": 5,
                                        "scale_min": -5
                                    },
                                    "subbands_per_file": 488,
                                    "channels_per_subband": 1,
                                    "time_integration_factor": 1
                                }
                            },
                            "station_groups": [
                                {
                                    "stations": [
                                        "CS001",
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007",
                                        "CS011",
                                        "CS013",
                                        "CS017",
                                        "CS021",
                                        "CS024",
                                        "CS026",
                                        "CS028",
                                        "CS030",
                                        "CS031",
                                        "CS032",
                                        "CS301",
                                        "CS302",
                                        "CS401",
                                        "CS501"
                                    ],
                                    "max_nr_missing": 1
                                }
                            ]
                        }
                    ]
                },
                "calibrator": {
                    "name": "calibrator",
                    "pointing": {
                        "angle1": 0.6624317181687094,
                        "angle2": 1.5579526427549426,
                        "target": "_target_name_",
                        "direction_type": "J2000"
                    },
                    "autoselect": false
                },
                "correlator": {
                    "storage_cluster": "CEP4",
                    "integration_time": 0.25,
                    "channels_per_subband": 16,
                    "topocentric_frequency_correction": false
                },
                "station_configuration": {
                    "SAPs": [
                        {
                            "name": "IMBF_Sun",
                            "subbands": [
                                410
                            ],
                            "digital_pointing": {
                                "angle1": 0.6624317181687094,
                                "angle2": 1.5579526427549426,
                                "target": "_target_name_",
                                "direction_type": "J2000"
                            }
                        }
                    ],
                    "filter": "HBA_110_190",
                    "tile_beam": {
                        "angle1": 0.6624317181687094,
                        "angle2": 1.5579526427549426,
                        "target": "_target_name_",
                        "direction_type": "J2000"
                    },
                    "antenna_set": "HBA_DUAL",
                    "station_groups": [
                        {
                            "stations": [
                                "DE601",
                                "DE602",
                                "DE603",
                                "DE604",
                                "DE605",
                                "DE609",
                                "FR606",
                                "SE607",
                                "UK608",
                                "PL610",
                                "PL611",
                                "PL612",
                                "IE613",
                                "LV614"
                            ],
                            "max_nr_missing": 12
                        },
                        {
                            "stations": [
                                "RS310",
                                "RS509"
                            ],
                            "max_nr_missing": 2
                        }
                    ]
                }
            },
            "specifications_template": "http://localhost:8008/api/task_template/"+(_.find(templates, {"name": "parallel calibrator target and beamforming observation"})).id,
            "specifications_template_id": +(_.find(templates, {"name": "parallel calibrator target and beamforming observation"})).id,
            "tags": [],
            "task_blueprints": [],
            "task_blueprints_ids": [],
            "task_type": "observation",
            "updated_at": "2022-05-26T05:21:28.889045"
        },
        "duration": 600.0,
        "error_reason": "",
        "first_scheduling_relation": [
            "http://localhost:8008/api/task_scheduling_relation_blueprint/1",
            "http://localhost:8008/api/task_scheduling_relation_blueprint/2",
            "http://localhost:8008/api/task_scheduling_relation_blueprint/3"
        ],
        "first_scheduling_relation_ids": [
            1,
            2,
            3
        ],
        "name": "IM+BF",
        "obsolete_since": null,
        "on_sky_duration": 600.0,
        "on_sky_start_time": "2022-05-30T06:19:00",
        "on_sky_stop_time": "2022-05-30T06:29:00",
        "output_pinned": false,
        "process_start_time": "2022-05-30T06:19:00",
        "process_stop_time": "2022-05-30T06:29:00",
        "produced_by": [],
        "produced_by_ids": [],
        "relative_start_time": 0.0,
        "relative_stop_time": 600.0,
        "scheduled_start_time": "2022-05-30T06:19:00",
        "scheduled_stop_time": "2022-05-30T06:29:00",
        "scheduling_unit_blueprint": "http://localhost:8008/api/scheduling_unit_blueprint/1",
        "scheduling_unit_blueprint_id": 1,
        "second_scheduling_relation": [],
        "second_scheduling_relation_ids": [],
        "short_description": "IM+BF",
        "specifications_doc": {
            "QA": {
                "plots": {
                    "enabled": false,
                    "autocorrelation": true,
                    "crosscorrelation": true
                },
                "file_conversion": {
                    "enabled": false,
                    "nr_of_subbands": -1,
                    "nr_of_timestamps": 256
                },
                "inspection_plots": "dynspec"
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20and%20beamforming%20observation/2/ref_resolved",
            "duration": 600,
            "beamformer": {
                "ppf": false,
                "pipelines": [
                    {
                        "name": "MultiRings_Beamformer",
                        "coherent": {
                            "SAPs": [
                                {
                                    "name": "IMBF_Sun",
                                    "tabs": [],
                                    "subbands": {
                                        "list": [
                                            96,
                                            97,
                                            98,
                                            99,
                                            100,
                                            101,
                                            102,
                                            103,
                                            104,
                                            105,
                                            106,
                                            107,
                                            108,
                                            109,
                                            110,
                                            111,
                                            112,
                                            113,
                                            114,
                                            115,
                                            116,
                                            117,
                                            118,
                                            119,
                                            120,
                                            121,
                                            122,
                                            123,
                                            124,
                                            125,
                                            126,
                                            127,
                                            128,
                                            129,
                                            130,
                                            131,
                                            132,
                                            133,
                                            134,
                                            135,
                                            136,
                                            137,
                                            138,
                                            139,
                                            140,
                                            141,
                                            142,
                                            143,
                                            144,
                                            145,
                                            146,
                                            147,
                                            148,
                                            149,
                                            150,
                                            151,
                                            152,
                                            153,
                                            154,
                                            155,
                                            156,
                                            157,
                                            158,
                                            159,
                                            160,
                                            161,
                                            162,
                                            163,
                                            164,
                                            165,
                                            166,
                                            167,
                                            168,
                                            169,
                                            170,
                                            171,
                                            172,
                                            173,
                                            174,
                                            175,
                                            176,
                                            177,
                                            178,
                                            179,
                                            180,
                                            181,
                                            182,
                                            183,
                                            184,
                                            185,
                                            186,
                                            187,
                                            188,
                                            189,
                                            190,
                                            191,
                                            192,
                                            193,
                                            194,
                                            195,
                                            196,
                                            197,
                                            198,
                                            199,
                                            200,
                                            201,
                                            202,
                                            203,
                                            204,
                                            205,
                                            206,
                                            207,
                                            208,
                                            209,
                                            210,
                                            211,
                                            212,
                                            213,
                                            214,
                                            215,
                                            216,
                                            217,
                                            218,
                                            219,
                                            220,
                                            221,
                                            222,
                                            223,
                                            224,
                                            225,
                                            226,
                                            227,
                                            228,
                                            229,
                                            230,
                                            231,
                                            232,
                                            233,
                                            234,
                                            235,
                                            236,
                                            237,
                                            238,
                                            239,
                                            240,
                                            241,
                                            242,
                                            243,
                                            244,
                                            245,
                                            246,
                                            247,
                                            248,
                                            249,
                                            250,
                                            251,
                                            252,
                                            253,
                                            254,
                                            255,
                                            256,
                                            257,
                                            258,
                                            259,
                                            260,
                                            261,
                                            262,
                                            263,
                                            264,
                                            265,
                                            266,
                                            267,
                                            268,
                                            269,
                                            270,
                                            271,
                                            272,
                                            273,
                                            274,
                                            282,
                                            284,
                                            286,
                                            288,
                                            290,
                                            292,
                                            294,
                                            296,
                                            298,
                                            300,
                                            302,
                                            304,
                                            306,
                                            308,
                                            310,
                                            312,
                                            314,
                                            316,
                                            318,
                                            320,
                                            322,
                                            324,
                                            326,
                                            328,
                                            330,
                                            332,
                                            334,
                                            336,
                                            338,
                                            340,
                                            342,
                                            344,
                                            346,
                                            348,
                                            350,
                                            352,
                                            354,
                                            356,
                                            358,
                                            360,
                                            362,
                                            364,
                                            366,
                                            368,
                                            370,
                                            372,
                                            374,
                                            376,
                                            378,
                                            380,
                                            382,
                                            384,
                                            386,
                                            388,
                                            390,
                                            392,
                                            394,
                                            396,
                                            398,
                                            400,
                                            402,
                                            404,
                                            406,
                                            408,
                                            410
                                        ],
                                        "method": "copy"
                                    },
                                    "tab_rings": {
                                        "count": 6,
                                        "width": 0.00174533
                                    }
                                }
                            ],
                            "settings": {
                                "stokes": "IQUV",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 16,
                                "time_integration_factor": 128
                            }
                        },
                        "flys eye": {
                            "enabled": false,
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "incoherent": {
                            "SAPs": [],
                            "settings": {
                                "stokes": "I",
                                "quantisation": {
                                    "bits": 8,
                                    "enabled": false,
                                    "scale_max": 5,
                                    "scale_min": -5
                                },
                                "subbands_per_file": 488,
                                "channels_per_subband": 1,
                                "time_integration_factor": 1
                            }
                        },
                        "station_groups": [
                            {
                                "stations": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                ]
            },
            "calibrator": {
                "name": "calibrator",
                "pointing": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "_target_name_",
                    "direction_type": "J2000"
                },
                "autoselect": false
            },
            "correlator": {
                "storage_cluster": "CEP4",
                "integration_time": 0.25,
                "channels_per_subband": 16,
                "topocentric_frequency_correction": false
            },
            "station_configuration": {
                "SAPs": [
                    {
                        "name": "IMBF_Sun",
                        "subbands": [
                            96,
                            97,
                            98,
                            99,
                            100,
                            101,
                            102,
                            103,
                            104,
                            105,
                            106,
                            107,
                            108,
                            109,
                            110,
                            111,
                            112,
                            113,
                            114,
                            115,
                            116,
                            117,
                            118,
                            119,
                            120,
                            121,
                            122,
                            123,
                            124,
                            125,
                            126,
                            127,
                            128,
                            129,
                            130,
                            131,
                            132,
                            133,
                            134,
                            135,
                            136,
                            137,
                            138,
                            139,
                            140,
                            141,
                            142,
                            143,
                            144,
                            145,
                            146,
                            147,
                            148,
                            149,
                            150,
                            151,
                            152,
                            153,
                            154,
                            155,
                            156,
                            157,
                            158,
                            159,
                            160,
                            161,
                            162,
                            163,
                            164,
                            165,
                            166,
                            167,
                            168,
                            169,
                            170,
                            171,
                            172,
                            173,
                            174,
                            175,
                            176,
                            177,
                            178,
                            179,
                            180,
                            181,
                            182,
                            183,
                            184,
                            185,
                            186,
                            187,
                            188,
                            189,
                            190,
                            191,
                            192,
                            193,
                            194,
                            195,
                            196,
                            197,
                            198,
                            199,
                            200,
                            201,
                            202,
                            203,
                            204,
                            205,
                            206,
                            207,
                            208,
                            209,
                            210,
                            211,
                            212,
                            213,
                            214,
                            215,
                            216,
                            217,
                            218,
                            219,
                            220,
                            221,
                            222,
                            223,
                            224,
                            225,
                            226,
                            227,
                            228,
                            229,
                            230,
                            231,
                            232,
                            233,
                            234,
                            235,
                            236,
                            237,
                            238,
                            239,
                            240,
                            241,
                            242,
                            243,
                            244,
                            245,
                            246,
                            247,
                            248,
                            249,
                            250,
                            251,
                            252,
                            253,
                            254,
                            255,
                            256,
                            257,
                            258,
                            259,
                            260,
                            261,
                            262,
                            263,
                            264,
                            265,
                            266,
                            267,
                            268,
                            269,
                            270,
                            271,
                            272,
                            273,
                            274,
                            282,
                            284,
                            286,
                            288,
                            290,
                            292,
                            294,
                            296,
                            298,
                            300,
                            302,
                            304,
                            306,
                            308,
                            310,
                            312,
                            314,
                            316,
                            318,
                            320,
                            322,
                            324,
                            326,
                            328,
                            330,
                            332,
                            334,
                            336,
                            338,
                            340,
                            342,
                            344,
                            346,
                            348,
                            350,
                            352,
                            354,
                            356,
                            358,
                            360,
                            362,
                            364,
                            366,
                            368,
                            370,
                            372,
                            374,
                            376,
                            378,
                            380,
                            382,
                            384,
                            386,
                            388,
                            390,
                            392,
                            394,
                            396,
                            398,
                            400,
                            402,
                            404,
                            406,
                            408,
                            410
                        ],
                        "digital_pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "_target_name_",
                            "direction_type": "J2000"
                        }
                    }
                ],
                "filter": "HBA_110_190",
                "tile_beam": {
                    "angle1": 0.6624317181687094,
                    "angle2": 1.5579526427549426,
                    "target": "_target_name_",
                    "direction_type": "J2000"
                },
                "antenna_set": "HBA_DUAL",
                "station_groups": [
                    {
                        "stations": [
                            "DE601",
                            "DE602",
                            "DE603",
                            "DE604",
                            "DE605",
                            "DE609",
                            "FR606",
                            "SE607",
                            "UK608",
                            "PL610",
                            "PL611",
                            "PL612",
                            "IE613",
                            "LV614"
                        ],
                        "max_nr_missing": 12
                    },
                    {
                        "stations": [
                            "RS310",
                            "RS509"
                        ],
                        "max_nr_missing": 2
                    }
                ]
            }
        },
        "specifications_template": "http://localhost:8008/api/task_template/"+ (_.find(templates, {"name": "parallel calibrator target and beamforming observation"})).id,
        "specifications_template_id": (_.find(templates, {"name": "parallel calibrator target and beamforming observation"})).id,
        "status": "schedulable",
        "status_value": "schedulable",
        "subtasks": [
            "http://localhost:8008/api/subtask/2000004"
        ],
        "subtasks_ids": [
            2000004
        ],
        "tags": [],
        "task_type": "observation",
        "unschedulable_reason": "",
        "updated_at": "2022-05-30T05:49:14.408812"
    }
  ],
  schedulingUnitDrafts: [
    {
      "id": 1,
      "url": "http://localhost:3000/api/scheduling_unit_draft/1",
      "created_at": "2022-04-14T06:20:19.156270",
      "description": "Test scheduling unit",
      "duration": 120,
      "ingest_permission_required": true,
      "interrupts_telescope": false,
      "name": "IM LBA LoDSS - 5 Beams",
      "observation_strategy_template": "http://localhost:3000/api/scheduling_unit_observing_strategy_template/1",
      "observation_strategy_template_id": 1,
      "piggyback_allowed_aartfaac": true,
      "piggyback_allowed_tbb": true,
      "priority_queue": "http://localhost:3000/api/priority_queue_type/A",
      "priority_queue_value": "A",
      "rank": 1,
      "scheduling_constraints_doc": {
          "sky": {
              "min_distance": {
                  "sun": 0.5,
                  "moon": 0.5,
                  "jupiter": 0.5
              },
              "min_elevation": {
                  "target": 0.5,
                  "calibrator": 0.5
              },
              "transit_offset": {
                  "to": 7200,
                  "from": -7200
              }
          },
          "time": {
              "at": "2022-04-14T06:25:19.054619"
          },
          "daily": {
              "require_day": false,
              "require_night": false,
              "avoid_twilight": false
          },
          "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
          "scheduler": "fixed_time"
      },
      "scheduling_constraints_template": "http://localhost:3000/api/scheduling_constraints_template/1",
      "scheduling_constraints_template_id": 1,
      "scheduling_set": "http://localhost:3000/api/scheduling_set/1",
      "scheduling_set_id": 1,
      "scheduling_unit_blueprints": [
          "http://localhost:3000/api/scheduling_unit_blueprint/1"
      ],
      "scheduling_unit_blueprints_ids": [
          1
      ],
      "specifications_template": "http://localhost:3000/api/scheduling_unit_template/1",
      "specifications_template_id": 1,
      "tags": [],
      "task_drafts": [
          {
              "id": 1,
              "url": "http://localhost:3000/api/task_draft/1",
              "connector_types": [
                  {
                      "id": 3,
                      "url": "http://localhost:3000/api/task_connector_type/3",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/output",
                      "iotype_value": "output",
                      "role": "http://localhost:3000/api/role/correlator",
                      "role_value": "correlator",
                      "task_template": "http://localhost:3000/api/task_template/5",
                      "task_template_id": 5
                  }
              ],
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_draft/1",
                  "http://localhost:3000/api/task_relation_draft/2",
                  "http://localhost:3000/api/task_relation_draft/3",
                  "http://localhost:3000/api/task_relation_draft/4",
                  "http://localhost:3000/api/task_relation_draft/5",
                  "http://localhost:3000/api/task_relation_draft/6"
              ],
              "consumed_by_ids": [
                  1,
                  2,
                  3,
                  4,
                  5,
                  6
              ],
              "created_at": "2022-04-14T06:20:19.292483",
              "description": "Combined parallel Calibrator & Target Observation for LBA with 5 target beams",
              "duration": 120,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Combined Observation",
              "output_pinned": false,
              "produced_by": [],
              "produced_by_ids": [],
              "relative_start_time": 0,
              "relative_stop_time": 120,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
              "scheduling_unit_draft_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "oXXX Paaa+01 3Cabc",
              "specifications_doc": {
                  "QA": {
                      "plots": {
                          "enabled": true,
                          "autocorrelation": true,
                          "crosscorrelation": true
                      },
                      "file_conversion": {
                          "enabled": true,
                          "nr_of_subbands": -1,
                          "nr_of_timestamps": 256
                      },
                      "inspection_plots": "msplots"
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                  "duration": 120,
                  "calibrator": {
                      "name": "calibrator",
                      "pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "_calibrator_name_",
                          "direction_type": "J2000"
                      },
                      "autoselect": false
                  },
                  "correlator": {
                      "storage_cluster": "CEP4",
                      "integration_time": 1,
                      "channels_per_subband": 64,
                      "topocentric_frequency_correction": false
                  },
                  "station_configuration": {
                      "SAPs": [
                          {
                              "name": "target1",
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+01",
                                  "direction_type": "J2000"
                              }
                          },
                          {
                              "name": "target2",
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+02",
                                  "direction_type": "J2000"
                              }
                          },
                          {
                              "name": "target3",
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+03",
                                  "direction_type": "J2000"
                              }
                          },
                          {
                              "name": "target4",
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+04",
                                  "direction_type": "J2000"
                              }
                          },
                          {
                              "name": "target5",
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+05",
                                  "direction_type": "J2000"
                              }
                          }
                      ],
                      "filter": "LBA_10_90",
                      "tile_beam": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "_target_name_",
                          "direction_type": "J2000"
                      },
                      "antenna_set": "LBA_SPARSE_EVEN",
                      "station_groups": [
                          {
                              "stations": [
                                  "CS001",
                                  "CS002",
                                  "CS003",
                                  "CS004",
                                  "CS005",
                                  "CS006",
                                  "CS007",
                                  "CS011",
                                  "CS013",
                                  "CS017",
                                  "CS021",
                                  "CS024",
                                  "CS026",
                                  "CS028",
                                  "CS030",
                                  "CS031",
                                  "CS032",
                                  "CS101",
                                  "CS103",
                                  "CS201",
                                  "CS301",
                                  "CS302",
                                  "CS401",
                                  "CS501",
                                  "RS106",
                                  "RS205",
                                  "RS208",
                                  "RS210",
                                  "RS305",
                                  "RS306",
                                  "RS307",
                                  "RS310",
                                  "RS406",
                                  "RS407",
                                  "RS409",
                                  "RS503",
                                  "RS508",
                                  "RS509"
                              ],
                              "max_nr_missing": 4
                          },
                          {
                              "stations": [
                                  "RS508",
                                  "RS509"
                              ],
                              "max_nr_missing": 1
                          },
                          {
                              "stations": [
                                  "RS310",
                                  "RS210"
                              ],
                              "max_nr_missing": 0
                          }
                      ]
                  }
              },
              "specifications_template": {
                  "id": 5,
                  "url": "http://localhost:3000/api/task_template/5",
                  "created_at": "2022-04-14T06:20:07.273898",
                  "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                  "name": "parallel calibrator target observation",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                      "type": "object",
                      "title": "parallel calibrator target observation",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "calibrator",
                          "correlator",
                          "duration",
                          "station_configuration"
                      ],
                      "properties": {
                          "QA": {
                              "$ref": "#/definitions/QA",
                              "default": {}
                          },
                          "duration": {
                              "$id": "#duration",
                              "$ref": "#/definitions/timedelta",
                              "title": "Duration",
                              "default": 600,
                              "minimum": 1,
                              "description": "Duration of this observation (seconds)"
                          },
                          "calibrator": {
                              "$ref": "#/definitions/calibrator",
                              "default": {}
                          },
                          "correlator": {
                              "$ref": "#/definitions/correlator",
                              "default": {}
                          },
                          "station_configuration": {
                              "$ref": "#/definitions/station_configuration",
                              "default": {}
                          }
                      },
                      "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "QA": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Perform all Quality Assurance (QA) tasks, including file conversion and plotting.",
                              "properties": {
                                  "file_conversion": {
                                      "$ref": "#/definitions/file_conversion",
                                      "default": {}
                                  },
                                  "inspection_plots": {
                                      "$ref": "#/definitions/inspection_plots",
                                      "default": "msplots"
                                  },
                                  "plots": {
                                      "$ref": "#/definitions/plots",
                                      "default": {}
                                  }
                              },
                              "required": [
                                  "file_conversion",
                                  "plots",
                                  "inspection_plots"
                              ],
                              "title": "QA",
                              "type": "object"
                          },
                          "plots": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Create dynamic spectrum plots",
                              "properties": {
                                  "autocorrelation": {
                                      "default": true,
                                      "description": "Create autocorrelation plots for all stations",
                                      "title": "autocorrelation",
                                      "type": "boolean"
                                  },
                                  "crosscorrelation": {
                                      "default": true,
                                      "description": "Create crosscorrelation plots for all baselines",
                                      "title": "crosscorrelation",
                                      "type": "boolean"
                                  },
                                  "enabled": {
                                      "default": true,
                                      "description": "Do/Don't create plots from the QA file from the observation",
                                      "title": "enabled",
                                      "type": "boolean"
                                  }
                              },
                              "required": [
                                  "enabled",
                                  "autocorrelation",
                                  "crosscorrelation"
                              ],
                              "title": "Plots",
                              "type": "object"
                          },
                          "file_conversion": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Create a QA file for the observation",
                              "properties": {
                                  "enabled": {
                                      "default": true,
                                      "description": "Do/Don't create a QA file for the observation",
                                      "title": "enabled",
                                      "type": "boolean"
                                  },
                                  "nr_of_subbands": {
                                      "default": -1,
                                      "description": "Keep this number of subbands from the observation in the QA file, or all if -1",
                                      "title": "#subbands",
                                      "type": "integer"
                                  },
                                  "nr_of_timestamps": {
                                      "default": 256,
                                      "description": "Extract this number of timestamps from the observation in the QA file (equidistantanly sampled, no averaging/interpolation)",
                                      "minimum": 1,
                                      "title": "#timestamps",
                                      "type": "integer"
                                  }
                              },
                              "required": [
                                  "enabled",
                                  "nr_of_subbands",
                                  "nr_of_timestamps"
                              ],
                              "title": "File Conversion",
                              "type": "object"
                          },
                          "inspection_plots": {
                              "decription": "Type of inspection plots to run",
                              "default": "msplots",
                              "enum": [
                                  "msplots",
                                  "dynspec",
                                  "none"
                              ],
                              "title": "Inspection Plots",
                              "type": "string"
                          },
                          "timedelta": {
                              "default": 0,
                              "description": "A time duration or delta expressed in seconds",
                              "type": "number"
                          },
                          "timestamp": {
                              "default": "1970-01-01T00:00:00Z",
                              "description": "A timestamp defined in UTC",
                              "format": "date-time",
                              "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                              "type": "string"
                          },
                          "calibrator": {
                              "additionalProperties": false,
                              "default": {},
                              "properties": {
                                  "autoselect": {
                                      "default": true,
                                      "description": "Auto-select calibrator based on elevation",
                                      "title": "Auto-select",
                                      "type": "boolean"
                                  },
                                  "name": {
                                      "default": "calibrator",
                                      "description": "Name of the calibrator SAP",
                                      "title": "Name",
                                      "type": "string"
                                  },
                                  "pointing": {
                                      "$ref": "#/definitions/pointing",
                                      "default": {},
                                      "description": "Manually selected calibrator",
                                      "title": "Digital pointing"
                                  }
                              },
                              "required": [
                                  "autoselect",
                                  "pointing",
                                  "name"
                              ],
                              "type": "object"
                          },
                          "pointing": {
                              "additionalProperties": false,
                              "properties": {
                                  "angle1": {
                                      "default": 0.6624317181687094,
                                      "description": "First angle (e.g. RA)",
                                      "title": "Angle 1",
                                      "type": "number"
                                  },
                                  "angle2": {
                                      "default": 1.5579526427549426,
                                      "description": "Second angle (e.g. DEC)",
                                      "title": "Angle 2",
                                      "type": "number"
                                  },
                                  "direction_type": {
                                      "default": "J2000",
                                      "description": "",
                                      "enum": [
                                          "J2000",
                                          "AZELGEO",
                                          "LMN",
                                          "SUN",
                                          "MOON",
                                          "MERCURY",
                                          "VENUS",
                                          "MARS",
                                          "JUPITER",
                                          "SATURN",
                                          "URANUS",
                                          "NEPTUNE",
                                          "PLUTO"
                                      ],
                                      "title": "Reference frame",
                                      "type": "string"
                                  },
                                  "target": {
                                      "default": "_target_name_",
                                      "description": "Description of where this beam points at",
                                      "minLength": 1,
                                      "title": "Target",
                                      "type": "string"
                                  }
                              },
                              "required": [
                                  "angle1",
                                  "angle2",
                                  "target"
                              ],
                              "type": "object"
                          },
                          "correlator": {
                              "additionalProperties": false,
                              "default": {},
                              "properties": {
                                  "channels_per_subband": {
                                      "default": 64,
                                      "description": "Number of frequency bands per subband",
                                      "enum": [
                                          8,
                                          16,
                                          32,
                                          64,
                                          128,
                                          256,
                                          512,
                                          1024
                                      ],
                                      "minimum": 8,
                                      "title": "Channels/subband",
                                      "type": "integer"
                                  },
                                  "integration_time": {
                                      "$ref": "#/definitions/timedelta",
                                      "default": 1,
                                      "description": "Desired integration period (seconds)",
                                      "minimum": 0.1,
                                      "title": "Integration time"
                                  },
                                  "storage_cluster": {
                                      "default": "CEP4",
                                      "description": "Cluster to write output to",
                                      "enum": [
                                          "CEP4",
                                          "DragNet"
                                      ],
                                      "title": "Storage cluster",
                                      "type": "string"
                                  },
                                  "topocentric_frequency_correction": {
                                      "default": false,
                                      "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation.",
                                      "title": "Topocentric Frequency Correction",
                                      "type": "boolean"
                                  }
                              }
                          },
                          "station_configuration": {
                              "default": {},
                              "properties": {
                                  "SAPs": {
                                      "$ref": "#/definitions/SAPs",
                                      "default": [
                                          {}
                                      ],
                                      "minItems": 1
                                  },
                                  "antenna_set": {
                                      "$ref": "#/definitions/antenna_set",
                                      "default": "HBA_DUAL"
                                  },
                                  "filter": {
                                      "$ref": "#/definitions/filter",
                                      "default": "HBA_110_190"
                                  },
                                  "station_groups": {
                                      "$ref": "#/definitions/station_groups",
                                      "default": [
                                          {
                                              "max_nr_missing": 1,
                                              "stations": [
                                                  "CS002",
                                                  "CS003",
                                                  "CS004",
                                                  "CS005",
                                                  "CS006",
                                                  "CS007"
                                              ]
                                          }
                                      ]
                                  },
                                  "tile_beam": {
                                      "$ref": "#/definitions/pointing",
                                      "default": {},
                                      "description": "HBA only",
                                      "title": "Tile beam"
                                  }
                              },
                              "required": [
                                  "station_groups",
                                  "antenna_set",
                                  "filter",
                                  "SAPs"
                              ],
                              "title": "station_configuration",
                              "type": "object"
                          },
                          "station": {
                              "description": "These are the LOFAR stations",
                              "enum": [
                                  "CS001",
                                  "CS002",
                                  "CS003",
                                  "CS004",
                                  "CS005",
                                  "CS006",
                                  "CS007",
                                  "CS011",
                                  "CS013",
                                  "CS017",
                                  "CS021",
                                  "CS024",
                                  "CS026",
                                  "CS028",
                                  "CS030",
                                  "CS031",
                                  "CS032",
                                  "CS101",
                                  "CS103",
                                  "CS201",
                                  "CS301",
                                  "CS302",
                                  "CS401",
                                  "CS501",
                                  "RS106",
                                  "RS205",
                                  "RS208",
                                  "RS210",
                                  "RS305",
                                  "RS306",
                                  "RS307",
                                  "RS310",
                                  "RS406",
                                  "RS407",
                                  "RS409",
                                  "RS503",
                                  "RS508",
                                  "RS509",
                                  "DE601",
                                  "DE602",
                                  "DE603",
                                  "DE604",
                                  "DE605",
                                  "FR606",
                                  "SE607",
                                  "UK608",
                                  "DE609",
                                  "PL610",
                                  "PL611",
                                  "PL612",
                                  "IE613",
                                  "LV614"
                              ],
                              "title": "Station",
                              "type": "string"
                          },
                          "station_groups": {
                              "additionalItems": false,
                              "additionalProperties": false,
                              "default": [
                                  {
                                      "max_nr_missing": 1,
                                      "stations": [
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007"
                                      ]
                                  }
                              ],
                              "description": "One or more predefined or custom groups of stations",
                              "items": {
                                  "$ref": "#/definitions/station_group"
                              },
                              "minItems": 1,
                              "title": "Station groups",
                              "type": "array"
                          },
                          "max_number_of_missing_stations": {
                              "default": 0,
                              "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                              "minimum": 0,
                              "title": "Maximum number of stations to omit",
                              "type": "integer"
                          },
                          "SAPs": {
                              "additionalItems": false,
                              "default": [
                                  {}
                              ],
                              "description": "Station beams",
                              "items": {
                                  "additionalProperties": false,
                                  "default": {},
                                  "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                  "properties": {
                                      "digital_pointing": {
                                          "$id": "#target_pointing",
                                          "$ref": "#/definitions/pointing",
                                          "default": {},
                                          "title": "Digital pointing"
                                      },
                                      "name": {
                                          "default": "_SAP_name_",
                                          "description": "Identifier for this beam",
                                          "minLength": 1,
                                          "title": "Name",
                                          "type": "string"
                                      },
                                      "subbands": {
                                          "additionalItems": false,
                                          "default": [
                                              255
                                          ],
                                          "items": {
                                              "maximum": 511,
                                              "minimum": 0,
                                              "title": "Subband",
                                              "type": "integer"
                                          },
                                          "maxItems": 488,
                                          "minItems": 1,
                                          "title": "Subband list",
                                          "type": "array"
                                      }
                                  },
                                  "required": [
                                      "name",
                                      "digital_pointing",
                                      "subbands"
                                  ],
                                  "title": "SAP",
                                  "type": "object"
                              },
                              "minItems": 0,
                              "title": "SAPs",
                              "type": "array"
                          },
                          "antenna_set": {
                              "default": "HBA_DUAL",
                              "description": "Fields & antennas to use",
                              "enum": [
                                  "HBA_DUAL",
                                  "HBA_DUAL_INNER",
                                  "HBA_ONE",
                                  "HBA_ONE_INNER",
                                  "HBA_ZERO",
                                  "HBA_ZERO_INNER",
                                  "LBA_INNER",
                                  "LBA_OUTER",
                                  "LBA_SPARSE_EVEN",
                                  "LBA_SPARSE_ODD",
                                  "LBA_ALL"
                              ],
                              "title": "Antenna set",
                              "type": "string"
                          },
                          "filter": {
                              "default": "HBA_110_190",
                              "description": "Must match antenna type",
                              "enum": [
                                  "LBA_10_70",
                                  "LBA_30_70",
                                  "LBA_10_90",
                                  "LBA_30_90",
                                  "HBA_110_190",
                                  "HBA_210_250"
                              ],
                              "title": "Band-pass filter",
                              "type": "string"
                          },
                          "station_list": {
                              "additionalItems": false,
                              "additionalProperties": false,
                              "default": [],
                              "items": {
                                  "$ref": "#/definitions/station"
                              },
                              "minItems": 0,
                              "type": "array",
                              "uniqueItems": true
                          },
                          "station_group": {
                              "anyOf": [
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 0,
                                          "stations": [
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007"
                                          ]
                                      },
                                      "description": "The group of all stations on the Superterp",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Superterp",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 4,
                                          "stations": [
                                              "CS001",
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007",
                                              "CS011",
                                              "CS013",
                                              "CS017",
                                              "CS021",
                                              "CS024",
                                              "CS026",
                                              "CS028",
                                              "CS030",
                                              "CS031",
                                              "CS032",
                                              "CS101",
                                              "CS103",
                                              "CS201",
                                              "CS301",
                                              "CS302",
                                              "CS401",
                                              "CS501"
                                          ]
                                      },
                                      "description": "The group of all Core stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS001",
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007",
                                                      "CS011",
                                                      "CS013",
                                                      "CS017",
                                                      "CS021",
                                                      "CS024",
                                                      "CS026",
                                                      "CS028",
                                                      "CS030",
                                                      "CS031",
                                                      "CS032",
                                                      "CS101",
                                                      "CS103",
                                                      "CS201",
                                                      "CS301",
                                                      "CS302",
                                                      "CS401",
                                                      "CS501"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Core",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 4,
                                          "stations": [
                                              "RS106",
                                              "RS205",
                                              "RS208",
                                              "RS210",
                                              "RS305",
                                              "RS306",
                                              "RS307",
                                              "RS310",
                                              "RS406",
                                              "RS407",
                                              "RS409",
                                              "RS503",
                                              "RS508",
                                              "RS509"
                                          ]
                                      },
                                      "description": "The group of all Dutch remote stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "RS106",
                                                      "RS205",
                                                      "RS208",
                                                      "RS210",
                                                      "RS305",
                                                      "RS306",
                                                      "RS307",
                                                      "RS310",
                                                      "RS406",
                                                      "RS407",
                                                      "RS409",
                                                      "RS503",
                                                      "RS508",
                                                      "RS509"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Remote",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 4,
                                          "stations": [
                                              "CS001",
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007",
                                              "CS011",
                                              "CS013",
                                              "CS017",
                                              "CS021",
                                              "CS024",
                                              "CS026",
                                              "CS028",
                                              "CS030",
                                              "CS031",
                                              "CS032",
                                              "CS101",
                                              "CS103",
                                              "CS201",
                                              "CS301",
                                              "CS302",
                                              "CS401",
                                              "CS501",
                                              "RS106",
                                              "RS205",
                                              "RS208",
                                              "RS210",
                                              "RS305",
                                              "RS306",
                                              "RS307",
                                              "RS310",
                                              "RS406",
                                              "RS407",
                                              "RS409",
                                              "RS503",
                                              "RS508",
                                              "RS509"
                                          ]
                                      },
                                      "description": "The group of all Dutch (Core + Remote) stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS001",
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007",
                                                      "CS011",
                                                      "CS013",
                                                      "CS017",
                                                      "CS021",
                                                      "CS024",
                                                      "CS026",
                                                      "CS028",
                                                      "CS030",
                                                      "CS031",
                                                      "CS032",
                                                      "CS101",
                                                      "CS103",
                                                      "CS201",
                                                      "CS301",
                                                      "CS302",
                                                      "CS401",
                                                      "CS501",
                                                      "RS106",
                                                      "RS205",
                                                      "RS208",
                                                      "RS210",
                                                      "RS305",
                                                      "RS306",
                                                      "RS307",
                                                      "RS310",
                                                      "RS406",
                                                      "RS407",
                                                      "RS409",
                                                      "RS503",
                                                      "RS508",
                                                      "RS509"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Dutch",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 2,
                                          "stations": [
                                              "DE601",
                                              "DE602",
                                              "DE603",
                                              "DE604",
                                              "DE605",
                                              "DE609",
                                              "FR606",
                                              "SE607",
                                              "UK608",
                                              "PL610",
                                              "PL611",
                                              "PL612",
                                              "IE613",
                                              "LV614"
                                          ]
                                      },
                                      "description": "The group of all international stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "DE601",
                                                      "DE602",
                                                      "DE603",
                                                      "DE604",
                                                      "DE605",
                                                      "DE609",
                                                      "FR606",
                                                      "SE607",
                                                      "UK608",
                                                      "PL610",
                                                      "PL611",
                                                      "PL612",
                                                      "IE613",
                                                      "LV614"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "International",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 1,
                                          "stations": [
                                              "DE601",
                                              "DE605"
                                          ]
                                      },
                                      "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "DE601",
                                                      "DE605"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "International required",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 6,
                                          "stations": [
                                              "CS001",
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007",
                                              "CS011",
                                              "CS013",
                                              "CS017",
                                              "CS021",
                                              "CS024",
                                              "CS026",
                                              "CS028",
                                              "CS030",
                                              "CS031",
                                              "CS032",
                                              "CS101",
                                              "CS103",
                                              "CS201",
                                              "CS301",
                                              "CS302",
                                              "CS401",
                                              "CS501",
                                              "RS106",
                                              "RS205",
                                              "RS208",
                                              "RS210",
                                              "RS305",
                                              "RS306",
                                              "RS307",
                                              "RS310",
                                              "RS406",
                                              "RS407",
                                              "RS409",
                                              "RS503",
                                              "RS508",
                                              "RS509",
                                              "DE601",
                                              "DE602",
                                              "DE603",
                                              "DE604",
                                              "DE605",
                                              "DE609",
                                              "FR606",
                                              "SE607",
                                              "UK608",
                                              "PL610",
                                              "PL611",
                                              "PL612",
                                              "IE613",
                                              "LV614"
                                          ]
                                      },
                                      "description": "The group of all (Core + Remote + International) stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS001",
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007",
                                                      "CS011",
                                                      "CS013",
                                                      "CS017",
                                                      "CS021",
                                                      "CS024",
                                                      "CS026",
                                                      "CS028",
                                                      "CS030",
                                                      "CS031",
                                                      "CS032",
                                                      "CS101",
                                                      "CS103",
                                                      "CS201",
                                                      "CS301",
                                                      "CS302",
                                                      "CS401",
                                                      "CS501",
                                                      "RS106",
                                                      "RS205",
                                                      "RS208",
                                                      "RS210",
                                                      "RS305",
                                                      "RS306",
                                                      "RS307",
                                                      "RS310",
                                                      "RS406",
                                                      "RS407",
                                                      "RS409",
                                                      "RS503",
                                                      "RS508",
                                                      "RS509",
                                                      "DE601",
                                                      "DE602",
                                                      "DE603",
                                                      "DE604",
                                                      "DE605",
                                                      "DE609",
                                                      "FR606",
                                                      "SE607",
                                                      "UK608",
                                                      "PL610",
                                                      "PL611",
                                                      "PL612",
                                                      "IE613",
                                                      "LV614"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "All",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 0,
                                          "stations": [
                                              "CS001"
                                          ]
                                      },
                                      "description": "A custom group of stations which can be defined by the user",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list"
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Custom",
                                      "type": "object"
                                  }
                              ],
                              "default": {
                                  "max_nr_missing": 1,
                                  "stations": [
                                      "CS002",
                                      "CS003",
                                      "CS004",
                                      "CS005",
                                      "CS006",
                                      "CS007"
                                  ]
                              },
                              "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                              "title": "Station group",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2#",
                      "type": "object",
                      "title": "parallel calibrator target observation",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "calibrator",
                          "correlator",
                          "duration",
                          "station_configuration"
                      ],
                      "properties": {
                          "QA": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/2#/definitions/QA",
                              "default": {}
                          },
                          "duration": {
                              "$id": "#duration",
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                              "title": "Duration",
                              "default": 600,
                              "minimum": 1,
                              "description": "Duration of this observation (seconds)"
                          },
                          "calibrator": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/calibrator/1#/definitions/calibrator",
                              "default": {}
                          },
                          "correlator": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/correlator/1#/definitions/correlator",
                              "default": {}
                          },
                          "station_configuration": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                              "default": {}
                          }
                      },
                      "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/observation",
                  "type_value": "observation",
                  "updated_at": "2022-04-14T06:20:07.273910",
                  "version": 2
              },
              "specifications_template_id": 5,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/1"
              ],
              "task_blueprints_ids": [
                  1
              ],
              "task_type": "observation",
              "updated_at": "2022-04-14T06:20:19.292495"
          },
          {
              "id": 2,
              "url": "http://localhost:3000/api/task_draft/2",
              "connector_types": [
                  {
                      "id": 13,
                      "url": "http://localhost:3000/api/task_connector_type/13",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/output",
                      "iotype_value": "output",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  },
                  {
                      "id": 12,
                      "url": "http://localhost:3000/api/task_connector_type/12",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  }
              ],
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_draft/7"
              ],
              "consumed_by_ids": [
                  7
              ],
              "created_at": "2022-04-14T06:20:19.318002",
              "description": "Preprocessing Pipeline for Calibrator Observation",
              "duration": 0,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Calibrator Pipeline",
              "output_pinned": false,
              "produced_by": [
                  "http://localhost:3000/api/task_relation_draft/1"
              ],
              "produced_by_ids": [
                  1
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
              "scheduling_unit_draft_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "3Cabc/1.0/CP",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 10,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 1,
                      "frequency_steps": 1
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/2"
              ],
              "task_blueprints_ids": [
                  2
              ],
              "task_type": "pipeline",
              "updated_at": "2022-04-14T06:20:19.318066"
          },
          {
              "id": 3,
              "url": "http://localhost:3000/api/task_draft/3",
              "connector_types": [
                  {
                      "id": 13,
                      "url": "http://localhost:3000/api/task_connector_type/13",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/output",
                      "iotype_value": "output",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  },
                  {
                      "id": 12,
                      "url": "http://localhost:3000/api/task_connector_type/12",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  }
              ],
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_draft/12"
              ],
              "consumed_by_ids": [
                  12
              ],
              "created_at": "2022-04-14T06:20:19.325739",
              "description": "Preprocessing Pipeline for Target Observation target5",
              "duration": 0,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target5",
              "output_pinned": false,
              "produced_by": [
                  "http://localhost:3000/api/task_relation_draft/6"
              ],
              "produced_by_ids": [
                  6
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
              "scheduling_unit_draft_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Paaa+05/1.4/PP",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 10,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 1,
                      "frequency_steps": 1
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/3"
              ],
              "task_blueprints_ids": [
                  3
              ],
              "task_type": "pipeline",
              "updated_at": "2022-04-14T06:20:19.325751"
          },
          {
              "id": 4,
              "url": "http://localhost:3000/api/task_draft/4",
              "connector_types": [
                  {
                      "id": 13,
                      "url": "http://localhost:3000/api/task_connector_type/13",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/output",
                      "iotype_value": "output",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  },
                  {
                      "id": 12,
                      "url": "http://localhost:3000/api/task_connector_type/12",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  }
              ],
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_draft/11"
              ],
              "consumed_by_ids": [
                  11
              ],
              "created_at": "2022-04-14T06:20:19.334536",
              "description": "Preprocessing Pipeline for Target Observation target4",
              "duration": 0,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target4",
              "output_pinned": false,
              "produced_by": [
                  "http://localhost:3000/api/task_relation_draft/5"
              ],
              "produced_by_ids": [
                  5
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
              "scheduling_unit_draft_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Paaa+04/1.3/PP",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 10,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 1,
                      "frequency_steps": 1
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/4"
              ],
              "task_blueprints_ids": [
                  4
              ],
              "task_type": "pipeline",
              "updated_at": "2022-04-14T06:20:19.334549"
          },
          {
              "id": 5,
              "url": "http://localhost:3000/api/task_draft/5",
              "connector_types": [
                  {
                      "id": 13,
                      "url": "http://localhost:3000/api/task_connector_type/13",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/output",
                      "iotype_value": "output",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  },
                  {
                      "id": 12,
                      "url": "http://localhost:3000/api/task_connector_type/12",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  }
              ],
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_draft/10"
              ],
              "consumed_by_ids": [
                  10
              ],
              "created_at": "2022-04-14T06:20:19.341298",
              "description": "Preprocessing Pipeline for Target Observation target3",
              "duration": 0,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target3",
              "output_pinned": false,
              "produced_by": [
                  "http://localhost:3000/api/task_relation_draft/4"
              ],
              "produced_by_ids": [
                  4
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
              "scheduling_unit_draft_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Paaa+03/1.2/PP",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 10,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 1,
                      "frequency_steps": 1
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/5"
              ],
              "task_blueprints_ids": [
                  5
              ],
              "task_type": "pipeline",
              "updated_at": "2022-04-14T06:20:19.341309"
          },
          {
              "id": 6,
              "url": "http://localhost:3000/api/task_draft/6",
              "connector_types": [
                  {
                      "id": 13,
                      "url": "http://localhost:3000/api/task_connector_type/13",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/output",
                      "iotype_value": "output",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  },
                  {
                      "id": 12,
                      "url": "http://localhost:3000/api/task_connector_type/12",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  }
              ],
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_draft/9"
              ],
              "consumed_by_ids": [
                  9
              ],
              "created_at": "2022-04-14T06:20:19.349574",
              "description": "Preprocessing Pipeline for Target Observation target2",
              "duration": 0,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target2",
              "output_pinned": false,
              "produced_by": [
                  "http://localhost:3000/api/task_relation_draft/3"
              ],
              "produced_by_ids": [
                  3
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
              "scheduling_unit_draft_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Paaa+02/1.1/PP",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 10,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 1,
                      "frequency_steps": 1
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/6"
              ],
              "task_blueprints_ids": [
                  6
              ],
              "task_type": "pipeline",
              "updated_at": "2022-04-14T06:20:19.349585"
          },
          {
              "id": 7,
              "url": "http://localhost:3000/api/task_draft/7",
              "connector_types": [
                  {
                      "id": 13,
                      "url": "http://localhost:3000/api/task_connector_type/13",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/output",
                      "iotype_value": "output",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  },
                  {
                      "id": 12,
                      "url": "http://localhost:3000/api/task_connector_type/12",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  }
              ],
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_draft/8"
              ],
              "consumed_by_ids": [
                  8
              ],
              "created_at": "2022-04-14T06:20:19.356637",
              "description": "Preprocessing Pipeline for Target Observation target1",
              "duration": 0,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target1",
              "output_pinned": false,
              "produced_by": [
                  "http://localhost:3000/api/task_relation_draft/2"
              ],
              "produced_by_ids": [
                  2
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
              "scheduling_unit_draft_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Paaa+01/1.0/PP",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 10,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 1,
                      "frequency_steps": 1
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/7"
              ],
              "task_blueprints_ids": [
                  7
              ],
              "task_type": "pipeline",
              "updated_at": "2022-04-14T06:20:19.356648"
          },
          {
              "id": 8,
              "url": "http://localhost:3000/api/task_draft/8",
              "connector_types": [
                  {
                      "id": 26,
                      "url": "http://localhost:3000/api/task_connector_type/26",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/correlator",
                      "role_value": "correlator",
                      "task_template": "http://localhost:3000/api/task_template/2",
                      "task_template_id": 2
                  },
                  {
                      "id": 21,
                      "url": "http://localhost:3000/api/task_connector_type/21",
                      "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                      "dataformat_value": "Beamformed",
                      "datatype": "http://localhost:3000/api/datatype/time%20series",
                      "datatype_value": "time series",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/2",
                      "task_template_id": 2
                  },
                  {
                      "id": 20,
                      "url": "http://localhost:3000/api/task_connector_type/20",
                      "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                      "dataformat_value": "Beamformed",
                      "datatype": "http://localhost:3000/api/datatype/time%20series",
                      "datatype_value": "time series",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/beamformer",
                      "role_value": "beamformer",
                      "task_template": "http://localhost:3000/api/task_template/2",
                      "task_template_id": 2
                  },
                  {
                      "id": 18,
                      "url": "http://localhost:3000/api/task_connector_type/18",
                      "dataformat": "http://localhost:3000/api/dataformat/pulp%20summary",
                      "dataformat_value": "pulp summary",
                      "datatype": "http://localhost:3000/api/datatype/quality",
                      "datatype_value": "quality",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/2",
                      "task_template_id": 2
                  },
                  {
                      "id": 16,
                      "url": "http://localhost:3000/api/task_connector_type/16",
                      "dataformat": "http://localhost:3000/api/dataformat/pulp%20analysis",
                      "dataformat_value": "pulp analysis",
                      "datatype": "http://localhost:3000/api/datatype/pulsar%20profile",
                      "datatype_value": "pulsar profile",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/2",
                      "task_template_id": 2
                  },
                  {
                      "id": 14,
                      "url": "http://localhost:3000/api/task_connector_type/14",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/2",
                      "task_template_id": 2
                  }
              ],
              "consumed_by": [],
              "consumed_by_ids": [],
              "created_at": "2022-04-14T06:20:19.364055",
              "description": "Ingest all preprocessed dataproducts",
              "duration": 0,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Ingest",
              "output_pinned": false,
              "produced_by": [
                  "http://localhost:3000/api/task_relation_draft/7",
                  "http://localhost:3000/api/task_relation_draft/8",
                  "http://localhost:3000/api/task_relation_draft/9",
                  "http://localhost:3000/api/task_relation_draft/10",
                  "http://localhost:3000/api/task_relation_draft/11",
                  "http://localhost:3000/api/task_relation_draft/12"
              ],
              "produced_by_ids": [
                  7,
                  8,
                  9,
                  10,
                  11,
                  12
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
              "scheduling_unit_draft_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "",
              "specifications_doc": {
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
              },
              "specifications_template": {
                  "id": 2,
                  "url": "http://localhost:3000/api/task_template/2",
                  "created_at": "2022-04-14T06:20:07.089989",
                  "description": "This schema defines the parameters to setup an ingest task.",
                  "name": "ingest",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved",
                      "type": "object",
                      "title": "ingest",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [],
                      "properties": {},
                      "description": "This schema defines the parameters to setup an ingest task.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {}
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2#",
                      "type": "object",
                      "title": "ingest",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [],
                      "properties": {},
                      "description": "This schema defines the parameters to setup an ingest task.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/ingest",
                  "type_value": "ingest",
                  "updated_at": "2022-04-14T06:20:07.090050",
                  "version": 2
              },
              "specifications_template_id": 2,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/8"
              ],
              "task_blueprints_ids": [
                  8
              ],
              "task_type": "ingest",
              "updated_at": "2022-04-14T06:20:19.364067"
          }
      ],
      "task_drafts_ids": [
          1,
          2,
          3,
          4,
          5,
          6,
          7,
          8
      ],
      "updated_at": "2022-04-14T06:20:19.156283"
    },
    {
      "id": 2,
      "url": "http://localhost:3000/api/scheduling_unit_draft/2",
      "created_at": "2022-04-14T06:20:22.208148",
      "description": "Test scheduling unit",
      "duration": 120,
      "ingest_permission_required": true,
      "interrupts_telescope": false,
      "name": "IM LBA Survey - 3 Beams",
      "observation_strategy_template": "http://localhost:3000/api/scheduling_unit_observing_strategy_template/2",
      "observation_strategy_template_id": 2,
      "piggyback_allowed_aartfaac": true,
      "piggyback_allowed_tbb": true,
      "priority_queue": "http://localhost:3000/api/priority_queue_type/A",
      "priority_queue_value": "A",
      "rank": 1,
      "scheduling_constraints_doc": {
          "sky": {
              "min_distance": {
                  "sun": 0.5,
                  "moon": 0.5,
                  "jupiter": 0.5
              },
              "min_elevation": {
                  "target": 0.5,
                  "calibrator": 0.5
              },
              "transit_offset": {
                  "to": 7200,
                  "from": -7200
              }
          },
          "time": {
              "at": "2022-04-14T06:42:19.054619"
          },
          "daily": {
              "require_day": false,
              "require_night": false,
              "avoid_twilight": false
          },
          "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
          "scheduler": "fixed_time"
      },
      "scheduling_constraints_template": "http://localhost:3000/api/scheduling_constraints_template/1",
      "scheduling_constraints_template_id": 1,
      "scheduling_set": "http://localhost:3000/api/scheduling_set/3",
      "scheduling_set_id": 3,
      "scheduling_unit_blueprints": [
          "http://localhost:3000/api/scheduling_unit_blueprint/2"
      ],
      "scheduling_unit_blueprints_ids": [
          2
      ],
      "specifications_template": "http://localhost:3000/api/scheduling_unit_template/1",
      "specifications_template_id": 1,
      "tags": [],
      "task_drafts": [
          {
              "id": 9,
              "url": "http://localhost:3000/api/task_draft/9",
              "connector_types": [
                  {
                      "id": 3,
                      "url": "http://localhost:3000/api/task_connector_type/3",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/output",
                      "iotype_value": "output",
                      "role": "http://localhost:3000/api/role/correlator",
                      "role_value": "correlator",
                      "task_template": "http://localhost:3000/api/task_template/5",
                      "task_template_id": 5
                  }
              ],
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_draft/13",
                  "http://localhost:3000/api/task_relation_draft/14",
                  "http://localhost:3000/api/task_relation_draft/15",
                  "http://localhost:3000/api/task_relation_draft/16",
                  "http://localhost:3000/api/task_relation_draft/21"
              ],
              "consumed_by_ids": [
                  13,
                  14,
                  15,
                  16,
                  21
              ],
              "created_at": "2022-04-14T06:20:22.242382",
              "description": "Combined parallel Calibrator & Target Observation for LBA with 3 target beams",
              "duration": 120,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Combined Observation",
              "output_pinned": false,
              "produced_by": [],
              "produced_by_ids": [],
              "relative_start_time": 0,
              "relative_stop_time": 120,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/2",
              "scheduling_unit_draft_id": 2,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "OOO.O Paaa+01 Paaa+02 Paaa+03",
              "specifications_doc": {
                  "QA": {
                      "plots": {
                          "enabled": true,
                          "autocorrelation": true,
                          "crosscorrelation": true
                      },
                      "file_conversion": {
                          "enabled": true,
                          "nr_of_subbands": -1,
                          "nr_of_timestamps": 256
                      },
                      "inspection_plots": "msplots"
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                  "duration": 120,
                  "calibrator": {
                      "name": "calibrator",
                      "pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "3Cabc",
                          "direction_type": "J2000"
                      },
                      "autoselect": false
                  },
                  "correlator": {
                      "storage_cluster": "CEP4",
                      "integration_time": 1,
                      "channels_per_subband": 64,
                      "topocentric_frequency_correction": false
                  },
                  "station_configuration": {
                      "SAPs": [
                          {
                              "name": "target1",
                              "subbands": [
                                  217,
                                  218,
                                  219,
                                  220,
                                  221,
                                  222,
                                  223,
                                  224,
                                  225,
                                  226,
                                  227,
                                  228,
                                  229,
                                  230,
                                  231,
                                  232,
                                  233,
                                  234,
                                  235,
                                  236,
                                  237,
                                  238,
                                  239,
                                  240,
                                  241,
                                  242,
                                  243,
                                  244,
                                  245,
                                  246,
                                  247,
                                  248,
                                  249,
                                  250,
                                  251,
                                  252,
                                  253,
                                  254,
                                  255,
                                  256,
                                  257,
                                  258,
                                  259,
                                  260,
                                  261,
                                  262,
                                  263,
                                  264,
                                  265,
                                  266,
                                  267,
                                  268,
                                  269,
                                  270,
                                  271,
                                  272,
                                  273,
                                  274,
                                  275,
                                  276,
                                  277,
                                  278,
                                  279,
                                  280,
                                  281,
                                  282,
                                  283,
                                  284,
                                  285,
                                  286,
                                  287,
                                  288,
                                  289,
                                  290,
                                  291,
                                  292,
                                  293,
                                  294,
                                  295,
                                  296,
                                  297,
                                  298,
                                  299,
                                  300,
                                  301,
                                  302,
                                  303,
                                  304,
                                  305,
                                  306,
                                  307,
                                  308,
                                  309,
                                  310,
                                  311,
                                  312,
                                  313,
                                  314,
                                  315,
                                  316,
                                  317,
                                  318,
                                  319,
                                  320,
                                  321,
                                  322,
                                  323,
                                  324,
                                  325,
                                  326,
                                  327,
                                  328,
                                  329,
                                  330,
                                  331,
                                  332,
                                  333,
                                  334,
                                  335,
                                  336,
                                  337,
                                  338
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+01",
                                  "direction_type": "J2000"
                              }
                          },
                          {
                              "name": "target2",
                              "subbands": [
                                  217,
                                  218,
                                  219,
                                  220,
                                  221,
                                  222,
                                  223,
                                  224,
                                  225,
                                  226,
                                  227,
                                  228,
                                  229,
                                  230,
                                  231,
                                  232,
                                  233,
                                  234,
                                  235,
                                  236,
                                  237,
                                  238,
                                  239,
                                  240,
                                  241,
                                  242,
                                  243,
                                  244,
                                  245,
                                  246,
                                  247,
                                  248,
                                  249,
                                  250,
                                  251,
                                  252,
                                  253,
                                  254,
                                  255,
                                  256,
                                  257,
                                  258,
                                  259,
                                  260,
                                  261,
                                  262,
                                  263,
                                  264,
                                  265,
                                  266,
                                  267,
                                  268,
                                  269,
                                  270,
                                  271,
                                  272,
                                  273,
                                  274,
                                  275,
                                  276,
                                  277,
                                  278,
                                  279,
                                  280,
                                  281,
                                  282,
                                  283,
                                  284,
                                  285,
                                  286,
                                  287,
                                  288,
                                  289,
                                  290,
                                  291,
                                  292,
                                  293,
                                  294,
                                  295,
                                  296,
                                  297,
                                  298,
                                  299,
                                  300,
                                  301,
                                  302,
                                  303,
                                  304,
                                  305,
                                  306,
                                  307,
                                  308,
                                  309,
                                  310,
                                  311,
                                  312,
                                  313,
                                  314,
                                  315,
                                  316,
                                  317,
                                  318,
                                  319,
                                  320,
                                  321,
                                  322,
                                  323,
                                  324,
                                  325,
                                  326,
                                  327,
                                  328,
                                  329,
                                  330,
                                  331,
                                  332,
                                  333,
                                  334,
                                  335,
                                  336,
                                  337,
                                  338
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+02",
                                  "direction_type": "J2000"
                              }
                          },
                          {
                              "name": "target3",
                              "subbands": [
                                  217,
                                  218,
                                  219,
                                  220,
                                  221,
                                  222,
                                  223,
                                  224,
                                  225,
                                  226,
                                  227,
                                  228,
                                  229,
                                  230,
                                  231,
                                  232,
                                  233,
                                  234,
                                  235,
                                  236,
                                  237,
                                  238,
                                  239,
                                  240,
                                  241,
                                  242,
                                  243,
                                  244,
                                  245,
                                  246,
                                  247,
                                  248,
                                  249,
                                  250,
                                  251,
                                  252,
                                  253,
                                  254,
                                  255,
                                  256,
                                  257,
                                  258,
                                  259,
                                  260,
                                  261,
                                  262,
                                  263,
                                  264,
                                  265,
                                  266,
                                  267,
                                  268,
                                  269,
                                  270,
                                  271,
                                  272,
                                  273,
                                  274,
                                  275,
                                  276,
                                  277,
                                  278,
                                  279,
                                  280,
                                  281,
                                  282,
                                  283,
                                  284,
                                  285,
                                  286,
                                  287,
                                  288,
                                  289,
                                  290,
                                  291,
                                  292,
                                  293,
                                  294,
                                  295,
                                  296,
                                  297,
                                  298,
                                  299,
                                  300,
                                  301,
                                  302,
                                  303,
                                  304,
                                  305,
                                  306,
                                  307,
                                  308,
                                  309,
                                  310,
                                  311,
                                  312,
                                  313,
                                  314,
                                  315,
                                  316,
                                  317,
                                  318,
                                  319,
                                  320,
                                  321,
                                  322,
                                  323,
                                  324,
                                  325,
                                  326,
                                  327,
                                  328,
                                  329,
                                  330,
                                  331,
                                  332,
                                  333,
                                  334,
                                  335,
                                  336,
                                  337,
                                  338
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+03",
                                  "direction_type": "J2000"
                              }
                          }
                      ],
                      "filter": "LBA_30_90",
                      "tile_beam": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "_target_name_",
                          "direction_type": "J2000"
                      },
                      "antenna_set": "LBA_SPARSE_EVEN",
                      "station_groups": [
                          {
                              "stations": [
                                  "CS001",
                                  "CS002",
                                  "CS003",
                                  "CS004",
                                  "CS005",
                                  "CS006",
                                  "CS007",
                                  "CS011",
                                  "CS013",
                                  "CS017",
                                  "CS021",
                                  "CS024",
                                  "CS026",
                                  "CS028",
                                  "CS030",
                                  "CS031",
                                  "CS032",
                                  "CS101",
                                  "CS103",
                                  "CS201",
                                  "CS301",
                                  "CS302",
                                  "CS401",
                                  "CS501",
                                  "RS106",
                                  "RS205",
                                  "RS208",
                                  "RS210",
                                  "RS305",
                                  "RS306",
                                  "RS307",
                                  "RS310",
                                  "RS406",
                                  "RS407",
                                  "RS409",
                                  "RS503",
                                  "RS508",
                                  "RS509"
                              ],
                              "max_nr_missing": 4
                          },
                          {
                              "stations": [
                                  "RS508",
                                  "RS509"
                              ],
                              "max_nr_missing": 1
                          },
                          {
                              "stations": [
                                  "RS310",
                                  "RS210"
                              ],
                              "max_nr_missing": 0
                          }
                      ]
                  }
              },
              "specifications_template": {
                  "id": 5,
                  "url": "http://localhost:3000/api/task_template/5",
                  "created_at": "2022-04-14T06:20:07.273898",
                  "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                  "name": "parallel calibrator target observation",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                      "type": "object",
                      "title": "parallel calibrator target observation",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "calibrator",
                          "correlator",
                          "duration",
                          "station_configuration"
                      ],
                      "properties": {
                          "QA": {
                              "$ref": "#/definitions/QA",
                              "default": {}
                          },
                          "duration": {
                              "$id": "#duration",
                              "$ref": "#/definitions/timedelta",
                              "title": "Duration",
                              "default": 600,
                              "minimum": 1,
                              "description": "Duration of this observation (seconds)"
                          },
                          "calibrator": {
                              "$ref": "#/definitions/calibrator",
                              "default": {}
                          },
                          "correlator": {
                              "$ref": "#/definitions/correlator",
                              "default": {}
                          },
                          "station_configuration": {
                              "$ref": "#/definitions/station_configuration",
                              "default": {}
                          }
                      },
                      "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "QA": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Perform all Quality Assurance (QA) tasks, including file conversion and plotting.",
                              "properties": {
                                  "file_conversion": {
                                      "$ref": "#/definitions/file_conversion",
                                      "default": {}
                                  },
                                  "inspection_plots": {
                                      "$ref": "#/definitions/inspection_plots",
                                      "default": "msplots"
                                  },
                                  "plots": {
                                      "$ref": "#/definitions/plots",
                                      "default": {}
                                  }
                              },
                              "required": [
                                  "file_conversion",
                                  "plots",
                                  "inspection_plots"
                              ],
                              "title": "QA",
                              "type": "object"
                          },
                          "plots": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Create dynamic spectrum plots",
                              "properties": {
                                  "autocorrelation": {
                                      "default": true,
                                      "description": "Create autocorrelation plots for all stations",
                                      "title": "autocorrelation",
                                      "type": "boolean"
                                  },
                                  "crosscorrelation": {
                                      "default": true,
                                      "description": "Create crosscorrelation plots for all baselines",
                                      "title": "crosscorrelation",
                                      "type": "boolean"
                                  },
                                  "enabled": {
                                      "default": true,
                                      "description": "Do/Don't create plots from the QA file from the observation",
                                      "title": "enabled",
                                      "type": "boolean"
                                  }
                              },
                              "required": [
                                  "enabled",
                                  "autocorrelation",
                                  "crosscorrelation"
                              ],
                              "title": "Plots",
                              "type": "object"
                          },
                          "file_conversion": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Create a QA file for the observation",
                              "properties": {
                                  "enabled": {
                                      "default": true,
                                      "description": "Do/Don't create a QA file for the observation",
                                      "title": "enabled",
                                      "type": "boolean"
                                  },
                                  "nr_of_subbands": {
                                      "default": -1,
                                      "description": "Keep this number of subbands from the observation in the QA file, or all if -1",
                                      "title": "#subbands",
                                      "type": "integer"
                                  },
                                  "nr_of_timestamps": {
                                      "default": 256,
                                      "description": "Extract this number of timestamps from the observation in the QA file (equidistantanly sampled, no averaging/interpolation)",
                                      "minimum": 1,
                                      "title": "#timestamps",
                                      "type": "integer"
                                  }
                              },
                              "required": [
                                  "enabled",
                                  "nr_of_subbands",
                                  "nr_of_timestamps"
                              ],
                              "title": "File Conversion",
                              "type": "object"
                          },
                          "inspection_plots": {
                              "decription": "Type of inspection plots to run",
                              "default": "msplots",
                              "enum": [
                                  "msplots",
                                  "dynspec",
                                  "none"
                              ],
                              "title": "Inspection Plots",
                              "type": "string"
                          },
                          "timedelta": {
                              "default": 0,
                              "description": "A time duration or delta expressed in seconds",
                              "type": "number"
                          },
                          "timestamp": {
                              "default": "1970-01-01T00:00:00Z",
                              "description": "A timestamp defined in UTC",
                              "format": "date-time",
                              "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                              "type": "string"
                          },
                          "calibrator": {
                              "additionalProperties": false,
                              "default": {},
                              "properties": {
                                  "autoselect": {
                                      "default": true,
                                      "description": "Auto-select calibrator based on elevation",
                                      "title": "Auto-select",
                                      "type": "boolean"
                                  },
                                  "name": {
                                      "default": "calibrator",
                                      "description": "Name of the calibrator SAP",
                                      "title": "Name",
                                      "type": "string"
                                  },
                                  "pointing": {
                                      "$ref": "#/definitions/pointing",
                                      "default": {},
                                      "description": "Manually selected calibrator",
                                      "title": "Digital pointing"
                                  }
                              },
                              "required": [
                                  "autoselect",
                                  "pointing",
                                  "name"
                              ],
                              "type": "object"
                          },
                          "pointing": {
                              "additionalProperties": false,
                              "properties": {
                                  "angle1": {
                                      "default": 0.6624317181687094,
                                      "description": "First angle (e.g. RA)",
                                      "title": "Angle 1",
                                      "type": "number"
                                  },
                                  "angle2": {
                                      "default": 1.5579526427549426,
                                      "description": "Second angle (e.g. DEC)",
                                      "title": "Angle 2",
                                      "type": "number"
                                  },
                                  "direction_type": {
                                      "default": "J2000",
                                      "description": "",
                                      "enum": [
                                          "J2000",
                                          "AZELGEO",
                                          "LMN",
                                          "SUN",
                                          "MOON",
                                          "MERCURY",
                                          "VENUS",
                                          "MARS",
                                          "JUPITER",
                                          "SATURN",
                                          "URANUS",
                                          "NEPTUNE",
                                          "PLUTO"
                                      ],
                                      "title": "Reference frame",
                                      "type": "string"
                                  },
                                  "target": {
                                      "default": "_target_name_",
                                      "description": "Description of where this beam points at",
                                      "minLength": 1,
                                      "title": "Target",
                                      "type": "string"
                                  }
                              },
                              "required": [
                                  "angle1",
                                  "angle2",
                                  "target"
                              ],
                              "type": "object"
                          },
                          "correlator": {
                              "additionalProperties": false,
                              "default": {},
                              "properties": {
                                  "channels_per_subband": {
                                      "default": 64,
                                      "description": "Number of frequency bands per subband",
                                      "enum": [
                                          8,
                                          16,
                                          32,
                                          64,
                                          128,
                                          256,
                                          512,
                                          1024
                                      ],
                                      "minimum": 8,
                                      "title": "Channels/subband",
                                      "type": "integer"
                                  },
                                  "integration_time": {
                                      "$ref": "#/definitions/timedelta",
                                      "default": 1,
                                      "description": "Desired integration period (seconds)",
                                      "minimum": 0.1,
                                      "title": "Integration time"
                                  },
                                  "storage_cluster": {
                                      "default": "CEP4",
                                      "description": "Cluster to write output to",
                                      "enum": [
                                          "CEP4",
                                          "DragNet"
                                      ],
                                      "title": "Storage cluster",
                                      "type": "string"
                                  },
                                  "topocentric_frequency_correction": {
                                      "default": false,
                                      "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation.",
                                      "title": "Topocentric Frequency Correction",
                                      "type": "boolean"
                                  }
                              }
                          },
                          "station_configuration": {
                              "default": {},
                              "properties": {
                                  "SAPs": {
                                      "$ref": "#/definitions/SAPs",
                                      "default": [
                                          {}
                                      ],
                                      "minItems": 1
                                  },
                                  "antenna_set": {
                                      "$ref": "#/definitions/antenna_set",
                                      "default": "HBA_DUAL"
                                  },
                                  "filter": {
                                      "$ref": "#/definitions/filter",
                                      "default": "HBA_110_190"
                                  },
                                  "station_groups": {
                                      "$ref": "#/definitions/station_groups",
                                      "default": [
                                          {
                                              "max_nr_missing": 1,
                                              "stations": [
                                                  "CS002",
                                                  "CS003",
                                                  "CS004",
                                                  "CS005",
                                                  "CS006",
                                                  "CS007"
                                              ]
                                          }
                                      ]
                                  },
                                  "tile_beam": {
                                      "$ref": "#/definitions/pointing",
                                      "default": {},
                                      "description": "HBA only",
                                      "title": "Tile beam"
                                  }
                              },
                              "required": [
                                  "station_groups",
                                  "antenna_set",
                                  "filter",
                                  "SAPs"
                              ],
                              "title": "station_configuration",
                              "type": "object"
                          },
                          "station": {
                              "description": "These are the LOFAR stations",
                              "enum": [
                                  "CS001",
                                  "CS002",
                                  "CS003",
                                  "CS004",
                                  "CS005",
                                  "CS006",
                                  "CS007",
                                  "CS011",
                                  "CS013",
                                  "CS017",
                                  "CS021",
                                  "CS024",
                                  "CS026",
                                  "CS028",
                                  "CS030",
                                  "CS031",
                                  "CS032",
                                  "CS101",
                                  "CS103",
                                  "CS201",
                                  "CS301",
                                  "CS302",
                                  "CS401",
                                  "CS501",
                                  "RS106",
                                  "RS205",
                                  "RS208",
                                  "RS210",
                                  "RS305",
                                  "RS306",
                                  "RS307",
                                  "RS310",
                                  "RS406",
                                  "RS407",
                                  "RS409",
                                  "RS503",
                                  "RS508",
                                  "RS509",
                                  "DE601",
                                  "DE602",
                                  "DE603",
                                  "DE604",
                                  "DE605",
                                  "FR606",
                                  "SE607",
                                  "UK608",
                                  "DE609",
                                  "PL610",
                                  "PL611",
                                  "PL612",
                                  "IE613",
                                  "LV614"
                              ],
                              "title": "Station",
                              "type": "string"
                          },
                          "station_groups": {
                              "additionalItems": false,
                              "additionalProperties": false,
                              "default": [
                                  {
                                      "max_nr_missing": 1,
                                      "stations": [
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007"
                                      ]
                                  }
                              ],
                              "description": "One or more predefined or custom groups of stations",
                              "items": {
                                  "$ref": "#/definitions/station_group"
                              },
                              "minItems": 1,
                              "title": "Station groups",
                              "type": "array"
                          },
                          "max_number_of_missing_stations": {
                              "default": 0,
                              "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                              "minimum": 0,
                              "title": "Maximum number of stations to omit",
                              "type": "integer"
                          },
                          "SAPs": {
                              "additionalItems": false,
                              "default": [
                                  {}
                              ],
                              "description": "Station beams",
                              "items": {
                                  "additionalProperties": false,
                                  "default": {},
                                  "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                  "properties": {
                                      "digital_pointing": {
                                          "$id": "#target_pointing",
                                          "$ref": "#/definitions/pointing",
                                          "default": {},
                                          "title": "Digital pointing"
                                      },
                                      "name": {
                                          "default": "_SAP_name_",
                                          "description": "Identifier for this beam",
                                          "minLength": 1,
                                          "title": "Name",
                                          "type": "string"
                                      },
                                      "subbands": {
                                          "additionalItems": false,
                                          "default": [
                                              255
                                          ],
                                          "items": {
                                              "maximum": 511,
                                              "minimum": 0,
                                              "title": "Subband",
                                              "type": "integer"
                                          },
                                          "maxItems": 488,
                                          "minItems": 1,
                                          "title": "Subband list",
                                          "type": "array"
                                      }
                                  },
                                  "required": [
                                      "name",
                                      "digital_pointing",
                                      "subbands"
                                  ],
                                  "title": "SAP",
                                  "type": "object"
                              },
                              "minItems": 0,
                              "title": "SAPs",
                              "type": "array"
                          },
                          "antenna_set": {
                              "default": "HBA_DUAL",
                              "description": "Fields & antennas to use",
                              "enum": [
                                  "HBA_DUAL",
                                  "HBA_DUAL_INNER",
                                  "HBA_ONE",
                                  "HBA_ONE_INNER",
                                  "HBA_ZERO",
                                  "HBA_ZERO_INNER",
                                  "LBA_INNER",
                                  "LBA_OUTER",
                                  "LBA_SPARSE_EVEN",
                                  "LBA_SPARSE_ODD",
                                  "LBA_ALL"
                              ],
                              "title": "Antenna set",
                              "type": "string"
                          },
                          "filter": {
                              "default": "HBA_110_190",
                              "description": "Must match antenna type",
                              "enum": [
                                  "LBA_10_70",
                                  "LBA_30_70",
                                  "LBA_10_90",
                                  "LBA_30_90",
                                  "HBA_110_190",
                                  "HBA_210_250"
                              ],
                              "title": "Band-pass filter",
                              "type": "string"
                          },
                          "station_list": {
                              "additionalItems": false,
                              "additionalProperties": false,
                              "default": [],
                              "items": {
                                  "$ref": "#/definitions/station"
                              },
                              "minItems": 0,
                              "type": "array",
                              "uniqueItems": true
                          },
                          "station_group": {
                              "anyOf": [
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 0,
                                          "stations": [
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007"
                                          ]
                                      },
                                      "description": "The group of all stations on the Superterp",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Superterp",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 4,
                                          "stations": [
                                              "CS001",
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007",
                                              "CS011",
                                              "CS013",
                                              "CS017",
                                              "CS021",
                                              "CS024",
                                              "CS026",
                                              "CS028",
                                              "CS030",
                                              "CS031",
                                              "CS032",
                                              "CS101",
                                              "CS103",
                                              "CS201",
                                              "CS301",
                                              "CS302",
                                              "CS401",
                                              "CS501"
                                          ]
                                      },
                                      "description": "The group of all Core stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS001",
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007",
                                                      "CS011",
                                                      "CS013",
                                                      "CS017",
                                                      "CS021",
                                                      "CS024",
                                                      "CS026",
                                                      "CS028",
                                                      "CS030",
                                                      "CS031",
                                                      "CS032",
                                                      "CS101",
                                                      "CS103",
                                                      "CS201",
                                                      "CS301",
                                                      "CS302",
                                                      "CS401",
                                                      "CS501"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Core",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 4,
                                          "stations": [
                                              "RS106",
                                              "RS205",
                                              "RS208",
                                              "RS210",
                                              "RS305",
                                              "RS306",
                                              "RS307",
                                              "RS310",
                                              "RS406",
                                              "RS407",
                                              "RS409",
                                              "RS503",
                                              "RS508",
                                              "RS509"
                                          ]
                                      },
                                      "description": "The group of all Dutch remote stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "RS106",
                                                      "RS205",
                                                      "RS208",
                                                      "RS210",
                                                      "RS305",
                                                      "RS306",
                                                      "RS307",
                                                      "RS310",
                                                      "RS406",
                                                      "RS407",
                                                      "RS409",
                                                      "RS503",
                                                      "RS508",
                                                      "RS509"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Remote",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 4,
                                          "stations": [
                                              "CS001",
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007",
                                              "CS011",
                                              "CS013",
                                              "CS017",
                                              "CS021",
                                              "CS024",
                                              "CS026",
                                              "CS028",
                                              "CS030",
                                              "CS031",
                                              "CS032",
                                              "CS101",
                                              "CS103",
                                              "CS201",
                                              "CS301",
                                              "CS302",
                                              "CS401",
                                              "CS501",
                                              "RS106",
                                              "RS205",
                                              "RS208",
                                              "RS210",
                                              "RS305",
                                              "RS306",
                                              "RS307",
                                              "RS310",
                                              "RS406",
                                              "RS407",
                                              "RS409",
                                              "RS503",
                                              "RS508",
                                              "RS509"
                                          ]
                                      },
                                      "description": "The group of all Dutch (Core + Remote) stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS001",
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007",
                                                      "CS011",
                                                      "CS013",
                                                      "CS017",
                                                      "CS021",
                                                      "CS024",
                                                      "CS026",
                                                      "CS028",
                                                      "CS030",
                                                      "CS031",
                                                      "CS032",
                                                      "CS101",
                                                      "CS103",
                                                      "CS201",
                                                      "CS301",
                                                      "CS302",
                                                      "CS401",
                                                      "CS501",
                                                      "RS106",
                                                      "RS205",
                                                      "RS208",
                                                      "RS210",
                                                      "RS305",
                                                      "RS306",
                                                      "RS307",
                                                      "RS310",
                                                      "RS406",
                                                      "RS407",
                                                      "RS409",
                                                      "RS503",
                                                      "RS508",
                                                      "RS509"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Dutch",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 2,
                                          "stations": [
                                              "DE601",
                                              "DE602",
                                              "DE603",
                                              "DE604",
                                              "DE605",
                                              "DE609",
                                              "FR606",
                                              "SE607",
                                              "UK608",
                                              "PL610",
                                              "PL611",
                                              "PL612",
                                              "IE613",
                                              "LV614"
                                          ]
                                      },
                                      "description": "The group of all international stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "DE601",
                                                      "DE602",
                                                      "DE603",
                                                      "DE604",
                                                      "DE605",
                                                      "DE609",
                                                      "FR606",
                                                      "SE607",
                                                      "UK608",
                                                      "PL610",
                                                      "PL611",
                                                      "PL612",
                                                      "IE613",
                                                      "LV614"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "International",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 1,
                                          "stations": [
                                              "DE601",
                                              "DE605"
                                          ]
                                      },
                                      "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "DE601",
                                                      "DE605"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "International required",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 6,
                                          "stations": [
                                              "CS001",
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007",
                                              "CS011",
                                              "CS013",
                                              "CS017",
                                              "CS021",
                                              "CS024",
                                              "CS026",
                                              "CS028",
                                              "CS030",
                                              "CS031",
                                              "CS032",
                                              "CS101",
                                              "CS103",
                                              "CS201",
                                              "CS301",
                                              "CS302",
                                              "CS401",
                                              "CS501",
                                              "RS106",
                                              "RS205",
                                              "RS208",
                                              "RS210",
                                              "RS305",
                                              "RS306",
                                              "RS307",
                                              "RS310",
                                              "RS406",
                                              "RS407",
                                              "RS409",
                                              "RS503",
                                              "RS508",
                                              "RS509",
                                              "DE601",
                                              "DE602",
                                              "DE603",
                                              "DE604",
                                              "DE605",
                                              "DE609",
                                              "FR606",
                                              "SE607",
                                              "UK608",
                                              "PL610",
                                              "PL611",
                                              "PL612",
                                              "IE613",
                                              "LV614"
                                          ]
                                      },
                                      "description": "The group of all (Core + Remote + International) stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS001",
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007",
                                                      "CS011",
                                                      "CS013",
                                                      "CS017",
                                                      "CS021",
                                                      "CS024",
                                                      "CS026",
                                                      "CS028",
                                                      "CS030",
                                                      "CS031",
                                                      "CS032",
                                                      "CS101",
                                                      "CS103",
                                                      "CS201",
                                                      "CS301",
                                                      "CS302",
                                                      "CS401",
                                                      "CS501",
                                                      "RS106",
                                                      "RS205",
                                                      "RS208",
                                                      "RS210",
                                                      "RS305",
                                                      "RS306",
                                                      "RS307",
                                                      "RS310",
                                                      "RS406",
                                                      "RS407",
                                                      "RS409",
                                                      "RS503",
                                                      "RS508",
                                                      "RS509",
                                                      "DE601",
                                                      "DE602",
                                                      "DE603",
                                                      "DE604",
                                                      "DE605",
                                                      "DE609",
                                                      "FR606",
                                                      "SE607",
                                                      "UK608",
                                                      "PL610",
                                                      "PL611",
                                                      "PL612",
                                                      "IE613",
                                                      "LV614"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "All",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 0,
                                          "stations": [
                                              "CS001"
                                          ]
                                      },
                                      "description": "A custom group of stations which can be defined by the user",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list"
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Custom",
                                      "type": "object"
                                  }
                              ],
                              "default": {
                                  "max_nr_missing": 1,
                                  "stations": [
                                      "CS002",
                                      "CS003",
                                      "CS004",
                                      "CS005",
                                      "CS006",
                                      "CS007"
                                  ]
                              },
                              "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                              "title": "Station group",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2#",
                      "type": "object",
                      "title": "parallel calibrator target observation",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "calibrator",
                          "correlator",
                          "duration",
                          "station_configuration"
                      ],
                      "properties": {
                          "QA": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/2#/definitions/QA",
                              "default": {}
                          },
                          "duration": {
                              "$id": "#duration",
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                              "title": "Duration",
                              "default": 600,
                              "minimum": 1,
                              "description": "Duration of this observation (seconds)"
                          },
                          "calibrator": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/calibrator/1#/definitions/calibrator",
                              "default": {}
                          },
                          "correlator": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/correlator/1#/definitions/correlator",
                              "default": {}
                          },
                          "station_configuration": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                              "default": {}
                          }
                      },
                      "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/observation",
                  "type_value": "observation",
                  "updated_at": "2022-04-14T06:20:07.273910",
                  "version": 2
              },
              "specifications_template_id": 5,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/9"
              ],
              "task_blueprints_ids": [
                  9
              ],
              "task_type": "observation",
              "updated_at": "2022-04-14T06:20:22.242394"
          },
          {
              "id": 10,
              "url": "http://localhost:3000/api/task_draft/10",
              "connector_types": [
                  {
                      "id": 13,
                      "url": "http://localhost:3000/api/task_connector_type/13",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/output",
                      "iotype_value": "output",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  },
                  {
                      "id": 12,
                      "url": "http://localhost:3000/api/task_connector_type/12",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  }
              ],
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_draft/17"
              ],
              "consumed_by_ids": [
                  17
              ],
              "created_at": "2022-04-14T06:20:22.251986",
              "description": "Preprocessing Pipeline for Calibrator Observation",
              "duration": 0,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Calibrator Pipeline",
              "output_pinned": false,
              "produced_by": [
                  "http://localhost:3000/api/task_relation_draft/13"
              ],
              "produced_by_ids": [
                  13
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/2",
              "scheduling_unit_draft_id": 2,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "c17 oOOO.O 3Cabc",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 8,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 4,
                      "frequency_steps": 8
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/10"
              ],
              "task_blueprints_ids": [
                  10
              ],
              "task_type": "pipeline",
              "updated_at": "2022-04-14T06:20:22.251997"
          },
          {
              "id": 11,
              "url": "http://localhost:3000/api/task_draft/11",
              "connector_types": [
                  {
                      "id": 13,
                      "url": "http://localhost:3000/api/task_connector_type/13",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/output",
                      "iotype_value": "output",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  },
                  {
                      "id": 12,
                      "url": "http://localhost:3000/api/task_connector_type/12",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  }
              ],
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_draft/20",
                  "http://localhost:3000/api/task_relation_draft/24"
              ],
              "consumed_by_ids": [
                  20,
                  24
              ],
              "created_at": "2022-04-14T06:20:22.259971",
              "description": "Preprocessing Pipeline for Target Observation target3",
              "duration": 0,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target3",
              "output_pinned": false,
              "produced_by": [
                  "http://localhost:3000/api/task_relation_draft/16"
              ],
              "produced_by_ids": [
                  16
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/2",
              "scheduling_unit_draft_id": 2,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "c17 oOOO.O Paaa+03",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 8,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 4,
                      "frequency_steps": 8
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/11"
              ],
              "task_blueprints_ids": [
                  11
              ],
              "task_type": "pipeline",
              "updated_at": "2022-04-14T06:20:22.259982"
          },
          {
              "id": 12,
              "url": "http://localhost:3000/api/task_draft/12",
              "connector_types": [
                  {
                      "id": 13,
                      "url": "http://localhost:3000/api/task_connector_type/13",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/output",
                      "iotype_value": "output",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  },
                  {
                      "id": 12,
                      "url": "http://localhost:3000/api/task_connector_type/12",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  }
              ],
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_draft/19",
                  "http://localhost:3000/api/task_relation_draft/23"
              ],
              "consumed_by_ids": [
                  19,
                  23
              ],
              "created_at": "2022-04-14T06:20:22.267248",
              "description": "Preprocessing Pipeline for Target Observation target2",
              "duration": 0,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target2",
              "output_pinned": false,
              "produced_by": [
                  "http://localhost:3000/api/task_relation_draft/15"
              ],
              "produced_by_ids": [
                  15
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/2",
              "scheduling_unit_draft_id": 2,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "c17 oOOO.O Paaa+02",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 8,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 4,
                      "frequency_steps": 8
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/12"
              ],
              "task_blueprints_ids": [
                  12
              ],
              "task_type": "pipeline",
              "updated_at": "2022-04-14T06:20:22.267260"
          },
          {
              "id": 13,
              "url": "http://localhost:3000/api/task_draft/13",
              "connector_types": [
                  {
                      "id": 13,
                      "url": "http://localhost:3000/api/task_connector_type/13",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/output",
                      "iotype_value": "output",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  },
                  {
                      "id": 12,
                      "url": "http://localhost:3000/api/task_connector_type/12",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/10",
                      "task_template_id": 10
                  }
              ],
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_draft/18",
                  "http://localhost:3000/api/task_relation_draft/22"
              ],
              "consumed_by_ids": [
                  18,
                  22
              ],
              "created_at": "2022-04-14T06:20:22.275062",
              "description": "Preprocessing Pipeline for Target Observation target1",
              "duration": 0,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target1",
              "output_pinned": false,
              "produced_by": [
                  "http://localhost:3000/api/task_relation_draft/14"
              ],
              "produced_by_ids": [
                  14
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/2",
              "scheduling_unit_draft_id": 2,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "c17 oOOO.O Paaa+01",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 8,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 4,
                      "frequency_steps": 8
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/13"
              ],
              "task_blueprints_ids": [
                  13
              ],
              "task_type": "pipeline",
              "updated_at": "2022-04-14T06:20:22.275073"
          },
          {
              "id": 14,
              "url": "http://localhost:3000/api/task_draft/14",
              "connector_types": [
                  {
                      "id": 48,
                      "url": "http://localhost:3000/api/task_connector_type/48",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/correlator",
                      "role_value": "correlator",
                      "task_template": "http://localhost:3000/api/task_template/1",
                      "task_template_id": 1
                  },
                  {
                      "id": 43,
                      "url": "http://localhost:3000/api/task_connector_type/43",
                      "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                      "dataformat_value": "Beamformed",
                      "datatype": "http://localhost:3000/api/datatype/time%20series",
                      "datatype_value": "time series",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/1",
                      "task_template_id": 1
                  },
                  {
                      "id": 42,
                      "url": "http://localhost:3000/api/task_connector_type/42",
                      "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                      "dataformat_value": "Beamformed",
                      "datatype": "http://localhost:3000/api/datatype/time%20series",
                      "datatype_value": "time series",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/beamformer",
                      "role_value": "beamformer",
                      "task_template": "http://localhost:3000/api/task_template/1",
                      "task_template_id": 1
                  },
                  {
                      "id": 40,
                      "url": "http://localhost:3000/api/task_connector_type/40",
                      "dataformat": "http://localhost:3000/api/dataformat/pulp%20summary",
                      "dataformat_value": "pulp summary",
                      "datatype": "http://localhost:3000/api/datatype/quality",
                      "datatype_value": "quality",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/1",
                      "task_template_id": 1
                  },
                  {
                      "id": 38,
                      "url": "http://localhost:3000/api/task_connector_type/38",
                      "dataformat": "http://localhost:3000/api/dataformat/pulp%20analysis",
                      "dataformat_value": "pulp analysis",
                      "datatype": "http://localhost:3000/api/datatype/pulsar%20profile",
                      "datatype_value": "pulsar profile",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/1",
                      "task_template_id": 1
                  },
                  {
                      "id": 36,
                      "url": "http://localhost:3000/api/task_connector_type/36",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/1",
                      "task_template_id": 1
                  }
              ],
              "consumed_by": [],
              "consumed_by_ids": [],
              "created_at": "2022-04-14T06:20:22.280424",
              "description": "Clean up all dataproducts from disk after ingest",
              "duration": 0,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Cleanup",
              "output_pinned": false,
              "produced_by": [
                  "http://localhost:3000/api/task_relation_draft/21",
                  "http://localhost:3000/api/task_relation_draft/22",
                  "http://localhost:3000/api/task_relation_draft/23",
                  "http://localhost:3000/api/task_relation_draft/24"
              ],
              "produced_by_ids": [
                  21,
                  22,
                  23,
                  24
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/2",
              "scheduling_unit_draft_id": 2,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "",
              "specifications_doc": {
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2/ref_resolved"
              },
              "specifications_template": {
                  "id": 1,
                  "url": "http://localhost:3000/api/task_template/1",
                  "created_at": "2022-04-14T06:20:07.052510",
                  "description": "This schema defines the parameters to setup a dataproduct(s) cleanup task.",
                  "name": "cleanup",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2/ref_resolved",
                      "type": "object",
                      "title": "cleanup",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [],
                      "properties": {},
                      "description": "This schema defines the parameters to setup a dataproduct(s) cleanup task.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {}
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2#",
                      "type": "object",
                      "title": "cleanup",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [],
                      "properties": {},
                      "description": "This schema defines the parameters to setup a dataproduct(s) cleanup task.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/cleanup",
                  "type_value": "cleanup",
                  "updated_at": "2022-04-14T06:20:07.052528",
                  "version": 2
              },
              "specifications_template_id": 1,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/14"
              ],
              "task_blueprints_ids": [
                  14
              ],
              "task_type": "cleanup",
              "updated_at": "2022-04-14T06:20:22.280436"
          },
          {
              "id": 15,
              "url": "http://localhost:3000/api/task_draft/15",
              "connector_types": [
                  {
                      "id": 26,
                      "url": "http://localhost:3000/api/task_connector_type/26",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/correlator",
                      "role_value": "correlator",
                      "task_template": "http://localhost:3000/api/task_template/2",
                      "task_template_id": 2
                  },
                  {
                      "id": 21,
                      "url": "http://localhost:3000/api/task_connector_type/21",
                      "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                      "dataformat_value": "Beamformed",
                      "datatype": "http://localhost:3000/api/datatype/time%20series",
                      "datatype_value": "time series",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/2",
                      "task_template_id": 2
                  },
                  {
                      "id": 20,
                      "url": "http://localhost:3000/api/task_connector_type/20",
                      "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                      "dataformat_value": "Beamformed",
                      "datatype": "http://localhost:3000/api/datatype/time%20series",
                      "datatype_value": "time series",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/beamformer",
                      "role_value": "beamformer",
                      "task_template": "http://localhost:3000/api/task_template/2",
                      "task_template_id": 2
                  },
                  {
                      "id": 18,
                      "url": "http://localhost:3000/api/task_connector_type/18",
                      "dataformat": "http://localhost:3000/api/dataformat/pulp%20summary",
                      "dataformat_value": "pulp summary",
                      "datatype": "http://localhost:3000/api/datatype/quality",
                      "datatype_value": "quality",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/2",
                      "task_template_id": 2
                  },
                  {
                      "id": 16,
                      "url": "http://localhost:3000/api/task_connector_type/16",
                      "dataformat": "http://localhost:3000/api/dataformat/pulp%20analysis",
                      "dataformat_value": "pulp analysis",
                      "datatype": "http://localhost:3000/api/datatype/pulsar%20profile",
                      "datatype_value": "pulsar profile",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/2",
                      "task_template_id": 2
                  },
                  {
                      "id": 14,
                      "url": "http://localhost:3000/api/task_connector_type/14",
                      "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                      "dataformat_value": "MeasurementSet",
                      "datatype": "http://localhost:3000/api/datatype/visibilities",
                      "datatype_value": "visibilities",
                      "iotype": "http://localhost:3000/api/iotype/input",
                      "iotype_value": "input",
                      "role": "http://localhost:3000/api/role/any",
                      "role_value": "any",
                      "task_template": "http://localhost:3000/api/task_template/2",
                      "task_template_id": 2
                  }
              ],
              "consumed_by": [],
              "consumed_by_ids": [],
              "created_at": "2022-04-14T06:20:22.286429",
              "description": "Ingest all preprocessed dataproducts",
              "duration": 0,
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Ingest",
              "output_pinned": false,
              "produced_by": [
                  "http://localhost:3000/api/task_relation_draft/17",
                  "http://localhost:3000/api/task_relation_draft/18",
                  "http://localhost:3000/api/task_relation_draft/19",
                  "http://localhost:3000/api/task_relation_draft/20"
              ],
              "produced_by_ids": [
                  17,
                  18,
                  19,
                  20
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/2",
              "scheduling_unit_draft_id": 2,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "",
              "specifications_doc": {
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
              },
              "specifications_template": {
                  "id": 2,
                  "url": "http://localhost:3000/api/task_template/2",
                  "created_at": "2022-04-14T06:20:07.089989",
                  "description": "This schema defines the parameters to setup an ingest task.",
                  "name": "ingest",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved",
                      "type": "object",
                      "title": "ingest",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [],
                      "properties": {},
                      "description": "This schema defines the parameters to setup an ingest task.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {}
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2#",
                      "type": "object",
                      "title": "ingest",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [],
                      "properties": {},
                      "description": "This schema defines the parameters to setup an ingest task.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/ingest",
                  "type_value": "ingest",
                  "updated_at": "2022-04-14T06:20:07.090050",
                  "version": 2
              },
              "specifications_template_id": 2,
              "tags": [],
              "task_blueprints": [
                  "http://localhost:3000/api/task_blueprint/15"
              ],
              "task_blueprints_ids": [
                  15
              ],
              "task_type": "ingest",
              "updated_at": "2022-04-14T06:20:22.286440"
          }
      ],
      "task_drafts_ids": [
          9,
          10,
          11,
          12,
          13,
          14,
          15
      ],
      "updated_at": "2022-04-14T06:20:22.208162"
    },
    {
        "id": 7,
        "url": "http://localhost:3000/api/scheduling_unit_draft/7",
        "created_at": "2022-05-03T08:40:25.612193",
        "description": "IM HBA LoTSS - 2 Beams",
        "duration": 30120.0,
        "ingest_permission_required": true,
        "interrupts_telescope": false,
        "name": "IM HBA LoTSS - 2 Beams",
        "observation_strategy_template": "http://localhost:3000/api/scheduling_unit_observing_strategy_template/4",
        "observation_strategy_template_id": 4,
        "piggyback_allowed_aartfaac": true,
        "piggyback_allowed_tbb": true,
        "priority_queue": "http://localhost:3000/api/priority_queue_type/A",
        "priority_queue_value": "A",
        "rank": 0.0016,
        "scheduling_constraints_doc": {
            "sky": {
                "min_distance": {
                    "sun": 0.5000368306963754,
                    "moon": 0.5000368306963754,
                    "jupiter": 0.5000368306963754
                },
                "min_elevation": {
                    "target": 0.5000368306963754,
                    "calibrator": 0.5000368306963754
                },
                "transit_offset": {
                    "to": 1440,
                    "from": -1440
                }
            },
            "time": {
                "between": [],
                "not_between": []
            },
            "daily": {
                "require_day": false,
                "require_night": false,
                "avoid_twilight": false
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
            "scheduler": "dynamic"
        },
        "scheduling_constraints_template": "http://localhost:3000/api/scheduling_constraints_template/1",
        "scheduling_constraints_template_id": 1,
        "scheduling_set": "http://localhost:3000/api/scheduling_set/1",
        "scheduling_set_id": 1,
        "scheduling_unit_blueprints": [],
        "scheduling_unit_blueprints_ids": [],
        "specifications_template": "http://localhost:3000/api/scheduling_unit_template/1",
        "specifications_template_id": 1,
        "tags": [],
        "task_drafts": [
            {
                "id": 21,
                "url": "http://127.0.0.100:8008/api/task_draft/21",
                "connector_types": [
                    {
                        "id": 1,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/1",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://127.0.0.100:8008/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://127.0.0.100:8008/api/role/correlator",
                        "role_value": "correlator",
                        "task_template": "http://127.0.0.100:8008/api/task_template/7",
                        "task_template_id": 7
                    }
                ],
                "consumed_by": [
                    "http://127.0.0.100:8008/api/task_relation_draft/23"
                ],
                "consumed_by_ids": [
                    23
                ],
                "created_at": "2022-05-03T08:40:25.626815",
                "description": "Calibrator Observation 1. Before the Target Observation",
                "duration": 600.0,
                "first_scheduling_relation": [
                    "http://127.0.0.100:8008/api/task_scheduling_relation_draft/2"
                ],
                "first_scheduling_relation_ids": [
                    2
                ],
                "name": "Calibrator Observation 1",
                "output_pinned": false,
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 0.0,
                "relative_stop_time": 600.0,
                "scheduling_unit_draft": "http://127.0.0.100:8008/api/scheduling_unit_draft/7",
                "scheduling_unit_draft_id": 7,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Cal1 3Cabc",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2/ref_resolved",
                    "duration": 600,
                    "calibrator": {
                        "name": "calibrator1",
                        "pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "3Cabc",
                            "direction_type": "J2000"
                        },
                        "autoselect": false
                    }
                },
                "specifications_template": (_.find(templates, {"name": "calibrator observation"})),
                "specifications_template_id": (_.find(templates, {"name": "calibrator observation"})).id,
                "tags": [],
                "task_blueprints": [],
                "task_blueprints_ids": [],
                "task_type": "observation",
                "updated_at": "2022-05-03T08:40:27.442602"
            },
            {
                "id": 22,
                "url": "http://127.0.0.100:8008/api/task_draft/22",
                "connector_types": [
                    {
                        "id": 13,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/13",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://127.0.0.100:8008/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://127.0.0.100:8008/api/role/any",
                        "role_value": "any",
                        "task_template": "http://127.0.0.100:8008/api/task_template/10",
                        "task_template_id": 10
                    },
                    {
                        "id": 12,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/12",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://127.0.0.100:8008/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://127.0.0.100:8008/api/role/any",
                        "role_value": "any",
                        "task_template": "http://127.0.0.100:8008/api/task_template/10",
                        "task_template_id": 10
                    }
                ],
                "consumed_by": [
                    "http://127.0.0.100:8008/api/task_relation_draft/27"
                ],
                "consumed_by_ids": [
                    27
                ],
                "created_at": "2022-05-03T08:40:25.634868",
                "description": "Preprocessing Pipeline for Calibrator Observation 1",
                "duration": 0.0,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Calibrator Pipeline 1",
                "output_pinned": false,
                "produced_by": [
                    "http://127.0.0.100:8008/api/task_relation_draft/23"
                ],
                "produced_by_ids": [
                    23
                ],
                "relative_start_time": 0.0,
                "relative_stop_time": 0.0,
                "scheduling_unit_draft": "http://127.0.0.100:8008/api/scheduling_unit_draft/7",
                "scheduling_unit_draft_id": 7,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Cal1 3Cabc/PP",
                "specifications_doc": {
                    "flag": {
                        "rfi_strategy": "HBAdefault",
                        "outerchannels": true,
                        "autocorrelations": true
                    },
                    "demix": {
                        "sources": [],
                        "time_steps": 10,
                        "ignore_target": false,
                        "frequency_steps": 64
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "average": {
                        "time_steps": 1,
                        "frequency_steps": 4
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    }
                },
                "specifications_template": (_.find(templates, {"name": "preprocessing pipeline"})),
                "specifications_template_id": (_.find(templates, {"name": "preprocessing pipeline"})).id,
                "tags": [],
                "task_blueprints": [],
                "task_blueprints_ids": [],
                "task_type": "pipeline",
                "updated_at": "2022-05-03T08:40:27.666159"
            },
            {
                "id": 25,
                "url": "http://127.0.0.100:8008/api/task_draft/25",
                "connector_types": [
                    {
                        "id": 13,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/13",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://127.0.0.100:8008/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://127.0.0.100:8008/api/role/any",
                        "role_value": "any",
                        "task_template": "http://127.0.0.100:8008/api/task_template/10",
                        "task_template_id": 10
                    },
                    {
                        "id": 12,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/12",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://127.0.0.100:8008/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://127.0.0.100:8008/api/role/any",
                        "role_value": "any",
                        "task_template": "http://127.0.0.100:8008/api/task_template/10",
                        "task_template_id": 10
                    }
                ],
                "consumed_by": [
                    "http://127.0.0.100:8008/api/task_relation_draft/28"
                ],
                "consumed_by_ids": [
                    28
                ],
                "created_at": "2022-05-03T08:40:25.680984",
                "description": "Preprocessing Pipeline for Calibrator Observation 2",
                "duration": 0.0,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Calibrator Pipeline 2",
                "output_pinned": false,
                "produced_by": [
                    "http://127.0.0.100:8008/api/task_relation_draft/24"
                ],
                "produced_by_ids": [
                    24
                ],
                "relative_start_time": 0.0,
                "relative_stop_time": 0.0,
                "scheduling_unit_draft": "http://127.0.0.100:8008/api/scheduling_unit_draft/7",
                "scheduling_unit_draft_id": 7,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Cal2 3Cdef/PP",
                "specifications_doc": {
                    "flag": {
                        "rfi_strategy": "HBAdefault",
                        "outerchannels": true,
                        "autocorrelations": true
                    },
                    "demix": {
                        "sources": [],
                        "time_steps": 10,
                        "ignore_target": false,
                        "frequency_steps": 64
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "average": {
                        "time_steps": 1,
                        "frequency_steps": 4
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    }
                },
                "specifications_template": (_.find(templates, {"name": "preprocessing pipeline"})),
                "specifications_template_id": (_.find(templates, {"name": "preprocessing pipeline"})).id,
                "tags": [],
                "task_blueprints": [],
                "task_blueprints_ids": [],
                "task_type": "pipeline",
                "updated_at": "2022-05-03T08:40:28.043015"
            },
            {
                "id": 28,
                "url": "http://127.0.0.100:8008/api/task_draft/28",
                "connector_types": [
                    {
                        "id": 26,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/26",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://127.0.0.100:8008/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://127.0.0.100:8008/api/role/correlator",
                        "role_value": "correlator",
                        "task_template": "http://127.0.0.100:8008/api/task_template/3",
                        "task_template_id": 3
                    },
                    {
                        "id": 21,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/21",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://127.0.0.100:8008/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://127.0.0.100:8008/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://127.0.0.100:8008/api/role/any",
                        "role_value": "any",
                        "task_template": "http://127.0.0.100:8008/api/task_template/3",
                        "task_template_id": 3
                    },
                    {
                        "id": 20,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/20",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://127.0.0.100:8008/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://127.0.0.100:8008/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://127.0.0.100:8008/api/role/beamformer",
                        "role_value": "beamformer",
                        "task_template": "http://127.0.0.100:8008/api/task_template/3",
                        "task_template_id": 3
                    },
                    {
                        "id": 18,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/18",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/pulp%20summary",
                        "dataformat_value": "pulp summary",
                        "datatype": "http://127.0.0.100:8008/api/datatype/quality",
                        "datatype_value": "quality",
                        "iotype": "http://127.0.0.100:8008/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://127.0.0.100:8008/api/role/any",
                        "role_value": "any",
                        "task_template": "http://127.0.0.100:8008/api/task_template/3",
                        "task_template_id": 3
                    },
                    {
                        "id": 16,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/16",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/pulp%20analysis",
                        "dataformat_value": "pulp analysis",
                        "datatype": "http://127.0.0.100:8008/api/datatype/pulsar%20profile",
                        "datatype_value": "pulsar profile",
                        "iotype": "http://127.0.0.100:8008/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://127.0.0.100:8008/api/role/any",
                        "role_value": "any",
                        "task_template": "http://127.0.0.100:8008/api/task_template/3",
                        "task_template_id": 3
                    },
                    {
                        "id": 14,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/14",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://127.0.0.100:8008/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://127.0.0.100:8008/api/role/any",
                        "role_value": "any",
                        "task_template": "http://127.0.0.100:8008/api/task_template/3",
                        "task_template_id": 3
                    }
                ],
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-05-03T08:40:25.701877",
                "description": "Ingest all preprocessed dataproducts",
                "duration": 0.0,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Ingest",
                "output_pinned": false,
                "produced_by": [
                    "http://127.0.0.100:8008/api/task_relation_draft/28",
                    "http://127.0.0.100:8008/api/task_relation_draft/29",
                    "http://127.0.0.100:8008/api/task_relation_draft/30",
                    "http://127.0.0.100:8008/api/task_relation_draft/27"
                ],
                "produced_by_ids": [
                    28,
                    29,
                    30,
                    27
                ],
                "relative_start_time": 0.0,
                "relative_stop_time": 0.0,
                "scheduling_unit_draft": (_.find(templates, {"name": "ingest"})),
                "scheduling_unit_draft_id": (_.find(templates, {"name": "ingest"})).id,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
                },
                "specifications_template": "http://127.0.0.100:8008/api/task_template/3",
                "specifications_template_id": 3,
                "tags": [],
                "task_blueprints": [],
                "task_blueprints_ids": [],
                "task_type": "ingest",
                "updated_at": "2022-05-03T08:40:26.825287"
            },
            {
                "id": 27,
                "url": "http://127.0.0.100:8008/api/task_draft/27",
                "connector_types": [
                    {
                        "id": 13,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/13",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://127.0.0.100:8008/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://127.0.0.100:8008/api/role/any",
                        "role_value": "any",
                        "task_template": "http://127.0.0.100:8008/api/task_template/10",
                        "task_template_id": 10
                    },
                    {
                        "id": 12,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/12",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://127.0.0.100:8008/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://127.0.0.100:8008/api/role/any",
                        "role_value": "any",
                        "task_template": "http://127.0.0.100:8008/api/task_template/10",
                        "task_template_id": 10
                    }
                ],
                "consumed_by": [
                    "http://127.0.0.100:8008/api/task_relation_draft/29"
                ],
                "consumed_by_ids": [
                    29
                ],
                "created_at": "2022-05-03T08:40:25.695816",
                "description": "Preprocessing Pipeline for Target Observation target1, SAP000, 120-168 MHz, 1s, 16ch/sb",
                "duration": 0.0,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Pipeline target1",
                "output_pinned": false,
                "produced_by": [
                    "http://127.0.0.100:8008/api/task_relation_draft/25"
                ],
                "produced_by_ids": [
                    25
                ],
                "relative_start_time": 0.0,
                "relative_stop_time": 0.0,
                "scheduling_unit_draft": "http://127.0.0.100:8008/api/scheduling_unit_draft/7",
                "scheduling_unit_draft_id": 7,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Paaa+01/TP",
                "specifications_doc": {
                    "flag": {
                        "rfi_strategy": "HBAdefault",
                        "outerchannels": true,
                        "autocorrelations": true
                    },
                    "demix": {
                        "sources": [],
                        "time_steps": 10,
                        "ignore_target": false,
                        "frequency_steps": 64
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "average": {
                        "time_steps": 1,
                        "frequency_steps": 4
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    }
                },
                "specifications_template": (_.find(templates, {"name": "preprocessing pipeline"})),
                "specifications_template_id": (_.find(templates, {"name": "preprocessing pipeline"})).id,
                "tags": [],
                "task_blueprints": [],
                "task_blueprints_ids": [],
                "task_type": "pipeline",
                "updated_at": "2022-05-03T08:40:27.006286"
            },
            {
                "id": 23,
                "url": "http://127.0.0.100:8008/api/task_draft/23",
                "connector_types": [
                    {
                        "id": 2,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/2",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://127.0.0.100:8008/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://127.0.0.100:8008/api/role/correlator",
                        "role_value": "correlator",
                        "task_template": "http://127.0.0.100:8008/api/task_template/5",
                        "task_template_id": 5
                    }
                ],
                "consumed_by": [
                    "http://127.0.0.100:8008/api/task_relation_draft/25",
                    "http://127.0.0.100:8008/api/task_relation_draft/26"
                ],
                "consumed_by_ids": [
                    25,
                    26
                ],
                "created_at": "2022-05-03T08:40:25.666199",
                "description": "Target Observation for LoTSS scheduling unit. HBA_DUAL_INNER, 120-168 MHz, 1s, 64ch/sb",
                "duration": 28800.0,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Target Observation",
                "output_pinned": false,
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 660.0,
                "relative_stop_time": 29460.0,
                "scheduling_unit_draft": "http://127.0.0.100:8008/api/scheduling_unit_draft/7",
                "scheduling_unit_draft_id": 7,
                "second_scheduling_relation": [
                    "http://127.0.0.100:8008/api/task_scheduling_relation_draft/2",
                    "http://127.0.0.100:8008/api/task_scheduling_relation_draft/3"
                ],
                "second_scheduling_relation_ids": [
                    2,
                    3
                ],
                "short_description": "Paaa+01 & Paaa+02",
                "specifications_doc": {
                    "QA": {
                        "plots": {
                            "enabled": true,
                            "autocorrelation": true,
                            "crosscorrelation": true
                        },
                        "file_conversion": {
                            "enabled": true,
                            "nr_of_subbands": -1,
                            "nr_of_timestamps": 256
                        },
                        "inspection_plots": "msplots"
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/2/ref_resolved",
                    "duration": 28800,
                    "correlator": {
                        "storage_cluster": "CEP4",
                        "integration_time": 1,
                        "channels_per_subband": 64,
                        "topocentric_frequency_correction": false
                    },
                    "station_configuration": {
                        "": {
                            "tile_beam": {
                                "angle1": 0,
                                "angle2": 0,
                                "target": "_target_name_"
                            }
                        },
                        "SAPs": [
                            {
                                "name": "sap1",
                                "subbands": [
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    182,
                                    183,
                                    184,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    212,
                                    213,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    242,
                                    243,
                                    244,
                                    245,
                                    246,
                                    247,
                                    248,
                                    249,
                                    250,
                                    251,
                                    252,
                                    253,
                                    254,
                                    255,
                                    257,
                                    258,
                                    259,
                                    260,
                                    261,
                                    262,
                                    263,
                                    264,
                                    265,
                                    266,
                                    267,
                                    268,
                                    269,
                                    270,
                                    271,
                                    272,
                                    273,
                                    275,
                                    276,
                                    277,
                                    278,
                                    279,
                                    280,
                                    281,
                                    282,
                                    283,
                                    284,
                                    285,
                                    286,
                                    287,
                                    288,
                                    289,
                                    290,
                                    291,
                                    292,
                                    293,
                                    294,
                                    295,
                                    296,
                                    297,
                                    298,
                                    299,
                                    300,
                                    302,
                                    303,
                                    304,
                                    305,
                                    306,
                                    307,
                                    308,
                                    309,
                                    310,
                                    311,
                                    312,
                                    313,
                                    314,
                                    315,
                                    316,
                                    317,
                                    318,
                                    319,
                                    320,
                                    321,
                                    322,
                                    323,
                                    324,
                                    325,
                                    326,
                                    327,
                                    328,
                                    330,
                                    331,
                                    332,
                                    333,
                                    334,
                                    335,
                                    336,
                                    337,
                                    338,
                                    339,
                                    340,
                                    341,
                                    342,
                                    343,
                                    344,
                                    345,
                                    346,
                                    347,
                                    349,
                                    364,
                                    372,
                                    380,
                                    388,
                                    396,
                                    404,
                                    413,
                                    421,
                                    430,
                                    438,
                                    447
                                ],
                                "digital_pointing": {
                                    "angle1": 0.6624317181687094,
                                    "angle2": 1.5579526427549426,
                                    "target": "Paaa+01",
                                    "direction_type": "J2000"
                                }
                            },
                            {
                                "name": "sap2",
                                "subbands": [
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    182,
                                    183,
                                    184,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    212,
                                    213,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    242,
                                    243,
                                    244,
                                    245,
                                    246,
                                    247,
                                    248,
                                    249,
                                    250,
                                    251,
                                    252,
                                    253,
                                    254,
                                    255,
                                    257,
                                    258,
                                    259,
                                    260,
                                    261,
                                    262,
                                    263,
                                    264,
                                    265,
                                    266,
                                    267,
                                    268,
                                    269,
                                    270,
                                    271,
                                    272,
                                    273,
                                    275,
                                    276,
                                    277,
                                    278,
                                    279,
                                    280,
                                    281,
                                    282,
                                    283,
                                    284,
                                    285,
                                    286,
                                    287,
                                    288,
                                    289,
                                    290,
                                    291,
                                    292,
                                    293,
                                    294,
                                    295,
                                    296,
                                    297,
                                    298,
                                    299,
                                    300,
                                    302,
                                    303,
                                    304,
                                    305,
                                    306,
                                    307,
                                    308,
                                    309,
                                    310,
                                    311,
                                    312,
                                    313,
                                    314,
                                    315,
                                    316,
                                    317,
                                    318,
                                    319,
                                    320,
                                    321,
                                    322,
                                    323,
                                    324,
                                    325,
                                    326,
                                    327,
                                    328,
                                    330,
                                    331,
                                    332,
                                    333,
                                    334,
                                    335,
                                    336,
                                    337,
                                    338,
                                    339,
                                    340,
                                    341,
                                    342,
                                    343,
                                    344,
                                    345,
                                    346,
                                    347,
                                    349,
                                    364,
                                    372,
                                    380,
                                    388,
                                    396,
                                    404,
                                    413,
                                    421,
                                    430,
                                    438,
                                    447
                                ],
                                "digital_pointing": {
                                    "angle1": 0.6624317181687094,
                                    "angle2": 1.5579526427549426,
                                    "target": "Paaa+02",
                                    "direction_type": "J2000"
                                }
                            }
                        ],
                        "filter": "HBA_110_190",
                        "tile_beam": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+01Paaa+02REF",
                            "direction_type": "J2000"
                        },
                        "antenna_set": "HBA_DUAL_INNER",
                        "station_groups": [
                            {
                                "stations": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501",
                                    "RS106",
                                    "RS205",
                                    "RS208",
                                    "RS210",
                                    "RS305",
                                    "RS306",
                                    "RS307",
                                    "RS310",
                                    "RS406",
                                    "RS407",
                                    "RS409",
                                    "RS503",
                                    "RS508",
                                    "RS509"
                                ],
                                "max_nr_missing": 4
                            },
                            {
                                "stations": [
                                    "DE601",
                                    "DE602",
                                    "DE603",
                                    "DE604",
                                    "DE605",
                                    "DE609",
                                    "FR606",
                                    "SE607",
                                    "UK608",
                                    "PL610",
                                    "PL611",
                                    "PL612",
                                    "IE613",
                                    "LV614"
                                ],
                                "max_nr_missing": 2
                            },
                            {
                                "stations": [
                                    "DE601",
                                    "DE605"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                },
                "specifications_template": (_.find(templates, {"name": "target observation"})),
                "specifications_template_id": (_.find(templates, {"name": "target observation"})).id,
                "tags": [],
                "task_blueprints": [],
                "task_blueprints_ids": [],
                "task_type": "observation",
                "updated_at": "2022-05-03T08:40:26.857829"
            },
            {
                "id": 24,
                "url": "http://127.0.0.100:8008/api/task_draft/24",
                "connector_types": [
                    {
                        "id": 1,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/1",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://127.0.0.100:8008/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://127.0.0.100:8008/api/role/correlator",
                        "role_value": "correlator",
                        "task_template": "http://127.0.0.100:8008/api/task_template/7",
                        "task_template_id": 7
                    }
                ],
                "consumed_by": [
                    "http://127.0.0.100:8008/api/task_relation_draft/24"
                ],
                "consumed_by_ids": [
                    24
                ],
                "created_at": "2022-05-03T08:40:25.673523",
                "description": "Calibrator Observation 2. After the Target Observation",
                "duration": 600.0,
                "first_scheduling_relation": [
                    "http://127.0.0.100:8008/api/task_scheduling_relation_draft/3"
                ],
                "first_scheduling_relation_ids": [
                    3
                ],
                "name": "Calibrator Observation 2",
                "output_pinned": false,
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 29520.0,
                "relative_stop_time": 30120.0,
                "scheduling_unit_draft": "http://127.0.0.100:8008/api/scheduling_unit_draft/7",
                "scheduling_unit_draft_id": 7,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Cal2 3Cdef",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2/ref_resolved",
                    "duration": 600,
                    "calibrator": {
                        "name": "calibrator1",
                        "pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "3Cdef",
                            "direction_type": "J2000"
                        },
                        "autoselect": false
                    }
                },
                "specifications_template": (_.find(templates, {"name": "calibrator observation"})),
                "specifications_template_id": (_.find(templates, {"name": "calibrator observation"})).id,
                "tags": [],
                "task_blueprints": [],
                "task_blueprints_ids": [],
                "task_type": "observation",
                "updated_at": "2022-05-03T08:40:27.881681"
            },
            {
                "id": 26,
                "url": "http://127.0.0.100:8008/api/task_draft/26",
                "connector_types": [
                    {
                        "id": 13,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/13",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://127.0.0.100:8008/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://127.0.0.100:8008/api/role/any",
                        "role_value": "any",
                        "task_template": "http://127.0.0.100:8008/api/task_template/10",
                        "task_template_id": 10
                    },
                    {
                        "id": 12,
                        "url": "http://127.0.0.100:8008/api/task_connector_type/12",
                        "dataformat": "http://127.0.0.100:8008/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://127.0.0.100:8008/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://127.0.0.100:8008/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://127.0.0.100:8008/api/role/any",
                        "role_value": "any",
                        "task_template": "http://127.0.0.100:8008/api/task_template/10",
                        "task_template_id": 10
                    }
                ],
                "consumed_by": [
                    "http://127.0.0.100:8008/api/task_relation_draft/30"
                ],
                "consumed_by_ids": [
                    30
                ],
                "created_at": "2022-05-03T08:40:25.688577",
                "description": "Preprocessing Pipeline for Target Observation target2, SAP001, 120-168 MHz, 1s, 16ch/sb",
                "duration": 0.0,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Pipeline target2",
                "output_pinned": false,
                "produced_by": [
                    "http://127.0.0.100:8008/api/task_relation_draft/26"
                ],
                "produced_by_ids": [
                    26
                ],
                "relative_start_time": 0.0,
                "relative_stop_time": 0.0,
                "scheduling_unit_draft": "http://127.0.0.100:8008/api/scheduling_unit_draft/7",
                "scheduling_unit_draft_id": 7,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Paaa+02/TP",
                "specifications_doc": {
                    "flag": {
                        "rfi_strategy": "HBAdefault",
                        "outerchannels": true,
                        "autocorrelations": true
                    },
                    "demix": {
                        "sources": [],
                        "time_steps": 10,
                        "ignore_target": false,
                        "frequency_steps": 64
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "average": {
                        "time_steps": 1,
                        "frequency_steps": 4
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    }
                },
                "specifications_template": (_.find(templates, {"name": "preprocessing pipeline"})),
                "specifications_template_id": (_.find(templates, {"name": "preprocessing pipeline"})).id,
                "tags": [],
                "task_blueprints": [],
                "task_blueprints_ids": [],
                "task_type": "pipeline",
                "updated_at": "2022-05-03T08:40:27.246310"
            }
        ],
        "task_drafts_ids": [
            21,
            22,
            25,
            28,
            27,
            23,
            24,
            26
        ],
        "updated_at": "2022-05-03T08:40:26.564170"
    },
    {
        "id": 9,
        "url": "http://localhost:3000/api/scheduling_unit_draft/9",
        "created_at": "2022-04-21T08:16:30.648088",
        "description": "Test scheduling unit",
        "duration": 120,
        "ingest_permission_required": true,
        "interrupts_telescope": false,
        "name": "BF Pulsar Timing",
        "observation_strategy_template": "http://localhost:3000/api/scheduling_unit_observing_strategy_template/9",
        "observation_strategy_template_id": 9,
        "piggyback_allowed_aartfaac": true,
        "piggyback_allowed_tbb": true,
        "priority_queue": "http://localhost:3000/api/priority_queue_type/A",
        "priority_queue_value": "A",
        "rank": 1,
        "scheduling_constraints_doc": {
            "sky": {
                "min_distance": {
                    "sun": 0.5,
                    "moon": 0.5,
                    "jupiter": 0.5
                },
                "min_elevation": {
                    "target": 0.5,
                    "calibrator": 0.5
                },
                "transit_offset": {
                    "to": 7200,
                    "from": -7200
                }
            },
            "time": {
                "at": "2022-04-22T02:52:06.166227"
            },
            "daily": {
                "require_day": false,
                "require_night": false,
                "avoid_twilight": false
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
            "scheduler": "fixed_time"
        },
        "scheduling_constraints_template": "http://localhost:3000/api/scheduling_constraints_template/1",
        "scheduling_constraints_template_id": 1,
        "scheduling_set": "http://localhost:3000/api/scheduling_set/2",
        "scheduling_set_id": 2,
        "scheduling_unit_blueprints": [
            "http://localhost:3000/api/scheduling_unit_blueprint/9"
        ],
        "scheduling_unit_blueprints_ids": [
            9
        ],
        "specifications_template": "http://localhost:3000/api/scheduling_unit_template/1",
        "specifications_template_id": 1,
        "tags": [],
        "task_drafts": [
            {
                "id": 49,
                "url": "http://localhost:3000/api/task_draft/49",
                "connector_types": [
                    {
                        "id": 6,
                        "url": "http://localhost:3000/api/task_connector_type/6",
                        "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://localhost:3000/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://localhost:3000/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://localhost:3000/api/role/beamformer",
                        "role_value": "beamformer",
                        "task_template": "http://localhost:3000/api/task_template/3",
                        "task_template_id": 3
                    }
                ],
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_draft/66",
                    "http://localhost:3000/api/task_relation_draft/69"
                ],
                "consumed_by_ids": [
                    66,
                    69
                ],
                "created_at": "2022-04-21T08:16:30.727162",
                "description": "Beamforming observation for default pulsar timing with HBA",
                "duration": 120,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Observation",
                "output_pinned": false,
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 0,
                "relative_stop_time": 120,
                "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/9",
                "scheduling_unit_draft_id": 9,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
                    "duration": 120,
                    "beamformer": {
                        "ppf": false,
                        "pipelines": [
                            {
                                "name": "Beamformer CV",
                                "coherent": {
                                    "SAPs": [
                                        {
                                            "name": "SAP0",
                                            "tabs": [
                                                {
                                                    "duration": 0,
                                                    "pointing": {
                                                        "angle1": 0,
                                                        "angle2": 0,
                                                        "target": "target1",
                                                        "direction_type": "J2000"
                                                    },
                                                    "relative": true,
                                                    "allow_after_previous": true
                                                }
                                            ],
                                            "subbands": {
                                                "list": [
                                                    51,
                                                    52,
                                                    53,
                                                    54,
                                                    55,
                                                    56,
                                                    57,
                                                    58,
                                                    59,
                                                    60,
                                                    61,
                                                    62,
                                                    63,
                                                    64,
                                                    65,
                                                    66,
                                                    67,
                                                    68,
                                                    69,
                                                    70,
                                                    71,
                                                    72,
                                                    73,
                                                    74,
                                                    75,
                                                    76,
                                                    77,
                                                    78,
                                                    79,
                                                    80,
                                                    81,
                                                    82,
                                                    83,
                                                    84,
                                                    85,
                                                    86,
                                                    87,
                                                    88,
                                                    89,
                                                    90,
                                                    91,
                                                    92,
                                                    93,
                                                    94,
                                                    95,
                                                    96,
                                                    97,
                                                    98,
                                                    99,
                                                    100,
                                                    101,
                                                    102,
                                                    103,
                                                    104,
                                                    105,
                                                    106,
                                                    107,
                                                    108,
                                                    109,
                                                    110,
                                                    111,
                                                    112,
                                                    113,
                                                    114,
                                                    115,
                                                    116,
                                                    117,
                                                    118,
                                                    119,
                                                    120,
                                                    121,
                                                    122,
                                                    123,
                                                    124,
                                                    125,
                                                    126,
                                                    127,
                                                    128,
                                                    129,
                                                    130,
                                                    131,
                                                    132,
                                                    133,
                                                    134,
                                                    135,
                                                    136,
                                                    137,
                                                    138,
                                                    139,
                                                    140,
                                                    141,
                                                    142,
                                                    143,
                                                    144,
                                                    145,
                                                    146,
                                                    147,
                                                    148,
                                                    149,
                                                    150,
                                                    151,
                                                    152,
                                                    153,
                                                    154,
                                                    155,
                                                    156,
                                                    157,
                                                    158,
                                                    159,
                                                    160,
                                                    161,
                                                    162,
                                                    163,
                                                    164,
                                                    165,
                                                    166,
                                                    167,
                                                    168,
                                                    169,
                                                    170,
                                                    171,
                                                    172,
                                                    173,
                                                    174,
                                                    175,
                                                    176,
                                                    177,
                                                    178,
                                                    179,
                                                    180,
                                                    181,
                                                    182,
                                                    183,
                                                    184,
                                                    185,
                                                    186,
                                                    187,
                                                    188,
                                                    189,
                                                    190,
                                                    191,
                                                    192,
                                                    193,
                                                    194,
                                                    195,
                                                    196,
                                                    197,
                                                    198,
                                                    199,
                                                    200,
                                                    201,
                                                    202,
                                                    203,
                                                    204,
                                                    205,
                                                    206,
                                                    207,
                                                    208,
                                                    209,
                                                    210,
                                                    211,
                                                    212,
                                                    213,
                                                    214,
                                                    215,
                                                    216,
                                                    217,
                                                    218,
                                                    219,
                                                    220,
                                                    221,
                                                    222,
                                                    223,
                                                    224,
                                                    225,
                                                    226,
                                                    227,
                                                    228,
                                                    229,
                                                    230,
                                                    231,
                                                    232,
                                                    233,
                                                    234,
                                                    235,
                                                    236,
                                                    237,
                                                    238,
                                                    239,
                                                    240,
                                                    241,
                                                    242,
                                                    243,
                                                    244,
                                                    245,
                                                    246,
                                                    247,
                                                    248,
                                                    249,
                                                    250,
                                                    251,
                                                    252,
                                                    253,
                                                    254,
                                                    255,
                                                    256,
                                                    257,
                                                    258,
                                                    259,
                                                    260,
                                                    261,
                                                    262,
                                                    263,
                                                    264,
                                                    265,
                                                    266,
                                                    267,
                                                    268,
                                                    269,
                                                    270,
                                                    271,
                                                    272,
                                                    273,
                                                    274,
                                                    275,
                                                    276,
                                                    277,
                                                    278,
                                                    279,
                                                    280,
                                                    281,
                                                    282,
                                                    283,
                                                    284,
                                                    285,
                                                    286,
                                                    287,
                                                    288,
                                                    289,
                                                    290,
                                                    291,
                                                    292,
                                                    293,
                                                    294,
                                                    295,
                                                    296,
                                                    297,
                                                    298,
                                                    299,
                                                    300,
                                                    301,
                                                    302,
                                                    303,
                                                    304,
                                                    305,
                                                    306,
                                                    307,
                                                    308,
                                                    309,
                                                    310,
                                                    311,
                                                    312,
                                                    313,
                                                    314,
                                                    315,
                                                    316,
                                                    317,
                                                    318,
                                                    319,
                                                    320,
                                                    321,
                                                    322,
                                                    323,
                                                    324,
                                                    325,
                                                    326,
                                                    327,
                                                    328,
                                                    329,
                                                    330,
                                                    331,
                                                    332,
                                                    333,
                                                    334,
                                                    335,
                                                    336,
                                                    337,
                                                    338,
                                                    339,
                                                    340,
                                                    341,
                                                    342,
                                                    343,
                                                    344,
                                                    345,
                                                    346,
                                                    347,
                                                    348,
                                                    349,
                                                    350,
                                                    351,
                                                    352,
                                                    353,
                                                    354,
                                                    355,
                                                    356,
                                                    357,
                                                    358,
                                                    359,
                                                    360,
                                                    361,
                                                    362,
                                                    363,
                                                    364,
                                                    365,
                                                    366,
                                                    367,
                                                    368,
                                                    369,
                                                    370,
                                                    371,
                                                    372,
                                                    373,
                                                    374,
                                                    375,
                                                    376,
                                                    377,
                                                    378,
                                                    379,
                                                    380,
                                                    381,
                                                    382,
                                                    383,
                                                    384,
                                                    385,
                                                    386,
                                                    387,
                                                    388,
                                                    389,
                                                    390,
                                                    391,
                                                    392,
                                                    393,
                                                    394,
                                                    395,
                                                    396,
                                                    397,
                                                    398,
                                                    399,
                                                    400,
                                                    401,
                                                    402,
                                                    403,
                                                    404,
                                                    405,
                                                    406,
                                                    407,
                                                    408,
                                                    409,
                                                    410,
                                                    411,
                                                    412,
                                                    413,
                                                    414,
                                                    415,
                                                    416,
                                                    417,
                                                    418,
                                                    419,
                                                    420,
                                                    421,
                                                    422,
                                                    423,
                                                    424,
                                                    425,
                                                    426,
                                                    427,
                                                    428,
                                                    429,
                                                    430,
                                                    431,
                                                    432,
                                                    433,
                                                    434,
                                                    435,
                                                    436,
                                                    437,
                                                    438,
                                                    439,
                                                    440,
                                                    441,
                                                    442,
                                                    443,
                                                    444,
                                                    445,
                                                    446,
                                                    447,
                                                    448,
                                                    449,
                                                    450
                                                ],
                                                "method": "copy"
                                            },
                                            "tab_rings": {
                                                "count": 0,
                                                "width": 0.01
                                            }
                                        }
                                    ],
                                    "settings": {
                                        "stokes": "XXYY",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 20,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "flys eye": {
                                    "enabled": false,
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "incoherent": {
                                    "SAPs": [],
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "station_groups": [
                                    {
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501"
                                        ],
                                        "max_nr_missing": 1
                                    }
                                ]
                            }
                        ]
                    },
                    "station_configuration": {
                        "SAPs": [
                            {
                                "name": "SAP0",
                                "subbands": [
                                    51,
                                    52,
                                    53,
                                    54,
                                    55,
                                    56,
                                    57,
                                    58,
                                    59,
                                    60,
                                    61,
                                    62,
                                    63,
                                    64,
                                    65,
                                    66,
                                    67,
                                    68,
                                    69,
                                    70,
                                    71,
                                    72,
                                    73,
                                    74,
                                    75,
                                    76,
                                    77,
                                    78,
                                    79,
                                    80,
                                    81,
                                    82,
                                    83,
                                    84,
                                    85,
                                    86,
                                    87,
                                    88,
                                    89,
                                    90,
                                    91,
                                    92,
                                    93,
                                    94,
                                    95,
                                    96,
                                    97,
                                    98,
                                    99,
                                    100,
                                    101,
                                    102,
                                    103,
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    137,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    164,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    181,
                                    182,
                                    183,
                                    184,
                                    185,
                                    186,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    210,
                                    211,
                                    212,
                                    213,
                                    214,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    241,
                                    242,
                                    243,
                                    244,
                                    245,
                                    246,
                                    247,
                                    248,
                                    249,
                                    250,
                                    251,
                                    252,
                                    253,
                                    254,
                                    255,
                                    256,
                                    257,
                                    258,
                                    259,
                                    260,
                                    261,
                                    262,
                                    263,
                                    264,
                                    265,
                                    266,
                                    267,
                                    268,
                                    269,
                                    270,
                                    271,
                                    272,
                                    273,
                                    274,
                                    275,
                                    276,
                                    277,
                                    278,
                                    279,
                                    280,
                                    281,
                                    282,
                                    283,
                                    284,
                                    285,
                                    286,
                                    287,
                                    288,
                                    289,
                                    290,
                                    291,
                                    292,
                                    293,
                                    294,
                                    295,
                                    296,
                                    297,
                                    298,
                                    299,
                                    300,
                                    301,
                                    302,
                                    303,
                                    304,
                                    305,
                                    306,
                                    307,
                                    308,
                                    309,
                                    310,
                                    311,
                                    312,
                                    313,
                                    314,
                                    315,
                                    316,
                                    317,
                                    318,
                                    319,
                                    320,
                                    321,
                                    322,
                                    323,
                                    324,
                                    325,
                                    326,
                                    327,
                                    328,
                                    329,
                                    330,
                                    331,
                                    332,
                                    333,
                                    334,
                                    335,
                                    336,
                                    337,
                                    338,
                                    339,
                                    340,
                                    341,
                                    342,
                                    343,
                                    344,
                                    345,
                                    346,
                                    347,
                                    348,
                                    349,
                                    350,
                                    351,
                                    352,
                                    353,
                                    354,
                                    355,
                                    356,
                                    357,
                                    358,
                                    359,
                                    360,
                                    361,
                                    362,
                                    363,
                                    364,
                                    365,
                                    366,
                                    367,
                                    368,
                                    369,
                                    370,
                                    371,
                                    372,
                                    373,
                                    374,
                                    375,
                                    376,
                                    377,
                                    378,
                                    379,
                                    380,
                                    381,
                                    382,
                                    383,
                                    384,
                                    385,
                                    386,
                                    387,
                                    388,
                                    389,
                                    390,
                                    391,
                                    392,
                                    393,
                                    394,
                                    395,
                                    396,
                                    397,
                                    398,
                                    399,
                                    400,
                                    401,
                                    402,
                                    403,
                                    404,
                                    405,
                                    406,
                                    407,
                                    408,
                                    409,
                                    410,
                                    411,
                                    412,
                                    413,
                                    414,
                                    415,
                                    416,
                                    417,
                                    418,
                                    419,
                                    420,
                                    421,
                                    422,
                                    423,
                                    424,
                                    425,
                                    426,
                                    427,
                                    428,
                                    429,
                                    430,
                                    431,
                                    432,
                                    433,
                                    434,
                                    435,
                                    436,
                                    437,
                                    438,
                                    439,
                                    440,
                                    441,
                                    442,
                                    443,
                                    444,
                                    445,
                                    446,
                                    447,
                                    448,
                                    449,
                                    450
                                ],
                                "digital_pointing": {
                                    "angle1": 0.92934186635,
                                    "angle2": 0.952579228492,
                                    "target": "target1",
                                    "direction_type": "J2000"
                                }
                            }
                        ],
                        "filter": "HBA_110_190",
                        "tile_beam": {
                            "angle1": 0.92934186635,
                            "angle2": 0.952579228492,
                            "target": "target1",
                            "direction_type": "J2000"
                        },
                        "antenna_set": "HBA_DUAL_INNER",
                        "station_groups": [
                            {
                                "stations": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                },
                "specifications_template": {
                    "id": 3,
                    "url": "http://localhost:3000/api/task_template/3",
                    "created_at": "2022-04-21T08:15:52.853823",
                    "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                    "name": "beamforming observation",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
                        "type": "object",
                        "title": "beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "beamformer",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "#/definitions/timedelta",
                                "title": "Duration",
                                "default": 300,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "#/definitions/beamformer",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "timedelta": {
                                "default": 0,
                                "description": "A time duration or delta expressed in seconds",
                                "type": "number"
                            },
                            "timestamp": {
                                "default": "1970-01-01T00:00:00Z",
                                "description": "A timestamp defined in UTC",
                                "format": "date-time",
                                "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                                "type": "string"
                            },
                            "beamformer": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "pipelines": {
                                        "additionalItems": false,
                                        "default": [
                                            {}
                                        ],
                                        "description": "This array of beamformer pipelines is currently restricted to exactly 1 item. Support for handling multiple pipelines (in cobalt) will be added in the future.",
                                        "items": {
                                            "$ref": "#/definitions/beamformer_pipeline"
                                        },
                                        "maxItems": 1,
                                        "minItems": 1,
                                        "title": "Pipelines",
                                        "type": "array"
                                    },
                                    "ppf": {
                                        "default": false,
                                        "description": "Use a PPF for the channellisation for beamforming. If not, an FFT is used.",
                                        "title": "Use PPF when beamforming",
                                        "type": "boolean"
                                    }
                                },
                                "required": [
                                    "pipelines",
                                    "ppf"
                                ],
                                "title": "Beamformer",
                                "type": "object"
                            },
                            "beamformer_pipeline": {
                                "additionalProperties": false,
                                "default": {},
                                "headerTemplate": "Beamformer Pipeline {{ self.index }}",
                                "properties": {
                                    "coherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to beamform.",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        },
                                                        "tab_rings": {
                                                            "additonalProperties": false,
                                                            "default": {},
                                                            "description": "Rings of TABs around the center of the beam.",
                                                            "properties": {
                                                                "count": {
                                                                    "default": 0,
                                                                    "maximum": 11,
                                                                    "minimum": 0,
                                                                    "title": "Number of rings",
                                                                    "type": "integer"
                                                                },
                                                                "width": {
                                                                    "default": 0.01,
                                                                    "description": "Distance between pointings.",
                                                                    "minimum": 0,
                                                                    "title": "Ring width",
                                                                    "type": "number"
                                                                }
                                                            },
                                                            "title": "Tied-Array Rings",
                                                            "type": "object"
                                                        },
                                                        "tabs": {
                                                            "additionalItems": false,
                                                            "default": [],
                                                            "description": "Tied-array beams to form",
                                                            "items": {
                                                                "additionalProperties": false,
                                                                "default": {},
                                                                "headerTemplate": "TAB {{ self.index }}",
                                                                "properties": {
                                                                    "allow_after_previous": {
                                                                        "default": true,
                                                                        "description": "This TAB can be recorded after the previous TAB, if duration allows.",
                                                                        "title": "Allow after previous",
                                                                        "type": "boolean"
                                                                    },
                                                                    "duration": {
                                                                        "default": 0,
                                                                        "description": "Exposure time of the TAB. 0 = observation duration.",
                                                                        "maximum": 0,
                                                                        "minimum": 0,
                                                                        "title": "Duration",
                                                                        "type": "integer"
                                                                    },
                                                                    "pointing": {
                                                                        "$ref": "#/definitions/pointing",
                                                                        "default": {}
                                                                    },
                                                                    "relative": {
                                                                        "default": false,
                                                                        "description": "Angular offset applied to the TAB pointing",
                                                                        "title": "Relative to SAP",
                                                                        "type": "boolean"
                                                                    }
                                                                },
                                                                "required": [
                                                                    "pointing",
                                                                    "relative"
                                                                ],
                                                                "title": "Tied-Array Beam",
                                                                "type": "object"
                                                            },
                                                            "title": "Tied-Array Beams",
                                                            "type": "array"
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "tabs"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "SAPs",
                                            "settings"
                                        ],
                                        "title": "Coherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "flys eye": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "description": "Produce beams containing the individual station signals",
                                        "properties": {
                                            "enabled": {
                                                "default": false,
                                                "title": "Enable Fly's Eye",
                                                "type": "boolean"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "enabled"
                                        ],
                                        "title": "Fly's Eye Settings",
                                        "type": "object"
                                    },
                                    "incoherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to create incoherent TABs for (empty list = all).",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "default": {},
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "subbands"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "settings",
                                            "SAPs"
                                        ],
                                        "title": "Incoherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "name": {
                                        "default": "_beamformer_name_",
                                        "description": "Beamformer name, used for identification purposes.",
                                        "minLength": 1,
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501"
                                                ]
                                            }
                                        ],
                                        "description": "While observing, COBALT will beamform on the intersection of all stations in this list and the used stations in the observation. So, specifying all possible stations here means that all observation-stations are used. Specifying a small subset here means that only the observing-stations in this small list are used. By default we let COBALT beamform on the Core stations.",
                                        "minItems": 1
                                    }
                                },
                                "required": [
                                    "name"
                                ],
                                "title": "Beamformer Pipeline",
                                "type": "object"
                            },
                            "subband_selection": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "list": {
                                        "additionalItems": false,
                                        "default": [],
                                        "description": "If method is 'select subset', only beamform these subbands, and only if they occur in the SAP.",
                                        "items": {
                                            "maximum": 511,
                                            "minimum": 0,
                                            "title": "Subband",
                                            "type": "integer"
                                        },
                                        "minItems": 0,
                                        "title": "Subset selection",
                                        "type": "array"
                                    },
                                    "method": {
                                        "default": "copy",
                                        "description": "How to select the subbands to beam form",
                                        "enum": [
                                            "copy",
                                            "largest continuous subset",
                                            "select subset"
                                        ],
                                        "title": "Method",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "method"
                                ],
                                "title": "Subband selection",
                                "type": "object"
                            },
                            "station_list": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [],
                                "items": {
                                    "$ref": "#/definitions/station"
                                },
                                "minItems": 0,
                                "type": "array",
                                "uniqueItems": true
                            },
                            "pointing": {
                                "additionalProperties": false,
                                "properties": {
                                    "angle1": {
                                        "default": 0.6624317181687094,
                                        "description": "First angle (e.g. RA)",
                                        "title": "Angle 1",
                                        "type": "number"
                                    },
                                    "angle2": {
                                        "default": 1.5579526427549426,
                                        "description": "Second angle (e.g. DEC)",
                                        "title": "Angle 2",
                                        "type": "number"
                                    },
                                    "direction_type": {
                                        "default": "J2000",
                                        "description": "",
                                        "enum": [
                                            "J2000",
                                            "AZELGEO",
                                            "LMN",
                                            "SUN",
                                            "MOON",
                                            "MERCURY",
                                            "VENUS",
                                            "MARS",
                                            "JUPITER",
                                            "SATURN",
                                            "URANUS",
                                            "NEPTUNE",
                                            "PLUTO"
                                        ],
                                        "title": "Reference frame",
                                        "type": "string"
                                    },
                                    "target": {
                                        "default": "_target_name_",
                                        "description": "Description of where this beam points at",
                                        "minLength": 1,
                                        "title": "Target",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "angle1",
                                    "angle2",
                                    "target"
                                ],
                                "type": "object"
                            },
                            "stokes_settings": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "channels_per_subband": {
                                        "default": 1,
                                        "description": "Number of frequency bands per subband",
                                        "enum": [
                                            1,
                                            8,
                                            16,
                                            32,
                                            64,
                                            128,
                                            256,
                                            512,
                                            1024
                                        ],
                                        "minimum": 1,
                                        "title": "Channels/subband",
                                        "type": "integer"
                                    },
                                    "quantisation": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "bits": {
                                                "default": 8,
                                                "description": "Number of bits for a single value",
                                                "enum": [
                                                    8
                                                ],
                                                "title": "Bits/sample",
                                                "type": "integer"
                                            },
                                            "enabled": {
                                                "default": false,
                                                "title": "Output quantisation into integers",
                                                "type": "boolean"
                                            },
                                            "scale_max": {
                                                "default": 5,
                                                "description": "Cut off values above this treshold",
                                                "title": "Maximum value",
                                                "type": "number"
                                            },
                                            "scale_min": {
                                                "default": -5,
                                                "description": "Cut off values below this treshold",
                                                "title": "Minimum value",
                                                "type": "number"
                                            }
                                        },
                                        "required": [
                                            "enabled",
                                            "bits",
                                            "scale_min",
                                            "scale_max"
                                        ],
                                        "title": "Output quantisation settings",
                                        "type": "object"
                                    },
                                    "stokes": {
                                        "$ref": "#/definitions/stokes",
                                        "default": "I"
                                    },
                                    "subbands_per_file": {
                                        "default": 488,
                                        "description": "The maximum number of subbands to write in each output dataproduct.",
                                        "maximum": 488,
                                        "minimum": 1,
                                        "title": "Subbands per file",
                                        "type": "integer"
                                    },
                                    "time_integration_factor": {
                                        "default": 1,
                                        "description": "The number of samples to integrate over",
                                        "maximum": 12288,
                                        "minimum": 1,
                                        "title": "Time integration",
                                        "type": "integer"
                                    }
                                },
                                "required": [
                                    "stokes",
                                    "time_integration_factor",
                                    "channels_per_subband"
                                ],
                                "type": "object"
                            },
                            "station": {
                                "description": "These are the LOFAR stations",
                                "enum": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501",
                                    "RS106",
                                    "RS205",
                                    "RS208",
                                    "RS210",
                                    "RS305",
                                    "RS306",
                                    "RS307",
                                    "RS310",
                                    "RS406",
                                    "RS407",
                                    "RS409",
                                    "RS503",
                                    "RS508",
                                    "RS509",
                                    "DE601",
                                    "DE602",
                                    "DE603",
                                    "DE604",
                                    "DE605",
                                    "FR606",
                                    "SE607",
                                    "UK608",
                                    "DE609",
                                    "PL610",
                                    "PL611",
                                    "PL612",
                                    "IE613",
                                    "LV614"
                                ],
                                "title": "Station",
                                "type": "string"
                            },
                            "max_number_of_missing_stations": {
                                "default": 0,
                                "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                                "minimum": 0,
                                "title": "Maximum number of stations to omit",
                                "type": "integer"
                            },
                            "stokes": {
                                "default": "I",
                                "description": "Which Stokes to produce",
                                "enum": [
                                    "I",
                                    "IQUV",
                                    "XXYY"
                                ],
                                "title": "Stokes",
                                "type": "string"
                            },
                            "station_groups": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [
                                    {
                                        "max_nr_missing": 1,
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ]
                                    }
                                ],
                                "description": "One or more predefined or custom groups of stations",
                                "items": {
                                    "$ref": "#/definitions/station_group"
                                },
                                "minItems": 1,
                                "title": "Station groups",
                                "type": "array"
                            },
                            "station_group": {
                                "anyOf": [
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007"
                                            ]
                                        },
                                        "description": "The group of all stations on the Superterp",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Superterp",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501"
                                            ]
                                        },
                                        "description": "The group of all Core stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Core",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch remote stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Remote",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch (Core + Remote) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Dutch",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 2,
                                            "stations": [
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 1,
                                            "stations": [
                                                "DE601",
                                                "DE605"
                                            ]
                                        },
                                        "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE605"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International required",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 6,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509",
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all (Core + Remote + International) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509",
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "All",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS001"
                                            ]
                                        },
                                        "description": "A custom group of stations which can be defined by the user",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list"
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Custom",
                                        "type": "object"
                                    }
                                ],
                                "default": {
                                    "max_nr_missing": 1,
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                },
                                "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                                "title": "Station group",
                                "type": "object"
                            },
                            "station_configuration": {
                                "default": {},
                                "properties": {
                                    "SAPs": {
                                        "$ref": "#/definitions/SAPs",
                                        "default": [
                                            {}
                                        ],
                                        "minItems": 1
                                    },
                                    "antenna_set": {
                                        "$ref": "#/definitions/antenna_set",
                                        "default": "HBA_DUAL"
                                    },
                                    "filter": {
                                        "$ref": "#/definitions/filter",
                                        "default": "HBA_110_190"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007"
                                                ]
                                            }
                                        ]
                                    },
                                    "tile_beam": {
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "description": "HBA only",
                                        "title": "Tile beam"
                                    }
                                },
                                "required": [
                                    "station_groups",
                                    "antenna_set",
                                    "filter",
                                    "SAPs"
                                ],
                                "title": "station_configuration",
                                "type": "object"
                            },
                            "filter": {
                                "default": "HBA_110_190",
                                "description": "Must match antenna type",
                                "enum": [
                                    "LBA_10_70",
                                    "LBA_30_70",
                                    "LBA_10_90",
                                    "LBA_30_90",
                                    "HBA_110_190",
                                    "HBA_210_250"
                                ],
                                "title": "Band-pass filter",
                                "type": "string"
                            },
                            "antenna_set": {
                                "default": "HBA_DUAL",
                                "description": "Fields & antennas to use",
                                "enum": [
                                    "HBA_DUAL",
                                    "HBA_DUAL_INNER",
                                    "HBA_ONE",
                                    "HBA_ONE_INNER",
                                    "HBA_ZERO",
                                    "HBA_ZERO_INNER",
                                    "LBA_INNER",
                                    "LBA_OUTER",
                                    "LBA_SPARSE_EVEN",
                                    "LBA_SPARSE_ODD",
                                    "LBA_ALL"
                                ],
                                "title": "Antenna set",
                                "type": "string"
                            },
                            "SAPs": {
                                "additionalItems": false,
                                "default": [
                                    {}
                                ],
                                "description": "Station beams",
                                "items": {
                                    "additionalProperties": false,
                                    "default": {},
                                    "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                    "properties": {
                                        "digital_pointing": {
                                            "$id": "#target_pointing",
                                            "$ref": "#/definitions/pointing",
                                            "default": {},
                                            "title": "Digital pointing"
                                        },
                                        "name": {
                                            "default": "_SAP_name_",
                                            "description": "Identifier for this beam",
                                            "minLength": 1,
                                            "title": "Name",
                                            "type": "string"
                                        },
                                        "subbands": {
                                            "additionalItems": false,
                                            "default": [
                                                255
                                            ],
                                            "items": {
                                                "maximum": 511,
                                                "minimum": 0,
                                                "title": "Subband",
                                                "type": "integer"
                                            },
                                            "maxItems": 488,
                                            "minItems": 1,
                                            "title": "Subband list",
                                            "type": "array"
                                        }
                                    },
                                    "required": [
                                        "name",
                                        "digital_pointing",
                                        "subbands"
                                    ],
                                    "title": "SAP",
                                    "type": "object"
                                },
                                "minItems": 0,
                                "title": "SAPs",
                                "type": "array"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2#",
                        "type": "object",
                        "title": "beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "beamformer",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                "title": "Duration",
                                "default": 300,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/2#/definitions/beamformer",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/observation",
                    "type_value": "observation",
                    "updated_at": "2022-04-21T08:15:52.853846",
                    "version": 2
                },
                "specifications_template_id": 3,
                "tags": [],
                "task_blueprints": [
                    "http://localhost:3000/api/task_blueprint/49"
                ],
                "task_blueprints_ids": [
                    49
                ],
                "task_type": "observation",
                "updated_at": "2022-04-21T08:16:30.727178"
            },
            {
                "id": 50,
                "url": "http://localhost:3000/api/task_draft/50",
                "connector_types": [
                    {
                        "id": 11,
                        "url": "http://localhost:3000/api/task_connector_type/11",
                        "dataformat": "http://localhost:3000/api/dataformat/pulp%20analysis",
                        "dataformat_value": "pulp analysis",
                        "datatype": "http://localhost:3000/api/datatype/pulsar%20profile",
                        "datatype_value": "pulsar profile",
                        "iotype": "http://localhost:3000/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/9",
                        "task_template_id": 9
                    },
                    {
                        "id": 10,
                        "url": "http://localhost:3000/api/task_connector_type/10",
                        "dataformat": "http://localhost:3000/api/dataformat/pulp%20summary",
                        "dataformat_value": "pulp summary",
                        "datatype": "http://localhost:3000/api/datatype/quality",
                        "datatype_value": "quality",
                        "iotype": "http://localhost:3000/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/9",
                        "task_template_id": 9
                    },
                    {
                        "id": 9,
                        "url": "http://localhost:3000/api/task_connector_type/9",
                        "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://localhost:3000/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/beamformer",
                        "role_value": "beamformer",
                        "task_template": "http://localhost:3000/api/task_template/9",
                        "task_template_id": 9
                    }
                ],
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_draft/67",
                    "http://localhost:3000/api/task_relation_draft/68",
                    "http://localhost:3000/api/task_relation_draft/70",
                    "http://localhost:3000/api/task_relation_draft/71"
                ],
                "consumed_by_ids": [
                    67,
                    68,
                    70,
                    71
                ],
                "created_at": "2022-04-21T08:16:30.745911",
                "description": "Pulsar Pipeline for default HBA pulsar timing observation",
                "duration": 0,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Pipeline",
                "output_pinned": false,
                "produced_by": [
                    "http://localhost:3000/api/task_relation_draft/66"
                ],
                "produced_by_ids": [
                    66
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/9",
                "scheduling_unit_draft_id": 9,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "dspsr": {
                        "digifil": {
                            "dm": 0,
                            "channels_per_part": 320,
                            "coherent_dedispersion": true,
                            "integration_time_factor": 4
                        },
                        "enabled": true,
                        "filterbank": {
                            "enabled": false,
                            "channels_per_part": 120,
                            "coherent_dedispersion": true
                        },
                        "rfi_excision": true,
                        "optimise_period_dm": true,
                        "subintegration_length": -1,
                        "single_pulse_subintegration": false
                    },
                    "output": {
                        "quantisation": {
                            "scale": 5,
                            "enabled": false
                        },
                        "dynamic_spectrum": {
                            "enabled": false,
                            "time_average": 0.5
                        }
                    },
                    "presto": {
                        "input": {
                            "nr_blocks": 100,
                            "decode_sigma": 3,
                            "samples_per_block": 8192
                        },
                        "rrats": {
                            "enabled": false,
                            "dm_range": 5
                        },
                        "rfifind": {
                            "blocks": 16
                        },
                        "prepdata": {
                            "dm": -1
                        },
                        "prepfold": false,
                        "fold_profile": true
                    },
                    "pulsar": {
                        "name": "",
                        "strategy": "meta"
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/pulsar%20pipeline/2/ref_resolved",
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    },
                    "single_pulse_search": false
                },
                "specifications_template": {
                    "id": 10,
                    "url": "http://localhost:3000/api/task_template/10",
                    "created_at": "2022-04-21T08:15:53.345870",
                    "description": "This schema defines the parameters for a pulsar pipeline.",
                    "name": "pulsar pipeline",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/pulsar%20pipeline/2/ref_resolved",
                        "type": "object",
                        "title": "pulsar pipeline",
                        "$schema": "http://json-schema.org/draft-07/schema#",
                        "version": 2,
                        "required": [
                            "pulsar",
                            "presto",
                            "dspsr",
                            "output"
                        ],
                        "properties": {
                            "dspsr": {
                                "type": "object",
                                "title": "DSPSR",
                                "default": {},
                                "properties": {
                                    "digifil": {
                                        "type": "object",
                                        "title": "Digifil",
                                        "default": {},
                                        "properties": {
                                            "dm": {
                                                "type": "number",
                                                "title": "DM",
                                                "default": -1,
                                                "minimum": -1,
                                                "desciption": "Dispersion Measure (-1 for none)"
                                            },
                                            "integration_time": {
                                                "type": "number",
                                                "title": "Integration time",
                                                "default": 4,
                                                "minimum": 0.1
                                            },
                                            "frequency_channels": {
                                                "type": "integer",
                                                "title": "Frequency channels",
                                                "default": 512,
                                                "maximum": 512,
                                                "minimum": 1,
                                                "description": "Number of frequency channels (multiple of subbands/part)"
                                            },
                                            "coherent_dedispersion": {
                                                "type": "boolean",
                                                "title": "Coherent Dedispersion",
                                                "default": true
                                            }
                                        },
                                        "description": "Processes single pulses in CV data if single-pulse search is enabled",
                                        "additionalProperties": false
                                    },
                                    "enabled": {
                                        "type": "boolean",
                                        "title": "Enabled",
                                        "default": true
                                    },
                                    "filterbank": {
                                        "type": "object",
                                        "title": "Create filter bank",
                                        "default": {},
                                        "properties": {
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            },
                                            "frequency_channels": {
                                                "type": "integer",
                                                "title": "Frequency channels",
                                                "default": 120,
                                                "maximum": 512,
                                                "minimum": 1,
                                                "description": "Number of frequency channels (multiple of subbands/part)"
                                            },
                                            "coherent_dedispersion": {
                                                "type": "boolean",
                                                "title": "Coherent Dedispersion",
                                                "default": true
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "rfi_excision": {
                                        "type": "boolean",
                                        "title": "RFI excision",
                                        "default": true,
                                        "description": "Excise/clean/remove detected RFI"
                                    },
                                    "optimise_period_dm": {
                                        "type": "boolean",
                                        "title": "Optimise period & DM",
                                        "default": true
                                    },
                                    "subintegration_length": {
                                        "type": "integer",
                                        "title": "Subintegration length",
                                        "default": -1,
                                        "minimum": -1
                                    },
                                    "single_pulse_subintegration": {
                                        "type": "boolean",
                                        "title": "Single pulse subintegration",
                                        "default": false,
                                        "description": "Create single-pulse subintegrations with inter-channel dispersion delays removed."
                                    }
                                },
                                "additionalProperties": false
                            },
                            "output": {
                                "type": "object",
                                "title": "Output",
                                "default": {},
                                "properties": {
                                    "quantisation": {
                                        "type": "object",
                                        "title": "Quantisation",
                                        "default": {},
                                        "properties": {
                                            "scale": {
                                                "type": "number",
                                                "title": "Conversion sigma",
                                                "default": 5,
                                                "minimum": 1,
                                                "description": "Conversion sigma to use when converting to 8-bit samples"
                                            },
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            }
                                        },
                                        "description": "Quantise output into 8-bit samples",
                                        "additionalProperties": false
                                    },
                                    "dynamic_spectrum": {
                                        "type": "object",
                                        "title": "Dynamic Spectrum",
                                        "default": {},
                                        "properties": {
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            },
                                            "time_average": {
                                                "type": "number",
                                                "title": "Time average",
                                                "default": 0.5,
                                                "minimum": 0.01
                                            }
                                        },
                                        "additionalProperties": false
                                    }
                                },
                                "additionalProperties": false
                            },
                            "presto": {
                                "type": "object",
                                "title": "PRESTO",
                                "default": {},
                                "properties": {
                                    "input": {
                                        "type": "object",
                                        "title": "Input",
                                        "properties": {
                                            "nr_blocks": {
                                                "type": "integer",
                                                "title": "Nr of blocks",
                                                "default": 100,
                                                "minimum": 1,
                                                "description": "Number of blocks to read at a time"
                                            },
                                            "decode_sigma": {
                                                "type": "number",
                                                "title": "Decode sigma",
                                                "default": 3,
                                                "minimum": 1,
                                                "description": "Sigma threshold for decoding"
                                            },
                                            "samples_per_block": {
                                                "type": "integer",
                                                "title": "Block size (samples)",
                                                "default": 8192,
                                                "minimum": 16
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "rrats": {
                                        "type": "object",
                                        "title": "RRATs analysis",
                                        "default": {},
                                        "properties": {
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            },
                                            "dm_range": {
                                                "type": "number",
                                                "title": "DM range",
                                                "default": 5,
                                                "minimum": 0
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "rfifind": {
                                        "type": "object",
                                        "title": "rfifind",
                                        "default": {},
                                        "properties": {
                                            "blocks": {
                                                "type": "integer",
                                                "title": "blocks",
                                                "default": 16,
                                                "minimum": 1
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "prepdata": {
                                        "type": "object",
                                        "title": "prepdata",
                                        "default": {},
                                        "properties": {
                                            "dm": {
                                                "type": "number",
                                                "title": "DM",
                                                "default": -1,
                                                "minimum": -1,
                                                "desciption": "Dispersion Measure (-1 for none)"
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "prepfold": {
                                        "type": "boolean",
                                        "title": "Enable prepfold",
                                        "default": true
                                    },
                                    "fold_profile": {
                                        "type": "boolean",
                                        "title": "Fold",
                                        "default": true,
                                        "description": "Fold the pulsar profile"
                                    }
                                },
                                "additionalProperties": false
                            },
                            "pulsar": {
                                "type": "object",
                                "title": "Pulsar to fold",
                                "default": {},
                                "properties": {
                                    "name": {
                                        "type": "string",
                                        "title": "Name",
                                        "default": "",
                                        "description": "Name of the pulsar to fold, if strategy=manual"
                                    },
                                    "strategy": {
                                        "enum": [
                                            "manual",
                                            "meta",
                                            "sapfind",
                                            "sapfind3",
                                            "tabfind",
                                            "tabfind+"
                                        ],
                                        "type": "string",
                                        "title": "Strategy",
                                        "default": "manual",
                                        "description": "How to look up the pulsar to fold",
                                        "enum_titles": [
                                            "Pulsar name specified here",
                                            "SAP target is pulsar name",
                                            "Brightest known pulsar in SAP",
                                            "Brightest 3 known pulsars in SAP",
                                            "Brightest known pulsar in TAB",
                                            "Brightest 3 known pulsars in TAB"
                                        ]
                                    }
                                },
                                "additionalProperties": false
                            },
                            "cluster_resources": {
                                "$ref": "#/definitions/cluster_resources",
                                "default": {}
                            },
                            "single_pulse_search": {
                                "type": "boolean",
                                "title": "Single-pulse search",
                                "default": false,
                                "description": "Instructs PRESTO to process single pulses, and enables Digifil for CV data"
                            }
                        },
                        "description": "This schema defines the parameters for a pulsar pipeline.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "cluster_resources": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Which cluster resources to claim for this pipeline.",
                                "properties": {
                                    "cores_per_task": {
                                        "default": 2,
                                        "description": "Number of cores to reserve for each process in the pipeline.",
                                        "maximum": 24,
                                        "minimum": 1,
                                        "title": "Cores/task",
                                        "type": "integer"
                                    },
                                    "parallel_tasks": {
                                        "default": 122,
                                        "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                        "maximum": 488,
                                        "minimum": 1,
                                        "title": "Max parallel tasks",
                                        "type": "integer"
                                    },
                                    "where": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "description": "Where to run this pipeline.",
                                        "properties": {
                                            "cluster": {
                                                "default": "CEP4",
                                                "enum": [
                                                    "CEP4"
                                                ],
                                                "title": "Cluster",
                                                "type": "string"
                                            },
                                            "partition": {
                                                "default": "cpu",
                                                "enum": [
                                                    "cpu",
                                                    "gpu",
                                                    "testing"
                                                ],
                                                "title": "Partition",
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "cluster",
                                            "partition"
                                        ],
                                        "title": "Where",
                                        "type": "object"
                                    }
                                },
                                "required": [
                                    "where",
                                    "cores_per_task",
                                    "parallel_tasks"
                                ],
                                "title": "Cluster resources",
                                "type": "object"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/pulsar%20pipeline/2#",
                        "type": "object",
                        "title": "pulsar pipeline",
                        "$schema": "http://json-schema.org/draft-07/schema#",
                        "version": 2,
                        "required": [
                            "pulsar",
                            "presto",
                            "dspsr",
                            "output"
                        ],
                        "properties": {
                            "dspsr": {
                                "type": "object",
                                "title": "DSPSR",
                                "default": {},
                                "properties": {
                                    "digifil": {
                                        "type": "object",
                                        "title": "Digifil",
                                        "default": {},
                                        "properties": {
                                            "dm": {
                                                "type": "number",
                                                "title": "DM",
                                                "default": -1,
                                                "minimum": -1,
                                                "desciption": "Dispersion Measure (-1 for none)"
                                            },
                                            "integration_time": {
                                                "type": "number",
                                                "title": "Integration time",
                                                "default": 4,
                                                "minimum": 0.1
                                            },
                                            "frequency_channels": {
                                                "type": "integer",
                                                "title": "Frequency channels",
                                                "default": 512,
                                                "maximum": 512,
                                                "minimum": 1,
                                                "description": "Number of frequency channels (multiple of subbands/part)"
                                            },
                                            "coherent_dedispersion": {
                                                "type": "boolean",
                                                "title": "Coherent Dedispersion",
                                                "default": true
                                            }
                                        },
                                        "description": "Processes single pulses in CV data if single-pulse search is enabled",
                                        "additionalProperties": false
                                    },
                                    "enabled": {
                                        "type": "boolean",
                                        "title": "Enabled",
                                        "default": true
                                    },
                                    "filterbank": {
                                        "type": "object",
                                        "title": "Create filter bank",
                                        "default": {},
                                        "properties": {
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            },
                                            "frequency_channels": {
                                                "type": "integer",
                                                "title": "Frequency channels",
                                                "default": 120,
                                                "maximum": 512,
                                                "minimum": 1,
                                                "description": "Number of frequency channels (multiple of subbands/part)"
                                            },
                                            "coherent_dedispersion": {
                                                "type": "boolean",
                                                "title": "Coherent Dedispersion",
                                                "default": true
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "rfi_excision": {
                                        "type": "boolean",
                                        "title": "RFI excision",
                                        "default": true,
                                        "description": "Excise/clean/remove detected RFI"
                                    },
                                    "optimise_period_dm": {
                                        "type": "boolean",
                                        "title": "Optimise period & DM",
                                        "default": true
                                    },
                                    "subintegration_length": {
                                        "type": "integer",
                                        "title": "Subintegration length",
                                        "default": -1,
                                        "minimum": -1
                                    },
                                    "single_pulse_subintegration": {
                                        "type": "boolean",
                                        "title": "Single pulse subintegration",
                                        "default": false,
                                        "description": "Create single-pulse subintegrations with inter-channel dispersion delays removed."
                                    }
                                },
                                "additionalProperties": false
                            },
                            "output": {
                                "type": "object",
                                "title": "Output",
                                "default": {},
                                "properties": {
                                    "quantisation": {
                                        "type": "object",
                                        "title": "Quantisation",
                                        "default": {},
                                        "properties": {
                                            "scale": {
                                                "type": "number",
                                                "title": "Conversion sigma",
                                                "default": 5,
                                                "minimum": 1,
                                                "description": "Conversion sigma to use when converting to 8-bit samples"
                                            },
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            }
                                        },
                                        "description": "Quantise output into 8-bit samples",
                                        "additionalProperties": false
                                    },
                                    "dynamic_spectrum": {
                                        "type": "object",
                                        "title": "Dynamic Spectrum",
                                        "default": {},
                                        "properties": {
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            },
                                            "time_average": {
                                                "type": "number",
                                                "title": "Time average",
                                                "default": 0.5,
                                                "minimum": 0.01
                                            }
                                        },
                                        "additionalProperties": false
                                    }
                                },
                                "additionalProperties": false
                            },
                            "presto": {
                                "type": "object",
                                "title": "PRESTO",
                                "default": {},
                                "properties": {
                                    "input": {
                                        "type": "object",
                                        "title": "Input",
                                        "properties": {
                                            "nr_blocks": {
                                                "type": "integer",
                                                "title": "Nr of blocks",
                                                "default": 100,
                                                "minimum": 1,
                                                "description": "Number of blocks to read at a time"
                                            },
                                            "decode_sigma": {
                                                "type": "number",
                                                "title": "Decode sigma",
                                                "default": 3,
                                                "minimum": 1,
                                                "description": "Sigma threshold for decoding"
                                            },
                                            "samples_per_block": {
                                                "type": "integer",
                                                "title": "Block size (samples)",
                                                "default": 8192,
                                                "minimum": 16
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "rrats": {
                                        "type": "object",
                                        "title": "RRATs analysis",
                                        "default": {},
                                        "properties": {
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            },
                                            "dm_range": {
                                                "type": "number",
                                                "title": "DM range",
                                                "default": 5,
                                                "minimum": 0
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "rfifind": {
                                        "type": "object",
                                        "title": "rfifind",
                                        "default": {},
                                        "properties": {
                                            "blocks": {
                                                "type": "integer",
                                                "title": "blocks",
                                                "default": 16,
                                                "minimum": 1
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "prepdata": {
                                        "type": "object",
                                        "title": "prepdata",
                                        "default": {},
                                        "properties": {
                                            "dm": {
                                                "type": "number",
                                                "title": "DM",
                                                "default": -1,
                                                "minimum": -1,
                                                "desciption": "Dispersion Measure (-1 for none)"
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "prepfold": {
                                        "type": "boolean",
                                        "title": "Enable prepfold",
                                        "default": true
                                    },
                                    "fold_profile": {
                                        "type": "boolean",
                                        "title": "Fold",
                                        "default": true,
                                        "description": "Fold the pulsar profile"
                                    }
                                },
                                "additionalProperties": false
                            },
                            "pulsar": {
                                "type": "object",
                                "title": "Pulsar to fold",
                                "default": {},
                                "properties": {
                                    "name": {
                                        "type": "string",
                                        "title": "Name",
                                        "default": "",
                                        "description": "Name of the pulsar to fold, if strategy=manual"
                                    },
                                    "strategy": {
                                        "enum": [
                                            "manual",
                                            "meta",
                                            "sapfind",
                                            "sapfind3",
                                            "tabfind",
                                            "tabfind+"
                                        ],
                                        "type": "string",
                                        "title": "Strategy",
                                        "default": "manual",
                                        "description": "How to look up the pulsar to fold",
                                        "enum_titles": [
                                            "Pulsar name specified here",
                                            "SAP target is pulsar name",
                                            "Brightest known pulsar in SAP",
                                            "Brightest 3 known pulsars in SAP",
                                            "Brightest known pulsar in TAB",
                                            "Brightest 3 known pulsars in TAB"
                                        ]
                                    }
                                },
                                "additionalProperties": false
                            },
                            "cluster_resources": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                                "default": {}
                            },
                            "single_pulse_search": {
                                "type": "boolean",
                                "title": "Single-pulse search",
                                "default": false,
                                "description": "Instructs PRESTO to process single pulses, and enables Digifil for CV data"
                            }
                        },
                        "description": "This schema defines the parameters for a pulsar pipeline.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/pipeline",
                    "type_value": "pipeline",
                    "updated_at": "2022-04-21T08:15:53.345958",
                    "version": 2
                },
                "specifications_template_id": 9,
                "tags": [],
                "task_blueprints": [
                    "http://localhost:3000/api/task_blueprint/50"
                ],
                "task_blueprints_ids": [
                    50
                ],
                "task_type": "pipeline",
                "updated_at": "2022-04-21T08:16:30.745924"
            },
            {
                "id": 51,
                "url": "http://localhost:3000/api/task_draft/51",
                "connector_types": [
                    {
                        "id": 48,
                        "url": "http://localhost:3000/api/task_connector_type/48",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/correlator",
                        "role_value": "correlator",
                        "task_template": "http://localhost:3000/api/task_template/1",
                        "task_template_id": 1
                    },
                    {
                        "id": 43,
                        "url": "http://localhost:3000/api/task_connector_type/43",
                        "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://localhost:3000/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/1",
                        "task_template_id": 1
                    },
                    {
                        "id": 42,
                        "url": "http://localhost:3000/api/task_connector_type/42",
                        "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://localhost:3000/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/beamformer",
                        "role_value": "beamformer",
                        "task_template": "http://localhost:3000/api/task_template/1",
                        "task_template_id": 1
                    },
                    {
                        "id": 40,
                        "url": "http://localhost:3000/api/task_connector_type/40",
                        "dataformat": "http://localhost:3000/api/dataformat/pulp%20summary",
                        "dataformat_value": "pulp summary",
                        "datatype": "http://localhost:3000/api/datatype/quality",
                        "datatype_value": "quality",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/1",
                        "task_template_id": 1
                    },
                    {
                        "id": 38,
                        "url": "http://localhost:3000/api/task_connector_type/38",
                        "dataformat": "http://localhost:3000/api/dataformat/pulp%20analysis",
                        "dataformat_value": "pulp analysis",
                        "datatype": "http://localhost:3000/api/datatype/pulsar%20profile",
                        "datatype_value": "pulsar profile",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/1",
                        "task_template_id": 1
                    },
                    {
                        "id": 36,
                        "url": "http://localhost:3000/api/task_connector_type/36",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/1",
                        "task_template_id": 1
                    }
                ],
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-04-21T08:16:30.756024",
                "description": "Cleanup all dataproducts from disk",
                "duration": 0,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Cleanup",
                "output_pinned": false,
                "produced_by": [
                    "http://localhost:3000/api/task_relation_draft/69",
                    "http://localhost:3000/api/task_relation_draft/70",
                    "http://localhost:3000/api/task_relation_draft/71"
                ],
                "produced_by_ids": [
                    69,
                    70,
                    71
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/9",
                "scheduling_unit_draft_id": 9,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2/ref_resolved"
                },
                "specifications_template": {
                    "id": 1,
                    "url": "http://localhost:3000/api/task_template/1",
                    "created_at": "2022-04-21T08:15:52.702755",
                    "description": "This schema defines the parameters to setup a dataproduct(s) cleanup task.",
                    "name": "cleanup",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2/ref_resolved",
                        "type": "object",
                        "title": "cleanup",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [],
                        "properties": {},
                        "description": "This schema defines the parameters to setup a dataproduct(s) cleanup task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {}
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2#",
                        "type": "object",
                        "title": "cleanup",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [],
                        "properties": {},
                        "description": "This schema defines the parameters to setup a dataproduct(s) cleanup task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/cleanup",
                    "type_value": "cleanup",
                    "updated_at": "2022-04-21T08:15:52.702788",
                    "version": 2
                },
                "specifications_template_id": 1,
                "tags": [],
                "task_blueprints": [
                    "http://localhost:3000/api/task_blueprint/51"
                ],
                "task_blueprints_ids": [
                    51
                ],
                "task_type": "cleanup",
                "updated_at": "2022-04-21T08:16:30.756038"
            },
            {
                "id": 52,
                "url": "http://localhost:3000/api/task_draft/52",
                "connector_types": [
                    {
                        "id": 26,
                        "url": "http://localhost:3000/api/task_connector_type/26",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/correlator",
                        "role_value": "correlator",
                        "task_template": "http://localhost:3000/api/task_template/2",
                        "task_template_id": 2
                    },
                    {
                        "id": 21,
                        "url": "http://localhost:3000/api/task_connector_type/21",
                        "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://localhost:3000/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/2",
                        "task_template_id": 2
                    },
                    {
                        "id": 20,
                        "url": "http://localhost:3000/api/task_connector_type/20",
                        "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://localhost:3000/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/beamformer",
                        "role_value": "beamformer",
                        "task_template": "http://localhost:3000/api/task_template/2",
                        "task_template_id": 2
                    },
                    {
                        "id": 18,
                        "url": "http://localhost:3000/api/task_connector_type/18",
                        "dataformat": "http://localhost:3000/api/dataformat/pulp%20summary",
                        "dataformat_value": "pulp summary",
                        "datatype": "http://localhost:3000/api/datatype/quality",
                        "datatype_value": "quality",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/2",
                        "task_template_id": 2
                    },
                    {
                        "id": 16,
                        "url": "http://localhost:3000/api/task_connector_type/16",
                        "dataformat": "http://localhost:3000/api/dataformat/pulp%20analysis",
                        "dataformat_value": "pulp analysis",
                        "datatype": "http://localhost:3000/api/datatype/pulsar%20profile",
                        "datatype_value": "pulsar profile",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/2",
                        "task_template_id": 2
                    },
                    {
                        "id": 14,
                        "url": "http://localhost:3000/api/task_connector_type/14",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/input",
                        "iotype_value": "input",
                        "role": "http://localhost:3000/api/role/any",
                        "role_value": "any",
                        "task_template": "http://localhost:3000/api/task_template/2",
                        "task_template_id": 2
                    }
                ],
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-04-21T08:16:30.766446",
                "description": "Ingest the pipeline outputs dataproducts",
                "duration": 0,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Ingest",
                "output_pinned": false,
                "produced_by": [
                    "http://localhost:3000/api/task_relation_draft/67",
                    "http://localhost:3000/api/task_relation_draft/68"
                ],
                "produced_by_ids": [
                    67,
                    68
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/9",
                "scheduling_unit_draft_id": 9,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
                },
                "specifications_template": {
                    "id": 2,
                    "url": "http://localhost:3000/api/task_template/2",
                    "created_at": "2022-04-21T08:15:52.742462",
                    "description": "This schema defines the parameters to setup an ingest task.",
                    "name": "ingest",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved",
                        "type": "object",
                        "title": "ingest",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [],
                        "properties": {},
                        "description": "This schema defines the parameters to setup an ingest task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {}
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2#",
                        "type": "object",
                        "title": "ingest",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [],
                        "properties": {},
                        "description": "This schema defines the parameters to setup an ingest task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/ingest",
                    "type_value": "ingest",
                    "updated_at": "2022-04-21T08:15:52.742474",
                    "version": 2
                },
                "specifications_template_id": 2,
                "tags": [],
                "task_blueprints": [
                    "http://localhost:3000/api/task_blueprint/52"
                ],
                "task_blueprints_ids": [
                    52
                ],
                "task_type": "ingest",
                "updated_at": "2022-04-21T08:16:30.766460"
            }
        ],
        "task_drafts_ids": [
            49,
            50,
            51,
            52
        ],
        "updated_at": "2022-04-21T08:16:30.648104"
    },
    {
        "id": 10,
        "url": "http://localhost:3000/api/scheduling_unit_draft/10",
        "created_at": "2022-04-21T08:16:32.564871",
        "description": "Test scheduling unit",
        "duration": 120,
        "ingest_permission_required": true,
        "interrupts_telescope": false,
        "name": "Simple Beamforming Observation",
        "observation_strategy_template": "http://localhost:3000/api/scheduling_unit_observing_strategy_template/10",
        "observation_strategy_template_id": 10,
        "piggyback_allowed_aartfaac": true,
        "piggyback_allowed_tbb": true,
        "priority_queue": "http://localhost:3000/api/priority_queue_type/A",
        "priority_queue_value": "A",
        "rank": 1,
        "scheduling_constraints_doc": {
            "sky": {
                "min_distance": {
                    "sun": 0.5,
                    "moon": 0.5,
                    "jupiter": 0.5
                },
                "min_elevation": {
                    "target": 0.5,
                    "calibrator": 0.5
                },
                "transit_offset": {
                    "to": 7200,
                    "from": -7200
                }
            },
            "time": {
                "at": "2022-04-22T03:06:06.166227"
            },
            "daily": {
                "require_day": false,
                "require_night": false,
                "avoid_twilight": false
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
            "scheduler": "fixed_time"
        },
        "scheduling_constraints_template": "http://localhost:3000/api/scheduling_constraints_template/1",
        "scheduling_constraints_template_id": 1,
        "scheduling_set": "http://localhost:3000/api/scheduling_set/1",
        "scheduling_set_id": 1,
        "scheduling_unit_blueprints": [
            "http://localhost:3000/api/scheduling_unit_blueprint/10"
        ],
        "scheduling_unit_blueprints_ids": [
            10
        ],
        "specifications_template": "http://localhost:3000/api/scheduling_unit_template/1",
        "specifications_template_id": 1,
        "tags": [],
        "task_drafts": [
            {
                "id": 53,
                "url": "http://localhost:3000/api/task_draft/53",
                "connector_types": [
                    {
                        "id": 6,
                        "url": "http://localhost:3000/api/task_connector_type/6",
                        "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://localhost:3000/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://localhost:3000/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://localhost:3000/api/role/beamformer",
                        "role_value": "beamformer",
                        "task_template": "http://localhost:3000/api/task_template/3",
                        "task_template_id": 3
                    }
                ],
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-04-21T08:16:32.621923",
                "description": "A simple short test beamforming observation",
                "duration": 120,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Observation",
                "output_pinned": false,
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 0,
                "relative_stop_time": 120,
                "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/10",
                "scheduling_unit_draft_id": 10,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
                    "duration": 120,
                    "beamformer": {
                        "ppf": false,
                        "pipelines": [
                            {
                                "name": "B0329+54",
                                "coherent": {
                                    "SAPs": [
                                        {
                                            "name": "B0329+54",
                                            "tabs": [
                                                {
                                                    "duration": 0,
                                                    "pointing": {
                                                        "angle1": 0.92934186635,
                                                        "angle2": 0.952579228492,
                                                        "target": "target1",
                                                        "direction_type": "J2000"
                                                    },
                                                    "relative": false,
                                                    "allow_after_previous": true
                                                }
                                            ],
                                            "subbands": {
                                                "list": [
                                                    0,
                                                    1,
                                                    2,
                                                    3,
                                                    4,
                                                    5,
                                                    6,
                                                    7,
                                                    8,
                                                    9,
                                                    10,
                                                    11,
                                                    12,
                                                    13,
                                                    14,
                                                    15,
                                                    16,
                                                    17,
                                                    18,
                                                    19,
                                                    20,
                                                    21,
                                                    22,
                                                    23,
                                                    24,
                                                    25,
                                                    26,
                                                    27,
                                                    28,
                                                    29,
                                                    30,
                                                    31,
                                                    32,
                                                    33,
                                                    34,
                                                    35,
                                                    36,
                                                    37,
                                                    38,
                                                    39,
                                                    40,
                                                    41,
                                                    42,
                                                    43,
                                                    44,
                                                    45,
                                                    46,
                                                    47,
                                                    48,
                                                    49,
                                                    50,
                                                    51,
                                                    52,
                                                    53,
                                                    54,
                                                    55,
                                                    56,
                                                    57,
                                                    58,
                                                    59,
                                                    60,
                                                    61,
                                                    62,
                                                    63,
                                                    64,
                                                    65,
                                                    66,
                                                    67,
                                                    68,
                                                    69,
                                                    70,
                                                    71,
                                                    72,
                                                    73,
                                                    74,
                                                    75,
                                                    76,
                                                    77,
                                                    78,
                                                    79,
                                                    80,
                                                    81,
                                                    82,
                                                    83,
                                                    84,
                                                    85,
                                                    86,
                                                    87,
                                                    88,
                                                    89,
                                                    90,
                                                    91,
                                                    92,
                                                    93,
                                                    94,
                                                    95,
                                                    96,
                                                    97,
                                                    98,
                                                    99,
                                                    100,
                                                    101,
                                                    102,
                                                    103,
                                                    104,
                                                    105,
                                                    106,
                                                    107,
                                                    108,
                                                    109,
                                                    110,
                                                    111,
                                                    112,
                                                    113,
                                                    114,
                                                    115,
                                                    116,
                                                    117,
                                                    118,
                                                    119,
                                                    120,
                                                    121,
                                                    122,
                                                    123,
                                                    124,
                                                    125,
                                                    126,
                                                    127,
                                                    128,
                                                    129,
                                                    130,
                                                    131,
                                                    132,
                                                    133,
                                                    134,
                                                    135,
                                                    136,
                                                    137,
                                                    138,
                                                    139,
                                                    140,
                                                    141,
                                                    142,
                                                    143,
                                                    144,
                                                    145,
                                                    146,
                                                    147,
                                                    148,
                                                    149,
                                                    150,
                                                    151,
                                                    152,
                                                    153,
                                                    154,
                                                    155,
                                                    156,
                                                    157,
                                                    158,
                                                    159,
                                                    160,
                                                    161,
                                                    162,
                                                    163,
                                                    164,
                                                    165,
                                                    166,
                                                    167,
                                                    168,
                                                    169,
                                                    170,
                                                    171,
                                                    172,
                                                    173,
                                                    174,
                                                    175,
                                                    176,
                                                    177,
                                                    178,
                                                    179,
                                                    180,
                                                    181,
                                                    182,
                                                    183,
                                                    184,
                                                    185,
                                                    186,
                                                    187,
                                                    188,
                                                    189,
                                                    190,
                                                    191,
                                                    192,
                                                    193,
                                                    194,
                                                    195,
                                                    196,
                                                    197,
                                                    198,
                                                    199,
                                                    200,
                                                    201,
                                                    202,
                                                    203,
                                                    204,
                                                    205,
                                                    206,
                                                    207,
                                                    208,
                                                    209,
                                                    210,
                                                    211,
                                                    212,
                                                    213,
                                                    214,
                                                    215,
                                                    216,
                                                    217,
                                                    218,
                                                    219,
                                                    220,
                                                    221,
                                                    222,
                                                    223,
                                                    224,
                                                    225,
                                                    226,
                                                    227,
                                                    228,
                                                    229,
                                                    230,
                                                    231,
                                                    232,
                                                    233,
                                                    234,
                                                    235,
                                                    236,
                                                    237,
                                                    238,
                                                    239,
                                                    240,
                                                    241,
                                                    242,
                                                    243
                                                ],
                                                "method": "copy"
                                            },
                                            "tab_rings": {
                                                "count": 0,
                                                "width": 0.01
                                            }
                                        }
                                    ],
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "flys eye": {
                                    "enabled": false,
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "incoherent": {
                                    "SAPs": [],
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "station_groups": [
                                    {
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ],
                                        "max_nr_missing": 1
                                    }
                                ]
                            }
                        ]
                    },
                    "station_configuration": {
                        "SAPs": [
                            {
                                "name": "B0329+54",
                                "subbands": [
                                    0,
                                    1,
                                    2,
                                    3,
                                    4,
                                    5,
                                    6,
                                    7,
                                    8,
                                    9,
                                    10,
                                    11,
                                    12,
                                    13,
                                    14,
                                    15,
                                    16,
                                    17,
                                    18,
                                    19,
                                    20,
                                    21,
                                    22,
                                    23,
                                    24,
                                    25,
                                    26,
                                    27,
                                    28,
                                    29,
                                    30,
                                    31,
                                    32,
                                    33,
                                    34,
                                    35,
                                    36,
                                    37,
                                    38,
                                    39,
                                    40,
                                    41,
                                    42,
                                    43,
                                    44,
                                    45,
                                    46,
                                    47,
                                    48,
                                    49,
                                    50,
                                    51,
                                    52,
                                    53,
                                    54,
                                    55,
                                    56,
                                    57,
                                    58,
                                    59,
                                    60,
                                    61,
                                    62,
                                    63,
                                    64,
                                    65,
                                    66,
                                    67,
                                    68,
                                    69,
                                    70,
                                    71,
                                    72,
                                    73,
                                    74,
                                    75,
                                    76,
                                    77,
                                    78,
                                    79,
                                    80,
                                    81,
                                    82,
                                    83,
                                    84,
                                    85,
                                    86,
                                    87,
                                    88,
                                    89,
                                    90,
                                    91,
                                    92,
                                    93,
                                    94,
                                    95,
                                    96,
                                    97,
                                    98,
                                    99,
                                    100,
                                    101,
                                    102,
                                    103,
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    137,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    164,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    181,
                                    182,
                                    183,
                                    184,
                                    185,
                                    186,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    210,
                                    211,
                                    212,
                                    213,
                                    214,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    241,
                                    242,
                                    243
                                ],
                                "digital_pointing": {
                                    "angle1": 0.92934186635,
                                    "angle2": 0.952579228492,
                                    "target": "target1",
                                    "direction_type": "J2000"
                                }
                            }
                        ],
                        "filter": "HBA_110_190",
                        "tile_beam": {
                            "angle1": 0.92934186635,
                            "angle2": 0.952579228492,
                            "target": "target1",
                            "direction_type": "J2000"
                        },
                        "antenna_set": "HBA_DUAL_INNER",
                        "station_groups": [
                            {
                                "stations": [
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                },
                "specifications_template": {
                    "id": 3,
                    "url": "http://localhost:3000/api/task_template/3",
                    "created_at": "2022-04-21T08:15:52.853823",
                    "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                    "name": "beamforming observation",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
                        "type": "object",
                        "title": "beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "beamformer",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "#/definitions/timedelta",
                                "title": "Duration",
                                "default": 300,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "#/definitions/beamformer",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "timedelta": {
                                "default": 0,
                                "description": "A time duration or delta expressed in seconds",
                                "type": "number"
                            },
                            "timestamp": {
                                "default": "1970-01-01T00:00:00Z",
                                "description": "A timestamp defined in UTC",
                                "format": "date-time",
                                "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                                "type": "string"
                            },
                            "beamformer": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "pipelines": {
                                        "additionalItems": false,
                                        "default": [
                                            {}
                                        ],
                                        "description": "This array of beamformer pipelines is currently restricted to exactly 1 item. Support for handling multiple pipelines (in cobalt) will be added in the future.",
                                        "items": {
                                            "$ref": "#/definitions/beamformer_pipeline"
                                        },
                                        "maxItems": 1,
                                        "minItems": 1,
                                        "title": "Pipelines",
                                        "type": "array"
                                    },
                                    "ppf": {
                                        "default": false,
                                        "description": "Use a PPF for the channellisation for beamforming. If not, an FFT is used.",
                                        "title": "Use PPF when beamforming",
                                        "type": "boolean"
                                    }
                                },
                                "required": [
                                    "pipelines",
                                    "ppf"
                                ],
                                "title": "Beamformer",
                                "type": "object"
                            },
                            "beamformer_pipeline": {
                                "additionalProperties": false,
                                "default": {},
                                "headerTemplate": "Beamformer Pipeline {{ self.index }}",
                                "properties": {
                                    "coherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to beamform.",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        },
                                                        "tab_rings": {
                                                            "additonalProperties": false,
                                                            "default": {},
                                                            "description": "Rings of TABs around the center of the beam.",
                                                            "properties": {
                                                                "count": {
                                                                    "default": 0,
                                                                    "maximum": 11,
                                                                    "minimum": 0,
                                                                    "title": "Number of rings",
                                                                    "type": "integer"
                                                                },
                                                                "width": {
                                                                    "default": 0.01,
                                                                    "description": "Distance between pointings.",
                                                                    "minimum": 0,
                                                                    "title": "Ring width",
                                                                    "type": "number"
                                                                }
                                                            },
                                                            "title": "Tied-Array Rings",
                                                            "type": "object"
                                                        },
                                                        "tabs": {
                                                            "additionalItems": false,
                                                            "default": [],
                                                            "description": "Tied-array beams to form",
                                                            "items": {
                                                                "additionalProperties": false,
                                                                "default": {},
                                                                "headerTemplate": "TAB {{ self.index }}",
                                                                "properties": {
                                                                    "allow_after_previous": {
                                                                        "default": true,
                                                                        "description": "This TAB can be recorded after the previous TAB, if duration allows.",
                                                                        "title": "Allow after previous",
                                                                        "type": "boolean"
                                                                    },
                                                                    "duration": {
                                                                        "default": 0,
                                                                        "description": "Exposure time of the TAB. 0 = observation duration.",
                                                                        "maximum": 0,
                                                                        "minimum": 0,
                                                                        "title": "Duration",
                                                                        "type": "integer"
                                                                    },
                                                                    "pointing": {
                                                                        "$ref": "#/definitions/pointing",
                                                                        "default": {}
                                                                    },
                                                                    "relative": {
                                                                        "default": false,
                                                                        "description": "Angular offset applied to the TAB pointing",
                                                                        "title": "Relative to SAP",
                                                                        "type": "boolean"
                                                                    }
                                                                },
                                                                "required": [
                                                                    "pointing",
                                                                    "relative"
                                                                ],
                                                                "title": "Tied-Array Beam",
                                                                "type": "object"
                                                            },
                                                            "title": "Tied-Array Beams",
                                                            "type": "array"
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "tabs"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "SAPs",
                                            "settings"
                                        ],
                                        "title": "Coherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "flys eye": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "description": "Produce beams containing the individual station signals",
                                        "properties": {
                                            "enabled": {
                                                "default": false,
                                                "title": "Enable Fly's Eye",
                                                "type": "boolean"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "enabled"
                                        ],
                                        "title": "Fly's Eye Settings",
                                        "type": "object"
                                    },
                                    "incoherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to create incoherent TABs for (empty list = all).",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "default": {},
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "subbands"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "settings",
                                            "SAPs"
                                        ],
                                        "title": "Incoherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "name": {
                                        "default": "_beamformer_name_",
                                        "description": "Beamformer name, used for identification purposes.",
                                        "minLength": 1,
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501"
                                                ]
                                            }
                                        ],
                                        "description": "While observing, COBALT will beamform on the intersection of all stations in this list and the used stations in the observation. So, specifying all possible stations here means that all observation-stations are used. Specifying a small subset here means that only the observing-stations in this small list are used. By default we let COBALT beamform on the Core stations.",
                                        "minItems": 1
                                    }
                                },
                                "required": [
                                    "name"
                                ],
                                "title": "Beamformer Pipeline",
                                "type": "object"
                            },
                            "subband_selection": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "list": {
                                        "additionalItems": false,
                                        "default": [],
                                        "description": "If method is 'select subset', only beamform these subbands, and only if they occur in the SAP.",
                                        "items": {
                                            "maximum": 511,
                                            "minimum": 0,
                                            "title": "Subband",
                                            "type": "integer"
                                        },
                                        "minItems": 0,
                                        "title": "Subset selection",
                                        "type": "array"
                                    },
                                    "method": {
                                        "default": "copy",
                                        "description": "How to select the subbands to beam form",
                                        "enum": [
                                            "copy",
                                            "largest continuous subset",
                                            "select subset"
                                        ],
                                        "title": "Method",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "method"
                                ],
                                "title": "Subband selection",
                                "type": "object"
                            },
                            "station_list": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [],
                                "items": {
                                    "$ref": "#/definitions/station"
                                },
                                "minItems": 0,
                                "type": "array",
                                "uniqueItems": true
                            },
                            "pointing": {
                                "additionalProperties": false,
                                "properties": {
                                    "angle1": {
                                        "default": 0.6624317181687094,
                                        "description": "First angle (e.g. RA)",
                                        "title": "Angle 1",
                                        "type": "number"
                                    },
                                    "angle2": {
                                        "default": 1.5579526427549426,
                                        "description": "Second angle (e.g. DEC)",
                                        "title": "Angle 2",
                                        "type": "number"
                                    },
                                    "direction_type": {
                                        "default": "J2000",
                                        "description": "",
                                        "enum": [
                                            "J2000",
                                            "AZELGEO",
                                            "LMN",
                                            "SUN",
                                            "MOON",
                                            "MERCURY",
                                            "VENUS",
                                            "MARS",
                                            "JUPITER",
                                            "SATURN",
                                            "URANUS",
                                            "NEPTUNE",
                                            "PLUTO"
                                        ],
                                        "title": "Reference frame",
                                        "type": "string"
                                    },
                                    "target": {
                                        "default": "_target_name_",
                                        "description": "Description of where this beam points at",
                                        "minLength": 1,
                                        "title": "Target",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "angle1",
                                    "angle2",
                                    "target"
                                ],
                                "type": "object"
                            },
                            "stokes_settings": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "channels_per_subband": {
                                        "default": 1,
                                        "description": "Number of frequency bands per subband",
                                        "enum": [
                                            1,
                                            8,
                                            16,
                                            32,
                                            64,
                                            128,
                                            256,
                                            512,
                                            1024
                                        ],
                                        "minimum": 1,
                                        "title": "Channels/subband",
                                        "type": "integer"
                                    },
                                    "quantisation": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "bits": {
                                                "default": 8,
                                                "description": "Number of bits for a single value",
                                                "enum": [
                                                    8
                                                ],
                                                "title": "Bits/sample",
                                                "type": "integer"
                                            },
                                            "enabled": {
                                                "default": false,
                                                "title": "Output quantisation into integers",
                                                "type": "boolean"
                                            },
                                            "scale_max": {
                                                "default": 5,
                                                "description": "Cut off values above this treshold",
                                                "title": "Maximum value",
                                                "type": "number"
                                            },
                                            "scale_min": {
                                                "default": -5,
                                                "description": "Cut off values below this treshold",
                                                "title": "Minimum value",
                                                "type": "number"
                                            }
                                        },
                                        "required": [
                                            "enabled",
                                            "bits",
                                            "scale_min",
                                            "scale_max"
                                        ],
                                        "title": "Output quantisation settings",
                                        "type": "object"
                                    },
                                    "stokes": {
                                        "$ref": "#/definitions/stokes",
                                        "default": "I"
                                    },
                                    "subbands_per_file": {
                                        "default": 488,
                                        "description": "The maximum number of subbands to write in each output dataproduct.",
                                        "maximum": 488,
                                        "minimum": 1,
                                        "title": "Subbands per file",
                                        "type": "integer"
                                    },
                                    "time_integration_factor": {
                                        "default": 1,
                                        "description": "The number of samples to integrate over",
                                        "maximum": 12288,
                                        "minimum": 1,
                                        "title": "Time integration",
                                        "type": "integer"
                                    }
                                },
                                "required": [
                                    "stokes",
                                    "time_integration_factor",
                                    "channels_per_subband"
                                ],
                                "type": "object"
                            },
                            "station": {
                                "description": "These are the LOFAR stations",
                                "enum": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501",
                                    "RS106",
                                    "RS205",
                                    "RS208",
                                    "RS210",
                                    "RS305",
                                    "RS306",
                                    "RS307",
                                    "RS310",
                                    "RS406",
                                    "RS407",
                                    "RS409",
                                    "RS503",
                                    "RS508",
                                    "RS509",
                                    "DE601",
                                    "DE602",
                                    "DE603",
                                    "DE604",
                                    "DE605",
                                    "FR606",
                                    "SE607",
                                    "UK608",
                                    "DE609",
                                    "PL610",
                                    "PL611",
                                    "PL612",
                                    "IE613",
                                    "LV614"
                                ],
                                "title": "Station",
                                "type": "string"
                            },
                            "max_number_of_missing_stations": {
                                "default": 0,
                                "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                                "minimum": 0,
                                "title": "Maximum number of stations to omit",
                                "type": "integer"
                            },
                            "stokes": {
                                "default": "I",
                                "description": "Which Stokes to produce",
                                "enum": [
                                    "I",
                                    "IQUV",
                                    "XXYY"
                                ],
                                "title": "Stokes",
                                "type": "string"
                            },
                            "station_groups": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [
                                    {
                                        "max_nr_missing": 1,
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ]
                                    }
                                ],
                                "description": "One or more predefined or custom groups of stations",
                                "items": {
                                    "$ref": "#/definitions/station_group"
                                },
                                "minItems": 1,
                                "title": "Station groups",
                                "type": "array"
                            },
                            "station_group": {
                                "anyOf": [
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007"
                                            ]
                                        },
                                        "description": "The group of all stations on the Superterp",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Superterp",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501"
                                            ]
                                        },
                                        "description": "The group of all Core stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Core",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch remote stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Remote",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch (Core + Remote) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Dutch",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 2,
                                            "stations": [
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 1,
                                            "stations": [
                                                "DE601",
                                                "DE605"
                                            ]
                                        },
                                        "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE605"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International required",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 6,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509",
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all (Core + Remote + International) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509",
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "All",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS001"
                                            ]
                                        },
                                        "description": "A custom group of stations which can be defined by the user",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list"
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Custom",
                                        "type": "object"
                                    }
                                ],
                                "default": {
                                    "max_nr_missing": 1,
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                },
                                "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                                "title": "Station group",
                                "type": "object"
                            },
                            "station_configuration": {
                                "default": {},
                                "properties": {
                                    "SAPs": {
                                        "$ref": "#/definitions/SAPs",
                                        "default": [
                                            {}
                                        ],
                                        "minItems": 1
                                    },
                                    "antenna_set": {
                                        "$ref": "#/definitions/antenna_set",
                                        "default": "HBA_DUAL"
                                    },
                                    "filter": {
                                        "$ref": "#/definitions/filter",
                                        "default": "HBA_110_190"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007"
                                                ]
                                            }
                                        ]
                                    },
                                    "tile_beam": {
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "description": "HBA only",
                                        "title": "Tile beam"
                                    }
                                },
                                "required": [
                                    "station_groups",
                                    "antenna_set",
                                    "filter",
                                    "SAPs"
                                ],
                                "title": "station_configuration",
                                "type": "object"
                            },
                            "filter": {
                                "default": "HBA_110_190",
                                "description": "Must match antenna type",
                                "enum": [
                                    "LBA_10_70",
                                    "LBA_30_70",
                                    "LBA_10_90",
                                    "LBA_30_90",
                                    "HBA_110_190",
                                    "HBA_210_250"
                                ],
                                "title": "Band-pass filter",
                                "type": "string"
                            },
                            "antenna_set": {
                                "default": "HBA_DUAL",
                                "description": "Fields & antennas to use",
                                "enum": [
                                    "HBA_DUAL",
                                    "HBA_DUAL_INNER",
                                    "HBA_ONE",
                                    "HBA_ONE_INNER",
                                    "HBA_ZERO",
                                    "HBA_ZERO_INNER",
                                    "LBA_INNER",
                                    "LBA_OUTER",
                                    "LBA_SPARSE_EVEN",
                                    "LBA_SPARSE_ODD",
                                    "LBA_ALL"
                                ],
                                "title": "Antenna set",
                                "type": "string"
                            },
                            "SAPs": {
                                "additionalItems": false,
                                "default": [
                                    {}
                                ],
                                "description": "Station beams",
                                "items": {
                                    "additionalProperties": false,
                                    "default": {},
                                    "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                    "properties": {
                                        "digital_pointing": {
                                            "$id": "#target_pointing",
                                            "$ref": "#/definitions/pointing",
                                            "default": {},
                                            "title": "Digital pointing"
                                        },
                                        "name": {
                                            "default": "_SAP_name_",
                                            "description": "Identifier for this beam",
                                            "minLength": 1,
                                            "title": "Name",
                                            "type": "string"
                                        },
                                        "subbands": {
                                            "additionalItems": false,
                                            "default": [
                                                255
                                            ],
                                            "items": {
                                                "maximum": 511,
                                                "minimum": 0,
                                                "title": "Subband",
                                                "type": "integer"
                                            },
                                            "maxItems": 488,
                                            "minItems": 1,
                                            "title": "Subband list",
                                            "type": "array"
                                        }
                                    },
                                    "required": [
                                        "name",
                                        "digital_pointing",
                                        "subbands"
                                    ],
                                    "title": "SAP",
                                    "type": "object"
                                },
                                "minItems": 0,
                                "title": "SAPs",
                                "type": "array"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2#",
                        "type": "object",
                        "title": "beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "beamformer",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                "title": "Duration",
                                "default": 300,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/2#/definitions/beamformer",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/observation",
                    "type_value": "observation",
                    "updated_at": "2022-04-21T08:15:52.853846",
                    "version": 2
                },
                "specifications_template_id": 3,
                "tags": [],
                "task_blueprints": [
                    "http://localhost:3000/api/task_blueprint/53"
                ],
                "task_blueprints_ids": [
                    53
                ],
                "task_type": "observation",
                "updated_at": "2022-04-21T08:16:32.621939"
            }
        ],
        "task_drafts_ids": [
            53
        ],
        "updated_at": "2022-04-21T08:16:32.564890"
    },
    {
        "id": 15,
        "url": "http://localhost:3000/api/scheduling_unit_draft/15",
        "created_at": "2022-04-22T07:25:37.871421",
        "description": "OSR-01",
        "duration": 600,
        "ingest_permission_required": true,
        "interrupts_telescope": false,
        "name": "OSR-01",
        "observation_strategy_template": "http://localhost:3000/api/scheduling_unit_observing_strategy_template/14",
        "observation_strategy_template_id": 14,
        "piggyback_allowed_aartfaac": true,
        "piggyback_allowed_tbb": true,
        "priority_queue": "http://localhost:3000/api/priority_queue_type/A",
        "priority_queue_value": "A",
        "rank": 0,
        "scheduling_constraints_doc": {
            "sky": {
                "min_distance": {
                    "sun": 0,
                    "moon": 0,
                    "jupiter": 0
                },
                "min_elevation": {
                    "target": 0.10000736613927509,
                    "calibrator": 0.5000368306963754
                },
                "transit_offset": {
                    "to": 21600,
                    "from": -21600
                }
            },
            "time": {
                "between": [],
                "not_between": []
            },
            "daily": {
                "require_day": false,
                "require_night": false,
                "avoid_twilight": false
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
            "scheduler": "dynamic"
        },
        "scheduling_constraints_template": "http://localhost:3000/api/scheduling_constraints_template/1",
        "scheduling_constraints_template_id": 1,
        "scheduling_set": "http://localhost:3000/api/scheduling_set/1",
        "scheduling_set_id": 1,
        "scheduling_unit_blueprints": [],
        "scheduling_unit_blueprints_ids": [],
        "specifications_template": "http://localhost:3000/api/scheduling_unit_template/1",
        "specifications_template_id": 1,
        "tags": [],
        "task_drafts": [
            {
                "id": 68,
                "url": "http://localhost:3000/api/task_draft/68",
                "connector_types": [
                    {
                        "id": 8,
                        "url": "http://localhost:3000/api/task_connector_type/8",
                        "dataformat": "http://localhost:3000/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://localhost:3000/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://localhost:3000/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://localhost:3000/api/role/beamformer",
                        "role_value": "beamformer",
                        "task_template": "http://localhost:3000/api/task_template/6",
                        "task_template_id": 6
                    },
                    {
                        "id": 5,
                        "url": "http://localhost:3000/api/task_connector_type/5",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://localhost:3000/api/role/correlator",
                        "role_value": "correlator",
                        "task_template": "http://localhost:3000/api/task_template/6",
                        "task_template_id": 6
                    }
                ],
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-04-22T07:25:37.890328",
                "description": "IM BF Cal&Tgt",
                "duration": 600,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Observation",
                "output_pinned": false,
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 0,
                "relative_stop_time": 600,
                "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/15",
                "scheduling_unit_draft_id": 15,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "QA": {
                        "plots": {
                            "enabled": true,
                            "autocorrelation": true,
                            "crosscorrelation": true
                        },
                        "file_conversion": {
                            "enabled": true,
                            "nr_of_subbands": -1,
                            "nr_of_timestamps": 256
                        },
                        "inspection_plots": "msplots"
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20and%20beamforming%20observation/2/ref_resolved",
                    "duration": 600,
                    "beamformer": {
                        "ppf": false,
                        "pipelines": [
                            {
                                "name": "_beamformer_name_",
                                "coherent": {
                                    "SAPs": [],
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "flys eye": {
                                    "enabled": false,
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "incoherent": {
                                    "SAPs": [],
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "station_groups": [
                                    {
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501"
                                        ],
                                        "max_nr_missing": 1
                                    }
                                ]
                            }
                        ]
                    },
                    "calibrator": {
                        "name": "calibrator",
                        "pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "_target_name_",
                            "direction_type": "J2000"
                        },
                        "autoselect": true
                    },
                    "correlator": {
                        "storage_cluster": "CEP4",
                        "integration_time": 1,
                        "channels_per_subband": 64,
                        "topocentric_frequency_correction": false
                    },
                    "station_configuration": {
                        "SAPs": [
                            {
                                "name": "_SAP_name_",
                                "subbands": [
                                    255
                                ],
                                "digital_pointing": {
                                    "angle1": 0.6624317181687094,
                                    "angle2": 1.5579526427549426,
                                    "target": "_target_name_",
                                    "direction_type": "J2000"
                                }
                            }
                        ],
                        "filter": "HBA_110_190",
                        "tile_beam": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "_target_name_",
                            "direction_type": "J2000"
                        },
                        "antenna_set": "HBA_DUAL",
                        "station_groups": [
                            {
                                "stations": [
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                },
                "specifications_template": {
                    "id": 6,
                    "url": "http://localhost:3000/api/task_template/6",
                    "created_at": "2022-04-21T08:15:53.120162",
                    "description": "This schema combines the calibrator, target and beamformer observation task schema's for parallel observing.",
                    "name": "parallel calibrator target and beamforming observation",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20and%20beamforming%20observation/2/ref_resolved",
                        "type": "object",
                        "title": "parallel calibrator target and beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "calibrator",
                            "correlator",
                            "duration",
                            "station_configuration",
                            "beamformer"
                        ],
                        "properties": {
                            "QA": {
                                "$ref": "#/definitions/QA",
                                "default": {}
                            },
                            "duration": {
                                "$id": "#duration",
                                "$ref": "#/definitions/timedelta",
                                "title": "Duration",
                                "default": 600,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "#/definitions/beamformer",
                                "default": {}
                            },
                            "calibrator": {
                                "$ref": "#/definitions/calibrator",
                                "default": {}
                            },
                            "correlator": {
                                "$ref": "#/definitions/correlator",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema combines the calibrator, target and beamformer observation task schema's for parallel observing.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "QA": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Perform all Quality Assurance (QA) tasks, including file conversion and plotting.",
                                "properties": {
                                    "file_conversion": {
                                        "$ref": "#/definitions/file_conversion",
                                        "default": {}
                                    },
                                    "inspection_plots": {
                                        "$ref": "#/definitions/inspection_plots",
                                        "default": "msplots"
                                    },
                                    "plots": {
                                        "$ref": "#/definitions/plots",
                                        "default": {}
                                    }
                                },
                                "required": [
                                    "file_conversion",
                                    "plots",
                                    "inspection_plots"
                                ],
                                "title": "QA",
                                "type": "object"
                            },
                            "file_conversion": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Create a QA file for the observation",
                                "properties": {
                                    "enabled": {
                                        "default": true,
                                        "description": "Do/Don't create a QA file for the observation",
                                        "title": "enabled",
                                        "type": "boolean"
                                    },
                                    "nr_of_subbands": {
                                        "default": -1,
                                        "description": "Keep this number of subbands from the observation in the QA file, or all if -1",
                                        "title": "#subbands",
                                        "type": "integer"
                                    },
                                    "nr_of_timestamps": {
                                        "default": 256,
                                        "description": "Extract this number of timestamps from the observation in the QA file (equidistantanly sampled, no averaging/interpolation)",
                                        "minimum": 1,
                                        "title": "#timestamps",
                                        "type": "integer"
                                    }
                                },
                                "required": [
                                    "enabled",
                                    "nr_of_subbands",
                                    "nr_of_timestamps"
                                ],
                                "title": "File Conversion",
                                "type": "object"
                            },
                            "plots": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Create dynamic spectrum plots",
                                "properties": {
                                    "autocorrelation": {
                                        "default": true,
                                        "description": "Create autocorrelation plots for all stations",
                                        "title": "autocorrelation",
                                        "type": "boolean"
                                    },
                                    "crosscorrelation": {
                                        "default": true,
                                        "description": "Create crosscorrelation plots for all baselines",
                                        "title": "crosscorrelation",
                                        "type": "boolean"
                                    },
                                    "enabled": {
                                        "default": true,
                                        "description": "Do/Don't create plots from the QA file from the observation",
                                        "title": "enabled",
                                        "type": "boolean"
                                    }
                                },
                                "required": [
                                    "enabled",
                                    "autocorrelation",
                                    "crosscorrelation"
                                ],
                                "title": "Plots",
                                "type": "object"
                            },
                            "inspection_plots": {
                                "decription": "Type of inspection plots to run",
                                "default": "msplots",
                                "enum": [
                                    "msplots",
                                    "dynspec",
                                    "none"
                                ],
                                "title": "Inspection Plots",
                                "type": "string"
                            },
                            "timedelta": {
                                "default": 0,
                                "description": "A time duration or delta expressed in seconds",
                                "type": "number"
                            },
                            "timestamp": {
                                "default": "1970-01-01T00:00:00Z",
                                "description": "A timestamp defined in UTC",
                                "format": "date-time",
                                "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                                "type": "string"
                            },
                            "beamformer": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "pipelines": {
                                        "additionalItems": false,
                                        "default": [
                                            {}
                                        ],
                                        "description": "This array of beamformer pipelines is currently restricted to exactly 1 item. Support for handling multiple pipelines (in cobalt) will be added in the future.",
                                        "items": {
                                            "$ref": "#/definitions/beamformer_pipeline"
                                        },
                                        "maxItems": 1,
                                        "minItems": 1,
                                        "title": "Pipelines",
                                        "type": "array"
                                    },
                                    "ppf": {
                                        "default": false,
                                        "description": "Use a PPF for the channellisation for beamforming. If not, an FFT is used.",
                                        "title": "Use PPF when beamforming",
                                        "type": "boolean"
                                    }
                                },
                                "required": [
                                    "pipelines",
                                    "ppf"
                                ],
                                "title": "Beamformer",
                                "type": "object"
                            },
                            "beamformer_pipeline": {
                                "additionalProperties": false,
                                "default": {},
                                "headerTemplate": "Beamformer Pipeline {{ self.index }}",
                                "properties": {
                                    "coherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to beamform.",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        },
                                                        "tab_rings": {
                                                            "additonalProperties": false,
                                                            "default": {},
                                                            "description": "Rings of TABs around the center of the beam.",
                                                            "properties": {
                                                                "count": {
                                                                    "default": 0,
                                                                    "maximum": 11,
                                                                    "minimum": 0,
                                                                    "title": "Number of rings",
                                                                    "type": "integer"
                                                                },
                                                                "width": {
                                                                    "default": 0.01,
                                                                    "description": "Distance between pointings.",
                                                                    "minimum": 0,
                                                                    "title": "Ring width",
                                                                    "type": "number"
                                                                }
                                                            },
                                                            "title": "Tied-Array Rings",
                                                            "type": "object"
                                                        },
                                                        "tabs": {
                                                            "additionalItems": false,
                                                            "default": [],
                                                            "description": "Tied-array beams to form",
                                                            "items": {
                                                                "additionalProperties": false,
                                                                "default": {},
                                                                "headerTemplate": "TAB {{ self.index }}",
                                                                "properties": {
                                                                    "allow_after_previous": {
                                                                        "default": true,
                                                                        "description": "This TAB can be recorded after the previous TAB, if duration allows.",
                                                                        "title": "Allow after previous",
                                                                        "type": "boolean"
                                                                    },
                                                                    "duration": {
                                                                        "default": 0,
                                                                        "description": "Exposure time of the TAB. 0 = observation duration.",
                                                                        "maximum": 0,
                                                                        "minimum": 0,
                                                                        "title": "Duration",
                                                                        "type": "integer"
                                                                    },
                                                                    "pointing": {
                                                                        "$ref": "#/definitions/pointing",
                                                                        "default": {}
                                                                    },
                                                                    "relative": {
                                                                        "default": false,
                                                                        "description": "Angular offset applied to the TAB pointing",
                                                                        "title": "Relative to SAP",
                                                                        "type": "boolean"
                                                                    }
                                                                },
                                                                "required": [
                                                                    "pointing",
                                                                    "relative"
                                                                ],
                                                                "title": "Tied-Array Beam",
                                                                "type": "object"
                                                            },
                                                            "title": "Tied-Array Beams",
                                                            "type": "array"
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "tabs"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "SAPs",
                                            "settings"
                                        ],
                                        "title": "Coherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "flys eye": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "description": "Produce beams containing the individual station signals",
                                        "properties": {
                                            "enabled": {
                                                "default": false,
                                                "title": "Enable Fly's Eye",
                                                "type": "boolean"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "enabled"
                                        ],
                                        "title": "Fly's Eye Settings",
                                        "type": "object"
                                    },
                                    "incoherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to create incoherent TABs for (empty list = all).",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "default": {},
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "subbands"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "settings",
                                            "SAPs"
                                        ],
                                        "title": "Incoherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "name": {
                                        "default": "_beamformer_name_",
                                        "description": "Beamformer name, used for identification purposes.",
                                        "minLength": 1,
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501"
                                                ]
                                            }
                                        ],
                                        "description": "While observing, COBALT will beamform on the intersection of all stations in this list and the used stations in the observation. So, specifying all possible stations here means that all observation-stations are used. Specifying a small subset here means that only the observing-stations in this small list are used. By default we let COBALT beamform on the Core stations.",
                                        "minItems": 1
                                    }
                                },
                                "required": [
                                    "name"
                                ],
                                "title": "Beamformer Pipeline",
                                "type": "object"
                            },
                            "subband_selection": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "list": {
                                        "additionalItems": false,
                                        "default": [],
                                        "description": "If method is 'select subset', only beamform these subbands, and only if they occur in the SAP.",
                                        "items": {
                                            "maximum": 511,
                                            "minimum": 0,
                                            "title": "Subband",
                                            "type": "integer"
                                        },
                                        "minItems": 0,
                                        "title": "Subset selection",
                                        "type": "array"
                                    },
                                    "method": {
                                        "default": "copy",
                                        "description": "How to select the subbands to beam form",
                                        "enum": [
                                            "copy",
                                            "largest continuous subset",
                                            "select subset"
                                        ],
                                        "title": "Method",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "method"
                                ],
                                "title": "Subband selection",
                                "type": "object"
                            },
                            "station_list": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [],
                                "items": {
                                    "$ref": "#/definitions/station"
                                },
                                "minItems": 0,
                                "type": "array",
                                "uniqueItems": true
                            },
                            "pointing": {
                                "additionalProperties": false,
                                "properties": {
                                    "angle1": {
                                        "default": 0.6624317181687094,
                                        "description": "First angle (e.g. RA)",
                                        "title": "Angle 1",
                                        "type": "number"
                                    },
                                    "angle2": {
                                        "default": 1.5579526427549426,
                                        "description": "Second angle (e.g. DEC)",
                                        "title": "Angle 2",
                                        "type": "number"
                                    },
                                    "direction_type": {
                                        "default": "J2000",
                                        "description": "",
                                        "enum": [
                                            "J2000",
                                            "AZELGEO",
                                            "LMN",
                                            "SUN",
                                            "MOON",
                                            "MERCURY",
                                            "VENUS",
                                            "MARS",
                                            "JUPITER",
                                            "SATURN",
                                            "URANUS",
                                            "NEPTUNE",
                                            "PLUTO"
                                        ],
                                        "title": "Reference frame",
                                        "type": "string"
                                    },
                                    "target": {
                                        "default": "_target_name_",
                                        "description": "Description of where this beam points at",
                                        "minLength": 1,
                                        "title": "Target",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "angle1",
                                    "angle2",
                                    "target"
                                ],
                                "type": "object"
                            },
                            "stokes_settings": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "channels_per_subband": {
                                        "default": 1,
                                        "description": "Number of frequency bands per subband",
                                        "enum": [
                                            1,
                                            8,
                                            16,
                                            32,
                                            64,
                                            128,
                                            256,
                                            512,
                                            1024
                                        ],
                                        "minimum": 1,
                                        "title": "Channels/subband",
                                        "type": "integer"
                                    },
                                    "quantisation": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "bits": {
                                                "default": 8,
                                                "description": "Number of bits for a single value",
                                                "enum": [
                                                    8
                                                ],
                                                "title": "Bits/sample",
                                                "type": "integer"
                                            },
                                            "enabled": {
                                                "default": false,
                                                "title": "Output quantisation into integers",
                                                "type": "boolean"
                                            },
                                            "scale_max": {
                                                "default": 5,
                                                "description": "Cut off values above this treshold",
                                                "title": "Maximum value",
                                                "type": "number"
                                            },
                                            "scale_min": {
                                                "default": -5,
                                                "description": "Cut off values below this treshold",
                                                "title": "Minimum value",
                                                "type": "number"
                                            }
                                        },
                                        "required": [
                                            "enabled",
                                            "bits",
                                            "scale_min",
                                            "scale_max"
                                        ],
                                        "title": "Output quantisation settings",
                                        "type": "object"
                                    },
                                    "stokes": {
                                        "$ref": "#/definitions/stokes",
                                        "default": "I"
                                    },
                                    "subbands_per_file": {
                                        "default": 488,
                                        "description": "The maximum number of subbands to write in each output dataproduct.",
                                        "maximum": 488,
                                        "minimum": 1,
                                        "title": "Subbands per file",
                                        "type": "integer"
                                    },
                                    "time_integration_factor": {
                                        "default": 1,
                                        "description": "The number of samples to integrate over",
                                        "maximum": 12288,
                                        "minimum": 1,
                                        "title": "Time integration",
                                        "type": "integer"
                                    }
                                },
                                "required": [
                                    "stokes",
                                    "time_integration_factor",
                                    "channels_per_subband"
                                ],
                                "type": "object"
                            },
                            "station": {
                                "description": "These are the LOFAR stations",
                                "enum": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501",
                                    "RS106",
                                    "RS205",
                                    "RS208",
                                    "RS210",
                                    "RS305",
                                    "RS306",
                                    "RS307",
                                    "RS310",
                                    "RS406",
                                    "RS407",
                                    "RS409",
                                    "RS503",
                                    "RS508",
                                    "RS509",
                                    "DE601",
                                    "DE602",
                                    "DE603",
                                    "DE604",
                                    "DE605",
                                    "FR606",
                                    "SE607",
                                    "UK608",
                                    "DE609",
                                    "PL610",
                                    "PL611",
                                    "PL612",
                                    "IE613",
                                    "LV614"
                                ],
                                "title": "Station",
                                "type": "string"
                            },
                            "max_number_of_missing_stations": {
                                "default": 0,
                                "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                                "minimum": 0,
                                "title": "Maximum number of stations to omit",
                                "type": "integer"
                            },
                            "stokes": {
                                "default": "I",
                                "description": "Which Stokes to produce",
                                "enum": [
                                    "I",
                                    "IQUV",
                                    "XXYY"
                                ],
                                "title": "Stokes",
                                "type": "string"
                            },
                            "station_groups": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [
                                    {
                                        "max_nr_missing": 1,
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ]
                                    }
                                ],
                                "description": "One or more predefined or custom groups of stations",
                                "items": {
                                    "$ref": "#/definitions/station_group"
                                },
                                "minItems": 1,
                                "title": "Station groups",
                                "type": "array"
                            },
                            "station_group": {
                                "anyOf": [
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007"
                                            ]
                                        },
                                        "description": "The group of all stations on the Superterp",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Superterp",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501"
                                            ]
                                        },
                                        "description": "The group of all Core stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Core",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch remote stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Remote",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch (Core + Remote) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Dutch",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 2,
                                            "stations": [
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 1,
                                            "stations": [
                                                "DE601",
                                                "DE605"
                                            ]
                                        },
                                        "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE605"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International required",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 6,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509",
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all (Core + Remote + International) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509",
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "All",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS001"
                                            ]
                                        },
                                        "description": "A custom group of stations which can be defined by the user",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list"
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Custom",
                                        "type": "object"
                                    }
                                ],
                                "default": {
                                    "max_nr_missing": 1,
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                },
                                "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                                "title": "Station group",
                                "type": "object"
                            },
                            "calibrator": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "autoselect": {
                                        "default": true,
                                        "description": "Auto-select calibrator based on elevation",
                                        "title": "Auto-select",
                                        "type": "boolean"
                                    },
                                    "name": {
                                        "default": "calibrator",
                                        "description": "Name of the calibrator SAP",
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "pointing": {
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "description": "Manually selected calibrator",
                                        "title": "Digital pointing"
                                    }
                                },
                                "required": [
                                    "autoselect",
                                    "pointing",
                                    "name"
                                ],
                                "type": "object"
                            },
                            "correlator": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "channels_per_subband": {
                                        "default": 64,
                                        "description": "Number of frequency bands per subband",
                                        "enum": [
                                            8,
                                            16,
                                            32,
                                            64,
                                            128,
                                            256,
                                            512,
                                            1024
                                        ],
                                        "minimum": 8,
                                        "title": "Channels/subband",
                                        "type": "integer"
                                    },
                                    "integration_time": {
                                        "$ref": "#/definitions/timedelta",
                                        "default": 1,
                                        "description": "Desired integration period (seconds)",
                                        "minimum": 0.1,
                                        "title": "Integration time"
                                    },
                                    "storage_cluster": {
                                        "default": "CEP4",
                                        "description": "Cluster to write output to",
                                        "enum": [
                                            "CEP4",
                                            "DragNet"
                                        ],
                                        "title": "Storage cluster",
                                        "type": "string"
                                    },
                                    "topocentric_frequency_correction": {
                                        "default": false,
                                        "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation.",
                                        "title": "Topocentric Frequency Correction",
                                        "type": "boolean"
                                    }
                                }
                            },
                            "station_configuration": {
                                "default": {},
                                "properties": {
                                    "SAPs": {
                                        "$ref": "#/definitions/SAPs",
                                        "default": [
                                            {}
                                        ],
                                        "minItems": 1
                                    },
                                    "antenna_set": {
                                        "$ref": "#/definitions/antenna_set",
                                        "default": "HBA_DUAL"
                                    },
                                    "filter": {
                                        "$ref": "#/definitions/filter",
                                        "default": "HBA_110_190"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007"
                                                ]
                                            }
                                        ]
                                    },
                                    "tile_beam": {
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "description": "HBA only",
                                        "title": "Tile beam"
                                    }
                                },
                                "required": [
                                    "station_groups",
                                    "antenna_set",
                                    "filter",
                                    "SAPs"
                                ],
                                "title": "station_configuration",
                                "type": "object"
                            },
                            "filter": {
                                "default": "HBA_110_190",
                                "description": "Must match antenna type",
                                "enum": [
                                    "LBA_10_70",
                                    "LBA_30_70",
                                    "LBA_10_90",
                                    "LBA_30_90",
                                    "HBA_110_190",
                                    "HBA_210_250"
                                ],
                                "title": "Band-pass filter",
                                "type": "string"
                            },
                            "antenna_set": {
                                "default": "HBA_DUAL",
                                "description": "Fields & antennas to use",
                                "enum": [
                                    "HBA_DUAL",
                                    "HBA_DUAL_INNER",
                                    "HBA_ONE",
                                    "HBA_ONE_INNER",
                                    "HBA_ZERO",
                                    "HBA_ZERO_INNER",
                                    "LBA_INNER",
                                    "LBA_OUTER",
                                    "LBA_SPARSE_EVEN",
                                    "LBA_SPARSE_ODD",
                                    "LBA_ALL"
                                ],
                                "title": "Antenna set",
                                "type": "string"
                            },
                            "SAPs": {
                                "additionalItems": false,
                                "default": [
                                    {}
                                ],
                                "description": "Station beams",
                                "items": {
                                    "additionalProperties": false,
                                    "default": {},
                                    "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                    "properties": {
                                        "digital_pointing": {
                                            "$id": "#target_pointing",
                                            "$ref": "#/definitions/pointing",
                                            "default": {},
                                            "title": "Digital pointing"
                                        },
                                        "name": {
                                            "default": "_SAP_name_",
                                            "description": "Identifier for this beam",
                                            "minLength": 1,
                                            "title": "Name",
                                            "type": "string"
                                        },
                                        "subbands": {
                                            "additionalItems": false,
                                            "default": [
                                                255
                                            ],
                                            "items": {
                                                "maximum": 511,
                                                "minimum": 0,
                                                "title": "Subband",
                                                "type": "integer"
                                            },
                                            "maxItems": 488,
                                            "minItems": 1,
                                            "title": "Subband list",
                                            "type": "array"
                                        }
                                    },
                                    "required": [
                                        "name",
                                        "digital_pointing",
                                        "subbands"
                                    ],
                                    "title": "SAP",
                                    "type": "object"
                                },
                                "minItems": 0,
                                "title": "SAPs",
                                "type": "array"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20and%20beamforming%20observation/2#",
                        "type": "object",
                        "title": "parallel calibrator target and beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "calibrator",
                            "correlator",
                            "duration",
                            "station_configuration",
                            "beamformer"
                        ],
                        "properties": {
                            "QA": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/2#/definitions/QA",
                                "default": {}
                            },
                            "duration": {
                                "$id": "#duration",
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                "title": "Duration",
                                "default": 600,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/2#/definitions/beamformer",
                                "default": {}
                            },
                            "calibrator": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/calibrator/1#/definitions/calibrator",
                                "default": {}
                            },
                            "correlator": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/correlator/1#/definitions/correlator",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema combines the calibrator, target and beamformer observation task schema's for parallel observing.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/observation",
                    "type_value": "observation",
                    "updated_at": "2022-04-21T08:15:53.120183",
                    "version": 2
                },
                "specifications_template_id": 6,
                "tags": [],
                "task_blueprints": [],
                "task_blueprints_ids": [],
                "task_type": "observation",
                "updated_at": "2022-04-22T07:25:38.319179"
            }
        ],
        "task_drafts_ids": [
            68
        ],
        "updated_at": "2022-04-22T07:25:38.206490"
    },
    {
        "id": 16,
        "url": "http://localhost:8008/api/scheduling_unit_draft/16",
        "created_at": "2022-05-26T05:21:21.317944",
        "description": "test",
        "duration": 1380.0,
        "ingest_permission_required": true,
        "interrupts_telescope": false,
        "name": "test",
        "observation_strategy_template": "http://localhost:8008/api/scheduling_unit_observing_strategy_template/1",
        "observation_strategy_template_id": 1,
        "piggyback_allowed_aartfaac": true,
        "piggyback_allowed_tbb": true,
        "priority_queue": "http://localhost:8008/api/priority_queue_type/A",
        "priority_queue_value": "A",
        "rank": 0.0,
        "scheduling_constraints_doc": {
            "sky": {
                "min_distance": {
                    "sun": 0.5000368306963754,
                    "moon": 0.5000368306963754,
                    "jupiter": 0.5000368306963754
                },
                "min_elevation": {
                    "target": 0.5000368306963754,
                    "calibrator": 0.5000368306963754
                },
                "transit_offset": {
                    "to": 7200,
                    "from": -7200
                }
            },
            "time": {
                "between": [],
                "not_between": []
            },
            "daily": {
                "require_day": false,
                "require_night": false,
                "avoid_twilight": false
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
            "scheduler": "fixed_time"
        },
        "scheduling_constraints_template": "http://localhost:8008/api/scheduling_constraints_template/1",
        "scheduling_constraints_template_id": 1,
        "scheduling_set": "http://localhost:8008/api/scheduling_set/1",
        "scheduling_set_id": 1,
        "scheduling_unit_blueprints": [],
        "scheduling_unit_blueprints_ids": [],
        "specifications_template": "http://localhost:8008/api/scheduling_unit_template/1",
        "specifications_template_id": 1,
        "tags": [],
        "task_drafts": [
            {
                "id": 69,
                "url": "http://localhost:8008/api/task_draft/69",
                "connector_types": [
                    {
                        "id": 9,
                        "url": "http://localhost:8008/api/task_connector_type/9",
                        "dataformat": "http://localhost:8008/api/dataformat/Beamformed",
                        "dataformat_value": "Beamformed",
                        "datatype": "http://localhost:8008/api/datatype/time%20series",
                        "datatype_value": "time series",
                        "iotype": "http://localhost:8008/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://localhost:8008/api/role/beamformer",
                        "role_value": "beamformer",
                        "task_template": "http://localhost:8008/api/task_template/11",
                        "task_template_id": 11
                    },
                    {
                        "id": 5,
                        "url": "http://localhost:8008/api/task_connector_type/5",
                        "dataformat": "http://localhost:8008/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:8008/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:8008/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://localhost:8008/api/role/correlator",
                        "role_value": "correlator",
                        "task_template": "http://localhost:8008/api/task_template/11",
                        "task_template_id": 11
                    }
                ],
                "consumed_by": [
                    "http://localhost:8008/api/task_relation_draft/9",
                    "http://localhost:8008/api/task_relation_draft/10",
                    "http://localhost:8008/api/task_relation_draft/15",
                    "http://localhost:8008/api/task_relation_draft/16",
                    "http://localhost:8008/api/task_relation_draft/21"
                ],
                "consumed_by_ids": [
                    9,
                    10,
                    15,
                    16,
                    21
                ],
                "created_at": "2022-05-30T05:48:41.242947",
                "description": "IM+BF Observation",
                "duration": 600.0,
                "first_scheduling_relation": [
                    "http://localhost:8008/api/task_scheduling_relation_draft/4",
                    "http://localhost:8008/api/task_scheduling_relation_draft/5",
                    "http://localhost:8008/api/task_scheduling_relation_draft/6"
                ],
                "first_scheduling_relation_ids": [
                    4,
                    5,
                    6
                ],
                "name": "IM+BF",
                "output_pinned": false,
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 0.0,
                "relative_stop_time": 600.0,
                "scheduling_unit_draft": "http://localhost:8008/api/scheduling_unit_draft/2",
                "scheduling_unit_draft_id": 2,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "IM+BF",
                "specifications_doc": {
                    "QA": {
                        "plots": {
                            "enabled": false,
                            "autocorrelation": true,
                            "crosscorrelation": true
                        },
                        "file_conversion": {
                            "enabled": false,
                            "nr_of_subbands": -1,
                            "nr_of_timestamps": 256
                        },
                        "inspection_plots": "dynspec"
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20and%20beamforming%20observation/2/ref_resolved",
                    "duration": 600,
                    "beamformer": {
                        "ppf": false,
                        "pipelines": [
                            {
                                "name": "MultiRings_Beamformer",
                                "coherent": {
                                    "SAPs": [
                                        {
                                            "name": "IMBF_Sun",
                                            "tabs": [],
                                            "subbands": {
                                                "list": [
                                                    96,
                                                    97,
                                                    98,
                                                    99,
                                                    100,
                                                    101,
                                                    102,
                                                    103,
                                                    104,
                                                    105,
                                                    106,
                                                    107,
                                                    108,
                                                    109,
                                                    110,
                                                    111,
                                                    112,
                                                    113,
                                                    114,
                                                    115,
                                                    116,
                                                    117,
                                                    118,
                                                    119,
                                                    120,
                                                    121,
                                                    122,
                                                    123,
                                                    124,
                                                    125,
                                                    126,
                                                    127,
                                                    128,
                                                    129,
                                                    130,
                                                    131,
                                                    132,
                                                    133,
                                                    134,
                                                    135,
                                                    136,
                                                    137,
                                                    138,
                                                    139,
                                                    140,
                                                    141,
                                                    142,
                                                    143,
                                                    144,
                                                    145,
                                                    146,
                                                    147,
                                                    148,
                                                    149,
                                                    150,
                                                    151,
                                                    152,
                                                    153,
                                                    154,
                                                    155,
                                                    156,
                                                    157,
                                                    158,
                                                    159,
                                                    160,
                                                    161,
                                                    162,
                                                    163,
                                                    164,
                                                    165,
                                                    166,
                                                    167,
                                                    168,
                                                    169,
                                                    170,
                                                    171,
                                                    172,
                                                    173,
                                                    174,
                                                    175,
                                                    176,
                                                    177,
                                                    178,
                                                    179,
                                                    180,
                                                    181,
                                                    182,
                                                    183,
                                                    184,
                                                    185,
                                                    186,
                                                    187,
                                                    188,
                                                    189,
                                                    190,
                                                    191,
                                                    192,
                                                    193,
                                                    194,
                                                    195,
                                                    196,
                                                    197,
                                                    198,
                                                    199,
                                                    200,
                                                    201,
                                                    202,
                                                    203,
                                                    204,
                                                    205,
                                                    206,
                                                    207,
                                                    208,
                                                    209,
                                                    210,
                                                    211,
                                                    212,
                                                    213,
                                                    214,
                                                    215,
                                                    216,
                                                    217,
                                                    218,
                                                    219,
                                                    220,
                                                    221,
                                                    222,
                                                    223,
                                                    224,
                                                    225,
                                                    226,
                                                    227,
                                                    228,
                                                    229,
                                                    230,
                                                    231,
                                                    232,
                                                    233,
                                                    234,
                                                    235,
                                                    236,
                                                    237,
                                                    238,
                                                    239,
                                                    240,
                                                    241,
                                                    242,
                                                    243,
                                                    244,
                                                    245,
                                                    246,
                                                    247,
                                                    248,
                                                    249,
                                                    250,
                                                    251,
                                                    252,
                                                    253,
                                                    254,
                                                    255,
                                                    256,
                                                    257,
                                                    258,
                                                    259,
                                                    260,
                                                    261,
                                                    262,
                                                    263,
                                                    264,
                                                    265,
                                                    266,
                                                    267,
                                                    268,
                                                    269,
                                                    270,
                                                    271,
                                                    272,
                                                    273,
                                                    274,
                                                    282,
                                                    284,
                                                    286,
                                                    288,
                                                    290,
                                                    292,
                                                    294,
                                                    296,
                                                    298,
                                                    300,
                                                    302,
                                                    304,
                                                    306,
                                                    308,
                                                    310,
                                                    312,
                                                    314,
                                                    316,
                                                    318,
                                                    320,
                                                    322,
                                                    324,
                                                    326,
                                                    328,
                                                    330,
                                                    332,
                                                    334,
                                                    336,
                                                    338,
                                                    340,
                                                    342,
                                                    344,
                                                    346,
                                                    348,
                                                    350,
                                                    352,
                                                    354,
                                                    356,
                                                    358,
                                                    360,
                                                    362,
                                                    364,
                                                    366,
                                                    368,
                                                    370,
                                                    372,
                                                    374,
                                                    376,
                                                    378,
                                                    380,
                                                    382,
                                                    384,
                                                    386,
                                                    388,
                                                    390,
                                                    392,
                                                    394,
                                                    396,
                                                    398,
                                                    400,
                                                    402,
                                                    404,
                                                    406,
                                                    408,
                                                    410
                                                ],
                                                "method": "copy"
                                            },
                                            "tab_rings": {
                                                "count": 6,
                                                "width": 0.00174533
                                            }
                                        }
                                    ],
                                    "settings": {
                                        "stokes": "IQUV",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 16,
                                        "time_integration_factor": 128
                                    }
                                },
                                "flys eye": {
                                    "enabled": false,
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "incoherent": {
                                    "SAPs": [],
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "station_groups": [
                                    {
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501"
                                        ],
                                        "max_nr_missing": 1
                                    }
                                ]
                            }
                        ]
                    },
                    "calibrator": {
                        "name": "calibrator",
                        "pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "_target_name_",
                            "direction_type": "J2000"
                        },
                        "autoselect": false
                    },
                    "correlator": {
                        "storage_cluster": "CEP4",
                        "integration_time": 0.25,
                        "channels_per_subband": 16,
                        "topocentric_frequency_correction": false
                    },
                    "station_configuration": {
                        "SAPs": [
                            {
                                "name": "IMBF_Sun",
                                "subbands": [
                                    96,
                                    97,
                                    98,
                                    99,
                                    100,
                                    101,
                                    102,
                                    103,
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    137,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    164,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    181,
                                    182,
                                    183,
                                    184,
                                    185,
                                    186,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    210,
                                    211,
                                    212,
                                    213,
                                    214,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    241,
                                    242,
                                    243,
                                    244,
                                    245,
                                    246,
                                    247,
                                    248,
                                    249,
                                    250,
                                    251,
                                    252,
                                    253,
                                    254,
                                    255,
                                    256,
                                    257,
                                    258,
                                    259,
                                    260,
                                    261,
                                    262,
                                    263,
                                    264,
                                    265,
                                    266,
                                    267,
                                    268,
                                    269,
                                    270,
                                    271,
                                    272,
                                    273,
                                    274,
                                    282,
                                    284,
                                    286,
                                    288,
                                    290,
                                    292,
                                    294,
                                    296,
                                    298,
                                    300,
                                    302,
                                    304,
                                    306,
                                    308,
                                    310,
                                    312,
                                    314,
                                    316,
                                    318,
                                    320,
                                    322,
                                    324,
                                    326,
                                    328,
                                    330,
                                    332,
                                    334,
                                    336,
                                    338,
                                    340,
                                    342,
                                    344,
                                    346,
                                    348,
                                    350,
                                    352,
                                    354,
                                    356,
                                    358,
                                    360,
                                    362,
                                    364,
                                    366,
                                    368,
                                    370,
                                    372,
                                    374,
                                    376,
                                    378,
                                    380,
                                    382,
                                    384,
                                    386,
                                    388,
                                    390,
                                    392,
                                    394,
                                    396,
                                    398,
                                    400,
                                    402,
                                    404,
                                    406,
                                    408,
                                    410
                                ],
                                "digital_pointing": {
                                    "angle1": 0.6624317181687094,
                                    "angle2": 1.5579526427549426,
                                    "target": "_target_name_",
                                    "direction_type": "J2000"
                                }
                            }
                        ],
                        "filter": "HBA_110_190",
                        "tile_beam": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "_target_name_",
                            "direction_type": "J2000"
                        },
                        "antenna_set": "HBA_DUAL",
                        "station_groups": [
                            {
                                "stations": [
                                    "DE601",
                                    "DE602",
                                    "DE603",
                                    "DE604",
                                    "DE605",
                                    "DE609",
                                    "FR606",
                                    "SE607",
                                    "UK608",
                                    "PL610",
                                    "PL611",
                                    "PL612",
                                    "IE613",
                                    "LV614"
                                ],
                                "max_nr_missing": 12
                            },
                            {
                                "stations": [
                                    "RS310",
                                    "RS509"
                                ],
                                "max_nr_missing": 2
                            }
                        ]
                    }
                },
                "specifications_template": "http://localhost:8008/api/task_template/11",
                "specifications_template_id": 11,
                "tags": [],
                "task_blueprints": [
                    "http://localhost:8008/api/task_blueprint/4"
                ],
                "task_blueprints_ids": [
                    4
                ],
                "task_type": "observation",
                "updated_at": "2022-05-30T05:48:44.333799"
            }
        
        ],
        "task_drafts_ids": [
            69
        ],
        "updated_at": "2022-05-26T05:21:25.938283"
    },
    {
        "id": 17,
        "url": "http://localhost:3000/api/scheduling_unit_draft/17",
        "created_at": "2022-06-17T05:53:55.002110",
        "description": "test",
        "duration": 120,
        "name": "test",
        "observation_strategy_template": "http://localhost:8008/api/scheduling_unit_observing_strategy_template/6",
        "observation_strategy_template_id": 6,
        "priority_queue_value": "A",
        "rank": 0,
        "scheduling_constraints_doc": {
            "sky": {
                "min_distance": {
                    "sun": 0,
                    "moon": 0,
                    "jupiter": 0
                },
                "min_elevation": {
                    "target": 0.10000736613927509,
                    "calibrator": 0.5000368306963754
                },
                "transit_offset": {
                    "to": 21600,
                    "from": -21600
                }
            },
            "time": {
                "between": [],
                "not_between": []
            },
            "daily": {
                "require_day": false,
                "require_night": false,
                "avoid_twilight": false
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
            "scheduler": "dynamic"
        },
        "scheduling_constraints_template": {
            "url": "http://localhost:3000/api/scheduling_constraints_template/1",
            "ref_resolved_schema": {
                "$id": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
                "type": "object",
                "title": "constraints",
                "$schema": "http://json-schema.org/draft-06/schema#",
                "default": {},
                "version": 1,
                "required": [
                    "scheduler"
                ],
                "properties": {
                    "sky": {
                        "type": "object",
                        "default": {},
                        "properties": {
                            "min_distance": {
                                "type": "object",
                                "default": {},
                                "properties": {
                                    "sun": {
                                        "$ref": "#/definitions/distance_on_sky",
                                        "default": 0.5
                                    },
                                    "moon": {
                                        "$ref": "#/definitions/distance_on_sky",
                                        "default": 0.5
                                    },
                                    "jupiter": {
                                        "$ref": "#/definitions/distance_on_sky",
                                        "default": 0.5
                                    }
                                },
                                "additionalProperties": false
                            },
                            "min_elevation": {
                                "type": "object",
                                "default": {},
                                "properties": {
                                    "target": {
                                        "$ref": "#/definitions/elevation",
                                        "default": 0.5,
                                        "description": "Minimum elevation for all target sources"
                                    },
                                    "calibrator": {
                                        "$ref": "#/definitions/elevation",
                                        "default": 0.5,
                                        "description": "Minimum elevation for all calibrator sources"
                                    }
                                },
                                "additionalProperties": false
                            },
                            "transit_offset": {
                                "type": "object",
                                "default": {},
                                "required": [
                                    "from",
                                    "to"
                                ],
                                "properties": {
                                    "to": {
                                        "$ref": "#/definitions/timedelta",
                                        "default": 7200,
                                        "maximum": 86400,
                                        "minimum": -86400
                                    },
                                    "from": {
                                        "$ref": "#/definitions/timedelta",
                                        "default": -7200,
                                        "maximum": 86400,
                                        "minimum": -86400
                                    }
                                },
                                "description": "Offset window to LST centering",
                                "additionalProperties": false
                            }
                        },
                        "additionalProperties": false
                    },
                    "time": {
                        "type": "object",
                        "default": {
                            "between": [],
                            "not_between": []
                        },
                        "properties": {
                            "at": {
                                "$ref": "#/definitions/timestamp",
                                "description": "Start at the specified date/time. Overrules dynamic scheduler priority. To be used only if really needed. Requires 'scheduler' to be set to 'fixed_time'."
                            },
                            "after": {
                                "$ref": "#/definitions/timestamp",
                                "description": "Start after this moment"
                            },
                            "before": {
                                "$ref": "#/definitions/timestamp",
                                "description": "End before this moment"
                            },
                            "between": {
                                "type": "array",
                                "items": {
                                    "$ref": "#/definitions/timewindow"
                                },
                                "default": [],
                                "minItems": 0,
                                "description": "Run within one of these time windows",
                                "uniqueItems": true
                            },
                            "not_between": {
                                "type": "array",
                                "items": {
                                    "$ref": "#/definitions/timewindow"
                                },
                                "default": [],
                                "minItems": 0,
                                "description": "Do NOT run within any of these time windows",
                                "uniqueItems": true
                            }
                        },
                        "additionalProperties": false
                    },
                    "daily": {
                        "type": "object",
                        "default": {
                            "require_day": false,
                            "require_night": false,
                            "avoid_twilight": false
                        },
                        "properties": {
                            "require_day": {
                                "type": "boolean",
                                "default": false,
                                "description": "Must run in daylight"
                            },
                            "require_night": {
                                "type": "boolean",
                                "default": false,
                                "description": "Must run at night"
                            },
                            "avoid_twilight": {
                                "type": "boolean",
                                "default": false,
                                "description": "Do not run during sunrise or sunset"
                            }
                        },
                        "additionalProperties": false
                    },
                    "scheduler": {
                        "enum": [
                            "fixed_time",
                            "dynamic"
                        ],
                        "name": "Scheduler",
                        "type": "string",
                        "default": "dynamic",
                        "description": "Schedule either at the fixed_time 'time.at' moment, of dynamically taking all time constraints into consideration."
                    }
                },
                "definitions": {
                    "elevation": {
                        "type": "number",
                        "maximum": 1.571,
                        "minimum": 0
                    },
                    "distance_on_sky": {
                        "type": "number",
                        "maximum": 3.142,
                        "minimum": 0
                    },
                    "timedelta": {
                        "default": 0,
                        "description": "A time duration or delta expressed in seconds",
                        "type": "number"
                    },
                    "timestamp": {
                        "default": "1970-01-01T00:00:00Z",
                        "description": "A timestamp defined in UTC",
                        "format": "date-time",
                        "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                        "type": "string"
                    },
                    "timewindow": {
                        "additionalProperties": false,
                        "description": "A timewindow interval: [from, to)",
                        "properties": {
                            "from": {
                                "$ref": "#/definitions/timestamp"
                            },
                            "to": {
                                "$ref": "#/definitions/timestamp"
                            }
                        },
                        "required": [
                            "from",
                            "to"
                        ],
                        "type": "object"
                    }
                },
                "description": "This schema defines the scheduling constraints for a scheduling unit",
                "patternProperties": {
                    "^[$]schema$": {}
                },
                "additionalProperties": false
            },
            "schema": {
                "$id": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1#",
                "type": "object",
                "title": "constraints",
                "$schema": "http://json-schema.org/draft-06/schema#",
                "default": {},
                "version": 1,
                "required": [
                    "scheduler"
                ],
                "properties": {
                    "sky": {
                        "type": "object",
                        "default": {},
                        "properties": {
                            "min_distance": {
                                "type": "object",
                                "default": {},
                                "properties": {
                                    "sun": {
                                        "$ref": "#/definitions/distance_on_sky",
                                        "default": 0.5
                                    },
                                    "moon": {
                                        "$ref": "#/definitions/distance_on_sky",
                                        "default": 0.5
                                    },
                                    "jupiter": {
                                        "$ref": "#/definitions/distance_on_sky",
                                        "default": 0.5
                                    }
                                },
                                "additionalProperties": false
                            },
                            "min_elevation": {
                                "type": "object",
                                "default": {},
                                "properties": {
                                    "target": {
                                        "$ref": "#/definitions/elevation",
                                        "default": 0.5,
                                        "description": "Minimum elevation for all target sources"
                                    },
                                    "calibrator": {
                                        "$ref": "#/definitions/elevation",
                                        "default": 0.5,
                                        "description": "Minimum elevation for all calibrator sources"
                                    }
                                },
                                "additionalProperties": false
                            },
                            "transit_offset": {
                                "type": "object",
                                "default": {},
                                "required": [
                                    "from",
                                    "to"
                                ],
                                "properties": {
                                    "to": {
                                        "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                        "default": 7200,
                                        "maximum": 86400,
                                        "minimum": -86400
                                    },
                                    "from": {
                                        "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                        "default": -7200,
                                        "maximum": 86400,
                                        "minimum": -86400
                                    }
                                },
                                "description": "Offset window to LST centering",
                                "additionalProperties": false
                            }
                        },
                        "additionalProperties": false
                    },
                    "time": {
                        "type": "object",
                        "default": {},
                        "properties": {
                            "at": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timestamp",
                                "description": "Start at the specified date/time. Overrules dynamic scheduler priority. To be used only if really needed. Requires 'scheduler' to be set to 'fixed_time'."
                            },
                            "after": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timestamp",
                                "description": "Start after this moment"
                            },
                            "before": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timestamp",
                                "description": "End before this moment"
                            },
                            "between": {
                                "type": "array",
                                "items": {
                                    "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timewindow"
                                },
                                "default": [],
                                "minItems": 0,
                                "description": "Run within one of these time windows",
                                "uniqueItems": true
                            },
                            "not_between": {
                                "type": "array",
                                "items": {
                                    "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timewindow"
                                },
                                "default": [],
                                "minItems": 0,
                                "description": "Do NOT run within any of these time windows",
                                "uniqueItems": true
                            }
                        },
                        "additionalProperties": false
                    },
                    "daily": {
                        "type": "object",
                        "default": {},
                        "properties": {
                            "require_day": {
                                "type": "boolean",
                                "default": false,
                                "description": "Must run in daylight"
                            },
                            "require_night": {
                                "type": "boolean",
                                "default": false,
                                "description": "Must run at night"
                            },
                            "avoid_twilight": {
                                "type": "boolean",
                                "default": false,
                                "description": "Do not run during sunrise or sunset"
                            }
                        },
                        "additionalProperties": false
                    },
                    "scheduler": {
                        "enum": [
                            "fixed_time",
                            "dynamic"
                        ],
                        "name": "Scheduler",
                        "type": "string",
                        "default": "dynamic",
                        "description": "Schedule either at the fixed_time 'time.at' moment, of dynamically taking all time constraints into consideration."
                    }
                },
                "definitions": {
                    "elevation": {
                        "type": "number",
                        "maximum": 1.571,
                        "minimum": 0
                    },
                    "distance_on_sky": {
                        "type": "number",
                        "maximum": 3.142,
                        "minimum": 0
                    }
                },
                "description": "This schema defines the scheduling constraints for a scheduling unit",
                "patternProperties": {
                    "^[$]schema$": {}
                },
                "additionalProperties": false
            }
        },
        "scheduling_constraints_template_id": 1,
        "scheduling_set": {
            "url": "http://localhost:3000/api/scheduling_set/1",
            "name": "Test Scheduling Set",
            "project_id": "high"
        },
        "scheduling_unit_blueprints": [],
        "scheduling_unit_blueprints_ids": [],
        "tags": [],
        "task_drafts": [
            {
                "id": 1,
                "url": "http://localhost:3000/api/task_draft/1",
                "connector_types": [
                    {
                        "id": 2,
                        "url": "http://localhost:3000/api/task_connector_type/2",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://localhost:3000/api/role/correlator",
                        "role_value": "correlator",
                        "task_template": "http://localhost:3000/api/task_template/6",
                        "task_template_id": 6
                    }
                ],
                "created_at": "2022-06-17T05:53:55.117168",
                "description": "A simple short test observation",
                "duration": 120,
                "name": "Observation",
                "output_pinned": false,
                "produced_by_ids": [],
                "relative_start_time": 0,
                "relative_stop_time": 120,
                "short_description": "",
                "specifications_doc": {
                    "QA": {
                        "plots": {
                            "enabled": true,
                            "autocorrelation": true,
                            "crosscorrelation": true
                        },
                        "file_conversion": {
                            "enabled": true,
                            "nr_of_subbands": -1,
                            "nr_of_timestamps": 256
                        },
                        "inspection_plots": "msplots"
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/2/ref_resolved",
                    "duration": 120,
                    "correlator": {
                        "storage_cluster": "CEP4",
                        "integration_time": 1,
                        "channels_per_subband": 64,
                        "topocentric_frequency_correction": false
                    },
                    "station_configuration": {
                        "SAPs": [
                            {
                                "name": "Polaris",
                                "subbands": [
                                    0,
                                    1,
                                    2,
                                    3,
                                    4,
                                    5,
                                    6,
                                    7,
                                    8,
                                    9,
                                    10,
                                    11,
                                    12,
                                    13,
                                    14,
                                    15,
                                    16,
                                    17,
                                    18,
                                    19,
                                    20,
                                    21,
                                    22,
                                    23,
                                    24,
                                    25,
                                    26,
                                    27,
                                    28,
                                    29,
                                    30,
                                    31,
                                    32,
                                    33,
                                    34,
                                    35,
                                    36,
                                    37,
                                    38,
                                    39,
                                    40,
                                    41,
                                    42,
                                    43,
                                    44,
                                    45,
                                    46,
                                    47,
                                    48,
                                    49,
                                    50,
                                    51,
                                    52,
                                    53,
                                    54,
                                    55,
                                    56,
                                    57,
                                    58,
                                    59,
                                    60,
                                    61,
                                    62,
                                    63,
                                    64,
                                    65,
                                    66,
                                    67,
                                    68,
                                    69,
                                    70,
                                    71,
                                    72,
                                    73,
                                    74,
                                    75,
                                    76,
                                    77,
                                    78,
                                    79,
                                    80,
                                    81,
                                    82,
                                    83,
                                    84,
                                    85,
                                    86,
                                    87,
                                    88,
                                    89,
                                    90,
                                    91,
                                    92,
                                    93,
                                    94,
                                    95,
                                    96,
                                    97,
                                    98,
                                    99,
                                    100,
                                    101,
                                    102,
                                    103,
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    137,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    164,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    181,
                                    182,
                                    183,
                                    184,
                                    185,
                                    186,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    210,
                                    211,
                                    212,
                                    213,
                                    214,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    241,
                                    242,
                                    243
                                ],
                                "digital_pointing": {
                                    "angle1": 0.6624317181687094,
                                    "angle2": 1.5579526427549426,
                                    "target": "target1",
                                    "direction_type": "J2000"
                                }
                            }
                        ],
                        "filter": "HBA_110_190",
                        "tile_beam": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "target1",
                            "direction_type": "J2000"
                        },
                        "antenna_set": "HBA_DUAL_INNER",
                        "station_groups": [
                            {
                                "stations": [
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                },
                "specifications_template": {
                    "id": 6,
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/2#",
                        "title": "target observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "correlator",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "QA": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/2#/definitions/QA",
                                "default": {}
                            },
                            "duration": {
                                "$id": "#duration",
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                "title": "Duration",
                                "default": 600,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "correlator": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/correlator/1#/definitions/correlator",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters to setup a target observation task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "type_value": "observation"
                },
                "tags": [],
                "task_blueprints_ids": [],
                "task_type": "observation",
                "updated_at": "2022-06-17T05:53:55.858755"
            }
        ],
        "updated_at": "2022-06-17T05:53:55.663285"
    },
    {
        "id": 18,
        "url": "http://localhost:3000/api/scheduling_unit_draft/18",
        "created_at": "2022-06-21T05:32:23.713380",
        "description": "sss",
        "duration": 120,
        "ingest_permission_required": true,
        "interrupts_telescope": false,
        "name": "ss",
        "observation_strategy_template": "http://localhost:3000/api/scheduling_unit_observing_strategy_template/7",
        "observation_strategy_template_id": 7,
        "piggyback_allowed_aartfaac": true,
        "piggyback_allowed_tbb": true,
        "priority_queue": "http://localhost:3000/api/priority_queue_type/A",
        "priority_queue_value": "A",
        "rank": 0,
        "scheduling_constraints_doc": {
            "sky": {
                "min_distance": {
                    "sun": 0,
                    "moon": 0,
                    "jupiter": 0
                },
                "min_elevation": {
                    "target": 0.10000736613927509,
                    "calibrator": 0.5000368306963754
                },
                "transit_offset": {
                    "to": 21600,
                    "from": -21600
                }
            },
            "time": {
                "between": [],
                "not_between": []
            },
            "daily": {
                "require_day": false,
                "require_night": false,
                "avoid_twilight": false
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
            "scheduler": "dynamic"
        },
        "scheduling_constraints_template": "http://localhost:3000/api/scheduling_constraints_template/1",
        "scheduling_constraints_template_id": 1,
        "scheduling_set": "http://localhost:3000/api/scheduling_set/1",
        "scheduling_set_id": 1,
        "scheduling_unit_blueprints": [],
        "scheduling_unit_blueprints_ids": [],
        "specifications_template": "http://localhost:3000/api/scheduling_unit_template/1",
        "specifications_template_id": 1,
        "tags": [],
        "task_drafts": [
            {
                "id": 71,
                "url": "http://localhost:3000/api/task_draft/71",
                "connector_types": [
                    {
                        "id": 2,
                        "url": "http://localhost:3000/api/task_connector_type/2",
                        "dataformat": "http://localhost:3000/api/dataformat/MeasurementSet",
                        "dataformat_value": "MeasurementSet",
                        "datatype": "http://localhost:3000/api/datatype/visibilities",
                        "datatype_value": "visibilities",
                        "iotype": "http://localhost:3000/api/iotype/output",
                        "iotype_value": "output",
                        "role": "http://localhost:3000/api/role/correlator",
                        "role_value": "correlator",
                        "task_template": "http://localhost:3000/api/task_template/7",
                        "task_template_id": 7
                    }
                ],
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-06-21T05:32:23.844672",
                "description": "A simple short test observation",
                "duration": 120,
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Observation",
                "output_pinned": false,
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 0,
                "relative_stop_time": 120,
                "scheduling_unit_draft": "http://localhost:3000/api/scheduling_unit_draft/1",
                "scheduling_unit_draft_id": 1,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "QA": {
                        "plots": {
                            "enabled": true,
                            "autocorrelation": true,
                            "crosscorrelation": true
                        },
                        "file_conversion": {
                            "enabled": true,
                            "nr_of_subbands": -1,
                            "nr_of_timestamps": 256
                        },
                        "inspection_plots": "msplots"
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/2/ref_resolved",
                    "duration": 120,
                    "correlator": {
                        "storage_cluster": "CEP4",
                        "integration_time": 1,
                        "channels_per_subband": 64,
                        "topocentric_frequency_correction": false
                    },
                    "station_configuration": {
                        "SAPs": [
                            {
                                "name": "Polaris",
                                "subbands": [
                                    0,
                                    1,
                                    2,
                                    3,
                                    4,
                                    5,
                                    6,
                                    7,
                                    8,
                                    9,
                                    10,
                                    11,
                                    12,
                                    13,
                                    14,
                                    15,
                                    16,
                                    17,
                                    18,
                                    19,
                                    20,
                                    21,
                                    22,
                                    23,
                                    24,
                                    25,
                                    26,
                                    27,
                                    28,
                                    29,
                                    30,
                                    31,
                                    32,
                                    33,
                                    34,
                                    35,
                                    36,
                                    37,
                                    38,
                                    39,
                                    40,
                                    41,
                                    42,
                                    43,
                                    44,
                                    45,
                                    46,
                                    47,
                                    48,
                                    49,
                                    50,
                                    51,
                                    52,
                                    53,
                                    54,
                                    55,
                                    56,
                                    57,
                                    58,
                                    59,
                                    60,
                                    61,
                                    62,
                                    63,
                                    64,
                                    65,
                                    66,
                                    67,
                                    68,
                                    69,
                                    70,
                                    71,
                                    72,
                                    73,
                                    74,
                                    75,
                                    76,
                                    77,
                                    78,
                                    79,
                                    80,
                                    81,
                                    82,
                                    83,
                                    84,
                                    85,
                                    86,
                                    87,
                                    88,
                                    89,
                                    90,
                                    91,
                                    92,
                                    93,
                                    94,
                                    95,
                                    96,
                                    97,
                                    98,
                                    99,
                                    100,
                                    101,
                                    102,
                                    103,
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    137,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    164,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    181,
                                    182,
                                    183,
                                    184,
                                    185,
                                    186,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    210,
                                    211,
                                    212,
                                    213,
                                    214,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    241,
                                    242,
                                    243
                                ],
                                "digital_pointing": {
                                    "angle1": 0.6624317181687094,
                                    "angle2": 1.5579526427549426,
                                    "target": "target1",
                                    "direction_type": "J2000"
                                }
                            }
                        ],
                        "filter": "HBA_110_190",
                        "tile_beam": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "target1",
                            "direction_type": "J2000"
                        },
                        "antenna_set": "HBA_DUAL_INNER",
                        "station_groups": [
                            {
                                "stations": [
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                },
                "specifications_template": {
                    "id": 7,
                    "url": "http://localhost:3000/api/task_template/7",
                    "created_at": "2022-06-21T05:29:02.199678",
                    "description": "This schema defines the parameters to setup a target observation task.",
                    "name": "target observation",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/2/ref_resolved",
                        "title": "target observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "correlator",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "QA": {
                                "$ref": "#/definitions/QA",
                                "default": {}
                            },
                            "duration": {
                                "$id": "#duration",
                                "$ref": "#/definitions/timedelta",
                                "title": "Duration",
                                "default": 600,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "correlator": {
                                "$ref": "#/definitions/correlator",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters to setup a target observation task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "QA": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Perform all Quality Assurance (QA) tasks, including file conversion and plotting.",
                                "properties": {
                                    "file_conversion": {
                                        "$ref": "#/definitions/file_conversion",
                                        "default": {}
                                    },
                                    "inspection_plots": {
                                        "$ref": "#/definitions/inspection_plots",
                                        "default": "msplots"
                                    },
                                    "plots": {
                                        "$ref": "#/definitions/plots",
                                        "default": {}
                                    }
                                },
                                "required": [
                                    "file_conversion",
                                    "plots",
                                    "inspection_plots"
                                ],
                                "title": "QA",
                                "type": "object"
                            },
                            "plots": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Create dynamic spectrum plots",
                                "properties": {
                                    "autocorrelation": {
                                        "default": true,
                                        "description": "Create autocorrelation plots for all stations",
                                        "title": "autocorrelation",
                                        "type": "boolean"
                                    },
                                    "crosscorrelation": {
                                        "default": true,
                                        "description": "Create crosscorrelation plots for all baselines",
                                        "title": "crosscorrelation",
                                        "type": "boolean"
                                    },
                                    "enabled": {
                                        "default": true,
                                        "description": "Do/Don't create plots from the QA file from the observation",
                                        "title": "enabled",
                                        "type": "boolean"
                                    }
                                },
                                "required": [
                                    "enabled",
                                    "autocorrelation",
                                    "crosscorrelation"
                                ],
                                "title": "Plots",
                                "type": "object"
                            },
                            "inspection_plots": {
                                "decription": "Type of inspection plots to run",
                                "default": "msplots",
                                "enum": [
                                    "msplots",
                                    "dynspec",
                                    "none"
                                ],
                                "title": "Inspection Plots",
                                "type": "string"
                            },
                            "file_conversion": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Create a QA file for the observation",
                                "properties": {
                                    "enabled": {
                                        "default": true,
                                        "description": "Do/Don't create a QA file for the observation",
                                        "title": "enabled",
                                        "type": "boolean"
                                    },
                                    "nr_of_subbands": {
                                        "default": -1,
                                        "description": "Keep this number of subbands from the observation in the QA file, or all if -1",
                                        "title": "#subbands",
                                        "type": "integer"
                                    },
                                    "nr_of_timestamps": {
                                        "default": 256,
                                        "description": "Extract this number of timestamps from the observation in the QA file (equidistantanly sampled, no averaging/interpolation)",
                                        "minimum": 1,
                                        "title": "#timestamps",
                                        "type": "integer"
                                    }
                                },
                                "required": [
                                    "enabled",
                                    "nr_of_subbands",
                                    "nr_of_timestamps"
                                ],
                                "title": "File Conversion",
                                "type": "object"
                            },
                            "timedelta": {
                                "default": 0,
                                "description": "A time duration or delta expressed in seconds",
                                "type": "number"
                            },
                            "timestamp": {
                                "default": "1970-01-01T00:00:00Z",
                                "description": "A timestamp defined in UTC",
                                "format": "date-time",
                                "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                                "type": "string"
                            },
                            "correlator": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "channels_per_subband": {
                                        "default": 64,
                                        "description": "Number of frequency bands per subband",
                                        "enum": [
                                            8,
                                            16,
                                            32,
                                            64,
                                            128,
                                            256,
                                            512,
                                            1024
                                        ],
                                        "minimum": 8,
                                        "title": "Channels/subband",
                                        "type": "integer"
                                    },
                                    "integration_time": {
                                        "$ref": "#/definitions/timedelta",
                                        "default": 1,
                                        "description": "Desired integration period (seconds)",
                                        "minimum": 0.1,
                                        "title": "Integration time"
                                    },
                                    "storage_cluster": {
                                        "default": "CEP4",
                                        "description": "Cluster to write output to",
                                        "enum": [
                                            "CEP4",
                                            "DragNet"
                                        ],
                                        "title": "Storage cluster",
                                        "type": "string"
                                    },
                                    "topocentric_frequency_correction": {
                                        "default": false,
                                        "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation.",
                                        "title": "Topocentric Frequency Correction",
                                        "type": "boolean"
                                    }
                                }
                            },
                            "station_configuration": {
                                "default": {},
                                "properties": {
                                    "SAPs": {
                                        "$ref": "#/definitions/SAPs",
                                        "default": [
                                            {}
                                        ],
                                        "minItems": 1
                                    },
                                    "antenna_set": {
                                        "$ref": "#/definitions/antenna_set",
                                        "default": "HBA_DUAL"
                                    },
                                    "filter": {
                                        "$ref": "#/definitions/filter",
                                        "default": "HBA_110_190"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 4,
                                                "stations": [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS101",
                                                    "CS103",
                                                    "CS201",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501"
                                                ]
                                            }
                                        ]
                                    },
                                    "tile_beam": {
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "description": "HBA only",
                                        "title": "Tile beam"
                                    }
                                },
                                "required": [
                                    "station_groups",
                                    "antenna_set",
                                    "filter",
                                    "SAPs"
                                ],
                                "title": "station_configuration",
                                "type": "object"
                            },
                            "max_number_of_missing_stations": {
                                "default": 0,
                                "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                                "minimum": 0,
                                "title": "Maximum number of stations to omit",
                                "type": "integer"
                            },
                            "station_list": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [],
                                "items": {
                                    "$ref": "#/definitions/station"
                                },
                                "minItems": 0,
                                "type": "array",
                                "uniqueItems": true
                            },
                            "antenna_set": {
                                "default": "HBA_DUAL",
                                "description": "Fields & antennas to use",
                                "enum": [
                                    "HBA_DUAL",
                                    "HBA_DUAL_INNER",
                                    "HBA_ONE",
                                    "HBA_ONE_INNER",
                                    "HBA_ZERO",
                                    "HBA_ZERO_INNER",
                                    "LBA_INNER",
                                    "LBA_OUTER",
                                    "LBA_SPARSE_EVEN",
                                    "LBA_SPARSE_ODD",
                                    "LBA_ALL"
                                ],
                                "title": "Antenna set",
                                "type": "string"
                            },
                            "filter": {
                                "default": "HBA_110_190",
                                "description": "Must match antenna type",
                                "enum": [
                                    "LBA_10_70",
                                    "LBA_30_70",
                                    "LBA_10_90",
                                    "LBA_30_90",
                                    "HBA_110_190",
                                    "HBA_210_250"
                                ],
                                "title": "Band-pass filter",
                                "type": "string"
                            },
                            "SAPs": {
                                "additionalItems": false,
                                "default": [
                                    {}
                                ],
                                "description": "Station beams",
                                "items": {
                                    "additionalProperties": false,
                                    "default": {},
                                    "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                    "properties": {
                                        "digital_pointing": {
                                            "$id": "#target_pointing",
                                            "$ref": "#/definitions/pointing",
                                            "default": {},
                                            "title": "Digital pointing"
                                        },
                                        "name": {
                                            "default": "_SAP_name_",
                                            "description": "Identifier for this beam",
                                            "minLength": 1,
                                            "title": "Name",
                                            "type": "string"
                                        },
                                        "subbands": {
                                            "additionalItems": false,
                                            "default": [
                                                255
                                            ],
                                            "items": {
                                                "maximum": 511,
                                                "minimum": 0,
                                                "title": "Subband",
                                                "type": "integer"
                                            },
                                            "maxItems": 488,
                                            "minItems": 1,
                                            "title": "Subband list",
                                            "type": "array"
                                        }
                                    },
                                    "required": [
                                        "name",
                                        "digital_pointing",
                                        "subbands"
                                    ],
                                    "title": "SAP",
                                    "type": "object"
                                },
                                "minItems": 0,
                                "title": "SAPs",
                                "type": "array"
                            },
                            "pointing": {
                                "additionalProperties": false,
                                "properties": {
                                    "angle1": {
                                        "default": 0.6624317181687094,
                                        "description": "First angle (e.g. RA)",
                                        "title": "Angle 1",
                                        "type": "number"
                                    },
                                    "angle2": {
                                        "default": 1.5579526427549426,
                                        "description": "Second angle (e.g. DEC)",
                                        "title": "Angle 2",
                                        "type": "number"
                                    },
                                    "direction_type": {
                                        "default": "J2000",
                                        "description": "",
                                        "enum": [
                                            "J2000",
                                            "AZELGEO",
                                            "LMN",
                                            "SUN",
                                            "MOON",
                                            "MERCURY",
                                            "VENUS",
                                            "MARS",
                                            "JUPITER",
                                            "SATURN",
                                            "URANUS",
                                            "NEPTUNE",
                                            "PLUTO"
                                        ],
                                        "title": "Reference frame",
                                        "type": "string"
                                    },
                                    "target": {
                                        "default": "_target_name_",
                                        "description": "Description of where this beam points at",
                                        "minLength": 1,
                                        "title": "Target",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "angle1",
                                    "angle2",
                                    "target"
                                ],
                                "type": "object"
                            },
                            "station_groups": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [
                                    {
                                        "max_nr_missing": 1,
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ]
                                    }
                                ],
                                "description": "One or more predefined or custom groups of stations",
                                "items": {
                                    "$ref": "#/definitions/station_group"
                                },
                                "minItems": 1,
                                "title": "Station groups",
                                "type": "array"
                            },
                            "station": {
                                "description": "These are the LOFAR stations",
                                "enum": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501",
                                    "RS106",
                                    "RS205",
                                    "RS208",
                                    "RS210",
                                    "RS305",
                                    "RS306",
                                    "RS307",
                                    "RS310",
                                    "RS406",
                                    "RS407",
                                    "RS409",
                                    "RS503",
                                    "RS508",
                                    "RS509",
                                    "DE601",
                                    "DE602",
                                    "DE603",
                                    "DE604",
                                    "DE605",
                                    "FR606",
                                    "SE607",
                                    "UK608",
                                    "DE609",
                                    "PL610",
                                    "PL611",
                                    "PL612",
                                    "IE613",
                                    "LV614"
                                ],
                                "title": "Station",
                                "type": "string"
                            },
                            "station_group": {
                                "anyOf": [
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007"
                                            ]
                                        },
                                        "description": "The group of all stations on the Superterp",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Superterp",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501"
                                            ]
                                        },
                                        "description": "The group of all Core stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Core",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch remote stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Remote",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch (Core + Remote) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Dutch",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 2,
                                            "stations": [
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 1,
                                            "stations": [
                                                "DE601",
                                                "DE605"
                                            ]
                                        },
                                        "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE605"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International required",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 6,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509",
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all (Core + Remote + International) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509",
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "All",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS001"
                                            ]
                                        },
                                        "description": "A custom group of stations which can be defined by the user",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list"
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Custom",
                                        "type": "object"
                                    }
                                ],
                                "default": {
                                    "max_nr_missing": 1,
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                },
                                "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                                "title": "Station group",
                                "type": "object"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/2#",
                        "title": "target observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "correlator",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "QA": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/2#/definitions/QA",
                                "default": {}
                            },
                            "duration": {
                                "$id": "#duration",
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                "title": "Duration",
                                "default": 600,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "correlator": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/correlator/1#/definitions/correlator",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters to setup a target observation task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/observation",
                    "type_value": "observation",
                    "updated_at": "2022-06-21T05:29:02.199834",
                    "version": 2
                },
                "specifications_template_id": 7,
                "tags": [],
                "task_blueprints": [],
                "task_blueprints_ids": [],
                "task_type": "observation",
                "updated_at": "2022-06-21T05:32:24.460594"
            }
        ],
        "task_drafts_ids": [
            1
        ],
        "updated_at": "2022-06-21T05:32:24.276417"
    }
  ],
  schedulingUnitBlueprints: [
    {
      "id": 1,
      "url": "http://localhost:3000/api/scheduling_unit_blueprint/1",
      "actual_on_sky_start_time": null,
      "actual_on_sky_stop_time": null,
      "actual_process_start_time": null,
      "actual_process_stop_time": null,
      "created_at": "2022-04-14T06:20:19.627266",
      "description": "Test scheduling unit",
      "draft": "http://localhost:3000/api/scheduling_unit_draft/1",
      "draft_id": 1,
      "duration": 1740,
      "error_reason": "",
      "global_identifier": "http://localhost:3000/api/sip_identifier/1",
      "global_identifier_id": 1,
      "ingest_permission_granted_since": null,
      "ingest_permission_required": true,
      "interrupts_telescope": false,
      "name": "IM LBA LoDSS - 5 Beams",
      "observed_duration": null,
      "observed_start_time": null,
      "observed_stop_time": null,
      "obsolete_since": null,
      "on_sky_duration": 120,
      "on_sky_start_time": "2022-04-14T06:25:19.054619",
      "on_sky_stop_time": "2022-04-14T06:27:19.054619",
      "output_pinned": false,
      "piggyback_allowed_aartfaac": true,
      "piggyback_allowed_tbb": true,
      "priority_queue": "http://localhost:3000/api/priority_queue_type/A",
      "priority_queue_value": "A",
      "process_start_time": "2022-04-14T06:25:19.054619",
      "process_stop_time": "2022-04-14T06:37:19.054619",
      "rank": 1,
      "scheduled_start_time": "2022-04-14T06:25:19.054619",
      "scheduled_stop_time": "2022-04-14T06:37:19.054619",
      "scheduling_constraints_doc": {
          "sky": {
              "min_distance": {
                  "sun": 0.5,
                  "moon": 0.5,
                  "jupiter": 0.5
              },
              "min_elevation": {
                  "target": 0.5,
                  "calibrator": 0.5
              },
              "transit_offset": {
                  "to": 7200,
                  "from": -7200
              }
          },
          "time": {
              "at": "2022-04-14T06:25:19.054619"
          },
          "daily": {
              "require_day": false,
              "require_night": false,
              "avoid_twilight": false
          },
          "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
          "scheduler": "fixed_time"
      },
      "scheduling_constraints_template": "http://localhost:3000/api/scheduling_constraints_template/1",
      "scheduling_constraints_template_id": 1,
      "specifications_template": "http://localhost:3000/api/scheduling_unit_template/1",
      "specifications_template_id": 1,
      "status": "scheduled",
      "status_value": "scheduled",
      "tags": [],
      "task_blueprints": [
          {
              "id": 5,
              "url": "http://localhost:3000/api/task_blueprint/5",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_blueprint/10"
              ],
              "consumed_by_ids": [
                  10
              ],
              "created_at": "2022-04-14T06:20:19.856020",
              "description": "Preprocessing Pipeline for Target Observation target3",
              "draft": "http://localhost:3000/api/task_draft/5",
              "draft_id": 5,
              "duration": 120,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target3",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:27:19.054619",
              "process_stop_time": "2022-04-14T06:29:19.054619",
              "produced_by": [
                  "http://localhost:3000/api/task_relation_blueprint/4"
              ],
              "produced_by_ids": [
                  4
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-04-14T06:27:19.054619",
              "scheduled_stop_time": "2022-04-14T06:29:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
              "scheduling_unit_blueprint_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Paaa+03/1.2/PP",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 10,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 1,
                      "frequency_steps": 1
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000004"
              ],
              "subtasks_ids": [
                  2000004
              ],
              "tags": [],
              "task_type": "pipeline",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:19.856102"
          },
          {
              "id": 6,
              "url": "http://localhost:3000/api/task_blueprint/6",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_blueprint/9"
              ],
              "consumed_by_ids": [
                  9
              ],
              "created_at": "2022-04-14T06:20:19.899035",
              "description": "Preprocessing Pipeline for Target Observation target2",
              "draft": "http://localhost:3000/api/task_draft/6",
              "draft_id": 6,
              "duration": 120,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target2",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:27:19.054619",
              "process_stop_time": "2022-04-14T06:29:19.054619",
              "produced_by": [
                  "http://localhost:3000/api/task_relation_blueprint/5"
              ],
              "produced_by_ids": [
                  5
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-04-14T06:27:19.054619",
              "scheduled_stop_time": "2022-04-14T06:29:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
              "scheduling_unit_blueprint_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Paaa+02/1.1/PP",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 10,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 1,
                      "frequency_steps": 1
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000008"
              ],
              "subtasks_ids": [
                  2000008
              ],
              "tags": [],
              "task_type": "pipeline",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:19.899049"
          },
          {
              "id": 2,
              "url": "http://localhost:3000/api/task_blueprint/2",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_blueprint/7"
              ],
              "consumed_by_ids": [
                  7
              ],
              "created_at": "2022-04-14T06:20:19.736118",
              "description": "Preprocessing Pipeline for Calibrator Observation",
              "draft": "http://localhost:3000/api/task_draft/2",
              "draft_id": 2,
              "duration": 120,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Calibrator Pipeline",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:27:19.054619",
              "process_stop_time": "2022-04-14T06:29:19.054619",
              "produced_by": [
                  "http://localhost:3000/api/task_relation_blueprint/1"
              ],
              "produced_by_ids": [
                  1
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-04-14T06:27:19.054619",
              "scheduled_stop_time": "2022-04-14T06:29:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
              "scheduling_unit_blueprint_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "3Cabc/1.0/CP",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 10,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 1,
                      "frequency_steps": 1
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000006"
              ],
              "subtasks_ids": [
                  2000006
              ],
              "tags": [],
              "task_type": "pipeline",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:19.736130"
          },
          {
              "id": 7,
              "url": "http://localhost:3000/api/task_blueprint/7",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_blueprint/8"
              ],
              "consumed_by_ids": [
                  8
              ],
              "created_at": "2022-04-14T06:20:19.939457",
              "description": "Preprocessing Pipeline for Target Observation target1",
              "draft": "http://localhost:3000/api/task_draft/7",
              "draft_id": 7,
              "duration": 120,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target1",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:27:19.054619",
              "process_stop_time": "2022-04-14T06:29:19.054619",
              "produced_by": [
                  "http://localhost:3000/api/task_relation_blueprint/6"
              ],
              "produced_by_ids": [
                  6
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-04-14T06:27:19.054619",
              "scheduled_stop_time": "2022-04-14T06:29:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
              "scheduling_unit_blueprint_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Paaa+01/1.0/PP",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 10,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 1,
                      "frequency_steps": 1
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000007"
              ],
              "subtasks_ids": [
                  2000007
              ],
              "tags": [],
              "task_type": "pipeline",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:19.939486"
          },
          {
              "id": 1,
              "url": "http://localhost:3000/api/task_blueprint/1",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_blueprint/1",
                  "http://localhost:3000/api/task_relation_blueprint/2",
                  "http://localhost:3000/api/task_relation_blueprint/3",
                  "http://localhost:3000/api/task_relation_blueprint/4",
                  "http://localhost:3000/api/task_relation_blueprint/5",
                  "http://localhost:3000/api/task_relation_blueprint/6"
              ],
              "consumed_by_ids": [
                  1,
                  2,
                  3,
                  4,
                  5,
                  6
              ],
              "created_at": "2022-04-14T06:20:19.668460",
              "description": "Combined parallel Calibrator & Target Observation for LBA with 5 target beams",
              "draft": "http://localhost:3000/api/task_draft/1",
              "draft_id": 1,
              "duration": 720,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Combined Observation",
              "obsolete_since": null,
              "on_sky_duration": 120,
              "on_sky_start_time": "2022-04-14T06:25:19.054619",
              "on_sky_stop_time": "2022-04-14T06:27:19.054619",
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:25:19.054619",
              "process_stop_time": "2022-04-14T06:37:19.054619",
              "produced_by": [],
              "produced_by_ids": [],
              "relative_start_time": 0,
              "relative_stop_time": 120,
              "scheduled_start_time": "2022-04-14T06:25:19.054619",
              "scheduled_stop_time": "2022-04-14T06:37:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
              "scheduling_unit_blueprint_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "oXXX Paaa+01 3Cabc",
              "specifications_doc": {
                  "QA": {
                      "plots": {
                          "enabled": true,
                          "autocorrelation": true,
                          "crosscorrelation": true
                      },
                      "file_conversion": {
                          "enabled": true,
                          "nr_of_subbands": -1,
                          "nr_of_timestamps": 256
                      },
                      "inspection_plots": "msplots"
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                  "duration": 120,
                  "calibrator": {
                      "name": "calibrator",
                      "pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "_calibrator_name_",
                          "direction_type": "J2000"
                      },
                      "autoselect": false
                  },
                  "correlator": {
                      "storage_cluster": "CEP4",
                      "integration_time": 1,
                      "channels_per_subband": 64,
                      "topocentric_frequency_correction": false
                  },
                  "station_configuration": {
                      "SAPs": [
                          {
                              "name": "target1",
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+01",
                                  "direction_type": "J2000"
                              }
                          },
                          {
                              "name": "target2",
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+02",
                                  "direction_type": "J2000"
                              }
                          },
                          {
                              "name": "target3",
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+03",
                                  "direction_type": "J2000"
                              }
                          },
                          {
                              "name": "target4",
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+04",
                                  "direction_type": "J2000"
                              }
                          },
                          {
                              "name": "target5",
                              "subbands": [
                                  74,
                                  75,
                                  76,
                                  77,
                                  78,
                                  79,
                                  80,
                                  81,
                                  82,
                                  83,
                                  84,
                                  85,
                                  86,
                                  87,
                                  88,
                                  89,
                                  90,
                                  91,
                                  92,
                                  93,
                                  94,
                                  95,
                                  96,
                                  97,
                                  98,
                                  99,
                                  100,
                                  101,
                                  102,
                                  103,
                                  104,
                                  105,
                                  106,
                                  107,
                                  108,
                                  109,
                                  110,
                                  111,
                                  112,
                                  113,
                                  114,
                                  115,
                                  116,
                                  117,
                                  118,
                                  119,
                                  120,
                                  121,
                                  122,
                                  123,
                                  124,
                                  125,
                                  126,
                                  127,
                                  128,
                                  129,
                                  130,
                                  131,
                                  132,
                                  133,
                                  134,
                                  135,
                                  136,
                                  137,
                                  138,
                                  139,
                                  140,
                                  141,
                                  142,
                                  143,
                                  144,
                                  145,
                                  146,
                                  147,
                                  148,
                                  149,
                                  150,
                                  151,
                                  152,
                                  153,
                                  154
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+05",
                                  "direction_type": "J2000"
                              }
                          }
                      ],
                      "filter": "LBA_10_90",
                      "tile_beam": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "_target_name_",
                          "direction_type": "J2000"
                      },
                      "antenna_set": "LBA_SPARSE_EVEN",
                      "station_groups": [
                          {
                              "stations": [
                                  "CS001",
                                  "CS002",
                                  "CS003",
                                  "CS004",
                                  "CS005",
                                  "CS006",
                                  "CS007",
                                  "CS011",
                                  "CS013",
                                  "CS017",
                                  "CS021",
                                  "CS024",
                                  "CS026",
                                  "CS028",
                                  "CS030",
                                  "CS031",
                                  "CS032",
                                  "CS101",
                                  "CS103",
                                  "CS201",
                                  "CS301",
                                  "CS302",
                                  "CS401",
                                  "CS501",
                                  "RS106",
                                  "RS205",
                                  "RS208",
                                  "RS210",
                                  "RS305",
                                  "RS306",
                                  "RS307",
                                  "RS310",
                                  "RS406",
                                  "RS407",
                                  "RS409",
                                  "RS503",
                                  "RS508",
                                  "RS509"
                              ],
                              "max_nr_missing": 4
                          },
                          {
                              "stations": [
                                  "RS508",
                                  "RS509"
                              ],
                              "max_nr_missing": 1
                          },
                          {
                              "stations": [
                                  "RS310",
                                  "RS210"
                              ],
                              "max_nr_missing": 0
                          }
                      ]
                  }
              },
              "specifications_template": {
                  "id": 5,
                  "url": "http://localhost:3000/api/task_template/5",
                  "created_at": "2022-04-14T06:20:07.273898",
                  "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                  "name": "parallel calibrator target observation",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                      "type": "object",
                      "title": "parallel calibrator target observation",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "calibrator",
                          "correlator",
                          "duration",
                          "station_configuration"
                      ],
                      "properties": {
                          "QA": {
                              "$ref": "#/definitions/QA",
                              "default": {}
                          },
                          "duration": {
                              "$id": "#duration",
                              "$ref": "#/definitions/timedelta",
                              "title": "Duration",
                              "default": 600,
                              "minimum": 1,
                              "description": "Duration of this observation (seconds)"
                          },
                          "calibrator": {
                              "$ref": "#/definitions/calibrator",
                              "default": {}
                          },
                          "correlator": {
                              "$ref": "#/definitions/correlator",
                              "default": {}
                          },
                          "station_configuration": {
                              "$ref": "#/definitions/station_configuration",
                              "default": {}
                          }
                      },
                      "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "QA": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Perform all Quality Assurance (QA) tasks, including file conversion and plotting.",
                              "properties": {
                                  "file_conversion": {
                                      "$ref": "#/definitions/file_conversion",
                                      "default": {}
                                  },
                                  "inspection_plots": {
                                      "$ref": "#/definitions/inspection_plots",
                                      "default": "msplots"
                                  },
                                  "plots": {
                                      "$ref": "#/definitions/plots",
                                      "default": {}
                                  }
                              },
                              "required": [
                                  "file_conversion",
                                  "plots",
                                  "inspection_plots"
                              ],
                              "title": "QA",
                              "type": "object"
                          },
                          "plots": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Create dynamic spectrum plots",
                              "properties": {
                                  "autocorrelation": {
                                      "default": true,
                                      "description": "Create autocorrelation plots for all stations",
                                      "title": "autocorrelation",
                                      "type": "boolean"
                                  },
                                  "crosscorrelation": {
                                      "default": true,
                                      "description": "Create crosscorrelation plots for all baselines",
                                      "title": "crosscorrelation",
                                      "type": "boolean"
                                  },
                                  "enabled": {
                                      "default": true,
                                      "description": "Do/Don't create plots from the QA file from the observation",
                                      "title": "enabled",
                                      "type": "boolean"
                                  }
                              },
                              "required": [
                                  "enabled",
                                  "autocorrelation",
                                  "crosscorrelation"
                              ],
                              "title": "Plots",
                              "type": "object"
                          },
                          "file_conversion": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Create a QA file for the observation",
                              "properties": {
                                  "enabled": {
                                      "default": true,
                                      "description": "Do/Don't create a QA file for the observation",
                                      "title": "enabled",
                                      "type": "boolean"
                                  },
                                  "nr_of_subbands": {
                                      "default": -1,
                                      "description": "Keep this number of subbands from the observation in the QA file, or all if -1",
                                      "title": "#subbands",
                                      "type": "integer"
                                  },
                                  "nr_of_timestamps": {
                                      "default": 256,
                                      "description": "Extract this number of timestamps from the observation in the QA file (equidistantanly sampled, no averaging/interpolation)",
                                      "minimum": 1,
                                      "title": "#timestamps",
                                      "type": "integer"
                                  }
                              },
                              "required": [
                                  "enabled",
                                  "nr_of_subbands",
                                  "nr_of_timestamps"
                              ],
                              "title": "File Conversion",
                              "type": "object"
                          },
                          "inspection_plots": {
                              "decription": "Type of inspection plots to run",
                              "default": "msplots",
                              "enum": [
                                  "msplots",
                                  "dynspec",
                                  "none"
                              ],
                              "title": "Inspection Plots",
                              "type": "string"
                          },
                          "timedelta": {
                              "default": 0,
                              "description": "A time duration or delta expressed in seconds",
                              "type": "number"
                          },
                          "timestamp": {
                              "default": "1970-01-01T00:00:00Z",
                              "description": "A timestamp defined in UTC",
                              "format": "date-time",
                              "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                              "type": "string"
                          },
                          "calibrator": {
                              "additionalProperties": false,
                              "default": {},
                              "properties": {
                                  "autoselect": {
                                      "default": true,
                                      "description": "Auto-select calibrator based on elevation",
                                      "title": "Auto-select",
                                      "type": "boolean"
                                  },
                                  "name": {
                                      "default": "calibrator",
                                      "description": "Name of the calibrator SAP",
                                      "title": "Name",
                                      "type": "string"
                                  },
                                  "pointing": {
                                      "$ref": "#/definitions/pointing",
                                      "default": {},
                                      "description": "Manually selected calibrator",
                                      "title": "Digital pointing"
                                  }
                              },
                              "required": [
                                  "autoselect",
                                  "pointing",
                                  "name"
                              ],
                              "type": "object"
                          },
                          "pointing": {
                              "additionalProperties": false,
                              "properties": {
                                  "angle1": {
                                      "default": 0.6624317181687094,
                                      "description": "First angle (e.g. RA)",
                                      "title": "Angle 1",
                                      "type": "number"
                                  },
                                  "angle2": {
                                      "default": 1.5579526427549426,
                                      "description": "Second angle (e.g. DEC)",
                                      "title": "Angle 2",
                                      "type": "number"
                                  },
                                  "direction_type": {
                                      "default": "J2000",
                                      "description": "",
                                      "enum": [
                                          "J2000",
                                          "AZELGEO",
                                          "LMN",
                                          "SUN",
                                          "MOON",
                                          "MERCURY",
                                          "VENUS",
                                          "MARS",
                                          "JUPITER",
                                          "SATURN",
                                          "URANUS",
                                          "NEPTUNE",
                                          "PLUTO"
                                      ],
                                      "title": "Reference frame",
                                      "type": "string"
                                  },
                                  "target": {
                                      "default": "_target_name_",
                                      "description": "Description of where this beam points at",
                                      "minLength": 1,
                                      "title": "Target",
                                      "type": "string"
                                  }
                              },
                              "required": [
                                  "angle1",
                                  "angle2",
                                  "target"
                              ],
                              "type": "object"
                          },
                          "correlator": {
                              "additionalProperties": false,
                              "default": {},
                              "properties": {
                                  "channels_per_subband": {
                                      "default": 64,
                                      "description": "Number of frequency bands per subband",
                                      "enum": [
                                          8,
                                          16,
                                          32,
                                          64,
                                          128,
                                          256,
                                          512,
                                          1024
                                      ],
                                      "minimum": 8,
                                      "title": "Channels/subband",
                                      "type": "integer"
                                  },
                                  "integration_time": {
                                      "$ref": "#/definitions/timedelta",
                                      "default": 1,
                                      "description": "Desired integration period (seconds)",
                                      "minimum": 0.1,
                                      "title": "Integration time"
                                  },
                                  "storage_cluster": {
                                      "default": "CEP4",
                                      "description": "Cluster to write output to",
                                      "enum": [
                                          "CEP4",
                                          "DragNet"
                                      ],
                                      "title": "Storage cluster",
                                      "type": "string"
                                  },
                                  "topocentric_frequency_correction": {
                                      "default": false,
                                      "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation.",
                                      "title": "Topocentric Frequency Correction",
                                      "type": "boolean"
                                  }
                              }
                          },
                          "station_configuration": {
                              "default": {},
                              "properties": {
                                  "SAPs": {
                                      "$ref": "#/definitions/SAPs",
                                      "default": [
                                          {}
                                      ],
                                      "minItems": 1
                                  },
                                  "antenna_set": {
                                      "$ref": "#/definitions/antenna_set",
                                      "default": "HBA_DUAL"
                                  },
                                  "filter": {
                                      "$ref": "#/definitions/filter",
                                      "default": "HBA_110_190"
                                  },
                                  "station_groups": {
                                      "$ref": "#/definitions/station_groups",
                                      "default": [
                                          {
                                              "max_nr_missing": 1,
                                              "stations": [
                                                  "CS002",
                                                  "CS003",
                                                  "CS004",
                                                  "CS005",
                                                  "CS006",
                                                  "CS007"
                                              ]
                                          }
                                      ]
                                  },
                                  "tile_beam": {
                                      "$ref": "#/definitions/pointing",
                                      "default": {},
                                      "description": "HBA only",
                                      "title": "Tile beam"
                                  }
                              },
                              "required": [
                                  "station_groups",
                                  "antenna_set",
                                  "filter",
                                  "SAPs"
                              ],
                              "title": "station_configuration",
                              "type": "object"
                          },
                          "station": {
                              "description": "These are the LOFAR stations",
                              "enum": [
                                  "CS001",
                                  "CS002",
                                  "CS003",
                                  "CS004",
                                  "CS005",
                                  "CS006",
                                  "CS007",
                                  "CS011",
                                  "CS013",
                                  "CS017",
                                  "CS021",
                                  "CS024",
                                  "CS026",
                                  "CS028",
                                  "CS030",
                                  "CS031",
                                  "CS032",
                                  "CS101",
                                  "CS103",
                                  "CS201",
                                  "CS301",
                                  "CS302",
                                  "CS401",
                                  "CS501",
                                  "RS106",
                                  "RS205",
                                  "RS208",
                                  "RS210",
                                  "RS305",
                                  "RS306",
                                  "RS307",
                                  "RS310",
                                  "RS406",
                                  "RS407",
                                  "RS409",
                                  "RS503",
                                  "RS508",
                                  "RS509",
                                  "DE601",
                                  "DE602",
                                  "DE603",
                                  "DE604",
                                  "DE605",
                                  "FR606",
                                  "SE607",
                                  "UK608",
                                  "DE609",
                                  "PL610",
                                  "PL611",
                                  "PL612",
                                  "IE613",
                                  "LV614"
                              ],
                              "title": "Station",
                              "type": "string"
                          },
                          "station_groups": {
                              "additionalItems": false,
                              "additionalProperties": false,
                              "default": [
                                  {
                                      "max_nr_missing": 1,
                                      "stations": [
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007"
                                      ]
                                  }
                              ],
                              "description": "One or more predefined or custom groups of stations",
                              "items": {
                                  "$ref": "#/definitions/station_group"
                              },
                              "minItems": 1,
                              "title": "Station groups",
                              "type": "array"
                          },
                          "max_number_of_missing_stations": {
                              "default": 0,
                              "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                              "minimum": 0,
                              "title": "Maximum number of stations to omit",
                              "type": "integer"
                          },
                          "SAPs": {
                              "additionalItems": false,
                              "default": [
                                  {}
                              ],
                              "description": "Station beams",
                              "items": {
                                  "additionalProperties": false,
                                  "default": {},
                                  "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                  "properties": {
                                      "digital_pointing": {
                                          "$id": "#target_pointing",
                                          "$ref": "#/definitions/pointing",
                                          "default": {},
                                          "title": "Digital pointing"
                                      },
                                      "name": {
                                          "default": "_SAP_name_",
                                          "description": "Identifier for this beam",
                                          "minLength": 1,
                                          "title": "Name",
                                          "type": "string"
                                      },
                                      "subbands": {
                                          "additionalItems": false,
                                          "default": [
                                              255
                                          ],
                                          "items": {
                                              "maximum": 511,
                                              "minimum": 0,
                                              "title": "Subband",
                                              "type": "integer"
                                          },
                                          "maxItems": 488,
                                          "minItems": 1,
                                          "title": "Subband list",
                                          "type": "array"
                                      }
                                  },
                                  "required": [
                                      "name",
                                      "digital_pointing",
                                      "subbands"
                                  ],
                                  "title": "SAP",
                                  "type": "object"
                              },
                              "minItems": 0,
                              "title": "SAPs",
                              "type": "array"
                          },
                          "antenna_set": {
                              "default": "HBA_DUAL",
                              "description": "Fields & antennas to use",
                              "enum": [
                                  "HBA_DUAL",
                                  "HBA_DUAL_INNER",
                                  "HBA_ONE",
                                  "HBA_ONE_INNER",
                                  "HBA_ZERO",
                                  "HBA_ZERO_INNER",
                                  "LBA_INNER",
                                  "LBA_OUTER",
                                  "LBA_SPARSE_EVEN",
                                  "LBA_SPARSE_ODD",
                                  "LBA_ALL"
                              ],
                              "title": "Antenna set",
                              "type": "string"
                          },
                          "filter": {
                              "default": "HBA_110_190",
                              "description": "Must match antenna type",
                              "enum": [
                                  "LBA_10_70",
                                  "LBA_30_70",
                                  "LBA_10_90",
                                  "LBA_30_90",
                                  "HBA_110_190",
                                  "HBA_210_250"
                              ],
                              "title": "Band-pass filter",
                              "type": "string"
                          },
                          "station_list": {
                              "additionalItems": false,
                              "additionalProperties": false,
                              "default": [],
                              "items": {
                                  "$ref": "#/definitions/station"
                              },
                              "minItems": 0,
                              "type": "array",
                              "uniqueItems": true
                          },
                          "station_group": {
                              "anyOf": [
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 0,
                                          "stations": [
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007"
                                          ]
                                      },
                                      "description": "The group of all stations on the Superterp",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Superterp",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 4,
                                          "stations": [
                                              "CS001",
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007",
                                              "CS011",
                                              "CS013",
                                              "CS017",
                                              "CS021",
                                              "CS024",
                                              "CS026",
                                              "CS028",
                                              "CS030",
                                              "CS031",
                                              "CS032",
                                              "CS101",
                                              "CS103",
                                              "CS201",
                                              "CS301",
                                              "CS302",
                                              "CS401",
                                              "CS501"
                                          ]
                                      },
                                      "description": "The group of all Core stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS001",
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007",
                                                      "CS011",
                                                      "CS013",
                                                      "CS017",
                                                      "CS021",
                                                      "CS024",
                                                      "CS026",
                                                      "CS028",
                                                      "CS030",
                                                      "CS031",
                                                      "CS032",
                                                      "CS101",
                                                      "CS103",
                                                      "CS201",
                                                      "CS301",
                                                      "CS302",
                                                      "CS401",
                                                      "CS501"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Core",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 4,
                                          "stations": [
                                              "RS106",
                                              "RS205",
                                              "RS208",
                                              "RS210",
                                              "RS305",
                                              "RS306",
                                              "RS307",
                                              "RS310",
                                              "RS406",
                                              "RS407",
                                              "RS409",
                                              "RS503",
                                              "RS508",
                                              "RS509"
                                          ]
                                      },
                                      "description": "The group of all Dutch remote stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "RS106",
                                                      "RS205",
                                                      "RS208",
                                                      "RS210",
                                                      "RS305",
                                                      "RS306",
                                                      "RS307",
                                                      "RS310",
                                                      "RS406",
                                                      "RS407",
                                                      "RS409",
                                                      "RS503",
                                                      "RS508",
                                                      "RS509"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Remote",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 4,
                                          "stations": [
                                              "CS001",
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007",
                                              "CS011",
                                              "CS013",
                                              "CS017",
                                              "CS021",
                                              "CS024",
                                              "CS026",
                                              "CS028",
                                              "CS030",
                                              "CS031",
                                              "CS032",
                                              "CS101",
                                              "CS103",
                                              "CS201",
                                              "CS301",
                                              "CS302",
                                              "CS401",
                                              "CS501",
                                              "RS106",
                                              "RS205",
                                              "RS208",
                                              "RS210",
                                              "RS305",
                                              "RS306",
                                              "RS307",
                                              "RS310",
                                              "RS406",
                                              "RS407",
                                              "RS409",
                                              "RS503",
                                              "RS508",
                                              "RS509"
                                          ]
                                      },
                                      "description": "The group of all Dutch (Core + Remote) stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS001",
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007",
                                                      "CS011",
                                                      "CS013",
                                                      "CS017",
                                                      "CS021",
                                                      "CS024",
                                                      "CS026",
                                                      "CS028",
                                                      "CS030",
                                                      "CS031",
                                                      "CS032",
                                                      "CS101",
                                                      "CS103",
                                                      "CS201",
                                                      "CS301",
                                                      "CS302",
                                                      "CS401",
                                                      "CS501",
                                                      "RS106",
                                                      "RS205",
                                                      "RS208",
                                                      "RS210",
                                                      "RS305",
                                                      "RS306",
                                                      "RS307",
                                                      "RS310",
                                                      "RS406",
                                                      "RS407",
                                                      "RS409",
                                                      "RS503",
                                                      "RS508",
                                                      "RS509"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Dutch",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 2,
                                          "stations": [
                                              "DE601",
                                              "DE602",
                                              "DE603",
                                              "DE604",
                                              "DE605",
                                              "DE609",
                                              "FR606",
                                              "SE607",
                                              "UK608",
                                              "PL610",
                                              "PL611",
                                              "PL612",
                                              "IE613",
                                              "LV614"
                                          ]
                                      },
                                      "description": "The group of all international stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "DE601",
                                                      "DE602",
                                                      "DE603",
                                                      "DE604",
                                                      "DE605",
                                                      "DE609",
                                                      "FR606",
                                                      "SE607",
                                                      "UK608",
                                                      "PL610",
                                                      "PL611",
                                                      "PL612",
                                                      "IE613",
                                                      "LV614"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "International",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 1,
                                          "stations": [
                                              "DE601",
                                              "DE605"
                                          ]
                                      },
                                      "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "DE601",
                                                      "DE605"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "International required",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 6,
                                          "stations": [
                                              "CS001",
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007",
                                              "CS011",
                                              "CS013",
                                              "CS017",
                                              "CS021",
                                              "CS024",
                                              "CS026",
                                              "CS028",
                                              "CS030",
                                              "CS031",
                                              "CS032",
                                              "CS101",
                                              "CS103",
                                              "CS201",
                                              "CS301",
                                              "CS302",
                                              "CS401",
                                              "CS501",
                                              "RS106",
                                              "RS205",
                                              "RS208",
                                              "RS210",
                                              "RS305",
                                              "RS306",
                                              "RS307",
                                              "RS310",
                                              "RS406",
                                              "RS407",
                                              "RS409",
                                              "RS503",
                                              "RS508",
                                              "RS509",
                                              "DE601",
                                              "DE602",
                                              "DE603",
                                              "DE604",
                                              "DE605",
                                              "DE609",
                                              "FR606",
                                              "SE607",
                                              "UK608",
                                              "PL610",
                                              "PL611",
                                              "PL612",
                                              "IE613",
                                              "LV614"
                                          ]
                                      },
                                      "description": "The group of all (Core + Remote + International) stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS001",
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007",
                                                      "CS011",
                                                      "CS013",
                                                      "CS017",
                                                      "CS021",
                                                      "CS024",
                                                      "CS026",
                                                      "CS028",
                                                      "CS030",
                                                      "CS031",
                                                      "CS032",
                                                      "CS101",
                                                      "CS103",
                                                      "CS201",
                                                      "CS301",
                                                      "CS302",
                                                      "CS401",
                                                      "CS501",
                                                      "RS106",
                                                      "RS205",
                                                      "RS208",
                                                      "RS210",
                                                      "RS305",
                                                      "RS306",
                                                      "RS307",
                                                      "RS310",
                                                      "RS406",
                                                      "RS407",
                                                      "RS409",
                                                      "RS503",
                                                      "RS508",
                                                      "RS509",
                                                      "DE601",
                                                      "DE602",
                                                      "DE603",
                                                      "DE604",
                                                      "DE605",
                                                      "DE609",
                                                      "FR606",
                                                      "SE607",
                                                      "UK608",
                                                      "PL610",
                                                      "PL611",
                                                      "PL612",
                                                      "IE613",
                                                      "LV614"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "All",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 0,
                                          "stations": [
                                              "CS001"
                                          ]
                                      },
                                      "description": "A custom group of stations which can be defined by the user",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list"
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Custom",
                                      "type": "object"
                                  }
                              ],
                              "default": {
                                  "max_nr_missing": 1,
                                  "stations": [
                                      "CS002",
                                      "CS003",
                                      "CS004",
                                      "CS005",
                                      "CS006",
                                      "CS007"
                                  ]
                              },
                              "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                              "title": "Station group",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2#",
                      "type": "object",
                      "title": "parallel calibrator target observation",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "calibrator",
                          "correlator",
                          "duration",
                          "station_configuration"
                      ],
                      "properties": {
                          "QA": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/2#/definitions/QA",
                              "default": {}
                          },
                          "duration": {
                              "$id": "#duration",
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                              "title": "Duration",
                              "default": 600,
                              "minimum": 1,
                              "description": "Duration of this observation (seconds)"
                          },
                          "calibrator": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/calibrator/1#/definitions/calibrator",
                              "default": {}
                          },
                          "correlator": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/correlator/1#/definitions/correlator",
                              "default": {}
                          },
                          "station_configuration": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                              "default": {}
                          }
                      },
                      "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/observation",
                  "type_value": "observation",
                  "updated_at": "2022-04-14T06:20:07.273910",
                  "version": 2
              },
              "specifications_template_id": 5,
              "status": "scheduled",
              "status_value": "scheduled",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000001",
                  "http://localhost:3000/api/subtask/2000002",
                  "http://localhost:3000/api/subtask/2000000"
              ],
              "subtasks_ids": [
                  2000001,
                  2000002,
                  2000000
              ],
              "tags": [],
              "task_type": "observation",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:19.668472"
          },
          {
              "id": 8,
              "url": "http://localhost:3000/api/task_blueprint/8",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [],
              "consumed_by_ids": [],
              "created_at": "2022-04-14T06:20:19.977881",
              "description": "Ingest all preprocessed dataproducts",
              "draft": "http://localhost:3000/api/task_draft/8",
              "draft_id": 8,
              "duration": 300,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Ingest",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:29:19.054619",
              "process_stop_time": "2022-04-14T06:34:19.054619",
              "produced_by": [
                  "http://localhost:3000/api/task_relation_blueprint/7",
                  "http://localhost:3000/api/task_relation_blueprint/8",
                  "http://localhost:3000/api/task_relation_blueprint/9",
                  "http://localhost:3000/api/task_relation_blueprint/10",
                  "http://localhost:3000/api/task_relation_blueprint/11",
                  "http://localhost:3000/api/task_relation_blueprint/12"
              ],
              "produced_by_ids": [
                  7,
                  8,
                  9,
                  10,
                  11,
                  12
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-04-14T06:29:19.054619",
              "scheduled_stop_time": "2022-04-14T06:34:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
              "scheduling_unit_blueprint_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "",
              "specifications_doc": {
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
              },
              "specifications_template": {
                  "id": 2,
                  "url": "http://localhost:3000/api/task_template/2",
                  "created_at": "2022-04-14T06:20:07.089989",
                  "description": "This schema defines the parameters to setup an ingest task.",
                  "name": "ingest",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved",
                      "type": "object",
                      "title": "ingest",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [],
                      "properties": {},
                      "description": "This schema defines the parameters to setup an ingest task.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {}
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2#",
                      "type": "object",
                      "title": "ingest",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [],
                      "properties": {},
                      "description": "This schema defines the parameters to setup an ingest task.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/ingest",
                  "type_value": "ingest",
                  "updated_at": "2022-04-14T06:20:07.090050",
                  "version": 2
              },
              "specifications_template_id": 2,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000009"
              ],
              "subtasks_ids": [
                  2000009
              ],
              "tags": [],
              "task_type": "ingest",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:19.977893"
          },
          {
              "id": 4,
              "url": "http://localhost:3000/api/task_blueprint/4",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_blueprint/11"
              ],
              "consumed_by_ids": [
                  11
              ],
              "created_at": "2022-04-14T06:20:19.813073",
              "description": "Preprocessing Pipeline for Target Observation target4",
              "draft": "http://localhost:3000/api/task_draft/4",
              "draft_id": 4,
              "duration": 120,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target4",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:27:19.054619",
              "process_stop_time": "2022-04-14T06:29:19.054619",
              "produced_by": [
                  "http://localhost:3000/api/task_relation_blueprint/3"
              ],
              "produced_by_ids": [
                  3
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-04-14T06:27:19.054619",
              "scheduled_stop_time": "2022-04-14T06:29:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
              "scheduling_unit_blueprint_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Paaa+04/1.3/PP",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 10,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 1,
                      "frequency_steps": 1
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000005"
              ],
              "subtasks_ids": [
                  2000005
              ],
              "tags": [],
              "task_type": "pipeline",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:19.813116"
          },
          {
              "id": 3,
              "url": "http://localhost:3000/api/task_blueprint/3",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_blueprint/12"
              ],
              "consumed_by_ids": [
                  12
              ],
              "created_at": "2022-04-14T06:20:19.774241",
              "description": "Preprocessing Pipeline for Target Observation target5",
              "draft": "http://localhost:3000/api/task_draft/3",
              "draft_id": 3,
              "duration": 120,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target5",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:27:19.054619",
              "process_stop_time": "2022-04-14T06:29:19.054619",
              "produced_by": [
                  "http://localhost:3000/api/task_relation_blueprint/2"
              ],
              "produced_by_ids": [
                  2
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-04-14T06:27:19.054619",
              "scheduled_stop_time": "2022-04-14T06:29:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/1",
              "scheduling_unit_blueprint_id": 1,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "Paaa+05/1.4/PP",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 10,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 1,
                      "frequency_steps": 1
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000003"
              ],
              "subtasks_ids": [
                  2000003
              ],
              "tags": [],
              "task_type": "pipeline",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:19.774253"
          }
      ],
      "task_blueprints_ids": [
          5,
          6,
          2,
          7,
          1,
          8,
          4,
          3
      ],
      "unschedulable_reason": "",
      "updated_at": "2022-04-14T06:20:19.627278"
    },
    {
      "id": 2,
      "url": "http://localhost:3000/api/scheduling_unit_blueprint/2",
      "actual_on_sky_start_time": null,
      "actual_on_sky_stop_time": null,
      "actual_process_start_time": null,
      "actual_process_stop_time": null,
      "created_at": "2022-04-14T06:20:22.543477",
      "description": "Test scheduling unit",
      "draft": "http://localhost:3000/api/scheduling_unit_draft/2",
      "draft_id": 2,
      "duration": 1800,
      "error_reason": "",
      "global_identifier": "http://localhost:3000/api/sip_identifier/22",
      "global_identifier_id": 22,
      "ingest_permission_granted_since": null,
      "ingest_permission_required": true,
      "interrupts_telescope": false,
      "name": "IM LBA Survey - 3 Beams",
      "observed_duration": null,
      "observed_start_time": null,
      "observed_stop_time": null,
      "obsolete_since": null,
      "on_sky_duration": 120,
      "on_sky_start_time": "2022-04-14T06:42:19.054619",
      "on_sky_stop_time": "2022-04-14T06:44:19.054619",
      "output_pinned": false,
      "piggyback_allowed_aartfaac": true,
      "piggyback_allowed_tbb": true,
      "priority_queue": "http://localhost:3000/api/priority_queue_type/A",
      "priority_queue_value": "A",
      "process_start_time": "2022-04-14T06:42:19.054619",
      "process_stop_time": "2022-04-14T06:54:19.054619",
      "rank": 1,
      "scheduled_start_time": "2022-04-14T06:42:19.054619",
      "scheduled_stop_time": "2022-04-14T06:54:19.054619",
      "scheduling_constraints_doc": {
          "sky": {
              "min_distance": {
                  "sun": 0.5,
                  "moon": 0.5,
                  "jupiter": 0.5
              },
              "min_elevation": {
                  "target": 0.5,
                  "calibrator": 0.5
              },
              "transit_offset": {
                  "to": 7200,
                  "from": -7200
              }
          },
          "time": {
              "at": "2022-04-14T06:42:19.054619"
          },
          "daily": {
              "require_day": false,
              "require_night": false,
              "avoid_twilight": false
          },
          "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
          "scheduler": "fixed_time"
      },
      "scheduling_constraints_template": "http://localhost:3000/api/scheduling_constraints_template/1",
      "scheduling_constraints_template_id": 1,
      "specifications_template": "http://localhost:3000/api/scheduling_unit_template/1",
      "specifications_template_id": 1,
      "status": "scheduled",
      "status_value": "scheduled",
      "tags": [],
      "task_blueprints": [
          {
              "id": 14,
              "url": "http://localhost:3000/api/task_blueprint/14",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [],
              "consumed_by_ids": [],
              "created_at": "2022-04-14T06:20:22.793703",
              "description": "Clean up all dataproducts from disk after ingest",
              "draft": "http://localhost:3000/api/task_draft/14",
              "draft_id": 14,
              "duration": 300,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Cleanup",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:46:19.054619",
              "process_stop_time": "2022-04-14T06:51:19.054619",
              "produced_by": [
                  "http://localhost:3000/api/task_relation_blueprint/17",
                  "http://localhost:3000/api/task_relation_blueprint/18",
                  "http://localhost:3000/api/task_relation_blueprint/19",
                  "http://localhost:3000/api/task_relation_blueprint/20"
              ],
              "produced_by_ids": [
                  17,
                  18,
                  19,
                  20
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-04-14T06:46:19.054619",
              "scheduled_stop_time": "2022-04-14T06:51:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/2",
              "scheduling_unit_blueprint_id": 2,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "",
              "specifications_doc": {
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2/ref_resolved"
              },
              "specifications_template": {
                  "id": 1,
                  "url": "http://localhost:3000/api/task_template/1",
                  "created_at": "2022-04-14T06:20:07.052510",
                  "description": "This schema defines the parameters to setup a dataproduct(s) cleanup task.",
                  "name": "cleanup",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2/ref_resolved",
                      "type": "object",
                      "title": "cleanup",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [],
                      "properties": {},
                      "description": "This schema defines the parameters to setup a dataproduct(s) cleanup task.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {}
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2#",
                      "type": "object",
                      "title": "cleanup",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [],
                      "properties": {},
                      "description": "This schema defines the parameters to setup a dataproduct(s) cleanup task.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/cleanup",
                  "type_value": "cleanup",
                  "updated_at": "2022-04-14T06:20:07.052528",
                  "version": 2
              },
              "specifications_template_id": 1,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000018"
              ],
              "subtasks_ids": [
                  2000018
              ],
              "tags": [],
              "task_type": "cleanup",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:22.793715"
          },
          {
              "id": 10,
              "url": "http://localhost:3000/api/task_blueprint/10",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_blueprint/21"
              ],
              "consumed_by_ids": [
                  21
              ],
              "created_at": "2022-04-14T06:20:22.628307",
              "description": "Preprocessing Pipeline for Calibrator Observation",
              "draft": "http://localhost:3000/api/task_draft/10",
              "draft_id": 10,
              "duration": 120,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Calibrator Pipeline",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:44:19.054619",
              "process_stop_time": "2022-04-14T06:46:19.054619",
              "produced_by": [
                  "http://localhost:3000/api/task_relation_blueprint/13"
              ],
              "produced_by_ids": [
                  13
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-04-14T06:44:19.054619",
              "scheduled_stop_time": "2022-04-14T06:46:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/2",
              "scheduling_unit_blueprint_id": 2,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "c17 oOOO.O 3Cabc",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 8,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 4,
                      "frequency_steps": 8
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000015"
              ],
              "subtasks_ids": [
                  2000015
              ],
              "tags": [],
              "task_type": "pipeline",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:22.628318"
          },
          {
              "id": 15,
              "url": "http://localhost:3000/api/task_blueprint/15",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [],
              "consumed_by_ids": [],
              "created_at": "2022-04-14T06:20:22.862472",
              "description": "Ingest all preprocessed dataproducts",
              "draft": "http://localhost:3000/api/task_draft/15",
              "draft_id": 15,
              "duration": 300,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Ingest",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:46:19.054619",
              "process_stop_time": "2022-04-14T06:51:19.054619",
              "produced_by": [
                  "http://localhost:3000/api/task_relation_blueprint/21",
                  "http://localhost:3000/api/task_relation_blueprint/22",
                  "http://localhost:3000/api/task_relation_blueprint/23",
                  "http://localhost:3000/api/task_relation_blueprint/24"
              ],
              "produced_by_ids": [
                  21,
                  22,
                  23,
                  24
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-04-14T06:46:19.054619",
              "scheduled_stop_time": "2022-04-14T06:51:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/2",
              "scheduling_unit_blueprint_id": 2,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "",
              "specifications_doc": {
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
              },
              "specifications_template": {
                  "id": 2,
                  "url": "http://localhost:3000/api/task_template/2",
                  "created_at": "2022-04-14T06:20:07.089989",
                  "description": "This schema defines the parameters to setup an ingest task.",
                  "name": "ingest",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved",
                      "type": "object",
                      "title": "ingest",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [],
                      "properties": {},
                      "description": "This schema defines the parameters to setup an ingest task.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {}
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2#",
                      "type": "object",
                      "title": "ingest",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [],
                      "properties": {},
                      "description": "This schema defines the parameters to setup an ingest task.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/ingest",
                  "type_value": "ingest",
                  "updated_at": "2022-04-14T06:20:07.090050",
                  "version": 2
              },
              "specifications_template_id": 2,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000017"
              ],
              "subtasks_ids": [
                  2000017
              ],
              "tags": [],
              "task_type": "ingest",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:22.862485"
          },
          {
              "id": 11,
              "url": "http://localhost:3000/api/task_blueprint/11",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_blueprint/20",
                  "http://localhost:3000/api/task_relation_blueprint/24"
              ],
              "consumed_by_ids": [
                  20,
                  24
              ],
              "created_at": "2022-04-14T06:20:22.664403",
              "description": "Preprocessing Pipeline for Target Observation target3",
              "draft": "http://localhost:3000/api/task_draft/11",
              "draft_id": 11,
              "duration": 120,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target3",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:44:19.054619",
              "process_stop_time": "2022-04-14T06:46:19.054619",
              "produced_by": [
                  "http://localhost:3000/api/task_relation_blueprint/14"
              ],
              "produced_by_ids": [
                  14
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-04-14T06:44:19.054619",
              "scheduled_stop_time": "2022-04-14T06:46:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/2",
              "scheduling_unit_blueprint_id": 2,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "c17 oOOO.O Paaa+03",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 8,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 4,
                      "frequency_steps": 8
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000013"
              ],
              "subtasks_ids": [
                  2000013
              ],
              "tags": [],
              "task_type": "pipeline",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:22.664415"
          },
          {
              "id": 13,
              "url": "http://localhost:3000/api/task_blueprint/13",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_blueprint/18",
                  "http://localhost:3000/api/task_relation_blueprint/22"
              ],
              "consumed_by_ids": [
                  18,
                  22
              ],
              "created_at": "2022-04-14T06:20:22.753080",
              "description": "Preprocessing Pipeline for Target Observation target1",
              "draft": "http://localhost:3000/api/task_draft/13",
              "draft_id": 13,
              "duration": 120,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target1",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:44:19.054619",
              "process_stop_time": "2022-04-14T06:46:19.054619",
              "produced_by": [
                  "http://localhost:3000/api/task_relation_blueprint/16"
              ],
              "produced_by_ids": [
                  16
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-04-14T06:44:19.054619",
              "scheduled_stop_time": "2022-04-14T06:46:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/2",
              "scheduling_unit_blueprint_id": 2,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "c17 oOOO.O Paaa+01",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 8,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 4,
                      "frequency_steps": 8
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000014"
              ],
              "subtasks_ids": [
                  2000014
              ],
              "tags": [],
              "task_type": "pipeline",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:22.753094"
          },
          {
              "id": 12,
              "url": "http://localhost:3000/api/task_blueprint/12",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_blueprint/19",
                  "http://localhost:3000/api/task_relation_blueprint/23"
              ],
              "consumed_by_ids": [
                  19,
                  23
              ],
              "created_at": "2022-04-14T06:20:22.708297",
              "description": "Preprocessing Pipeline for Target Observation target2",
              "draft": "http://localhost:3000/api/task_draft/12",
              "draft_id": 12,
              "duration": 120,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Pipeline target2",
              "obsolete_since": null,
              "on_sky_duration": null,
              "on_sky_start_time": null,
              "on_sky_stop_time": null,
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:44:19.054619",
              "process_stop_time": "2022-04-14T06:46:19.054619",
              "produced_by": [
                  "http://localhost:3000/api/task_relation_blueprint/15"
              ],
              "produced_by_ids": [
                  15
              ],
              "relative_start_time": 0,
              "relative_stop_time": 0,
              "scheduled_start_time": "2022-04-14T06:44:19.054619",
              "scheduled_stop_time": "2022-04-14T06:46:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/2",
              "scheduling_unit_blueprint_id": 2,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "c17 oOOO.O Paaa+02",
              "specifications_doc": {
                  "flag": {
                      "rfi_strategy": "LBAdefault",
                      "outerchannels": true,
                      "autocorrelations": true
                  },
                  "demix": {
                      "sources": [],
                      "time_steps": 8,
                      "ignore_target": false,
                      "frequency_steps": 64
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                  "average": {
                      "time_steps": 4,
                      "frequency_steps": 8
                  },
                  "storagemanager": "dysco",
                  "cluster_resources": {
                      "where": {
                          "cluster": "CEP4",
                          "partition": "cpu"
                      },
                      "cores_per_task": 2,
                      "parallel_tasks": 122
                  }
              },
              "specifications_template": {
                  "id": 10,
                  "url": "http://localhost:3000/api/task_template/10",
                  "created_at": "2022-04-14T06:20:07.571599",
                  "description": "This schema defines the parameters for a preprocessing pipeline.",
                  "name": "preprocessing pipeline",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "cluster_resources": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Which cluster resources to claim for this pipeline.",
                              "properties": {
                                  "cores_per_task": {
                                      "default": 2,
                                      "description": "Number of cores to reserve for each process in the pipeline.",
                                      "maximum": 24,
                                      "minimum": 1,
                                      "title": "Cores/task",
                                      "type": "integer"
                                  },
                                  "parallel_tasks": {
                                      "default": 122,
                                      "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                      "maximum": 488,
                                      "minimum": 1,
                                      "title": "Max parallel tasks",
                                      "type": "integer"
                                  },
                                  "where": {
                                      "additionalProperties": false,
                                      "default": {},
                                      "description": "Where to run this pipeline.",
                                      "properties": {
                                          "cluster": {
                                              "default": "CEP4",
                                              "enum": [
                                                  "CEP4"
                                              ],
                                              "title": "Cluster",
                                              "type": "string"
                                          },
                                          "partition": {
                                              "default": "cpu",
                                              "enum": [
                                                  "cpu",
                                                  "gpu",
                                                  "testing"
                                              ],
                                              "title": "Partition",
                                              "type": "string"
                                          }
                                      },
                                      "required": [
                                          "cluster",
                                          "partition"
                                      ],
                                      "title": "Where",
                                      "type": "object"
                                  }
                              },
                              "required": [
                                  "where",
                                  "cores_per_task",
                                  "parallel_tasks"
                              ],
                              "title": "Cluster resources",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                      "type": "object",
                      "title": "preprocessing pipeline",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "average"
                      ],
                      "properties": {
                          "flag": {
                              "type": "object",
                              "title": "Flagging",
                              "default": {},
                              "required": [
                                  "outerchannels",
                                  "autocorrelations",
                                  "rfi_strategy"
                              ],
                              "properties": {
                                  "rfi_strategy": {
                                      "enum": [
                                          "none",
                                          "HBAdefault",
                                          "LBAdefault"
                                      ],
                                      "type": "string",
                                      "title": "RFI flagging strategy",
                                      "default": "HBAdefault"
                                  },
                                  "outerchannels": {
                                      "type": "boolean",
                                      "title": "Flag outer channels",
                                      "default": true
                                  },
                                  "autocorrelations": {
                                      "type": "boolean",
                                      "title": "Flag auto correlations",
                                      "default": true
                                  }
                              },
                              "additionalProperties": false
                          },
                          "demix": {
                              "type": "object",
                              "title": "Demixing",
                              "default": {},
                              "required": [],
                              "properties": {
                                  "sources": {
                                      "type": "array",
                                      "items": {
                                          "enum": [
                                              "CasA",
                                              "CygA",
                                              "HerA",
                                              "HydraA",
                                              "TauA",
                                              "VirA"
                                          ],
                                          "type": "string",
                                          "default": "CasA"
                                      },
                                      "maxItems": 2,
                                      "minItems": 0,
                                      "uniqueItems": true,
                                      "additionalItems": false
                                  },
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 10,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging time steps"
                                  },
                                  "ignore_target": {
                                      "type": "boolean",
                                      "title": "Ignore target",
                                      "default": false
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 64,
                                      "minimum": 1,
                                      "description": "Must be a multiple of the averaging frequency steps"
                                  }
                              },
                              "additionalProperties": false
                          },
                          "average": {
                              "type": "object",
                              "title": "Averaging",
                              "default": {},
                              "required": [
                                  "frequency_steps",
                                  "time_steps"
                              ],
                              "properties": {
                                  "time_steps": {
                                      "type": "integer",
                                      "title": "Time steps",
                                      "default": 1,
                                      "minimum": 1
                                  },
                                  "frequency_steps": {
                                      "type": "integer",
                                      "title": "Frequency steps",
                                      "default": 4,
                                      "minimum": 1
                                  }
                              },
                              "additionalProperties": false
                          },
                          "storagemanager": {
                              "enum": [
                                  "standard",
                                  "dysco"
                              ],
                              "type": "string",
                              "title": "Storage Manager",
                              "default": "dysco"
                          },
                          "cluster_resources": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                              "default": {}
                          }
                      },
                      "description": "This schema defines the parameters for a preprocessing pipeline.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/pipeline",
                  "type_value": "pipeline",
                  "updated_at": "2022-04-14T06:20:07.571613",
                  "version": 2
              },
              "specifications_template_id": 10,
              "status": "schedulable",
              "status_value": "schedulable",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000016"
              ],
              "subtasks_ids": [
                  2000016
              ],
              "tags": [],
              "task_type": "pipeline",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:22.708311"
          },
          {
              "id": 9,
              "url": "http://localhost:3000/api/task_blueprint/9",
              "actual_on_sky_start_time": null,
              "actual_on_sky_stop_time": null,
              "actual_process_start_time": null,
              "actual_process_stop_time": null,
              "consumed_by": [
                  "http://localhost:3000/api/task_relation_blueprint/13",
                  "http://localhost:3000/api/task_relation_blueprint/14",
                  "http://localhost:3000/api/task_relation_blueprint/15",
                  "http://localhost:3000/api/task_relation_blueprint/16",
                  "http://localhost:3000/api/task_relation_blueprint/17"
              ],
              "consumed_by_ids": [
                  13,
                  14,
                  15,
                  16,
                  17
              ],
              "created_at": "2022-04-14T06:20:22.577268",
              "description": "Combined parallel Calibrator & Target Observation for LBA with 3 target beams",
              "draft": "http://localhost:3000/api/task_draft/9",
              "draft_id": 9,
              "duration": 720,
              "error_reason": "",
              "first_scheduling_relation": [],
              "first_scheduling_relation_ids": [],
              "name": "Combined Observation",
              "obsolete_since": null,
              "on_sky_duration": 120,
              "on_sky_start_time": "2022-04-14T06:42:19.054619",
              "on_sky_stop_time": "2022-04-14T06:44:19.054619",
              "output_pinned": false,
              "process_start_time": "2022-04-14T06:42:19.054619",
              "process_stop_time": "2022-04-14T06:54:19.054619",
              "produced_by": [],
              "produced_by_ids": [],
              "relative_start_time": 0,
              "relative_stop_time": 120,
              "scheduled_start_time": "2022-04-14T06:42:19.054619",
              "scheduled_stop_time": "2022-04-14T06:54:19.054619",
              "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/2",
              "scheduling_unit_blueprint_id": 2,
              "second_scheduling_relation": [],
              "second_scheduling_relation_ids": [],
              "short_description": "OOO.O Paaa+01 Paaa+02 Paaa+03",
              "specifications_doc": {
                  "QA": {
                      "plots": {
                          "enabled": true,
                          "autocorrelation": true,
                          "crosscorrelation": true
                      },
                      "file_conversion": {
                          "enabled": true,
                          "nr_of_subbands": -1,
                          "nr_of_timestamps": 256
                      },
                      "inspection_plots": "msplots"
                  },
                  "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                  "duration": 120,
                  "calibrator": {
                      "name": "calibrator",
                      "pointing": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "3Cabc",
                          "direction_type": "J2000"
                      },
                      "autoselect": false
                  },
                  "correlator": {
                      "storage_cluster": "CEP4",
                      "integration_time": 1,
                      "channels_per_subband": 64,
                      "topocentric_frequency_correction": false
                  },
                  "station_configuration": {
                      "SAPs": [
                          {
                              "name": "target1",
                              "subbands": [
                                  217,
                                  218,
                                  219,
                                  220,
                                  221,
                                  222,
                                  223,
                                  224,
                                  225,
                                  226,
                                  227,
                                  228,
                                  229,
                                  230,
                                  231,
                                  232,
                                  233,
                                  234,
                                  235,
                                  236,
                                  237,
                                  238,
                                  239,
                                  240,
                                  241,
                                  242,
                                  243,
                                  244,
                                  245,
                                  246,
                                  247,
                                  248,
                                  249,
                                  250,
                                  251,
                                  252,
                                  253,
                                  254,
                                  255,
                                  256,
                                  257,
                                  258,
                                  259,
                                  260,
                                  261,
                                  262,
                                  263,
                                  264,
                                  265,
                                  266,
                                  267,
                                  268,
                                  269,
                                  270,
                                  271,
                                  272,
                                  273,
                                  274,
                                  275,
                                  276,
                                  277,
                                  278,
                                  279,
                                  280,
                                  281,
                                  282,
                                  283,
                                  284,
                                  285,
                                  286,
                                  287,
                                  288,
                                  289,
                                  290,
                                  291,
                                  292,
                                  293,
                                  294,
                                  295,
                                  296,
                                  297,
                                  298,
                                  299,
                                  300,
                                  301,
                                  302,
                                  303,
                                  304,
                                  305,
                                  306,
                                  307,
                                  308,
                                  309,
                                  310,
                                  311,
                                  312,
                                  313,
                                  314,
                                  315,
                                  316,
                                  317,
                                  318,
                                  319,
                                  320,
                                  321,
                                  322,
                                  323,
                                  324,
                                  325,
                                  326,
                                  327,
                                  328,
                                  329,
                                  330,
                                  331,
                                  332,
                                  333,
                                  334,
                                  335,
                                  336,
                                  337,
                                  338
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+01",
                                  "direction_type": "J2000"
                              }
                          },
                          {
                              "name": "target2",
                              "subbands": [
                                  217,
                                  218,
                                  219,
                                  220,
                                  221,
                                  222,
                                  223,
                                  224,
                                  225,
                                  226,
                                  227,
                                  228,
                                  229,
                                  230,
                                  231,
                                  232,
                                  233,
                                  234,
                                  235,
                                  236,
                                  237,
                                  238,
                                  239,
                                  240,
                                  241,
                                  242,
                                  243,
                                  244,
                                  245,
                                  246,
                                  247,
                                  248,
                                  249,
                                  250,
                                  251,
                                  252,
                                  253,
                                  254,
                                  255,
                                  256,
                                  257,
                                  258,
                                  259,
                                  260,
                                  261,
                                  262,
                                  263,
                                  264,
                                  265,
                                  266,
                                  267,
                                  268,
                                  269,
                                  270,
                                  271,
                                  272,
                                  273,
                                  274,
                                  275,
                                  276,
                                  277,
                                  278,
                                  279,
                                  280,
                                  281,
                                  282,
                                  283,
                                  284,
                                  285,
                                  286,
                                  287,
                                  288,
                                  289,
                                  290,
                                  291,
                                  292,
                                  293,
                                  294,
                                  295,
                                  296,
                                  297,
                                  298,
                                  299,
                                  300,
                                  301,
                                  302,
                                  303,
                                  304,
                                  305,
                                  306,
                                  307,
                                  308,
                                  309,
                                  310,
                                  311,
                                  312,
                                  313,
                                  314,
                                  315,
                                  316,
                                  317,
                                  318,
                                  319,
                                  320,
                                  321,
                                  322,
                                  323,
                                  324,
                                  325,
                                  326,
                                  327,
                                  328,
                                  329,
                                  330,
                                  331,
                                  332,
                                  333,
                                  334,
                                  335,
                                  336,
                                  337,
                                  338
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+02",
                                  "direction_type": "J2000"
                              }
                          },
                          {
                              "name": "target3",
                              "subbands": [
                                  217,
                                  218,
                                  219,
                                  220,
                                  221,
                                  222,
                                  223,
                                  224,
                                  225,
                                  226,
                                  227,
                                  228,
                                  229,
                                  230,
                                  231,
                                  232,
                                  233,
                                  234,
                                  235,
                                  236,
                                  237,
                                  238,
                                  239,
                                  240,
                                  241,
                                  242,
                                  243,
                                  244,
                                  245,
                                  246,
                                  247,
                                  248,
                                  249,
                                  250,
                                  251,
                                  252,
                                  253,
                                  254,
                                  255,
                                  256,
                                  257,
                                  258,
                                  259,
                                  260,
                                  261,
                                  262,
                                  263,
                                  264,
                                  265,
                                  266,
                                  267,
                                  268,
                                  269,
                                  270,
                                  271,
                                  272,
                                  273,
                                  274,
                                  275,
                                  276,
                                  277,
                                  278,
                                  279,
                                  280,
                                  281,
                                  282,
                                  283,
                                  284,
                                  285,
                                  286,
                                  287,
                                  288,
                                  289,
                                  290,
                                  291,
                                  292,
                                  293,
                                  294,
                                  295,
                                  296,
                                  297,
                                  298,
                                  299,
                                  300,
                                  301,
                                  302,
                                  303,
                                  304,
                                  305,
                                  306,
                                  307,
                                  308,
                                  309,
                                  310,
                                  311,
                                  312,
                                  313,
                                  314,
                                  315,
                                  316,
                                  317,
                                  318,
                                  319,
                                  320,
                                  321,
                                  322,
                                  323,
                                  324,
                                  325,
                                  326,
                                  327,
                                  328,
                                  329,
                                  330,
                                  331,
                                  332,
                                  333,
                                  334,
                                  335,
                                  336,
                                  337,
                                  338
                              ],
                              "digital_pointing": {
                                  "angle1": 0.6624317181687094,
                                  "angle2": 1.5579526427549426,
                                  "target": "Paaa+03",
                                  "direction_type": "J2000"
                              }
                          }
                      ],
                      "filter": "LBA_30_90",
                      "tile_beam": {
                          "angle1": 0.6624317181687094,
                          "angle2": 1.5579526427549426,
                          "target": "_target_name_",
                          "direction_type": "J2000"
                      },
                      "antenna_set": "LBA_SPARSE_EVEN",
                      "station_groups": [
                          {
                              "stations": [
                                  "CS001",
                                  "CS002",
                                  "CS003",
                                  "CS004",
                                  "CS005",
                                  "CS006",
                                  "CS007",
                                  "CS011",
                                  "CS013",
                                  "CS017",
                                  "CS021",
                                  "CS024",
                                  "CS026",
                                  "CS028",
                                  "CS030",
                                  "CS031",
                                  "CS032",
                                  "CS101",
                                  "CS103",
                                  "CS201",
                                  "CS301",
                                  "CS302",
                                  "CS401",
                                  "CS501",
                                  "RS106",
                                  "RS205",
                                  "RS208",
                                  "RS210",
                                  "RS305",
                                  "RS306",
                                  "RS307",
                                  "RS310",
                                  "RS406",
                                  "RS407",
                                  "RS409",
                                  "RS503",
                                  "RS508",
                                  "RS509"
                              ],
                              "max_nr_missing": 4
                          },
                          {
                              "stations": [
                                  "RS508",
                                  "RS509"
                              ],
                              "max_nr_missing": 1
                          },
                          {
                              "stations": [
                                  "RS310",
                                  "RS210"
                              ],
                              "max_nr_missing": 0
                          }
                      ]
                  }
              },
              "specifications_template": {
                  "id": 5,
                  "url": "http://localhost:3000/api/task_template/5",
                  "created_at": "2022-04-14T06:20:07.273898",
                  "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                  "name": "parallel calibrator target observation",
                  "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                  "purpose_value": "technical_commissioning",
                  "ref_resolved_schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2/ref_resolved",
                      "type": "object",
                      "title": "parallel calibrator target observation",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "calibrator",
                          "correlator",
                          "duration",
                          "station_configuration"
                      ],
                      "properties": {
                          "QA": {
                              "$ref": "#/definitions/QA",
                              "default": {}
                          },
                          "duration": {
                              "$id": "#duration",
                              "$ref": "#/definitions/timedelta",
                              "title": "Duration",
                              "default": 600,
                              "minimum": 1,
                              "description": "Duration of this observation (seconds)"
                          },
                          "calibrator": {
                              "$ref": "#/definitions/calibrator",
                              "default": {}
                          },
                          "correlator": {
                              "$ref": "#/definitions/correlator",
                              "default": {}
                          },
                          "station_configuration": {
                              "$ref": "#/definitions/station_configuration",
                              "default": {}
                          }
                      },
                      "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false,
                      "definitions": {
                          "QA": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Perform all Quality Assurance (QA) tasks, including file conversion and plotting.",
                              "properties": {
                                  "file_conversion": {
                                      "$ref": "#/definitions/file_conversion",
                                      "default": {}
                                  },
                                  "inspection_plots": {
                                      "$ref": "#/definitions/inspection_plots",
                                      "default": "msplots"
                                  },
                                  "plots": {
                                      "$ref": "#/definitions/plots",
                                      "default": {}
                                  }
                              },
                              "required": [
                                  "file_conversion",
                                  "plots",
                                  "inspection_plots"
                              ],
                              "title": "QA",
                              "type": "object"
                          },
                          "plots": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Create dynamic spectrum plots",
                              "properties": {
                                  "autocorrelation": {
                                      "default": true,
                                      "description": "Create autocorrelation plots for all stations",
                                      "title": "autocorrelation",
                                      "type": "boolean"
                                  },
                                  "crosscorrelation": {
                                      "default": true,
                                      "description": "Create crosscorrelation plots for all baselines",
                                      "title": "crosscorrelation",
                                      "type": "boolean"
                                  },
                                  "enabled": {
                                      "default": true,
                                      "description": "Do/Don't create plots from the QA file from the observation",
                                      "title": "enabled",
                                      "type": "boolean"
                                  }
                              },
                              "required": [
                                  "enabled",
                                  "autocorrelation",
                                  "crosscorrelation"
                              ],
                              "title": "Plots",
                              "type": "object"
                          },
                          "file_conversion": {
                              "additionalProperties": false,
                              "default": {},
                              "description": "Create a QA file for the observation",
                              "properties": {
                                  "enabled": {
                                      "default": true,
                                      "description": "Do/Don't create a QA file for the observation",
                                      "title": "enabled",
                                      "type": "boolean"
                                  },
                                  "nr_of_subbands": {
                                      "default": -1,
                                      "description": "Keep this number of subbands from the observation in the QA file, or all if -1",
                                      "title": "#subbands",
                                      "type": "integer"
                                  },
                                  "nr_of_timestamps": {
                                      "default": 256,
                                      "description": "Extract this number of timestamps from the observation in the QA file (equidistantanly sampled, no averaging/interpolation)",
                                      "minimum": 1,
                                      "title": "#timestamps",
                                      "type": "integer"
                                  }
                              },
                              "required": [
                                  "enabled",
                                  "nr_of_subbands",
                                  "nr_of_timestamps"
                              ],
                              "title": "File Conversion",
                              "type": "object"
                          },
                          "inspection_plots": {
                              "decription": "Type of inspection plots to run",
                              "default": "msplots",
                              "enum": [
                                  "msplots",
                                  "dynspec",
                                  "none"
                              ],
                              "title": "Inspection Plots",
                              "type": "string"
                          },
                          "timedelta": {
                              "default": 0,
                              "description": "A time duration or delta expressed in seconds",
                              "type": "number"
                          },
                          "timestamp": {
                              "default": "1970-01-01T00:00:00Z",
                              "description": "A timestamp defined in UTC",
                              "format": "date-time",
                              "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                              "type": "string"
                          },
                          "calibrator": {
                              "additionalProperties": false,
                              "default": {},
                              "properties": {
                                  "autoselect": {
                                      "default": true,
                                      "description": "Auto-select calibrator based on elevation",
                                      "title": "Auto-select",
                                      "type": "boolean"
                                  },
                                  "name": {
                                      "default": "calibrator",
                                      "description": "Name of the calibrator SAP",
                                      "title": "Name",
                                      "type": "string"
                                  },
                                  "pointing": {
                                      "$ref": "#/definitions/pointing",
                                      "default": {},
                                      "description": "Manually selected calibrator",
                                      "title": "Digital pointing"
                                  }
                              },
                              "required": [
                                  "autoselect",
                                  "pointing",
                                  "name"
                              ],
                              "type": "object"
                          },
                          "pointing": {
                              "additionalProperties": false,
                              "properties": {
                                  "angle1": {
                                      "default": 0.6624317181687094,
                                      "description": "First angle (e.g. RA)",
                                      "title": "Angle 1",
                                      "type": "number"
                                  },
                                  "angle2": {
                                      "default": 1.5579526427549426,
                                      "description": "Second angle (e.g. DEC)",
                                      "title": "Angle 2",
                                      "type": "number"
                                  },
                                  "direction_type": {
                                      "default": "J2000",
                                      "description": "",
                                      "enum": [
                                          "J2000",
                                          "AZELGEO",
                                          "LMN",
                                          "SUN",
                                          "MOON",
                                          "MERCURY",
                                          "VENUS",
                                          "MARS",
                                          "JUPITER",
                                          "SATURN",
                                          "URANUS",
                                          "NEPTUNE",
                                          "PLUTO"
                                      ],
                                      "title": "Reference frame",
                                      "type": "string"
                                  },
                                  "target": {
                                      "default": "_target_name_",
                                      "description": "Description of where this beam points at",
                                      "minLength": 1,
                                      "title": "Target",
                                      "type": "string"
                                  }
                              },
                              "required": [
                                  "angle1",
                                  "angle2",
                                  "target"
                              ],
                              "type": "object"
                          },
                          "correlator": {
                              "additionalProperties": false,
                              "default": {},
                              "properties": {
                                  "channels_per_subband": {
                                      "default": 64,
                                      "description": "Number of frequency bands per subband",
                                      "enum": [
                                          8,
                                          16,
                                          32,
                                          64,
                                          128,
                                          256,
                                          512,
                                          1024
                                      ],
                                      "minimum": 8,
                                      "title": "Channels/subband",
                                      "type": "integer"
                                  },
                                  "integration_time": {
                                      "$ref": "#/definitions/timedelta",
                                      "default": 1,
                                      "description": "Desired integration period (seconds)",
                                      "minimum": 0.1,
                                      "title": "Integration time"
                                  },
                                  "storage_cluster": {
                                      "default": "CEP4",
                                      "description": "Cluster to write output to",
                                      "enum": [
                                          "CEP4",
                                          "DragNet"
                                      ],
                                      "title": "Storage cluster",
                                      "type": "string"
                                  },
                                  "topocentric_frequency_correction": {
                                      "default": false,
                                      "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation.",
                                      "title": "Topocentric Frequency Correction",
                                      "type": "boolean"
                                  }
                              }
                          },
                          "station_configuration": {
                              "default": {},
                              "properties": {
                                  "SAPs": {
                                      "$ref": "#/definitions/SAPs",
                                      "default": [
                                          {}
                                      ],
                                      "minItems": 1
                                  },
                                  "antenna_set": {
                                      "$ref": "#/definitions/antenna_set",
                                      "default": "HBA_DUAL"
                                  },
                                  "filter": {
                                      "$ref": "#/definitions/filter",
                                      "default": "HBA_110_190"
                                  },
                                  "station_groups": {
                                      "$ref": "#/definitions/station_groups",
                                      "default": [
                                          {
                                              "max_nr_missing": 1,
                                              "stations": [
                                                  "CS002",
                                                  "CS003",
                                                  "CS004",
                                                  "CS005",
                                                  "CS006",
                                                  "CS007"
                                              ]
                                          }
                                      ]
                                  },
                                  "tile_beam": {
                                      "$ref": "#/definitions/pointing",
                                      "default": {},
                                      "description": "HBA only",
                                      "title": "Tile beam"
                                  }
                              },
                              "required": [
                                  "station_groups",
                                  "antenna_set",
                                  "filter",
                                  "SAPs"
                              ],
                              "title": "station_configuration",
                              "type": "object"
                          },
                          "station": {
                              "description": "These are the LOFAR stations",
                              "enum": [
                                  "CS001",
                                  "CS002",
                                  "CS003",
                                  "CS004",
                                  "CS005",
                                  "CS006",
                                  "CS007",
                                  "CS011",
                                  "CS013",
                                  "CS017",
                                  "CS021",
                                  "CS024",
                                  "CS026",
                                  "CS028",
                                  "CS030",
                                  "CS031",
                                  "CS032",
                                  "CS101",
                                  "CS103",
                                  "CS201",
                                  "CS301",
                                  "CS302",
                                  "CS401",
                                  "CS501",
                                  "RS106",
                                  "RS205",
                                  "RS208",
                                  "RS210",
                                  "RS305",
                                  "RS306",
                                  "RS307",
                                  "RS310",
                                  "RS406",
                                  "RS407",
                                  "RS409",
                                  "RS503",
                                  "RS508",
                                  "RS509",
                                  "DE601",
                                  "DE602",
                                  "DE603",
                                  "DE604",
                                  "DE605",
                                  "FR606",
                                  "SE607",
                                  "UK608",
                                  "DE609",
                                  "PL610",
                                  "PL611",
                                  "PL612",
                                  "IE613",
                                  "LV614"
                              ],
                              "title": "Station",
                              "type": "string"
                          },
                          "station_groups": {
                              "additionalItems": false,
                              "additionalProperties": false,
                              "default": [
                                  {
                                      "max_nr_missing": 1,
                                      "stations": [
                                          "CS002",
                                          "CS003",
                                          "CS004",
                                          "CS005",
                                          "CS006",
                                          "CS007"
                                      ]
                                  }
                              ],
                              "description": "One or more predefined or custom groups of stations",
                              "items": {
                                  "$ref": "#/definitions/station_group"
                              },
                              "minItems": 1,
                              "title": "Station groups",
                              "type": "array"
                          },
                          "max_number_of_missing_stations": {
                              "default": 0,
                              "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                              "minimum": 0,
                              "title": "Maximum number of stations to omit",
                              "type": "integer"
                          },
                          "SAPs": {
                              "additionalItems": false,
                              "default": [
                                  {}
                              ],
                              "description": "Station beams",
                              "items": {
                                  "additionalProperties": false,
                                  "default": {},
                                  "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                  "properties": {
                                      "digital_pointing": {
                                          "$id": "#target_pointing",
                                          "$ref": "#/definitions/pointing",
                                          "default": {},
                                          "title": "Digital pointing"
                                      },
                                      "name": {
                                          "default": "_SAP_name_",
                                          "description": "Identifier for this beam",
                                          "minLength": 1,
                                          "title": "Name",
                                          "type": "string"
                                      },
                                      "subbands": {
                                          "additionalItems": false,
                                          "default": [
                                              255
                                          ],
                                          "items": {
                                              "maximum": 511,
                                              "minimum": 0,
                                              "title": "Subband",
                                              "type": "integer"
                                          },
                                          "maxItems": 488,
                                          "minItems": 1,
                                          "title": "Subband list",
                                          "type": "array"
                                      }
                                  },
                                  "required": [
                                      "name",
                                      "digital_pointing",
                                      "subbands"
                                  ],
                                  "title": "SAP",
                                  "type": "object"
                              },
                              "minItems": 0,
                              "title": "SAPs",
                              "type": "array"
                          },
                          "antenna_set": {
                              "default": "HBA_DUAL",
                              "description": "Fields & antennas to use",
                              "enum": [
                                  "HBA_DUAL",
                                  "HBA_DUAL_INNER",
                                  "HBA_ONE",
                                  "HBA_ONE_INNER",
                                  "HBA_ZERO",
                                  "HBA_ZERO_INNER",
                                  "LBA_INNER",
                                  "LBA_OUTER",
                                  "LBA_SPARSE_EVEN",
                                  "LBA_SPARSE_ODD",
                                  "LBA_ALL"
                              ],
                              "title": "Antenna set",
                              "type": "string"
                          },
                          "filter": {
                              "default": "HBA_110_190",
                              "description": "Must match antenna type",
                              "enum": [
                                  "LBA_10_70",
                                  "LBA_30_70",
                                  "LBA_10_90",
                                  "LBA_30_90",
                                  "HBA_110_190",
                                  "HBA_210_250"
                              ],
                              "title": "Band-pass filter",
                              "type": "string"
                          },
                          "station_list": {
                              "additionalItems": false,
                              "additionalProperties": false,
                              "default": [],
                              "items": {
                                  "$ref": "#/definitions/station"
                              },
                              "minItems": 0,
                              "type": "array",
                              "uniqueItems": true
                          },
                          "station_group": {
                              "anyOf": [
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 0,
                                          "stations": [
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007"
                                          ]
                                      },
                                      "description": "The group of all stations on the Superterp",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Superterp",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 4,
                                          "stations": [
                                              "CS001",
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007",
                                              "CS011",
                                              "CS013",
                                              "CS017",
                                              "CS021",
                                              "CS024",
                                              "CS026",
                                              "CS028",
                                              "CS030",
                                              "CS031",
                                              "CS032",
                                              "CS101",
                                              "CS103",
                                              "CS201",
                                              "CS301",
                                              "CS302",
                                              "CS401",
                                              "CS501"
                                          ]
                                      },
                                      "description": "The group of all Core stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS001",
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007",
                                                      "CS011",
                                                      "CS013",
                                                      "CS017",
                                                      "CS021",
                                                      "CS024",
                                                      "CS026",
                                                      "CS028",
                                                      "CS030",
                                                      "CS031",
                                                      "CS032",
                                                      "CS101",
                                                      "CS103",
                                                      "CS201",
                                                      "CS301",
                                                      "CS302",
                                                      "CS401",
                                                      "CS501"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Core",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 4,
                                          "stations": [
                                              "RS106",
                                              "RS205",
                                              "RS208",
                                              "RS210",
                                              "RS305",
                                              "RS306",
                                              "RS307",
                                              "RS310",
                                              "RS406",
                                              "RS407",
                                              "RS409",
                                              "RS503",
                                              "RS508",
                                              "RS509"
                                          ]
                                      },
                                      "description": "The group of all Dutch remote stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "RS106",
                                                      "RS205",
                                                      "RS208",
                                                      "RS210",
                                                      "RS305",
                                                      "RS306",
                                                      "RS307",
                                                      "RS310",
                                                      "RS406",
                                                      "RS407",
                                                      "RS409",
                                                      "RS503",
                                                      "RS508",
                                                      "RS509"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Remote",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 4,
                                          "stations": [
                                              "CS001",
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007",
                                              "CS011",
                                              "CS013",
                                              "CS017",
                                              "CS021",
                                              "CS024",
                                              "CS026",
                                              "CS028",
                                              "CS030",
                                              "CS031",
                                              "CS032",
                                              "CS101",
                                              "CS103",
                                              "CS201",
                                              "CS301",
                                              "CS302",
                                              "CS401",
                                              "CS501",
                                              "RS106",
                                              "RS205",
                                              "RS208",
                                              "RS210",
                                              "RS305",
                                              "RS306",
                                              "RS307",
                                              "RS310",
                                              "RS406",
                                              "RS407",
                                              "RS409",
                                              "RS503",
                                              "RS508",
                                              "RS509"
                                          ]
                                      },
                                      "description": "The group of all Dutch (Core + Remote) stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS001",
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007",
                                                      "CS011",
                                                      "CS013",
                                                      "CS017",
                                                      "CS021",
                                                      "CS024",
                                                      "CS026",
                                                      "CS028",
                                                      "CS030",
                                                      "CS031",
                                                      "CS032",
                                                      "CS101",
                                                      "CS103",
                                                      "CS201",
                                                      "CS301",
                                                      "CS302",
                                                      "CS401",
                                                      "CS501",
                                                      "RS106",
                                                      "RS205",
                                                      "RS208",
                                                      "RS210",
                                                      "RS305",
                                                      "RS306",
                                                      "RS307",
                                                      "RS310",
                                                      "RS406",
                                                      "RS407",
                                                      "RS409",
                                                      "RS503",
                                                      "RS508",
                                                      "RS509"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Dutch",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 2,
                                          "stations": [
                                              "DE601",
                                              "DE602",
                                              "DE603",
                                              "DE604",
                                              "DE605",
                                              "DE609",
                                              "FR606",
                                              "SE607",
                                              "UK608",
                                              "PL610",
                                              "PL611",
                                              "PL612",
                                              "IE613",
                                              "LV614"
                                          ]
                                      },
                                      "description": "The group of all international stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "DE601",
                                                      "DE602",
                                                      "DE603",
                                                      "DE604",
                                                      "DE605",
                                                      "DE609",
                                                      "FR606",
                                                      "SE607",
                                                      "UK608",
                                                      "PL610",
                                                      "PL611",
                                                      "PL612",
                                                      "IE613",
                                                      "LV614"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "International",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 1,
                                          "stations": [
                                              "DE601",
                                              "DE605"
                                          ]
                                      },
                                      "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "DE601",
                                                      "DE605"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "International required",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 6,
                                          "stations": [
                                              "CS001",
                                              "CS002",
                                              "CS003",
                                              "CS004",
                                              "CS005",
                                              "CS006",
                                              "CS007",
                                              "CS011",
                                              "CS013",
                                              "CS017",
                                              "CS021",
                                              "CS024",
                                              "CS026",
                                              "CS028",
                                              "CS030",
                                              "CS031",
                                              "CS032",
                                              "CS101",
                                              "CS103",
                                              "CS201",
                                              "CS301",
                                              "CS302",
                                              "CS401",
                                              "CS501",
                                              "RS106",
                                              "RS205",
                                              "RS208",
                                              "RS210",
                                              "RS305",
                                              "RS306",
                                              "RS307",
                                              "RS310",
                                              "RS406",
                                              "RS407",
                                              "RS409",
                                              "RS503",
                                              "RS508",
                                              "RS509",
                                              "DE601",
                                              "DE602",
                                              "DE603",
                                              "DE604",
                                              "DE605",
                                              "DE609",
                                              "FR606",
                                              "SE607",
                                              "UK608",
                                              "PL610",
                                              "PL611",
                                              "PL612",
                                              "IE613",
                                              "LV614"
                                          ]
                                      },
                                      "description": "The group of all (Core + Remote + International) stations",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list",
                                              "enum": [
                                                  [
                                                      "CS001",
                                                      "CS002",
                                                      "CS003",
                                                      "CS004",
                                                      "CS005",
                                                      "CS006",
                                                      "CS007",
                                                      "CS011",
                                                      "CS013",
                                                      "CS017",
                                                      "CS021",
                                                      "CS024",
                                                      "CS026",
                                                      "CS028",
                                                      "CS030",
                                                      "CS031",
                                                      "CS032",
                                                      "CS101",
                                                      "CS103",
                                                      "CS201",
                                                      "CS301",
                                                      "CS302",
                                                      "CS401",
                                                      "CS501",
                                                      "RS106",
                                                      "RS205",
                                                      "RS208",
                                                      "RS210",
                                                      "RS305",
                                                      "RS306",
                                                      "RS307",
                                                      "RS310",
                                                      "RS406",
                                                      "RS407",
                                                      "RS409",
                                                      "RS503",
                                                      "RS508",
                                                      "RS509",
                                                      "DE601",
                                                      "DE602",
                                                      "DE603",
                                                      "DE604",
                                                      "DE605",
                                                      "DE609",
                                                      "FR606",
                                                      "SE607",
                                                      "UK608",
                                                      "PL610",
                                                      "PL611",
                                                      "PL612",
                                                      "IE613",
                                                      "LV614"
                                                  ]
                                              ]
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "All",
                                      "type": "object"
                                  },
                                  {
                                      "additionalProperties": false,
                                      "default": {
                                          "max_nr_missing": 0,
                                          "stations": [
                                              "CS001"
                                          ]
                                      },
                                      "description": "A custom group of stations which can be defined by the user",
                                      "properties": {
                                          "max_nr_missing": {
                                              "$ref": "#/definitions/max_number_of_missing_stations"
                                          },
                                          "stations": {
                                              "$ref": "#/definitions/station_list"
                                          }
                                      },
                                      "required": [
                                          "stations",
                                          "max_nr_missing"
                                      ],
                                      "title": "Custom",
                                      "type": "object"
                                  }
                              ],
                              "default": {
                                  "max_nr_missing": 1,
                                  "stations": [
                                      "CS002",
                                      "CS003",
                                      "CS004",
                                      "CS005",
                                      "CS006",
                                      "CS007"
                                  ]
                              },
                              "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                              "title": "Station group",
                              "type": "object"
                          }
                      }
                  },
                  "schema": {
                      "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20observation/2#",
                      "type": "object",
                      "title": "parallel calibrator target observation",
                      "$schema": "http://json-schema.org/draft-06/schema#",
                      "version": 2,
                      "required": [
                          "calibrator",
                          "correlator",
                          "duration",
                          "station_configuration"
                      ],
                      "properties": {
                          "QA": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/2#/definitions/QA",
                              "default": {}
                          },
                          "duration": {
                              "$id": "#duration",
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                              "title": "Duration",
                              "default": 600,
                              "minimum": 1,
                              "description": "Duration of this observation (seconds)"
                          },
                          "calibrator": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/calibrator/1#/definitions/calibrator",
                              "default": {}
                          },
                          "correlator": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/correlator/1#/definitions/correlator",
                              "default": {}
                          },
                          "station_configuration": {
                              "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                              "default": {}
                          }
                      },
                      "description": "This schema combines the calibrator and target observation task schema's for parallel observing.",
                      "patternProperties": {
                          "^[$]schema$": {}
                      },
                      "additionalProperties": false
                  },
                  "state": "http://localhost:3000/api/template_state/active",
                  "state_value": "active",
                  "tags": [],
                  "type": "http://localhost:3000/api/task_type/observation",
                  "type_value": "observation",
                  "updated_at": "2022-04-14T06:20:07.273910",
                  "version": 2
              },
              "specifications_template_id": 5,
              "status": "scheduled",
              "status_value": "scheduled",
              "subtasks": [
                  "http://localhost:3000/api/subtask/2000011",
                  "http://localhost:3000/api/subtask/2000012",
                  "http://localhost:3000/api/subtask/2000010"
              ],
              "subtasks_ids": [
                  2000011,
                  2000012,
                  2000010
              ],
              "tags": [],
              "task_type": "observation",
              "unschedulable_reason": "",
              "updated_at": "2022-04-14T06:20:22.577280"
          }
      ],
      "task_blueprints_ids": [
          14,
          10,
          15,
          11,
          13,
          12,
          9
      ],
      "unschedulable_reason": "",
      "updated_at": "2022-04-14T06:20:22.543489"
    },
    {
        "id": 4,
        "url": "http://localhost:3000/api/scheduling_unit_blueprint/4",
        "actual_on_sky_start_time": null,
        "actual_on_sky_stop_time": null,
        "actual_process_start_time": null,
        "actual_process_stop_time": null,
        "created_at": "2022-05-09T05:13:05.262982",
        "description": "Test scheduling unit",
        "draft": "http://localhost:3000/api/scheduling_unit_draft/4",
        "draft_id": 4,
        "duration": 90900,
        "error_reason": "",
        "global_identifier": "http://localhost:3000/api/sip_identifier/58",
        "global_identifier_id": 58,
        "ingest_permission_granted_since": null,
        "ingest_permission_required": true,
        "interrupts_telescope": false,
        "name": "IM HBA LoTSS - 2 Beams",
        "observed_duration": null,
        "observed_start_time": null,
        "observed_stop_time": null,
        "obsolete_since": null,
        "on_sky_duration": 30000,
        "on_sky_start_time": "2022-05-09T06:08:49.932056",
        "on_sky_stop_time": "2022-05-09T14:30:49.932056",
        "output_pinned": false,
        "piggyback_allowed_aartfaac": true,
        "piggyback_allowed_tbb": true,
        "priority_queue": "http://localhost:3000/api/priority_queue_type/A",
        "priority_queue_value": "A",
        "process_start_time": "2022-05-09T06:08:49.932056",
        "process_stop_time": "2022-05-09T22:24:49.932056",
        "rank": 1,
        "scheduled_start_time": "2022-05-09T06:08:49.932056",
        "scheduled_stop_time": "2022-05-09T22:24:49.932056",
        "scheduling_constraints_doc": {
            "sky": {
                "min_distance": {
                    "sun": 0.5,
                    "moon": 0.5,
                    "jupiter": 0.5
                },
                "min_elevation": {
                    "target": 0.5,
                    "calibrator": 0.5
                },
                "transit_offset": {
                    "to": 7200,
                    "from": -7200
                }
            },
            "time": {
                "at": "2022-05-09T06:08:49.932056"
            },
            "daily": {
                "require_day": false,
                "require_night": false,
                "avoid_twilight": false
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
            "scheduler": "fixed_time"
        },
        "scheduling_constraints_template": "http://localhost:3000/api/scheduling_constraints_template/1",
        "scheduling_constraints_template_id": 1,
        "specifications_template": "http://localhost:3000/api/scheduling_unit_template/1",
        "specifications_template_id": 1,
        "status": "scheduled",
        "status_value": "scheduled",
        "tags": [],
        "task_blueprints": [
            {
                "id": 23,
                "url": "http://localhost:3000/api/task_blueprint/23",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_blueprint/38"
                ],
                "consumed_by_ids": [
                    38
                ],
                "created_at": "2022-05-09T05:13:05.378424",
                "description": "Preprocessing Pipeline for Calibrator Observation 1",
                "draft": "http://localhost:3000/api/task_draft/23",
                "draft_id": 23,
                "duration": 600,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Calibrator Pipeline 1",
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "output_pinned": false,
                "process_start_time": "2022-05-09T06:18:49.932056",
                "process_stop_time": "2022-05-09T06:28:49.932056",
                "produced_by": [
                    "http://localhost:3000/api/task_relation_blueprint/34"
                ],
                "produced_by_ids": [
                    34
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduled_start_time": "2022-05-09T06:18:49.932056",
                "scheduled_stop_time": "2022-05-09T06:28:49.932056",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
                "scheduling_unit_blueprint_id": 4,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Cal1 3Cabc/PP",
                "specifications_doc": {
                    "flag": {
                        "rfi_strategy": "HBAdefault",
                        "outerchannels": true,
                        "autocorrelations": true
                    },
                    "demix": {
                        "sources": [],
                        "time_steps": 10,
                        "ignore_target": false,
                        "frequency_steps": 64
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "average": {
                        "time_steps": 1,
                        "frequency_steps": 4
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    }
                },
                "specifications_template": {
                    "id": 10,
                    "url": "http://localhost:3000/api/task_template/10",
                    "created_at": "2022-05-09T05:12:34.774684",
                    "description": "This schema defines the parameters for a preprocessing pipeline.",
                    "name": "preprocessing pipeline",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "type": "object",
                        "title": "preprocessing pipeline",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "average"
                        ],
                        "properties": {
                            "flag": {
                                "type": "object",
                                "title": "Flagging",
                                "default": {},
                                "required": [
                                    "outerchannels",
                                    "autocorrelations",
                                    "rfi_strategy"
                                ],
                                "properties": {
                                    "rfi_strategy": {
                                        "enum": [
                                            "none",
                                            "HBAdefault",
                                            "LBAdefault"
                                        ],
                                        "type": "string",
                                        "title": "RFI flagging strategy",
                                        "default": "HBAdefault"
                                    },
                                    "outerchannels": {
                                        "type": "boolean",
                                        "title": "Flag outer channels",
                                        "default": true
                                    },
                                    "autocorrelations": {
                                        "type": "boolean",
                                        "title": "Flag auto correlations",
                                        "default": true
                                    }
                                },
                                "additionalProperties": false
                            },
                            "demix": {
                                "type": "object",
                                "title": "Demixing",
                                "default": {},
                                "required": [],
                                "properties": {
                                    "sources": {
                                        "type": "array",
                                        "items": {
                                            "enum": [
                                                "CasA",
                                                "CygA",
                                                "HerA",
                                                "HydraA",
                                                "TauA",
                                                "VirA"
                                            ],
                                            "type": "string",
                                            "default": "CasA"
                                        },
                                        "maxItems": 2,
                                        "minItems": 0,
                                        "uniqueItems": true,
                                        "additionalItems": false
                                    },
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 10,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging time steps"
                                    },
                                    "ignore_target": {
                                        "type": "boolean",
                                        "title": "Ignore target",
                                        "default": false
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 64,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging frequency steps"
                                    }
                                },
                                "additionalProperties": false
                            },
                            "average": {
                                "type": "object",
                                "title": "Averaging",
                                "default": {},
                                "required": [
                                    "frequency_steps",
                                    "time_steps"
                                ],
                                "properties": {
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 1,
                                        "minimum": 1
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 4,
                                        "minimum": 1
                                    }
                                },
                                "additionalProperties": false
                            },
                            "storagemanager": {
                                "enum": [
                                    "standard",
                                    "dysco"
                                ],
                                "type": "string",
                                "title": "Storage Manager",
                                "default": "dysco"
                            },
                            "cluster_resources": {
                                "$ref": "#/definitions/cluster_resources",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for a preprocessing pipeline.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "cluster_resources": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Which cluster resources to claim for this pipeline.",
                                "properties": {
                                    "cores_per_task": {
                                        "default": 2,
                                        "description": "Number of cores to reserve for each process in the pipeline.",
                                        "maximum": 24,
                                        "minimum": 1,
                                        "title": "Cores/task",
                                        "type": "integer"
                                    },
                                    "parallel_tasks": {
                                        "default": 122,
                                        "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                        "maximum": 488,
                                        "minimum": 1,
                                        "title": "Max parallel tasks",
                                        "type": "integer"
                                    },
                                    "where": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "description": "Where to run this pipeline.",
                                        "properties": {
                                            "cluster": {
                                                "default": "CEP4",
                                                "enum": [
                                                    "CEP4"
                                                ],
                                                "title": "Cluster",
                                                "type": "string"
                                            },
                                            "partition": {
                                                "default": "cpu",
                                                "enum": [
                                                    "cpu",
                                                    "gpu",
                                                    "testing"
                                                ],
                                                "title": "Partition",
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "cluster",
                                            "partition"
                                        ],
                                        "title": "Where",
                                        "type": "object"
                                    }
                                },
                                "required": [
                                    "where",
                                    "cores_per_task",
                                    "parallel_tasks"
                                ],
                                "title": "Cluster resources",
                                "type": "object"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                        "type": "object",
                        "title": "preprocessing pipeline",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "average"
                        ],
                        "properties": {
                            "flag": {
                                "type": "object",
                                "title": "Flagging",
                                "default": {},
                                "required": [
                                    "outerchannels",
                                    "autocorrelations",
                                    "rfi_strategy"
                                ],
                                "properties": {
                                    "rfi_strategy": {
                                        "enum": [
                                            "none",
                                            "HBAdefault",
                                            "LBAdefault"
                                        ],
                                        "type": "string",
                                        "title": "RFI flagging strategy",
                                        "default": "HBAdefault"
                                    },
                                    "outerchannels": {
                                        "type": "boolean",
                                        "title": "Flag outer channels",
                                        "default": true
                                    },
                                    "autocorrelations": {
                                        "type": "boolean",
                                        "title": "Flag auto correlations",
                                        "default": true
                                    }
                                },
                                "additionalProperties": false
                            },
                            "demix": {
                                "type": "object",
                                "title": "Demixing",
                                "default": {},
                                "required": [],
                                "properties": {
                                    "sources": {
                                        "type": "array",
                                        "items": {
                                            "enum": [
                                                "CasA",
                                                "CygA",
                                                "HerA",
                                                "HydraA",
                                                "TauA",
                                                "VirA"
                                            ],
                                            "type": "string",
                                            "default": "CasA"
                                        },
                                        "maxItems": 2,
                                        "minItems": 0,
                                        "uniqueItems": true,
                                        "additionalItems": false
                                    },
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 10,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging time steps"
                                    },
                                    "ignore_target": {
                                        "type": "boolean",
                                        "title": "Ignore target",
                                        "default": false
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 64,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging frequency steps"
                                    }
                                },
                                "additionalProperties": false
                            },
                            "average": {
                                "type": "object",
                                "title": "Averaging",
                                "default": {},
                                "required": [
                                    "frequency_steps",
                                    "time_steps"
                                ],
                                "properties": {
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 1,
                                        "minimum": 1
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 4,
                                        "minimum": 1
                                    }
                                },
                                "additionalProperties": false
                            },
                            "storagemanager": {
                                "enum": [
                                    "standard",
                                    "dysco"
                                ],
                                "type": "string",
                                "title": "Storage Manager",
                                "default": "dysco"
                            },
                            "cluster_resources": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for a preprocessing pipeline.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/pipeline",
                    "type_value": "pipeline",
                    "updated_at": "2022-05-09T05:12:34.774747",
                    "version": 2
                },
                "specifications_template_id": 10,
                "status": "schedulable",
                "status_value": "schedulable",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000038"
                ],
                "subtasks_ids": [
                    2000038
                ],
                "tags": [],
                "task_type": "pipeline",
                "unschedulable_reason": "",
                "updated_at": "2022-05-09T05:13:05.378438"
            },
            {
                "id": 24,
                "url": "http://localhost:3000/api/task_blueprint/24",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_blueprint/36",
                    "http://localhost:3000/api/task_relation_blueprint/37"
                ],
                "consumed_by_ids": [
                    36,
                    37
                ],
                "created_at": "2022-05-09T05:13:05.477848",
                "description": "Target Observation for LoTSS scheduling unit. HBA_DUAL_INNER, 120-168 MHz, 1s, 64ch/sb ",
                "draft": "http://localhost:3000/api/task_draft/24",
                "draft_id": 24,
                "duration": 29400,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Target Observation",
                "obsolete_since": null,
                "on_sky_duration": 28800,
                "on_sky_start_time": "2022-05-09T06:19:49.932056",
                "on_sky_stop_time": "2022-05-09T14:19:49.932056",
                "output_pinned": false,
                "process_start_time": "2022-05-09T06:19:49.932056",
                "process_stop_time": "2022-05-09T14:29:49.932056",
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 660,
                "relative_stop_time": 29460,
                "scheduled_start_time": "2022-05-09T06:19:49.932056",
                "scheduled_stop_time": "2022-05-09T14:29:49.932056",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
                "scheduling_unit_blueprint_id": 4,
                "second_scheduling_relation": [
                    "http://localhost:3000/api/task_scheduling_relation_blueprint/1",
                    "http://localhost:3000/api/task_scheduling_relation_blueprint/2"
                ],
                "second_scheduling_relation_ids": [
                    1,
                    2
                ],
                "short_description": "Paaa+01 & Paaa+02",
                "specifications_doc": {
                    "QA": {
                        "plots": {
                            "enabled": true,
                            "autocorrelation": true,
                            "crosscorrelation": true
                        },
                        "file_conversion": {
                            "enabled": true,
                            "nr_of_subbands": -1,
                            "nr_of_timestamps": 256
                        },
                        "inspection_plots": "msplots"
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/2/ref_resolved",
                    "duration": 28800,
                    "correlator": {
                        "storage_cluster": "CEP4",
                        "integration_time": 1,
                        "channels_per_subband": 64,
                        "topocentric_frequency_correction": false
                    },
                    "station_configuration": {
                        "SAPs": [
                            {
                                "name": "sap1",
                                "subbands": [
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    182,
                                    183,
                                    184,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    212,
                                    213,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    242,
                                    243,
                                    244,
                                    245,
                                    246,
                                    247,
                                    248,
                                    249,
                                    250,
                                    251,
                                    252,
                                    253,
                                    254,
                                    255,
                                    257,
                                    258,
                                    259,
                                    260,
                                    261,
                                    262,
                                    263,
                                    264,
                                    265,
                                    266,
                                    267,
                                    268,
                                    269,
                                    270,
                                    271,
                                    272,
                                    273,
                                    275,
                                    276,
                                    277,
                                    278,
                                    279,
                                    280,
                                    281,
                                    282,
                                    283,
                                    284,
                                    285,
                                    286,
                                    287,
                                    288,
                                    289,
                                    290,
                                    291,
                                    292,
                                    293,
                                    294,
                                    295,
                                    296,
                                    297,
                                    298,
                                    299,
                                    300,
                                    302,
                                    303,
                                    304,
                                    305,
                                    306,
                                    307,
                                    308,
                                    309,
                                    310,
                                    311,
                                    312,
                                    313,
                                    314,
                                    315,
                                    316,
                                    317,
                                    318,
                                    319,
                                    320,
                                    321,
                                    322,
                                    323,
                                    324,
                                    325,
                                    326,
                                    327,
                                    328,
                                    330,
                                    331,
                                    332,
                                    333,
                                    334,
                                    335,
                                    336,
                                    337,
                                    338,
                                    339,
                                    340,
                                    341,
                                    342,
                                    343,
                                    344,
                                    345,
                                    346,
                                    347,
                                    349,
                                    364,
                                    372,
                                    380,
                                    388,
                                    396,
                                    404,
                                    413,
                                    421,
                                    430,
                                    438,
                                    447
                                ],
                                "digital_pointing": {
                                    "angle1": 0.6624317181687094,
                                    "angle2": 1.5579526427549426,
                                    "target": "Paaa+01",
                                    "direction_type": "J2000"
                                }
                            },
                            {
                                "name": "sap2",
                                "subbands": [
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    182,
                                    183,
                                    184,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    212,
                                    213,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    242,
                                    243,
                                    244,
                                    245,
                                    246,
                                    247,
                                    248,
                                    249,
                                    250,
                                    251,
                                    252,
                                    253,
                                    254,
                                    255,
                                    257,
                                    258,
                                    259,
                                    260,
                                    261,
                                    262,
                                    263,
                                    264,
                                    265,
                                    266,
                                    267,
                                    268,
                                    269,
                                    270,
                                    271,
                                    272,
                                    273,
                                    275,
                                    276,
                                    277,
                                    278,
                                    279,
                                    280,
                                    281,
                                    282,
                                    283,
                                    284,
                                    285,
                                    286,
                                    287,
                                    288,
                                    289,
                                    290,
                                    291,
                                    292,
                                    293,
                                    294,
                                    295,
                                    296,
                                    297,
                                    298,
                                    299,
                                    300,
                                    302,
                                    303,
                                    304,
                                    305,
                                    306,
                                    307,
                                    308,
                                    309,
                                    310,
                                    311,
                                    312,
                                    313,
                                    314,
                                    315,
                                    316,
                                    317,
                                    318,
                                    319,
                                    320,
                                    321,
                                    322,
                                    323,
                                    324,
                                    325,
                                    326,
                                    327,
                                    328,
                                    330,
                                    331,
                                    332,
                                    333,
                                    334,
                                    335,
                                    336,
                                    337,
                                    338,
                                    339,
                                    340,
                                    341,
                                    342,
                                    343,
                                    344,
                                    345,
                                    346,
                                    347,
                                    349,
                                    364,
                                    372,
                                    380,
                                    388,
                                    396,
                                    404,
                                    413,
                                    421,
                                    430,
                                    438,
                                    447
                                ],
                                "digital_pointing": {
                                    "angle1": 0.6624317181687094,
                                    "angle2": 1.5579526427549426,
                                    "target": "Paaa+02",
                                    "direction_type": "J2000"
                                }
                            }
                        ],
                        "filter": "HBA_110_190",
                        "tile_beam": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "Paaa+01Paaa+02REF",
                            "direction_type": "J2000"
                        },
                        "antenna_set": "HBA_DUAL_INNER",
                        "station_groups": [
                            {
                                "stations": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501",
                                    "RS106",
                                    "RS205",
                                    "RS208",
                                    "RS210",
                                    "RS305",
                                    "RS306",
                                    "RS307",
                                    "RS310",
                                    "RS406",
                                    "RS407",
                                    "RS409",
                                    "RS503",
                                    "RS508",
                                    "RS509"
                                ],
                                "max_nr_missing": 4
                            },
                            {
                                "stations": [
                                    "DE601",
                                    "DE602",
                                    "DE603",
                                    "DE604",
                                    "DE605",
                                    "DE609",
                                    "FR606",
                                    "SE607",
                                    "UK608",
                                    "PL610",
                                    "PL611",
                                    "PL612",
                                    "IE613",
                                    "LV614"
                                ],
                                "max_nr_missing": 2
                            },
                            {
                                "stations": [
                                    "DE601",
                                    "DE605"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                },
                "specifications_template": {
                    "id": 4,
                    "url": "http://localhost:3000/api/task_template/4",
                    "created_at": "2022-05-09T05:12:31.829842",
                    "description": "This schema defines the parameters to setup a target observation task.",
                    "name": "target observation",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/2/ref_resolved",
                        "title": "target observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "correlator",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "QA": {
                                "$ref": "#/definitions/QA",
                                "default": {}
                            },
                            "duration": {
                                "$id": "#duration",
                                "$ref": "#/definitions/timedelta",
                                "title": "Duration",
                                "default": 600,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "correlator": {
                                "$ref": "#/definitions/correlator",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters to setup a target observation task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "QA": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Perform all Quality Assurance (QA) tasks, including file conversion and plotting.",
                                "properties": {
                                    "file_conversion": {
                                        "$ref": "#/definitions/file_conversion",
                                        "default": {}
                                    },
                                    "inspection_plots": {
                                        "$ref": "#/definitions/inspection_plots",
                                        "default": "msplots"
                                    },
                                    "plots": {
                                        "$ref": "#/definitions/plots",
                                        "default": {}
                                    }
                                },
                                "required": [
                                    "file_conversion",
                                    "plots",
                                    "inspection_plots"
                                ],
                                "title": "QA",
                                "type": "object"
                            },
                            "plots": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Create dynamic spectrum plots",
                                "properties": {
                                    "autocorrelation": {
                                        "default": true,
                                        "description": "Create autocorrelation plots for all stations",
                                        "title": "autocorrelation",
                                        "type": "boolean"
                                    },
                                    "crosscorrelation": {
                                        "default": true,
                                        "description": "Create crosscorrelation plots for all baselines",
                                        "title": "crosscorrelation",
                                        "type": "boolean"
                                    },
                                    "enabled": {
                                        "default": true,
                                        "description": "Do/Don't create plots from the QA file from the observation",
                                        "title": "enabled",
                                        "type": "boolean"
                                    }
                                },
                                "required": [
                                    "enabled",
                                    "autocorrelation",
                                    "crosscorrelation"
                                ],
                                "title": "Plots",
                                "type": "object"
                            },
                            "file_conversion": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Create a QA file for the observation",
                                "properties": {
                                    "enabled": {
                                        "default": true,
                                        "description": "Do/Don't create a QA file for the observation",
                                        "title": "enabled",
                                        "type": "boolean"
                                    },
                                    "nr_of_subbands": {
                                        "default": -1,
                                        "description": "Keep this number of subbands from the observation in the QA file, or all if -1",
                                        "title": "#subbands",
                                        "type": "integer"
                                    },
                                    "nr_of_timestamps": {
                                        "default": 256,
                                        "description": "Extract this number of timestamps from the observation in the QA file (equidistantanly sampled, no averaging/interpolation)",
                                        "minimum": 1,
                                        "title": "#timestamps",
                                        "type": "integer"
                                    }
                                },
                                "required": [
                                    "enabled",
                                    "nr_of_subbands",
                                    "nr_of_timestamps"
                                ],
                                "title": "File Conversion",
                                "type": "object"
                            },
                            "inspection_plots": {
                                "decription": "Type of inspection plots to run",
                                "default": "msplots",
                                "enum": [
                                    "msplots",
                                    "dynspec",
                                    "none"
                                ],
                                "title": "Inspection Plots",
                                "type": "string"
                            },
                            "timedelta": {
                                "default": 0,
                                "description": "A time duration or delta expressed in seconds",
                                "type": "number"
                            },
                            "timestamp": {
                                "default": "1970-01-01T00:00:00Z",
                                "description": "A timestamp defined in UTC",
                                "format": "date-time",
                                "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                                "type": "string"
                            },
                            "correlator": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "channels_per_subband": {
                                        "default": 64,
                                        "description": "Number of frequency bands per subband",
                                        "enum": [
                                            8,
                                            16,
                                            32,
                                            64,
                                            128,
                                            256,
                                            512,
                                            1024
                                        ],
                                        "minimum": 8,
                                        "title": "Channels/subband",
                                        "type": "integer"
                                    },
                                    "integration_time": {
                                        "$ref": "#/definitions/timedelta",
                                        "default": 1,
                                        "description": "Desired integration period (seconds)",
                                        "minimum": 0.1,
                                        "title": "Integration time"
                                    },
                                    "storage_cluster": {
                                        "default": "CEP4",
                                        "description": "Cluster to write output to",
                                        "enum": [
                                            "CEP4",
                                            "DragNet"
                                        ],
                                        "title": "Storage cluster",
                                        "type": "string"
                                    },
                                    "topocentric_frequency_correction": {
                                        "default": false,
                                        "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation.",
                                        "title": "Topocentric Frequency Correction",
                                        "type": "boolean"
                                    }
                                }
                            },
                            "station_configuration": {
                                "default": {},
                                "properties": {
                                    "SAPs": {
                                        "$ref": "#/definitions/SAPs",
                                        "default": [
                                            {}
                                        ],
                                        "minItems": 1
                                    },
                                    "antenna_set": {
                                        "$ref": "#/definitions/antenna_set",
                                        "default": "HBA_DUAL"
                                    },
                                    "filter": {
                                        "$ref": "#/definitions/filter",
                                        "default": "HBA_110_190"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 4,
                                                "stations": [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS101",
                                                    "CS103",
                                                    "CS201",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501"
                                                ]
                                            }
                                        ]
                                    },
                                    "tile_beam": {
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "description": "HBA only",
                                        "title": "Tile beam"
                                    }
                                },
                                "required": [
                                    "station_groups",
                                    "antenna_set",
                                    "filter",
                                    "SAPs"
                                ],
                                "title": "station_configuration",
                                "type": "object"
                            },
                            "antenna_set": {
                                "default": "HBA_DUAL",
                                "description": "Fields & antennas to use",
                                "enum": [
                                    "HBA_DUAL",
                                    "HBA_DUAL_INNER",
                                    "HBA_ONE",
                                    "HBA_ONE_INNER",
                                    "HBA_ZERO",
                                    "HBA_ZERO_INNER",
                                    "LBA_INNER",
                                    "LBA_OUTER",
                                    "LBA_SPARSE_EVEN",
                                    "LBA_SPARSE_ODD",
                                    "LBA_ALL"
                                ],
                                "title": "Antenna set",
                                "type": "string"
                            },
                            "filter": {
                                "default": "HBA_110_190",
                                "description": "Must match antenna type",
                                "enum": [
                                    "LBA_10_70",
                                    "LBA_30_70",
                                    "LBA_10_90",
                                    "LBA_30_90",
                                    "HBA_110_190",
                                    "HBA_210_250"
                                ],
                                "title": "Band-pass filter",
                                "type": "string"
                            },
                            "station": {
                                "description": "These are the LOFAR stations",
                                "enum": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501",
                                    "RS106",
                                    "RS205",
                                    "RS208",
                                    "RS210",
                                    "RS305",
                                    "RS306",
                                    "RS307",
                                    "RS310",
                                    "RS406",
                                    "RS407",
                                    "RS409",
                                    "RS503",
                                    "RS508",
                                    "RS509",
                                    "DE601",
                                    "DE602",
                                    "DE603",
                                    "DE604",
                                    "DE605",
                                    "FR606",
                                    "SE607",
                                    "UK608",
                                    "DE609",
                                    "PL610",
                                    "PL611",
                                    "PL612",
                                    "IE613",
                                    "LV614"
                                ],
                                "title": "Station",
                                "type": "string"
                            },
                            "max_number_of_missing_stations": {
                                "default": 0,
                                "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                                "minimum": 0,
                                "title": "Maximum number of stations to omit",
                                "type": "integer"
                            },
                            "SAPs": {
                                "additionalItems": false,
                                "default": [
                                    {}
                                ],
                                "description": "Station beams",
                                "items": {
                                    "additionalProperties": false,
                                    "default": {},
                                    "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                    "properties": {
                                        "digital_pointing": {
                                            "$id": "#target_pointing",
                                            "$ref": "#/definitions/pointing",
                                            "default": {},
                                            "title": "Digital pointing"
                                        },
                                        "name": {
                                            "default": "_SAP_name_",
                                            "description": "Identifier for this beam",
                                            "minLength": 1,
                                            "title": "Name",
                                            "type": "string"
                                        },
                                        "subbands": {
                                            "additionalItems": false,
                                            "default": [
                                                255
                                            ],
                                            "items": {
                                                "maximum": 511,
                                                "minimum": 0,
                                                "title": "Subband",
                                                "type": "integer"
                                            },
                                            "maxItems": 488,
                                            "minItems": 1,
                                            "title": "Subband list",
                                            "type": "array"
                                        }
                                    },
                                    "required": [
                                        "name",
                                        "digital_pointing",
                                        "subbands"
                                    ],
                                    "title": "SAP",
                                    "type": "object"
                                },
                                "minItems": 0,
                                "title": "SAPs",
                                "type": "array"
                            },
                            "station_group": {
                                "anyOf": [
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007"
                                            ]
                                        },
                                        "description": "The group of all stations on the Superterp",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Superterp",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501"
                                            ]
                                        },
                                        "description": "The group of all Core stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Core",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch remote stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Remote",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch (Core + Remote) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Dutch",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 2,
                                            "stations": [
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 1,
                                            "stations": [
                                                "DE601",
                                                "DE605"
                                            ]
                                        },
                                        "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE605"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International required",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 6,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509",
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all (Core + Remote + International) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509",
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "All",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS001"
                                            ]
                                        },
                                        "description": "A custom group of stations which can be defined by the user",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list"
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Custom",
                                        "type": "object"
                                    }
                                ],
                                "default": {
                                    "max_nr_missing": 1,
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                },
                                "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                                "title": "Station group",
                                "type": "object"
                            },
                            "station_list": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [],
                                "items": {
                                    "$ref": "#/definitions/station"
                                },
                                "minItems": 0,
                                "type": "array",
                                "uniqueItems": true
                            },
                            "station_groups": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [
                                    {
                                        "max_nr_missing": 1,
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ]
                                    }
                                ],
                                "description": "One or more predefined or custom groups of stations",
                                "items": {
                                    "$ref": "#/definitions/station_group"
                                },
                                "minItems": 1,
                                "title": "Station groups",
                                "type": "array"
                            },
                            "pointing": {
                                "additionalProperties": false,
                                "properties": {
                                    "angle1": {
                                        "default": 0.6624317181687094,
                                        "description": "First angle (e.g. RA)",
                                        "title": "Angle 1",
                                        "type": "number"
                                    },
                                    "angle2": {
                                        "default": 1.5579526427549426,
                                        "description": "Second angle (e.g. DEC)",
                                        "title": "Angle 2",
                                        "type": "number"
                                    },
                                    "direction_type": {
                                        "default": "J2000",
                                        "description": "",
                                        "enum": [
                                            "J2000",
                                            "AZELGEO",
                                            "LMN",
                                            "SUN",
                                            "MOON",
                                            "MERCURY",
                                            "VENUS",
                                            "MARS",
                                            "JUPITER",
                                            "SATURN",
                                            "URANUS",
                                            "NEPTUNE",
                                            "PLUTO"
                                        ],
                                        "title": "Reference frame",
                                        "type": "string"
                                    },
                                    "target": {
                                        "default": "_target_name_",
                                        "description": "Description of where this beam points at",
                                        "minLength": 1,
                                        "title": "Target",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "angle1",
                                    "angle2",
                                    "target"
                                ],
                                "type": "object"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/target%20observation/2#",
                        "title": "target observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "correlator",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "QA": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/2#/definitions/QA",
                                "default": {}
                            },
                            "duration": {
                                "$id": "#duration",
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                "title": "Duration",
                                "default": 600,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "correlator": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/correlator/1#/definitions/correlator",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters to setup a target observation task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/observation",
                    "type_value": "observation",
                    "updated_at": "2022-05-09T05:12:31.829889",
                    "version": 2
                },
                "specifications_template_id": 4,
                "status": "scheduled",
                "status_value": "scheduled",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000033",
                    "http://localhost:3000/api/subtask/2000032",
                    "http://localhost:3000/api/subtask/2000031"
                ],
                "subtasks_ids": [
                    2000033,
                    2000032,
                    2000031
                ],
                "tags": [],
                "task_type": "observation",
                "unschedulable_reason": "",
                "updated_at": "2022-05-09T05:13:05.477861"
            },
            {
                "id": 28,
                "url": "http://localhost:3000/api/task_blueprint/28",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_blueprint/40"
                ],
                "consumed_by_ids": [
                    40
                ],
                "created_at": "2022-05-09T05:13:05.670728",
                "description": "Preprocessing Pipeline for Target Observation target1, SAP000, 120-168 MHz, 1s, 16ch/sb",
                "draft": "http://localhost:3000/api/task_draft/28",
                "draft_id": 28,
                "duration": 28800,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Pipeline target1",
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "output_pinned": false,
                "process_start_time": "2022-05-09T14:19:49.932056",
                "process_stop_time": "2022-05-09T22:19:49.932056",
                "produced_by": [
                    "http://localhost:3000/api/task_relation_blueprint/37"
                ],
                "produced_by_ids": [
                    37
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduled_start_time": "2022-05-09T14:19:49.932056",
                "scheduled_stop_time": "2022-05-09T22:19:49.932056",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
                "scheduling_unit_blueprint_id": 4,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Paaa+01/TP",
                "specifications_doc": {
                    "flag": {
                        "rfi_strategy": "HBAdefault",
                        "outerchannels": true,
                        "autocorrelations": true
                    },
                    "demix": {
                        "sources": [],
                        "time_steps": 10,
                        "ignore_target": false,
                        "frequency_steps": 64
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "average": {
                        "time_steps": 1,
                        "frequency_steps": 4
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    }
                },
                "specifications_template": {
                    "id": 10,
                    "url": "http://localhost:3000/api/task_template/10",
                    "created_at": "2022-05-09T05:12:34.774684",
                    "description": "This schema defines the parameters for a preprocessing pipeline.",
                    "name": "preprocessing pipeline",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "type": "object",
                        "title": "preprocessing pipeline",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "average"
                        ],
                        "properties": {
                            "flag": {
                                "type": "object",
                                "title": "Flagging",
                                "default": {},
                                "required": [
                                    "outerchannels",
                                    "autocorrelations",
                                    "rfi_strategy"
                                ],
                                "properties": {
                                    "rfi_strategy": {
                                        "enum": [
                                            "none",
                                            "HBAdefault",
                                            "LBAdefault"
                                        ],
                                        "type": "string",
                                        "title": "RFI flagging strategy",
                                        "default": "HBAdefault"
                                    },
                                    "outerchannels": {
                                        "type": "boolean",
                                        "title": "Flag outer channels",
                                        "default": true
                                    },
                                    "autocorrelations": {
                                        "type": "boolean",
                                        "title": "Flag auto correlations",
                                        "default": true
                                    }
                                },
                                "additionalProperties": false
                            },
                            "demix": {
                                "type": "object",
                                "title": "Demixing",
                                "default": {},
                                "required": [],
                                "properties": {
                                    "sources": {
                                        "type": "array",
                                        "items": {
                                            "enum": [
                                                "CasA",
                                                "CygA",
                                                "HerA",
                                                "HydraA",
                                                "TauA",
                                                "VirA"
                                            ],
                                            "type": "string",
                                            "default": "CasA"
                                        },
                                        "maxItems": 2,
                                        "minItems": 0,
                                        "uniqueItems": true,
                                        "additionalItems": false
                                    },
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 10,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging time steps"
                                    },
                                    "ignore_target": {
                                        "type": "boolean",
                                        "title": "Ignore target",
                                        "default": false
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 64,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging frequency steps"
                                    }
                                },
                                "additionalProperties": false
                            },
                            "average": {
                                "type": "object",
                                "title": "Averaging",
                                "default": {},
                                "required": [
                                    "frequency_steps",
                                    "time_steps"
                                ],
                                "properties": {
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 1,
                                        "minimum": 1
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 4,
                                        "minimum": 1
                                    }
                                },
                                "additionalProperties": false
                            },
                            "storagemanager": {
                                "enum": [
                                    "standard",
                                    "dysco"
                                ],
                                "type": "string",
                                "title": "Storage Manager",
                                "default": "dysco"
                            },
                            "cluster_resources": {
                                "$ref": "#/definitions/cluster_resources",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for a preprocessing pipeline.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "cluster_resources": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Which cluster resources to claim for this pipeline.",
                                "properties": {
                                    "cores_per_task": {
                                        "default": 2,
                                        "description": "Number of cores to reserve for each process in the pipeline.",
                                        "maximum": 24,
                                        "minimum": 1,
                                        "title": "Cores/task",
                                        "type": "integer"
                                    },
                                    "parallel_tasks": {
                                        "default": 122,
                                        "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                        "maximum": 488,
                                        "minimum": 1,
                                        "title": "Max parallel tasks",
                                        "type": "integer"
                                    },
                                    "where": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "description": "Where to run this pipeline.",
                                        "properties": {
                                            "cluster": {
                                                "default": "CEP4",
                                                "enum": [
                                                    "CEP4"
                                                ],
                                                "title": "Cluster",
                                                "type": "string"
                                            },
                                            "partition": {
                                                "default": "cpu",
                                                "enum": [
                                                    "cpu",
                                                    "gpu",
                                                    "testing"
                                                ],
                                                "title": "Partition",
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "cluster",
                                            "partition"
                                        ],
                                        "title": "Where",
                                        "type": "object"
                                    }
                                },
                                "required": [
                                    "where",
                                    "cores_per_task",
                                    "parallel_tasks"
                                ],
                                "title": "Cluster resources",
                                "type": "object"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                        "type": "object",
                        "title": "preprocessing pipeline",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "average"
                        ],
                        "properties": {
                            "flag": {
                                "type": "object",
                                "title": "Flagging",
                                "default": {},
                                "required": [
                                    "outerchannels",
                                    "autocorrelations",
                                    "rfi_strategy"
                                ],
                                "properties": {
                                    "rfi_strategy": {
                                        "enum": [
                                            "none",
                                            "HBAdefault",
                                            "LBAdefault"
                                        ],
                                        "type": "string",
                                        "title": "RFI flagging strategy",
                                        "default": "HBAdefault"
                                    },
                                    "outerchannels": {
                                        "type": "boolean",
                                        "title": "Flag outer channels",
                                        "default": true
                                    },
                                    "autocorrelations": {
                                        "type": "boolean",
                                        "title": "Flag auto correlations",
                                        "default": true
                                    }
                                },
                                "additionalProperties": false
                            },
                            "demix": {
                                "type": "object",
                                "title": "Demixing",
                                "default": {},
                                "required": [],
                                "properties": {
                                    "sources": {
                                        "type": "array",
                                        "items": {
                                            "enum": [
                                                "CasA",
                                                "CygA",
                                                "HerA",
                                                "HydraA",
                                                "TauA",
                                                "VirA"
                                            ],
                                            "type": "string",
                                            "default": "CasA"
                                        },
                                        "maxItems": 2,
                                        "minItems": 0,
                                        "uniqueItems": true,
                                        "additionalItems": false
                                    },
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 10,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging time steps"
                                    },
                                    "ignore_target": {
                                        "type": "boolean",
                                        "title": "Ignore target",
                                        "default": false
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 64,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging frequency steps"
                                    }
                                },
                                "additionalProperties": false
                            },
                            "average": {
                                "type": "object",
                                "title": "Averaging",
                                "default": {},
                                "required": [
                                    "frequency_steps",
                                    "time_steps"
                                ],
                                "properties": {
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 1,
                                        "minimum": 1
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 4,
                                        "minimum": 1
                                    }
                                },
                                "additionalProperties": false
                            },
                            "storagemanager": {
                                "enum": [
                                    "standard",
                                    "dysco"
                                ],
                                "type": "string",
                                "title": "Storage Manager",
                                "default": "dysco"
                            },
                            "cluster_resources": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for a preprocessing pipeline.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/pipeline",
                    "type_value": "pipeline",
                    "updated_at": "2022-05-09T05:12:34.774747",
                    "version": 2
                },
                "specifications_template_id": 10,
                "status": "schedulable",
                "status_value": "schedulable",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000039"
                ],
                "subtasks_ids": [
                    2000039
                ],
                "tags": [],
                "task_type": "pipeline",
                "unschedulable_reason": "",
                "updated_at": "2022-05-09T05:13:05.670749"
            },
            {
                "id": 27,
                "url": "http://localhost:3000/api/task_blueprint/27",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_blueprint/41"
                ],
                "consumed_by_ids": [
                    41
                ],
                "created_at": "2022-05-09T05:13:05.624931",
                "description": "Preprocessing Pipeline for Target Observation target2, SAP001, 120-168 MHz, 1s, 16ch/sb",
                "draft": "http://localhost:3000/api/task_draft/27",
                "draft_id": 27,
                "duration": 28800,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Pipeline target2",
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "output_pinned": false,
                "process_start_time": "2022-05-09T14:19:49.932056",
                "process_stop_time": "2022-05-09T22:19:49.932056",
                "produced_by": [
                    "http://localhost:3000/api/task_relation_blueprint/36"
                ],
                "produced_by_ids": [
                    36
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduled_start_time": "2022-05-09T14:19:49.932056",
                "scheduled_stop_time": "2022-05-09T22:19:49.932056",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
                "scheduling_unit_blueprint_id": 4,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Paaa+02/TP",
                "specifications_doc": {
                    "flag": {
                        "rfi_strategy": "HBAdefault",
                        "outerchannels": true,
                        "autocorrelations": true
                    },
                    "demix": {
                        "sources": [],
                        "time_steps": 10,
                        "ignore_target": false,
                        "frequency_steps": 64
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "average": {
                        "time_steps": 1,
                        "frequency_steps": 4
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    }
                },
                "specifications_template": {
                    "id": 10,
                    "url": "http://localhost:3000/api/task_template/10",
                    "created_at": "2022-05-09T05:12:34.774684",
                    "description": "This schema defines the parameters for a preprocessing pipeline.",
                    "name": "preprocessing pipeline",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "type": "object",
                        "title": "preprocessing pipeline",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "average"
                        ],
                        "properties": {
                            "flag": {
                                "type": "object",
                                "title": "Flagging",
                                "default": {},
                                "required": [
                                    "outerchannels",
                                    "autocorrelations",
                                    "rfi_strategy"
                                ],
                                "properties": {
                                    "rfi_strategy": {
                                        "enum": [
                                            "none",
                                            "HBAdefault",
                                            "LBAdefault"
                                        ],
                                        "type": "string",
                                        "title": "RFI flagging strategy",
                                        "default": "HBAdefault"
                                    },
                                    "outerchannels": {
                                        "type": "boolean",
                                        "title": "Flag outer channels",
                                        "default": true
                                    },
                                    "autocorrelations": {
                                        "type": "boolean",
                                        "title": "Flag auto correlations",
                                        "default": true
                                    }
                                },
                                "additionalProperties": false
                            },
                            "demix": {
                                "type": "object",
                                "title": "Demixing",
                                "default": {},
                                "required": [],
                                "properties": {
                                    "sources": {
                                        "type": "array",
                                        "items": {
                                            "enum": [
                                                "CasA",
                                                "CygA",
                                                "HerA",
                                                "HydraA",
                                                "TauA",
                                                "VirA"
                                            ],
                                            "type": "string",
                                            "default": "CasA"
                                        },
                                        "maxItems": 2,
                                        "minItems": 0,
                                        "uniqueItems": true,
                                        "additionalItems": false
                                    },
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 10,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging time steps"
                                    },
                                    "ignore_target": {
                                        "type": "boolean",
                                        "title": "Ignore target",
                                        "default": false
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 64,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging frequency steps"
                                    }
                                },
                                "additionalProperties": false
                            },
                            "average": {
                                "type": "object",
                                "title": "Averaging",
                                "default": {},
                                "required": [
                                    "frequency_steps",
                                    "time_steps"
                                ],
                                "properties": {
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 1,
                                        "minimum": 1
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 4,
                                        "minimum": 1
                                    }
                                },
                                "additionalProperties": false
                            },
                            "storagemanager": {
                                "enum": [
                                    "standard",
                                    "dysco"
                                ],
                                "type": "string",
                                "title": "Storage Manager",
                                "default": "dysco"
                            },
                            "cluster_resources": {
                                "$ref": "#/definitions/cluster_resources",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for a preprocessing pipeline.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "cluster_resources": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Which cluster resources to claim for this pipeline.",
                                "properties": {
                                    "cores_per_task": {
                                        "default": 2,
                                        "description": "Number of cores to reserve for each process in the pipeline.",
                                        "maximum": 24,
                                        "minimum": 1,
                                        "title": "Cores/task",
                                        "type": "integer"
                                    },
                                    "parallel_tasks": {
                                        "default": 122,
                                        "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                        "maximum": 488,
                                        "minimum": 1,
                                        "title": "Max parallel tasks",
                                        "type": "integer"
                                    },
                                    "where": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "description": "Where to run this pipeline.",
                                        "properties": {
                                            "cluster": {
                                                "default": "CEP4",
                                                "enum": [
                                                    "CEP4"
                                                ],
                                                "title": "Cluster",
                                                "type": "string"
                                            },
                                            "partition": {
                                                "default": "cpu",
                                                "enum": [
                                                    "cpu",
                                                    "gpu",
                                                    "testing"
                                                ],
                                                "title": "Partition",
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "cluster",
                                            "partition"
                                        ],
                                        "title": "Where",
                                        "type": "object"
                                    }
                                },
                                "required": [
                                    "where",
                                    "cores_per_task",
                                    "parallel_tasks"
                                ],
                                "title": "Cluster resources",
                                "type": "object"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                        "type": "object",
                        "title": "preprocessing pipeline",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "average"
                        ],
                        "properties": {
                            "flag": {
                                "type": "object",
                                "title": "Flagging",
                                "default": {},
                                "required": [
                                    "outerchannels",
                                    "autocorrelations",
                                    "rfi_strategy"
                                ],
                                "properties": {
                                    "rfi_strategy": {
                                        "enum": [
                                            "none",
                                            "HBAdefault",
                                            "LBAdefault"
                                        ],
                                        "type": "string",
                                        "title": "RFI flagging strategy",
                                        "default": "HBAdefault"
                                    },
                                    "outerchannels": {
                                        "type": "boolean",
                                        "title": "Flag outer channels",
                                        "default": true
                                    },
                                    "autocorrelations": {
                                        "type": "boolean",
                                        "title": "Flag auto correlations",
                                        "default": true
                                    }
                                },
                                "additionalProperties": false
                            },
                            "demix": {
                                "type": "object",
                                "title": "Demixing",
                                "default": {},
                                "required": [],
                                "properties": {
                                    "sources": {
                                        "type": "array",
                                        "items": {
                                            "enum": [
                                                "CasA",
                                                "CygA",
                                                "HerA",
                                                "HydraA",
                                                "TauA",
                                                "VirA"
                                            ],
                                            "type": "string",
                                            "default": "CasA"
                                        },
                                        "maxItems": 2,
                                        "minItems": 0,
                                        "uniqueItems": true,
                                        "additionalItems": false
                                    },
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 10,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging time steps"
                                    },
                                    "ignore_target": {
                                        "type": "boolean",
                                        "title": "Ignore target",
                                        "default": false
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 64,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging frequency steps"
                                    }
                                },
                                "additionalProperties": false
                            },
                            "average": {
                                "type": "object",
                                "title": "Averaging",
                                "default": {},
                                "required": [
                                    "frequency_steps",
                                    "time_steps"
                                ],
                                "properties": {
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 1,
                                        "minimum": 1
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 4,
                                        "minimum": 1
                                    }
                                },
                                "additionalProperties": false
                            },
                            "storagemanager": {
                                "enum": [
                                    "standard",
                                    "dysco"
                                ],
                                "type": "string",
                                "title": "Storage Manager",
                                "default": "dysco"
                            },
                            "cluster_resources": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for a preprocessing pipeline.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/pipeline",
                    "type_value": "pipeline",
                    "updated_at": "2022-05-09T05:12:34.774747",
                    "version": 2
                },
                "specifications_template_id": 10,
                "status": "schedulable",
                "status_value": "schedulable",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000034"
                ],
                "subtasks_ids": [
                    2000034
                ],
                "tags": [],
                "task_type": "pipeline",
                "unschedulable_reason": "",
                "updated_at": "2022-05-09T05:13:05.624946"
            },
            {
                "id": 25,
                "url": "http://localhost:3000/api/task_blueprint/25",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_blueprint/35"
                ],
                "consumed_by_ids": [
                    35
                ],
                "created_at": "2022-05-09T05:13:05.534984",
                "description": "Calibrator Observation 2. After the Target Observation",
                "draft": "http://localhost:3000/api/task_draft/25",
                "draft_id": 25,
                "duration": 1200,
                "error_reason": "",
                "first_scheduling_relation": [
                    "http://localhost:3000/api/task_scheduling_relation_blueprint/2"
                ],
                "first_scheduling_relation_ids": [
                    2
                ],
                "name": "Calibrator Observation 2",
                "obsolete_since": null,
                "on_sky_duration": 600,
                "on_sky_start_time": "2022-05-09T14:20:49.932056",
                "on_sky_stop_time": "2022-05-09T14:30:49.932056",
                "output_pinned": false,
                "process_start_time": "2022-05-09T14:20:49.932056",
                "process_stop_time": "2022-05-09T14:40:49.932056",
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 29520,
                "relative_stop_time": 30120,
                "scheduled_start_time": "2022-05-09T14:20:49.932056",
                "scheduled_stop_time": "2022-05-09T14:40:49.932056",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
                "scheduling_unit_blueprint_id": 4,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Cal2 3Cdef",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2/ref_resolved",
                    "duration": 600,
                    "calibrator": {
                        "name": "calibrator1",
                        "pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "3Cdef",
                            "direction_type": "J2000"
                        },
                        "autoselect": false
                    }
                },
                "specifications_template": {
                    "id": 7,
                    "url": "http://localhost:3000/api/task_template/7",
                    "created_at": "2022-05-09T05:12:33.393061",
                    "description": "This schema defines the (extra) parameters to setup a calibrator observation task, which uses all parameters from the target observation task which it is linked to, plus these calibrator overrides.",
                    "name": "calibrator observation",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2/ref_resolved",
                        "title": "calibrator observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "calibrator",
                            "duration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "#/definitions/timedelta",
                                "title": "Duration",
                                "default": 600,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "calibrator": {
                                "$ref": "#/definitions/calibrator",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the (extra) parameters to setup a calibrator observation task, which uses all parameters from the target observation task which it is linked to, plus these calibrator overrides.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "timedelta": {
                                "default": 0,
                                "description": "A time duration or delta expressed in seconds",
                                "type": "number"
                            },
                            "timestamp": {
                                "default": "1970-01-01T00:00:00Z",
                                "description": "A timestamp defined in UTC",
                                "format": "date-time",
                                "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                                "type": "string"
                            },
                            "calibrator": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "autoselect": {
                                        "default": true,
                                        "description": "Auto-select calibrator based on elevation",
                                        "title": "Auto-select",
                                        "type": "boolean"
                                    },
                                    "name": {
                                        "default": "calibrator",
                                        "description": "Name of the calibrator SAP",
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "pointing": {
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "description": "Manually selected calibrator",
                                        "title": "Digital pointing"
                                    }
                                },
                                "required": [
                                    "autoselect",
                                    "pointing",
                                    "name"
                                ],
                                "type": "object"
                            },
                            "pointing": {
                                "additionalProperties": false,
                                "properties": {
                                    "angle1": {
                                        "default": 0.6624317181687094,
                                        "description": "First angle (e.g. RA)",
                                        "title": "Angle 1",
                                        "type": "number"
                                    },
                                    "angle2": {
                                        "default": 1.5579526427549426,
                                        "description": "Second angle (e.g. DEC)",
                                        "title": "Angle 2",
                                        "type": "number"
                                    },
                                    "direction_type": {
                                        "default": "J2000",
                                        "description": "",
                                        "enum": [
                                            "J2000",
                                            "AZELGEO",
                                            "LMN",
                                            "SUN",
                                            "MOON",
                                            "MERCURY",
                                            "VENUS",
                                            "MARS",
                                            "JUPITER",
                                            "SATURN",
                                            "URANUS",
                                            "NEPTUNE",
                                            "PLUTO"
                                        ],
                                        "title": "Reference frame",
                                        "type": "string"
                                    },
                                    "target": {
                                        "default": "_target_name_",
                                        "description": "Description of where this beam points at",
                                        "minLength": 1,
                                        "title": "Target",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "angle1",
                                    "angle2",
                                    "target"
                                ],
                                "type": "object"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2#",
                        "title": "calibrator observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "calibrator",
                            "duration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                "title": "Duration",
                                "default": 600,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "calibrator": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/calibrator/1#/definitions/calibrator",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the (extra) parameters to setup a calibrator observation task, which uses all parameters from the target observation task which it is linked to, plus these calibrator overrides.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/observation",
                    "type_value": "observation",
                    "updated_at": "2022-05-09T05:12:33.393095",
                    "version": 2
                },
                "specifications_template_id": 7,
                "status": "scheduled",
                "status_value": "scheduled",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000027",
                    "http://localhost:3000/api/subtask/2000028",
                    "http://localhost:3000/api/subtask/2000029"
                ],
                "subtasks_ids": [
                    2000027,
                    2000028,
                    2000029
                ],
                "tags": [],
                "task_type": "observation",
                "unschedulable_reason": "",
                "updated_at": "2022-05-09T05:13:05.535000"
            },
            {
                "id": 26,
                "url": "http://localhost:3000/api/task_blueprint/26",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_blueprint/39"
                ],
                "consumed_by_ids": [
                    39
                ],
                "created_at": "2022-05-09T05:13:05.575103",
                "description": "Preprocessing Pipeline for Calibrator Observation 2",
                "draft": "http://localhost:3000/api/task_draft/26",
                "draft_id": 26,
                "duration": 600,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Calibrator Pipeline 2",
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "output_pinned": false,
                "process_start_time": "2022-05-09T14:30:49.932056",
                "process_stop_time": "2022-05-09T14:40:49.932056",
                "produced_by": [
                    "http://localhost:3000/api/task_relation_blueprint/35"
                ],
                "produced_by_ids": [
                    35
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduled_start_time": "2022-05-09T14:30:49.932056",
                "scheduled_stop_time": "2022-05-09T14:40:49.932056",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
                "scheduling_unit_blueprint_id": 4,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Cal2 3Cdef/PP",
                "specifications_doc": {
                    "flag": {
                        "rfi_strategy": "HBAdefault",
                        "outerchannels": true,
                        "autocorrelations": true
                    },
                    "demix": {
                        "sources": [],
                        "time_steps": 10,
                        "ignore_target": false,
                        "frequency_steps": 64
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                    "average": {
                        "time_steps": 1,
                        "frequency_steps": 4
                    },
                    "storagemanager": "dysco",
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    }
                },
                "specifications_template": {
                    "id": 10,
                    "url": "http://localhost:3000/api/task_template/10",
                    "created_at": "2022-05-09T05:12:34.774684",
                    "description": "This schema defines the parameters for a preprocessing pipeline.",
                    "name": "preprocessing pipeline",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2/ref_resolved",
                        "type": "object",
                        "title": "preprocessing pipeline",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "average"
                        ],
                        "properties": {
                            "flag": {
                                "type": "object",
                                "title": "Flagging",
                                "default": {},
                                "required": [
                                    "outerchannels",
                                    "autocorrelations",
                                    "rfi_strategy"
                                ],
                                "properties": {
                                    "rfi_strategy": {
                                        "enum": [
                                            "none",
                                            "HBAdefault",
                                            "LBAdefault"
                                        ],
                                        "type": "string",
                                        "title": "RFI flagging strategy",
                                        "default": "HBAdefault"
                                    },
                                    "outerchannels": {
                                        "type": "boolean",
                                        "title": "Flag outer channels",
                                        "default": true
                                    },
                                    "autocorrelations": {
                                        "type": "boolean",
                                        "title": "Flag auto correlations",
                                        "default": true
                                    }
                                },
                                "additionalProperties": false
                            },
                            "demix": {
                                "type": "object",
                                "title": "Demixing",
                                "default": {},
                                "required": [],
                                "properties": {
                                    "sources": {
                                        "type": "array",
                                        "items": {
                                            "enum": [
                                                "CasA",
                                                "CygA",
                                                "HerA",
                                                "HydraA",
                                                "TauA",
                                                "VirA"
                                            ],
                                            "type": "string",
                                            "default": "CasA"
                                        },
                                        "maxItems": 2,
                                        "minItems": 0,
                                        "uniqueItems": true,
                                        "additionalItems": false
                                    },
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 10,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging time steps"
                                    },
                                    "ignore_target": {
                                        "type": "boolean",
                                        "title": "Ignore target",
                                        "default": false
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 64,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging frequency steps"
                                    }
                                },
                                "additionalProperties": false
                            },
                            "average": {
                                "type": "object",
                                "title": "Averaging",
                                "default": {},
                                "required": [
                                    "frequency_steps",
                                    "time_steps"
                                ],
                                "properties": {
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 1,
                                        "minimum": 1
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 4,
                                        "minimum": 1
                                    }
                                },
                                "additionalProperties": false
                            },
                            "storagemanager": {
                                "enum": [
                                    "standard",
                                    "dysco"
                                ],
                                "type": "string",
                                "title": "Storage Manager",
                                "default": "dysco"
                            },
                            "cluster_resources": {
                                "$ref": "#/definitions/cluster_resources",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for a preprocessing pipeline.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "cluster_resources": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Which cluster resources to claim for this pipeline.",
                                "properties": {
                                    "cores_per_task": {
                                        "default": 2,
                                        "description": "Number of cores to reserve for each process in the pipeline.",
                                        "maximum": 24,
                                        "minimum": 1,
                                        "title": "Cores/task",
                                        "type": "integer"
                                    },
                                    "parallel_tasks": {
                                        "default": 122,
                                        "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                        "maximum": 488,
                                        "minimum": 1,
                                        "title": "Max parallel tasks",
                                        "type": "integer"
                                    },
                                    "where": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "description": "Where to run this pipeline.",
                                        "properties": {
                                            "cluster": {
                                                "default": "CEP4",
                                                "enum": [
                                                    "CEP4"
                                                ],
                                                "title": "Cluster",
                                                "type": "string"
                                            },
                                            "partition": {
                                                "default": "cpu",
                                                "enum": [
                                                    "cpu",
                                                    "gpu",
                                                    "testing"
                                                ],
                                                "title": "Partition",
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "cluster",
                                            "partition"
                                        ],
                                        "title": "Where",
                                        "type": "object"
                                    }
                                },
                                "required": [
                                    "where",
                                    "cores_per_task",
                                    "parallel_tasks"
                                ],
                                "title": "Cluster resources",
                                "type": "object"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/preprocessing%20pipeline/2#",
                        "type": "object",
                        "title": "preprocessing pipeline",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "average"
                        ],
                        "properties": {
                            "flag": {
                                "type": "object",
                                "title": "Flagging",
                                "default": {},
                                "required": [
                                    "outerchannels",
                                    "autocorrelations",
                                    "rfi_strategy"
                                ],
                                "properties": {
                                    "rfi_strategy": {
                                        "enum": [
                                            "none",
                                            "HBAdefault",
                                            "LBAdefault"
                                        ],
                                        "type": "string",
                                        "title": "RFI flagging strategy",
                                        "default": "HBAdefault"
                                    },
                                    "outerchannels": {
                                        "type": "boolean",
                                        "title": "Flag outer channels",
                                        "default": true
                                    },
                                    "autocorrelations": {
                                        "type": "boolean",
                                        "title": "Flag auto correlations",
                                        "default": true
                                    }
                                },
                                "additionalProperties": false
                            },
                            "demix": {
                                "type": "object",
                                "title": "Demixing",
                                "default": {},
                                "required": [],
                                "properties": {
                                    "sources": {
                                        "type": "array",
                                        "items": {
                                            "enum": [
                                                "CasA",
                                                "CygA",
                                                "HerA",
                                                "HydraA",
                                                "TauA",
                                                "VirA"
                                            ],
                                            "type": "string",
                                            "default": "CasA"
                                        },
                                        "maxItems": 2,
                                        "minItems": 0,
                                        "uniqueItems": true,
                                        "additionalItems": false
                                    },
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 10,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging time steps"
                                    },
                                    "ignore_target": {
                                        "type": "boolean",
                                        "title": "Ignore target",
                                        "default": false
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 64,
                                        "minimum": 1,
                                        "description": "Must be a multiple of the averaging frequency steps"
                                    }
                                },
                                "additionalProperties": false
                            },
                            "average": {
                                "type": "object",
                                "title": "Averaging",
                                "default": {},
                                "required": [
                                    "frequency_steps",
                                    "time_steps"
                                ],
                                "properties": {
                                    "time_steps": {
                                        "type": "integer",
                                        "title": "Time steps",
                                        "default": 1,
                                        "minimum": 1
                                    },
                                    "frequency_steps": {
                                        "type": "integer",
                                        "title": "Frequency steps",
                                        "default": 4,
                                        "minimum": 1
                                    }
                                },
                                "additionalProperties": false
                            },
                            "storagemanager": {
                                "enum": [
                                    "standard",
                                    "dysco"
                                ],
                                "type": "string",
                                "title": "Storage Manager",
                                "default": "dysco"
                            },
                            "cluster_resources": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for a preprocessing pipeline.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/pipeline",
                    "type_value": "pipeline",
                    "updated_at": "2022-05-09T05:12:34.774747",
                    "version": 2
                },
                "specifications_template_id": 10,
                "status": "schedulable",
                "status_value": "schedulable",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000030"
                ],
                "subtasks_ids": [
                    2000030
                ],
                "tags": [],
                "task_type": "pipeline",
                "unschedulable_reason": "",
                "updated_at": "2022-05-09T05:13:05.575118"
            },
            {
                "id": 29,
                "url": "http://localhost:3000/api/task_blueprint/29",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-05-09T05:13:05.717903",
                "description": "Ingest all preprocessed dataproducts",
                "draft": "http://localhost:3000/api/task_draft/29",
                "draft_id": 29,
                "duration": 300,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Ingest",
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "output_pinned": false,
                "process_start_time": "2022-05-09T22:19:49.932056",
                "process_stop_time": "2022-05-09T22:24:49.932056",
                "produced_by": [
                    "http://localhost:3000/api/task_relation_blueprint/38",
                    "http://localhost:3000/api/task_relation_blueprint/39",
                    "http://localhost:3000/api/task_relation_blueprint/40",
                    "http://localhost:3000/api/task_relation_blueprint/41"
                ],
                "produced_by_ids": [
                    38,
                    39,
                    40,
                    41
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduled_start_time": "2022-05-09T22:19:49.932056",
                "scheduled_stop_time": "2022-05-09T22:24:49.932056",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
                "scheduling_unit_blueprint_id": 4,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
                },
                "specifications_template": {
                    "id": 2,
                    "url": "http://localhost:3000/api/task_template/2",
                    "created_at": "2022-05-09T05:12:30.954249",
                    "description": "This schema defines the parameters to setup an ingest task.",
                    "name": "ingest",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved",
                        "type": "object",
                        "title": "ingest",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [],
                        "properties": {},
                        "description": "This schema defines the parameters to setup an ingest task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {}
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2#",
                        "type": "object",
                        "title": "ingest",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [],
                        "properties": {},
                        "description": "This schema defines the parameters to setup an ingest task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/ingest",
                    "type_value": "ingest",
                    "updated_at": "2022-05-09T05:12:30.954327",
                    "version": 2
                },
                "specifications_template_id": 2,
                "status": "schedulable",
                "status_value": "schedulable",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000040"
                ],
                "subtasks_ids": [
                    2000040
                ],
                "tags": [],
                "task_type": "ingest",
                "unschedulable_reason": "",
                "updated_at": "2022-05-09T05:13:05.717918"
            },
            {
                "id": 22,
                "url": "http://localhost:3000/api/task_blueprint/22",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_blueprint/34"
                ],
                "consumed_by_ids": [
                    34
                ],
                "created_at": "2022-05-09T05:13:05.319201",
                "description": "Calibrator Observation 1. Before the Target Observation",
                "draft": "http://localhost:3000/api/task_draft/22",
                "draft_id": 22,
                "duration": 1200,
                "error_reason": "",
                "first_scheduling_relation": [
                    "http://localhost:3000/api/task_scheduling_relation_blueprint/1"
                ],
                "first_scheduling_relation_ids": [
                    1
                ],
                "name": "Calibrator Observation 1",
                "obsolete_since": null,
                "on_sky_duration": 600,
                "on_sky_start_time": "2022-05-09T06:08:49.932056",
                "on_sky_stop_time": "2022-05-09T06:18:49.932056",
                "output_pinned": false,
                "process_start_time": "2022-05-09T06:08:49.932056",
                "process_stop_time": "2022-05-09T06:28:49.932056",
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 0,
                "relative_stop_time": 600,
                "scheduled_start_time": "2022-05-09T06:08:49.932056",
                "scheduled_stop_time": "2022-05-09T06:28:49.932056",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/4",
                "scheduling_unit_blueprint_id": 4,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "Cal1 3Cabc",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2/ref_resolved",
                    "duration": 600,
                    "calibrator": {
                        "name": "calibrator1",
                        "pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "3Cabc",
                            "direction_type": "J2000"
                        },
                        "autoselect": false
                    }
                },
                "specifications_template": {
                    "id": 7,
                    "url": "http://localhost:3000/api/task_template/7",
                    "created_at": "2022-05-09T05:12:33.393061",
                    "description": "This schema defines the (extra) parameters to setup a calibrator observation task, which uses all parameters from the target observation task which it is linked to, plus these calibrator overrides.",
                    "name": "calibrator observation",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2/ref_resolved",
                        "title": "calibrator observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "calibrator",
                            "duration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "#/definitions/timedelta",
                                "title": "Duration",
                                "default": 600,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "calibrator": {
                                "$ref": "#/definitions/calibrator",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the (extra) parameters to setup a calibrator observation task, which uses all parameters from the target observation task which it is linked to, plus these calibrator overrides.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "timedelta": {
                                "default": 0,
                                "description": "A time duration or delta expressed in seconds",
                                "type": "number"
                            },
                            "timestamp": {
                                "default": "1970-01-01T00:00:00Z",
                                "description": "A timestamp defined in UTC",
                                "format": "date-time",
                                "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                                "type": "string"
                            },
                            "calibrator": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "autoselect": {
                                        "default": true,
                                        "description": "Auto-select calibrator based on elevation",
                                        "title": "Auto-select",
                                        "type": "boolean"
                                    },
                                    "name": {
                                        "default": "calibrator",
                                        "description": "Name of the calibrator SAP",
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "pointing": {
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "description": "Manually selected calibrator",
                                        "title": "Digital pointing"
                                    }
                                },
                                "required": [
                                    "autoselect",
                                    "pointing",
                                    "name"
                                ],
                                "type": "object"
                            },
                            "pointing": {
                                "additionalProperties": false,
                                "properties": {
                                    "angle1": {
                                        "default": 0.6624317181687094,
                                        "description": "First angle (e.g. RA)",
                                        "title": "Angle 1",
                                        "type": "number"
                                    },
                                    "angle2": {
                                        "default": 1.5579526427549426,
                                        "description": "Second angle (e.g. DEC)",
                                        "title": "Angle 2",
                                        "type": "number"
                                    },
                                    "direction_type": {
                                        "default": "J2000",
                                        "description": "",
                                        "enum": [
                                            "J2000",
                                            "AZELGEO",
                                            "LMN",
                                            "SUN",
                                            "MOON",
                                            "MERCURY",
                                            "VENUS",
                                            "MARS",
                                            "JUPITER",
                                            "SATURN",
                                            "URANUS",
                                            "NEPTUNE",
                                            "PLUTO"
                                        ],
                                        "title": "Reference frame",
                                        "type": "string"
                                    },
                                    "target": {
                                        "default": "_target_name_",
                                        "description": "Description of where this beam points at",
                                        "minLength": 1,
                                        "title": "Target",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "angle1",
                                    "angle2",
                                    "target"
                                ],
                                "type": "object"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/calibrator%20observation/2#",
                        "title": "calibrator observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "calibrator",
                            "duration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                "title": "Duration",
                                "default": 600,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "calibrator": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/calibrator/1#/definitions/calibrator",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the (extra) parameters to setup a calibrator observation task, which uses all parameters from the target observation task which it is linked to, plus these calibrator overrides.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/observation",
                    "type_value": "observation",
                    "updated_at": "2022-05-09T05:12:33.393095",
                    "version": 2
                },
                "specifications_template_id": 7,
                "status": "scheduled",
                "status_value": "scheduled",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000037",
                    "http://localhost:3000/api/subtask/2000035",
                    "http://localhost:3000/api/subtask/2000036"
                ],
                "subtasks_ids": [
                    2000037,
                    2000035,
                    2000036
                ],
                "tags": [],
                "task_type": "observation",
                "unschedulable_reason": "",
                "updated_at": "2022-05-09T05:13:05.319244"
            }
        ],
        "task_blueprints_ids": [
            23,
            24,
            28,
            27,
            25,
            26,
            29,
            22
        ],
        "unschedulable_reason": "",
        "updated_at": "2022-05-09T05:13:05.262996"
    },
    {
        "id": 9,
        "url": "http://localhost:3000/api/scheduling_unit_blueprint/9",
        "actual_on_sky_start_time": null,
        "actual_on_sky_stop_time": null,
        "actual_process_start_time": null,
        "actual_process_stop_time": null,
        "created_at": "2022-04-21T08:16:31.086555",
        "description": "Test scheduling unit",
        "draft": "http://localhost:3000/api/scheduling_unit_draft/9",
        "draft_id": 9,
        "duration": 840,
        "error_reason": "",
        "global_identifier": "http://localhost:3000/api/sip_identifier/145",
        "global_identifier_id": 145,
        "ingest_permission_granted_since": null,
        "ingest_permission_required": true,
        "interrupts_telescope": false,
        "name": "BF Pulsar Timing",
        "observed_duration": null,
        "observed_start_time": null,
        "observed_stop_time": null,
        "obsolete_since": null,
        "on_sky_duration": 120,
        "on_sky_start_time": "2022-04-22T02:52:06.166227",
        "on_sky_stop_time": "2022-04-22T02:54:06.166227",
        "output_pinned": false,
        "piggyback_allowed_aartfaac": true,
        "piggyback_allowed_tbb": true,
        "priority_queue": "http://localhost:3000/api/priority_queue_type/A",
        "priority_queue_value": "A",
        "process_start_time": "2022-04-22T02:52:06.166227",
        "process_stop_time": "2022-04-22T03:01:06.166227",
        "rank": 1,
        "scheduled_start_time": "2022-04-22T02:52:06.166227",
        "scheduled_stop_time": "2022-04-22T03:01:06.166227",
        "scheduling_constraints_doc": {
            "sky": {
                "min_distance": {
                    "sun": 0.5,
                    "moon": 0.5,
                    "jupiter": 0.5
                },
                "min_elevation": {
                    "target": 0.5,
                    "calibrator": 0.5
                },
                "transit_offset": {
                    "to": 7200,
                    "from": -7200
                }
            },
            "time": {
                "at": "2022-04-22T02:52:06.166227"
            },
            "daily": {
                "require_day": false,
                "require_night": false,
                "avoid_twilight": false
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
            "scheduler": "fixed_time"
        },
        "scheduling_constraints_template": "http://localhost:3000/api/scheduling_constraints_template/1",
        "scheduling_constraints_template_id": 1,
        "specifications_template": "http://localhost:3000/api/scheduling_unit_template/1",
        "specifications_template_id": 1,
        "status": "scheduled",
        "status_value": "scheduled",
        "tags": [],
        "task_blueprints": [
            {
                "id": 50,
                "url": "http://localhost:3000/api/task_blueprint/50",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_blueprint/68",
                    "http://localhost:3000/api/task_relation_blueprint/69",
                    "http://localhost:3000/api/task_relation_blueprint/70",
                    "http://localhost:3000/api/task_relation_blueprint/71"
                ],
                "consumed_by_ids": [
                    68,
                    69,
                    70,
                    71
                ],
                "created_at": "2022-04-21T08:16:31.195314",
                "description": "Pulsar Pipeline for default HBA pulsar timing observation",
                "draft": "http://localhost:3000/api/task_draft/50",
                "draft_id": 50,
                "duration": 120,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Pipeline",
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "output_pinned": false,
                "process_start_time": "2022-04-22T02:54:06.166227",
                "process_stop_time": "2022-04-22T02:56:06.166227",
                "produced_by": [
                    "http://localhost:3000/api/task_relation_blueprint/66"
                ],
                "produced_by_ids": [
                    66
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduled_start_time": "2022-04-22T02:54:06.166227",
                "scheduled_stop_time": "2022-04-22T02:56:06.166227",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/9",
                "scheduling_unit_blueprint_id": 9,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "dspsr": {
                        "digifil": {
                            "dm": 0,
                            "channels_per_part": 320,
                            "coherent_dedispersion": true,
                            "integration_time_factor": 4
                        },
                        "enabled": true,
                        "filterbank": {
                            "enabled": false,
                            "channels_per_part": 120,
                            "coherent_dedispersion": true
                        },
                        "rfi_excision": true,
                        "optimise_period_dm": true,
                        "subintegration_length": -1,
                        "single_pulse_subintegration": false
                    },
                    "output": {
                        "quantisation": {
                            "scale": 5,
                            "enabled": false
                        },
                        "dynamic_spectrum": {
                            "enabled": false,
                            "time_average": 0.5
                        }
                    },
                    "presto": {
                        "input": {
                            "nr_blocks": 100,
                            "decode_sigma": 3,
                            "samples_per_block": 8192
                        },
                        "rrats": {
                            "enabled": false,
                            "dm_range": 5
                        },
                        "rfifind": {
                            "blocks": 16
                        },
                        "prepdata": {
                            "dm": -1
                        },
                        "prepfold": false,
                        "fold_profile": true
                    },
                    "pulsar": {
                        "name": "",
                        "strategy": "meta"
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/pulsar%20pipeline/2/ref_resolved",
                    "cluster_resources": {
                        "where": {
                            "cluster": "CEP4",
                            "partition": "cpu"
                        },
                        "cores_per_task": 2,
                        "parallel_tasks": 122
                    },
                    "single_pulse_search": false
                },
                "specifications_template": {
                    "id": 9,
                    "url": "http://localhost:3000/api/task_template/9",
                    "created_at": "2022-04-21T08:15:53.345870",
                    "description": "This schema defines the parameters for a pulsar pipeline.",
                    "name": "pulsar pipeline",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/pulsar%20pipeline/2/ref_resolved",
                        "type": "object",
                        "title": "pulsar pipeline",
                        "$schema": "http://json-schema.org/draft-07/schema#",
                        "version": 2,
                        "required": [
                            "pulsar",
                            "presto",
                            "dspsr",
                            "output"
                        ],
                        "properties": {
                            "dspsr": {
                                "type": "object",
                                "title": "DSPSR",
                                "default": {},
                                "properties": {
                                    "digifil": {
                                        "type": "object",
                                        "title": "Digifil",
                                        "default": {},
                                        "properties": {
                                            "dm": {
                                                "type": "number",
                                                "title": "DM",
                                                "default": -1,
                                                "minimum": -1,
                                                "desciption": "Dispersion Measure (-1 for none)"
                                            },
                                            "integration_time": {
                                                "type": "number",
                                                "title": "Integration time",
                                                "default": 4,
                                                "minimum": 0.1
                                            },
                                            "frequency_channels": {
                                                "type": "integer",
                                                "title": "Frequency channels",
                                                "default": 512,
                                                "maximum": 512,
                                                "minimum": 1,
                                                "description": "Number of frequency channels (multiple of subbands/part)"
                                            },
                                            "coherent_dedispersion": {
                                                "type": "boolean",
                                                "title": "Coherent Dedispersion",
                                                "default": true
                                            }
                                        },
                                        "description": "Processes single pulses in CV data if single-pulse search is enabled",
                                        "additionalProperties": false
                                    },
                                    "enabled": {
                                        "type": "boolean",
                                        "title": "Enabled",
                                        "default": true
                                    },
                                    "filterbank": {
                                        "type": "object",
                                        "title": "Create filter bank",
                                        "default": {},
                                        "properties": {
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            },
                                            "frequency_channels": {
                                                "type": "integer",
                                                "title": "Frequency channels",
                                                "default": 120,
                                                "maximum": 512,
                                                "minimum": 1,
                                                "description": "Number of frequency channels (multiple of subbands/part)"
                                            },
                                            "coherent_dedispersion": {
                                                "type": "boolean",
                                                "title": "Coherent Dedispersion",
                                                "default": true
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "rfi_excision": {
                                        "type": "boolean",
                                        "title": "RFI excision",
                                        "default": true,
                                        "description": "Excise/clean/remove detected RFI"
                                    },
                                    "optimise_period_dm": {
                                        "type": "boolean",
                                        "title": "Optimise period & DM",
                                        "default": true
                                    },
                                    "subintegration_length": {
                                        "type": "integer",
                                        "title": "Subintegration length",
                                        "default": -1,
                                        "minimum": -1
                                    },
                                    "single_pulse_subintegration": {
                                        "type": "boolean",
                                        "title": "Single pulse subintegration",
                                        "default": false,
                                        "description": "Create single-pulse subintegrations with inter-channel dispersion delays removed."
                                    }
                                },
                                "additionalProperties": false
                            },
                            "output": {
                                "type": "object",
                                "title": "Output",
                                "default": {},
                                "properties": {
                                    "quantisation": {
                                        "type": "object",
                                        "title": "Quantisation",
                                        "default": {},
                                        "properties": {
                                            "scale": {
                                                "type": "number",
                                                "title": "Conversion sigma",
                                                "default": 5,
                                                "minimum": 1,
                                                "description": "Conversion sigma to use when converting to 8-bit samples"
                                            },
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            }
                                        },
                                        "description": "Quantise output into 8-bit samples",
                                        "additionalProperties": false
                                    },
                                    "dynamic_spectrum": {
                                        "type": "object",
                                        "title": "Dynamic Spectrum",
                                        "default": {},
                                        "properties": {
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            },
                                            "time_average": {
                                                "type": "number",
                                                "title": "Time average",
                                                "default": 0.5,
                                                "minimum": 0.01
                                            }
                                        },
                                        "additionalProperties": false
                                    }
                                },
                                "additionalProperties": false
                            },
                            "presto": {
                                "type": "object",
                                "title": "PRESTO",
                                "default": {},
                                "properties": {
                                    "input": {
                                        "type": "object",
                                        "title": "Input",
                                        "properties": {
                                            "nr_blocks": {
                                                "type": "integer",
                                                "title": "Nr of blocks",
                                                "default": 100,
                                                "minimum": 1,
                                                "description": "Number of blocks to read at a time"
                                            },
                                            "decode_sigma": {
                                                "type": "number",
                                                "title": "Decode sigma",
                                                "default": 3,
                                                "minimum": 1,
                                                "description": "Sigma threshold for decoding"
                                            },
                                            "samples_per_block": {
                                                "type": "integer",
                                                "title": "Block size (samples)",
                                                "default": 8192,
                                                "minimum": 16
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "rrats": {
                                        "type": "object",
                                        "title": "RRATs analysis",
                                        "default": {},
                                        "properties": {
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            },
                                            "dm_range": {
                                                "type": "number",
                                                "title": "DM range",
                                                "default": 5,
                                                "minimum": 0
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "rfifind": {
                                        "type": "object",
                                        "title": "rfifind",
                                        "default": {},
                                        "properties": {
                                            "blocks": {
                                                "type": "integer",
                                                "title": "blocks",
                                                "default": 16,
                                                "minimum": 1
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "prepdata": {
                                        "type": "object",
                                        "title": "prepdata",
                                        "default": {},
                                        "properties": {
                                            "dm": {
                                                "type": "number",
                                                "title": "DM",
                                                "default": -1,
                                                "minimum": -1,
                                                "desciption": "Dispersion Measure (-1 for none)"
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "prepfold": {
                                        "type": "boolean",
                                        "title": "Enable prepfold",
                                        "default": true
                                    },
                                    "fold_profile": {
                                        "type": "boolean",
                                        "title": "Fold",
                                        "default": true,
                                        "description": "Fold the pulsar profile"
                                    }
                                },
                                "additionalProperties": false
                            },
                            "pulsar": {
                                "type": "object",
                                "title": "Pulsar to fold",
                                "default": {},
                                "properties": {
                                    "name": {
                                        "type": "string",
                                        "title": "Name",
                                        "default": "",
                                        "description": "Name of the pulsar to fold, if strategy=manual"
                                    },
                                    "strategy": {
                                        "enum": [
                                            "manual",
                                            "meta",
                                            "sapfind",
                                            "sapfind3",
                                            "tabfind",
                                            "tabfind+"
                                        ],
                                        "type": "string",
                                        "title": "Strategy",
                                        "default": "manual",
                                        "description": "How to look up the pulsar to fold",
                                        "enum_titles": [
                                            "Pulsar name specified here",
                                            "SAP target is pulsar name",
                                            "Brightest known pulsar in SAP",
                                            "Brightest 3 known pulsars in SAP",
                                            "Brightest known pulsar in TAB",
                                            "Brightest 3 known pulsars in TAB"
                                        ]
                                    }
                                },
                                "additionalProperties": false
                            },
                            "cluster_resources": {
                                "$ref": "#/definitions/cluster_resources",
                                "default": {}
                            },
                            "single_pulse_search": {
                                "type": "boolean",
                                "title": "Single-pulse search",
                                "default": false,
                                "description": "Instructs PRESTO to process single pulses, and enables Digifil for CV data"
                            }
                        },
                        "description": "This schema defines the parameters for a pulsar pipeline.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "cluster_resources": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Which cluster resources to claim for this pipeline.",
                                "properties": {
                                    "cores_per_task": {
                                        "default": 2,
                                        "description": "Number of cores to reserve for each process in the pipeline.",
                                        "maximum": 24,
                                        "minimum": 1,
                                        "title": "Cores/task",
                                        "type": "integer"
                                    },
                                    "parallel_tasks": {
                                        "default": 122,
                                        "description": "Maximum number of tasks to run in parallel. Determines the size of the reservation. A smaller reservation results in a longer pipeline run. A bigger reservation results in more in resource waste.",
                                        "maximum": 488,
                                        "minimum": 1,
                                        "title": "Max parallel tasks",
                                        "type": "integer"
                                    },
                                    "where": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "description": "Where to run this pipeline.",
                                        "properties": {
                                            "cluster": {
                                                "default": "CEP4",
                                                "enum": [
                                                    "CEP4"
                                                ],
                                                "title": "Cluster",
                                                "type": "string"
                                            },
                                            "partition": {
                                                "default": "cpu",
                                                "enum": [
                                                    "cpu",
                                                    "gpu",
                                                    "testing"
                                                ],
                                                "title": "Partition",
                                                "type": "string"
                                            }
                                        },
                                        "required": [
                                            "cluster",
                                            "partition"
                                        ],
                                        "title": "Where",
                                        "type": "object"
                                    }
                                },
                                "required": [
                                    "where",
                                    "cores_per_task",
                                    "parallel_tasks"
                                ],
                                "title": "Cluster resources",
                                "type": "object"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/pulsar%20pipeline/2#",
                        "type": "object",
                        "title": "pulsar pipeline",
                        "$schema": "http://json-schema.org/draft-07/schema#",
                        "version": 2,
                        "required": [
                            "pulsar",
                            "presto",
                            "dspsr",
                            "output"
                        ],
                        "properties": {
                            "dspsr": {
                                "type": "object",
                                "title": "DSPSR",
                                "default": {},
                                "properties": {
                                    "digifil": {
                                        "type": "object",
                                        "title": "Digifil",
                                        "default": {},
                                        "properties": {
                                            "dm": {
                                                "type": "number",
                                                "title": "DM",
                                                "default": -1,
                                                "minimum": -1,
                                                "desciption": "Dispersion Measure (-1 for none)"
                                            },
                                            "integration_time": {
                                                "type": "number",
                                                "title": "Integration time",
                                                "default": 4,
                                                "minimum": 0.1
                                            },
                                            "frequency_channels": {
                                                "type": "integer",
                                                "title": "Frequency channels",
                                                "default": 512,
                                                "maximum": 512,
                                                "minimum": 1,
                                                "description": "Number of frequency channels (multiple of subbands/part)"
                                            },
                                            "coherent_dedispersion": {
                                                "type": "boolean",
                                                "title": "Coherent Dedispersion",
                                                "default": true
                                            }
                                        },
                                        "description": "Processes single pulses in CV data if single-pulse search is enabled",
                                        "additionalProperties": false
                                    },
                                    "enabled": {
                                        "type": "boolean",
                                        "title": "Enabled",
                                        "default": true
                                    },
                                    "filterbank": {
                                        "type": "object",
                                        "title": "Create filter bank",
                                        "default": {},
                                        "properties": {
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            },
                                            "frequency_channels": {
                                                "type": "integer",
                                                "title": "Frequency channels",
                                                "default": 120,
                                                "maximum": 512,
                                                "minimum": 1,
                                                "description": "Number of frequency channels (multiple of subbands/part)"
                                            },
                                            "coherent_dedispersion": {
                                                "type": "boolean",
                                                "title": "Coherent Dedispersion",
                                                "default": true
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "rfi_excision": {
                                        "type": "boolean",
                                        "title": "RFI excision",
                                        "default": true,
                                        "description": "Excise/clean/remove detected RFI"
                                    },
                                    "optimise_period_dm": {
                                        "type": "boolean",
                                        "title": "Optimise period & DM",
                                        "default": true
                                    },
                                    "subintegration_length": {
                                        "type": "integer",
                                        "title": "Subintegration length",
                                        "default": -1,
                                        "minimum": -1
                                    },
                                    "single_pulse_subintegration": {
                                        "type": "boolean",
                                        "title": "Single pulse subintegration",
                                        "default": false,
                                        "description": "Create single-pulse subintegrations with inter-channel dispersion delays removed."
                                    }
                                },
                                "additionalProperties": false
                            },
                            "output": {
                                "type": "object",
                                "title": "Output",
                                "default": {},
                                "properties": {
                                    "quantisation": {
                                        "type": "object",
                                        "title": "Quantisation",
                                        "default": {},
                                        "properties": {
                                            "scale": {
                                                "type": "number",
                                                "title": "Conversion sigma",
                                                "default": 5,
                                                "minimum": 1,
                                                "description": "Conversion sigma to use when converting to 8-bit samples"
                                            },
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            }
                                        },
                                        "description": "Quantise output into 8-bit samples",
                                        "additionalProperties": false
                                    },
                                    "dynamic_spectrum": {
                                        "type": "object",
                                        "title": "Dynamic Spectrum",
                                        "default": {},
                                        "properties": {
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            },
                                            "time_average": {
                                                "type": "number",
                                                "title": "Time average",
                                                "default": 0.5,
                                                "minimum": 0.01
                                            }
                                        },
                                        "additionalProperties": false
                                    }
                                },
                                "additionalProperties": false
                            },
                            "presto": {
                                "type": "object",
                                "title": "PRESTO",
                                "default": {},
                                "properties": {
                                    "input": {
                                        "type": "object",
                                        "title": "Input",
                                        "properties": {
                                            "nr_blocks": {
                                                "type": "integer",
                                                "title": "Nr of blocks",
                                                "default": 100,
                                                "minimum": 1,
                                                "description": "Number of blocks to read at a time"
                                            },
                                            "decode_sigma": {
                                                "type": "number",
                                                "title": "Decode sigma",
                                                "default": 3,
                                                "minimum": 1,
                                                "description": "Sigma threshold for decoding"
                                            },
                                            "samples_per_block": {
                                                "type": "integer",
                                                "title": "Block size (samples)",
                                                "default": 8192,
                                                "minimum": 16
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "rrats": {
                                        "type": "object",
                                        "title": "RRATs analysis",
                                        "default": {},
                                        "properties": {
                                            "enabled": {
                                                "type": "boolean",
                                                "title": "Enabled",
                                                "default": false
                                            },
                                            "dm_range": {
                                                "type": "number",
                                                "title": "DM range",
                                                "default": 5,
                                                "minimum": 0
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "rfifind": {
                                        "type": "object",
                                        "title": "rfifind",
                                        "default": {},
                                        "properties": {
                                            "blocks": {
                                                "type": "integer",
                                                "title": "blocks",
                                                "default": 16,
                                                "minimum": 1
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "prepdata": {
                                        "type": "object",
                                        "title": "prepdata",
                                        "default": {},
                                        "properties": {
                                            "dm": {
                                                "type": "number",
                                                "title": "DM",
                                                "default": -1,
                                                "minimum": -1,
                                                "desciption": "Dispersion Measure (-1 for none)"
                                            }
                                        },
                                        "additionalProperties": false
                                    },
                                    "prepfold": {
                                        "type": "boolean",
                                        "title": "Enable prepfold",
                                        "default": true
                                    },
                                    "fold_profile": {
                                        "type": "boolean",
                                        "title": "Fold",
                                        "default": true,
                                        "description": "Fold the pulsar profile"
                                    }
                                },
                                "additionalProperties": false
                            },
                            "pulsar": {
                                "type": "object",
                                "title": "Pulsar to fold",
                                "default": {},
                                "properties": {
                                    "name": {
                                        "type": "string",
                                        "title": "Name",
                                        "default": "",
                                        "description": "Name of the pulsar to fold, if strategy=manual"
                                    },
                                    "strategy": {
                                        "enum": [
                                            "manual",
                                            "meta",
                                            "sapfind",
                                            "sapfind3",
                                            "tabfind",
                                            "tabfind+"
                                        ],
                                        "type": "string",
                                        "title": "Strategy",
                                        "default": "manual",
                                        "description": "How to look up the pulsar to fold",
                                        "enum_titles": [
                                            "Pulsar name specified here",
                                            "SAP target is pulsar name",
                                            "Brightest known pulsar in SAP",
                                            "Brightest 3 known pulsars in SAP",
                                            "Brightest known pulsar in TAB",
                                            "Brightest 3 known pulsars in TAB"
                                        ]
                                    }
                                },
                                "additionalProperties": false
                            },
                            "cluster_resources": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/pipeline/2#/definitions/cluster_resources",
                                "default": {}
                            },
                            "single_pulse_search": {
                                "type": "boolean",
                                "title": "Single-pulse search",
                                "default": false,
                                "description": "Instructs PRESTO to process single pulses, and enables Digifil for CV data"
                            }
                        },
                        "description": "This schema defines the parameters for a pulsar pipeline.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/pipeline",
                    "type_value": "pipeline",
                    "updated_at": "2022-04-21T08:15:53.345958",
                    "version": 2
                },
                "specifications_template_id": 9,
                "status": "schedulable",
                "status_value": "schedulable",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000069"
                ],
                "subtasks_ids": [
                    2000069
                ],
                "tags": [],
                "task_type": "pipeline",
                "unschedulable_reason": "",
                "updated_at": "2022-04-21T08:16:31.195328"
            },
            {
                "id": 49,
                "url": "http://localhost:3000/api/task_blueprint/49",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [
                    "http://localhost:3000/api/task_relation_blueprint/66",
                    "http://localhost:3000/api/task_relation_blueprint/67"
                ],
                "consumed_by_ids": [
                    66,
                    67
                ],
                "created_at": "2022-04-21T08:16:31.148213",
                "description": "Beamforming observation for default pulsar timing with HBA",
                "draft": "http://localhost:3000/api/task_draft/49",
                "draft_id": 49,
                "duration": 120,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Observation",
                "obsolete_since": null,
                "on_sky_duration": 120,
                "on_sky_start_time": "2022-04-22T02:52:06.166227",
                "on_sky_stop_time": "2022-04-22T02:54:06.166227",
                "output_pinned": false,
                "process_start_time": "2022-04-22T02:52:06.166227",
                "process_stop_time": "2022-04-22T02:54:06.166227",
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 0,
                "relative_stop_time": 120,
                "scheduled_start_time": "2022-04-22T02:52:06.166227",
                "scheduled_stop_time": "2022-04-22T02:54:06.166227",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/9",
                "scheduling_unit_blueprint_id": 9,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
                    "duration": 120,
                    "beamformer": {
                        "ppf": false,
                        "pipelines": [
                            {
                                "name": "Beamformer CV",
                                "coherent": {
                                    "SAPs": [
                                        {
                                            "name": "SAP0",
                                            "tabs": [
                                                {
                                                    "duration": 0,
                                                    "pointing": {
                                                        "angle1": 0,
                                                        "angle2": 0,
                                                        "target": "target1",
                                                        "direction_type": "J2000"
                                                    },
                                                    "relative": true,
                                                    "allow_after_previous": true
                                                }
                                            ],
                                            "subbands": {
                                                "list": [
                                                    51,
                                                    52,
                                                    53,
                                                    54,
                                                    55,
                                                    56,
                                                    57,
                                                    58,
                                                    59,
                                                    60,
                                                    61,
                                                    62,
                                                    63,
                                                    64,
                                                    65,
                                                    66,
                                                    67,
                                                    68,
                                                    69,
                                                    70,
                                                    71,
                                                    72,
                                                    73,
                                                    74,
                                                    75,
                                                    76,
                                                    77,
                                                    78,
                                                    79,
                                                    80,
                                                    81,
                                                    82,
                                                    83,
                                                    84,
                                                    85,
                                                    86,
                                                    87,
                                                    88,
                                                    89,
                                                    90,
                                                    91,
                                                    92,
                                                    93,
                                                    94,
                                                    95,
                                                    96,
                                                    97,
                                                    98,
                                                    99,
                                                    100,
                                                    101,
                                                    102,
                                                    103,
                                                    104,
                                                    105,
                                                    106,
                                                    107,
                                                    108,
                                                    109,
                                                    110,
                                                    111,
                                                    112,
                                                    113,
                                                    114,
                                                    115,
                                                    116,
                                                    117,
                                                    118,
                                                    119,
                                                    120,
                                                    121,
                                                    122,
                                                    123,
                                                    124,
                                                    125,
                                                    126,
                                                    127,
                                                    128,
                                                    129,
                                                    130,
                                                    131,
                                                    132,
                                                    133,
                                                    134,
                                                    135,
                                                    136,
                                                    137,
                                                    138,
                                                    139,
                                                    140,
                                                    141,
                                                    142,
                                                    143,
                                                    144,
                                                    145,
                                                    146,
                                                    147,
                                                    148,
                                                    149,
                                                    150,
                                                    151,
                                                    152,
                                                    153,
                                                    154,
                                                    155,
                                                    156,
                                                    157,
                                                    158,
                                                    159,
                                                    160,
                                                    161,
                                                    162,
                                                    163,
                                                    164,
                                                    165,
                                                    166,
                                                    167,
                                                    168,
                                                    169,
                                                    170,
                                                    171,
                                                    172,
                                                    173,
                                                    174,
                                                    175,
                                                    176,
                                                    177,
                                                    178,
                                                    179,
                                                    180,
                                                    181,
                                                    182,
                                                    183,
                                                    184,
                                                    185,
                                                    186,
                                                    187,
                                                    188,
                                                    189,
                                                    190,
                                                    191,
                                                    192,
                                                    193,
                                                    194,
                                                    195,
                                                    196,
                                                    197,
                                                    198,
                                                    199,
                                                    200,
                                                    201,
                                                    202,
                                                    203,
                                                    204,
                                                    205,
                                                    206,
                                                    207,
                                                    208,
                                                    209,
                                                    210,
                                                    211,
                                                    212,
                                                    213,
                                                    214,
                                                    215,
                                                    216,
                                                    217,
                                                    218,
                                                    219,
                                                    220,
                                                    221,
                                                    222,
                                                    223,
                                                    224,
                                                    225,
                                                    226,
                                                    227,
                                                    228,
                                                    229,
                                                    230,
                                                    231,
                                                    232,
                                                    233,
                                                    234,
                                                    235,
                                                    236,
                                                    237,
                                                    238,
                                                    239,
                                                    240,
                                                    241,
                                                    242,
                                                    243,
                                                    244,
                                                    245,
                                                    246,
                                                    247,
                                                    248,
                                                    249,
                                                    250,
                                                    251,
                                                    252,
                                                    253,
                                                    254,
                                                    255,
                                                    256,
                                                    257,
                                                    258,
                                                    259,
                                                    260,
                                                    261,
                                                    262,
                                                    263,
                                                    264,
                                                    265,
                                                    266,
                                                    267,
                                                    268,
                                                    269,
                                                    270,
                                                    271,
                                                    272,
                                                    273,
                                                    274,
                                                    275,
                                                    276,
                                                    277,
                                                    278,
                                                    279,
                                                    280,
                                                    281,
                                                    282,
                                                    283,
                                                    284,
                                                    285,
                                                    286,
                                                    287,
                                                    288,
                                                    289,
                                                    290,
                                                    291,
                                                    292,
                                                    293,
                                                    294,
                                                    295,
                                                    296,
                                                    297,
                                                    298,
                                                    299,
                                                    300,
                                                    301,
                                                    302,
                                                    303,
                                                    304,
                                                    305,
                                                    306,
                                                    307,
                                                    308,
                                                    309,
                                                    310,
                                                    311,
                                                    312,
                                                    313,
                                                    314,
                                                    315,
                                                    316,
                                                    317,
                                                    318,
                                                    319,
                                                    320,
                                                    321,
                                                    322,
                                                    323,
                                                    324,
                                                    325,
                                                    326,
                                                    327,
                                                    328,
                                                    329,
                                                    330,
                                                    331,
                                                    332,
                                                    333,
                                                    334,
                                                    335,
                                                    336,
                                                    337,
                                                    338,
                                                    339,
                                                    340,
                                                    341,
                                                    342,
                                                    343,
                                                    344,
                                                    345,
                                                    346,
                                                    347,
                                                    348,
                                                    349,
                                                    350,
                                                    351,
                                                    352,
                                                    353,
                                                    354,
                                                    355,
                                                    356,
                                                    357,
                                                    358,
                                                    359,
                                                    360,
                                                    361,
                                                    362,
                                                    363,
                                                    364,
                                                    365,
                                                    366,
                                                    367,
                                                    368,
                                                    369,
                                                    370,
                                                    371,
                                                    372,
                                                    373,
                                                    374,
                                                    375,
                                                    376,
                                                    377,
                                                    378,
                                                    379,
                                                    380,
                                                    381,
                                                    382,
                                                    383,
                                                    384,
                                                    385,
                                                    386,
                                                    387,
                                                    388,
                                                    389,
                                                    390,
                                                    391,
                                                    392,
                                                    393,
                                                    394,
                                                    395,
                                                    396,
                                                    397,
                                                    398,
                                                    399,
                                                    400,
                                                    401,
                                                    402,
                                                    403,
                                                    404,
                                                    405,
                                                    406,
                                                    407,
                                                    408,
                                                    409,
                                                    410,
                                                    411,
                                                    412,
                                                    413,
                                                    414,
                                                    415,
                                                    416,
                                                    417,
                                                    418,
                                                    419,
                                                    420,
                                                    421,
                                                    422,
                                                    423,
                                                    424,
                                                    425,
                                                    426,
                                                    427,
                                                    428,
                                                    429,
                                                    430,
                                                    431,
                                                    432,
                                                    433,
                                                    434,
                                                    435,
                                                    436,
                                                    437,
                                                    438,
                                                    439,
                                                    440,
                                                    441,
                                                    442,
                                                    443,
                                                    444,
                                                    445,
                                                    446,
                                                    447,
                                                    448,
                                                    449,
                                                    450
                                                ],
                                                "method": "copy"
                                            },
                                            "tab_rings": {
                                                "count": 0,
                                                "width": 0.01
                                            }
                                        }
                                    ],
                                    "settings": {
                                        "stokes": "XXYY",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 20,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "flys eye": {
                                    "enabled": false,
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "incoherent": {
                                    "SAPs": [],
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "station_groups": [
                                    {
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS101",
                                            "CS103",
                                            "CS201",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501"
                                        ],
                                        "max_nr_missing": 1
                                    }
                                ]
                            }
                        ]
                    },
                    "station_configuration": {
                        "SAPs": [
                            {
                                "name": "SAP0",
                                "subbands": [
                                    51,
                                    52,
                                    53,
                                    54,
                                    55,
                                    56,
                                    57,
                                    58,
                                    59,
                                    60,
                                    61,
                                    62,
                                    63,
                                    64,
                                    65,
                                    66,
                                    67,
                                    68,
                                    69,
                                    70,
                                    71,
                                    72,
                                    73,
                                    74,
                                    75,
                                    76,
                                    77,
                                    78,
                                    79,
                                    80,
                                    81,
                                    82,
                                    83,
                                    84,
                                    85,
                                    86,
                                    87,
                                    88,
                                    89,
                                    90,
                                    91,
                                    92,
                                    93,
                                    94,
                                    95,
                                    96,
                                    97,
                                    98,
                                    99,
                                    100,
                                    101,
                                    102,
                                    103,
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    137,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    164,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    181,
                                    182,
                                    183,
                                    184,
                                    185,
                                    186,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    210,
                                    211,
                                    212,
                                    213,
                                    214,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    241,
                                    242,
                                    243,
                                    244,
                                    245,
                                    246,
                                    247,
                                    248,
                                    249,
                                    250,
                                    251,
                                    252,
                                    253,
                                    254,
                                    255,
                                    256,
                                    257,
                                    258,
                                    259,
                                    260,
                                    261,
                                    262,
                                    263,
                                    264,
                                    265,
                                    266,
                                    267,
                                    268,
                                    269,
                                    270,
                                    271,
                                    272,
                                    273,
                                    274,
                                    275,
                                    276,
                                    277,
                                    278,
                                    279,
                                    280,
                                    281,
                                    282,
                                    283,
                                    284,
                                    285,
                                    286,
                                    287,
                                    288,
                                    289,
                                    290,
                                    291,
                                    292,
                                    293,
                                    294,
                                    295,
                                    296,
                                    297,
                                    298,
                                    299,
                                    300,
                                    301,
                                    302,
                                    303,
                                    304,
                                    305,
                                    306,
                                    307,
                                    308,
                                    309,
                                    310,
                                    311,
                                    312,
                                    313,
                                    314,
                                    315,
                                    316,
                                    317,
                                    318,
                                    319,
                                    320,
                                    321,
                                    322,
                                    323,
                                    324,
                                    325,
                                    326,
                                    327,
                                    328,
                                    329,
                                    330,
                                    331,
                                    332,
                                    333,
                                    334,
                                    335,
                                    336,
                                    337,
                                    338,
                                    339,
                                    340,
                                    341,
                                    342,
                                    343,
                                    344,
                                    345,
                                    346,
                                    347,
                                    348,
                                    349,
                                    350,
                                    351,
                                    352,
                                    353,
                                    354,
                                    355,
                                    356,
                                    357,
                                    358,
                                    359,
                                    360,
                                    361,
                                    362,
                                    363,
                                    364,
                                    365,
                                    366,
                                    367,
                                    368,
                                    369,
                                    370,
                                    371,
                                    372,
                                    373,
                                    374,
                                    375,
                                    376,
                                    377,
                                    378,
                                    379,
                                    380,
                                    381,
                                    382,
                                    383,
                                    384,
                                    385,
                                    386,
                                    387,
                                    388,
                                    389,
                                    390,
                                    391,
                                    392,
                                    393,
                                    394,
                                    395,
                                    396,
                                    397,
                                    398,
                                    399,
                                    400,
                                    401,
                                    402,
                                    403,
                                    404,
                                    405,
                                    406,
                                    407,
                                    408,
                                    409,
                                    410,
                                    411,
                                    412,
                                    413,
                                    414,
                                    415,
                                    416,
                                    417,
                                    418,
                                    419,
                                    420,
                                    421,
                                    422,
                                    423,
                                    424,
                                    425,
                                    426,
                                    427,
                                    428,
                                    429,
                                    430,
                                    431,
                                    432,
                                    433,
                                    434,
                                    435,
                                    436,
                                    437,
                                    438,
                                    439,
                                    440,
                                    441,
                                    442,
                                    443,
                                    444,
                                    445,
                                    446,
                                    447,
                                    448,
                                    449,
                                    450
                                ],
                                "digital_pointing": {
                                    "angle1": 0.92934186635,
                                    "angle2": 0.952579228492,
                                    "target": "target1",
                                    "direction_type": "J2000"
                                }
                            }
                        ],
                        "filter": "HBA_110_190",
                        "tile_beam": {
                            "angle1": 0.92934186635,
                            "angle2": 0.952579228492,
                            "target": "target1",
                            "direction_type": "J2000"
                        },
                        "antenna_set": "HBA_DUAL_INNER",
                        "station_groups": [
                            {
                                "stations": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                },
                "specifications_template": {
                    "id": 3,
                    "url": "http://localhost:3000/api/task_template/3",
                    "created_at": "2022-04-21T08:15:52.853823",
                    "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                    "name": "beamforming observation",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
                        "type": "object",
                        "title": "beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "beamformer",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "#/definitions/timedelta",
                                "title": "Duration",
                                "default": 300,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "#/definitions/beamformer",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "timedelta": {
                                "default": 0,
                                "description": "A time duration or delta expressed in seconds",
                                "type": "number"
                            },
                            "timestamp": {
                                "default": "1970-01-01T00:00:00Z",
                                "description": "A timestamp defined in UTC",
                                "format": "date-time",
                                "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                                "type": "string"
                            },
                            "beamformer": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "pipelines": {
                                        "additionalItems": false,
                                        "default": [
                                            {}
                                        ],
                                        "description": "This array of beamformer pipelines is currently restricted to exactly 1 item. Support for handling multiple pipelines (in cobalt) will be added in the future.",
                                        "items": {
                                            "$ref": "#/definitions/beamformer_pipeline"
                                        },
                                        "maxItems": 1,
                                        "minItems": 1,
                                        "title": "Pipelines",
                                        "type": "array"
                                    },
                                    "ppf": {
                                        "default": false,
                                        "description": "Use a PPF for the channellisation for beamforming. If not, an FFT is used.",
                                        "title": "Use PPF when beamforming",
                                        "type": "boolean"
                                    }
                                },
                                "required": [
                                    "pipelines",
                                    "ppf"
                                ],
                                "title": "Beamformer",
                                "type": "object"
                            },
                            "beamformer_pipeline": {
                                "additionalProperties": false,
                                "default": {},
                                "headerTemplate": "Beamformer Pipeline {{ self.index }}",
                                "properties": {
                                    "coherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to beamform.",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        },
                                                        "tab_rings": {
                                                            "additonalProperties": false,
                                                            "default": {},
                                                            "description": "Rings of TABs around the center of the beam.",
                                                            "properties": {
                                                                "count": {
                                                                    "default": 0,
                                                                    "maximum": 11,
                                                                    "minimum": 0,
                                                                    "title": "Number of rings",
                                                                    "type": "integer"
                                                                },
                                                                "width": {
                                                                    "default": 0.01,
                                                                    "description": "Distance between pointings.",
                                                                    "minimum": 0,
                                                                    "title": "Ring width",
                                                                    "type": "number"
                                                                }
                                                            },
                                                            "title": "Tied-Array Rings",
                                                            "type": "object"
                                                        },
                                                        "tabs": {
                                                            "additionalItems": false,
                                                            "default": [],
                                                            "description": "Tied-array beams to form",
                                                            "items": {
                                                                "additionalProperties": false,
                                                                "default": {},
                                                                "headerTemplate": "TAB {{ self.index }}",
                                                                "properties": {
                                                                    "allow_after_previous": {
                                                                        "default": true,
                                                                        "description": "This TAB can be recorded after the previous TAB, if duration allows.",
                                                                        "title": "Allow after previous",
                                                                        "type": "boolean"
                                                                    },
                                                                    "duration": {
                                                                        "default": 0,
                                                                        "description": "Exposure time of the TAB. 0 = observation duration.",
                                                                        "maximum": 0,
                                                                        "minimum": 0,
                                                                        "title": "Duration",
                                                                        "type": "integer"
                                                                    },
                                                                    "pointing": {
                                                                        "$ref": "#/definitions/pointing",
                                                                        "default": {}
                                                                    },
                                                                    "relative": {
                                                                        "default": false,
                                                                        "description": "Angular offset applied to the TAB pointing",
                                                                        "title": "Relative to SAP",
                                                                        "type": "boolean"
                                                                    }
                                                                },
                                                                "required": [
                                                                    "pointing",
                                                                    "relative"
                                                                ],
                                                                "title": "Tied-Array Beam",
                                                                "type": "object"
                                                            },
                                                            "title": "Tied-Array Beams",
                                                            "type": "array"
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "tabs"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "SAPs",
                                            "settings"
                                        ],
                                        "title": "Coherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "flys eye": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "description": "Produce beams containing the individual station signals",
                                        "properties": {
                                            "enabled": {
                                                "default": false,
                                                "title": "Enable Fly's Eye",
                                                "type": "boolean"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "enabled"
                                        ],
                                        "title": "Fly's Eye Settings",
                                        "type": "object"
                                    },
                                    "incoherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to create incoherent TABs for (empty list = all).",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "default": {},
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "subbands"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "settings",
                                            "SAPs"
                                        ],
                                        "title": "Incoherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "name": {
                                        "default": "_beamformer_name_",
                                        "description": "Beamformer name, used for identification purposes.",
                                        "minLength": 1,
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501"
                                                ]
                                            }
                                        ],
                                        "description": "While observing, COBALT will beamform on the intersection of all stations in this list and the used stations in the observation. So, specifying all possible stations here means that all observation-stations are used. Specifying a small subset here means that only the observing-stations in this small list are used. By default we let COBALT beamform on the Core stations.",
                                        "minItems": 1
                                    }
                                },
                                "required": [
                                    "name"
                                ],
                                "title": "Beamformer Pipeline",
                                "type": "object"
                            },
                            "subband_selection": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "list": {
                                        "additionalItems": false,
                                        "default": [],
                                        "description": "If method is 'select subset', only beamform these subbands, and only if they occur in the SAP.",
                                        "items": {
                                            "maximum": 511,
                                            "minimum": 0,
                                            "title": "Subband",
                                            "type": "integer"
                                        },
                                        "minItems": 0,
                                        "title": "Subset selection",
                                        "type": "array"
                                    },
                                    "method": {
                                        "default": "copy",
                                        "description": "How to select the subbands to beam form",
                                        "enum": [
                                            "copy",
                                            "largest continuous subset",
                                            "select subset"
                                        ],
                                        "title": "Method",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "method"
                                ],
                                "title": "Subband selection",
                                "type": "object"
                            },
                            "station_list": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [],
                                "items": {
                                    "$ref": "#/definitions/station"
                                },
                                "minItems": 0,
                                "type": "array",
                                "uniqueItems": true
                            },
                            "pointing": {
                                "additionalProperties": false,
                                "properties": {
                                    "angle1": {
                                        "default": 0.6624317181687094,
                                        "description": "First angle (e.g. RA)",
                                        "title": "Angle 1",
                                        "type": "number"
                                    },
                                    "angle2": {
                                        "default": 1.5579526427549426,
                                        "description": "Second angle (e.g. DEC)",
                                        "title": "Angle 2",
                                        "type": "number"
                                    },
                                    "direction_type": {
                                        "default": "J2000",
                                        "description": "",
                                        "enum": [
                                            "J2000",
                                            "AZELGEO",
                                            "LMN",
                                            "SUN",
                                            "MOON",
                                            "MERCURY",
                                            "VENUS",
                                            "MARS",
                                            "JUPITER",
                                            "SATURN",
                                            "URANUS",
                                            "NEPTUNE",
                                            "PLUTO"
                                        ],
                                        "title": "Reference frame",
                                        "type": "string"
                                    },
                                    "target": {
                                        "default": "_target_name_",
                                        "description": "Description of where this beam points at",
                                        "minLength": 1,
                                        "title": "Target",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "angle1",
                                    "angle2",
                                    "target"
                                ],
                                "type": "object"
                            },
                            "stokes_settings": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "channels_per_subband": {
                                        "default": 1,
                                        "description": "Number of frequency bands per subband",
                                        "enum": [
                                            1,
                                            8,
                                            16,
                                            32,
                                            64,
                                            128,
                                            256,
                                            512,
                                            1024
                                        ],
                                        "minimum": 1,
                                        "title": "Channels/subband",
                                        "type": "integer"
                                    },
                                    "quantisation": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "bits": {
                                                "default": 8,
                                                "description": "Number of bits for a single value",
                                                "enum": [
                                                    8
                                                ],
                                                "title": "Bits/sample",
                                                "type": "integer"
                                            },
                                            "enabled": {
                                                "default": false,
                                                "title": "Output quantisation into integers",
                                                "type": "boolean"
                                            },
                                            "scale_max": {
                                                "default": 5,
                                                "description": "Cut off values above this treshold",
                                                "title": "Maximum value",
                                                "type": "number"
                                            },
                                            "scale_min": {
                                                "default": -5,
                                                "description": "Cut off values below this treshold",
                                                "title": "Minimum value",
                                                "type": "number"
                                            }
                                        },
                                        "required": [
                                            "enabled",
                                            "bits",
                                            "scale_min",
                                            "scale_max"
                                        ],
                                        "title": "Output quantisation settings",
                                        "type": "object"
                                    },
                                    "stokes": {
                                        "$ref": "#/definitions/stokes",
                                        "default": "I"
                                    },
                                    "subbands_per_file": {
                                        "default": 488,
                                        "description": "The maximum number of subbands to write in each output dataproduct.",
                                        "maximum": 488,
                                        "minimum": 1,
                                        "title": "Subbands per file",
                                        "type": "integer"
                                    },
                                    "time_integration_factor": {
                                        "default": 1,
                                        "description": "The number of samples to integrate over",
                                        "maximum": 12288,
                                        "minimum": 1,
                                        "title": "Time integration",
                                        "type": "integer"
                                    }
                                },
                                "required": [
                                    "stokes",
                                    "time_integration_factor",
                                    "channels_per_subband"
                                ],
                                "type": "object"
                            },
                            "station": {
                                "description": "These are the LOFAR stations",
                                "enum": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501",
                                    "RS106",
                                    "RS205",
                                    "RS208",
                                    "RS210",
                                    "RS305",
                                    "RS306",
                                    "RS307",
                                    "RS310",
                                    "RS406",
                                    "RS407",
                                    "RS409",
                                    "RS503",
                                    "RS508",
                                    "RS509",
                                    "DE601",
                                    "DE602",
                                    "DE603",
                                    "DE604",
                                    "DE605",
                                    "FR606",
                                    "SE607",
                                    "UK608",
                                    "DE609",
                                    "PL610",
                                    "PL611",
                                    "PL612",
                                    "IE613",
                                    "LV614"
                                ],
                                "title": "Station",
                                "type": "string"
                            },
                            "max_number_of_missing_stations": {
                                "default": 0,
                                "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                                "minimum": 0,
                                "title": "Maximum number of stations to omit",
                                "type": "integer"
                            },
                            "stokes": {
                                "default": "I",
                                "description": "Which Stokes to produce",
                                "enum": [
                                    "I",
                                    "IQUV",
                                    "XXYY"
                                ],
                                "title": "Stokes",
                                "type": "string"
                            },
                            "station_groups": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [
                                    {
                                        "max_nr_missing": 1,
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ]
                                    }
                                ],
                                "description": "One or more predefined or custom groups of stations",
                                "items": {
                                    "$ref": "#/definitions/station_group"
                                },
                                "minItems": 1,
                                "title": "Station groups",
                                "type": "array"
                            },
                            "station_group": {
                                "anyOf": [
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007"
                                            ]
                                        },
                                        "description": "The group of all stations on the Superterp",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Superterp",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501"
                                            ]
                                        },
                                        "description": "The group of all Core stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Core",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch remote stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Remote",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch (Core + Remote) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Dutch",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 2,
                                            "stations": [
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 1,
                                            "stations": [
                                                "DE601",
                                                "DE605"
                                            ]
                                        },
                                        "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE605"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International required",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 6,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509",
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all (Core + Remote + International) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509",
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "All",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS001"
                                            ]
                                        },
                                        "description": "A custom group of stations which can be defined by the user",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list"
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Custom",
                                        "type": "object"
                                    }
                                ],
                                "default": {
                                    "max_nr_missing": 1,
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                },
                                "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                                "title": "Station group",
                                "type": "object"
                            },
                            "station_configuration": {
                                "default": {},
                                "properties": {
                                    "SAPs": {
                                        "$ref": "#/definitions/SAPs",
                                        "default": [
                                            {}
                                        ],
                                        "minItems": 1
                                    },
                                    "antenna_set": {
                                        "$ref": "#/definitions/antenna_set",
                                        "default": "HBA_DUAL"
                                    },
                                    "filter": {
                                        "$ref": "#/definitions/filter",
                                        "default": "HBA_110_190"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007"
                                                ]
                                            }
                                        ]
                                    },
                                    "tile_beam": {
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "description": "HBA only",
                                        "title": "Tile beam"
                                    }
                                },
                                "required": [
                                    "station_groups",
                                    "antenna_set",
                                    "filter",
                                    "SAPs"
                                ],
                                "title": "station_configuration",
                                "type": "object"
                            },
                            "filter": {
                                "default": "HBA_110_190",
                                "description": "Must match antenna type",
                                "enum": [
                                    "LBA_10_70",
                                    "LBA_30_70",
                                    "LBA_10_90",
                                    "LBA_30_90",
                                    "HBA_110_190",
                                    "HBA_210_250"
                                ],
                                "title": "Band-pass filter",
                                "type": "string"
                            },
                            "antenna_set": {
                                "default": "HBA_DUAL",
                                "description": "Fields & antennas to use",
                                "enum": [
                                    "HBA_DUAL",
                                    "HBA_DUAL_INNER",
                                    "HBA_ONE",
                                    "HBA_ONE_INNER",
                                    "HBA_ZERO",
                                    "HBA_ZERO_INNER",
                                    "LBA_INNER",
                                    "LBA_OUTER",
                                    "LBA_SPARSE_EVEN",
                                    "LBA_SPARSE_ODD",
                                    "LBA_ALL"
                                ],
                                "title": "Antenna set",
                                "type": "string"
                            },
                            "SAPs": {
                                "additionalItems": false,
                                "default": [
                                    {}
                                ],
                                "description": "Station beams",
                                "items": {
                                    "additionalProperties": false,
                                    "default": {},
                                    "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                    "properties": {
                                        "digital_pointing": {
                                            "$id": "#target_pointing",
                                            "$ref": "#/definitions/pointing",
                                            "default": {},
                                            "title": "Digital pointing"
                                        },
                                        "name": {
                                            "default": "_SAP_name_",
                                            "description": "Identifier for this beam",
                                            "minLength": 1,
                                            "title": "Name",
                                            "type": "string"
                                        },
                                        "subbands": {
                                            "additionalItems": false,
                                            "default": [
                                                255
                                            ],
                                            "items": {
                                                "maximum": 511,
                                                "minimum": 0,
                                                "title": "Subband",
                                                "type": "integer"
                                            },
                                            "maxItems": 488,
                                            "minItems": 1,
                                            "title": "Subband list",
                                            "type": "array"
                                        }
                                    },
                                    "required": [
                                        "name",
                                        "digital_pointing",
                                        "subbands"
                                    ],
                                    "title": "SAP",
                                    "type": "object"
                                },
                                "minItems": 0,
                                "title": "SAPs",
                                "type": "array"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2#",
                        "type": "object",
                        "title": "beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "beamformer",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                "title": "Duration",
                                "default": 300,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/2#/definitions/beamformer",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/observation",
                    "type_value": "observation",
                    "updated_at": "2022-04-21T08:15:52.853846",
                    "version": 2
                },
                "specifications_template_id": 3,
                "status": "scheduled",
                "status_value": "scheduled",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000068"
                ],
                "subtasks_ids": [
                    2000068
                ],
                "tags": [],
                "task_type": "observation",
                "unschedulable_reason": "",
                "updated_at": "2022-04-21T08:16:31.148228"
            },
            {
                "id": 51,
                "url": "http://localhost:3000/api/task_blueprint/51",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-04-21T08:16:31.257688",
                "description": "Cleanup all dataproducts from disk",
                "draft": "http://localhost:3000/api/task_draft/51",
                "draft_id": 51,
                "duration": 300,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Cleanup",
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "output_pinned": false,
                "process_start_time": "2022-04-22T02:56:06.166227",
                "process_stop_time": "2022-04-22T03:01:06.166227",
                "produced_by": [
                    "http://localhost:3000/api/task_relation_blueprint/67",
                    "http://localhost:3000/api/task_relation_blueprint/68",
                    "http://localhost:3000/api/task_relation_blueprint/69"
                ],
                "produced_by_ids": [
                    67,
                    68,
                    69
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduled_start_time": "2022-04-22T02:56:06.166227",
                "scheduled_stop_time": "2022-04-22T03:01:06.166227",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/9",
                "scheduling_unit_blueprint_id": 9,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2/ref_resolved"
                },
                "specifications_template": {
                    "id": 1,
                    "url": "http://localhost:3000/api/task_template/1",
                    "created_at": "2022-04-21T08:15:52.702755",
                    "description": "This schema defines the parameters to setup a dataproduct(s) cleanup task.",
                    "name": "cleanup",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2/ref_resolved",
                        "type": "object",
                        "title": "cleanup",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [],
                        "properties": {},
                        "description": "This schema defines the parameters to setup a dataproduct(s) cleanup task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {}
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/cleanup/2#",
                        "type": "object",
                        "title": "cleanup",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [],
                        "properties": {},
                        "description": "This schema defines the parameters to setup a dataproduct(s) cleanup task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/cleanup",
                    "type_value": "cleanup",
                    "updated_at": "2022-04-21T08:15:52.702788",
                    "version": 2
                },
                "specifications_template_id": 1,
                "status": "schedulable",
                "status_value": "schedulable",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000071"
                ],
                "subtasks_ids": [
                    2000071
                ],
                "tags": [],
                "task_type": "cleanup",
                "unschedulable_reason": "",
                "updated_at": "2022-04-21T08:16:31.257700"
            },
            {
                "id": 52,
                "url": "http://localhost:3000/api/task_blueprint/52",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-04-21T08:16:31.327526",
                "description": "Ingest the pipeline outputs dataproducts",
                "draft": "http://localhost:3000/api/task_draft/52",
                "draft_id": 52,
                "duration": 300,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Ingest",
                "obsolete_since": null,
                "on_sky_duration": null,
                "on_sky_start_time": null,
                "on_sky_stop_time": null,
                "output_pinned": false,
                "process_start_time": "2022-04-22T02:56:06.166227",
                "process_stop_time": "2022-04-22T03:01:06.166227",
                "produced_by": [
                    "http://localhost:3000/api/task_relation_blueprint/70",
                    "http://localhost:3000/api/task_relation_blueprint/71"
                ],
                "produced_by_ids": [
                    70,
                    71
                ],
                "relative_start_time": 0,
                "relative_stop_time": 0,
                "scheduled_start_time": "2022-04-22T02:56:06.166227",
                "scheduled_stop_time": "2022-04-22T03:01:06.166227",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/9",
                "scheduling_unit_blueprint_id": 9,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved"
                },
                "specifications_template": {
                    "id": 2,
                    "url": "http://localhost:3000/api/task_template/2",
                    "created_at": "2022-04-21T08:15:52.742462",
                    "description": "This schema defines the parameters to setup an ingest task.",
                    "name": "ingest",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2/ref_resolved",
                        "type": "object",
                        "title": "ingest",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [],
                        "properties": {},
                        "description": "This schema defines the parameters to setup an ingest task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {}
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/ingest/2#",
                        "type": "object",
                        "title": "ingest",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [],
                        "properties": {},
                        "description": "This schema defines the parameters to setup an ingest task.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/ingest",
                    "type_value": "ingest",
                    "updated_at": "2022-04-21T08:15:52.742474",
                    "version": 2
                },
                "specifications_template_id": 2,
                "status": "schedulable",
                "status_value": "schedulable",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000070"
                ],
                "subtasks_ids": [
                    2000070
                ],
                "tags": [],
                "task_type": "ingest",
                "unschedulable_reason": "",
                "updated_at": "2022-04-21T08:16:31.327539"
            }
        ],
        "task_blueprints_ids": [
            50,
            49,
            51,
            52
        ],
        "unschedulable_reason": "",
        "updated_at": "2022-04-21T08:16:31.086568"
    },
    {
        "id": 10,
        "url": "http://localhost:3000/api/scheduling_unit_blueprint/10",
        "actual_on_sky_start_time": null,
        "actual_on_sky_stop_time": null,
        "actual_process_start_time": null,
        "actual_process_stop_time": null,
        "created_at": "2022-04-21T08:16:32.661330",
        "description": "Test scheduling unit",
        "draft": "http://localhost:3000/api/scheduling_unit_draft/10",
        "draft_id": 10,
        "duration": 120,
        "error_reason": "",
        "global_identifier": "http://localhost:3000/api/sip_identifier/154",
        "global_identifier_id": 154,
        "ingest_permission_granted_since": null,
        "ingest_permission_required": true,
        "interrupts_telescope": false,
        "name": "Simple Beamforming Observation",
        "observed_duration": null,
        "observed_start_time": null,
        "observed_stop_time": null,
        "obsolete_since": null,
        "on_sky_duration": 120,
        "on_sky_start_time": "2022-04-22T03:06:06.166227",
        "on_sky_stop_time": "2022-04-22T03:08:06.166227",
        "output_pinned": false,
        "piggyback_allowed_aartfaac": true,
        "piggyback_allowed_tbb": true,
        "priority_queue": "http://localhost:3000/api/priority_queue_type/A",
        "priority_queue_value": "A",
        "process_start_time": "2022-04-22T03:06:06.166227",
        "process_stop_time": "2022-04-22T03:08:06.166227",
        "rank": 1,
        "scheduled_start_time": "2022-04-22T03:06:06.166227",
        "scheduled_stop_time": "2022-04-22T03:08:06.166227",
        "scheduling_constraints_doc": {
            "sky": {
                "min_distance": {
                    "sun": 0.5,
                    "moon": 0.5,
                    "jupiter": 0.5
                },
                "min_elevation": {
                    "target": 0.5,
                    "calibrator": 0.5
                },
                "transit_offset": {
                    "to": 7200,
                    "from": -7200
                }
            },
            "time": {
                "at": "2022-04-22T03:06:06.166227"
            },
            "daily": {
                "require_day": false,
                "require_night": false,
                "avoid_twilight": false
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
            "scheduler": "fixed_time"
        },
        "scheduling_constraints_template": "http://localhost:3000/api/scheduling_constraints_template/1",
        "scheduling_constraints_template_id": 1,
        "specifications_template": "http://localhost:3000/api/scheduling_unit_template/1",
        "specifications_template_id": 1,
        "status": "scheduled",
        "status_value": "scheduled",
        "tags": [],
        "task_blueprints": [
            {
                "id": 53,
                "url": "http://localhost:3000/api/task_blueprint/53",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-04-21T08:16:32.722465",
                "description": "A simple short test beamforming observation",
                "draft": "http://localhost:3000/api/task_draft/53",
                "draft_id": 53,
                "duration": 120,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Observation",
                "obsolete_since": null,
                "on_sky_duration": 120,
                "on_sky_start_time": "2022-04-22T03:06:06.166227",
                "on_sky_stop_time": "2022-04-22T03:08:06.166227",
                "output_pinned": false,
                "process_start_time": "2022-04-22T03:06:06.166227",
                "process_stop_time": "2022-04-22T03:08:06.166227",
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 0,
                "relative_stop_time": 120,
                "scheduled_start_time": "2022-04-22T03:06:06.166227",
                "scheduled_stop_time": "2022-04-22T03:08:06.166227",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/10",
                "scheduling_unit_blueprint_id": 10,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
                    "duration": 120,
                    "beamformer": {
                        "ppf": false,
                        "pipelines": [
                            {
                                "name": "B0329+54",
                                "coherent": {
                                    "SAPs": [
                                        {
                                            "name": "B0329+54",
                                            "tabs": [
                                                {
                                                    "duration": 0,
                                                    "pointing": {
                                                        "angle1": 0.92934186635,
                                                        "angle2": 0.952579228492,
                                                        "target": "target1",
                                                        "direction_type": "J2000"
                                                    },
                                                    "relative": false,
                                                    "allow_after_previous": true
                                                }
                                            ],
                                            "subbands": {
                                                "list": [
                                                    0,
                                                    1,
                                                    2,
                                                    3,
                                                    4,
                                                    5,
                                                    6,
                                                    7,
                                                    8,
                                                    9,
                                                    10,
                                                    11,
                                                    12,
                                                    13,
                                                    14,
                                                    15,
                                                    16,
                                                    17,
                                                    18,
                                                    19,
                                                    20,
                                                    21,
                                                    22,
                                                    23,
                                                    24,
                                                    25,
                                                    26,
                                                    27,
                                                    28,
                                                    29,
                                                    30,
                                                    31,
                                                    32,
                                                    33,
                                                    34,
                                                    35,
                                                    36,
                                                    37,
                                                    38,
                                                    39,
                                                    40,
                                                    41,
                                                    42,
                                                    43,
                                                    44,
                                                    45,
                                                    46,
                                                    47,
                                                    48,
                                                    49,
                                                    50,
                                                    51,
                                                    52,
                                                    53,
                                                    54,
                                                    55,
                                                    56,
                                                    57,
                                                    58,
                                                    59,
                                                    60,
                                                    61,
                                                    62,
                                                    63,
                                                    64,
                                                    65,
                                                    66,
                                                    67,
                                                    68,
                                                    69,
                                                    70,
                                                    71,
                                                    72,
                                                    73,
                                                    74,
                                                    75,
                                                    76,
                                                    77,
                                                    78,
                                                    79,
                                                    80,
                                                    81,
                                                    82,
                                                    83,
                                                    84,
                                                    85,
                                                    86,
                                                    87,
                                                    88,
                                                    89,
                                                    90,
                                                    91,
                                                    92,
                                                    93,
                                                    94,
                                                    95,
                                                    96,
                                                    97,
                                                    98,
                                                    99,
                                                    100,
                                                    101,
                                                    102,
                                                    103,
                                                    104,
                                                    105,
                                                    106,
                                                    107,
                                                    108,
                                                    109,
                                                    110,
                                                    111,
                                                    112,
                                                    113,
                                                    114,
                                                    115,
                                                    116,
                                                    117,
                                                    118,
                                                    119,
                                                    120,
                                                    121,
                                                    122,
                                                    123,
                                                    124,
                                                    125,
                                                    126,
                                                    127,
                                                    128,
                                                    129,
                                                    130,
                                                    131,
                                                    132,
                                                    133,
                                                    134,
                                                    135,
                                                    136,
                                                    137,
                                                    138,
                                                    139,
                                                    140,
                                                    141,
                                                    142,
                                                    143,
                                                    144,
                                                    145,
                                                    146,
                                                    147,
                                                    148,
                                                    149,
                                                    150,
                                                    151,
                                                    152,
                                                    153,
                                                    154,
                                                    155,
                                                    156,
                                                    157,
                                                    158,
                                                    159,
                                                    160,
                                                    161,
                                                    162,
                                                    163,
                                                    164,
                                                    165,
                                                    166,
                                                    167,
                                                    168,
                                                    169,
                                                    170,
                                                    171,
                                                    172,
                                                    173,
                                                    174,
                                                    175,
                                                    176,
                                                    177,
                                                    178,
                                                    179,
                                                    180,
                                                    181,
                                                    182,
                                                    183,
                                                    184,
                                                    185,
                                                    186,
                                                    187,
                                                    188,
                                                    189,
                                                    190,
                                                    191,
                                                    192,
                                                    193,
                                                    194,
                                                    195,
                                                    196,
                                                    197,
                                                    198,
                                                    199,
                                                    200,
                                                    201,
                                                    202,
                                                    203,
                                                    204,
                                                    205,
                                                    206,
                                                    207,
                                                    208,
                                                    209,
                                                    210,
                                                    211,
                                                    212,
                                                    213,
                                                    214,
                                                    215,
                                                    216,
                                                    217,
                                                    218,
                                                    219,
                                                    220,
                                                    221,
                                                    222,
                                                    223,
                                                    224,
                                                    225,
                                                    226,
                                                    227,
                                                    228,
                                                    229,
                                                    230,
                                                    231,
                                                    232,
                                                    233,
                                                    234,
                                                    235,
                                                    236,
                                                    237,
                                                    238,
                                                    239,
                                                    240,
                                                    241,
                                                    242,
                                                    243
                                                ],
                                                "method": "copy"
                                            },
                                            "tab_rings": {
                                                "count": 0,
                                                "width": 0.01
                                            }
                                        }
                                    ],
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "flys eye": {
                                    "enabled": false,
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "incoherent": {
                                    "SAPs": [],
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "station_groups": [
                                    {
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ],
                                        "max_nr_missing": 1
                                    }
                                ]
                            }
                        ]
                    },
                    "station_configuration": {
                        "SAPs": [
                            {
                                "name": "B0329+54",
                                "subbands": [
                                    0,
                                    1,
                                    2,
                                    3,
                                    4,
                                    5,
                                    6,
                                    7,
                                    8,
                                    9,
                                    10,
                                    11,
                                    12,
                                    13,
                                    14,
                                    15,
                                    16,
                                    17,
                                    18,
                                    19,
                                    20,
                                    21,
                                    22,
                                    23,
                                    24,
                                    25,
                                    26,
                                    27,
                                    28,
                                    29,
                                    30,
                                    31,
                                    32,
                                    33,
                                    34,
                                    35,
                                    36,
                                    37,
                                    38,
                                    39,
                                    40,
                                    41,
                                    42,
                                    43,
                                    44,
                                    45,
                                    46,
                                    47,
                                    48,
                                    49,
                                    50,
                                    51,
                                    52,
                                    53,
                                    54,
                                    55,
                                    56,
                                    57,
                                    58,
                                    59,
                                    60,
                                    61,
                                    62,
                                    63,
                                    64,
                                    65,
                                    66,
                                    67,
                                    68,
                                    69,
                                    70,
                                    71,
                                    72,
                                    73,
                                    74,
                                    75,
                                    76,
                                    77,
                                    78,
                                    79,
                                    80,
                                    81,
                                    82,
                                    83,
                                    84,
                                    85,
                                    86,
                                    87,
                                    88,
                                    89,
                                    90,
                                    91,
                                    92,
                                    93,
                                    94,
                                    95,
                                    96,
                                    97,
                                    98,
                                    99,
                                    100,
                                    101,
                                    102,
                                    103,
                                    104,
                                    105,
                                    106,
                                    107,
                                    108,
                                    109,
                                    110,
                                    111,
                                    112,
                                    113,
                                    114,
                                    115,
                                    116,
                                    117,
                                    118,
                                    119,
                                    120,
                                    121,
                                    122,
                                    123,
                                    124,
                                    125,
                                    126,
                                    127,
                                    128,
                                    129,
                                    130,
                                    131,
                                    132,
                                    133,
                                    134,
                                    135,
                                    136,
                                    137,
                                    138,
                                    139,
                                    140,
                                    141,
                                    142,
                                    143,
                                    144,
                                    145,
                                    146,
                                    147,
                                    148,
                                    149,
                                    150,
                                    151,
                                    152,
                                    153,
                                    154,
                                    155,
                                    156,
                                    157,
                                    158,
                                    159,
                                    160,
                                    161,
                                    162,
                                    163,
                                    164,
                                    165,
                                    166,
                                    167,
                                    168,
                                    169,
                                    170,
                                    171,
                                    172,
                                    173,
                                    174,
                                    175,
                                    176,
                                    177,
                                    178,
                                    179,
                                    180,
                                    181,
                                    182,
                                    183,
                                    184,
                                    185,
                                    186,
                                    187,
                                    188,
                                    189,
                                    190,
                                    191,
                                    192,
                                    193,
                                    194,
                                    195,
                                    196,
                                    197,
                                    198,
                                    199,
                                    200,
                                    201,
                                    202,
                                    203,
                                    204,
                                    205,
                                    206,
                                    207,
                                    208,
                                    209,
                                    210,
                                    211,
                                    212,
                                    213,
                                    214,
                                    215,
                                    216,
                                    217,
                                    218,
                                    219,
                                    220,
                                    221,
                                    222,
                                    223,
                                    224,
                                    225,
                                    226,
                                    227,
                                    228,
                                    229,
                                    230,
                                    231,
                                    232,
                                    233,
                                    234,
                                    235,
                                    236,
                                    237,
                                    238,
                                    239,
                                    240,
                                    241,
                                    242,
                                    243
                                ],
                                "digital_pointing": {
                                    "angle1": 0.92934186635,
                                    "angle2": 0.952579228492,
                                    "target": "target1",
                                    "direction_type": "J2000"
                                }
                            }
                        ],
                        "filter": "HBA_110_190",
                        "tile_beam": {
                            "angle1": 0.92934186635,
                            "angle2": 0.952579228492,
                            "target": "target1",
                            "direction_type": "J2000"
                        },
                        "antenna_set": "HBA_DUAL_INNER",
                        "station_groups": [
                            {
                                "stations": [
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                },
                "specifications_template": {
                    "id": 3,
                    "url": "http://localhost:3000/api/task_template/3",
                    "created_at": "2022-04-21T08:15:52.853823",
                    "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                    "name": "beamforming observation",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2/ref_resolved",
                        "type": "object",
                        "title": "beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "beamformer",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "#/definitions/timedelta",
                                "title": "Duration",
                                "default": 300,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "#/definitions/beamformer",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "timedelta": {
                                "default": 0,
                                "description": "A time duration or delta expressed in seconds",
                                "type": "number"
                            },
                            "timestamp": {
                                "default": "1970-01-01T00:00:00Z",
                                "description": "A timestamp defined in UTC",
                                "format": "date-time",
                                "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                                "type": "string"
                            },
                            "beamformer": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "pipelines": {
                                        "additionalItems": false,
                                        "default": [
                                            {}
                                        ],
                                        "description": "This array of beamformer pipelines is currently restricted to exactly 1 item. Support for handling multiple pipelines (in cobalt) will be added in the future.",
                                        "items": {
                                            "$ref": "#/definitions/beamformer_pipeline"
                                        },
                                        "maxItems": 1,
                                        "minItems": 1,
                                        "title": "Pipelines",
                                        "type": "array"
                                    },
                                    "ppf": {
                                        "default": false,
                                        "description": "Use a PPF for the channellisation for beamforming. If not, an FFT is used.",
                                        "title": "Use PPF when beamforming",
                                        "type": "boolean"
                                    }
                                },
                                "required": [
                                    "pipelines",
                                    "ppf"
                                ],
                                "title": "Beamformer",
                                "type": "object"
                            },
                            "beamformer_pipeline": {
                                "additionalProperties": false,
                                "default": {},
                                "headerTemplate": "Beamformer Pipeline {{ self.index }}",
                                "properties": {
                                    "coherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to beamform.",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        },
                                                        "tab_rings": {
                                                            "additonalProperties": false,
                                                            "default": {},
                                                            "description": "Rings of TABs around the center of the beam.",
                                                            "properties": {
                                                                "count": {
                                                                    "default": 0,
                                                                    "maximum": 11,
                                                                    "minimum": 0,
                                                                    "title": "Number of rings",
                                                                    "type": "integer"
                                                                },
                                                                "width": {
                                                                    "default": 0.01,
                                                                    "description": "Distance between pointings.",
                                                                    "minimum": 0,
                                                                    "title": "Ring width",
                                                                    "type": "number"
                                                                }
                                                            },
                                                            "title": "Tied-Array Rings",
                                                            "type": "object"
                                                        },
                                                        "tabs": {
                                                            "additionalItems": false,
                                                            "default": [],
                                                            "description": "Tied-array beams to form",
                                                            "items": {
                                                                "additionalProperties": false,
                                                                "default": {},
                                                                "headerTemplate": "TAB {{ self.index }}",
                                                                "properties": {
                                                                    "allow_after_previous": {
                                                                        "default": true,
                                                                        "description": "This TAB can be recorded after the previous TAB, if duration allows.",
                                                                        "title": "Allow after previous",
                                                                        "type": "boolean"
                                                                    },
                                                                    "duration": {
                                                                        "default": 0,
                                                                        "description": "Exposure time of the TAB. 0 = observation duration.",
                                                                        "maximum": 0,
                                                                        "minimum": 0,
                                                                        "title": "Duration",
                                                                        "type": "integer"
                                                                    },
                                                                    "pointing": {
                                                                        "$ref": "#/definitions/pointing",
                                                                        "default": {}
                                                                    },
                                                                    "relative": {
                                                                        "default": false,
                                                                        "description": "Angular offset applied to the TAB pointing",
                                                                        "title": "Relative to SAP",
                                                                        "type": "boolean"
                                                                    }
                                                                },
                                                                "required": [
                                                                    "pointing",
                                                                    "relative"
                                                                ],
                                                                "title": "Tied-Array Beam",
                                                                "type": "object"
                                                            },
                                                            "title": "Tied-Array Beams",
                                                            "type": "array"
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "tabs"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "SAPs",
                                            "settings"
                                        ],
                                        "title": "Coherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "flys eye": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "description": "Produce beams containing the individual station signals",
                                        "properties": {
                                            "enabled": {
                                                "default": false,
                                                "title": "Enable Fly's Eye",
                                                "type": "boolean"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "enabled"
                                        ],
                                        "title": "Fly's Eye Settings",
                                        "type": "object"
                                    },
                                    "incoherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to create incoherent TABs for (empty list = all).",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "default": {},
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "subbands"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "settings",
                                            "SAPs"
                                        ],
                                        "title": "Incoherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "name": {
                                        "default": "_beamformer_name_",
                                        "description": "Beamformer name, used for identification purposes.",
                                        "minLength": 1,
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501"
                                                ]
                                            }
                                        ],
                                        "description": "While observing, COBALT will beamform on the intersection of all stations in this list and the used stations in the observation. So, specifying all possible stations here means that all observation-stations are used. Specifying a small subset here means that only the observing-stations in this small list are used. By default we let COBALT beamform on the Core stations.",
                                        "minItems": 1
                                    }
                                },
                                "required": [
                                    "name"
                                ],
                                "title": "Beamformer Pipeline",
                                "type": "object"
                            },
                            "subband_selection": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "list": {
                                        "additionalItems": false,
                                        "default": [],
                                        "description": "If method is 'select subset', only beamform these subbands, and only if they occur in the SAP.",
                                        "items": {
                                            "maximum": 511,
                                            "minimum": 0,
                                            "title": "Subband",
                                            "type": "integer"
                                        },
                                        "minItems": 0,
                                        "title": "Subset selection",
                                        "type": "array"
                                    },
                                    "method": {
                                        "default": "copy",
                                        "description": "How to select the subbands to beam form",
                                        "enum": [
                                            "copy",
                                            "largest continuous subset",
                                            "select subset"
                                        ],
                                        "title": "Method",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "method"
                                ],
                                "title": "Subband selection",
                                "type": "object"
                            },
                            "station_list": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [],
                                "items": {
                                    "$ref": "#/definitions/station"
                                },
                                "minItems": 0,
                                "type": "array",
                                "uniqueItems": true
                            },
                            "pointing": {
                                "additionalProperties": false,
                                "properties": {
                                    "angle1": {
                                        "default": 0.6624317181687094,
                                        "description": "First angle (e.g. RA)",
                                        "title": "Angle 1",
                                        "type": "number"
                                    },
                                    "angle2": {
                                        "default": 1.5579526427549426,
                                        "description": "Second angle (e.g. DEC)",
                                        "title": "Angle 2",
                                        "type": "number"
                                    },
                                    "direction_type": {
                                        "default": "J2000",
                                        "description": "",
                                        "enum": [
                                            "J2000",
                                            "AZELGEO",
                                            "LMN",
                                            "SUN",
                                            "MOON",
                                            "MERCURY",
                                            "VENUS",
                                            "MARS",
                                            "JUPITER",
                                            "SATURN",
                                            "URANUS",
                                            "NEPTUNE",
                                            "PLUTO"
                                        ],
                                        "title": "Reference frame",
                                        "type": "string"
                                    },
                                    "target": {
                                        "default": "_target_name_",
                                        "description": "Description of where this beam points at",
                                        "minLength": 1,
                                        "title": "Target",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "angle1",
                                    "angle2",
                                    "target"
                                ],
                                "type": "object"
                            },
                            "stokes_settings": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "channels_per_subband": {
                                        "default": 1,
                                        "description": "Number of frequency bands per subband",
                                        "enum": [
                                            1,
                                            8,
                                            16,
                                            32,
                                            64,
                                            128,
                                            256,
                                            512,
                                            1024
                                        ],
                                        "minimum": 1,
                                        "title": "Channels/subband",
                                        "type": "integer"
                                    },
                                    "quantisation": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "bits": {
                                                "default": 8,
                                                "description": "Number of bits for a single value",
                                                "enum": [
                                                    8
                                                ],
                                                "title": "Bits/sample",
                                                "type": "integer"
                                            },
                                            "enabled": {
                                                "default": false,
                                                "title": "Output quantisation into integers",
                                                "type": "boolean"
                                            },
                                            "scale_max": {
                                                "default": 5,
                                                "description": "Cut off values above this treshold",
                                                "title": "Maximum value",
                                                "type": "number"
                                            },
                                            "scale_min": {
                                                "default": -5,
                                                "description": "Cut off values below this treshold",
                                                "title": "Minimum value",
                                                "type": "number"
                                            }
                                        },
                                        "required": [
                                            "enabled",
                                            "bits",
                                            "scale_min",
                                            "scale_max"
                                        ],
                                        "title": "Output quantisation settings",
                                        "type": "object"
                                    },
                                    "stokes": {
                                        "$ref": "#/definitions/stokes",
                                        "default": "I"
                                    },
                                    "subbands_per_file": {
                                        "default": 488,
                                        "description": "The maximum number of subbands to write in each output dataproduct.",
                                        "maximum": 488,
                                        "minimum": 1,
                                        "title": "Subbands per file",
                                        "type": "integer"
                                    },
                                    "time_integration_factor": {
                                        "default": 1,
                                        "description": "The number of samples to integrate over",
                                        "maximum": 12288,
                                        "minimum": 1,
                                        "title": "Time integration",
                                        "type": "integer"
                                    }
                                },
                                "required": [
                                    "stokes",
                                    "time_integration_factor",
                                    "channels_per_subband"
                                ],
                                "type": "object"
                            },
                            "station": {
                                "description": "These are the LOFAR stations",
                                "enum": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501",
                                    "RS106",
                                    "RS205",
                                    "RS208",
                                    "RS210",
                                    "RS305",
                                    "RS306",
                                    "RS307",
                                    "RS310",
                                    "RS406",
                                    "RS407",
                                    "RS409",
                                    "RS503",
                                    "RS508",
                                    "RS509",
                                    "DE601",
                                    "DE602",
                                    "DE603",
                                    "DE604",
                                    "DE605",
                                    "FR606",
                                    "SE607",
                                    "UK608",
                                    "DE609",
                                    "PL610",
                                    "PL611",
                                    "PL612",
                                    "IE613",
                                    "LV614"
                                ],
                                "title": "Station",
                                "type": "string"
                            },
                            "max_number_of_missing_stations": {
                                "default": 0,
                                "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                                "minimum": 0,
                                "title": "Maximum number of stations to omit",
                                "type": "integer"
                            },
                            "stokes": {
                                "default": "I",
                                "description": "Which Stokes to produce",
                                "enum": [
                                    "I",
                                    "IQUV",
                                    "XXYY"
                                ],
                                "title": "Stokes",
                                "type": "string"
                            },
                            "station_groups": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [
                                    {
                                        "max_nr_missing": 1,
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ]
                                    }
                                ],
                                "description": "One or more predefined or custom groups of stations",
                                "items": {
                                    "$ref": "#/definitions/station_group"
                                },
                                "minItems": 1,
                                "title": "Station groups",
                                "type": "array"
                            },
                            "station_group": {
                                "anyOf": [
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007"
                                            ]
                                        },
                                        "description": "The group of all stations on the Superterp",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Superterp",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501"
                                            ]
                                        },
                                        "description": "The group of all Core stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Core",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch remote stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Remote",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch (Core + Remote) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Dutch",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 2,
                                            "stations": [
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 1,
                                            "stations": [
                                                "DE601",
                                                "DE605"
                                            ]
                                        },
                                        "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE605"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International required",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 6,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509",
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all (Core + Remote + International) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509",
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "All",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS001"
                                            ]
                                        },
                                        "description": "A custom group of stations which can be defined by the user",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list"
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Custom",
                                        "type": "object"
                                    }
                                ],
                                "default": {
                                    "max_nr_missing": 1,
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                },
                                "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                                "title": "Station group",
                                "type": "object"
                            },
                            "station_configuration": {
                                "default": {},
                                "properties": {
                                    "SAPs": {
                                        "$ref": "#/definitions/SAPs",
                                        "default": [
                                            {}
                                        ],
                                        "minItems": 1
                                    },
                                    "antenna_set": {
                                        "$ref": "#/definitions/antenna_set",
                                        "default": "HBA_DUAL"
                                    },
                                    "filter": {
                                        "$ref": "#/definitions/filter",
                                        "default": "HBA_110_190"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007"
                                                ]
                                            }
                                        ]
                                    },
                                    "tile_beam": {
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "description": "HBA only",
                                        "title": "Tile beam"
                                    }
                                },
                                "required": [
                                    "station_groups",
                                    "antenna_set",
                                    "filter",
                                    "SAPs"
                                ],
                                "title": "station_configuration",
                                "type": "object"
                            },
                            "filter": {
                                "default": "HBA_110_190",
                                "description": "Must match antenna type",
                                "enum": [
                                    "LBA_10_70",
                                    "LBA_30_70",
                                    "LBA_10_90",
                                    "LBA_30_90",
                                    "HBA_110_190",
                                    "HBA_210_250"
                                ],
                                "title": "Band-pass filter",
                                "type": "string"
                            },
                            "antenna_set": {
                                "default": "HBA_DUAL",
                                "description": "Fields & antennas to use",
                                "enum": [
                                    "HBA_DUAL",
                                    "HBA_DUAL_INNER",
                                    "HBA_ONE",
                                    "HBA_ONE_INNER",
                                    "HBA_ZERO",
                                    "HBA_ZERO_INNER",
                                    "LBA_INNER",
                                    "LBA_OUTER",
                                    "LBA_SPARSE_EVEN",
                                    "LBA_SPARSE_ODD",
                                    "LBA_ALL"
                                ],
                                "title": "Antenna set",
                                "type": "string"
                            },
                            "SAPs": {
                                "additionalItems": false,
                                "default": [
                                    {}
                                ],
                                "description": "Station beams",
                                "items": {
                                    "additionalProperties": false,
                                    "default": {},
                                    "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                    "properties": {
                                        "digital_pointing": {
                                            "$id": "#target_pointing",
                                            "$ref": "#/definitions/pointing",
                                            "default": {},
                                            "title": "Digital pointing"
                                        },
                                        "name": {
                                            "default": "_SAP_name_",
                                            "description": "Identifier for this beam",
                                            "minLength": 1,
                                            "title": "Name",
                                            "type": "string"
                                        },
                                        "subbands": {
                                            "additionalItems": false,
                                            "default": [
                                                255
                                            ],
                                            "items": {
                                                "maximum": 511,
                                                "minimum": 0,
                                                "title": "Subband",
                                                "type": "integer"
                                            },
                                            "maxItems": 488,
                                            "minItems": 1,
                                            "title": "Subband list",
                                            "type": "array"
                                        }
                                    },
                                    "required": [
                                        "name",
                                        "digital_pointing",
                                        "subbands"
                                    ],
                                    "title": "SAP",
                                    "type": "object"
                                },
                                "minItems": 0,
                                "title": "SAPs",
                                "type": "array"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/beamforming%20observation/2#",
                        "type": "object",
                        "title": "beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "default": {},
                        "version": 2,
                        "required": [
                            "beamformer",
                            "duration",
                            "station_configuration"
                        ],
                        "properties": {
                            "duration": {
                                "$id": "#duration",
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                "title": "Duration",
                                "default": 300,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/2#/definitions/beamformer",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema defines the parameters for an observation that forms tied-array beams in COBALT.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/observation",
                    "type_value": "observation",
                    "updated_at": "2022-04-21T08:15:52.853846",
                    "version": 2
                },
                "specifications_template_id": 3,
                "status": "scheduled",
                "status_value": "scheduled",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000072"
                ],
                "subtasks_ids": [
                    2000072
                ],
                "tags": [],
                "task_type": "observation",
                "unschedulable_reason": "",
                "updated_at": "2022-04-21T08:16:32.722481"
            }
        ],
        "task_blueprints_ids": [
            53
        ],
        "unschedulable_reason": "",
        "updated_at": "2022-04-21T08:16:32.661348"
    },
    {
        "id": 15,
        "url": "http://localhost:3000/api/scheduling_unit_blueprint/15",
        "actual_on_sky_start_time": null,
        "actual_on_sky_stop_time": null,
        "actual_process_start_time": null,
        "actual_process_stop_time": null,
        "created_at": "2022-04-26T11:43:13.199356",
        "description": "OSR-01",
        "draft": "http://localhost:3000/api/scheduling_unit_draft/15",
        "draft_id": 15,
        "duration": 1200,
        "error_reason": "",
        "global_identifier": "http://localhost:3000/api/sip_identifier/8777",
        "global_identifier_id": 8777,
        "ingest_permission_granted_since": null,
        "ingest_permission_required": true,
        "interrupts_telescope": false,
        "name": "OSR-01",
        "observed_duration": null,
        "observed_start_time": null,
        "observed_stop_time": null,
        "obsolete_since": null,
        "on_sky_duration": 600,
        "on_sky_start_time": "2022-04-22T07:18:00",
        "on_sky_stop_time": "2022-04-22T07:28:00",
        "output_pinned": false,
        "piggyback_allowed_aartfaac": true,
        "piggyback_allowed_tbb": true,
        "priority_queue": "http://localhost:3000/api/priority_queue_type/A",
        "priority_queue_value": "A",
        "process_start_time": "2022-04-22T07:18:00",
        "process_stop_time": "2022-04-22T07:38:00",
        "rank": 0,
        "scheduled_start_time": "2022-04-22T07:18:00",
        "scheduled_stop_time": "2022-04-22T07:38:00",
        "scheduling_constraints_doc": {
            "sky": {
                "min_distance": {
                    "sun": 0,
                    "moon": 0,
                    "jupiter": 0
                },
                "min_elevation": {
                    "target": 0.10000736613927509,
                    "calibrator": 0.5000368306963754
                },
                "transit_offset": {
                    "to": 21600,
                    "from": -21600
                }
            },
            "time": {
                "between": [],
                "not_between": []
            },
            "daily": {
                "require_day": false,
                "require_night": false,
                "avoid_twilight": false
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
            "scheduler": "dynamic"
        },
        "scheduling_constraints_template": "http://localhost:3000/api/scheduling_constraints_template/1",
        "scheduling_constraints_template_id": 1,
        "specifications_template": "http://localhost:3000/api/scheduling_unit_template/1",
        "specifications_template_id": 1,
        "status": "schedulable",
        "status_value": "schedulable",
        "tags": [],
        "task_blueprints": [
            {
                "id": 68,
                "url": "http://localhost:3000/api/task_blueprint/68",
                "actual_on_sky_start_time": null,
                "actual_on_sky_stop_time": null,
                "actual_process_start_time": null,
                "actual_process_stop_time": null,
                "consumed_by": [],
                "consumed_by_ids": [],
                "created_at": "2022-04-26T11:43:13.243456",
                "description": "IM BF Cal&Tgt",
                "draft": "http://localhost:3000/api/task_draft/68",
                "draft_id": 68,
                "duration": 1200,
                "error_reason": "",
                "first_scheduling_relation": [],
                "first_scheduling_relation_ids": [],
                "name": "Observation",
                "obsolete_since": null,
                "on_sky_duration": 600,
                "on_sky_start_time": "2022-04-22T07:18:00",
                "on_sky_stop_time": "2022-04-22T07:28:00",
                "output_pinned": false,
                "process_start_time": "2022-04-22T07:18:00",
                "process_stop_time": "2022-04-22T07:38:00",
                "produced_by": [],
                "produced_by_ids": [],
                "relative_start_time": 0,
                "relative_stop_time": 600,
                "scheduled_start_time": "2022-04-22T07:18:00",
                "scheduled_stop_time": "2022-04-22T07:38:00",
                "scheduling_unit_blueprint": "http://localhost:3000/api/scheduling_unit_blueprint/15",
                "scheduling_unit_blueprint_id": 15,
                "second_scheduling_relation": [],
                "second_scheduling_relation_ids": [],
                "short_description": "",
                "specifications_doc": {
                    "QA": {
                        "plots": {
                            "enabled": true,
                            "autocorrelation": true,
                            "crosscorrelation": true
                        },
                        "file_conversion": {
                            "enabled": true,
                            "nr_of_subbands": -1,
                            "nr_of_timestamps": 256
                        },
                        "inspection_plots": "msplots"
                    },
                    "$schema": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20and%20beamforming%20observation/2/ref_resolved",
                    "duration": 600,
                    "beamformer": {
                        "ppf": false,
                        "pipelines": [
                            {
                                "name": "_beamformer_name_",
                                "coherent": {
                                    "SAPs": [],
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "flys eye": {
                                    "enabled": false,
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "incoherent": {
                                    "SAPs": [],
                                    "settings": {
                                        "stokes": "I",
                                        "quantisation": {
                                            "bits": 8,
                                            "enabled": false,
                                            "scale_max": 5,
                                            "scale_min": -5
                                        },
                                        "subbands_per_file": 488,
                                        "channels_per_subband": 1,
                                        "time_integration_factor": 1
                                    }
                                },
                                "station_groups": [
                                    {
                                        "stations": [
                                            "CS001",
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007",
                                            "CS011",
                                            "CS013",
                                            "CS017",
                                            "CS021",
                                            "CS024",
                                            "CS026",
                                            "CS028",
                                            "CS030",
                                            "CS031",
                                            "CS032",
                                            "CS301",
                                            "CS302",
                                            "CS401",
                                            "CS501"
                                        ],
                                        "max_nr_missing": 1
                                    }
                                ]
                            }
                        ]
                    },
                    "calibrator": {
                        "name": "calibrator",
                        "pointing": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "_target_name_",
                            "direction_type": "J2000"
                        },
                        "autoselect": true
                    },
                    "correlator": {
                        "storage_cluster": "CEP4",
                        "integration_time": 1,
                        "channels_per_subband": 64,
                        "topocentric_frequency_correction": false
                    },
                    "station_configuration": {
                        "SAPs": [
                            {
                                "name": "_SAP_name_",
                                "subbands": [
                                    255
                                ],
                                "digital_pointing": {
                                    "angle1": 0.6624317181687094,
                                    "angle2": 1.5579526427549426,
                                    "target": "_target_name_",
                                    "direction_type": "J2000"
                                }
                            }
                        ],
                        "filter": "HBA_110_190",
                        "tile_beam": {
                            "angle1": 0.6624317181687094,
                            "angle2": 1.5579526427549426,
                            "target": "_target_name_",
                            "direction_type": "J2000"
                        },
                        "antenna_set": "HBA_DUAL",
                        "station_groups": [
                            {
                                "stations": [
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007"
                                ],
                                "max_nr_missing": 1
                            }
                        ]
                    }
                },
                "specifications_template": {
                    "id": 6,
                    "url": "http://localhost:3000/api/task_template/6",
                    "created_at": "2022-04-21T08:15:53.120162",
                    "description": "This schema combines the calibrator, target and beamformer observation task schema's for parallel observing.",
                    "name": "parallel calibrator target and beamforming observation",
                    "purpose": "http://localhost:3000/api/template_purpose/technical_commissioning",
                    "purpose_value": "technical_commissioning",
                    "ref_resolved_schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20and%20beamforming%20observation/2/ref_resolved",
                        "type": "object",
                        "title": "parallel calibrator target and beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "calibrator",
                            "correlator",
                            "duration",
                            "station_configuration",
                            "beamformer"
                        ],
                        "properties": {
                            "QA": {
                                "$ref": "#/definitions/QA",
                                "default": {}
                            },
                            "duration": {
                                "$id": "#duration",
                                "$ref": "#/definitions/timedelta",
                                "title": "Duration",
                                "default": 600,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "#/definitions/beamformer",
                                "default": {}
                            },
                            "calibrator": {
                                "$ref": "#/definitions/calibrator",
                                "default": {}
                            },
                            "correlator": {
                                "$ref": "#/definitions/correlator",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema combines the calibrator, target and beamformer observation task schema's for parallel observing.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false,
                        "definitions": {
                            "QA": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Perform all Quality Assurance (QA) tasks, including file conversion and plotting.",
                                "properties": {
                                    "file_conversion": {
                                        "$ref": "#/definitions/file_conversion",
                                        "default": {}
                                    },
                                    "inspection_plots": {
                                        "$ref": "#/definitions/inspection_plots",
                                        "default": "msplots"
                                    },
                                    "plots": {
                                        "$ref": "#/definitions/plots",
                                        "default": {}
                                    }
                                },
                                "required": [
                                    "file_conversion",
                                    "plots",
                                    "inspection_plots"
                                ],
                                "title": "QA",
                                "type": "object"
                            },
                            "file_conversion": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Create a QA file for the observation",
                                "properties": {
                                    "enabled": {
                                        "default": true,
                                        "description": "Do/Don't create a QA file for the observation",
                                        "title": "enabled",
                                        "type": "boolean"
                                    },
                                    "nr_of_subbands": {
                                        "default": -1,
                                        "description": "Keep this number of subbands from the observation in the QA file, or all if -1",
                                        "title": "#subbands",
                                        "type": "integer"
                                    },
                                    "nr_of_timestamps": {
                                        "default": 256,
                                        "description": "Extract this number of timestamps from the observation in the QA file (equidistantanly sampled, no averaging/interpolation)",
                                        "minimum": 1,
                                        "title": "#timestamps",
                                        "type": "integer"
                                    }
                                },
                                "required": [
                                    "enabled",
                                    "nr_of_subbands",
                                    "nr_of_timestamps"
                                ],
                                "title": "File Conversion",
                                "type": "object"
                            },
                            "plots": {
                                "additionalProperties": false,
                                "default": {},
                                "description": "Create dynamic spectrum plots",
                                "properties": {
                                    "autocorrelation": {
                                        "default": true,
                                        "description": "Create autocorrelation plots for all stations",
                                        "title": "autocorrelation",
                                        "type": "boolean"
                                    },
                                    "crosscorrelation": {
                                        "default": true,
                                        "description": "Create crosscorrelation plots for all baselines",
                                        "title": "crosscorrelation",
                                        "type": "boolean"
                                    },
                                    "enabled": {
                                        "default": true,
                                        "description": "Do/Don't create plots from the QA file from the observation",
                                        "title": "enabled",
                                        "type": "boolean"
                                    }
                                },
                                "required": [
                                    "enabled",
                                    "autocorrelation",
                                    "crosscorrelation"
                                ],
                                "title": "Plots",
                                "type": "object"
                            },
                            "inspection_plots": {
                                "decription": "Type of inspection plots to run",
                                "default": "msplots",
                                "enum": [
                                    "msplots",
                                    "dynspec",
                                    "none"
                                ],
                                "title": "Inspection Plots",
                                "type": "string"
                            },
                            "timedelta": {
                                "default": 0,
                                "description": "A time duration or delta expressed in seconds",
                                "type": "number"
                            },
                            "timestamp": {
                                "default": "1970-01-01T00:00:00Z",
                                "description": "A timestamp defined in UTC",
                                "format": "date-time",
                                "pattern": "\\d{4}-[01]\\d-[0-3]\\dT[0-2]\\d:[0-5]\\d:[0-5]\\d(\\.\\d+)?Z?",
                                "type": "string"
                            },
                            "beamformer": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "pipelines": {
                                        "additionalItems": false,
                                        "default": [
                                            {}
                                        ],
                                        "description": "This array of beamformer pipelines is currently restricted to exactly 1 item. Support for handling multiple pipelines (in cobalt) will be added in the future.",
                                        "items": {
                                            "$ref": "#/definitions/beamformer_pipeline"
                                        },
                                        "maxItems": 1,
                                        "minItems": 1,
                                        "title": "Pipelines",
                                        "type": "array"
                                    },
                                    "ppf": {
                                        "default": false,
                                        "description": "Use a PPF for the channellisation for beamforming. If not, an FFT is used.",
                                        "title": "Use PPF when beamforming",
                                        "type": "boolean"
                                    }
                                },
                                "required": [
                                    "pipelines",
                                    "ppf"
                                ],
                                "title": "Beamformer",
                                "type": "object"
                            },
                            "beamformer_pipeline": {
                                "additionalProperties": false,
                                "default": {},
                                "headerTemplate": "Beamformer Pipeline {{ self.index }}",
                                "properties": {
                                    "coherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to beamform.",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        },
                                                        "tab_rings": {
                                                            "additonalProperties": false,
                                                            "default": {},
                                                            "description": "Rings of TABs around the center of the beam.",
                                                            "properties": {
                                                                "count": {
                                                                    "default": 0,
                                                                    "maximum": 11,
                                                                    "minimum": 0,
                                                                    "title": "Number of rings",
                                                                    "type": "integer"
                                                                },
                                                                "width": {
                                                                    "default": 0.01,
                                                                    "description": "Distance between pointings.",
                                                                    "minimum": 0,
                                                                    "title": "Ring width",
                                                                    "type": "number"
                                                                }
                                                            },
                                                            "title": "Tied-Array Rings",
                                                            "type": "object"
                                                        },
                                                        "tabs": {
                                                            "additionalItems": false,
                                                            "default": [],
                                                            "description": "Tied-array beams to form",
                                                            "items": {
                                                                "additionalProperties": false,
                                                                "default": {},
                                                                "headerTemplate": "TAB {{ self.index }}",
                                                                "properties": {
                                                                    "allow_after_previous": {
                                                                        "default": true,
                                                                        "description": "This TAB can be recorded after the previous TAB, if duration allows.",
                                                                        "title": "Allow after previous",
                                                                        "type": "boolean"
                                                                    },
                                                                    "duration": {
                                                                        "default": 0,
                                                                        "description": "Exposure time of the TAB. 0 = observation duration.",
                                                                        "maximum": 0,
                                                                        "minimum": 0,
                                                                        "title": "Duration",
                                                                        "type": "integer"
                                                                    },
                                                                    "pointing": {
                                                                        "$ref": "#/definitions/pointing",
                                                                        "default": {}
                                                                    },
                                                                    "relative": {
                                                                        "default": false,
                                                                        "description": "Angular offset applied to the TAB pointing",
                                                                        "title": "Relative to SAP",
                                                                        "type": "boolean"
                                                                    }
                                                                },
                                                                "required": [
                                                                    "pointing",
                                                                    "relative"
                                                                ],
                                                                "title": "Tied-Array Beam",
                                                                "type": "object"
                                                            },
                                                            "title": "Tied-Array Beams",
                                                            "type": "array"
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "tabs"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "SAPs",
                                            "settings"
                                        ],
                                        "title": "Coherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "flys eye": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "description": "Produce beams containing the individual station signals",
                                        "properties": {
                                            "enabled": {
                                                "default": false,
                                                "title": "Enable Fly's Eye",
                                                "type": "boolean"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "enabled"
                                        ],
                                        "title": "Fly's Eye Settings",
                                        "type": "object"
                                    },
                                    "incoherent": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "SAPs": {
                                                "additionalItems": false,
                                                "default": [],
                                                "description": "Which SAPs in the observation to create incoherent TABs for (empty list = all).",
                                                "items": {
                                                    "additionalProperties": false,
                                                    "default": {},
                                                    "properties": {
                                                        "name": {
                                                            "default": "_SAP_to_beamform_",
                                                            "description": "Name of the SAP to beamform",
                                                            "minLength": 1,
                                                            "title": "SAP name",
                                                            "type": "string"
                                                        },
                                                        "subbands": {
                                                            "$ref": "#/definitions/subband_selection",
                                                            "default": {}
                                                        }
                                                    },
                                                    "required": [
                                                        "name",
                                                        "subbands"
                                                    ],
                                                    "type": "object"
                                                },
                                                "minItems": 0,
                                                "title": "SAPs",
                                                "type": "array"
                                            },
                                            "settings": {
                                                "$ref": "#/definitions/stokes_settings",
                                                "default": {}
                                            }
                                        },
                                        "required": [
                                            "settings",
                                            "SAPs"
                                        ],
                                        "title": "Incoherent Tied-Array Beams",
                                        "type": "object"
                                    },
                                    "name": {
                                        "default": "_beamformer_name_",
                                        "description": "Beamformer name, used for identification purposes.",
                                        "minLength": 1,
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS001",
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007",
                                                    "CS011",
                                                    "CS013",
                                                    "CS017",
                                                    "CS021",
                                                    "CS024",
                                                    "CS026",
                                                    "CS028",
                                                    "CS030",
                                                    "CS031",
                                                    "CS032",
                                                    "CS301",
                                                    "CS302",
                                                    "CS401",
                                                    "CS501"
                                                ]
                                            }
                                        ],
                                        "description": "While observing, COBALT will beamform on the intersection of all stations in this list and the used stations in the observation. So, specifying all possible stations here means that all observation-stations are used. Specifying a small subset here means that only the observing-stations in this small list are used. By default we let COBALT beamform on the Core stations.",
                                        "minItems": 1
                                    }
                                },
                                "required": [
                                    "name"
                                ],
                                "title": "Beamformer Pipeline",
                                "type": "object"
                            },
                            "subband_selection": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "list": {
                                        "additionalItems": false,
                                        "default": [],
                                        "description": "If method is 'select subset', only beamform these subbands, and only if they occur in the SAP.",
                                        "items": {
                                            "maximum": 511,
                                            "minimum": 0,
                                            "title": "Subband",
                                            "type": "integer"
                                        },
                                        "minItems": 0,
                                        "title": "Subset selection",
                                        "type": "array"
                                    },
                                    "method": {
                                        "default": "copy",
                                        "description": "How to select the subbands to beam form",
                                        "enum": [
                                            "copy",
                                            "largest continuous subset",
                                            "select subset"
                                        ],
                                        "title": "Method",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "method"
                                ],
                                "title": "Subband selection",
                                "type": "object"
                            },
                            "station_list": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [],
                                "items": {
                                    "$ref": "#/definitions/station"
                                },
                                "minItems": 0,
                                "type": "array",
                                "uniqueItems": true
                            },
                            "pointing": {
                                "additionalProperties": false,
                                "properties": {
                                    "angle1": {
                                        "default": 0.6624317181687094,
                                        "description": "First angle (e.g. RA)",
                                        "title": "Angle 1",
                                        "type": "number"
                                    },
                                    "angle2": {
                                        "default": 1.5579526427549426,
                                        "description": "Second angle (e.g. DEC)",
                                        "title": "Angle 2",
                                        "type": "number"
                                    },
                                    "direction_type": {
                                        "default": "J2000",
                                        "description": "",
                                        "enum": [
                                            "J2000",
                                            "AZELGEO",
                                            "LMN",
                                            "SUN",
                                            "MOON",
                                            "MERCURY",
                                            "VENUS",
                                            "MARS",
                                            "JUPITER",
                                            "SATURN",
                                            "URANUS",
                                            "NEPTUNE",
                                            "PLUTO"
                                        ],
                                        "title": "Reference frame",
                                        "type": "string"
                                    },
                                    "target": {
                                        "default": "_target_name_",
                                        "description": "Description of where this beam points at",
                                        "minLength": 1,
                                        "title": "Target",
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "angle1",
                                    "angle2",
                                    "target"
                                ],
                                "type": "object"
                            },
                            "stokes_settings": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "channels_per_subband": {
                                        "default": 1,
                                        "description": "Number of frequency bands per subband",
                                        "enum": [
                                            1,
                                            8,
                                            16,
                                            32,
                                            64,
                                            128,
                                            256,
                                            512,
                                            1024
                                        ],
                                        "minimum": 1,
                                        "title": "Channels/subband",
                                        "type": "integer"
                                    },
                                    "quantisation": {
                                        "additionalProperties": false,
                                        "default": {},
                                        "properties": {
                                            "bits": {
                                                "default": 8,
                                                "description": "Number of bits for a single value",
                                                "enum": [
                                                    8
                                                ],
                                                "title": "Bits/sample",
                                                "type": "integer"
                                            },
                                            "enabled": {
                                                "default": false,
                                                "title": "Output quantisation into integers",
                                                "type": "boolean"
                                            },
                                            "scale_max": {
                                                "default": 5,
                                                "description": "Cut off values above this treshold",
                                                "title": "Maximum value",
                                                "type": "number"
                                            },
                                            "scale_min": {
                                                "default": -5,
                                                "description": "Cut off values below this treshold",
                                                "title": "Minimum value",
                                                "type": "number"
                                            }
                                        },
                                        "required": [
                                            "enabled",
                                            "bits",
                                            "scale_min",
                                            "scale_max"
                                        ],
                                        "title": "Output quantisation settings",
                                        "type": "object"
                                    },
                                    "stokes": {
                                        "$ref": "#/definitions/stokes",
                                        "default": "I"
                                    },
                                    "subbands_per_file": {
                                        "default": 488,
                                        "description": "The maximum number of subbands to write in each output dataproduct.",
                                        "maximum": 488,
                                        "minimum": 1,
                                        "title": "Subbands per file",
                                        "type": "integer"
                                    },
                                    "time_integration_factor": {
                                        "default": 1,
                                        "description": "The number of samples to integrate over",
                                        "maximum": 12288,
                                        "minimum": 1,
                                        "title": "Time integration",
                                        "type": "integer"
                                    }
                                },
                                "required": [
                                    "stokes",
                                    "time_integration_factor",
                                    "channels_per_subband"
                                ],
                                "type": "object"
                            },
                            "station": {
                                "description": "These are the LOFAR stations",
                                "enum": [
                                    "CS001",
                                    "CS002",
                                    "CS003",
                                    "CS004",
                                    "CS005",
                                    "CS006",
                                    "CS007",
                                    "CS011",
                                    "CS013",
                                    "CS017",
                                    "CS021",
                                    "CS024",
                                    "CS026",
                                    "CS028",
                                    "CS030",
                                    "CS031",
                                    "CS032",
                                    "CS101",
                                    "CS103",
                                    "CS201",
                                    "CS301",
                                    "CS302",
                                    "CS401",
                                    "CS501",
                                    "RS106",
                                    "RS205",
                                    "RS208",
                                    "RS210",
                                    "RS305",
                                    "RS306",
                                    "RS307",
                                    "RS310",
                                    "RS406",
                                    "RS407",
                                    "RS409",
                                    "RS503",
                                    "RS508",
                                    "RS509",
                                    "DE601",
                                    "DE602",
                                    "DE603",
                                    "DE604",
                                    "DE605",
                                    "FR606",
                                    "SE607",
                                    "UK608",
                                    "DE609",
                                    "PL610",
                                    "PL611",
                                    "PL612",
                                    "IE613",
                                    "LV614"
                                ],
                                "title": "Station",
                                "type": "string"
                            },
                            "max_number_of_missing_stations": {
                                "default": 0,
                                "description": "Maximum number of stations that can be omitted from a group (due to maintenance for example)",
                                "minimum": 0,
                                "title": "Maximum number of stations to omit",
                                "type": "integer"
                            },
                            "stokes": {
                                "default": "I",
                                "description": "Which Stokes to produce",
                                "enum": [
                                    "I",
                                    "IQUV",
                                    "XXYY"
                                ],
                                "title": "Stokes",
                                "type": "string"
                            },
                            "station_groups": {
                                "additionalItems": false,
                                "additionalProperties": false,
                                "default": [
                                    {
                                        "max_nr_missing": 1,
                                        "stations": [
                                            "CS002",
                                            "CS003",
                                            "CS004",
                                            "CS005",
                                            "CS006",
                                            "CS007"
                                        ]
                                    }
                                ],
                                "description": "One or more predefined or custom groups of stations",
                                "items": {
                                    "$ref": "#/definitions/station_group"
                                },
                                "minItems": 1,
                                "title": "Station groups",
                                "type": "array"
                            },
                            "station_group": {
                                "anyOf": [
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007"
                                            ]
                                        },
                                        "description": "The group of all stations on the Superterp",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Superterp",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501"
                                            ]
                                        },
                                        "description": "The group of all Core stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Core",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch remote stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Remote",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 4,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509"
                                            ]
                                        },
                                        "description": "The group of all Dutch (Core + Remote) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Dutch",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 2,
                                            "stations": [
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 1,
                                            "stations": [
                                                "DE601",
                                                "DE605"
                                            ]
                                        },
                                        "description": "A subgroup of the international stations which are required when doing observation with international stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "DE601",
                                                        "DE605"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "International required",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 6,
                                            "stations": [
                                                "CS001",
                                                "CS002",
                                                "CS003",
                                                "CS004",
                                                "CS005",
                                                "CS006",
                                                "CS007",
                                                "CS011",
                                                "CS013",
                                                "CS017",
                                                "CS021",
                                                "CS024",
                                                "CS026",
                                                "CS028",
                                                "CS030",
                                                "CS031",
                                                "CS032",
                                                "CS101",
                                                "CS103",
                                                "CS201",
                                                "CS301",
                                                "CS302",
                                                "CS401",
                                                "CS501",
                                                "RS106",
                                                "RS205",
                                                "RS208",
                                                "RS210",
                                                "RS305",
                                                "RS306",
                                                "RS307",
                                                "RS310",
                                                "RS406",
                                                "RS407",
                                                "RS409",
                                                "RS503",
                                                "RS508",
                                                "RS509",
                                                "DE601",
                                                "DE602",
                                                "DE603",
                                                "DE604",
                                                "DE605",
                                                "DE609",
                                                "FR606",
                                                "SE607",
                                                "UK608",
                                                "PL610",
                                                "PL611",
                                                "PL612",
                                                "IE613",
                                                "LV614"
                                            ]
                                        },
                                        "description": "The group of all (Core + Remote + International) stations",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list",
                                                "enum": [
                                                    [
                                                        "CS001",
                                                        "CS002",
                                                        "CS003",
                                                        "CS004",
                                                        "CS005",
                                                        "CS006",
                                                        "CS007",
                                                        "CS011",
                                                        "CS013",
                                                        "CS017",
                                                        "CS021",
                                                        "CS024",
                                                        "CS026",
                                                        "CS028",
                                                        "CS030",
                                                        "CS031",
                                                        "CS032",
                                                        "CS101",
                                                        "CS103",
                                                        "CS201",
                                                        "CS301",
                                                        "CS302",
                                                        "CS401",
                                                        "CS501",
                                                        "RS106",
                                                        "RS205",
                                                        "RS208",
                                                        "RS210",
                                                        "RS305",
                                                        "RS306",
                                                        "RS307",
                                                        "RS310",
                                                        "RS406",
                                                        "RS407",
                                                        "RS409",
                                                        "RS503",
                                                        "RS508",
                                                        "RS509",
                                                        "DE601",
                                                        "DE602",
                                                        "DE603",
                                                        "DE604",
                                                        "DE605",
                                                        "DE609",
                                                        "FR606",
                                                        "SE607",
                                                        "UK608",
                                                        "PL610",
                                                        "PL611",
                                                        "PL612",
                                                        "IE613",
                                                        "LV614"
                                                    ]
                                                ]
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "All",
                                        "type": "object"
                                    },
                                    {
                                        "additionalProperties": false,
                                        "default": {
                                            "max_nr_missing": 0,
                                            "stations": [
                                                "CS001"
                                            ]
                                        },
                                        "description": "A custom group of stations which can be defined by the user",
                                        "properties": {
                                            "max_nr_missing": {
                                                "$ref": "#/definitions/max_number_of_missing_stations"
                                            },
                                            "stations": {
                                                "$ref": "#/definitions/station_list"
                                            }
                                        },
                                        "required": [
                                            "stations",
                                            "max_nr_missing"
                                        ],
                                        "title": "Custom",
                                        "type": "object"
                                    }
                                ],
                                "default": {
                                    "max_nr_missing": 1,
                                    "stations": [
                                        "CS002",
                                        "CS003",
                                        "CS004",
                                        "CS005",
                                        "CS006",
                                        "CS007"
                                    ]
                                },
                                "description": "A set of predefined list of stations, and a constraint on how many stations are allowed to be missing (due to maintenance for example)",
                                "title": "Station group",
                                "type": "object"
                            },
                            "calibrator": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "autoselect": {
                                        "default": true,
                                        "description": "Auto-select calibrator based on elevation",
                                        "title": "Auto-select",
                                        "type": "boolean"
                                    },
                                    "name": {
                                        "default": "calibrator",
                                        "description": "Name of the calibrator SAP",
                                        "title": "Name",
                                        "type": "string"
                                    },
                                    "pointing": {
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "description": "Manually selected calibrator",
                                        "title": "Digital pointing"
                                    }
                                },
                                "required": [
                                    "autoselect",
                                    "pointing",
                                    "name"
                                ],
                                "type": "object"
                            },
                            "correlator": {
                                "additionalProperties": false,
                                "default": {},
                                "properties": {
                                    "channels_per_subband": {
                                        "default": 64,
                                        "description": "Number of frequency bands per subband",
                                        "enum": [
                                            8,
                                            16,
                                            32,
                                            64,
                                            128,
                                            256,
                                            512,
                                            1024
                                        ],
                                        "minimum": 8,
                                        "title": "Channels/subband",
                                        "type": "integer"
                                    },
                                    "integration_time": {
                                        "$ref": "#/definitions/timedelta",
                                        "default": 1,
                                        "description": "Desired integration period (seconds)",
                                        "minimum": 0.1,
                                        "title": "Integration time"
                                    },
                                    "storage_cluster": {
                                        "default": "CEP4",
                                        "description": "Cluster to write output to",
                                        "enum": [
                                            "CEP4",
                                            "DragNet"
                                        ],
                                        "title": "Storage cluster",
                                        "type": "string"
                                    },
                                    "topocentric_frequency_correction": {
                                        "default": false,
                                        "description": "Correct frequencies for the Doppler shift caused by the Earth's rotation.",
                                        "title": "Topocentric Frequency Correction",
                                        "type": "boolean"
                                    }
                                }
                            },
                            "station_configuration": {
                                "default": {},
                                "properties": {
                                    "SAPs": {
                                        "$ref": "#/definitions/SAPs",
                                        "default": [
                                            {}
                                        ],
                                        "minItems": 1
                                    },
                                    "antenna_set": {
                                        "$ref": "#/definitions/antenna_set",
                                        "default": "HBA_DUAL"
                                    },
                                    "filter": {
                                        "$ref": "#/definitions/filter",
                                        "default": "HBA_110_190"
                                    },
                                    "station_groups": {
                                        "$ref": "#/definitions/station_groups",
                                        "default": [
                                            {
                                                "max_nr_missing": 1,
                                                "stations": [
                                                    "CS002",
                                                    "CS003",
                                                    "CS004",
                                                    "CS005",
                                                    "CS006",
                                                    "CS007"
                                                ]
                                            }
                                        ]
                                    },
                                    "tile_beam": {
                                        "$ref": "#/definitions/pointing",
                                        "default": {},
                                        "description": "HBA only",
                                        "title": "Tile beam"
                                    }
                                },
                                "required": [
                                    "station_groups",
                                    "antenna_set",
                                    "filter",
                                    "SAPs"
                                ],
                                "title": "station_configuration",
                                "type": "object"
                            },
                            "filter": {
                                "default": "HBA_110_190",
                                "description": "Must match antenna type",
                                "enum": [
                                    "LBA_10_70",
                                    "LBA_30_70",
                                    "LBA_10_90",
                                    "LBA_30_90",
                                    "HBA_110_190",
                                    "HBA_210_250"
                                ],
                                "title": "Band-pass filter",
                                "type": "string"
                            },
                            "antenna_set": {
                                "default": "HBA_DUAL",
                                "description": "Fields & antennas to use",
                                "enum": [
                                    "HBA_DUAL",
                                    "HBA_DUAL_INNER",
                                    "HBA_ONE",
                                    "HBA_ONE_INNER",
                                    "HBA_ZERO",
                                    "HBA_ZERO_INNER",
                                    "LBA_INNER",
                                    "LBA_OUTER",
                                    "LBA_SPARSE_EVEN",
                                    "LBA_SPARSE_ODD",
                                    "LBA_ALL"
                                ],
                                "title": "Antenna set",
                                "type": "string"
                            },
                            "SAPs": {
                                "additionalItems": false,
                                "default": [
                                    {}
                                ],
                                "description": "Station beams",
                                "items": {
                                    "additionalProperties": false,
                                    "default": {},
                                    "headerTemplate": "{{ i0 }} - {{ self.name }}",
                                    "properties": {
                                        "digital_pointing": {
                                            "$id": "#target_pointing",
                                            "$ref": "#/definitions/pointing",
                                            "default": {},
                                            "title": "Digital pointing"
                                        },
                                        "name": {
                                            "default": "_SAP_name_",
                                            "description": "Identifier for this beam",
                                            "minLength": 1,
                                            "title": "Name",
                                            "type": "string"
                                        },
                                        "subbands": {
                                            "additionalItems": false,
                                            "default": [
                                                255
                                            ],
                                            "items": {
                                                "maximum": 511,
                                                "minimum": 0,
                                                "title": "Subband",
                                                "type": "integer"
                                            },
                                            "maxItems": 488,
                                            "minItems": 1,
                                            "title": "Subband list",
                                            "type": "array"
                                        }
                                    },
                                    "required": [
                                        "name",
                                        "digital_pointing",
                                        "subbands"
                                    ],
                                    "title": "SAP",
                                    "type": "object"
                                },
                                "minItems": 0,
                                "title": "SAPs",
                                "type": "array"
                            }
                        }
                    },
                    "schema": {
                        "$id": "http://127.0.0.1:8008/api/schemas/tasktemplate/parallel%20calibrator%20target%20and%20beamforming%20observation/2#",
                        "type": "object",
                        "title": "parallel calibrator target and beamforming observation",
                        "$schema": "http://json-schema.org/draft-06/schema#",
                        "version": 2,
                        "required": [
                            "calibrator",
                            "correlator",
                            "duration",
                            "beamformer",
                            "station_configuration",
                        ],
                        "properties": {
                            "QA": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/QA/2#/definitions/QA",
                                "default": {}
                            },
                            "duration": {
                                "$id": "#duration",
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/datetime/2/#/definitions/timedelta",
                                "title": "Duration",
                                "default": 600,
                                "minimum": 1,
                                "description": "Duration of this observation (seconds)"
                            },
                            "beamformer": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/beamforming/2#/definitions/beamformer",
                                "default": {}
                            },
                            "calibrator": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/calibrator/1#/definitions/calibrator",
                                "default": {}
                            },
                            "correlator": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/correlator/1#/definitions/correlator",
                                "default": {}
                            },
                            "station_configuration": {
                                "$ref": "http://127.0.0.1:8008/api/schemas/commonschematemplate/station_configuration/1#/definitions/station_configuration",
                                "default": {}
                            }
                        },
                        "description": "This schema combines the calibrator, target and beamformer observation task schema's for parallel observing.",
                        "patternProperties": {
                            "^[$]schema$": {}
                        },
                        "additionalProperties": false
                    },
                    "state": "http://localhost:3000/api/template_state/active",
                    "state_value": "active",
                    "tags": [],
                    "type": "http://localhost:3000/api/task_type/observation",
                    "type_value": "observation",
                    "updated_at": "2022-04-21T08:15:53.120183",
                    "version": 2
                },
                "specifications_template_id": 6,
                "status": "schedulable",
                "status_value": "schedulable",
                "subtasks": [
                    "http://localhost:3000/api/subtask/2000099",
                    "http://localhost:3000/api/subtask/2000100",
                    "http://localhost:3000/api/subtask/2000101"
                ],
                "subtasks_ids": [
                    2000099,
                    2000100,
                    2000101
                ],
                "tags": [],
                "task_type": "observation",
                "unschedulable_reason": "",
                "updated_at": "2022-04-26T11:43:13.243471"
            }
        ],
        "task_blueprints_ids": [
            68
        ],
        "unschedulable_reason": "",
        "updated_at": "2022-04-26T11:43:13.199439"
    },
    {
        "id": 16,
        "url": "http://localhost:8008/api/scheduling_unit_blueprint/16",
        "actual_on_sky_start_time": null,
        "actual_on_sky_stop_time": null,
        "actual_process_start_time": null,
        "actual_process_stop_time": null,
        "created_at": "2022-05-30T05:49:13.964291",
        "description": "test",
        "draft": "http://localhost:8008/api/scheduling_unit_draft/16",
        "draft_id": 16,
        "duration": 4200.0,
        "error_reason": "",
        "global_identifier": "http://localhost:8008/api/sip_identifier/1",
        "global_identifier_id": 1,
        "ingest_permission_granted_since": null,
        "ingest_permission_required": true,
        "interrupts_telescope": false,
        "name": "testSolar",
        "observed_duration": null,
        "observed_start_time": null,
        "observed_stop_time": null,
        "obsolete_since": null,
        "on_sky_duration": 2400.0,
        "on_sky_start_time": "2022-05-30T06:19:00",
        "on_sky_stop_time": "2022-05-30T06:42:00",
        "output_pinned": false,
        "piggyback_allowed_aartfaac": true,
        "piggyback_allowed_tbb": true,
        "priority_queue": "http://localhost:8008/api/priority_queue_type/A",
        "priority_queue_value": "A",
        "process_start_time": "2022-05-30T06:19:00",
        "process_stop_time": "2022-05-30T06:44:00",
        "rank": 0.0,
        "scheduled_start_time": "2022-05-30T06:19:00",
        "scheduled_stop_time": "2022-05-30T06:44:00",
        "scheduling_constraints_doc": {
            "sky": {
                "min_distance": {
                    "sun": 0.5000368306963754,
                    "moon": 0.5000368306963754,
                    "jupiter": 0.5000368306963754
                },
                "min_elevation": {
                    "target": 0.5000368306963754,
                    "calibrator": 0.5000368306963754
                },
                "transit_offset": {
                    "to": 7200,
                    "from": -7200
                }
            },
            "time": {
                "between": [],
                "not_between": []
            },
            "daily": {
                "require_day": false,
                "require_night": false,
                "avoid_twilight": false
            },
            "$schema": "http://127.0.0.1:8008/api/schemas/schedulingconstraintstemplate/constraints/1/ref_resolved",
            "scheduler": "fixed_time"
        },
        "scheduling_constraints_template": "http://localhost:8008/api/scheduling_constraints_template/1",
        "scheduling_constraints_template_id": 1,
        "specifications_template": "http://localhost:8008/api/scheduling_unit_template/17",
        "specifications_template_id": 17,
        "status": "schedulable",
        "status_value": "schedulable",
        "tags": [],
        "task_blueprints": [
            "http://localhost:8008/api/task_blueprint/3",
            "http://localhost:8008/api/task_blueprint/2",
            "http://localhost:8008/api/task_blueprint/4",
            "http://localhost:8008/api/task_blueprint/5",
            "http://localhost:8008/api/task_blueprint/6",
            "http://localhost:8008/api/task_blueprint/7",
            "http://localhost:8008/api/task_blueprint/1",
            "http://localhost:8008/api/task_blueprint/9",
            "http://localhost:8008/api/task_blueprint/10",
            "http://localhost:8008/api/task_blueprint/8",
            "http://localhost:8008/api/task_blueprint/11"
        ],
        "task_blueprints_ids": [
            3,
            2,
            4,
            5,
            6,
            7,
            1,
            9,
            10,
            8,
            11
        ],
        "unschedulable_reason": "",
        "updated_at": "2022-05-30T05:49:13.964320"
    }
    
  ]
};

export default TaskServiceMock;